# -*- coding: UTF-8 -*-
#This is generated code - do not edit
encoding = 'UTF-8'
dict = {'Add subnote': '\xe0\xa4\x89\xe0\xa4\xaa\xe0\xa4\xa8\xe0\xa5\x8b\xe0\xa4\x82\xe0\xa4\xa6  \xe0\xa4\x9c\xe0\xa5\x8b\xe0\xa4\xa1\xe0\xa4\xbe', 'Cut': '\xe0\xa4\x95\xe0\xa4\xbe\xe0\xa4\xaa\xe0\xa4\xbe', 'New subcategory': '\xe0\xa4\xa8\xe0\xa4\xb5\xe0\xa5\x80\xe0\xa4\xa8 \xe0\xa4\x85\xe0\xa4\x82\xe0\xa4\xa4\xe0\xa4\xb0\xe0\xa4\xb5\xe0\xa4\xb0\xe0\xa5\x8d\xe0\xa4\x97', 'Drag and drop': '\xe0\xa4\x93\xe0\xa4\xa2\xe0\xa4\xbe \xe0\xa4\x86\xe0\xa4\xa3\xe0\xa4\xbf \xe0\xa4\xb8\xe0\xa5\x8b\xe0\xa4\xa1\xe0\xa4\xbe', 'New category': '\xe0\xa4\xa8\xe0\xa4\xb5\xe0\xa5\x80\xe0\xa4\xa8 \xe0\xa4\xb5\xe0\xa4\xb0\xe0\xa5\x8d\xe0\xa4\x97', 'Copy': '\xe0\xa4\xaa\xe0\xa5\x8d\xe0\xa4\xb0\xe0\xa4\xa4\xe0\xa4\xbf\xe0\xa4\xb2\xe0\xa4\xbf\xe0\xa4\xaa\xe0\xa5\x80', 'Paste': '\xe0\xa4\x9a\xe0\xa4\xbf\xe0\xa4\x9f\xe0\xa4\x95\xe0\xa4\xb5\xe0\xa4\xbe', 'Delete': '\xe0\xa4\x95\xe0\xa4\xbe\xe0\xa4\xa2\xe0\xa5\x82\xe0\xa4\xa8 \xe0\xa4\x9f\xe0\xa4\xbe\xe0\xa4\x95\xe0\xa4\xbe'}