# -*- coding: UTF-8 -*-
'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import test
from taskcoachlib.gui.dialog import entry 


class AmountEntryTest(test.wxTestCase):
    def setUp(self):
        super(AmountEntryTest, self).setUp()
        self.amountEntry = entry.AmountEntry(self.frame)

    def testCreate(self):
        self.assertEqual(0.0, self.amountEntry.GetValue())

    def testSetValue(self):
        self.amountEntry.SetValue(1.0)
        self.assertEqual(1.0, self.amountEntry.GetValue())
