# -*- coding: UTF-8 -*-
#This is generated code - do not edit
encoding = 'UTF-8'
dict = {'Insert a new attachment': 'Neue Datei hinzuf\xc3\xbcgen', 'Closed %s': '%s geschlossen', 'Monthly': 'Monatlich', 'Minimize priority': 'Niedrigste Priorit\xc3\xa4t', 'Access': 'Zugriff', 'New subnote': 'Neue Unternotiz', '&New task...': '&Neue Aufgabe...', 'Language not found?': 'Sprache nicht gefunden?', 'Task': 'Aufgabe', 'Notes': 'Notizen', 'Decrease priority': 'Priorit\xc3\xa4t verringern', 'Add attachment': 'Anhang hinzuf\xc3\xbcgen', 'Yearly': 'J\xc3\xa4hrlich', '%s file error': '%s Dateifehler', 'Edit location of attachments': 'Bearbeite den Ort des Anhangs', 'Copy "%s"': 'Kopiere "%s"', 'Weekly': 'W\xc3\xb6chentlich', 'New task': 'Neue Aufgabe', 'Browse': 'Durchsuchen', 'Error while saving %s.ini:\n%s\n': 'Fehler beim Speichern von %s.ini:\n%s\n', 'Dates': 'Daten', 'Priority': 'Priorit\xc3\xa4t', 'Location': 'Ort', 'Show tips window on startup': 'Hilfefenster beim Start anzeigen', 'keeping dates on the same weekday': 'gleichen Wochentag beibehalten', 'New subtask': 'Neue Unteraufgabe', 'Insert a new category': 'Neue Kategorie einf\xc3\xbcgen', 'Enable SyncML': 'SyncML aktivieren', 'User name/ID': 'Benutzername/ID', 'Always': 'Immer', 'Categories': 'Kategorien', 'New &subtask...': 'Neue &Teilaufgabe...', 'None': 'Keine', 'Edit attachment "%s" location': 'Bearbeite Speicherort des Anhangs "%s"', 'Budget': 'Budget', 'Daily': 'T\xc3\xa4glich', 'Delete': 'Entfernen', 'Tasks': 'Aufgaben', 'Completion date': 'Abschlussdatum', 'Due date': 'F\xc3\xa4lligkeitsdatum', 'Maximize priority': 'H\xc3\xb6chste Priorit\xc3\xa4t', 'day(s),': 'Tag(e),', 'Yes': 'Ja', 'Subject': 'Betreff', 'Cut': 'Ausschneiden', 'No': 'Nein', 'Start': 'Start', 'Reminder': 'Erinnerung', 'Check for new version of %(name)s on startup': 'Beim Start auf neue Version von %(name)s pr\xc3\xbcfen', 'Start tracking': 'Stoppuhr starten', 'Open the selected attachments': 'Ausgew\xc3\xa4hlte Anh\xc3\xa4ngen \xc3\xb6ffnen', 'Drag and drop': 'Ziehen und Ablegen', 'New attachment': 'Neuer Anhang', 'Stop tracking': 'Stoppuhr anhalten', 'Language': 'Sprache', 'Save error': 'Fehler beim Speichern', 'Insert a new note': 'Neue Notiz einf\xc3\xbcgen', 'Error': 'Failer', 'Edit task': 'Aufgabe editieren', 'Attachments': 'Anh\xc3\xa4nge', 'month(s),': 'Monat(e),', 'Window behavior': 'Fensterverhalten', 'Insert a new task': 'Neue Aufgabe einf\xc3\xbcgen', 'Auto save after every change': 'Nach jeder \xc3\x84nderung speichern', 'Error while reading %s:\n': 'Fehler beim Lesen %s\n', 'Open': '\xc3\x96ffnen', 'Editor': 'Editor', 'Slow': 'Langsam', 'Hourly fee': 'Stundensatz', 'New subcategory': 'Neue Unterkategorie', 'Recurrence': 'Wiederholung', 'Increase priority': 'Priorit\xc3\xa4t erh\xc3\xb6hen', 'New note': 'Neue Notiz', 'New version of %(name)s available': 'Neue Version von %(name)s verf\xc3\xbcgbar', 'Description': 'Beschreibung', 'Never': 'Nie', 'Stop': 'Stop', 'Copy': 'Kopieren', 'Paste': 'Einf\xc3\xbcgen', ', every': ', jeden', 'Errors have occured. Please see "%s"': 'Es sind Fehler aufgetreten. Weitere Informationen siehe "%s"', 'period,': 'Zeitspanne', 'Add an effort period to the selected task(s)': 'Aufwand auf das Budget der ausgew\xc3\xa4hlten Aufgabe(n) buchen', 'New attachment...': 'Neuer Anhang...', "Couldn't load settings from TaskCoach.ini:\n%s": 'Konnte Einstellungen von TaskCoach.ini nicht laden:\n%s', 'Add attachment to "%s"': 'Anhang hinzuf\xc3\xbcgen zu "%s"', 'Errors have occured. Please see "taskcoachlog.txt" in your "My Documents" folder.': 'Es sind Fehler aufgetreten. Weitere Informationen wurde in der Datei "taskcoachlog.txt" im Ordner "Eigene Dateien" gespeichert.', 'Open attachment': 'Anhang \xc3\xb6ffnen', 'New category': 'Neue Kategorie', 'year(s),': 'Jahr(e)', 'Remove attachment': 'Anhang entfernen', 'week(s),': 'Woche(n),'}