# -*- coding: UTF-8 -*-
#This is generated code - do not edit
encoding = 'UTF-8'
dict = {'Cut': 'Ebaki', 'New task': 'Zeregin berria', 'No': 'Ez', 'Monthly': 'Hilero', 'Priority': 'Lehentasuna', 'Editor': 'Editorea', 'Files': 'Fitxategiak', 'Description': 'Deskribapena', 'Never': 'Inoiz ez', 'New attachment': 'Eranskin berria', 'Copy': 'Kopiatu', 'Paste': 'Itsatsi', 'Delete': 'Ezabatu', 'Language': 'Hizkuntza', 'Always': 'Beti', 'Notes': 'Oharrak', 'Daily': 'Egunero', "Couldn't load settings from TaskCoach.ini:\n%s": 'Ezin da TaskCoach.ini-tik konfigurazioa kargatu:\n%s', 'Attachments': 'Eranskinak', 'Yearly': 'Urtero', '%s file error': '%s fitxategi akatsa', 'Due date': 'Iraungipen-data', 'Insert a new task': 'Txertatu zeregin berria', 'Yes': 'Bai', 'Weekly': 'Astero', 'Subject': 'Gaia'}