# -*- coding: UTF-8 -*-
#This is generated code - do not edit
encoding = 'UTF-8'
dict = {'Insert a new attachment': 'Legg til nytt vedlegg', 'now': 'no', 'Cut': 'Klypp ut', '&Reminder': '&P\xc3\xa5minning', 'New task': 'Ny oppg\xc3\xa5ve', 'year(s),': '\xc3\xa5r', 'No': 'Nei', 'Monthly': 'Kvar m\xc3\xa5nad', 'Minimize priority': 'Sett l\xc3\xa6gste prioritet', 'Edit task': 'Endra oppg\xc3\xa5va', 'Merge': 'Sl\xc3\xa5 saman', 'Every %(frequency)d days': 'Kvar %(frequency)d. dag', 'New subcategory': 'Ny underkategori', 'Recurrence': 'Repetering', 'Increase priority': 'Auk prioritet', '&Merge...': '&Sl\xc3\xa5 saman \xe2\x80\xa6', 'New note': 'Nytt notat', '&Recurrence': '&Gjentaking', 'Reminder': 'P\xc3\xa5minning', 'Every %(frequency)d weeks': 'Kvar %(frequency)d. veke', 'Every other year': 'Annakvart \xc3\xa5r', '&Select': '&Vel', '&Columns': '&Kolonnar', 'New subtask': 'Ny underoppg\xc3\xa5ve', 'Insert a new category': 'Sett inn ein ny kategori', 'New subnote': 'Nytt undernotat', 'Every other week': 'Annakvar veke', 'Open the selected attachments': 'Opna dei valde vedlegga', 'Stop': 'Stogg', 'Every %(frequency)d years': 'Kvart %(frequency)d. \xc3\xa5r', 'Page': 'Side', 'Start tracking': 'Start timeregistrering', 'New attachment': 'Nytt vedlegg', 'Every %(frequency)d months': 'Kvar %(frequency)d. m\xc3\xa5nad', '&Undo': 'A&ngra', '&Ascending': '&Stigande', 'Error while saving %s.ini:\n%s\n': 'Noko gjekk gale under lagring av %s.ini:\n%s\n', 'Every other month': 'Annankvar m\xc3\xa5nad', 'Copy': 'Kopi\xc3\xa9r', 'Paste': 'Lim inn', 'period,': 'periode,', 'Stop tracking': 'Stogg timeregistrering', 'New &subtask...': 'Ny &underoppg\xc3\xa5ve...', 'Dates': 'Datoar', 'Task': 'Oppg\xc3\xa5ve', 'Save error': 'Kunne ikkje lagra', 'Always': 'Kvar g\xc3\xa5ng', 'Notes': 'Notatar', '&Open...\tCtrl+O': '&Opne...\tCtrl+O', 'Decrease priority': 'Mink prioritet', 'Daily': 'Dagleg', '&Note': '&Notat', 'New attachment...': 'Nytt vedlegg...', '&New task...': '&Ny oppg\xc3\xa5ve...', 'Open attachment': 'Opna vedlegg', 'Add attachment': 'Legg til vedlegg', 'month(s),': 'm\xc3\xa5nad(ar)', 'Start': 'Start', 'Yes': 'Jau', 'Delete': 'Fjern', 'Yearly': '\xc3\x85rleg', 'Print preview': 'F\xc3\xb8rehandsvising av utskrift', 'Every other day': 'Annankvar dag', 'Tasks': 'Oppg\xc3\xa5ver', '%s file error': 'Filfeil: %s', 'Maximize priority': 'Set h\xc3\xb8gste prioritet', 'Revenue': 'Forteneste', '&Description': 'S&kildring', 'Insert a new task': 'Sett inn ny oppg\xc3\xa5ve', '&Redo': '&Gjer om', '&Task': '&Oppg\xc3\xa5ve', '&Hide': '&G\xc3\xb8ym', 'Categories': 'Kategoriar', 'Close the current file': 'Lukk noverande fil', 'Features': 'Funksjonar', 'week(s),': 'veke(r),', '&Category': '&Kategori', 'New category': 'Ny kategori', 'Open': 'Opne', 'Drag and drop': 'Dra og slepp', 'Weekly': 'Kvar veke'}