#----------------------------------------------------------------------
# This file was generated by make.py
#
from wx.lib.embeddedimage import PyEmbeddedImage

catalog = {}
index = []

undo16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJVSURBVHjaYmTAAqyMjeXtWFk3"
    "fPz48dn069e9GfAAgABiQhdwVVOzD1NSulC7bZuBBCenOAMBABBAKMhLXr5+W3Pz//9v3/7/"
    "/+bN/0YdnaceSkrxQBfx49IDEEAwJ/PnGhntf7Rq1f//N278/3/t2v//V678/3/x4v9Hy5b9"
    "nxUZ+R5kODa9AAHECLRB31pW9kBRVZUAFy8vA8OfPwwMf/9i0I+vXGGoWrjwwD1e3oBjZ89+"
    "hBkAEEBMLL9/OwTHxgpwgXgvXzIwvH4NoV+8YGB49gyCHz9mkOXmZuj383NQevlyGbILAAKI"
    "+danTyfvnzv3mY2V1V1bTo6B4eZNBoa3bxkY3r2DGPbqFcTAJ08YuIC09p8/qjc4OT/fe//+"
    "OMgAgABiBhG3P306/vr58+0PHz0KNfPy4mAFafjwgeEwUO69uDgDn64uA+vv3wwMjx4xiLCy"
    "Mnz59Mnqmbj4rBevX38HCCBmmFMefPz49AM7+5ybJ09a6rq4yPNzcTHMW7fuZtfp04k7V6/+"
    "/ltCwsggJISB4epVBomfP9m3f/jwGKjnFEAAYY0VD3n5pm2Vlf8b/f3PwMTM9PWNay0tL32d"
    "M+f/f1vb/1lSUntA4gABhBPZKSm52UhIJKGn0Al+fu//t7T8n6Ko+B8oxAkQQAykAlDCujZ5"
    "8v+d2tr/dTU0bAACiJlUA+68f39R6v//BEt1dYF1z56tBwggJgYywPGrVye8FRNjYP32TQwg"
    "gMgy4J2Y2IHTFy4wCH//rgUQQMzkGPD4+fOXApycZo9//94GEGAA2rH5UKps700AAAAASUVO"
    "RK5CYII=")
index.append('undo16x16')
catalog['undo16x16'] = undo16x16

#----------------------------------------------------------------------
computer_desktop_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOWSURBVHjaYvz//z8DCKSmZmy0"
    "tDRT4eJkY2dlZmVlZmJhY2FiYWFiZGVjZAJyOLlYb919/K6gIMXy168fz4BafoL0AQQQCwMU"
    "ODu7OaioaPCV125j+McuwvCHiZHhNxMzw5//jAx/fjMwsPx+zFAQJSPKzMxkA1S+C4hfA/E/"
    "gACCG3D58uVzivLqDvtO/2NgkOJnYBDlYWDg52ZgYGVmYPj/l4Hh1SuG91/YmISFRXSfPHl0"
    "AqjlHcgAgABighlw796ji+Ji3Ayy6kDNYkDNsgIMDHJiDAwKUgwM0sIMDNycDK++MDLISEnJ"
    "AJVzwlwPEEBwA969e3/+37+/DGpyQM1vPwN9CLT1NzB8vv5iYHgJtOz9N4bHH4HmyStKA5Vz"
    "wAwACCC4Ad++/bj86tWbnwZ6okDNwPB5/4mB8fVLBub3LxkYWYAGiQsy3P3MxKCoqCyB7AKA"
    "AIKHwadP724/ePD8uaG2ogID3zeg//kYmPhZGbi4WBj+AUPxJ9CQv8zfGFRVNcQYGRmFgLHH"
    "CtIHEEBwA1hYOL6+fP3hk4MBP4Oq2m8GaVUGBm2gY0VYPjNI8bEzSPDxM3AB4cXzj/4zM7Pw"
    "//nzG2QAI0AAwQ0ICor3FxEV0/nz/RvDkiwlhm+fvzKcP3fi3YlTJ+9vvX/v5YePH1+8fvP6"
    "1afPHx7////vAVDLH5A+gACCG6CkJBWsr6/M9PLlG4avXz8xvH37gWHrrh3v9u7bOQUo/RKI"
    "gUHI8AOIv4N8DMX/AQIIbsCZMyfvi4ry/BAREeRgZ2dhYGRkYwgLT1QRFpMt0jW0YuLi5OO+"
    "df3iwUvnT047fmLXA6hhDAABhBSI34Tevv3G/vr1dwZJGTEGeRVNBmVdCwaXwHDdb8CY/AK0"
    "V9fMScHM9Ejg9x8fGy9cODkHqO0LQADBo/H8+RO7T588ep6N6f8vhr+MwDT2n4GN6Q/D9w9v"
    "GL6/fsjA9PEpgwowAiMjfPk8Pf3ygVrkQNEJEEBwF5w9e3DH8zdPfv8SVpxjz/pP4uf3twxa"
    "OjoM/IIcDJ9Y/jA8evT454pdR16uXbvqyoULp/cAtXCDfAAQQIyw3Ghlbsr9VCziPL+ej6rc"
    "30cM+vxfGe5c3Hbz8qXjl549e/r48+ePj//9+/cU5GxoPngOxK8AAgjuAnZO7j8s9zdst1H8"
    "JSgmKcN55tSB+9s2zp8LtOAOUPoNVCMwhTF8hcYECP8BCDAASVROeCfL8fUAAAAASUVORK5C"
    "YII=")
index.append('computer_desktop_icon16x16')
catalog['computer_desktop_icon16x16'] = computer_desktop_icon16x16

#----------------------------------------------------------------------
led_grey_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAQAAAC1+jfqAAAAAmJLR0QA/4ePzL8AAAAJcEhZ"
    "cwAACxMAAAsTAQCanBgAAAAHdElNRQfUCw0UEzCc2GfHAAABbklEQVR42n2Rv0tbYRSGn3Pu"
    "p73GJCCFdhA0BGkHt4g61xKLlIASB4ciCILon9DJ3b2tFCqC4lKTilDs0G6lg0YEFSl0E4u/"
    "WjS3BjW539fhGsHF8y7vOe/zTkdOuH8MzNBEMxUuRvzRloyfuDg/2Did06LHJUuYiKu2P3z7"
    "bCBFDCAW5HZzX4unU3pIBNTaWr/1peMIDsHRTDcdQwtPNp9zpOA8/31v+gEhFofFYgmJk+80"
    "7xAF8qn+kPAmCG8ReDn4+IWB5PgjAmL4OATB4bBUCEiTGTPQkmlCOCNBIx4AIVf8Q/HpyBiI"
    "xQ0ehmtCFMFhCWlAsCSSClI2NGAwKAIIiiG6BWWFYN3hoSgeHgbvdrP8LCn8nj1GEPSOBOEX"
    "G/MKldUfKzUEQW7aka+x8OnvmgJuf+LjTh2pt6u82f0yiVMAPSpkp5e3qQIOxxVbvC58yNr6"
    "L6B6WBz+PtDz6mlXMnlW3iutL/75HCX/AWZafCjm+7rjAAAAAElFTkSuQmCC")
index.append('led_grey_icon16x16')
catalog['led_grey_icon16x16'] = led_grey_icon16x16

#----------------------------------------------------------------------
lock_locked_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAN8SURBVHjaYvz//z+Dr28MAwz8"
    "+/ePgZeX39fExLBCTU3ZgIWFlfHhw8fXzp+/3Pv58/cVzMxM/2FqlyzpZwAIIBYQg42NFSwA"
    "MoyXlzfPw8Oz7/9/JuYLF67fYWJi+qerq22sqqqybMuW3SqfPn1uZmRkhFsIEEBMIIKFlQ2C"
    "WViMTUzMO96+/ci4dsPmstMXrpqdOHfVdMny9WlvXr//6ehoW8vCwubExsbOAMIgABBAYAN+"
    "MvIx/PrPwcAvIJrMycHDefrs+Q3sXx53y/+7+17h351P7F8fzT5w5PRSPl5eVgkJ8aSffxgZ"
    "fjFwgQ0ACCCwAVqMFxj+/PnDwMLBK/fu03eGj79Z98kwPmNQe7ubQfXNLgY5pmcMn/6wb3/7"
    "8TsDv7C4/N9/TAyKv46DDQAIIHAYaLJckxJnecV/95OF2Lf3nAyGTKd5FYReK735wc8Lklfj"
    "e/OV++8x4S9vVRkYPz4VNmHcYyb0+/4zoNQTgABiBAXc1mrTO9qWWso/f7Mw/GdiZGD+/eE/"
    "MKAYmTiYGRgZ/zD8/faT4f+vv/8ZOAQYGf7+ZmBm/Mlw98rLJ64NJ2UBAgjsAjk1KWUFNzcG"
    "hq+fGRh+vWJgYBQDKnzF8OLqVYZ/f78ySBnyMzB85mJk+PkSKAeMRT5eoJdFZEB6AQIIbMC/"
    "X9//Mnx4zMzw4wcDw5+3wHj9yHBqw0GGmMpHDD//MTDMr+RkcHKUYmD4Bgyyv38YGH6wMfz7"
    "LvEXpBcggFggqecX0OYXDAy/QQa8Y2Bgfs/w4elHhv9fGRhAyeb9w58MDN+B8j+gBjCzM/z/"
    "KwjWChBAUAOAgn/eAA34BqQ/ABPGFwZrc04Ge8GPDF9+MzDYGwJN+Q+U+wdObaDkCmSAHcAA"
    "EEDgaPwPTllA6xg+Af0IxP8/Mrw795lB/DswfICO+3AdqIwHGO/8QMzHAaSBiYgJrJUBIIAg"
    "LmAGYkag0/8BA/E/EH/9yiCiwsSQ284DFGdk4OUHKmZhY2BgB1r0D2gzFzDpszCDtQIEEAs0"
    "CwENAYYwM9AVzEAJVmYGTjtBBk42aJoH+pDhBywPgQwAuoAZ4gKAAAIbwMQG1CEBDJRvQFv+"
    "Q80E6YXnOyDgh+dXBgZudgZGDlawEwACCKz60dV3Fzk2iyr9/cPxB6GJEUU/AvwH+oaF5f6F"
    "9880ghkYAAIMAKTvLNzpbJnhAAAAAElFTkSuQmCC")
index.append('lock_locked_icon16x16')
catalog['lock_locked_icon16x16'] = lock_locked_icon16x16

#----------------------------------------------------------------------
music_note_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAKRSURBVHjaYvz//z8DDDCaGjEw"
    "sDO2MHx/Fc/w5Kc+g4rEOwamvwwMTEA1bEB88wUDw+MPDMh6AAKIiQEZMP1jYPj5c4tKrKq0"
    "XLDGTYYvXy0Y/v5nwAcAAgjVABagbX//nmB8zPx4Xm+jiFumzVEG9l8rGb79EmL4h90ggABC"
    "M+APAwP3f4bb5+5uOvbgDENNRjJT+/KcML0I1ecMDH+6GX5jGgIQQIwoYeCkBmF8/6OsHC1+"
    "g4H1P4sSk9xfRQcp5hsvHjIcKz9++M/xl3bIegACCBwgMMzgqgTBDooM3BnqlxWmGoBURjEI"
    "sVZJFGp+klis959BmqsfWQ9AAKF6gfkPBLP/Zfj68NuM74K/GVj0BWIY3v1uezH/vsirQ+8f"
    "MxhyxyFrAQggVANY/0DCARSYv/7O//L02w9mBw5HoAwXgzDzr3+v/8UzPPt9A1kLQAChGsD1"
    "lYGB8zvQBb+A9N9v329/P/jPhIGDgY81heH+NwaGtz/3A2lrZC0AAcSENW7+/mJi+PuN6d+D"
    "Xyt/M/xmYDBlSwFH45VPDAy//6EoBQggVANAQfb7dziPAv9DVW+NnzwenLPAYrasmsA4UmR4"
    "D3TZlz8oWgACCDUWQrjKzBfb/dv9c9X/S/83/I/66fWf4TUwDe8W+s8gxdaFTQ9AALGgmKbF"
    "UiPlw814i/kow4p3dxh2vbsC9A4PA4MEOwOD2g8Xhme/MHwLEECoBghxcK+/e5Vhw/crDP//"
    "AF30FyjNIgwMPGA4vP71CFtwAQQQqgF3vl5m4OPT/8/FBdTMBHIrA8OHLwwMm14+Zrj2Jx2b"
    "AQABhJqU45l5GJ4wL2aQ5nVi4GFmY3j14xPD2S8bGB78LwZKf0EOAxgACDAA0QkXZ2L68s0A"
    "AAAASUVORK5CYII=")
index.append('music_note_icon16x16')
catalog['music_note_icon16x16'] = music_note_icon16x16

#----------------------------------------------------------------------
box_out_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAFWSURBVHjaYvz//z8DJQAggFjw"
    "Sc5iZASbfpaBQXrm///PsKkBCCAGkAuw4ZkMQGrPnv//16//nwZkA7EUNnUAAcSIzQsgm9NO"
    "nWJgWL+egeHpUwaG588Z0nfvBklhuAQggJiwar55k4EBZAASmKmlBaKepjMySiGLAwQQE4bm"
    "Fy8YGG7dwurdmcLCYEOQxQACCCUQQYE1S0ICrADuBShIv3YNxpRG1gMQQCgGQP3HCAt9DBf8"
    "/8+ILgYQQEwMFAKAAKLYAIAAwhqN0JB+iiaMNTEBBBDYgFnpJnBT0maeYcRnY2eY8v+/f/8y"
    "gHDthseMAAEED0S/zGyGTdOngg0DGfrv3z8UDNIAokHAw1OSYeuWJ2A2QAChxIJ3ciLD399f"
    "Gf79+gKkv4HxPyj+++cHw38g/vf3J8O/P7/gegACCG4AyHbk/IHLBTDnwwBAADFSmp0BAgwA"
    "Jh3X6dZdyrgAAAAASUVORK5CYII=")
index.append('box_out_icon16x16')
catalog['box_out_icon16x16'] = box_out_icon16x16

#----------------------------------------------------------------------
cake_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAP0SURBVHjaYrwxz5Hh+88fDB/W"
    "3mZgvPWeQZDhL8N/BiAAEmz/GHrm/2DQ6XnHEPL/P8MXBiwAIIBYQATTn38Mby3FGJhkGRmM"
    "nr9mYGAF6v/FoMf3gyHd/QsD16wPDM4f/zJsBKllZmRg4GdnYPj8i4Hh9z8GBoAAYoKYw8jA"
    "+usvwydmdoZ9XEIMBx4zMJz9wmjy312IZ58A5yeg5oeszEwMelJsDO6aPAwinIwMLIwQnQAB"
    "xAJzCogPNJDhET8fw4Nf3xnYX/K+VeLPZDj3fu19JeHrF7w0hRg+fv/O8Os/E8O//wwQbwIB"
    "QACxQLT+Z/j35yfDv98/mRj+/NMX1xdS4/kiGsT0gY9BW4BXSlVQOJvh598rP378PP3jH8O3"
    "P38RYQAQAEEAvv8B2p5AAPXz9BDp8geACx06VxQiMxUNGR8DAvH3AAG9zAADLiIA/CggAO7k"
    "4/7e1s/1xb63yLG7y33c7gDJ/AAAAAKI4eJ0R5ZrqzKPfnh++z8y+Pnq9f83c+b+/3HpMor4"
    "/98f/i9qS/sowMbgDNIPEEBMfzhl5fmU7TT4JaQZGL4BQ+/bUyB+xsDK+Y1B0N+WgU2eF8h/"
    "wsDw/RkDw1cgzcLFYObmxScmwBQGMgAggFh+fH76/seHJ98Y/v4XYmBihCj+/YWB8ecnBkZG"
    "oGe/AlWxAg1h4WZg4BQH2cnw6fmz/1++/vsEMgAggJh+fXz679XdC/++fPrC8O7+LYbri1sZ"
    "np8/zvBfwpaBQdyBgUHGneHrLzaGy+taGR5dOABKCQyvH91n+POL4RvIAIAAYmEQVPv1/P6t"
    "vw+bQhh4P9xg4P7ymuH9hR1ABzEySHsWMHy+e4zhzqpchrd3bzI8unKK4cW9uwzXjx9mAMYm"
    "yG0MAAEAQQC+/wPgLshYHvvGW/jXtwQJAvwAEA7+AAED/QDy28QA7tKsAOnJngD58OUABAsC"
    "APf3/ADUxcoA9v4PAeYOQwrn5OGmAogpwl1gkpmpKvv3X8Ck8fcXw39gKmFiBqYOpl8MX59e"
    "Z/j78yMDCwcwkf3/B5T7w/Dtyw8GPV15hqZpU60FlG0mAAQQy8c3t3+o+xQx/AIm7A9HlzP8"
    "BYc4MKGDkhgb0I1A5t8/oFTKzMDMJcpg4m/PYBHfxLDpxheG319fSAIEECOfQaqNj7PWgti4"
    "CCVpQRZGxjeXGP6/vcvw7+tbBmBqABrCwsDEyc/AwCfLwCysx/DhnwjDxm0H/s7uaT/9/uHx"
    "dIAAAEEAvv8BDCVRAPrt1BALERVSHiAeUyQwNigWGBkODAsIBAQEAQL9/f8A9/j8/ezr7Prg"
    "2dbw2czF0+7h1akAAPm0AAAA+AIMAD6Gp+zOcXqNAAAAAElFTkSuQmCC")
index.append('cake_icon16x16')
catalog['cake_icon16x16'] = cake_icon16x16

#----------------------------------------------------------------------
folder_blue_arrow_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAbqSURBVHjaYmSI3M/A8PsPgwAP"
    "G8MvRjYG02edDNyMXxj+MTAz/P//n+Hfv3/Mzs6uS7W1tS337Nm949KlS7tYWFgvXL9+/a6X"
    "lzfD2bOnwepkZWUZHj16xGBsbMqwevVKBoAAYmFAAv+B+LWwPQP/h20Mf6AGs7Gxa/r5+QVr"
    "amqweHl5pj179iztwoWL944fP3n6y5fPh37+/HkYqO3y79+/GP78+QM3CyCAUAxmZmRgeMhm"
    "wMD0cT+DsuBvhm8//zDIycl5KyoqgNUxMTExyMjIgLCSj4+30qtXr8ODggLfAi06d/HihYOf"
    "P3/d+e3btztACz4ABBCKwWDN//8yPHvxmkGKnY3hz38mViMjI1cODg4GbEBMTBSEhe3t7Vy/"
    "fPnmevfu3cZz5y6cOX36VBNAADGhBAXQ68zMzAzvnj9nYPz3k0FQgF8eaLAuAxGAh4eLQV9f"
    "lzkhIdZcWVk1BSCAWICmMTD8+CP8iYlRkpWTVfjfvz9/GRj+nnj5jeuPq6GsqY6mmihE6z+G"
    "37//MXz9+Y/h3dc/DG+//mX4/P0Pw+cf/xkcNHgZeNiZGJhZmBnu3LnHcOvWjbcAAcTCzcke"
    "52cvmy0nwi4rIsgtdvv1rz+zDuhVXnjMM81OUM973+1fjE/ePWf49P0vw29QpDCDQo+R4fe/"
    "/wzsnBwMF+69Y/jw4gFDtLMmw38WTobTp8/+ePjw4RmAAGIxUBOp8XNQUf3ynYHhF1CnGicD"
    "s1NWZysv0/dMbiVRhYefWRj4+DgYZCWYGTjYmRlYgK4CReJfRkagJQwMX379Zbh2+xbDt6+K"
    "DMx/WRmOHj387O/fP2cBAjBILicAwkAUfIiaBGzDk7VYiKXagFdb8INxk4g6BSzMzps6XfHo"
    "qMED4hyu2GmchpDT0xOcgm8Us0QgKkjChDAgA+LlrlSt9vuTWdK5GcTzwkvrL4BYfn3+dF9H"
    "jMH4z29QUAMxUOObzwwMnz99YxDk42L4BjTsD1AM6HMwDTL8N1TsK1APJwcXw4cfjAzfvv9i"
    "uH3r5t9rl87sAqattwABxPThxdNHr778B3oRmI5ZgWHIBgxBoEt4OXmAuZGFgZEJ6lKgpb+g"
    "LgbRIFd/+QkymJnhy392hnfvPzGcuXLn11eJcC1GISMFgABi+fzk+r0vPxkZ7jw5znDxyS0G"
    "RmY2hp9Ak1gZQbnuH4OujBWDrKQqwx+o4T//QnwGdCDDr3/geGRgZOFgePzoIcMdZlVO0RDP"
    "jDezPW8DBBDTd3aVhwcuXfy/4ep8BgUZBgYpiW8MirI/GBTl/jJw8b5nmL27j+Huy18Mz4HB"
    "8/ILA8O7bwwMn35AXAyyBOR6bn5+hmt3nzI8+MnL8O/Z6bf/P928BxBALL++/3xx5d7jr8p6"
    "4jwhlu4Mn/8AdQNdy8rIyvDj7zeGHWduMtx8+YNBUJCNgQniQHCZ8htqKNDhDEKiogwXbnxg"
    "eM3zn+HLjUMPgCouAgQQy79fDO/fvvr8QeTfJ54zX84yfP/2ExjGjAxcbBwMX/99Yvj66wfD"
    "py+/Gdi5/jKwAMUZmZnAeQoUND+A+D8wbm7d/cjw6PFvhn+KPxn+PD1xFmjXc4AAYmH4vvfV"
    "84curyV//5B59PMhwy+gM5iYGBkevXvNcPjpOYb77zgYRN8JMbz895eB8/9nBi52FgYONmZg"
    "RLMzsABz28OHHxhASecdsywD59vnv/69vX4caPAPgAAAWQCm/wQBH81W6Aj8Iuv9/ADh+OIA"
    "9ff3ANbv4QDT6eIA8Pf3AAIEFgDS234ADgosABoiVAAC9/sABP35AAUE+gD7+voACPDzADU3"
    "HADW6e4A9Nu44//99dYAAAAAAoiJ4f5HFh7tM1wMGm8YTr8/x3D43XGGO1/vATUyMAhy8jEw"
    "mb5keP59HsOPL38Yvv5lZvj0l4Ph7X9+hqe/eRlePPnCYGogwcAkwsMgzCfI8PH6gUtAM++A"
    "DAYIIFBpfvb7FU1RATFZcyUtYJpk+M7wD5j6mYD59fffrww/jgsySDJMYxBR92P4C60NQOX2"
    "h6dfGUT+fWX4CIzFhzeeMjw6c+DXp0tL5gOjcz9IGUAAAFkApv8BAAoZAAD8DxoHL2C8OFjx"
    "KRFI0QDiAOgA8er9AO2vHwAO830ADBgqAPrz3QD78Ob69/Dn9vvy5/T16Nzr//Tm5Pv06ecA"
    "9ujpAP/z5AD99LoAAADgAAAAAAKIkYFdAZhfgSmBT8eGQ9lvIb/uDkX+wGeMT2Z9/vD9hMot"
    "Ni6+178/33n29/OT+wy/Pj0DuugtUMMboF4QBsYcw1cg/o5ePgMEECMDEyeUBUqhnMB6icmS"
    "Seyf+b+nH68AvbIWqvkHFP9nIBIABBgAILXz+CUSDSQAAAAASUVORK5CYII=")
index.append('folder_blue_arrow_icon22x22')
catalog['folder_blue_arrow_icon22x22'] = folder_blue_arrow_icon22x22

#----------------------------------------------------------------------
earth_blue_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAbsSURBVHjaYmRgaGMAAUZGRgYm"
    "ZmaGv3/+A3m/gZjJkE9a2FteTdBcTkFQhVuAk+/b15/fHj/99OjRnSvn3j+6uYPh+4WjDKwM"
    "PxhYRRgYvh1gQAYAAcSIbDAI//v3V0FUQbwyMNE4KNxfQURDkY9BmI+BgZ2BCWzdmy//Ga4+"
    "+c2wYc/jH5sWr937+PyeHgaGDwcYfp9GMRgggBggBoNwKwMjQ4eDTdDuG2vOffj/9t+3/7/+"
    "f/r/88/X/7/+/vj//df3/x++vf///tur/19/v/7/5s+3/6uu/fvvVnL+M6tAYB66qQABxMzA"
    "4ASk/gMNZXJwSbZZUdSiJW8g/Zbhw4cXDO++fGP4+ecPw59//xhefn7F8OHbB4YvP78xfPj+"
    "leHXj3cMsgKfGUQ1eNi+8Jq5P7n07Pef77eOwgwGCCCowX8VrAIt1vrlc8gqcz5k+P2LkYGd"
    "TYxBmEeKgZeDj4GbnYvhxafnDO++vWX49ecnw/df3xi+/frF8A1oiQDzS4avfPcZ/wv42T06"
    "c/rW/78vr4IMBgggJgaGP6wySkrV8h6flIT/XWfg41Bj4OXWZeDllAX6g5OBkZmN4euvfwzv"
    "Pn9mYPzPyfD520+g4cAY+8nM8Or9B4YfP1gYlNm+MvCpXmVVccpvZmBgVQIZDBBATMyMbNqG"
    "vkrBFx8eZrjwUIyBmVMBGIkcDN+AMfXzLwPDu6+/GM7cOMfw4uVbhq+f/zH8+83D8OcXJ8P7"
    "Dx8Zfv/4x/D81UeGjbuPMBzeP41Bz9VElU3APBJkMEAAAFkApv8EFx0WTfwQ6BEKBj0A5AA6"
    "AKXRAAAuFQAAIhMAAADx7gDY18cA9/n0AAMDBADy7+MAAwIEAP39/ADz9/gA2eDhALvIvwDh"
    "4+AAAP/7APcu2wAhNE9P2bzEnAKIRURO0OIX83cGbi55Bm5eQYbXPxgYDjxkYLj1AWjwpw8M"
    "BzavZWBjYWBgY+cEuYPh24+fDB8+fmB4/OI5w6e/nAx/f/1lYOGRYPjz4RnDg9uHGcQVHRRu"
    "MnBpAQQQC78Yl/KHb+cZLu/vYriraMugphfAoKzmwvD8M9AwVnmG/3IuDIcOzwK6mJXh65dX"
    "wCj5A0r0QPSfgYtfnIGDjZ9BREwdaCUoKXxj4OJg5GZg4pYACCAWBqZfvB++XGX4y/mN4cXz"
    "HQyvXu5n+Pm7Dmi4H8NvoIu0jKOAloUAU8oHhu2bMhnev7nGwMrGDU5Sv389Z5BTCGKQl7Nm"
    "+PP3A4OsqBDDryfcLAz/GbkBAojp358/3zl42RhY+FkZuEQEGVj4/jNcvjyR4S0wPH8Bc/eH"
    "rwwMP/6yAbOeGIOQpCYDM88/Bmagub+Z3zFIq1gyOLk1MshIqzKIcLEx2BrYMTD/Z/4PDO9/"
    "AAHE9PX1r4c8PHwMbMBsy8j1m+Ev+2cGPaMUBlERJYY/wJTxD2j4L6Dv/wBTCDsfPwMrz3+w"
    "OmEpNQZLi3wGIQ4Whqcv9zJw8AJLF6ZvDK8fvwDG0vdvAAHE9P7Z/bPSvAYMTLw/GTj4OBms"
    "jXMZdFX8GbhZWBi4gQ7lZAViYAByALOSnJQlAwP3PwYmrn8MBjoxDBrS+kCD/zM8eneAYcGB"
    "RIar908xPL1zDxjtP54ABBDz188//hgaR4V/ljzJYiYfzuBvXszAwsQFjgx2qIEcrBBalF+G"
    "4S/zBwYdBU8GW41whv//XjP8/f2UgZeLiYFPGJhRWIMY1s/qPfTv+8OFAAEELN14uTR1G1fZ"
    "NQl6CwpyMJhLRDC8/grJHMCUBCwngMGByP8MnEBfcLBA8M/fLxl4WP8z6EpJMPxj+s+QmDz/"
    "+5l1GfnAGJgDEEDMoJT95tXjR3KCYSH6zuzsclwSDFwsbGCNoGDgAmIeIOZlh2KgwXxAzM/B"
    "wCDDx8OgIszDICvEwDB9yT2GVRPytjL8fTkFlLUAAghoMNDRDC8f3Tr35IumdISbn6cMkwDQ"
    "IDZgKSLMgTCQH4gFgXlECCgmwgUMFh6gwbwgw4GGrnvGUJOfffH/l2NNQMMugnwHEECQcghs"
    "OJBktqtILj3688mH//9//v///+L7e//vfvj6/9mn//9fAvGbr///fwVK/Pr3Hww+/f7/v3r6"
    "jf+swl4XgNr9oYaBAUAAMSLYsJBUjdAxTa8Lz3TQ/Kh3kuETxz0GYS4uBjZgzgOV2zo8+gxa"
    "TPYM50/+ZZg4dc3309sm7WL4e20SUHI/WAEUAAQQFoMFgPiLIrAoipTTdvEXVBNS4FZg5mHl"
    "ZWD5/v33/3cPf/z4evfL++dXj19k+HFyG1DfTqCGh+g1CEAAYTFYHIjfQStUBglgCKsDE68U"
    "sJwFhuxfoIuA1QfD+6dAudugyGHAAQACDAAV94IoYBDl6gAAAABJRU5ErkJggg==")
index.append('earth_blue_icon22x22')
catalog['earth_blue_icon22x22'] = earth_blue_icon22x22

#----------------------------------------------------------------------
copy22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANCSURBVHjaYvz//z8DLQBAALEw"
    "GiQjeP/+iUnISO+1MDfUYvjP+J+QpSBZxv8QGoSZmBgZfv/5y3Di+OnDAAHEwgDS/B8q+/N3"
    "REK4j057vAVcMSGDwe4B4r9QNiMQp/by2AEEEAsDEzMDw58/IOtAmJOfixOugBGfoVC3/PsH"
    "MRhEg8RY2RgYeHk4GQACiAUs/RtoMAvQAkbGv3/+/sXvyv8Q1/0DEn//Q137D8IGGfUPiP/8"
    "/scAEECQoAAZ/PM3SNcXRiZGvAYzMiIZzgA1HIphBoOMAAggFgZGpnA2AYEQJgbGn/9//9E+"
    "fv4GwzSWbwy/fv1BjSQg/vXnH4OSlACDr70+UIwJ7FJwMPyHBgU0wD9++88AEEAs+tqqC5pz"
    "gzjYmJnAin78+MnwA2Qo0Gkwt/+DmswIZKw+dIWBl4uDwdlcE+xJWPj+hbr0J9ArTz7+YQAI"
    "IBZlWXF2X3MVotPn43efGR69eA825C80Qf2DhvX/fxBLQHIAAcQCTHcgPiMTmgHgSPgHwSAv"
    "A+ODARjZwOj4x8DCyQoPXwakcGaE6gNZBhBALNhcBTb0P8JQEBukGRTq3379A7L/g9mwcEU2"
    "GGYZQAAxYRjKADUUmpz+QMPvL5T/7utf1CCAYuTUATIbIIBQXPwX2ZUwmgFqKFTsP9BdsJTw"
    "FymTwNh/oRkFIIBQDUYzFOxSKP8f1NXgHAe1EJzrGFFdCktBAAEEN/gPcmRBw/gvGv83LPZh"
    "joDlROQwhiZ8gABiYWREdS3Mlcj8P0h8ZIP+/kO4EsRmRIpQgABi+fDtN8OPv5Di788/1NTw"
    "FxqJfxiQguQfIuL+/0dkEHhygxoMEEAsX4CmAlMQA+t/LC7+jxrOsLD+h6TmH7KLGRGFEUAA"
    "sYDCAlRigjj/GSE0uKBBLosYEUUoqJBiBHoPpAaEmaDhzAhNuyB9TEADAQKI5ffff4w/QOUo"
    "UPQ3EyTw//xHxPofJmhEAvnA4gTsu3/M7Ay/QGxmaJAwQtQxg8oKIP7+6ycjQAAxctoUXjQx"
    "NtJhZWH+/xcaG7CY/49UBkNcy8Rw6/ZdBlFpOQYRUVFg9oY69T+iavrx6zfjtdNHzwEEECOt"
    "KlOAAAMAvNkfamZRgt0AAAAASUVORK5CYII=")
index.append('copy22x22')
catalog['copy22x22'] = copy22x22

#----------------------------------------------------------------------
up16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAKfSURBVHjaYvz//z8DJQAggJgY"
    "KAQAAcQCIhhNZ2HK/P/PyfDj7y5GRkbO/+zMdgxMzN8YGFHt+38ykQEggKAijKj4H5B6873S"
    "y0bWJsJT2Zjh3bc2BhZORgY+CQYGPkkEBgKAAMLuhR9/zISFOTO7Sq0YAqItGXjE+FMYfv12"
    "YQCFFzIGAoAAwjTg3382hu+/pzcWmokwiQszXPrCy6DjZsXNyMA4GSjHC9b47y8EAwFAAEEN"
    "+A/Bf4Bu//yzzd9dycg7QJdh4zUGhgcvGBiYJVUZ2OVU1Rn+/etj4OBhYBCUZWAQVgTrBAgg"
    "VBf8/usuJcaVU5ZvxXDsBQvD+dvfGW7f/8pw5zUTwy8JPQYGLt5kBiaWCAZ2bgYGdl6wFoAA"
    "ghjACAy4v/+FmH//7ajIs2T/wCfMsPfGP4aDG44x3D16goGH6R/DPxZuBiYpTaBCxiag8+UY"
    "/v8DawUIIIgBf4HOf/+9LshbzUDDUoNh700GhvNH7zC8fPDi/K+PH48rMzxiEAFayCIozsAu"
    "LKUKdGkDAxNEK0AAQcjPP7wUlARzgmMtGU48Y2K4cv0jw7VjV78zsLJmff7Dlnzl8pMPSkLf"
    "GPiALpdVk2Lg5OdOZPjPFg7SChBAYAP4+Tn7s/IdmN/x8DFcuv+f4cze8ww/v36bxsDKfIKB"
    "+f/1Z69/9b2+/4RBXvQfg4YKK4OVlTgDGydLL0gvQACBUyK/GI/ak68sDDe2v2C4dfUlw7un"
    "L88y8HA3gdMVnxhIWf/DJz9c//O8teVl42NgYWZm4OTjkgbpBQggRlBmYjWbfYaZkVX7DzM7"
    "w18mlo8MLKyRQD/uB8e5sBIDOMCY2HQZ2bi2srJziDKwcjL8YWG//3cquxZAADFSmhsBAoji"
    "3AgQYAAsQtEvrVL0fwAAAABJRU5ErkJggg==")
index.append('up16x16')
catalog['up16x16'] = up16x16

#----------------------------------------------------------------------
clock_resume_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9sCDA82CAig9O4AAANZ"
    "SURBVDjLXZNbT5sFAIaf70ALPX+lHfQMhTIpdCRQFOIWt5i5zItFM5ddqJgQvfXWv6Bemnin"
    "Juq8mNHMZJskEpNFJF3mnGwZK1AOAuNQpDB6/Np+By+WafS9fp/34k0egf/npeuvdCeUtxIx"
    "16lOn90tILBbqJSWN4qZleXDK9S0KTKv6c/qwj/g0LfeYK/y0YXzve+m407a5SaNUhHTMLC6"
    "PRxoLcxtVLh2I3fl8VLhAx5c3gaQAEh87el5znd1cmL4UtjcJyTsMTYYQqts0SZVGOmPUNrM"
    "gaoyNJ48sbN99Py+evYGh9eqEpEv2nwB5yeX3h67OODSGO9qEui0EooE8fkVgqEOPB4HklQl"
    "5JFoViUcsXBsLZfvKmtnpyRZPndm/PX0x165JJ7vq3FiJEm7zw+A1WrBYrFgmgYexYf/mA+7"
    "nudutkCLLzCwnlm+Jyod7gm73y+/GNXY3dtmZuZ3isUyovjvPaIoIoogSQKWNoHRoIpV8eIN"
    "eydli+I5aZFNzp1M4HAlmbu/yPR0hnhPmGQygbV0iCGI/GVIHB08oSsW5VV3kJubRVp93hEZ"
    "m8vdrBRZXfmTgaEUo6Mp4vEIDx6tcn3qFiwtYI92YyoKx7xO1LrK4/UVaLjQWp1W0RSsiBjo"
    "TQ1d0wBob/dwZvg4kUaFHyv9ZHZdnBofIp1O0SK3UK/XEdExsSDrlUZJUdxKX38Qu82OCQiq"
    "ijY7ywvDg8ygc/p0Hx9+9xCX3uD9NwfpTaSwz5Wg9qQuV/fV27WmFP31zgIDx910dSfQ/7iH"
    "GQhATw/Zn5bYuAtScBCHCyY+nce1v0YzOEo1X70vl3bUr7ZWG29kPaIYjxroC1lMmx0plQJg"
    "uQAVCWQBTMB0J/GGB9iam6fatl8Ssdpv3ZnOX92T+znIG+SWVllzODBNqFZVVA0ME3QTDOPp"
    "yFEdyqaE5GZTovplsylOzuZWzbFAKhAVbA20UhGP00M2m2V220Jn1IfXCR1+KCwusvj9vd38"
    "zbV0M3v5m6cuVD4rV/X3pm4/FCMtvshgKOylXCzSaBjMl0WGh31szC2w/nOOhV9cU8V120X2"
    "X17+r40AvWsWjNYLnT22d7rjcjrol1oPtd/cWytyrViIZXbWxc+h/gMrsdoz5G+TaF5TOyGi"
    "EAAAAABJRU5ErkJggg==")
index.append('clock_resume_icon16x16')
catalog['clock_resume_icon16x16'] = clock_resume_icon16x16

#----------------------------------------------------------------------
envelopes_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJYSURBVHjaYvz//z8DJQAggJgY"
    "KAQAAUSxAQABxAIiamt33LO0VJT58+c/w9+/yJgBTIPE//9nZPjw4fv/5uaTFX///un/94+R"
    "4devfwwAAQQ2QFdXQs7aWoaZn58bp01/gaa9ffuFYerUS9r377+TARkIBN8BAghsABcXxz9W"
    "Vhbmy5efMcjJizMwMjEz/PsL0cjGxsDw9s0nhp8/fzCIiAgwBAWpxpuZiUcAw55x06bbnwEC"
    "CGwAyClcXOwM0tICDOfPPWCQV5BlYGRmY2AFyt679xpo2A8GIyNZhpcvfzBoa4uyhIaqs/z8"
    "+R1o6G8ugAACB+KfPwwMP38xMHBycTGoa0gynD1zi+Ed0LlXrzwB+vszg7aOLFAx0KLfjAyv"
    "Xn9juHHzBTj4WFjYGAACCGwASPPnLwwM798DhVm5GAyNVRgOH7zC8PXrdwY9fSWGDx8ZGD4B"
    "5b9+BQXkNwYZGUEGdnZWoKF/GQACiAViADPLp88MDL9/AzlfGRiYmDgYPLwtGJiZGRjeAQ0F"
    "pTVmoMqPn5kYhIUFGDg42Bn+AMPoN9DlAAEENuDevTcvmVm4RX/9AkUXAzB4GZlAmkBR+O8f"
    "CIMM+c/w7t13Bh4uRgaQZSDw4wcDA0AAgQ24cum24ZmTN/i/ff/7X1ycOzkq1q2Mi4sNGv8M"
    "cKygwAfEggyv3jAAXQE04CczA0AAgQ1gZmZ6wcrG+IL1DyMw2pjOP33y4RcrGwsTKHD/gRIU"
    "yBWgRAWkL196BfTqPwZgTDOcPvX4A0AAMVKamQACiOK8ABBAFBsAEGAAz9P7BPaRxLgAAAAA"
    "SUVORK5CYII=")
index.append('envelopes_icon16x16')
catalog['envelopes_icon16x16'] = envelopes_icon16x16

#----------------------------------------------------------------------
markcompleted16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAPDSURBVHjaYvz//z8DJQAggBiR"
    "OYrBNnrWEVpTjY00TQ6cOX33XsHGqXsl/jBxcDBy3Hr6Y5fJI4YbQGV/gBhuK0AAMZ5Q4HAV"
    "NNfL/c8vKrFQkkfpsvcvYVZhJob3vz8zfNpyh2GyQjaDkIIsg+C2lW87+9cm979n2APU941h"
    "OgMT409GTYAAAEEAvv8EFTIcnQeMhh2u5C0AVxAFANnFcQDD7rsA/fb/AC4DPQAJMWIACRUQ"
    "ANoVPRMQtcyDAMnh1wFwBAwzKx2W4abv/gIAQQC+/wDWOgPw/c6H/5K5zv/x3f//7t78/9Tb"
    "3v/b2+b/ttLE/+Pq8f/18///7vf//6WSlf4wUQ26buVB+jV5IdcAAQBNAgBBAL7/Aznz/yMv"
    "LwYy4hMqAPEBGAAZ/QAAEBwSAA4KDgCV2YgAxei3ACgIOgDW6NMAyRjAAUFhFiOvgdXbzo/g"
    "gwAAAOwCAEEAvv8DIfoA7ysK8V0bQjADwegVACwPEAD2/foAAAYDAA38GADkAcQAzOjGAO8J"
    "zgAdHPMAv64DAMWEzaz8/vvSAAAA3wKIqfcfw6njp16fl/11heHLhwcMz5mZGL68ZmD49Pc3"
    "g5qIEQPzazUGXT1GBk6eb2DNJbYTGALzOlwZGBm/Ag14BxBAzEDin9JnBj4nK1a3N7yyQBvV"
    "GMQZWBl+cjAxMLOzMvz7xs+w4vw8hteXRBlCNOoZfjILMEjpmwptPXj69b83d88CBBDIAAaB"
    "fwyvNPm+p+g5ybFuuyXJoKEgx/Dl4x+GX5xMDEK8Egzf7wkzuOhGM4jLyjN8+PqLgZWXieHr"
    "igbxW28+7gQIICaQAVsZGB4eOPTuoAjbfQbmD+cZXjIxAr3zj+Hrb0aGr38ZGbzcAhmklWQZ"
    "WLgYGJg4uBm2n7jJ0JrIoaLGzhAOEEDM0AT1T/YTA6OzI2fQL3Y2his/DBmUJISBSY6R4Qcz"
    "I8Pd538YLtx5yXDp8iWGN4/3MVjxHmFwNOVgOL/tHidAALHAkuTVfwzbj25/9NKhWF98+Yrz"
    "DD/1/jG8fX2fgZvlKYMi9x0GD967DFoSTxg4f75hYHj7heHK5A8MWx59vwwQQMh5gblRimlm"
    "3RG75BNX+Rie3XrEoML5noH10+f/zx58/37h1q/PFx/+fXvrNcOrp18Znr/+y/D4JwPDaYAA"
    "QslMygwMOnaazDM5uP4LXnr07+X99wxP3/5heA5U+BwoDYxcho9A/B2If0DpjwABxIiWO1mB"
    "WByI5YAYGGQMP6EKYRjEB6Yyht/QXPkLIMAAglZq9MCCdyoAAAAASUVORK5CYII=")
index.append('markcompleted16x16')
catalog['markcompleted16x16'] = markcompleted16x16

#----------------------------------------------------------------------
redo32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAXhSURBVHjaYvz//z8DIyMjAyXA"
    "3th4P9AEhV8MDAnHzp49SIpegABiYqAQWBkb2/MzMDiUBskoCDEwHLAzNs4nRT9AAFHsAJgh"
    "XqG+DJNnNDJoivJOAIbIeqDD+InRCxBAVHEAGPz/x6Bg7MHQu2A6g7eZegAXA8MFoCP0CWkD"
    "CCDqOeDvDwaGXx8YuAVEGUq7OxjSo90VBCCOwBslAAHESG4iBBocz8rAEMAJjH8eBgaB1QvC"
    "GRjkA0FGQjAQXTt5mKG9fgrDwx//F/xlYCgAJtCP6OYABBDJDgBaXA+0sEBRTFDAylKXwcRS"
    "n0FLXwNoIcxiVPrr++cM3Q1TGY5cf3nhOySXXEQ2DyCAiHYAKLUD43WBviyfQlKcHYOWiQnU"
    "EiYkS6FsZHEQ/vebYdH0FQwrt1748AESEgth5gIEEFEOAMUjMItNyAzTZPDytWRgYOGCGs6E"
    "ZDETwgEMSGwk8dNHzjBMmLyV4fH3fwsOnz2bCDIbIIAIOsDW2Hi+PPu/hOYCQwYFdXU0y5At"
    "YUbiM+OQZ2R4/eIlQ2f7eoazjz9fABZcDgABhNcBQMvrgZY3NGcrMSioKKAZDGUzgUKBGUJj"
    "k0Pm//8NpN8zfOVlYyhM2c5w+PqTWIAAYsET7P5SbEDLk/kZFKSBQf7jLdRQZiRDYRajs2Hq"
    "mEEFBDCLfmFg+A3EorwMD37yM0wq3Mdw4/GHs89fvXoAEEBYQwBUivExMDyocH0tYK8DDCgO"
    "QQYGVl4gBqZ/Zjag4cAMyMKNcAyIBqWF/38h7P9/IBb/+wnxNSsHA4OWAcPBQ68YpvceZ7jz"
    "+uOss9euTQYqugsQQFhDAGjFBCelLwL2mp+BBgCV/HwP9AGQzcQCsQBEM7Kg8pmQ+IxIfFFJ"
    "BgZ1PYbpLUcZNm6++fnhx48NN+7e3Qi05gGo+AIIIBZs2Q3oz4QE29fY44YLWMRLAxOjlArQ"
    "cBmE+MmdDAwf3qCqVdFieM0kytAZt4nh9PVXN289fVry4vXrU0CZVzAlAAGE4QBg4BZ4aXwA"
    "RtdvVGluoMU6DgwMinoMpw88YDiz/g7DzVsnGe7fegsKMYbVzewQH4MAL1CtlinD6bMfGPob"
    "1jLceft52ekrV7qAMreA+DuyfQABxILme3lgTAeEmKL5Xt2CgcHYk+Hg5psMS/OWMdx/+unZ"
    "x9+/D3z+/v0sOxubuCIXVxlcrZwaA4OSDsOiSWcYViy7/PnRly+9V2/dWg6Kb1CQo3sYIIDQ"
    "QyDBTvETgygfku8tgxhec6oyTE3fzHD49NNnL79+nQU0cBU0GD8AHW0DpMsYWIDhoGfD8JVL"
    "jKE7ZyfDwVNPbt5//brx4dOnB4Dyz3HlNoAAQnEAMF0HmCl8QgqSEIYHf+QYaqPWMtx692Xz"
    "zQcPuj5+/nwZKINRqTA4BDNcu/SGobV4OcMdhNoLQJkv+EpZgABiQQ5+YCwamChCHaBiwvDg"
    "rwJDbfoWhqsv3/eev3FjLjQOMYLxHxCvmX+ZYf6sswwPP39ugIYQVrXoACCAGEDlAKx6LXbT"
    "+v9/Dtv//6uk/3950/m/MNLtv5mu7iagtCakuMMO7IyMNtnq65+Rl5aOBHJlSKnWAQIIHgJA"
    "0w30pKChZRPJsHbBNYYz116ePXX5chUh3xw6dw7UEBAG4m+EghwdAAQQvEUECn55YWAOkVBh"
    "ePBJkGHF8isMt589a8CVetHbQ9BE+YXUhg1AADEh5X8HUV5gsavtwLCg7zjDq+/fZwELjQvo"
    "+ZbaACCAUNqEWmpswMpCmuHUmWef7zx+vBhf9qEWAAggJljxyw5iyOsxbAcG/VtgIQPMQo+J"
    "SsUUAoAAgoeAKA+w5lKzZDgMLGZfvnu3nB6+BwGAAEI4QJiZ4fSVvwyvP/14Biy9QHH/ix4O"
    "AAggRBoQkWW4fvYZw6c/f84CeW8Z6AQAAgglEV44+5zh07dvoLL7M70cABBA8ILo7s23DB++"
    "/GIANpNO0TrrIQOAAIIXxca6upHkFKWUAoAAAwAf8grGXXN2BAAAAABJRU5ErkJggg==")
index.append('redo32x32')
catalog['redo32x32'] = redo32x32

#----------------------------------------------------------------------
folder_green_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAG6SURBVHjaYmRgYGDSmS/0lwEN"
    "XEl8pw2kbgPxPyDGkIcBgABiAGr+//XfZwwMEgfKOwExFxCzQDEzEDMi6wcIIJAgw/Vf5zEM"
    "Phn/kMGcQX4vIZcBBBDYgM//P4Al//3/x8DEyASmT/zYCzYEHQANvQo0xBHIPAoyACCAwAa8"
    "//uG4dGfOwxz1s9gIAYAvbcfZghAAEEM+PcGrBmbjbgA0CUgQywAAghuAAic/3kM4hVgoP9l"
    "+MPw+/9vIP7F8JvhJ8Ov/z+B4v8Ytn5bxuDKGQwzxxUggMAGfPz3lgEWFkxA+Of/H5B2sGaI"
    "xr9gNkjs+/+vDF/+fYQZIAwQQFAD3kEM+PcBbAsoEGGuABkAM+gPlP/p3weYAYwAAYRmwEeG"
    "/0Ct/4EQpAlkxM//P4Au+g027A/USyBXwABAAEENeM+ACIv/YDayN0CuAfFhLkI2ACCAwAZ8"
    "+/+FAdklIENgGkCu+Atk/wOnGwaw2I//32H6fwAEEChZagHj9SoDiQAYhauA1HqAAAIZwArE"
    "hkDsDMRCROoHOeECEO8FCCBYxgDR7FDDiAGg3PkLFFQAAcT4//9/BkoAQIABAL8lCnGIvCO+"
    "AAAAAElFTkSuQmCC")
index.append('folder_green_icon16x16')
catalog['folder_green_icon16x16'] = folder_green_icon16x16

#----------------------------------------------------------------------
markuncompleted22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAVYSURBVHjaYvz//z8DLQBAALGA"
    "iAuKjAwsTAwMbIwMDPy/GBhMnQPZVLVYW9xczOJZORjZl63dvO55w4H6p8oMLAxsDGIMfxje"
    "/2dnuMXIDNRzjoHBjIeB4egXhKEgxwIEEAuExcDw7xuDNgMHgyIXD7O4E9drP9lQJz8Wwf8M"
    "3/98Y7BMME78+OWDzyceFVZWeQkezk+vvjJu2LXk47MPJUDdP+x4GRgCBBgYap8BOf8ghgME"
    "ENjg30wcSQrVqTMZtLVZjrwWZFB69YJh1bFlDDL6AgwvvnwAOvAXw22lh6JcX3MYTMVdGDik"
    "X/FH6Wtkz4lpf/yP4Xdn+3MGBlV2oOOQQhUggMDOvmTJe/3/u63/Nx8/8H/Svq//bz7/979y"
    "Xft/4RbJ/yxNXP+5SkX+25T5/r9x99F/ENh+9sf/fXtW/39SaPSEj4GBlwUtfEFmAgQQmLhm"
    "wvDz/8GK/9eubv0/Z/eT/zBw6+Hd/2eun/l/9/79/39+QMR+/f3///uf///7ll34/39Lxv9U"
    "YaYsZiwGAwQQE4jx5BnDtr+nDzKoS/xlePnsAcPX3xAFqnJKDMYaxgxKCgoMTECv/vrLwPAH"
    "GIZMQF3S8uoMW75oMxT7yeUDXcyOnioAAghs8MuPDAtenb7IwPTjFYMs93OGi/d/IWwH4r9A"
    "4vdfSBiCDP/wnYHBUp+D4cQ7UwZFe201bx6GEHSDAQIIbPDXPwx77l/9do/hziUGE/mvDOeu"
    "P4MrABkGwiAL/gCJnyBXAzEH0JnqOgYM234ZMhS7CBYDUypKiAAEEBPUVV/vPmXY9ufSKWBw"
    "/GJ4/+IOPDhgBv8FBsGP3xAa5IMvQFeb67MznPzuxqBpI29oz87giWwwQAAxwRgvPjKse3rh"
    "OgMTwwcGGY77DOfv/IRGBMSw73+AwfEPEsYwzAp0pqq+BcPe/xYMBZYspcgGAwQQE4z4/o/h"
    "+NWLHx8wvLzLYCr3juHMlSdgBaAwBRv6F+JSsIuh+OsPBgZjXVaG00xhDPrmYnYGzAy2MIMB"
    "AggSxv/AwfHj+g2Gzb9uX2PQkv3B8PLhNYanXxCuhBv6H4JB4Q1OIUD9Ksa2DCc43RjS9RiK"
    "YQYDBBDY4LfQ8Hz1hWH1vXO3GJg4vzEocFxnOHfzJwMzC8TAP0gWgCISJM4GTGR/gcGhocXC"
    "cOBvNIO5maiXCgODCcgsgAACZxpGWApgYDh55fjrOxopr1Ws5L4zbLj8iMHRWJWBCWQI0AmM"
    "IG8BI+3VOwaGh09/MTx6+JHhxaPXDN/fvWAQ/PuQQc1HkTXp4LtqoFGBAAEEz43/IBb8unPx"
    "38afjx4WaykpMiw4c5rh2DVVhp/A5HDn7geGZw9fMnx59YSB7dcTBgnOVwwa4h8ZfDV+MShI"
    "MzIIinEAvWLLoKLwyhBkHkAAscDCA1ooMbz/zLD27tl7xVrR6gzpRlsYFi58ysDD8pVBWfQD"
    "g4P8bwZ5SyYGIWFWBgZOULIFxuiPbwwMnz8wMDx+w8Bw9RnD5uOPt4UCZQACCGzwVyDmRuS0"
    "M1f2PbuhlcCgoaavxtBq8BSYYYGGsACt/g0Mh6/vgVkPaMjdNwxfn7xjeP3oC9AnP38/fPL3"
    "+elbDOc3fGToApkDEEBgg68AsT3UYKDW33dO/ln3/d7eKk4VeWB+/8Tw5eZbhrf3PzI8vfPt"
    "9/07v1/efvj34Z3n/27ee/f/2tPfDNffAq0BOg7oAgZ4cQ8QQIygkoiPkZHBAmqLDhADs4YU"
    "mzr7fFl5JoWbd37fvPXi7+WH3/5ffsnAcPMTMN6ASt7hq5ZAZgIEECOt6jyAAGJioBEACDAA"
    "y3FaM7Cl764AAAAASUVORK5CYII=")
index.append('markuncompleted22x22')
catalog['markuncompleted22x22'] = markuncompleted22x22

#----------------------------------------------------------------------
trafficlight_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAKWSURBVHjaYvz//z8DMpg0adJT"
    "QwNDiY9fv/77DeT//fePgfHfX0Zedk7GBw/u/6qoKK969+5dP0w9QACxMKABTQ0NsRfv3jJd"
    "On6CiffJY4b/P34wPBUUZBBVVWMw0tbl4OPjcwUqgxsAEEBM6AZ8+fnr/4OrVxn4Lp1niHS0"
    "Z0ji42MwXr6M4cmFcwzvgIb9//f/N7J6gADCcMGvf/8ZmH79YLBi+s0ge+8hA8OVawwu374z"
    "3Pvzg+EHIxMDupcBAgjDAIZfvxl+C4sxnPryiUF11iQGtk9fGY5oKjA85RdiUAC5gAHVAIAA"
    "wjDg/5/fDCxANRdU9BkElYQZuP7+YFj/VYiBk5WbgRkoh+YABoAAwjDg99//DD+//2IwF/nG"
    "EJWWx8DC+5CBZcZihu1PZRl+/gVaAIwVZAAQQBiB+PfvH4bfP/4w/OX7zcAk85aBQewBA5vs"
    "T4Yf338z/P3zD80DDAwAAYThgn9//zJwcLAynLjMxvB2yhwGDq5vDOf2STBw8zMyMDL8xghE"
    "gADCMOAP0JZ/P38wfBP+w8BlmMDwj5OB4cOT9Qy8z4Au+PUPwwCAAMKMBaAfv33/wcCo+J/B"
    "z9GC4Q/jW4Zz7/4xfFn0HRjA/zGUAwQQlkD8w8AIVPfl9GeGpnX1DIz8Pxmer/vIIPmLH5hG"
    "fmMYABBAGAaws3MwqWirM/wAeuPngT/AZPGbwZBFhUHYWISBj58f5AV2ZPUAAYQRC2vWrLx+"
    "4/qN399+//whJir2T0lBieEX859fH95//rV16+afHz58uI2sHiCAwIGCjIGAA4gtgFgxMDDg"
    "am5eLkiwGIgNoeIiyOoBAggzEBkYfgDxCWCuY3jx4uXO9+8/CAH5Z4D4PBa1DAABBgAbiBnq"
    "gVv6KwAAAABJRU5ErkJggg==")
index.append('trafficlight_icon16x16')
catalog['trafficlight_icon16x16'] = trafficlight_icon16x16

#----------------------------------------------------------------------
delete32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAafSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsyBxG/UR1QQm5fjFpGROG////UMsSRhBmYmZ58/L5zddP7hf/vzD3FEwOIIBQHMAt"
    "KNbSWZ/t6W8qxvDrNwPDf1JsAIJ//xA0SC+UywCKZVagTSfufhMtrOrrAAo5wbQCBBCKAwTE"
    "xNVt1MUYxFiBHFbKfP3vP8QBIMf8BRkHjGwTJS4GCVk5WWR1AAGE4oD///7//fX3P8JLpFgG"
    "xSDfgoz4D7X8HzQE/jIzMPz4CRKDhRMEAAQQCyW+/A81HO5bqAMYoPz/jCBPIclh8RpAAFHm"
    "ACTDQZgBGMxMQPwHxP+FcBBYnhHCRk9XAAFEdDZ8++gRw+UVKxg+v3+PGsdQNiMwiH99+8pw"
    "a9kqhtcXLzMws0Md+B8RUiDvo1sIEEBEOeDxnTsMF4KDGf6npTGcTkxk+PL1CwMTI9QRQEuY"
    "gOH489dPhgvFJQxME3sYXhTkMTzed4CBiQuhBuSAn8Cc9eMvqtkAAUTQAS+Blt8JDGQw19Fh"
    "0DtzhkEFGAJHAwIZPnz4wMAE9DUDKHF9/8FwJjmVQerOLQbtBQsZjGOiGb421DO82H+UgZED"
    "EkqMQAc/+8LA8PQTqvkAAUTQAXdmz2bQvn2bgScigoHh1SsGucpKBo137xhOBAYzfP7yheHP"
    "3z8MpxKTGKRvXGNQKSxkYHjxgoFVRYVBS1WZ4cOSxQy/ofEPT6BoACCACDpACRjkt+XkGL7l"
    "5zMwnDvHwPDkCYN8aiqD+svnDMd8/BgOBQUzSJ87w6AeFwf04jMGhmvXGH6vWM5w++ZtBu7w"
    "KAZGkMV/kdIBGgAIIIK5QFJDg+HP1m0Mp3x9GAw7Oxn4Q0IYGDg5GRR9fBj+rl7F8BMY/JpR"
    "URDLf/9m+H7nNsPVR08YWDt7GcTc7Bh+fYTmBGiCRAcAAcRCKJ+DNMmqqjAwbtnKcAZoqd78"
    "+QyidnYMDOzsDCqGhhDT798H0x+BjrgKdBDP5JkM4g6WDL8/IeUUHHYABBALPtthWe0vkJBU"
    "UWZg2rOH4YJ/AIPW7t0M0qqqkJQFzdxvfnxnuMrNwyAydwGDiKkRxOfQsgJeKv7HLIgAAgin"
    "A/4iuRxkCCifs7JzMDDy8jJ8+fWL4dmtW8AcwMzwH1jyMAKj5DswRBiERRmYeXjADgZnPUYk"
    "M/5jr9wAAogJW7AzIBWrIMuZgRXT6ydPGa4FBTKoHdzPwA2U/wSM79c/fjC8ATrmA8gnXFwM"
    "qh/fMXxIiGd4e+4yA5MgUv3wDxGa6AAggPA6AISZgWH08vEThrtAyzWPHGbghIYOC1TtX6Ar"
    "WYBR8R/oZWYeXga1/78ZfuVlMLw9choYWtDa8D8iFNABQABhZsN/iDocZMsrYAJ7DCwF9U6f"
    "ZuCAWg4qf4CFGsNrYzOGN5Y2DD8+fmRgBjoCVNExAh2hwcvJwFxdyvDxyFmG/5xQh/7D7gCA"
    "AGJCb1f8RwuJR9NnMWicPgUO9j9Qy38A8XMzSwblzdsYNHbsYHjn6sXw9c1rYEXEzPDv718G"
    "Jh4+BjVONoZfC+cw/PkHLYhwhABAAKE44O13RFH5F1rTiYWFMtyRV2AAVuUMwGTGAFTC8NTS"
    "lkF+zVoGLhFhYDrkYJBdtJDhta0Tw7dXzxlYWVkZGIE54v47YMrw8oXUGX9wpwGAAEJxwE+g"
    "wi+/EWngN9Cr4iZGDLxAyy4pKDG8BIWInQODzLr1DNwykgy/vkLUsPLwM4gtWMTw3MKB4d3t"
    "mwy3nz5jeFdcw8Dj78Xw5ytSLsASAgABxILReGREFEAg+s83BgYJoCOY129keLF1J4N0TAQD"
    "J9Dnvz4hitffQEtYOHgZBCdNYXg6ZwEDm5Exg6CTHcOf90jtBRxpACCAMMqB//9R8y6I/gOs"
    "xfi0dRj4DHQY/v6AWPgfqU0AZgPjholTkEGotJABmBEY/nxCagmBopMZexQABBALrvYdct4F"
    "W/ADYsl/5DYgepMM1JL+hdZE+4fWXEMDAAGEEQUoBqKVYv+RDEY2FIOPbjlSlKIDgABCDQFg"
    "PmJjZWRgByZNNmYkw2DpAtYKYkLEK4rFjEgh8w+qDqqGlRnSN2BkYkJJ+AABhOKAv59fPzh2"
    "+52ukqgQw/efiM4FzCcMSM1vlOhCCjnkBAzXB7IIaNOFx78Yfrx/8QrZToAAYkTunDIaJBiz"
    "iqlPFZDVUGX8//cPRvnMiMZnICCOErrMLJ9fPnj6/emVkv/n5+6CCQMEEONA944BAmjAe8cA"
    "AQYAYCApoMU3AcAAAAAASUVORK5CYII=")
index.append('delete32x32')
catalog['delete32x32'] = delete32x32

#----------------------------------------------------------------------
clock_stop_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9sCDBQWGwnp3QMAAAMW"
    "SURBVDjLbZPfT5t1FMY/74/1hba0fUu70ZYWaCmTso4EqA6dMZjgsl0sGrdwM2ey+Bd4459g"
    "1BtjTLwzUXczb3axKVnwYnFbUKKTTSLIr46NMYot0L5reW3ffr9eGKcQT3Kuznk+OXnyHIWD"
    "9cq113rS5oV0l+/ljpDHr6CwWapayw8r0yvLO5fZcyaZfr35z7ryTDj4dTDaa35w9nTvOyPJ"
    "Ntr1BnWrghQCwx9g2znE7MMqV68vXV5fLL3H/YkNAA2A9FeB1HOhK5cuDp3vlEViyhYnjsVw"
    "qo9p1aoM98exHi2BbTM4mjn+ZKP8fNEev87O1ZpG/PPWUKTtk/NvnXhzwOcw2t0g0mEQi0cJ"
    "hU2isSMEAl40rUYsoNGoaXi7OrvyS4Xup874pKbrp8ZG3xj5MKhb6um+PY4PZ2gPhQEwDBcu"
    "lwspBQEzRPhwCE+zwE/zJQ6FIgNr08t3VfOI/6InHNZfSjhsbm1w69bPVCpPUdV/7VFVFVUF"
    "TVNwtSrkojaGGSTYGbyku8zASZcuOXUyjdeXYfbe70xNTZNMdZLJpDGsHYSi8ofQKG/v0t2V"
    "4Iw/yjePKrSEgsM6bp+/Ua2wuvKAgcEsuVyWZDLO/d9WuTZ5ExYX8CR6kKbJ4WAb9p8262sr"
    "UPfhtLQZulQMVATNhkPTcQBobw8wNnSUH7/9jtlPPyKQTIIQ7AIzAFJyQdV4Mb/m15vVumWa"
    "frOvP4rH7UECim3j3LnDC0PHKPl85DIZRKOxL2+qrkO5rOi1ov3DXkNL3J5ZYOCon+6eNM1f"
    "7iIjEUil2Ftfp3DjBkKIfQBFUaht70jdemJ/+Xi1fm4+oKrJhKC5MI90e9CyWQAcy8La3UUe"
    "SLwC1EGqGJ6bM1OFK1t6P9sFwdLiKnmvFymhVrNBymeC/zaAClKlOFatlRrvfvHZ5u3vi3EW"
    "XBE2HmxRLlr8em/uwMPsv2D/LDbX4Y6ZH0+cMyfO5OqEDQtRr1B5NcuwEIj/AcxBcz+8N+9C"
    "tJztSLnf7knqI9Gw1jL+vs/fJxxFSikVpFT42w4FyIPzF3AeMm8RA/zJAAAAAElFTkSuQmCC")
index.append('clock_stop_icon16x16')
catalog['clock_stop_icon16x16'] = clock_stop_icon16x16

#----------------------------------------------------------------------
palette_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQDSURBVHjaYsznYmD4+5OBgYGR"
    "gYGJhYHhzy8Ghu//GISBXDNOAfYIQWkOG2FxZhFerq+svz///HH3JsOley8Yev4yMGzlYGD4"
    "DxBAQC0I8PsnA+e//wxhavqyWZbOCmbalqIMfPJMDGxs3xi+fWFhuPVGgtP4xWf7y1uP2u/Y"
    "+qj83z+GboAAghvw+w+DvpiCWI9HvruLtr0Iw67ftxhuvnjCEPVJkOH8GxOGjkcpDB9YpBgk"
    "hL8wVGV0MfDxLGlbsfzpc4AAAhvw5y+DmYad4bKwzihlKekPDLOvnGIoZbnM8P37C4Zbj0QY"
    "ghiVGF6++MzwXYmN4fsPXobPn/8zuEUpMx878rEIIIBYfv9iEJUzUJkc2hCrLM54k+Hv/a8M"
    "qt/YGeweMQON5mMw5pBjsNJ5zbBZsoJhxVMHhnff+RnUpO4y8AnzMyhq8coDBBALBxd7klWw"
    "mZmY8jaGHy++MPx9K8BgxibAsNrIi+H/rz8MHHxAg/79ZBBW5WRIYtnO8P/TNwZOYVmG/wws"
    "DOxcDEwAAcSkaGIUo2SozMDM9piBW4KR4eu7Twyf/jAzsMvwM3BI8TAwgDzJzsJwZ8tDhqsF"
    "5xg+lVxmeL78LsNPht8MH978+gUQQCyyhmpSArKSDI83/mNg/PeDgUVaj0FMRxoYqsD4/AuK"
    "XmaG34z/GV5tvsugfv8jKLYZLq26wcAj+5fh+eOfdwECiIWF+R8Tr6wwA/tvTwZmfmEGJjEh"
    "BsbvQM2fXjMwMP8DOv8PAyszM4NcpCbDvXffGT7/+MfA4SLCcPPqG4Y3T74tBggglo9PHn/5"
    "/59NgE1GkOHV4zcMT/efYZCzDGAQFmFiYHj/COgCIP39N4OMrSwDv74I0LB/DO8/PGPYmnf2"
    "EsOff2sBAojl8YXLJ98+/yDz/sEDhvNzN75je/dhCdd/7ijhME8Rhv8PIIkE6AWGr38ZePnY"
    "GT59ec6wbtLV76/ufa/mYWZ4CRBALN8/vF2wvaUt+O/713c4Pn0IY+Zk+MPIxBINTtP//kIw"
    "UD8DBzPD5/cvGLbMv8Rw99j7HmZGhi0gYYAAYgKGyt77R25nfH/5wY+BleH8568MGsyCUvzg"
    "DPIbmkk4WRjevn/FsHn+eYbre19MYGZiaAD5DAQAAggUSd+BnJkgzo8fDAy/vgHD/+uX/wwc"
    "bAwMrEBVTH8Z7t+4z3Bg6TmGp6ffdjH9Z6hlYGL4B8sCAAEEzwv/ge75+QOcrA9e3b79iZju"
    "X8W/H58zXDp4j+HKrnvvvz75ls/CxLD4DyMDCgAIIMZ6YJQDY4lBSAAY7aBY+wem09hFufr/"
    "MzIxfXn+dSfjr/9NjGxM5/7//sfwAxg0X78xMHz5DAkagAADAM+FexkTkAFpAAAAAElFTkSu"
    "QmCC")
index.append('palette_icon16x16')
catalog['palette_icon16x16'] = palette_icon16x16

#----------------------------------------------------------------------
prev16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAKbSURBVHjaYvz//z8DJQAggFhg"
    "DEajmQwMP34zMLADhViZGRi+AdmMQAl2VgaGf/8YGN5/9+IX5e4XleRTe/j619lfx2JNQPoA"
    "AogJv/lAE/7+EwJqniCnKLhpZqerWmWRPQMHM7s2TAVAAOE34O8/d6Zvv/YG+mvmz50ayGzg"
    "qMDw7j8Tw19mNrgSgABiwbQUaOu//2wMv/+2iYpw5RRnWbJbOGgw3PvKxHD76j+GIyffMPxk"
    "YPkIUw4QQCwYmn/8MWP4/We6s4OSUVamFcMXLmGGtdcYGD69+cRwYs8VhpvXX59l4GQvhWkB"
    "CCBkAzgZvv2q5BfiyswptBIxddRl2HGLheH8/X8M357cZ7h/7ub3L2+/T2Pg525iYGH6BNME"
    "EEAIA/783WVur2KTmWbF8OifMEP+SmDAf/jO8P/uGYbPr96cZ2BmyWLgZD3BwACKdkTUAwQQ"
    "3AAmRkZOTj4OhoNP2Bl23mVgeP4OKPbzPwM/EyMDHzfHj09/uT4ycHAzMHABMTs33ACAAILH"
    "wj8WJrsDO25MXDJx61fO93cZNGT+MXDyczHw61kxWDvoWEqLcxwDhk0t0HIe5GADCCBENP5n"
    "+AZ0YuHv378C7x07f/P1hQsMEtzfGN7/ZWI4/U2WQc5IV0BFQ6SJieH/NmAs6cK0AQQQajr4"
    "D4QszLv/s7Gbvnnyes7jMxf/8/x8A4yd/wznnnMx/BJRZhCUErdlYGLaCtMCEEDYExIj42eg"
    "otSfP/5GPb3x4Pa/t08ZBLn+MLz6wsjwmYGXgZGNQxSmFCCA8KdERsYV/5nYXN69/Dz/w9OX"
    "DDxMwCTEwsjwn5kDrgQggJgIZjcmhkdAXUk/vv+LeP/669M/v34zMLKy3odJAwQQI6XZGSDA"
    "ABOQ3NieFWOEAAAAAElFTkSuQmCC")
index.append('prev16x16')
catalog['prev16x16'] = prev16x16

#----------------------------------------------------------------------
cross_red_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAP/SURBVHjaYiyXlGT4/vu3vIag"
    "4BKh//8NL3/7lvL/378VbP/+MbD+/8/ABKQZgfjv378MzAwM3hri4st//vt39+G9e1GcTEzX"
    "AQKI2YKHR16Oh2dJpLm5jZmmJtu3O3e8n/7794Dx///LLAwMDECa4T8I//vnoy8ktMw7NZVP"
    "Xl9f4vWdOxYfP3w4ChBAzDGKirtiTEwshfPzGRhCQxmUnjxh/XfxoudTJqZ7TAwMV0AGAK33"
    "MmZnX+YUFcXHmpfHwK6uziDPyyv1/sULB4AAYhHm4NAVEhZmYODmZmD49ImBJSuLwe7LF86/"
    "q1fPPs/N/Quo+ZPunz8rbIKDednj4hgY3r0DGcjAIy/PICQmpgYQQMzq///f//nhg4fykyds"
    "jBISQDczMrAYGTFIPX/O9uvixSChX78iHNzcuHiBBoM0Mnz4wPD/1CmGM0uW/Lx06FAhQAAx"
    "27CzX3706dN9ptevveRevWJlAhkCVMhiYMAg/PQpk4y4OIsAUDMjVPO/M2cYLmzc+OfE0aNF"
    "///8mQYQQMx2PDwM/3//vvLkx48Hf1688JJ79oyFWUQEHHgcGhoMXPr6DEygQHz7luHvyZMM"
    "Z7Zv/3PqwoWCf3//TmUBuhYggJhtgX5nBIY2w58/l1/8/Hn15+vXQZJv3zKzgQxhYmJg/POH"
    "4f/HjwzfgZrP7t//9/ydO0n//v2bwwyKIaABAAHEwv/1KwNQgOEfUPF/NrYP/7i4/n0GOvPL"
    "ixcMYO8AwV+gGlAA/2Vh+Qe07AMbUAyEQekEIICYXVlZGf6CbOHg8DGWlV1j9fEj91egcz++"
    "fs3w9v59ho9A/BvodyY2NgY5GRlmFkHBgNevXj1ggKYTgABiAiYQhv/s7F4WampL7T594v0F"
    "tPk/SAKI+YFOFABqZObgYPgPdOFPoCv05OQ4jaytZzMyM0eC1AEEEAsjO7ufiabmEru3b3m/"
    "P3rE8JsBAniALnvEyvrvDyPjPyVmZpY/QOf+BeJfwHSgq6LCCfTu7JsHD7ICBBCLhpbWEvfv"
    "33l/3r7N8AtqMw8XF8MdTs6fF79/zwDG0EeWv38Xq3BwcP8FuugPUP73s2cMJiYm3MBwmwkQ"
    "QEw/v3599AXoTCagjaDY4BIQYHgoJPTn3LdvxX///VsAdOb6q+/fpz76+vU7JzDGQFEKCo8f"
    "v34xfH///gVAADGbffp08BUPj7mCmJgULzMzw2V+/j9nXr0qAAbsVGYgHxzFf/9eef3p0wNW"
    "Li5PGTk51v/AKD5+9OiNJ+fPxwIEELMnG9vrD2/eHP7CyWn5hotL+PLjx/l/gCmMCZQGgE5m"
    "AEUVMKBBof7p/fsHf5iYPB49ePDg4fXr8cyMjCcBAgwAkC29pEiNXvgAAAAASUVORK5CYII=")
index.append('cross_red_icon16x16')
catalog['cross_red_icon16x16'] = cross_red_icon16x16

#----------------------------------------------------------------------
sign_warning_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAM7SURBVHjaYlzOgB38Z2BQ4xMX"
    "3fqfhYXzzdPnfkChc9jUAQQQ03+IYhT8j4GBmZGZKc+koU7FrKVRmpGVtfgnAwPLb6AcOgYI"
    "IKwG/GFgUJcyN40Xd3RmENMzYpAzMw7+wcBgCjSE4RcaBgggpn8QG+H4LxCzsbKUqISF8jB8"
    "+cbA8Pwlg5qDAzsbO1vpV6DcNzQMEEAoLmCAGGAuaWIYLGlgxMDAzc3AICLCIC0ixqCgoe4B"
    "dJnrfzQLAQIIxQVABSwsLMxpqj6+fIwc3Awf+PkZ3snJMjCwczDoqqhwsrOxpgKdzfEHohaM"
    "AQKIiRFIMEJtBwoYyZoahUnqGzAwiIsxNPT3M5R3dTH81dJikObiZlCRlfH9wsBgCQwPhp9Q"
    "DBBATLDoAIU8BxdnrrqnFw/IdgYZGYaH9+8z3L12jYHJzIyBUUCQwZiPn4OHg73wO9BNsEAE"
    "CCB4GID8Lq2rGSamIM/AICwE8gyDqqIig5KsLAMjJycDg5UVg8zXrwxqggLewOhzgnkBIICY"
    "/kDjk5WHp1CB+x/bj71rGf7Ky4FdJS4uziAtLQ1JWC7ODA8FuRnEGX4ycXNx5QG9wgR0CQNA"
    "ADFB491RRkXai//tY4bPKloMvwWFwJoMDAwYzM3NIV4UEWX4FpvAwPDlC4MyD4cjUI8PKAwA"
    "AogJ6HRWDj6+TCUeJq7ffMIMf0MTGX7/+s3w/98/BmlJSQY5oBdA7J+/fzMI+AUy/FHWZJD8"
    "/o2dn4M9G2gAD0AAgbxgKyMvGcjz5TXDe+8whi+yygw//v5j+MvExNA+dSpDY28vw28gGxRg"
    "fyWlGDjjga749YdBkZXFDeh6V4AAYuEREixWEGBm+XX/A8OvV88Z/u/aBPTwP4ZvnFwMmaZG"
    "DH+Bhj3evpXh+48fDD+YmBm+Pn7M8B7obAmgq4RZWQsAAohxq7riN0slXs5vL14w/Pn5m+HX"
    "f2aG33/+Mfz6/ReYfrgZ/gCd+P7DZ4Y/QE1/gJr+/PwFdA0jAxfQsPP//v0ACCDGOVwcu4Ql"
    "xRyBTv7759cvhr9Av4IUgmz+8/cPw++//xn+gvj/QPRfhr+MTMBoZwTK/WV+9vvPCYAAAwBs"
    "ZCJJWgpZCQAAAABJRU5ErkJggg==")
index.append('sign_warning_icon16x16')
catalog['sign_warning_icon16x16'] = sign_warning_icon16x16

#----------------------------------------------------------------------
person_photo_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJ+SURBVHjaYvz//z8DJQAggJgY"
    "KAQAAcQCY0yZMuW5vp6eMAPD//9///5jALnsLwMzw79//xgY/v0G8qFif/8zbt686cyMGTOs"
    "QPoAAghugK6urqiElCLznTtPGRhZWYF6fjKw/njO8J+BieEnixQDIxD++fWLwcBQheHy5cuS"
    "MH0AAcSCcMy/vxcv32N+/UeDQZD3J4Mq83kGKWVOhv+fHzDcv/+E4fLnUIZnj78xcHE9ZGBm"
    "BjoOCgACCG7A/3//Gbh4OBgkeIQZ9NmPMihpajEwsAF99JGPQUr0BAPLjQsM337ZMLCyfmD4"
    "hxTwAAGEMAAoxsTEyMDJ/INBiO0FA8OXXwwMjE8ZGH68Yvj75z8DN8tNBmY2C7AaBqSIAwgg"
    "JC8AfcvIyMDKwcHw/T8nA+OdIwwXzz9hAAWlrNR3hrcMPgws7KwMjCADkABAAKG6gBnklX8M"
    "Z3/YM4i+vMrw/Mljhl+/vjG8YjZn+C4XyMD45xfEBUgAIIDg6QBoOQNIat/Nfwznv/EzfDGt"
    "ZBD+Y84gyu3HwOjWy/CKl4fh6tt/DEz/Ia6FAYAAQvICIwMLExPD6ydnGET+cDBc/cLEIGjh"
    "CQoYhs9XzjJ8/MzA8PP3RwYOTkkGUNKAAYAAghvw/ccvBmsrTQZNlY8MoIT0BxhR/4DOAif1"
    "v38ZQC5ntBJl4OQRBHrrN9wAgABCCoP/QE1AjQzsDP+B7mRkBOJ//8HO/c/MwvAXKP/v738G"
    "lt9/UcIAIIDgBrABAScnB1AjE0QTSMO//+DABbFhmJOTE5gWWNlg+gACCG7A9u1bj165clkc"
    "lPaRNaDzGYHeOnr0yBWYPoAAYqQ0OwMEGAC73v/etd/FEQAAAABJRU5ErkJggg==")
index.append('person_photo_icon16x16')
catalog['person_photo_icon16x16'] = person_photo_icon16x16

#----------------------------------------------------------------------
exportashtml16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANJSURBVHjaYqyqqjrt4eFhzMDA"
    "8B8IGNABIyMjCp+JiYmBjY2N8cCBA99bWloKAAKIYceOHT/+4wP//v7/+/fv/z9//vz/9evX"
    "/58/f4KFL1++/F9eXv4QQAAOxyAFABiEYXUIu/j/rw4UddaecgihWjOWRwDxSEPQeJ6IyOWr"
    "guyCR61jZuvz8RdALFD9DH///mNgZWZg+AbUePP5T4aPX/8xsLMyMfz89YeBg/kfg6o4CwMX"
    "BxPDt59AbwG9AdIHcglAAMENYGX+z/Dx21+GnVe/MQjzsTPoK3IzMAENfP/1D8Pjt38Zdlz7"
    "wmCr/J9BiJuB4d/ffzADGAACiAkWOH+BAstOfWB494uBwVyVl4Gfk4Hh+buvDO8+/2AQ4PzD"
    "cPPFO4ZZe58BvfWPgYXpP9wAgABiAQYQ2IAnb38ybD3ziEFNQ4Hh4nsGhiunLjBcuXaLgZmV"
    "jeHh8/cMh8/dZRAWlWPwM/BgsBDmY4DFGEAAsfyDMr78YmJ4ePsuw5FDxxnefAxikBJTZrjy"
    "6TnDsX2HGf4ysDBomZoxSArxMzx//RXoB4gLQAAggFj+/4MYwMnKwMDHK8jA/Pobw/tv/xkE"
    "ufgZ5AytGW7f/cSgqqHIIK0gz/Dv7UsGfm4mht9AV8O8ABBALMCIBqUhBgne/wyelqoMemZG"
    "DNpaEgxvPzMw8IvwMURFezHISvAwfP3wmeHTz18MUkJcDH/ALoBYDBBALCBTgOmEgQloSLSV"
    "IMO+e8B4Z/jFwM/HxsDCzMTAp8DP8PfXb4ZX794y2BlwMwjycgGj9i8DyGKQXoAAYoEEBijh"
    "/GOQ4GdmcFf5wXD2/lOGi0+YGD7/ZGKQ4v/NIMX1lcFIlIlBXoofrI4V6nwQAAggFphf/v37"
    "C0wkjAw8nCwMDhqcDPrSPxk+f/3JwAqMMgFePgYWVhag/B8GUKxBXA2hAQKIhZmZmZGFBZjK"
    "uLggBgENZmT4xyDJycUgCVIIFPj95x/DH6AGDg4WsEZOTk4wBmY0JoAAYjl37tx3CwsLjt+/"
    "fzPA0gQ4iv6D0H+woTDnwtggtcDMxPD58+dfAAHEyM3NnSglJZUGSs3/IVGCkpXRszhIDIhZ"
    "Xr9+/fX9+/eTAAIMAK4Kvkpe2FQBAAAAAElFTkSuQmCC")
index.append('exportashtml16x16')
catalog['exportashtml16x16'] = exportashtml16x16

#----------------------------------------------------------------------
computer_handheld_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMESURBVHjaYvz//z8DJQAggFhg"
    "jMmTJ9+2tbVV/v7t2/9///4xMDIyMAARmAYBEMXExMTAw8vHeOrsuV9lpaVFL1++nAYQQHAD"
    "DA0M5dlYORjP373OyMrOxfAPKPYXSPz5B6WB/PefvzGYaSkyWJiasvPy8YUAhaYBBBDcgI+f"
    "Pv1jYuVkUDYwYuARFmf49O0fw9dfDAxfgPjrbwaG338ZGZ6+/MJw58NTBlG2dwxgZwIBQADB"
    "Dfjz6xcDKxsbAzOXKAMLhwADG8hmoOxfVqAk0IC/QCew8XAxcP7/yPD7z1uGP3//ggMPIAAP"
    "ZLQCAAiDwPv/r62XtrE1MogeRBBOxF+QWdxsRdOGHGyDCw5pasnwJkVUqe2dfwQgYt5RAIBB"
    "IIia+181bT4qLhlIYfGqgelBJFfZpgkHziCwMC+Y+JBVlkl/8AQQkguABgAFv/9hBCoAKvwD"
    "8jfEoFdAzc+ALnoFxD9BLgDCf//BQcAAEEAoBvwFavgMtIUNqBDoI4ZvQINeANmPvjEwPADi"
    "+18ZGb6wA134/zfcBQABBDfgJ9CtTP9+MahwvGfgYP0L9Mp3hu///zKIM/1jUGb5xWDOw8Tw"
    "g4uJQfznB2AM/YeHAUAAIVwAtJ4F6D8Jjh8MHGzAAPv2nuHXv59Am/4w/P7/k4GDg4WBl4uL"
    "4SHQPy/+/Afqh3gBIIDgBrCxsTL8+PmTYfP2Ewxs7BxgJ7KxcwJTItDZX78A4/0vwy9gVCsp"
    "KjNwc7AzwDIAQAAxIQxgBpL/GDg5+Bh+A+OPk52H4cvHTwwvnwETjoAoAzMQigoC0wgwObMC"
    "1UJTOANAACES0p+/TEZGxgzS0tJAm34yMDMzM3z69Inh589fDOLiosAw+s3Awc7OICwsxnDx"
    "4gVQWgDZyAAQQHADFi9e9On581fC7EDns7KyMrx79xao+ScwA4Hs+s8gIiLOAMq5IL+fOnUc"
    "ZPhnkD6AAGKEZWegX92AVCwjIxM3kM36799fNmQLgOJ/gMHxE5gF/oLyFRAvAOo9BBBgABw4"
    "UNjZxhsSAAAAAElFTkSuQmCC")
index.append('computer_handheld_icon16x16')
catalog['computer_handheld_icon16x16'] = computer_handheld_icon16x16

#----------------------------------------------------------------------
clock_menu_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9gGARIBM7wsgW4AAANC"
    "SURBVDjLXY/LT1x1AIW/+5g382QQZobhTSvC2JRSoUpjbKLGmDQxys62Se3Kf8C/wZWJSdO4"
    "Mpq6YKUJtcTgoknFYn0UTcMAAzMU5DHMo8wd7p3XvffnokrUszlncb6THImRr0CSUAIuFI+D"
    "pi290T8cfn+4N3CxK+oLSkgclPTqxrb2YHPj6W1q5jwty0JvgBBI/wzgUCPxofDHl98aujEx"
    "4KddbdGsagjbxhUMUTYdLG/rfH0nc/vP9dJHtKw9hECh8z1oWaHB56Oz16+Oz3SLIgnpkKmx"
    "BKa+i0fROTeSpLqTgXqdMxdeeHF/r/JScUe7g0M2FNxve6Ix/6czV6beHQ2YXOhrEetykUjG"
    "iXaEiSc6CYXaUBSDREihZSi09Xb35jL5vuN9bV5V9fr02YuTV8zyAanTEqmzowDYtsDn8/6d"
    "bQaHTvE/zTjbb83K4c7gVV9Hh/pKj8nB4R737/+Kph0jy9JJU5ZlhBD/od/5pECkO3JddoZD"
    "005V8Ob0MJdefxW328nCwgMeLadpNE0oFbDLJfLFI9bWcgBoZZ02xcQdjZxT8QaCLV0ju7nF"
    "6JkU58+nGBhI8sdKlrn5e7C+iq+nHxEO81zET6VSYSe7Bs0AptvvUoXkQsbGaplYpglAe3uI"
    "18ZP89Pd78kke5h4eZx4ZxSn00HNqNFoNJCxEDhRLb1ZDYeD4VMjcXxeHwKQ6nXMxUUmx8eY"
    "HBw8+S2EwO3xMDScwrdchdpRQzaK9aVaS+GHh6vktjZAgPXoN0QsBv+Cn0li50mWxZ9XMCwn"
    "Rt74Xa7u17/czTbtdEGmVrOxVtPYXh9KKnWC2bZAiGfeqJmk87C33eRoW/9cxuW793AhP3uo"
    "jlDO22TWs+Ta2hACDKOObhgIJHK5LJn0BgXNRUkd4ce7+Tnh8H6rEP2w1arai5msmIqlYj2S"
    "t4lZ1Qj5Q6TTaXZ39wj6I2yu59g+rPO42sfNW8Vfytmja3jUgkLgBijKsfGU+aXHctIRTY4l"
    "uiMcaxrNpo1lOylUJIp2nO9WEtz8rDJXymofoJpPwEYiuQQo4PIBHie2+3LXoPda/4A6Ee9Q"
    "3EhwULAaWzlzeXej9gU0vgG9RqMKwuQvgMaDfMvLULsAAAAASUVORK5CYII=")
index.append('clock_menu_icon16x16')
catalog['clock_menu_icon16x16'] = clock_menu_icon16x16

#----------------------------------------------------------------------
maxpriority16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOfSURBVHjaYvz//z8Do+lCBoav"
    "PxkYfv9hYGBhZmBg+sfA8I9Bk1+YfTkHBxPHy9e/YhjY2M4wMLEB5VgZGDi4GBjYORn+73Bi"
    "AAggJgZs4NsfMzEB1g2V1Y76lqFu6ixsXGsZ/vxxxqYUIIAwDfj911tRiW9DY4+72g1GRYb9"
    "z6QYpFzd5LiEhVcx/Psbia4cIIAQBvwH4j//YzS1BZe19nlI3mOUZVi//S3DtxevGb5xiTFI"
    "2DgI8YiLzWP49y8N2QCAAIIY8A+o+/vvDCNzifntvR58T1nEGXbues7w+cyu51y39j5S+vOc"
    "4Qe7IIOktQ2HgIzETGBYlTL8/w/WChBAYAOY//3LDwpRmz53rjfLb15BhgM7HzHc2bXv0b8f"
    "37w/fv3t+ujksdvCn58yfGbgZpC1s2YQV5PvYvr/vwakFyCAGEGxIO629H9NqSUDAycXw8ET"
    "bxi2rjl/7cePX+EMnNxXgNHCwPD3jwqPAOfSf1JaZmwiIgyiXH8Znly9xfBtrRUjQACBDWAz"
    "n13NwcDs+vMv478///8/+sfMVM/AyPaQgZkDGG1AA1iADv33V5KBhbWegYVNjZmFhek/E/OR"
    "v1tsawACCGwAo9FMBobPvxkZmIEKuTj/M7CyAwOUhQWoMYOZjYXjLzvXZAbGfz/BAc3wjxHs"
    "KjaO//+32TEABBALJAaACYeNBagRaCM7LygdABX97ff1UMp595eV4eiRx7oMnJypDOzsv4De"
    "+Q90DdCcf2CtAAEEiQVWPgYGHn4GBm5+UAoEJrH/i2LD1XMsPIwY3ovrMwhrKscx/vu7AijO"
    "D7aAA6iOgw+sFSCAoAZwAqMCmET//GFn+vdvTW6adoylpyHD3KNMDNceAh2noMkgpaMWyMTI"
    "sBHoAkFwuLBxgrUCBBDEAFCc/vgjyc7MsLsyU9/L2kOXYc25/wwvL19l4HlzhYET6GQmSQUG"
    "aR1le2Zmxj0Mf/+qwNIBQABBDPjzV02Mn2NDV5mJrZOPGsOeq78Ybhy9zPD16ePen6+etQi8"
    "u8XA8ec3A4uoFIOygZIROyfrWmDiMwRpBQggsAGiQhzrJzSam7n6KjIcu/mNYc+mSwzPbj5p"
    "YeBgK/n9l6n2wtXX5T+f3Pn74c0Phj88IgwqBkp6bJys60B6AQIIHAvAGOOYvenxn49rn/x9"
    "9urHzxdPvzYyMDP3wdz4j4Gp69HTz++Y39/v/PiUi4uXm5n5PxMrK0gaIMAAAe08keAYC0oA"
    "AAAASUVORK5CYII=")
index.append('maxpriority16x16')
catalog['maxpriority16x16'] = maxpriority16x16

#----------------------------------------------------------------------
led_orange_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAObSURBVHjaAEEAvv8BhCQUAADv"
    "+wAG8PQAB/39B/YBAD73FwdlBCsePggbGA4AAQAB9PDw8/Dt7sj9/AGbAAIDvAMEA/cHBwYA"
    "////AAKIhQEK/v/7L/OfgaFLwUI9Uj0tjkHWzouBR0SKASjG8O3DK457Fw76nty2xvfahZvr"
    "GJk4SxkZGe4B9TMABBCzkxAXw79vf+XY/vxcZZXr4ek0o5tBxiqYgUtQloGFnZuBFYi5+MUY"
    "pNV1GfRtdRh4mN9p3r1+3/XXb6adzMwM7wECiNlRkIvj/+evE+zjNb0sGzMZmNnEGRh+/QQ5"
    "CYiBVvz7xcDw5yMDw/dXDCz/fzEoaQgxsH+5L3rl2islJmbWLQABxPLrL6ONigJjjIkBUPGX"
    "dwwMzI8ZGDh+Qwxg+gr0ACPQkB9AQ98zMPx4ycDw9z2DjdhnhptiP30uf+T2AAggFmZGxgw9"
    "XTYGtmd3GBjunWVgUOYAehpo2B+gZmYOiAF/gS768wFoEBA/ucjAcu8Kg4UMN8ONr4xZAAHE"
    "IsDJaKMgzwxUAFR0YgNQMVCjjBFQsRjQBeyg4AWygS76+YaB4dl5BoYzu4Cu+cGgKMLOIPyM"
    "0RAggFj42Bj4OLn+QKLi9ycGhrNAQx6dYWAQU2Jg4BaCuOAb0Pmv7zMwvHoE9AZQLRdQiukP"
    "Ax8HAztAALGw/v7DwMjMCFbHwAzCfxkYPj4EagJiWCQDhRiAYcnACjGPAeggBiYQk5ERIIBY"
    "/rz9/eXnPzZONm6IaxlAruaA0sxQsX9QA4BhyfATYsgPRg6Gz79YfgEEENPH979OPHkOVM3F"
    "BLGBC4pBBvIAMS+UhomzQQx/8J2f4e2nv5cBAojpGzvzrDM3ORh+MQJVckIVcEIV80IxFwNC"
    "Dqj5Hycvw4mHvAw/f/6eARBATKws//dfevZn7bFHcgzA4IQoYoV6AeYSdqgYSI6XneHkSwWG"
    "C3f/7WNl+rsZIICY7eQEfv/gYTlz4x2XKxuviKiszE8GJt4fCFewQgMS6MN//wUYjt9WYVh3"
    "kOn+/9/fYlhZmJ4ABABBAL7/AzM0HAAFCQTv+Pb5uNbgAyLk7RYnBAgSAAoPBwAEBQMA//0J"
    "APX2FADa3h0AtL8XALzH/Lr7+PVdCQgC2P//AQACiIXpzz8GIA3MpUz3/vz5Hnnlyi/PG7fZ"
    "s4RFxIz4eUU5gDHF8OUT4883b39e+fnj03RgnG5kZmH6DMomIAAQYADJPDDWrsuzpAAAAABJ"
    "RU5ErkJggg==")
index.append('led_orange_icon16x16')
catalog['led_orange_icon16x16'] = led_orange_icon16x16

#----------------------------------------------------------------------
squaremapviewer16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9kBCxMCM4ACV0sAAAFX"
    "SURBVDjLpVE9SwNBEH27t4ZAWjs1hSiSRrDQykJEg4VtCgtt/AeCghistDH4F+ytRAsRrUzh"
    "X9DGIgT8SBE4DxJztzNjkbsk6l3U+OCx7Hs7y5sZNTm1ez89Oz7RCkQsASQAcfu0BNjwbjnU"
    "Q4+UVrWHx6qZX5kZXdgqmMpTu8CPaLtshQxs23sPgFQGUKfnw6b+xryRBZDFnzFyQmJcjwAA"
    "2wfXiQ9Lxfw3v1TMo9FkmEDpT+JX9BZGfqSx0tAk+seovm9jdYKGoZ4ESW2kUibWF+0Ac2tn"
    "rgyIzNKla+AM9R1gP4jWQK5w8csELMKBBH5LRJoiIpJevHI7M0hKEbfCSBflwJByOuLR3jKI"
    "GMwMIsb+8W3sijsfKg1D6G5h5/Amsd+4FEppILdZ9gbdgrN655mX6nNtbL2c9hpWRBRIAA5J"
    "DDAUmABIDxmA0QqN17oSEfwHHxaeVDnvOYo6AAAAAElFTkSuQmCC")
index.append('squaremapviewer16x16')
catalog['squaremapviewer16x16'] = squaremapviewer16x16

#----------------------------------------------------------------------
symbol_plus_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAaQSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsMAZjLB+UgSNE/jNaMHz/Wsbw558dAwMTC5D/n+EfI0gciIH+AGn7z8TE8I/pKgM7"
    "z0QGRpaVEEE08Beo9hcbw/9dd8FcgACCO4CB/RdCwT9mVE3//psx/P65NdAqTCjUIoSBlYWV"
    "ARx1IMvBjoO6nZGJ4djVk5bTt8wz//H7Dw8DM/NcQiEAEEAIBzD/RRiI6gBmhp/fqoKsw4XW"
    "5i8nGKQhFj4MovyiTFWzauoZOHh3AoWeIHwPtO4PM0ooAwQQC2owQwmmPwhHMDJKAkNAP8DE"
    "D8z98ecHw+NPj4Gq/jMwAiEEMTL8/f+XQZxLnEGAQ4AhxS2aoWXlBOlv378pMTAxP4FGIQQz"
    "IoUcEAAEEAtWb4AUMYJChBmihpGBmZWZFSx1480NBrclbgy///9mYGVlZWBkYgTjz38+M/Q7"
    "9jOk6acxcLJxgqMJnsj/Q6MWS5IACCAWnGEJDqZ/oBj4DtL+798/qFn/GX7//Q12AMh4RqBv"
    "QBgk9hcIwTH47y8DNHv/APsWnFixWwMQQNgcwA7EKuC4B2v7Lwmk2VDyLjCxgSGQZmRkBGMw"
    "m4ERLecwqAOd+AUaEqBgfQqk3yErAQggNAf8t2D48bsDaI4OAxMwq/1j+gf0ATPDHwYeWAgQ"
    "A0Ce/fPnNxPD3z9TgK77BRRgYvgDFP395wkDO1c7MIsug6kFCCBkB+gy/PqzLsI2WjLGIpqB"
    "m4Ob4f8/SLj9//uPQVtWm2gHcLNzMmxvWM7w9y8wK4JDBhyHDAcuHBfoXj51yU+IuWBHAAQQ"
    "wgE//+Q4aDtJLk5ZwMDCjDtpcLBw4PD1fwZ2JnaIr4D67XQsMNQ4G9gA08pfxs4FfYUwBwAE"
    "EMKmvwxG/kb+YM1/gYno1ONTwFD7A4ljaHZjAcYKKBdAcidqfIPkbn24xXD6xWlwloQoghWi"
    "/xm0hbQZ+Fj5GHwsXBi6lk5VhekDCCAWdF+AwJdfXxh85vswfPr1CViYMTMwMkMTGjMw4QFL"
    "W2YWZgZmJkRhBXIgDwsPw9xLcxnmXpkLcRzQZHAWBbJ/M/5m2Oi5kcFa3BpYpADLDyZEFQQQ"
    "QMiV0X9YzQj28X+E4eCsBhX7j6OugDkeTEELJ2SIZAlS4c3AABBAOCMb5EMYhhU2MB/hrFpB"
    "CQ6qBhn/Y/yHmUWhACCABrw6BgggFtx5GQExCkkGRgZ8+ogRgwGAAGJBNxqm4fPPzwy/fv1i"
    "+MX8C1ImMkEwCwsLAw8HD5agZGL48fcHsKz5DYkGZkS0gRIhPGegAYAAQnYAMJFDYoSLlYth"
    "QdgCcHkPildYtmNjYWO48+4OQ9fxLrCBLAwscAd/+/uNIVormsFd0Z3h179fkIQITS//gFCd"
    "Xx3MZgXmKmBChEc9QACxIHnh+v5r+43znXMZQDVfhEEEVheDyoGeEz0Mv//9hoQMrBz7+5PB"
    "TNKMIUAlAG+cH7lymuH/zx+PYHyAAEJqEbHM2HR2g1/xylI+UFEswC0IyzJA+h+DOJ8YOOi/"
    "//kOFkdPByA+yBGQ2vAfw6M3Txkg9QckJ/z8/RNUFDO0LZ7AwMDJOQumDyCAkKKA8eh/FuaY"
    "vs09DRN2TlFlZWHnAuYfUA3G8PfrN5bFeUsYIixDiErZX358ZTAt8mD49OXzX0ZGVmAMMTL+"
    "/fP395+vXx8xMLPPBLYuJsPUAgQQaiJkZNzMwMG2/9/fPyo/f//jBTYygRU7kyTDr59TgcWz"
    "OCnZ68fvn/9//f5ZwMD47ywwibAAy/UfDBxcj4DmvUTOFAABhC0bAutvxgsMjEywpC8GpL7j"
    "K4BwFErAeGI8CcSnISkSe3UOEEAsBCt2YKYAmwd1ACj+cbWIYG0GpMYJN9ji/7jLO4AAwuEA"
    "UDOKCdqIBLfLgE0DiOFqQmoMW6O3MoD4oIoJVlP++feHQVUIUsn9+fsHlE0ZoW06oIuAFCu4"
    "xsVomgEEEAtGOQRuw7Eg1aUMz4Hk7X1XDshHW4Uz8LDzMNjK2eINuJVHNzJ8+fDmNQM79xMU"
    "s8EtYlS1AAGE6gCQr0GtV9Qs9huYeLoWHFxkw8XGzRFkHsDADAxieJfyP6IEBQX9mVvnGZqX"
    "9wCLTLYJQOF7qLb9gbaOEeYDBBAjvApOZwV3mSAOwJoefBi+AbtmTOzaTEyszODiD6VrxgiM"
    "J6Aff/wEtfvmMzCyTgTFBoY5P9kgandC3AYQQIwD3TsGCKABr44BAgwApEVL7WLoUDYAAAAA"
    "SUVORK5CYII=")
index.append('symbol_plus_icon32x32')
catalog['symbol_plus_icon32x32'] = symbol_plus_icon32x32

#----------------------------------------------------------------------
led_green_light_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAPESURBVHjaAEEAvv8BAEsbAAAA"
    "9QAACfIAAAv+BwDyAD4C8Q9lFwYlPhULGA4CAAEB6+/v8+jp7cj9+v+bAAECvAAFBPcACQcA"
    "AP//AAKIEW4A+0cZBq7/XWbKhpFxFqkMDtpODOKCogxAAxhefXjHcPjcCYaFGzYwHD1ycR0D"
    "M3spAx/XPYa//xgAAogBbIAnnxxjONPxnA3x/199u/MfF3j37cH/8nm1/1n19a4yaBgoMxga"
    "MwAEEDODCi8Hw7+vE+IiPb0agysZ2Fl5GX4yfGf4x/CfAQT+Mfxl+AXkf2f4yMDA+pfB3FCJ"
    "4TPXa9HTB+8oMTAxbwEIIBaGX/9sxDUEY4zsxBm+MHxm+AHUyMXAB9YO0gox5DdQHGLIX6B3"
    "DXxYGGR3c/g8PsPoARBAzMzq/F12vtJa/PJfGXhZxRj4WISAiv4AMYj8BdT6Heiir0DyK5D9"
    "k+HRz5sMl7/tZWBhZmW4c/KvKEAAsQiIsNmoaosw/P3/g+Hshz0Mf4V+MUixqTKwM/AwMDOw"
    "AIOQAWzUD6ABL3/dY7jw4RDD7z9/GFQMeBhEJP8ZAgQQC48wMx+vABsDGxsrwz/G3wzXfh5l"
    "eMV4j0GIRZqBk5EHqJ0RqPkLw7s/zxne/nrG8Jv5OwMXEy/DHwEmBh4hFnaAABTQwQ3AIBAD"
    "QZ/viOm/S955oEgBYRpYjbbYF/QITRst5VBh1ovhCC24gW3Bnx/UBdLb1kSaxog4AlBIbikA"
    "g0AMjDRK3fvfU5C1FMVHJN8ZBhK2UX1NZrO7eMRLQ5I+kQWIAmxFgPCj03UrlffE5w9aWf0I"
    "IKZ3r76ceHLnEwMPUCM7CwcDBzAG2IGYk0EAiIWAWBgoJgQWA2E2oCt5gOqe32BiePP062WA"
    "AGL685Fl1sntLxkYf4A0cwIDjhPoAl6gYpgBIM38QMwLdBXQEqAq1r88DCfWv2f4+e3XDIAA"
    "YmJgY9p/cd/rtYdXfwQq4wcrYAZikDdAruACGgAykAkoxg42VoDh5PrvDGf3PNkH5G4GCCBm"
    "Bnnx3wyf2c/cOPrFlY9bUFRLU4qBn5kfqIkbrBwYP0AD/wDZwKT15x/D3rWPGeZ3nL3//f+v"
    "GGCAPQEIIGYGPjUGBjb293++/dhx9tBjpbs3PqlzsAIdzs8HjpG///4wfPrwleHciQcM8yac"
    "YFg99+yu7z9/xDLwcNwA5UaAAGJkMDVkAKeWr/8YGH794QayPDk42LMkZISNRESA+YSRkeHd"
    "2y8/nz9+c+X7tx/TgbGyERjanxk4WcHZGSDAAMyhUSpfaf0ZAAAAAElFTkSuQmCC")
index.append('led_green_light_icon16x16')
catalog['led_green_light_icon16x16'] = led_green_light_icon16x16

#----------------------------------------------------------------------
clock_stop_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9sCDBQZGvl28VoAAAjU"
    "SURBVFjDpZdrcJTVGcd/573su1eSbHZzIRAIJLCBhBBAQETpIFpLEUFQMWMdFaeKQ53qdOxH"
    "p9OZ9kPHmQ71AzodB62l3ASmjrYqiKFAEIIQQAnkhhBIQi6bzW729l5OP8QBGRW29syceb+c"
    "8zy/53n/z3nOEeQ6Vn9iYMk6LHkPiqjSDHWKoYlCISHjyGEzbV3EoQ2Vw2jqF+xdlsjFrLjt"
    "iuUf+cna6xSvvn7aBG/dghmFnvpIkJJCDx5DQSBImw790TQt56M0nR20WrsTp81E9l105W98"
    "+MDAjwdY8v4TXr/+u2V3lk55YvlkppSPQwKxqMngtQSWJRBCIHAIFnkpCBqoGnRfHWX7x1/z"
    "YeOVK8NDmT/iVt7gk59buQMs+aePrP3a3Fnh5zb8YiY11fl0dWXpONsNiQSqmcarCYYGL2Nm"
    "M5SWVZLISCzdQHr9lEdKiUS8dHTGeePvX3HweO97qMqvOLiy5/YAd+4OYTrbG9ZMX/pkw0wG"
    "R6B533kCyUHm1/qZPi2Iz2+Ql5/H/v0fMTwcZc2adYyMxEmOpunsGObzlhGuyXHU3xthYqnG"
    "nvfb2bzlzGlbsJajD7d92516k/O5O/2krd0Nj0SWPvZ4Lc0tcTr2nyMQP8LMSIr7H7gHTddR"
    "NQ3D7aGzs5NUOktNbR224+D2GFRMKcI2W7ny5RH6u71cHPVw95IyQgG9uPnY1cWy9NFd9OxM"
    "fRdg9naVZHbT2jXVa1euq+NA4yDyq7NsWFfKvAWVHDjwGZHITAIBH6qqIAQUFYWYPLkCwzBQ"
    "FIGm6WQyGT744AMefWwFd88J09J4nuYuncXLJlHgVktPHr00k4nr3qN3l30zgGfFutq64t8/"
    "/ss7lUMnRvF3nOGFJycRDBfgdvuoq5uN3+9D01RUVUVVBX6/H7/fD8jrUEJATU0tBQWFaIaL"
    "+pkBOpo7OHbVx8Il5Qx1R6d1nx9IMLz38A2Aii0+t0vZ8/SL9xR0xr2YJ7/klWcmkB/KB0DX"
    "FQIBP5qmousKuj4GoSgCIUBRBKqqfPNVcbs9OI5ESomi6cyu9nPi4CU6CXPXwmJOHe5cmHKt"
    "eIfY3rgCQDyzbuGiSeWiOMS5g22UKZ8zNDIECLxeD6qqAfJ69EKMOb6uZDFWjqqqomkqQjC2"
    "VlNRVQWpCSoLTtN1qIUB1cdP7qvyM5p5YSwDxW+6dE15fdXzSyYc79KYbV7gjlrJseZ2env7"
    "KSwMEArlf2P49ueWEAJFEYBAVRRs22LLli2Ewi5CnhAHr4SorQtx+mBrWVZb/g8Vcf+88inB"
    "346/d57e3nyJjUth6U8XESrM49KlXo4fP0siMUoolIfH487p1L4BIbFtSVXVNOrnLsBLil2f"
    "RvFWjcfuvJzf2zZ4TMW87/H6e6cvvxaeRHjwMs8sDyJVF6UlRcyZU43Ho3PqVBsnT7YCknA4"
    "H03TcoRQkBLcbg/ptElwnMIXp4c4TymTA6Zo/U97j4K0pwUrx9N91abcl8bjN0CCZZnYtsP8"
    "+XWsX7+KSGQSjY0n2Lr1X7S2dtzs7cCncLoFAMexiUZjnDlzgX37DuM4NlI6gER3G1QWmFy5"
    "nMQoDuPSxVTNbYgK2xOgtyPB5OmCwLg8bHQcx0JRBFJKAgE/y5cvYd68Gvbta2L37gNUVbUx"
    "Y/YMtIudBHbsIPnCRto/OkTP1QGi0Tggyc/3Mn++hctl4PO50BSb6ZP8xNpTjE5w4/ep4zXD"
    "MArTUiOdFgxHL7Nvfw+WozF//h2Ew4U3Ca+oqJCGhhWcO9fB/gPHiDYe4esLF1CKqyhq+gpD"
    "E5SUBKmvr2LixFKCwXykFMRicZqamsDJMNBvo9llxCwVzeXyawiXcFBQBZjZNMPRLKajY9v2"
    "D/aq6uqpeH0etr2zg9E3/0xBNkPUpSOkpA9o+Z49jpQgJSD4NQrKLpvqFFO1jKVGVcch4JWU"
    "lVWxdu0sLKkjpYll2ej695SfbVO67V3W3L2IrX9I8+KqVbjLy3HS6ZyqRDEMzP5+Xt++Q9HS"
    "pnrRSaQoLysiOioZGYmDauByaaiq8t3dUmJv3oxZVUXlksUIRUExDFSvFzkm/1t7lxLV7Ua6"
    "3UihoCH19qHOAUqWldF3yU0qmcYTcCGEwHEkjiNR1RtGnZ07Sakq3tWrx+w5DvGuLjLxOKZp"
    "5pQBTdeRsRjSMqWGYRzqPjeUjTyEaxAfnR3DzFkQJpPJ4HLpaJqCoow1GtnYSPLyZdwvv8z1"
    "3DgOmb4+7GQSy7JyAlBVFZFKgeNIDY/7i76edKs9lJwVrCilueUUSfNTTp5s4emnn8Vw5WEJ"
    "0FtbSR09irFxI9q30iwdh3RPDwwMjAktx5NStW2EbUmN/gcTTv6Hb585cOm1VRsidPeOY+iz"
    "wzQ8uRK34SGNwOjtI7l3L8pTT6H7fN+5UmVjMSTg5HjBVr5pwxLkWCY97rfPHRu4drE1yZRF"
    "EfKKluJx52MrAjmSwNy2DWv1GrSSEmzbQUrJt4MV/PgxBtCzdNBKy1f37bjIKBqhWZW8t7uD"
    "1FAUfdc2ErPn4VRUkE6kyGYtslkby7KwbQdyTPutAQD8nrd62kb37nq3F60kj5KaKva8dYK+"
    "8eWYtTP566ZNDA/HkBJM08KybEZGRvl/xw2AjjuyePTnmz++9uVfNg+SV1VCZOVd7O4uY+tf"
    "9xAMjqOgIJ90KkMmY6IoOk1NR3IW3u0BANrr+zDU1cf/3d/86p+GGHQV8tDaCAWFtZjZGg4f"
    "7iIWT2CaWWzbxrLS1438mCl+UD8z2kJkxVvBCv+Dz64P8czPVHzJFEcPdhGNJclkTMJhH1eu"
    "thP/zaM0ODYKYOUYtfrN3AbmDwu47qJBVryEob8yo95X0LDKz8OLFUp9kqErMVLJDIpQ2DF7"
    "PBssCxdg5ggwdsOEN24JcP290DORrHgOQ30kr1ifVlvjprrSxaQSQTAA1+7TecmxcP+PAA6w"
    "CazcS3juQDE2d2GzGCGmoogyFALPtkydOsWOKWN/c6znCqQUY0He6kUsemD4v39+qpBol3NJ"
    "AAAAAElFTkSuQmCC")
index.append('clock_stop_icon32x32')
catalog['clock_stop_icon32x32'] = clock_stop_icon32x32

#----------------------------------------------------------------------
treeview22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAAGXRFWHRTb2Z0d2FyZQBBZG9i"
    "ZSBJbWFnZVJlYWR5ccllPAAAAsRJREFUeJy1lT2LVUkQhp/q7jvjzB1HHGcUVpdNFhTBQEyM"
    "/QWmaiIDGhiYab6RZiIImw0LC7vJBv4HMwUzRRE/8AO8jqiMH3PvOV1VBn3OuffusntB1oKm"
    "u9/T1Hmr+q0ucXe+h8ny8oX7h4/8cEhC9KyOAW5ggHkZ3szargGzie/tWQRTk/fPXr5IR44e"
    "OHjp14uy5QtSDY1sUCnUCtkgN+tam7X9bd9iGVQCo7rm9rUbB1JKPR6963HnuaGVdszUxsMc"
    "VMd4bnErmDUEPDi7lxOkOZJJjyf3Bixujdi9FFAb5ynGQEyJuqrwAAIQS+jS5rKZQ4RRZdx9"
    "s0CVI+lLJaz1jV/O//y/XNqZGwPuVEIwCdQu/3rw9KmzqBqF57SpOlWVGQ5r6tEIgOwCIZBU"
    "Iqr/zSLGAMDm+lKHrW18QqKQR4q74+L0aBQlkWQSmOGX06fOAnB9AdY2PrO53gda2Slm4FIi"
    "MgcLQlIJMxn/8edvALw4eYvN9UGHZ8DUMXfaVJk7hEhyieiM4lNzYhB+vPl4Cq8+bTdSNAzr"
    "GIOQNITZjnOGmMhBcCm6XQygtWLmqDrSMRYoOf7n5bmDyPTeKNFqTRFvANWSX1VFWsbmEAJB"
    "ZZqxT+Srtfn5HnG4zXyA/hz0ewVfXd2FqqJqjSRLNHSqsGnH7hDjtLZDf5EH546xRx8CRW4l"
    "TYqaF51BQ7LRsU0QNCvOpegcGMvtanrM2u9jub3+mAtjc7Amn04pEJMw5TilwtQ1A6mTWn77"
    "jveX/+qcAki9TZBSla0qIlLkJjExrCZzKoAgTbV1P1xd6cJvbd/KTljZOYVtj3JxHKPIlx3L"
    "nLjyCp98JptH3AxyMxsBQ8oTqo6ZjxuAOWrwwZdgIYjMHd94umf/3p/Uk2d1XMo9tF3BJzpI"
    "+xS5jjtM19mawhBUfGswkO/V88LsI99mXwEIftJkA1QP5QAAAABJRU5ErkJggg==")
index.append('treeview22x22')
catalog['treeview22x22'] = treeview22x22

#----------------------------------------------------------------------
weather_lightning_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAKcSURBVHjaYvz//z8DDOQyMoIo"
    "LT0Zlp0GrkbS/7nE/v378xso9J/hPxsHAz/Td8ZDO47/rLz5pfT9//9TQYoBAogBZAAMFwIF"
    "qhkY+q4VGP3//+MWUOgPEAOV/v8OZr9aW/c/XIT5J1BZMUwPQAChGFDBwCA0T5rx3rfDNUDu"
    "WyB+CMVv/7/a2vE/XJzjO1BzObLFAAGEYkADA0P88SSNv/9/bAZyLwLxESA+9//d3pb/BTKc"
    "f3kZGNa7MDLKSiEZABBATAxIQEaaPUUzzoWJgV0AyHsBxB8YGH7cYfjx7CKDd6QZ47EOL58I"
    "E4lLPxgYUmF6AAKIBdkAOR0pAyZxQYa3N44ycDD/ZeCWF2ZgYGdhkIz0YpCMEWL8cOgIS8+U"
    "I5zvGBiYYXoAAgjFC7Z8HP2eolxnK8R4Pt1bEAUU6gXibiCe+v/Zzvz/obKCX4B6MpD1AQQQ"
    "ihcufvpRaPj6W75rsBazXJg6UOQ92Bt/X99juLj6MIMMH8MfC15WEWBkw/UBBBCKC+YxMMie"
    "t+d/9P+5G5Ab8P//P08g7fX/7x8PINvr/4c1Fv/Dhdlfg9IKTA9AAKEYMJ+BQaNFi+P25jT+"
    "f+/P8AGFBKBY8P+bLYL/E6RYvwE1VwIxE0wPQAChGDALKOPMwCDZpsxw5+c5hNS79Qz/IyUZ"
    "QP7PRnc5QAChGLAJKLGegcHiWiLD1/+vgEKfGP6/XsjwP0YcrDkVObxgegACCMWAzUCJ3TwM"
    "k99MB3LfMPx/O4Xhf7wQWHM6AxqA6QEIIBQDdgOT8ll9hvv/DwOd3QfUzM3wGd1mdAMAAgjF"
    "gJ3MDDHfEhj+fqhj+J/Az/AJqC6JAQeA6QEIIJSUeImR4fmNgwzPjnxgYFr9EZQ1QDGLHwAE"
    "GAC5oLsuGNqCCQAAAABJRU5ErkJggg==")
index.append('weather_lightning_icon16x16')
catalog['weather_lightning_icon16x16'] = weather_lightning_icon16x16

#----------------------------------------------------------------------
copy16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJsSURBVHjaYmTQjmdg+P+fgYGJ"
    "Sd/ByWqboZay2O8/f/8xYAH/YQQjAwMzEzPTkVOXzgMEEAsDSO2PXwxsfHxOZfGeUp5GMgzE"
    "gsnyIloAAcTCyMnB8B9kHxPT/z+//2JV+A9o6z+gmj//Iew/QGVcHAxAu///BQgglv8g57Oy"
    "Mf77+fvH779/gQr/o+pmBGFGht9A4b8gDDIIiH+DDAaqBQggFh5uzjoRXl5LZkYmmQNnbjC8"
    "ePGc4RfUJWDbgGwHExUGIw1Zhl+/IIb8gxr24dt/BoAAYokLcKyqTHJn/wO0/eOnrwxAZwAD"
    "iBFsABMTE8PdJ28Ytp+4waAuJwZ0KTvD3z8Il7z5+pcBIIBYJEV4f8kIcbGDdYjyYvhfRJCH"
    "4f7ztww/f/9hYGFhh2sGYSagPEAAsQCdi+JpkP9+AX0A8gUzKwPD+++/GX4BBf8DXQXzP0gH"
    "OECB6gECiAVFM1DwF1QRiA1S8QMYel9+/Qf7mwHqfxgGCQEEEAuyzSBb/8BCGWQ8lA+KqL8M"
    "iKiEuQRkCEAAwQ34C9X0+x/CMFAUwhSCaGCYwv3/F2owQACxMEICHOH0f5CEAqIZkGwCafiH"
    "pBnsMqASgABi+fDtD9wFMENgrvmPbAAQMyKlRhAf5AKAAGL58RuSb/5BNfyDhjLcNmgc/YcG"
    "KkjN/7+wQGRkAAggFlYWZmaQAg4uoA1AQWaQLUBHMUGjkZWTjYGZjY2Bg5uNgREYBox/IAZz"
    "cALZQL0AAcSy4+jFh0l/mNRAWRiYOSBOY4DlcCaGdx8/Mzx//ZHh0fs/QFuZweL/geHGCEz7"
    "Z06euwcQYAAj12LGoFkdIAAAAABJRU5ErkJggg==")
index.append('copy16x16')
catalog['copy16x16'] = copy16x16

#----------------------------------------------------------------------
clock_alarm_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAARNSURBVHjaAEEAvv8BdQAAAAMA"
    "AAkZEQViKjICaxok/CH57f0ExscAw8rlAGQLAAD5NxUAljE8AEkHGQP+5doC49DL/prZ8f2Y"
    "9wAA8wKI5Q8DA5+aEfcC70K2QEnNzwwMf/8xMFwHCvIxOP65x7CZ9S/DdxtVBmNlO0YGBo1/"
    "DOqs/1V0jrJ2rZ7I6HL07p8YgABi7NfgmpE8UySdV/Q50LzfDAy3gE76y8DwTYGPgYVXlIHh"
    "NyPD33dvGTgevWdg5ATK6QMNYmFg+LKNlaG/989qgAAAQQC+/wHEjh71O1kBCuGi4QALHQAA"
    "FBoKAP4GTgD9FlkAAAgiAP//AAD+9+QABOGnAATmrgD08/QA9P0BABgQAgDIrv/7Aojl5/Of"
    "P/Z0/v8NdDyvVxQH08f7JQzMykUMXGL8DD9+MTD8ARrw57smwz9OawYefgsG7mdlDJfXfPl3"
    "4T7Dx39sDIwAAQBBAL7/Ax/G9tnU5vX9WZYAMg/4ZgH0E3kAz8nBAAoLCwAREg8A+vf4APjd"
    "2AD9CggA+BVAAAn38QAWFsUBu6v/scDS/4oCAEEAvv8DNhIB4CpQAHk9SRMb+gGTAAweIAD8"
    "+PcAwqytAAEHCAAJ8/YA//z8AP4TEwD5+/0A/QcjABD6tAD66fAN49cAcwKISUDnv6ysEoMh"
    "wx8Nhn888gzff/5lUDU0ZFAxNWGYPWUyw5yVSxi+8QsyMDMyA73ymeEbiwgw0rUYBKUY5LmY"
    "GZQBAjBIxjgAwjAQM7kIBlY2xP8/x4hArZqU7J5s++4sLgxtxDSiB83EcV4838tdWVPOaJ3M"
    "yaCkaEU1T+H+CwBBAL7/BPTaAPj3MwDuAAfgFP728gD/BxkA9PLyAM++vgBXVFkAKjxAAP/+"
    "/gD29vcA+/4EAAYA9gAA/eEA7u8A+fT8AO8CAEEAvv8C7uQA6ePDALD5+d4ACPquAPv6AgDz"
    "8fEAKDo+ADRGSQABAAAA+fj6APz7+wD+AQwAFPi+APzv1ADa1AC5+v8A5gKICajg16PTDMtu"
    "r2H4xiraysD+/iADC5cMg4C0KsOH918Zvn79yfDrL9D/LGIMEv8uMvB+aWC4fer3vxsXGWYD"
    "ffEdIICYvYA2MP1nePDmMAOzhPxXG1HDbcwcvzgYfv+VBCYidgYmRkYGLpYvDDLMmxkEPhcy"
    "PNzz8M/KKQwTX3xhmALUygAQQCz/WBkYGIHJ//1vht0vzjGU/Hr3ilXcPI9BXGIyMMmqgkOK"
    "4eNdhq93rjNcvwBMwn8Y/jD9YtgrBNTHBJQCCCCWvxxAP3xlYNX3YChiVmDg21DKkCMgziCq"
    "bHg7mE3ktiwwKTP9fMXw7Oplhk1PXjPc8mtjmG7iz5B/aTXDYTZuhs8AAcTCCvQIMxeD1ps7"
    "DHr3DzCE//zDsOrrU6Db3jBMYxBi4AJ6n/HfR4bvDz8zvPgKFN7UwPBIQYFhKjMPgy5Q7hhA"
    "gAEAOmWRJjLoFvYAAAAASUVORK5CYII=")
index.append('clock_alarm_icon16x16')
catalog['clock_alarm_icon16x16'] = clock_alarm_icon16x16

#----------------------------------------------------------------------
heart_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMkSURBVHjaYvz//z8DJQAgAABB"
    "AL7/BGElJQD5//8A/wICbh8ICHMjDg4bBvz8Adnq6vHM+/uvHgcHFSsHB0MTEBAJ8v///MPl"
    "5djB6+t7AQQEs+719QACAEEAvv8AXCQkAGIgIKazXl7+87q6///a2v//19f//Ken/85BQf/X"
    "WFj//729///b2///1tb/7Kam/5M/P/shCAh8AgICAAKIMUFOriDc1rbfo7KSgYGbm4Hh2zcG"
    "BqAEAzs7A8OrVwwMnJwMDDw8DAw/fkDERUQYGB4+ZOgsKPhTc+SIP0AAsahzc+c7ODszMLx5"
    "w8Bw9y4DAyMjwoNsbAwM794xMPz9CzHw92+G71u2MPx7/JghmoODZQMvbyNAALGo8fBIc/z5"
    "AzYVpAAFgAIYZAgLC8Of27cZvmzYwPDxwgUGoFsYJHl5GXS5uJQAAoiF6+dPZoZnzxgYuLgw"
    "DQA5HxhQvw8cYPh26BDDD6A8zH3/gAYDdbAABBDLi/fvvzHcvs3DICnJwPDrF0SWmRls698r"
    "Vxh+ATX+fvqU4Q/IQUAMoplA3gRa+Ozbt98AAcQMdKCz458/SlwgQZALfv5k+Af097cTJ8C2"
    "/v7wAawJhEHu+w7EEsLCDDeAXuh99eoSQAAxP/vz5zrr799Rln//sjF9+sTw8sYNsD+/P3nC"
    "8O/fP4a/UM2/oJolRUUZfoiLM5Q9e/bjzLdvhQABxAx0zrOj375d/PHnT6AxCwvrjxcvGN5/"
    "+cLABNUIMuAnEIMCTlpWluEjMBpzHj36venTp3qg0FyAAGLgA/kXCFgZGMyzBATevDA0/P9C"
    "Sur/OaCXrwDxJSC+yMT0/7O6+v+zOjr/bdnZPwGVJ8HCGSCAwAaAjOAGhoE8A4NaBDf3w0cm"
    "Jv9fKCn9P8rI+P8MG9v/LwYG/w9oav7XYWJ6AVTqjRxRAAEEN4AXaIAWEHMA05o7G9u5K0CX"
    "fATiz6am/1cCDZNjYLgJVGaInpkAAgjFAE0gFgJFEzD5WDAz7zqmq/t/tpzcf0EGhuOg8MOW"
    "GwECCMMAYVAKBmIxINZkYOgCumg2kCmAKzsDBBgATpIuuOCCfHQAAAAASUVORK5CYII=")
index.append('heart_icon16x16')
catalog['heart_icon16x16'] = heart_icon16x16

#----------------------------------------------------------------------
earth_blue_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQUSURBVHjaYmRgaGBgZGBh+M/A"
    "xMDAxMTDJ8pfom0kGWlsJi4jKcHL9uX7n3+Xbzz7ePLQ0ROvb+ztYOIUOcbK/pXh54f9DCAA"
    "EEAMIAMYGJqAuM3cwHH93XnbH/1/8/3HfwT4/f/Tv1//99789j+gZN8PdvHUKcwc5jDNDAAB"
    "xMzAYA90AbO5dajxjtZJupLWer8Y/vz8yvD73z+Gt1/fMTz98Izhx493DLKi3xiMreVZPgso"
    "mt06fUn195fr60AGAAQQ0AAHIXVrld0p9fwSWuL/gJqFGLg5xBl4OXkZXn56wfDi43OGT9+/"
    "M7z99I2B6fdLBnH5zwyvWTR1H52+8uvfnzdHAAKISVBIIl/U8ofCy3cvGDhZdRk42aUYGJnY"
    "gRo/MTx6/hJoIAfD37+8DN++MzK8fveL4faFcwyfWU4wqthFFzMwMEoCBBCLpAZPBKs4H8O1"
    "rzYMm+6wMWiLAMPyy0OGa1eAgcQIDF4WoAFA77z58J7h3PXrDPuO72Zg4uBgMDWsFn54XCkO"
    "IIBYBOQ4Zd+9vsLw6z8bw3kWO4ZnX7gYfr77zbB/xy6GH9/fMjAxsjF8/wbywkeGn4xcDOz8"
    "8gwcrP8Zfv5/wcAtJGcLEABBAL7/Ax40SYDa7J4C3NOuAPX15AAMFCwABQ0gAA0ZLwAGEiYA"
    "2OC/AO/25AD/CPwADBMOAC81WwAhKDIAHhghAODJye4CiOXv71//OEQ5GL4zAP38cgOD1Asn"
    "BnYuGQZuQRUGPlEVhscvNjM8ffIE7B0zi0wGZVlrBm6mJwwSHMoM2398/QcQAEEAvv8C/e8B"
    "8REWEgADEv8ALzNcACYiRAAB/gAA+/oAAP77AAAOFB0A+wntACcrTQA/MlMA5+y4AMjfmADk"
    "4toA9fLt6wKImf2/gq2cjYK6mromg59xPQMrEycDOzBRcrIyMHADMT+vEAM/Py+Ds146Az/b"
    "P6BDHgMD8SPD4yM//906vmoKQAAxMjPIW9gnVhzwKdJgV+FwYHj1hYHhO9DmP0BX/GdkYGAD"
    "GsbNzsDAxQKMnf/vGcyUBRnOXPjMkODvde/zyyOmAAHE/J/h35PX936I6Gs5Wfg4yzLwAxUK"
    "sTEwCHMwMIhyMTCIA7EoJwODND8Dg4UCJ8PLl/8YcjJrfj25urIImBBPAgQQEIsCsQEDN2/w"
    "ktZJx/89ff////kfd/+f+3r//7Wvz/7f//Xy/y9gjvgJxDtPvP6vY1f0E6ihCpYXAAII6EgR"
    "oL/UGP7//wPkfi43sfcqkXSXEGZT/sbIJ8TKwPibmUH6kxbD/cPv/61dMf3h97fHG4EKF8IM"
    "AAgguAEMwOT7/+9FkJg4I4NQEqeggBUTF4fw79+//v98//IVw+8nuxkY/q4Ayr9jQAIAAQYA"
    "gtx+AyErEIwAAAAASUVORK5CYII=")
index.append('earth_blue_icon16x16')
catalog['earth_blue_icon16x16'] = earth_blue_icon16x16

#----------------------------------------------------------------------
folder_yellow_open_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAH7SURBVHjaYmRgYGDqLhP4y4AG"
    "Srs+aAOp20D8mwEPAAggBqDm/z/e7cXAIHGgvBMQcwExCxAzAzEjun6AAAJJMPz+cg3D4NyK"
    "tUAyeC8hVwEEENiA//++Q7n/QD4C078+nYYaggyCrwINcQYyTgAxSNN/gAACG/Dv9zsgE2go"
    "IytY2Y+3WxkWr3uMy8tgV125/bto4fqvMwECCGrAG7iCvz8egDVnF89lYGIVwBl2kzuC+4DU"
    "NYAAghjw5wNc4tuHqwyQcLkI9NovsHf+/wN5+Q/D91frGThEvJDN0QMIIEgY/P0E5v35fp9h"
    "1Y7vDKkpyUBDP4IkGP7//wWh//1h+P37LwPb7w8M85fsAgXoBqCWxwABBPUCxAVv3n6GBCVI"
    "M8NfsAsgtv8DhTTDz9//Gbj+foXZ/giIDwEEEDwQP3/5wrDj8A+GxBg3iJf+g2LkP9wbIPY/"
    "ILVg+VGY7ceB+BVAAIENePXmM8Ouoz8ZIN75DFYM4fwBeuEP1Ct/GH7++o9s+2GQrwECCGwA"
    "SLOPAweDiKQhUO0XSMD9/wd3BcwgkDqo7aB0wA3SCxBAoKSpBYzbqwxEAqAB/UCqF4ifgUwH"
    "CCCQAaDUYwjEoBQmREA/yHkHgPgoOF6BACCAYJkDRLNDDcMHQLn2F0wzCAAEEOP///8ZKAEA"
    "AQYAMIz+kBTrA/gAAAAASUVORK5CYII=")
index.append('folder_yellow_open_icon16x16')
catalog['folder_yellow_open_icon16x16'] = folder_yellow_open_icon16x16

#----------------------------------------------------------------------
redo22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOdSURBVHjaYvj//z8DscDK2Ng/"
    "1tn4v5OxcT4htQABxMJAGjCoiNdkuPJOdQITA4PDNwaGhGNnz37EphAggJgYSAUs3AxhuZUM"
    "E/trA2T5eC8AfaGPTRlAADERGQT8QBzPBXQlw79fDAzfXzJoGZswTJ7RrGCqonDBDkvQAAQQ"
    "IyiMGRkZcRrIzsBQoCwuVBDgbSKgqKbOoGWgCXEPSA8I//vJsGDKAoYlW84u+A5UCwsagADC"
    "aTDIi7Lc7Bsyo80V7F0tgYYwoWImGJsZTJ8+fIKhd8rmC4+//gWF+0WAAMJqsAvQUG1JxgPV"
    "ec4CopJiUIOYoYYwI9hMTChir1++Y2hvX/rh8N1P4QABxILNpdoS/w60ZmsKcPMA4/3rE4hB"
    "TCxQA1igGGYBlP3nC4OoEjeDhYeWwNEJh50BAgjFxaAwlWX/e2Fy7HsFUUlxBgY2XmAq4GBg"
    "YOUBamZjYGAGGcLKwPD/L8Qwhr8QNiczw1cJZYZJ3WcYNuy4MevUtWsNAAGE4mJQRMWZvVMQ"
    "5f4KdOlPBoYfrBADeYUZGAQlGRgEgJa9f83A8Okt0BI2iCVSCgwP/kowNKft/Hzi8sOCaw8e"
    "rAUa9REggFiQU4Ay368CL/33QB4bRFBanYFBx47hwUtmhlMHHzLcvvmGIdrkM4OCECg9Aw3V"
    "NmPYf+IzQ1vd+pvXX7zIevrixRGgzC+QVoAAghsM9HBCjMlrATCHDcgz9WW49l6coTdu7+cb"
    "999ufvf9+3ZeHp6AQGXOYAZlGYavCsYMc6deYli59vKyY1euVAF1PUT2PUAAMcDKCi9j4/Nf"
    "pnH9/z+f9///t/X/5/fE/7fX09svxM9vDJTmhPqq/urWxP/3b7T+z49y+6SrogLKGMLY8gBA"
    "ALFANchbKXw04Gb/x8BgFsiwYP4DhjkLz20+evlyClD6FbKG7Xs/Mhzv2nzz/P1n5fcePdoF"
    "FPqOzWCAAIIFhYG6CFAeGLOnX8kBDd2C1VAgmLBo+/XHL169Ov7q7dtbkGSBHQAEEDgoXEFe"
    "bOD//+Vm7v8Qe4unwoKC+gwUAoAAYoISBvI60gxbD/xkuP383ay3799fp9RggABigqZfAW4d"
    "U4adW24+u3jr1gJYkqEEAAQQ2GBglAmcfiDAAExWB7CEK1kAIIDABsuoCRtcvvKB4c23bzhj"
    "mVQAEEDwgv782ecMt+7d28tAJQAQQODkdvnay/Qv378DCwOGD9QyGCDAAAXyQUQ42DyzAAAA"
    "AElFTkSuQmCC")
index.append('redo22x22')
catalog['redo22x22'] = redo22x22

#----------------------------------------------------------------------
exportascsv16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJwSURBVHjaYqyqqjrt4eFhzMDI"
    "8B8IGBiACAYYGcEkkgADAxMjEwM7Gxvj/gMHvre0tBQABGBZzo0AAGEYCNKJ++/AMSXgSsjA"
    "n3g1s5muMPM4Jfq3l/lEbJ5wD7gFVA1z6v2INBBRXQKIBaQeZOmbpz8Z/oFY0v8Z/v36B3QJ"
    "kAZioEkIGij/9+8/BnExXoZ/f/+CnMwAEEAsf4EMkCNZ2SGKGFgZwb4A+wbI/fcf7DcgZgQb"
    "8O8vIwMzEyPDX6BakEsAAogFpAmk4TfQZJBbmP8AFf0GYgYI/z/UdpCBYBeAXQLBIDmAAGKB"
    "hRnIJSAbGYA2/Pv/D6oRZADE6SAIEv8LNuQ/2ABQiAMEEAvILyDAxATi/mNgZmZiYAQKAV0M"
    "xCCFoND/D3YBGAAZjGBxiBhAALGAAwgIf//7CwnEP0Cn/oUG4P+/cNvB4QFy+l8I/g92LgMD"
    "QACxgJ3KwAD3039woMHYYFfC2RAayoaGD0AAscACCBjnoLQEFvwLthVkJQMkPP5B/A8NWrDr"
    "oLHAABBATGBnM0IwGIKd9g+aIiHpAcQBp0qwGMRwmBcAAogFxGWE+heilhHmbohzwXb+hzuf"
    "AWThX4jtIAAQQGAvgJz88/cfcGJnBlJ/f6NG439wqmRggKj9x/AHRENTIkAAsTABkxUrMwsD"
    "Dy832G9cXCxwA/7BExE0XEBpAqiRi5OTgYODE+gtRiaAAGI5d+78dwtLS45fjL+AMfGX4dc7"
    "RngIg30McwUDIhbeM/1muHz5MsPnz59/AQQQIzc3d6KUlFQaI9Ah4Dz4Dzk7M8INQhYDYpbX"
    "r19/ff/+/SSAAAMAiaIhq2T+dDEAAAAASUVORK5CYII=")
index.append('exportascsv16x16')
catalog['exportascsv16x16'] = exportascsv16x16

#----------------------------------------------------------------------
taskcoach128x128 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAIAAAACACAYAAADDPmHLAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAHD9SURBVHjaYvz//z/DKBi5ACCA"
    "mEaDYGQDgAAaTQAjHAAE0GgCGOEAIIBGE8AIBwABNJoARjgACKDRBDDCAUAAjSaAEQ4AAogF"
    "mcPIyEh3B/AryTJIupgw/GP4zfAXiP8z/AHCX0A+CP8Fwl8c//7/Mf7x95vmrz8/DH79+yH5"
    "n+HXXwaGX4wMTH+fMTD9v87AwnCD4S/zeYYfnB8Z/rACpdgYGH4AvfbhPwPDpdcMDEzAdP71"
    "J1HuCRXEFFuphGCH34PQq98P3UhHHvsBCCBGFA6dE4CipyMDt6wYNMJREgDjv/+/LX/9/RX6"
    "8+83SwbmXyqCgjzCQvz8DFycnEA1oITxh+Hbz88M776+Y3j+/snHvx//3GX4xXqRgYF7FcMf"
    "9kPABPANJQGwsjCY/v7KwA4KACAulwD6F+ZvsN8hbE9+4ty+4xOEbn/OwHD4y9BNAAABNCAJ"
    "gEdKkkHCWJ+BR06c4c/vH8gJgOP//7+RP//8TP/D9MVcWIqFwVBXncFCx5RBSVyJQYhfiIGH"
    "mRfiCSD8+h8Yx18+MLx8/4Lh8p1LDHtPHmG4eP4mw/+PLFcZGDj7gQlgIcOlV38YmJmBlnIx"
    "PBP/yCDJ8g8R6yjFHyxwiPMDI1LleeAjA0MjMCEc+Dz0EgBAAA1IAjDOzgDn4j+/vwEjHlbk"
    "//H68+dP3W/mz+Yiyn8YHG2NGOz07RkkueXATZXfQAiUZ/j7/y848sHuBjqXCRgTzMAIBtHf"
    "/35jOHvjFMPSdWsYrh5+wcDwnXMbw5VXOQw//95n4OViuCP2iUGRBZTUIDkeWwnAREYQwBLD"
    "UEkIyHEOEEB0SQDmXIjAZgJa9z02GZwAQFEBjHiRP//+tv359zOVReYFg7GtBIOHqQ+DDL8a"
    "MMKBLYA/PyERDo10FBqJ/Y/hHzgqOdjZGT78fM2wZOtihj1LbzH8e855j+Heq2hgNXDijvhn"
    "YhIAiMUDxAJAzAdqpkDZIkD8DIiBdQrDbSD+hi8hON4aGgkAIIDITgCmnJilaK4QzBzUktWJ"
    "B6GWExhPDg6wBPDH9u+f/3P+cX9U49V/BMz15gwW8h7ARMIGbOb9wBrRMDYownElBmYmFoY/"
    "TN8Zth7ZwLBl9nWGP4+43nC8fJN2h//tcSmW//y/gBELdKMA0E3AlgCDJJAWBfKlgWwJRkik"
    "84DkoQmAE82bv6EJ4BioLQiKbyD+hZ4QBnMiQI5zgAAiOgGYcDAwZAkhItWBGzUHIRef6Lnr"
    "+38EnwOYALydQAngX9y/v/+mMkg85+E2usdQblPLwM4vwfDy5wtwRYweuf/+/8OZGLCJMQJj"
    "gZH1H8PBi7sZts+6xvDvBvffnt9PnucwfBT4xcTA858BNbEyIvmD8R+kpQiiGf9DW40wSWZo"
    "3wkRVMCGJ8M0IF4GxF+GQiJAjnOAACKYAEARny4AjXBGREAhRyqhBIDMByUAN5cMW5Zff/f9"
    "l3nMwmRyjoGDlZ1hvvU2hhds7xjOfz7HwMLIgjuiCVQFyCUEyGIWViaGEzf2M+yYdofh3zUB"
    "hr6/Dxny/n9g+PEH0uBDLv5Bkc0Iyu9cHAxM3EAPc3EyMHJxMTDyAYswAWBN8A7Y93sGTKBP"
    "X0DyPCtKR/oaELdCEwI8ESwAVhiJDwZvAgAIIBa8jTVg5C+RggTSt3+oCYCM5gKoOAU14fnU"
    "Pr+2v8cuzPKf8zvDb6bfDJ+/fme4+O4sg56sBbAh9x3FocTW/7gSA8NvRgYNJUOG94nvGY7P"
    "fcdQdFWBgYX7JUOWthjDLwFBBkZBfgZGYPeSUUQYyBZgYAT2Fhg4OIA0MAFwcoATAQMbK6QY"
    "AAXc23cM/+/dZ2A4chKITzAwPP8AlAeHpBaQXArEAUCcD8TPgYUWgz0w7bACw+r3IF12ARBA"
    "OEsAUOQvBtaQP6ARDsrd6AmAEVI/8kHrTQGgGl5wPcoIrkuBSYdBDErzQutcXrZ/DPyPuPm5"
    "ck1CGH4Iv2b4bXaU4cufzwwhijEMFWatDMteLWV48+sNsN3PRFb9jy2xgPz1m+Enw8Hzuxju"
    "LGJj4HwhybCqLpjBx9kAmJN/I/X9gPQ/UEQDY+7vXyANxP/+QcRg8tAxBXCAPHvO8H/HfgaG"
    "zTsZGF5+hIQGJAhBw0UJQHz4GjA9G12H1CKDJREgxzlAAOEsARaIA3P9f5RuESid2wCxHahh"
    "DzQD1DrmB4Yt738QDUwEoHACp6H/iGrzPyOkaAWFJUgQNIbH/+cjg9C3DwxPOfkYfn9lYWBk"
    "Z2Y4/PwQQ/qftwxyXHIMd7/eRVQDFFYFsBKCBdgwlJdTYXjheYrhyzI2huzujQzKAhwMmppy"
    "kEQAy+EMSJH9nwHBh8n9BSUOaJtPRIiBMTmSgcHVhuH/knUMDLuA7cF/4KoBNHa4FRgeEcDk"
    "so0BGvnqwEx18wcdYlgNiH3RxDYDMZb2CEAAYSQAFmCEuXJB6nikyI8AcouBYWCC3FhigDWS"
    "kBpNWNsA7EBzeTnB9SoLNxeDEC8PgxYPE8ODH6wM/94JMPyW/MBw7+MDht2PdjA4KbkzHH17"
    "lOHdr3cMzIzMZNf/2MQ5OHkYuMTYGL47PGN4tIaNoXbhPoaF5UEM3HzArPvnH2qkIyeG/1BP"
    "Ild74CwNTAi/fjIwiIsxMJZmMvw30WX4P20BA+ObzyA/8/5jZFj86R+DK1DnOZCWWGAjetIr"
    "BoZXf2gY+cVArItFHCSWAsRodgMEEEoC4AIm13phYCONE9LXAbV5gf6cCqTTwWEAKi3/QBtL"
    "QJ0sfGwMzMBIZQbWm8zAxhIzHy8Dq5AgA4swEIuIALEQA4uQEAMTHzdYHROwXmUC1qlMwgIM"
    "jkevM+xYcICB5ZU4w2/R+8CS4Q/DqhvLGewUnBj0+PUYtjzfAh7cgcQJ/kgntofAyMzIwMrM"
    "zfBXFtid1/zMsOnkI4YtBy8yhPuYQYp9eO5nhLTgGKHJHUQzAZv//4AB8O8HamIABcwvSLZm"
    "dAIWkNISDP9bJjH8f/KSAdi2FQKm86kcjAwuX/4zfK0BWivITKtcD3SQDyiige7BNe0BGt34"
    "gCoEEEAoCWARsM5XBxb0n/+CSwJQ5M8EhkkyyI//gXUZu5wIg4CzLQOvhSkDKzCSmXl4gLka"
    "GPFAzARqLfNyI+UaGAGtV8F1KbROBRa51kriDOJCHAwv3ogy/PvEzfCf6w/DyadnGdbdWsPg"
    "pxHAcOXjFYYLHy8wsDGykVj/w+TREg6oRPvHxPD9509IiaXxjuH3NUGGTafuMXhb6zLwSItB"
    "3AmOeCbIRCksEfxnYvgD7Dv+BfvhJ8OnL18Yfv36DjaXm+UvgyAXyEyguq9fGBjVgaV/UTLD"
    "v8ouhn9//jBoczFY+PEzJC57zzAFFCLv/9JgSrcQ2F7SBVW3eCIfqTZDBgABhJIAlIGN3S9/"
    "4WrD4JEPTPji0X4MUvlpDKxyMpBIBEn8hUYoKPeA6G9fUetNWN8KVrQyIYpYEQluBm9DGYZZ"
    "ez4xcD2XY/ikeBFszIzTMxiURVQZ7MXtGZ7/eM5w+8ttBlYmViy9AhifAR7B//8j9R7AfEZE"
    "+w4Ymb9+fmf4/O4zpBjkA6Zo/l8MB29/ZSja/JNBSluI4cvP/wxPvwCbn8BI//H3P8Ozr0zg"
    "NAFkMnz8ycjwF5zt+Rk+/RZn+AWMXJDR3MBs4iP3maHN+CGDGPcPcBgw6msC+81mDH+3HAOW"
    "egwMGaIMqSvfM2wChtYjqrbgVZkYGL2BiVQHFPH/UQczGBiAjROGaGjbbRFQfBm0skYBAAGE"
    "kgB+gDInEAO7zixAj+eDEv8/YDhJxAQzyHfXAgMOmBI+f0FtKMEimRVLzgf6+Ce00fMPaOCH"
    "HwzwiPr4l4lBWVaLgYvrFgPzKzmGN8J3GX5zfmF48+EdQ9v+NoZa11oGdyl3ht9P/jJcf38L"
    "7DNIlcAIN+M/UhWNKoZQA+oB/AfNJQAd8urJPYbf339B+u7/IAny6cc/DLOPA7PNK1YGLh5O"
    "YGMRNJLIyMDPASmr2YFFIS+wkSoETINCXMwMAhxMYN+J87AA2cwML77+ZZh17BPDtY8cDAc8"
    "LzGwMQFT119mBiZfF4a/B08B24t/GGx5GfR8+RnCNnxk6KFm5DOVsYDD/v/P/8gRrwzEwOKH"
    "IQguxsrgyvCYwQjo8Ar0UUuAAGLBUUKAahKTf0Cl7DJiDDLFleDIZPj5Hda2R2nq/wA2cZee"
    "YGI4ep8JHMlvgBXeP2DA//rzn+ETKOeAim5gxL3/DslNwNqY4fNPkDkKDAK/VYH9xPMMYk+0"
    "GZ4qngSXvLef3Wao21rHEG8dz+Aoac/AzyrIcOHVDYb3wCIW0oVhhPQsUHocjPBcD/ELI1jt"
    "H2D9/P3zO4Yv714x/P39GxFQ74FN8k+sDKqK3Ay53iIMZvYyDJLC/MCqD5hcgAlAgB1S/APj"
    "nuAQOajamX3wL0PLBRmGJtN74IYho6ocA6OZHsOfPeeAVSUDQ6EYQwwwAaxioLAUYFRhZmD2"
    "AkabDtB/P/+h5/oUYMS3AbEoN7DdJSYtBfQPI8ODB/cZfgv9LmDgAjZGPzGsYYD07sEAIIBQ"
    "EsDPv5CWPzCz+oMagKAEwKcmxcAqowisG54gRfx/REIAduy7d7Iw1K1ngQyTsjGBpTiAuYMd"
    "WJQwA7n8nCwMzMAGmJQQMGdxMoO18XMzM4jycjCw/fdnuHLhPsPPt8C6lEOZ4b34XbD+By8e"
    "MHRs7WCwVrNm0JBVZzCXBCaQb68Znn9+w/Dpx3eGn7//Mfz9A6mFQD2zP7//Avl/gc2MP0D6"
    "N7Cx+o3hz8/PDL9/fAVWYX8QAQVy42cgcUEK3PI3UZJn0JbkYTCW4mBg4WDBWmni2zwDShzt"
    "TqIMe+9+Y2g9K8PgLP6WwV7mPdgiZl8nhr/HLjL8BrrLkodB15GHwWf/F/CwMXl1PSjyS9ix"
    "5XoZoFO7gbkrgomJiUFESoxBVEqcQUNAkOEDMNG/e/Oe4e3v14wMksBu/AvwHMY9mEaAAELx"
    "8T9I1w9U/PvA+vBC4n8grV9mYN/wD9IymN9fII0Dpn8MH74AkziosgN2EdIcpBlyXSQYeIBZ"
    "B1R8glwpAIx00PgJC9BwZoz5VjmGTeqJDHkzOxgEnqsw/GEG1tOCz8Alwc/vPxn2ndvHcPDq"
    "QQYRPhEGTi4Ohr+Mf4EJ9SewdPnL8Ps3IzCCfwMTwX9wxP8HD978gbZLGFAH+WH4LTAAz8gz"
    "MDzlZBAW52ew0tRk4ODhAzYLmME1A6k7pUDqhYH+m+IlyuC14DlD1kk1huPCZxj42L8zMOmo"
    "MDBa6jH83nuegQvYASoRZ4gHJoAN0FlFEiIemIHcWRkYdViAEf8XPddHACOq5//ff9LcwO61"
    "mJwUg4iQAIO/hASDArBhPvvhQ2hRCRrA+g9spf8XQk4AAAGEbSAIVPwbgMKQgw+YU//dBwbW"
    "VQYGKR1g0fYa0igAt5JBDT9gEfGHkSFc7wdD/2FgSx4YIRcff2XQkeLCHWDA6oHhBpAWBxoj"
    "Cym6/SzsgcX+Q4beDQsYhJ9qMDD9ZGP4JPQI2Pj+B3b3X6CnX758idqKxVUqMzJgzu78hRb5"
    "jwUZGO6LABsgjAycvGwMqW5uDCqysgwiYmIM7OzsWCOfuATxn8FThZchw+oLw4wD/xkazisw"
    "9FkCPQlsvLJ42jD8OXwePPfgystgBiwF/ICJYAbRka/MwsBaCAxb0CALaq4XBVoLjPj/cYzA"
    "3CUmJcogJC3KoCYA7HLIygO7u0wME+/eZXj86i3Dp3efGJi+M//8d+/PU+gMBhwABBDGolBg"
    "7vcGYmZQZuIHRhLLP2DL/voBYAkAzDlMnJDiH1xUQM0Bekxf6heDvvRvcHfp3P3PDKfvf0Fq"
    "jSNhBogH/u8A4nWQRuJ/aKwWB8UxFAREM3wDNtZ4X8qB2wQcnwQZGP8wIVyKjNFL6v9IkQ1q"
    "XwDrd4YXwIR4XZiB4ZgCA8NhYPfsqjC4myMvI85QGhjMYG9gwCAqLs4gLS0Nref/w1oPkMEs"
    "UPcfWGJhx4jhcZgjWh3FGFRlWRn6L8kx7HkEzGh/vzEw66kyMBlrMPz5AZkTAPYIokBtSMIR"
    "z8rAlsnLwFbAC4n4nygJMQho3Yl/v//HcXJyMciqyzPIKcsw+MnIMNSpaTLwsrAw9N+9w3AP"
    "GPnP7jwDd5wZdvzfx/Dm/130dQwAAeiyepaGoSh6sDQFY0u/FEIotKUOdXBQOhSH4lQLDiL4"
    "CwQRHNzcnRxdHfsPdHDp4iS0gxRxERGsSzsIfgzmJaWvL56XBj8KEu6SR5J3c27uOfdkugPE"
    "iO1GAJJ2Ou2QM+8ugdqedn7Y+l9DXzcyCSURY2NpLHq47RlkBomL7hsqhblvufgHKG0VrE7+"
    "ovs9LWp0x1bQ3HW0vYu0GcfJ+RncjyiSoggZczA0yOVRATUzCozdn+AxVoEIA+d0eATd49Qj"
    "GC6/eMH0Rjyvi4i8maegrZWXUV+pIGdZiJMji4UCTNMkT1NTMPmARbjPT0fC9caQMrCQ4BEA"
    "va7CYhOuXlOB/tASI28bKJdmSQULqDcH2O8soZNuI5vyYWytQ9w8QPBemwlUqyYabQfN/8CP"
    "FAn+YSKc69Xvgs/w2cd8/Qe68ubJ9alcBqVUAjuWjbVkFr2hg9OnRzy/vGNw32eDZkJX/rVq"
    "yRav70/Tz5cAXJo7S8NgFIbftInpVYvEWx3qlToIdqqT4KCIoyC6u+svcPAP6KBIF/0FVScH"
    "BwURFyUoXbpY0SKo4NIiik2T1vckoV6yfHyBkOS8b77znHznz2bQaVKZ43DIeEbDFGpylq6V"
    "KlJhYFeZugazBCiWZM67Jz4dDufLrTRNip/d5apEOEunoihsZAiBilv+4f+a88TLcgxrhtMl"
    "pfUFiwnkMIu32Drew1X5BpValfcn3Cl2q2PYaVqwm5bfUWR5ismvSxHc0r2uYBnr7YhrBkaN"
    "FMb700gnh5gfuxGJR/leCSQ6ewinumskEVolszRpRk3qYEJrmLmdAXIBVvZ/5Jy0FwrYxlgG"
    "hsg5MpeVQA16+0SBQBBrJ6/YOatiZeIB+1MFPp+Gz/Uc6tclEM6Rr+Bi+RGLfnfRj/DDbdBm"
    "IgiMaUQZxzOAPB1TIVPAPCfbDbsxEorpMAYM8lkHpo0uLPQl0auGULY+sHl/h9JbBS/FZ7IS"
    "Y3RuXzoHtSM/75tigt+afwsg9EZgGtAv3KBcICgGjHwu6Njxd2Do3tjLwKBiDXQl0Ad/P0HK"
    "WkZmSMwB1ehJ/mIwlPoNrAJYGW49/c5w7O4nBkcNAch8Anp9Daz7mVSBJtz4x8D4HtgKEITU"
    "Kj9//QV3Fw1UDRn6YjsYth45zHDw2jGGc8A2yLufrxh+/QcWqUDLOEDuApYGjMA+NzMw1FmA"
    "mB3YvuEXEmEQYBFjEGIDRvI/QQYJIVEGCX4xBgE+PmAdz8HADezn8/PzM4iICjLw8vAAMQu4"
    "9ABHLKiRyoLUfCC0OBA22MSAOhYBahs1OYgy7LrzjWHeNXkGX6mXDAHarxnYvK0Zfp27wwAs"
    "WBjceBms9DgYnC/9YFgBj3x5NgbOfCFgYxY6GIPI9QJAS+qArd8cUP0tKCHIIKQkxKAIzPUh"
    "EtIM5oJCwDBhZHjy+zvDdGB37/arDwwvr74AGgHMMKf+nUOK/AvQEgAFAAQQC5qf7EANRtAY"
    "CTCjICVNIL68jYHBKRdSDfx8Ca1sodUAsAXOxvWPwVPtOzABsDH8//6HYeuF9+AE8B1YhD16"
    "9hMcOD9+/YeMBINXhvxn+Hj3DwMjsFf6WxNoLNDiv0DPMwNjAlQlcAK7ZNoqDgzaCiYM796+"
    "Zvjy7T3QrA/AcPkLzJks0NwG6nUCA46VExh5TOAczMbGCszBrAzsHOwMbOzABhSQz8nFycAN"
    "bIZzcLBDJjGgM1n//0G6s8iRCO/8/SVn7hbUG/kPHiACVQVu858y5J/WZrAUO8QgbqHOwKyv"
    "yPD77H1QKcCSI8qQkPaYYQd4dF6fk4EzmA/StUMtMe1BuR7Y2dJj42JjEFEWZuCX4GGwFhZm"
    "CJOWZRBlYWcALZF99fsnw+T7dxiuvn7P8Praa2AXGRj5Z/+d/7Ps+0qg/ofQVUsPsbkYIIBQ"
    "SwBgbDLCRtCY0FQ9BRb9D4AliKYTMCaBWfD3R4giUCyAO+KMDIFa3xna9/MCe41MDGvPvWVo"
    "CJADj/6+/fifgYMNVFT+Z+DiZGTgBvVIjBkZZK5zMHB9ZWZgNQD2PdlAljOCi1JG2HwM2BGg"
    "7C4GLKJ/Mvz5BawAfv8GJxTEjC0EsoHqKkZGcDUCwqCVwizALI08iAMey2f4S1QrnxF5dRMj"
    "7sEgcBUHjTdQW+DPX9BI4l8GFyUehnwbPoYJexgY8k7rMKx0v8jA4W3B8PncfXApEMjH4DiF"
    "k8H90neGlQxCwJIImDkYOOF2cAHL/FpgIiwC2swmIM7HIKAiyCAnxAvO9VZCwuAWEKhF9B2Y"
    "+WY9vMdwBVjnv7nyhuHnT6Dhl/5d/r3s63KgV0E5/xIQ41yTBBBA6CXASaAzPEDB9PYVA4OE"
    "IlKXCtQYuQ70jZYraG4X6Nv30CwDnd4CelxX8iewGvjFcPYBO8ODZz8YDt78yOCtL8RgZciF"
    "dSbir9E/hj/HgUXwS2A1oMwIblv+R5mGRwxdMwF7HeycbEBM3GIH8MTRv38oEQpLWEwoxTsj"
    "sv/BpQLIBGAVDEx0kLQNavCBGoWgqhE0ugkS/wUehPoPlPsDVgsbkQTpFwFGqJkuN0MDqCq4"
    "+51h1U1ZBh+ZFwyxlsoMzJpSDL+A9bMAFwNbshBDbP5Thm3AsENeSG4FxL3///y3YGVnZRAG"
    "Rjy/JA+DmbAQQ6i0FIMkMAC+g6otoIe+Aev4mfceMJwHRv7by+8Yvn37xfD/1t+bv1Z8WQWM"
    "/AfQnH8XX3gBBBD6UPASIOUIDBP2V8C2ohqwjcfKAY0HkMorwBLLFZgogQ0ohh/PIANB4Bkz"
    "SG8AVA04KP1gOHuPAxx6m86/AycAUKD8/fcPw+Z/OkDxY8AAPw/0kDITzhkrjIWMyAs5mVBz"
    "J3JGhVkJbqn/huRQkNjnz8AW/k9IxP0E5jxQZIJb9sBW/y9gQv8HnWQCrxAHllqgEU1uLmaw"
    "XazMoKXnwAYiO7DUYWNi4AeW56wskJILvFCICdomBVrKD6yCpnqKMTgDq4JSYFXgIPuOQTrQ"
    "guHt1XUMQCcwBPEzOE5+xeB05x/DRqCFoNZoCTDiqv//Z+TkF+NlEFDmZxAX4mbwl5RgsBcV"
    "AbvoMzDHswI9+fPfX4Y59x4ynH71DmgesHr8CvTg7X+3fi76tIThx3/QquXL0MjHW5cBBBB6"
    "I3ATKJqB5ht/+gQtBRSgCQCU0V8Aq5H7p4AtPh9gaHFDuoSgyAeFzD9Ivz5I+xtD7yE+cNdr"
    "28X3DB++/mIQ4GbFnIcCtXPkgEEsB2ReA/YIPgCxALAIxzHCg5w7IUUtJGK/A3Pn5y9/wEUv"
    "SPwXNKJBXTZQ7vz9C9anZ2LgALaU/zMCcwnzbwYRHmAiBhbVjIw/gXUypBEoI8EG7BKyABuV"
    "DODWPShCYYmMCXmVC1KOAdmDPCmFnM7/AjOBA7A7XGTDz9CzC1gVHNViWGf1hoFNU4Lhx/UX"
    "DOLAGrCEhyEql4fpMwMfc+3/n/8cWIC5XkgR2HuR5mEwEuZnCJaRZJDi5AS26CE7H5ihkb/4"
    "wROGUy/fM7y78oHh2ydgq+rpv0c/l39aAYz8u9DIv00o8kEAIIDQxwE+A3XsBmJjUHw+fwxN"
    "ALDsBirqrgJLAX1gAmDlB4b0G+jiCGg1ACwejaR+MuhI/GK48pid4cmL7ww7Ln9giLAQgw+e"
    "oNSdwGr7vw6wj73hL8MPUBvVjInh07tfwD43sGr4ywiMzP/gHApSCoron7/+Qad4/4Nb7qCc"
    "yg7MhVyc0Jk7oHmgtgYPsC5lA+ZaPj4uYCQygXMtSN3mW1sZpp2ezfAR2LXUE9diqLYvZVAQ"
    "UIIvk4HMav9HbdlDB5f+MpDeKAS5mxlYr9XZizLsBPYKNtyUY5gpr8WQ4a0HrJlfMLwHuvWl"
    "E48fmy534P+fDKy8wlwM/CrAXC/CxeAhLsJgLyYCjHAmhq+/IXUMKPJBi18WP3jKcPjlW4b3"
    "1z8yfHsPjPxnfx/+WPx+8f/Pf2+CMjADeKwVc+oXGwAIIBYsOQ3UbSgFRjjzy+fAIhJYO4EG"
    "euDVAKgd8A6YMriFgNUAqEP/E7GCBlhJcvD8BfcGrjxkB4fAlgvvwAkAlCsv3/wOHlAB5dYf"
    "P/8Ci8n/4CT36yvQ8C2gNZXMwBY7MwMbKNJY/oOLVX5Q7gTayyPNzsANbv3/B0coOHcyIUbr"
    "sC1+gOVOUGNw6YWVDPFrs8B9a3ZgLjt1/xzDoXunGBb4TmOwVDYHuukPwWFf8H4DWAMV2lVk"
    "ZsIsqWD2gvwMqmr4eFgZpgJ7BY7znjBUnNRjcLZxZvhpeYGhmPc3wx55bg5eoH8E5XkZeGS4"
    "GXSBDT1/GTEGBW5eYPgwMQCbvuBBL0Zo5K9/8pLh6It3DB+uf2L48hoY9u//v/6+7P2yfx//"
    "XgdlT1IiHwQAAogFS7idB+ILwMA1/vqFgeE1sMcno4KUAEACd08wMJiGAZM3sDH45yuiGgDl"
    "JGCkBml9Zeg+CKwGgLl415WPDG8//wLPCIJ6YVwcoJzKDKw7OcD9blY2YPcPtATp0j8GDllg"
    "5Csz42htw6oARqQqAVrUEuiygSJ39qnF4JhT0ZdlYBdiY/j44AvDvTuPGCJWJjPM9Ohn8DB0"
    "ZfjzD7JCGL20gtkPW9gEqmZADUJQFfMJWJn/BpVUvyENQ1CpBSqtfv+BVD2gtGlpyM1gq8DN"
    "UO0owdC06zVDyNn3DD+MuBjufvvDIMrDzsCvyscgIsLJ4Azs4zsAcz0HE6jIBy1G+QMZxgXn"
    "fmDOfPSKYc/ztwwfb31h+PzyB2j7+6tvi98s/vfuz3VoxF9jgK/mIw4ABBC29QB/gGGwBVwN"
    "gAbtgNW+jDKaokubgbLBoGEJyJgA8vQw0OMG0j/AA0OXHrEzvH7zA1gNvGeIthJn0FHjgvS/"
    "/0OLWujawt96wGrgPDDwLgIDTQko/uc/AwOVllCzAVPZ04/PGZ5/eMnAy83NwA0MaDVgUXuP"
    "jQW8jOvF3TcM6duKGKb97WFw13Nn+AJsLX7+/Bvc6geVvN9/MEAbi8Bq6sc/cOnFBN1AyAZM"
    "vFzQMQlOYOLmA2LQ6KAAMHGzsjLAG4eQ2ThgN1ntLcP0Y3UM118fAE8+ySkJMHDLcTGoCXIz"
    "+EoLM6jwCQITNSfYXsj5CJAhb1Dk73zylmHvs3cMn4CR/wnYwwK2Gt5/Wfx68b/Xv0EtfVju"
    "/01q+AAEEEoC+A0eCACHPWjRQDmo3QTK8F+BXX5ufmgbAKTj1iEGhjfALqagFDAbALP036/Q"
    "MV5GcMuMg/sPsBr4ynDpAQc4caw88QaYAMTAkf7nD1rp9Ac08QEMUAlgI+7qXwZWByAbtMic"
    "wNo5+B4F8HpNRtQFu9AEBh5dBK3H/gdsCzBygicvfwGrHhVuLgYtXm6GdcDIZPzPxPDq7juG"
    "9K0lDMmXvjHYqLgxMLH9BkcuGysjuF3BywraXc7GwANqILKAEhWkkcjICCkpGNHqfVjiBlUZ"
    "rEzs4IhcemY1Q/2eNoYfX54xCIsIMvCqcDMIiXIy2AP7+Pbiwgy8LAIM//4CG6nAXP8XPMQN"
    "WdAOWoi2Cxjx25++Zfh0+xvDp8fAFPmd4dPXla+W/339C1Tf34Q2+n6Sk0EAAgh9HAAyW/cf"
    "XJScAbJtvn0HNlReMDAoCSIN/n0ApojbRxgYrJMgI4N/PiKqAVBnHjTQAawGug4AUzSwKDt0"
    "8xPDQ2BJIC/CDlmihVzXgtqQwIKBWZeJ4e+Ovwx/r/9jYLdmBnfBMIphaOCC++Og1v5fSCPx"
    "06c/DD+BDUfQGMgPUCQDU/KP7//ANCiBcLJyMShzGzLcfX2b4fvL3wynBD4z5AL7479BbZT/"
    "oNVFjAwv77xnWPCkEdh/52DwNvcEd3GR3fkPqYGIaO1D3IJ1Ng9oMRsw9Tx885ihZkcbw5qL"
    "m4ClAguDsLIEA7cCJ4OKEBeDuxQ/gwa/MLChCAonFtBpKOBTUf6BF7/+BSayXwyHn39g2P7k"
    "HcPn+98ZPj/6zsD4neHLl+UvFv958vMMdKU/2ZEPAgABhD4SyADNoCD/rQdtBAF5+NF9BgZF"
    "NbReENBDDOYxwAod2D/9+RR1gR6oGpD6waAv+YPhwiNOho/vfjLsvvKeIcVBChgh/yCjfYyw"
    "AZ//kL0U2owM3w8BI/PcXwY2bWDL98cfcMSCGlKghiOofgVF9lfQMBooEf2DTC2DcimoGAY1"
    "CkG5FdRoBLX4+fnZwY1JUI7lBZbPfHIxDEdmb2f48vg7w3Ng/+v0q88MjtL84PbD9n+Qlb+v"
    "7nxgyNpayTAZaLafmRcwgf1iIP0oXWAjk4Ud7LeVZzcwNO7qZrj96j6DsBg/A78aD4OAKDuD"
    "mRg3g5OkIIMQqzgD4z8+yNaV/7/A+6XBhT4oEwFj+sizjwxbn7xn+HzvO8PHu99Bo60/vq5/"
    "ufrPkx/noN08UPH/nZIqEiCAWNA5oBFJFsiysA1Aqg6YqfnfAHt7n94DA1cIqRoAjQe8BLY7"
    "xFQgE0S/PyHW0AM1swOrAQ/VrwwXHnCCY3rVybfABCABLgFuPQCWYZ//AQOYERypP4DdO3Cv"
    "AOjx3yeBVYggCwO3EgsDO2iSB1gUs4OLYEYGTg5mBj5eDnDdCopw2A4tXFUArBv3688vBlM5"
    "IwYXVXuGTZd3MvC/5mU4w/+VQQ/YFrCR4AWqBRaz4JKAieH17Q8MuVurwaOIAeY+DL+A9Qb6"
    "iCJsIyyyvbDJIBZmDoYn758DG3u9DMvOrAWqYWKQVRdn4JLnZJAVZGdwl+Fl0BQQYWBnkGBg"
    "+scBntkEHZIBivq///+Bj75hYvoCTKAfGTY/es/w6dEPhk93foL2OH7/suHZil93voKWdN2B"
    "Tu58pbSNBBBAKAngLjBzySAa4feA3j4J9LAbqCH0HJjJ+UWQqoGPXyFtAWl9oK+B9cOv90gT"
    "CJBBIQ+1bwwdB/6BG0FHbn5kuPnsG4OaJDfD56+M4FYzNzBt8AEjlo+XnYFHiJnhH7AL+Wfl"
    "bwYuYF3LbcWK2GmEpX6FLU38A1u3TSBXsrGwMSSaRjLsuHKA4dvjnwxPgTnx2rsvwPpXm8FZ"
    "ShhoxE2GvdDq4PXtjwzZm6sZPvz4xJBgH8XwnwU0//AX0gP4A+3eARuEHz79BiZcSAn1/x8r"
    "sPXPwHD0yTaGtQ8mM9x6cZdBUJiXgU+Nm4FPgo3BGNj4tJXkY5DgkGZgYxCDLFgFltyQIv8f"
    "NAGApr0/M1x4Dcz5jz4yfHr4A9ji/wFy058v21+s/3X7yzGknP+VGo1kgABCSQArgR4o4YY0"
    "JRkhqXo1MGjdQJyHDxgY1DWRhlpBcX1hI7CSSAaWvaLAxuB9RL8M2hswl/vOoCH2k+HGcw6G"
    "7x9/M2y/+IFBXYqHwUSXE7yGD6V+BbZ0/2kwMnwRB/YIrvxhYLVjYmDkI9wYJBb8+v2LwUnd"
    "msFS0Zjh6N0zDDxvuBjO8n9j0Bd8z6DB6QsskkF1/j2G/cDqgAnYwntz4yNDBbDu/vjlH4O1"
    "bAjD26/fGb59Q7iZFVj1cHOxgKsdET5uhvc/XjEsuTSDYf3l1eB1klIqogxcSuwMkoJsDI5S"
    "nMD+vSgDL5M8A8t/XnCuBzf0oBH/D5rz/zJ+YLj85gPDpoefGT48+cnw+eYvYCZg/PN5x7N1"
    "P658PAwqd6E5/yO1lpcDBBBKArgF9N8dYICrMUM2ugAz13ZghL8BZmyRt28ZGN69Y2AQFocO"
    "nIF0PgImxCeXGBjkDBgYPvMCQ/kdtCHICM6VHNy/GbyAvYEbjznBKWfD2bcMuW4S4MT1+w/m"
    "TByo9c+sycTw5wCw7r8JbAxasgAbR8TVwSi9AlA3jRHRk4Q0HEHdKQ6GcP0IYGl0luH7498M"
    "T0TZGC6/f8ggz/GSQZM9muGP5AJgGn7McAA6q/j22meGul0dDNkGPxhyPGMZOHhAi0H+gXeL"
    "gwaA2NjYwI3fAzcOMlTv6GK49PAGA68gDzDXczFwA3O9nggrg60UL4MUpywDF4MMAxNohxEw"
    "1//9/wca8X/hOf8P43uG2+8/Mmx7+IXhw+NfDJ+u/QSNxf3/vOfJpu+X3u+Djuufp2bkgwBA"
    "ALGgjwNvBRb3mtzwoaSnwLA7BAzLINCOqiePgQlAAqkEAJ29d+sAsIVoAWkM/noDLTug++yA"
    "VIDWZ4a+Q4LgRHHi9meGi4++MRiBlov9/YfZwALtLwE2AH8cBzb4LgEbg8bM8OEFSMkD2bSB"
    "POEDG3n7DR2IAa05+PDhF8P3n6DBGUYw/e0r6Kyhf0Anf2XgYzJl0BTTZrj34jYD1xsOhot8"
    "fxi0BI4ymHLrAxNBBMNfyeVAMx8zHPwH6b+/ByaCRbemMCjJsjAkOcaAZr3BiYmNlZPhzce3"
    "DL17ZjHMPbocaP5vBlFFYQYuZXYGMWAbxkaKHZjrhRiEWdWARb4QuIX/+/9vaJH/F3zQHSzn"
    "/2J4y3D73XuGLQ++Mrx7Boz8qz+BPRSmP98PPd397dybvdDpXFDkv6P2tjKAAMIYCbwPLAHe"
    "/0NMTQMDeAUwHIJAvbxHDxkYdPQgfWAG2EwwqBqwzwImAGDR8OUOZFk2tC8JKinMgNWAlsQv"
    "hmtP2Rl+fvrFsPX8O2AC4IN3B1Fm9kD2KgJziRITw+8H/0DJj4FJkZHh9zdIiQGaigXVu1+/"
    "/gP370H17/cf/8E9Blh2By3fAs3ccYJm7DiAPQBgG0MA2ADj4gLNCfxn4OMRZnjBFMBQsbqD"
    "4deTPwwPhVgZbnx4ziDNdoZBjs2GQZ09GJgI1gDd9oThELAYBBbBDG+vfmWo2TYR3K1Md41n"
    "YAM2Rg9cP85Qv6mX4cy9iwwCgrwMQrq8DFySLAwawqDI52SQ41Fg4GNUBuZ6FkhdD871kFPO"
    "kBPBz/+vGR59+cCw7f43hnfPfzN8vvyTgRnoHcVjT36cOfX6AlKD7y0t9pQCBBBGAngJDPfb"
    "QAcYsULaAsBw3QsknoIOUQJVAW9eMzCIS0OrAVACeArsCTwC9kpUrIDZVwCYdV8glu0CExM7"
    "F6ga+Mxw7REHeJRpCzABVPrJQFf+QNYMgoZPQT2Cr9/+MnwCRuYHrj8M758CA2wDsFNvBEwM"
    "3/7Bi3TQxA8nBxN40kcAWOtIi7Mw8PFxABt5jOCiGbI2D6kKgI3N/4Os9/316zdDoKEnw5yD"
    "KxmePn3OwCHLznDl9S9gf/wcgxCzJgMnswiDGnsAwz+JdcBE8IzhyH/wiATD28tfGVp2TwMv"
    "Wfv0/yvDlH0LgKXNTwYJRREGDmVWBhFgI9ZCihVY7AsyiLJpMHAzioNz9+//P6ER/wce8TD6"
    "x/83DA8/vwNG/g+GN8+B3d6Lvxh+/WVhUAZ2mb2vveb5xs4gc/EnuJ//ila7yQECCOsBEUeA"
    "MW+IOAjpHTD89gEDNBZUzD58BEwAMkjVwPe/kAkiNQdIYxC0TgC53AZK+2t9Yug5JAysEpgY"
    "zt37zHD2/hcGc1V+hpv3/zBcuf0NMlwKalgB++08vMwM7IrMDNwifxg4nv9nkAPmXk4hJvCG"
    "I1Zw3YtaBaCOvEGGbWG9AmytB9A2dCkBcYYQY2+Gjs3TGH49/cvwQJCZ4fbHxwySrDcYRFg1"
    "GTgYBRlUOXwZ/kpsBJr5nOHof9B0Ng/Dh6s/GJp3TgFWN38YOHmAbXkDAQZOKRYGFWEmcOQr"
    "AXO9EIsmMJ2zA4vwH+AEgB7pMPa3/68Ynn59w7D17g+G16/+Mny99Auc0D+8lmO4/UWHgZ/v"
    "CUPe7+8Wma8YeIG12nNaJQCAAMKaAO7+hRyzy4mompcBAzcW1L4DbTQxNGYERwa4qmcDyl7e"
    "wsDgUsDAwCHJwPAJdB7KL2joM4F7A6Yy3xm0JX4wXH3CyfDny2+GdaffghOAkhwwEEWYgTn3"
    "P3gFMaj4Bm3DB+3j//yMmeHHSWCX8BWwMSgP7CKCl2VD1gDg3g2Cd00J0ha4fwwhpj4Miw6v"
    "Y/jy5AvDJ2lWhmtvfjMo8Z1n4GNWAA/MsDPyMyizewITwTZgInjBcByYCEBdxE9XfzHwyHAw"
    "cKqyMggAc70RsE1kIC7AIMmmDWzlS0NONwZGPqzIh0Q8rAT4A16E+h1Y7D/9+pph1/1fDK+B"
    "Re7nC7/AM6J6XEoMN36pMDyRUWLYIcrDMOfyJFUPbobwTV8YmmmVAAACCOtp4bBqgAUxuHcE"
    "iO+AEgBoFPjlK+imYyboPpNXwN7Jg9PAFAMsAdiEIVvJGKDr7/6BppN/M3irfwLPDoJWW4Cq"
    "gW8/fjNwA4tyAV4m8MoaUPH8GzzpAtmjyKDJDGwcAev388B8BN7SzwjdsoHYB4K+K/gf7Igf"
    "jN3C0G2BoH1vwKoCtJxcXVqBIcDYDTyf/vvFf4Z77xkZHnx+xPDuz32wnp//vjKwM/ExKHG4"
    "MZiIizFYSDMycAETCr8pJwOPNjuDqjQzg6cyM4OVpAKDArstAw+jLDDXA836B2zl//sDTPdA"
    "/A/odhD7HyQBgFZFff33kuHJlxcMO+/+YngODOhvl4CNwE//GLQ5FBm8RI0ZMt10GNillRi2"
    "6mczrFV0Z8jmZohmY2RQplUCAAggnMfFH4UeowbNcMDeMHihCLiIvX8fOhQGmjlihuRyhuu7"
    "IPMBHBKIXsB/2DkC/xl81D8DE8tfsJ4bT74CG0+fGRiZIBNIsKlWeKQByzwWBWYGZllgIgBW"
    "E78fQ/ShRyzS6TTwJQmgCAb10UEYxAZVF6BBp0/AhuTjF78ZLt34znDq4neG3Ue/MogzuTBw"
    "s/Ez/Hzyl+EjsMi78fonw7Ofl4D19g+gl36DEwEbAw+DIocTg5G4CIO5zD8GcQlGBnP5/wwu"
    "SlwMugImwGrDDJgP2MFL1kF6QJH/G0qD+H+giQG0Q/rrv1cMz789Z9gP9NPzl5DE/e39HwYN"
    "YDfRQcCAQVJekSHR25yhwkcH3MZqMihj4BMRVXdhZwikVQIACMDJ1aw0DITB6ZqEmBZjq6HS"
    "IhahKC2I0KNUH8OTFy8+gfoKPkwvHosnb+boQRH8JUixtFZR27QkJk42saigB/cRZvf7mW9n"
    "vl+XRN0mZUAPxhU9KgM7BFk4Tgj3TSAyj8hBTbSC+4IP4IWlyiiQqRqxh/Az9HyBWnGAsjXC"
    "ZVtnOvfQsB+xXsmOzaLhj1FuijRErSoYXfsYnvtQFpVE0PldACKFF1I5FP/F9xjRfZeRRTrb"
    "HxBk9ihh0iPozDgZMgRDJzMwfWyWV3HWq+PQbmKyreHGFHBm72ApLWQmLF6aJ7W3KtIoaRtI"
    "zdlYmnnCtJZDXq1CF6aM+EDuL35P0v5XmhebWUJ5+R08uC0cE9j7DvE7JZN5DrCcLmJtagVW"
    "YQFmvgIjO4/9ukDz6hUnTgkHtV1sd/e2joZoeH+oe/97PgQQzgTwClQNACPXmAXaG/gP3lVy"
    "FZgAdD8Cq4EXwMa+oiYjZM8aKzBW3j0BphpgB14/ENIY/HoXugYfUg1w8YKmiD8z3H7CBW6q"
    "bz73jqEl/DcDLwczOEJBM3NM0MEc8IIM0A4VXRaGX8eAvYDrwGLS8h/DL2BZ+B3YUHr/HtgV"
    "/A4MxJ/AtgKwj//z51/4wlDQbh4eYJcPtPBEWICZQUAA2IgETe2yQdf4IbUg2VlYGCKs/Rl2"
    "nT3E8OvRP4Z3YiwMd9/9YJDmus6gwCEAzsGwU0tBDTsFDiugG4CNVkbIbqIf/74Bpf8iDefC"
    "Gnl/4I09kOe//n3N8ObXE4ajD/4zPHkJbDdfAep995dB4LcIg8RPTYZPLFIMn34rMXz+Lc3w"
    "DphoFfkYGSZ6iDE4zn/CsFHWn8FW+4Ce17ttARt/M0ygdgIACCC8B0UeBxZDhszwauAHeGSQ"
    "gUEXVGTfufMPmACYoQsIGCEd9Zs7GRj0AoCxIANJAOB13tBqAjwo9IFh0mERcO/gwfPvDEdu"
    "fGTwMhIBj66B1iL8gOZi0CqbD6ApXqCRr4FmvAf2hVhX/WZgU2FhYAJ2HXm5gb0EYAnBywPs"
    "BkqyMQjwsYK7gOBuIAsj1mVh4LOc/qCWNb///GIwUzVkMFczYjhy5TQDx2sWhjv8zAxKQvcZ"
    "hFlUgQ1BHnBLHn46OVgvMFH++wk9fA4ymvcfJfL/wht+IEu//XvH8OrnU4YTwMh/+JKR4ccl"
    "YMIF5i5pThEGY14tBmV5ZQZjc0MGLS0N8GIS0B7FD8BEbSbPxVBlJ8hQs/MDQ69xJUPyw0uR"
    "O+4+WfWTxK3lhABAAOFNAOeBfrgNVKHIDA/P5UBcBCwFWJ4Aq4EvwJKAB7Z4A9QovL2PgeE9"
    "sJ8ILNbAx2L8gc4Q/oNs0LSQ+8qgJv6D4dYLTnBsbD77DpwAQNMCuw59AY/igSAoIHh5gEU1"
    "DyODFLAIknnzn0GAiZVB1JobMgyLlIuRu4HgyZpf/4nuEYDqZXYOYClgG8Bw7PI5YFsDWJCJ"
    "MzM8ePeNQZL9FhDrQ6sBxOFTkFNJ/0GncP+iJARYAvgPPdIeFPlvfj1iOAlsM917xcTw6yow"
    "8l8AI59LmMFeSJtBWUGJQVtfn0FNXQO8keXnL8iCDNDkPmiyqchGmGH7na8MRx9KMhyxKzcx"
    "fZLvf+Tnv+mQ1hsLeCk+pQAggPDeGQQy/tZf6DlPkGrgChCfA7X1PgLj9skT0MpeRmhvAOig"
    "j8Dy7cFRYNkqBKxwxeEOBAUIKJFw8vxi8FL7BD6+FdSx3whMAK8//mDgBNbJtqY8DM7WPAze"
    "jnwMnvY8DLYmnAzGWhwMenbcDJLAxiDLE2BOfP0XfFDTT/AaPAgGrb37C2tEIvUQ/iE1EPHh"
    "779+M9homzEYKmszfHsO7MK9ZmS4/4aV4dmPBwxf/7wHN+LArXtgQgDRf8DsXxA2WOw3VP4X"
    "tNEHxED/fv37nuH1z4cMZx7+Z3jwEpizLzMyfAPmDVF2PgYbQS0GBVklBjVtXQYlFXVg45UF"
    "vNQM2V2gld6cwNJsoqcYAw/rD4YDMh5MzGahCZywreVsPFQpAQACiOClUSf+QnMZfOE3wybY"
    "9rVb16ESTNDWGShjXt8K6QZyyTP8Z2ICN44gw8P/4NUA6FgZUDXw/NV3hv1XP4JHBYWA9R4H"
    "B6TVDhryBUcwsH3xGyjGrMXK8PszqMsEWtNPOFJxdRVhk0aghicreLEIE3iVMS8nF4OniSdk"
    "rOExI8PrD8wMjz58ZXjz+wG4BQ/p3iHj3xhif/7BEggk8l/9fMBwFhj5d58Dq4wbwLYLsHQR"
    "5eBlsBXWZlCQkQdGvhaDmoYmAwsrG7h7+B/tmANQGIPaA8ayXAw1DsKgOWeGk1YlJn+ltP3g"
    "ZZtXPcUJACCACCYAYOnLcPsfeIc2zGFrgPgHqMv17Ok/ho+gU2NYoV1C0MFKD4ENwbd3gAlA"
    "GrJcDFwX/oWUBsCcby77hUFDHHRaFBN49ckWYGMQFDvM0AYgRj8eaDebJisDIy8Tw3dgY/DP"
    "R9B2aUbo/gBU/A+KQeMFjOB9hkzgrd4gzAyMbNBw8+dvjAxPX/1lOH/tG8Pxc98Zdh35xrBq"
    "23sG9j/GDNIiCgw/nwNb5++ZGB4C2wOvgDn4+98vKLkcEfm/gW2B33CxX9DI//7vE8PrXw8Y"
    "LgNLx3vP2YANWGaG7/cYGPjZuBisRTQYlGQUgZGvw6CqoQVMgGzgiSVY4mSCjlOAMGhQ7DOw"
    "LvgMbPQWWAky2KmyM/xg4mP65dVYzMDGBRmLlTMCYmOKEgBAALEQUvAD6Dhge49BCdEOAC1I"
    "OA4s9h0/A7v2D+79Z9CXYIIe1QWsoL+9BQoeYWAwTQNWA9KQVcNA8f/AQGIE1h0c4GrgA8ON"
    "R9zgamDnxQ8MT98CW95C7OAdOb/RT2QFLRqVYGFgUWBl+HH9J8OPO78ZuIzYwdPEjFi6hZA9"
    "B//BawbfvPvF8PnLP3CP4fOXv+BVR6CEAVpRxMfHwsADLE8VpFgYBAVZGYT5QQcquTNMWD6H"
    "gfMxK8MrQTaGpx+/MAiwPGXgZ5EElwSQU06gh1NA5/JhDUDwMDOwV/D290OGa8DcfuclO8Pv"
    "m0wMP4BtYX4OLgZLEVWgXZCcr6mjw8DNzQk5GAMY0aBuLKgB/BHY8H0LdPOX74xgNuiACkNt"
    "LgYzdXaGCcBegcPcRwyfpAzVGeyyShkOTstn+A3s68YtZmBo0SI7AQAEEAsxik4CI8GZCbJG"
    "ADq9DlouBtpDyHDz2l8GfXMW6CGQjJAJoutbGBgMYoCNQQVgZXYJ2hX8C0kg4GrgPUPfIQlw"
    "g+fVqx8Muy59YEh0kmRgY/7LgDSxBymyQQdHsDMycOmxMvy5+ZPhz7VfDMwG7OAt5j9+gLZ/"
    "Afv+734zfAb2/b98Ax3W+Qfc8gd3CYF9fj5gj4GXmxFY7HIw8AAblhxskEkjWKKBb+Rg/Mfg"
    "aenMsGrPFoYPj9+DB6EeA9sC4rxPgM0cAXhbBtYURDQEIZXjr//fGd7/echw+xkjw+3nnAx/"
    "bjMDEyvoeHQ2BltJdQZVWUUGFWCuV1LVAfqRk+HRne8MH4HV2idQAv36Fxyo7OwswMYvI3ir"
    "mqwUJwM/L2St4yugGkNZToY6J1GGks2vGBjME9IZXt9dAXTEcYZf3yDnNz27QlYCAAggohLA"
    "W2Ag3QX6UxE6sAPkbgDiJiZmBv7nz/4xvAZ2a0RlmKCXKABD9/l5YPa7ycAoosLwn1UIMkMI"
    "XigCzJZ/2BhMpb8yqIp8Z7j9kgs8uA/qDSQ4SICHE1jBi1EQkQPOzcBi/yMvI8PdP38YvgJT"
    "IxMPsH3A/h/eOAXlZlC3UFyYiUFEjRs8WwhaRwha2o25sQQymom+0AQ0cicjJsbgZeXCMG/d"
    "SgaOZ6wMLwXYGV6KfGbgZX7FwMksCFaD6BFAI58R1DD6yfDhzxNg5IPOOORi+AuM/J83QXsR"
    "WRjUOeUZmP9KMHz8rcDw+K0Sw5NTf4BaPjBwcbEAI5iRQQZYugkLc4Gnr0GJkwm62RVU2sDm"
    "PX6B5iCAVUGmMR/DtltfGPbd/c0OLAUmMPz8DDo/4AfDmwcMDLaZDAxHZ5PcMwAIQNkVqzQM"
    "RdETnhVDLIKQaokNdShuFRwc3IpQEBc/pe79KteCUugHFNTFgtGS6qCUktK+mqbGe+8jcSs6"
    "BEIyvXB455537zn5EwBYljANHFp5q5/DDnt0fzknAAb00q0qIwdZniyoMHjpAuUTWPYBUh3K"
    "85T0s5VsiG+gWZtgMDI0cPc4wfBDw3e3xLn78BzLlhhNjVuXVUfBJiXgb0L1Nfajb1Sudkin"
    "Qz6aSfYwfQIOaMjqiCwF/lcSrk/94OLz4uwcN7cd6NcZFkQP4SdRhPMON7Vzo0ZOBTLy8IVo"
    "9YaAMD4Ii0gChfiJI+cU9pYeHFVByT9C/bgOz9vFtmNJKzs7pjZH4QaYMt7Ixlf+WVbMF4+3"
    "L6F1QnX1CiUC/XWtgP5whnHRO4Wt27Sqtiyy0QJG96Q3e/8CwI8AYiFW4RlgAnBmhpx7D13z"
    "swZIe4M8cf3yXwYTOzZwixpcDYAq81vbGBiMkxkYeFQYGD6chVQD///DB4f8Nd8zTD0C6dF8"
    "ePsTXArkeMmA5w1ACz1AxbacJOhIFxZwbuYAFo2/5X4zvHsNrDeBHQk+UPMVtK3sL+z4/v94"
    "loQSB0Ana6jIyTHYGVswbNqzi4HtBbAhyM/F8FbkA7AL+wHY1uWCDu/CIuw3MPJfMDwGuunO"
    "Y36GP/dYgX19ZvCRNfrisgyaMkoMGjpaDLoGhgwCAgLAkucvotr5C6lLf/8GnXcIbHgC674f"
    "QAyO7D//GP4AE8Bf0GGYf0H7G/4yvPz6j+EoUOwrsCgTBGYG0EDZP2bWMobf30GHPx0GamBg"
    "COphYJgbysDw/jHRCQAggIhOAO9g1QAjpC0A9MQ2oNBLYDUg/urFP4YXwKpACtRSBA1mgE7r"
    "eAOsk14BsYQ+sHcgBuxwPwJXA/+BrWbGv5wM1gpfGJSB1cDd15zgpA9KABlukgyywCKRXxjY"
    "4PsLmfz5B+0f/wbW9/+BOfK/NDPDt3u/Gdhv/WLgNODAu2aQ2IiHdQ3BNNALgU6eDHsOH2X4"
    "9wjYqpcA+u0tCwMf+2tgVSABHeiBbNz48u8Vw9PXwJz/EBj5T4Bd1ats4IMb9MWlGTRkFBiU"
    "1NWACUCfgYtHAFii/QVH5vdvwIj+DolsUKT/BZ00Bj71FLJuEaTuFbAd8wqo7gWw2L8PxL9Z"
    "gG0XLjYGLl5OBhVg2yBXnZWhbscThk8/GYAB9RlYAjB6MixJ+Meg4Q7MGZIkJQCAACI6AfyE"
    "rhNXZoYPsIAmhQ8Bi4NQ0ETMnet/GaRAq0lBi+bA1cAnYDWwn4FBxgo8JgBeNQzeOgY0iQno"
    "GZ4/DF7q7xkmg+YGgH39w9c/MNx98Z1BXYqLgfEfpBWPEVGgPQJa7AzfgZH/9fIPBg5tdozD"
    "yXFFPPx0EOh6fri/fkFKnI+fgKXLh9/gJeufP0swyEnoMtx7dI6B5RWwFBDgZhATesfAzPEZ"
    "WLCzgiP/6783DC/fA7t6j4QY/jwCtvgvsYGXtOlISTGoA7t6CmrqDJq6Bgx8AsLAiIUcVfvs"
    "yReG1y+/Qba3A/nfgL2SV0B7X3z5zfAcWPU9BPZUQJHNxsHKwC/AwaAoz8UQJcXNICPAxiAP"
    "7JUIcUD2H264+gW8dxF8XMqnF4rA+kGJ4dHZO0BMciMQIIBYSFF8FLRj9D/kqD/omMBSoD9C"
    "QaXxjUt/GKzcgDmAFbKcGLyB7h6wdDIBNk541IBFyHHIyaKgBhSoSwgMSl9gb2DyYXFwzHz/"
    "8Ith+/n3wMDjAZ+4DVqIhX7CHKhLyAbsDzMJAetnYDvh14s/DKySrODNpP/RVwjDDnmENihh"
    "3cNvwJLkLbDK+fAJ1GsAbSv7DV2IwsQgLMACumGWQVOVh4GP35uhsfcKw7+HQD3i7Ayv37Ey"
    "cEh8YGBn4GP4/v89wytgkXj/oQjDr6fAvvxFDtBKcAYtSTGg++UY5JSUGbSBkS8kLAo+JALi"
    "JkgbBZTLX7//ybD1xieGN8DEx8bJxsDLz8GgJCvAEAr0u5QgO4McEAtyQW9sgd5SA+oevwG2"
    "t0C9pCPA9sk/0EliwKqA7fmlP7/+/JSCrh0kGQAEEEkJADQ9fBeIDRkgGR3IPADEj4GNNNk3"
    "wHrw6cN/DPLaLJDiAnTq0/ubwEriPAOjjDXDfw6gG7/cgqwZAJUCv7kZrOQ/M6iIfgf2mSFn"
    "CK098YohE1gNgHoC7Mz/Gb7+QTsvBLQsQBjYJlBhZ/hy8hvDt+s/GfjlIDkPUoRDWs+wXP3h"
    "4x9oFxHE/g2uTkApgw/YveIGWikLrFKEdbjAQ9GwFjisDDHT12XQ0dBkuHjlKgMzsBR4w8/N"
    "ICj0luEn809gaQE6gVsUGvmc4M6NprQYg6asHIO8sio45wuKiII3kyAfWSwqzs3w7csvhhef"
    "fjHc/PCHoSxSB9hO4GbgZYfMl/2HrngC3av0HDqNwszAAD1nGXIONyghnHn8BTLwBixZHN6c"
    "lz/DwKD0DlQakwEAAoiFVA3ngA7QZ4L3BkBr1HcDcRLI0Tcu/2GQ14Xu1wJNBf/6zMDwEFgN"
    "yDkBSwElYAK4Ad06BhoJBJ3bB6kGJj3mAq8pOHXzE8O1x18ZDJVApcB/hi//MXM1M7Ds5NHj"
    "ZPh56QfDjxs/GFiMuBi+Ax3z5tUPSK7+CppNhORqVmBKEuBnAverleS5wRNMoDMKQPsJ0XcR"
    "gw+r+ItIAKD9hp5OLgwXL14HJnEWhs8ifxk+fvwILOH+Mty/J8bw8xmwV3AOGPm/GBlUJAWB"
    "OV+GQUZRmUFLz5BBREwCHPmMjJDDJWF2MQG7K0wcQH3ArpMEsI1z/s47Bitgt/U5MBS//YZW"
    "T4yQ2XU2ZkiXGHxJGRQDawaGN8Ci/+brH2AJpp9fGAw+3OQCVg2eKyBnDpJ8JRVAAMquWCdh"
    "KIoe5DWYFokNE2BsRIhDdQAX3NjYTPwFd3/OwcFBoxOTmujEQGAg1iqpxhBoAeu5ryWOhuGl"
    "H3DOufe++27PXZsA4jQoa2fyf05tsivvXDjRIwHa4ihiJj+GyNIcDK6ZBi6QyR8gVnepoQTT"
    "AEt3QoRTlwS4T9JA9B3h6mGMRq3ACLBATiU9AT2iFcb6tfDLi/BONY8oheBxpt/9lYxqWYZW"
    "tbNroMj4KUZOcrJpk1IDnQ4ohfP430pRbGpax03UGc77wwFyewqet82CjXf8VwL/ZCKebWC/"
    "bMN1KnCqNRw1mqjslJluWNWHGd2B9AmWNHvGwUL7Cwghpj8GOq6Ny+c3vBwWUS+ZmHymvofZ"
    "ROkCvqherb48FoNqdzhFQIJDbcL+6MGa+DjbQudmjhN/idt18fwVQCQngE/Q3oARI3z30ElQ"
    "3APjSe3tu38MD+/+ZVAzZYXsWQWN/H1+wMDwAlhIKbgB6yxZYGvrMnQq8wf4SFdLYDWgBqwG"
    "boGqAUZgb+D0G4ZiP1nwMm/QJonL9/+CR8pAR7VBRsuAuVqElUFMh51BCiguAgwpKRcBcMBB"
    "jmlDTA+Djn/9Q3SPAHWMAFQi8AFb3a6ODgxTp89nYL/DyfCNDWjuV2C1dJmH4f83RgZFST4G"
    "HSVpBilZFQZZFUOGr79EGI6d/cLw/gNkuzqo3QE6U0BIgIlBTZmTQViIBbxY5dtXLoan9z4y"
    "fATWHfN3P2DoS9ZiEAM2rIDpH5zTYZEOy/nICeHyc2AjElTcAsNB9M1tBv6vPxgk+Rn4vTgZ"
    "ohd8YTjKgHYjCCEAEEAs5NQbF4ChqYcIs6+gLiEoAYDqp+vnfwMTABuk8gJN3YGGKh/tBfYf"
    "PSClwMdL0MoOdM4wNwMX9x8GV7UPDLcec4N7A6dvfWQ4c/czg422ALBhBZo/+M8gL8vGIALs"
    "GsJyNQuwvv4lw8rw6v4vhr+PfzL8fgOMZlFIYxBbZJLSJUQGoF1FzvbWDEePnmC4evUWA9s7"
    "fqAdjOA6X5CXi4GHWZjh7RcJBsYPSgw/7vIAI/sXA+jCUXkZNgZRUTbI6B47UnXzF3LgNAfo"
    "vEMeNgZzZT6Gs0deMmw584Yh3EqE4Qn0Ogb0SIdhkBnnnnyDbpP6xyD37zb40E2Qt324GQI2"
    "f2dY+PYvw2FS/AgQQEzkJIDn0GrgH+Jo1RWgTAeK89tXfzN8BW0tYoVWaGygc/r3AZMJsPLg"
    "BfYGWKEzhMAewX/QNbGgkymB7QAGVsgkwL+vvxm2nnkDNldNnpXB3pyHQUWJg4Gflwm80BM0"
    "jAuaA/gvxMrApMAOXlH79ep3yKJRLNPBuFYK/8exepgJurAUhEEtdiYWLoaw0GQGMRE1hj8f"
    "gW4AFt987NwM/JzCDGLSigzmVsYMfn5aDG6O/AxOtjwM5sa8YPeCBrJAYwqgFU5gDCqN/iPW"
    "LQhLcAOrSlYGH01+htWHHzO8AjZYxXgQRT8rE2rkg0q4D8BS8MqLr5Bw/fObwVjlMQOfMORA"
    "SwlmBmFPToYYUjM1QACRVQKAGqj3Qb2B/5CT2IGeOg9MlBeBzjJ4B6wj7t38y6Brzwy5ehTU"
    "GPz2DFgNnAK2VQMZGLgUgb2DU5DqAZQAfnMwOCh/YpAX/s7w8A0neHHJtjOvGWpCFcA3jjAD"
    "W7o/fzEzMKFf2gg0lkOLExj5oN7AdwZuYEJhYoM4hpjcDm9UMjPC5wq+gSeU/gHrbWA3ETT/"
    "AOybg4Zjubh4GeIT0xnePDvP8OndY/BaAklpKQYVDQ1g8S8PPZX0P3hWD7ms+U9gJZKACDeD"
    "IrCqUH38jWHW3scM7eGK4AQOKklZ0BIAF7BQvfD0B8PrT7/BPSmeP28Y1CXeMig68DO83fAR"
    "2DthYHDjYPDd8JVhLrDxfIrYuAQIILISAAiANqjrMcAXivwC+n8rKAGAGtJXTv9i0LVlR1QD"
    "oIh+uBNYDfgzMPJqM/x/fwayEfAfqBrgZeDh+QvsDXxgmP4MMih0+d5nhhM3PzA46wsDu7rA"
    "XAi+QRR9TADYh1ZiZ2AWZWH4CWwY/rgPbBACu3T/fuKJbOhoH6irBRoT+AqM7JevfjJ8BobY"
    "uw+gRtpfBlZgVuPjAWFgz0GRB7I0jQPUowAdJOEOrL+/gcfqQaePs7Cygodx0asdYhIgKMEJ"
    "inAxfPr4k8FdQ4Bh6qk3DIdvijDYqfECW/rQI+iRqgLQ5VWXn31j+P3tL/hyaxnGBwxiTG8Z"
    "1L3FGc6f+sbw9cVvBnk2Bkk3YFtg3TcG6Ng7YQAQQGQngBdovQEGyHrBMlDv5e613wwfXv1j"
    "EBBjgqwrA1UDL08Au4GPgNWAKmTJ2M/XYDeCqgFGRi4GX+13DNNBg0LgSyr/MKw/8ZrB2VAE"
    "3B1kAyaCb3+YEKXAf4j3mLiB3To1DoYfz34yfL3ynYFDkwsc04zwXcSQwSRQix40bfz6zU9g"
    "A+0vuLsIWkkMyrn8fMAeBB8Tg5Y6F4OQICtKNxF2LhB4RxLorGSgyWwcXOBVcNiOvyWlnQGy"
    "gZ0dWJLI8DD8+fGHwUaSm2HSjscMVipaDHwcoJvYICe1gAoo6IktwP7/V7hFckz3GUTZfzLw"
    "SQsySNkLMlxf/IrhF7BQDeBmCNv9g2HR538MRA0LAgQQ2QkAdLoxaJG6EQP8hCLQOXWngQnA"
    "6hOwjr5z5TeDiScnJBRB1cAPYJJ5cQxYsccC+zMqDOA1UozAcu3vF2CZx8Vgp/yRQUHkO8MD"
    "0NwA0Me7zr1heP/xFwMfFzMDNzABfP3NjCWAGRm4tLgYvgFb3r8f/GD4/w7YU+BnYfgGzNFv"
    "3iIiG3TsGyjE+YCRzQtsNakqsTEIC7OBB4BAR9Cg9x5+/sZ5YDHGmYTkVDegquLjV2CCfP0N"
    "2D38x3D7JrAx+/Y/w/u33xgWHn/JUOAkzvAWGtew9sB3oB5wAgCb8YdBhekWsJRiA7tJxkWI"
    "4f7Otwzfgf4Ftpcl7NgZwrd+B28nJ3hgJEAAkZ0AQAC01EMfEQigIgd01rAVqGV56cRPBhNX"
    "ToiDQf1F0EzhE2A1oBzKwMinx/D/7RGI+/4Aq4E/vxi4gUWtt+Y7hqlPpYHdxf8Mtx99ZjgO"
    "rAa8TEUZOFj+gfvz0IE8yFQqeBwfmJP5WRleAyP2yTVg4Kx9y/BPgQOYo/6C9wcICjAzyEqz"
    "MoiKcENG+0BrDhlgc+3QyP5F3oGE/wlEOOywyf/QhSugTSovXv4ARjhoQOkPuIHJwwMas2Bm"
    "sLUTYfj0hp1B8TUHw6JDLxi8dQUZlIAJ9MsvSPBxgm5kefSD4cmHX+AJNeb/3xlU2e4Dexys"
    "kMW20hwMssBEcGPpa4ZfwBj1B7Zb9/1gWAws9C4T8gdAAFGUAECH0YOOrQBtXIPO3awG4jpg"
    "KcD1+M4fYKMJ2E+XY4HOIQPLtbfngEXHPcgUMeg8AXApACyK/3wFRgsfuBqYelACkm5//mFY"
    "e/Qlg6eJKLBDAew6Mf1l+PKXGRyYd+9/B+fsD8B6+z8wYf3nhJxKIvTyJ4OmrzADDz/kfABG"
    "RtQDo36hNdKoEfGMDKiXS4ESFuisIFAD8sXLX8BSCFICgY6VBVU3gkC3qSpzAtkIN4IMecUK"
    "bOv8BTaeBVgZOrc/YViQoASpeqD7bq4A+/+/vvwGTwCJM9xnkGR/D0kAkIOSGORdhRke7ACW"
    "AsCyX4mVQd6RgyF623eGSkIeBgggihIAKJ5A11CoIO0oA+JjQPe6fAHmrJsXfjGIqLBCsy4z"
    "5ASRF8BuqkY6MBGoQzaPgG4f+wOsBn7xMtgqfWJQEv3GcO8VF7js23n2NcOLdz8ZRPjYGLhY"
    "/oOPWP8PPtYdNEjDyKCmAmykAQOS7ddfhtfAwPkNTBBcwFYymySwcfbzH3HF9H9GkiIevUEJ"
    "WpYObBcyvHv/G9h7AEb4x7/gE0lAC1EFBFgYJMRYGAz0uMEXZbCzI9bUQQ6c+g83k1+Ii+Hj"
    "hx8Mjsq8DJPPvGXYeOkjQ6AeP/i2HhA49fALtG/LzCDD+ppBkusrMAEJQNpMwITNJcPOIA1s"
    "C9xa95bhDzBIPTgYQnZ/Z5j/G3KQJE4AEEBMDBSCS6jNTZAT14IY4Ftmjv2EbB1jhq7eBJXj"
    "z3aDh4MZBQxBi/2gKRhYfP/5ycDF84vBR+s9MGQgVcbT518Zjlx7D2xtMwNbwZAjU9mAxbi5"
    "MQ+Dvg4Pg6gwC7CBCKrcWRmYFTkZfgL7yR8vfoHeG4m0ShgfxrOkHHpKDGTTKXTDKShXfvz8"
    "n+H67e8Mh098Ydi1/xPDngPvGa7d/A6WU1bkYHC0F2Bwc+ZnsLbgYdBQ5wQnVpD3YWMCf/4i"
    "7t+GbWgBbYkXFudh4ORmY/AE9j4m7n7G8AmYiEHD4aCBv9OgBABqBf//w6DA9Z1BWEQC2KZA"
    "Ppv+P4OSlwgDB7BKAc0rqLEwKFuzM4QRij+AAKI4AXyBVgVsCKGNQPwRlACeP/wDxKBuC/SS"
    "PRZghH+4ApQFJkpuZcgWMvAU8S9gKv4CLgud1YAJgOUv7IA/hg3HXoKLVVDDnJMZ1AhjRA1I"
    "aCByanIzMAKrgm93vzP8fAXaP8BI2v4BBsQuY9B6ARbo7mLw6mJgQ+3S1W8MB45+Ydi++wPD"
    "/sMfGB4//QM+6FoH2Ah1cRJkcHXkZzA35WFQlGdn4OZihLcv0N2JvLMZGYMal1y87AxcAhwM"
    "GhKcDKzAdsykQ6/AwXb39Q+G26CTwZkgZ/arc71i4BHXZWBiYUPckAK0hweY+KRt+MGLhUGZ"
    "0p+TIRZYGKjhiz+AAKI4AfxmwLiNCDRQuB9kMOievOtnfiD2DYCGxkDXz746DB4RZOTTgmwi"
    "AQ8NfwLfluUA7A3ICH2HjG8Ci9GdZ14yPHr1HdyC5mL5B1uUippTgZ5nlWVnYJViY/j96Q/D"
    "1xvfwANK2M4QQI8EWOscskYPdKgUA7Dk+c1w5vxXhn2HQBH+nuHo8Y8Mr9/+ZeAF5mRTE14G"
    "N2CEO9nzMpgYcjNISrCCj4oDJdJfv6Ajfv9w7UpixIpBpdRfMGZgEAGVAlysDGEa3AwbTr5i"
    "ePj+F8O1F98ZvoHqP2BRxMP4iUGRC1j/CypC5laQF4ECY13JUxgYtIzgqgNYCqgaszEE4Ys/"
    "gACiOAFgqQZAYA2sgXEVWA38+warBhghd7O92AOM7A8MDAImQD4XJAGAqgFg0uXj+83gAewN"
    "gKsBIHr7+jvD/gtvGNjYgH1+YJEHahD+w3IoBGi1EJcGD2Qr9o0vDH+/g+7aY8TI3aDiHDSl"
    "DMrhoCoAtCjk7sOfDCfOfAUX5zuAEX7u4lfw9nIxMWAL3UaAwd1ViMHBhpdBX5cb2KNgAed8"
    "2O0kkNzNCD7AgmB1g+Rm2GknIDboynoeYKknwvaHQUnwH4O+IjuDgRQrgzewLdOy8xnD/tuf"
    "oEUIsFcDrP8lOH+C9xUwcGswgI9UgV1yDcxA/GpcDLJ2AuBSAJQI/TgZophB17TjAAABxEKN"
    "BPAVWg2oIqaiQNehvQRaLP4SWFQ+vvWbQd6QHTJbDer7f77BwPDpGjABADuRXAqQGULQaN/v"
    "TwyMXCIMfsDewBzQvgHoGu5tp14yxLjIgdsVXCygDR6s4EkilAYpMEI4gJ5nPsbK8P3pT4Zv"
    "D38wcKtzgVeuMDPDpnghp4q9BHXH3v0FNtz+gEfyQDedg7qMykocDMLCrOAl5mxsiMWb8BXG"
    "ZK49hJVEsAYfKBGzARu1oATNCTp3EFi1gfYBgi7CALUVuMQ4GZ58/M3wn+0rw9ZzbxlAl4KA"
    "d10BzZBhfcMgxgfZ28DAIgX0NLAU+P4AMrQObZmr+ogwPDzwgeE7sETSZmHQMWFjCDn5i6EH"
    "m9sAAogqCeA3dFBIFWkrARDvAfo1+gfQQddO/GCQN+VADA3//gysBg4CyzsrBkZ+fYb/H85D"
    "JziApcJPQQYHlY8MsiLfGR6D5gaAOXXPuVcMd559ZVCU4GLgAhax78EBitZ6B0YUKzDyuFQ4"
    "Gb6c/cTw7eoXBhYFLobPH/8xPH8BGtuHnCsAigfQhZGg0T89HS7wriDQcXKg4VbYhRCQ+puM"
    "CEe6dxBZDTjCwZH8DxzpoLhkZwGJMUG7jowMD9/9Yrj54gfDkTufGY7e/8pwC1jnv/sEXSXC"
    "CJ2lAvpZhf05Az8vFyTjgxIUrybDf1B3GnrNFbgUUOFikDLnY7i35yMDBwsDozsHQ9SZXwwr"
    "gEH0BN3JAAFElQQAqwYcUYVAlxZGgyy4AUwALvHAepoDeislaLnY673AUE4FlgLGwJ7BGsid"
    "AyD85xuwa8fA4AmsBmbtlQafVPXpzTeG3WdfMuQEKjOwA8te0NDw979M8D44qFYB3+ABWtwJ"
    "zD3XP7xn+HbsMwPLH2YGFk5m8BIwQX4mBjVVXvDuINAiElB4QoZ6ocXxX/JXFSPncMjoHeie"
    "AGCjDpTDQdUWNMLBk1vQq1RBi0FvvvzGcPLeF4YDwEi//vQ7w7P3vyGOYYKeqQC6K4cBevLq"
    "v78MRtyXGSz57jFwgLbfM0FTLKgEADWmwauuEVefqvoIMTwGtl1AawhVmRhUQLXZ87/gVUMo"
    "3gIIIKolAFg1oI4YGgatTnkAqn9ev/jL8PDaLwYVG2BsfvsH6TOAii3Q2gBhK2BdBtT1/jh4"
    "TOD/748MjMD6zVf3LcMs2KAQMJa2n3zOkOylCA4bLvDQMBM4jEBz9jcu/WB4/eY3eOYOPPgj"
    "xM7A//4Hg6oAE4O0kwD4HCtGSBjCL3jEG9H/iYhwpEgHDdeC1jCCcjeooQqZvwBGOKhxyQxp"
    "XH4EdlGvPfvOcOERMMJvfWG49Pgbw+1XP6CbIZEayeBb2ZnA5w0LMH9mkGV/xaDP9QCY858x"
    "KHK8ZpAS5gImaGD4/QaWlr+/A3uFX2DDUUhLuH8wCCjyMXBJ8DF8fPAJVJ1xcoIv7GUA3Qf7"
    "EtkvAAFYO3cVhIEgil4VIr7TJBYBOwst/QR/ws/xtyysBG0sRRAEQbEQHwi+EiUx3tldUbET"
    "06ROZnbvnNll7t8SIDQ0UPskxI44potEjPu+TgBVDBoZ2DJH3CYSdgPxfmAG/nCDDxwlAxXH"
    "x2KTVTQwGG0pAydUvbz6yU+nOt2iJwMTgVyHEkB9PBdi7Lo3pISdbzZCQcL7bz3ALwcx0+PQ"
    "AderXIItQU8bHZeABySTKWuN0fKCHou44YzbOmnmavwO9ClP0rhzq6ufTBYfnrVGLbNAPTNH"
    "xVrBS++QtyLY2TsKuRDlEmnkMkF8DPQpK6LXRExBankXq4jKLZY/bX73AU/Tcz4y7339nuIP"
    "AVi7mtSEgSj8NegkppGoJ1CwULoo4sJDFHqBnqA38Co9iCvdiWiPYF0ItVZXJjUNiSWQ6Xtv"
    "BnRb6KxmNTDz3vve/7x/YwBenH7iZuXw7BUw5DwzT7+RGjhFpANDG9SvcGh4SsQmr7FJ3sC2"
    "aYgvCfEUQb2Oh7sILxNf1EAa5RjN9xg+3UKRReaRhKSFI52+/Z5v28LM1Srda5SLCBnBavae"
    "o3bjm4n3fwjxXkq41LUwgRnWRY8biFcyPsaR0vNSO9gcfrDcZZitEryuv2mfIz4W9ltdm9Zj"
    "n1HbkWdgyz9Gx93i3luhrTZou59oVBMEXkFE1wiDK7QCQha3lElqfBF9Ks8S79B5it5OtUx4"
    "vTEgWX9EPJ4gWX8If32R+XPUYpfx14/Vy7KxXwFE1QTwDTo3YMEAv8YCtF7wBtCrGu+Bre57"
    "l34yaLlxA+uLf5CK+yewTfLhHLAUcAOmWi1g2twL9tD/X++A1QAPg5/OW4YZByShner/DDtP"
    "PWfIClQFN5x4gBHx6RekF/v3N+r4NKsoGwO7HLAxeOUTw+drnxg4VLkQORlHjCM33MAXQoFz"
    "NKThxgnN4aDVOqAIZwYffAFsxnz+zXDrGeTIuwM3PgLr8W8MT9/8hCRwJmiEgzQxsMLrck5g"
    "E1YWGNGawAjXZL/FIMf6mEGS/Q0DNzswl/P8B+ZyVgYhYAOVjxN0eOZ/YOcIOqAB7FT/B93L"
    "ANpsyyYKGQPgVmZgBB3OCTqbEZQAQJd4sQDD9/05hoezGsCXZoImkq79ZrgCzHtvsHX9AQKI"
    "qgkABC5DSwHY4log3gIMBg1QBrx88DuDlgs3dIKbGbI8/B2oGvBkYBS0ZPj/ej8kGkD1209J"
    "BkvFTwxSAqDGESd4qezpa28Ybj76zKCtyA/MhX+BxoCWWTFirgAE3eKlyQvsbX5m+AIsevle"
    "/wGGDQtKKYBcpIP0s0IjGTTnwAkt1sHLs0FjBuBxfyZgPf6H4Towwi/cB0b49U8MVx5BinWG"
    "H9Dr3sGrNxghjVxosQ4KAimW+wxKrLcZdNkvAnP5fQY5YH3Ox/6HgQ9YuIG2gAvwMgMjnwWY"
    "y4GJC1SssAiAujRAy8UY/oMaeVzy4E22jKAIBkU26NJO0MJaRhbo3TvQou8r6IieJQx/DvUy"
    "PDn9ENzjBu2ZOf4bfMLbN2jOR5kiBgjA2rXrNAwEwYl0MVZklDNWKCnTIVHmC/JBfBEdX0CN"
    "REORCopEYFJEIg+MhDCJ5TMQhdk9W4Se4nSlpVvv3Iy9O/vvL4AwDJFp9vcauOQ6l26B6ajE"
    "JtsiOq67TGUw8TvJnyMSkAeovZwYSgiWVzls12BINXBxTdbbkTbxCle3zzjrxwi+tszOHfJP"
    "bym3z8JbRISQEtActeHmDkW6hh3E2nnbRN0TNWiwO21P3Ew9G1iCLhBdkaClS4fxvMDNJMdo"
    "uka6KuG0vmCvWoNaS2Ednpl3zSv1+gyn4T36wYR3+Qy9IFdDCi1AiUIklpL2MMFBlHirfQl0"
    "eOJ3yWYNtOVjjOdMzUAu/YXOvPrgmZVM6s2CB/7ElQKrO3Vszx4old98HSGP++XxW009XA3M"
    "f75o/AggqieA79BqwBJRDYAKhQtA5xuB7gm6ffoHg2EwL/TQX9BU8QtgbwBYU4gHAasBfaAn"
    "NoInif7/BlYDjAIM/sDewILDEtDTKf4zbD/2lCE7WAM8CQSqBj78ZMJY9QDets/BzMBrKcLA"
    "A1pKLs8FXkIGKsq5oMU6BxMk17MwQU4MAeVwUEvpKageBxblR29+YDh28yPDLSD7/QfIeYHw"
    "hhs7NLJB7gfW/czA9q40MGersV1j0GG/AMzldxmkOV4x8ADrbdAdgryCEgxCoqoM/IJiDJz8"
    "cgyMPAqQATBwkS0EOWUdPaLBw+PfIespwRENbCt9BIbsB2BkfwJ2+b4Aw+3ba8jVroidJ8AE"
    "xMnw4OYP8P0JnEDu9d8Mlz/9Bzf8fkEXcaFUfgABWDt/XoPBKIw/jb+NyG1o081gEDZXrAaf"
    "gG9ru8MdDUQECWFBCJFKBQktzjntG8Fq6Nih+fV5znlO8p736z+AIl59nRPxRpEKg+r/HfHb"
    "TIdlIMxmDlm/3YSWreG+aYVvOTK6qxf2FH2oDOx0OQ7eG+8wmO1RLWXpA33J3Neb9mHr/GFW"
    "haJQnKBrPgVPTxo3BVxtCNm6F4Hcmbr4H1Idnx+xYFv38bR1WTQRC1QumdyDGV2Ssqco611q"
    "4ibIJVfI8DYSw4JhFfFjN5Cx89CNHGIpM1A1WztP7DRVHhRovgtwHYA+sSAoUDsj4MAqJ8gn"
    "4ndx32bakeBJpJ5NDB8do5g9J8nLTb433Nue9Obn0AHcd1YPAUSTBACqBkBXWwgiqgHQQpFa"
    "0GWaD4ENwQ9P/jAIgBaKgE8UYQem5rOQcQF+A8i5QqA7icGzgR8Z+PnZGDw03zLM2wcsGoE5"
    "99/XXww7TjxlsNAVBTbG/jJwA8Vef2eG7ycE5XJuFkgDClSsg1vwoIYbsDEF6p59/gasx598"
    "Ybh4DxjhV94xXALSoO4aA2j9AGwTHnhHBiiiocHzB9Tle88gxfaYQYfrKoM69wMGBR5gwuT7"
    "x8AtIM0gKObKwCemyiAkpsDAKyQJLNbFUNsBsBwN6qZ9fwrEryAR/ekeJFeD1kqCIhqEgVUf"
    "4go+BkiuBpnDxo3o7v3H0UdlZwSW9/8Y3r6ATCO/+s/w4s5fcPH/E9otx1gyCxBANEkAP6DV"
    "gBWiGgBdcHgcGBR2oGXXN45+Y7BQFYBcUgheNQy6l+4EsBSIAaYaYDfmKWihCAewGvgATOtC"
    "DH667xjmHZKCLixhYNgJbAfkhWmD7wcUYP8Hrcv/gxMAM3R2D7R0G1RPg3YHgcYPrj78xHDk"
    "8luGs3c+gHM8vB4HjQGDh+qYoJEFykagUbjfDJKcbxmUuJ8x6PLdZVDg/cggJ8jIwC8gwiAg"
    "4s7AJ6zAICguzyAgCCyJOLmBnRd26OHY0Dr681PIAphvwOzwBRjRn+4D6QfQHP0CkqNRVpow"
    "Q6oBNi7MVipyRDMwoon/RyxqAZZst87+Au8T4GKFFP+fEcX/J2xxBRBANEkAWKoBUOkPuojS"
    "DuTUq3u/MlhECSCqAZAnPgKrAdFQSDXwfAO0N/AJtCmQwQFYDUgLA7tYbyELRq8Cq4Hzt94w"
    "2BpKMLD9/cvABewygVbggPrjoGnZ52+/M9wARviJq28Zjl59w3ALmOPfv4UmfmboxjtQhINH"
    "kyCnmILinpfrF4Mc/08GHdEvDKoCH4GR/plBDNgl4xU0ZBAQkgDmbnEGYWER8EwcK6i1Dtrg"
    "8hNUPz8ERuxLyHK3z3cgEQ0S/wGM7B8fUPua8IjmRopUBtyR+h+HGHKCAAlzMjLcPfKD4cap"
    "XwzMrJAm1pk/4NY/rPGHNQEABBDNEsAraDUghKgGQLHaClov+OTaT4bX934xiGqwQ1YMgdYL"
    "frvEAD5TjU8PGBMawARxAdKX/vkemOvYGTx13jPM2QUMNK6/4DXT2489YfCylAIfOf/mww+G"
    "2zc/MZy7/pbh4KWXDLeAkf/4xTfImgJYhLNBIxyUw/9C5iSY2P4xyAj+YFADtjG0xb4xyAsA"
    "E4AQEwMPDzeDoKAiAx8wdwsKCTHwAZvvnEzAEvQvsF3y/TQDw5N7kMj+CovoN5B7E5EjE1zP"
    "Q4tuBmyRDMvljKTlclhPnokBflUa6BDN69t/MBxZ8xWc+7mBsXrzL8OFa3/BVwD/go7UY71e"
    "FiCAaJYAQMkOdGKBDaIaAE0YHgS62/Prr/8M1w5+ZbDX44SMhYMi5R8wp3w+CqwGUhgYFbMY"
    "/l+vBY8S/v/xDBiWbxn8VH8wzNkHnRsAFvH7Tj5hmLKGn+HE5dcMF26+BRbznxn+gzZNMDMh"
    "inUOZkhggkoZUDAAqwhhPmAkC/5i0JP4xaAs8odBAdg+E+JhZeDjFwG2N3gZ+LlA/D/gCGf5"
    "C+ydfABG9FNoDv/xCrKOgQFpSQ8solm58Ofm/0jD9XB1eHI5bMyZEToUCTPz+3+GL+/+Mnz9"
    "+I/hI7CP9/zeb4ZHV34zvHz4Fzy2xgnU8+Yfw4vlvxkW/YH0yH/gyv0gABBAjMhXtyEfpU4N"
    "AJp9SEJdLJIMxHNA3QJ5LXaG9KWykONaQHU7aLcwtykDg8IUyPDm55sM/x/PA3rhJAPjvy8M"
    "n35wMmj2JDA8+yAODGzoSgrwRAqs4cYMiXwGyD3AkMV8jAwcXIwMUoL/GbQk/gDxTwYF4f8M"
    "ksDah4f9F7DV/oeBl/M3gxDXFwYexqcM7P+AdTOw+wnetAIsecA9lP9IsQGLbPSGGL7GGdYE"
    "gVwlQHMzM5oYaJva+38MXz/8Y/j0+i/D60d/GF4//sPw6c1fsBgI//6FmCkGnR0A6qw8/89w"
    "f95vhumP/4HbXa+gjb+byMO/yHEOEEA0TQAg00CrEnUQ5Q/oIIDroF1RoLjKWCbLIGXECTmO"
    "FDZbI9MJrAacofcOASPgC7A79PEcMDKeMNSsVGRoXcINOaOGmRGxRBcc4dC6Fdh6l+AHpiPR"
    "Pwz60j8ZlIU/MsjyfWDgZ//CIMDxCUiD2O8ZBDk+M3AwfWJg+v8JEdH//iPFBhPuYhtXpOKK"
    "aOSimxlNDFgFfnn7l+EzMFeDcjQokt88+g1kAxMAKKLf/wUvRmFEmvOD7xxmRCwXeA9s7J35"
    "x7B/31+GPdCG3xtosIOO9HuNur8F4TCAAKJZFQALB1C5r43aQ9wHdH8waPQU1BiUMucGrwWE"
    "VANAn74EJgAmYHHKbQHJ3aA2AQgDI7wi+xPDhSdHGbYeA82EscIPqebl+ccgLfCZQVfqA4OG"
    "6HsGOb5XDOJcrxl42d4yCHB+ZeDn+ArM7T8ZuDj+g8cA4MUvyE5Qv5yJFXWUHFskYyu24ZHM"
    "iDrJALtOhxEpJ4By9Nt/kIh+CYpoULH9h+Hjiz8Mn4E5/fObv+BgYELSwgKNbC4WROT/h9Rm"
    "X96ANhL9Z/jw5T+w2/+f4fbd/wzXgYngGXTI9zO04f0cPfLRAUAA0bQEAAFQZi1Ana0OBeJV"
    "oFFZSUU2hqwNCuDFwpB6AlRf/wTdFwOMdB8g9gKWawqQuwf+g8dvGd48vsEwe8V+hrPARt/v"
    "36DIfsGgLPSKQYb/EwMfxxcGQe7v4O1fQnzMDHxcoPsFoVXDf+jFFv/+Y4lUPEU0eoJAjmhm"
    "RkSDDFZ0g3YBASP501tQRAOL7Ye/GZ7e+c3w4cVfYJH+FxzRP/+j5WjodBELE1I4QSL6K7D1"
    "9h6YlT8A6WfP/zHcB2Zn0Prgdx//M7z+CrlG9jc0p/+F0n+gxf1zaCmAmTGR4hwggGieAEAl"
    "HmhZqh6iGgBNE4AOmgdfIpyyWJZBwY4HMkMID/S/kNExJh5IAmDkY4BfF874ieHnh2cMt++/"
    "Ae++AWU20D1A/LysDIK8LMBczgTsBiEtFviHr0+NFqnYEgQjLEejpmLQeYjfgJH58fUfYORC"
    "Ivrl/T8M7579BjfSPgFbYj+Q9hEyIgaP4Rv4YSuagLH17TPkaub3wEgFHT0Iuo/jJTC83n8G"
    "TTpChnB/IeG/iIVw8KsRfiMN976FJgQGQgkAIIBoWgXAXAiqBnQRQh+gC0USQDnhyo4vDAoO"
    "vNCrZ2CRAKrPoTdS/boBHUlDJCl2ThYGHW1e8MDaP9CZe7CcCIvsf9CQhUUgIyP+fja86GbC"
    "GtFfgZH8GZqjQaeSvLr/m+Htkz/g3Pzx1R8GpE1I8IhmhpZ+yBuaQTn6E+hgVEhEP3sL6l/8"
    "Z3gOimig+MuvkLD5jRTRsNXlf5HYMPnf0Bb+D2je+sVA4vEwIAAQQDRPACBwlQF0Bz1KuK4C"
    "4gSQ5Tf3fWFwAxaNbNzQhSIoEQMqE9mxD3v+h1Td4JWB//6j7t1iwlI/oxTdTIgG2X/kHA2M"
    "VGCL+xMwUl/e/cXw4vYvYEQDc/QbSNEN2ufAiFa6gYpuHrR2DzBmvgOb3h+AOfrDF0iOBl27"
    "DI5oYCS/+Qzpnv1Cisi/aBENy9Gw4vwnUmTDEsB/asQNQADRJQH8hI4FI1UDoDPt7gPjQPEt"
    "sMh8cOobg5ovsOn++S/xAyLI3S8mLIMpjLB9CGg5+vd/hu/AHA3KzZ+ARfcrYESDIvsdsOX9"
    "GZoAvv1GLbqZoAHFg144ACMaFJnIORpYRz8HrXIARfQn1Ij+hVQs/0OK9N9IGDZq95PaEY0L"
    "AAQQXRIArBrQR11DuhkYmHkgudPLPzCoefNDOrK/CQx7oicIBkakxhgjQgMoooE5+ROo+H4O"
    "zNH3gEX37Z8Mbx/9AuZwYC4HJoDvaOcAwOpo9KIbOUdDI/rJYwYG0Gk974Dirz5DBj1/o9XR"
    "/wlENHKO/s1AxF5+WgCAAKJ5IxAGQHNj6ahDw0bQJWMsoGzhXizK4FAuDu0y/Ue9BQpWnLPA"
    "js6ErgBhhET0D2Akg4ptUI5+ffcnw7MrPxneP/4FEQPi77+wRzRG4QDMed8gORoU0S+ARfcD"
    "UER/Q+Tod0itbmIi+idSjv41kBGNqxEIEEB0SwAM0MmhYGj2h4JJQJz7H1o2GvrwMdhkiTBI"
    "63OBJzcgQQWNfNBFUsAi+sMzUI4G5uY7wKL71g+Gt8CcDSq2P7/8zfD1B9ER/QvohrfAiP4I"
    "pJ8Dy+nHoDoaFPlA/PoDJKJ/IdW/+CL6D1Ju/j7QOZrUBAAQQHRNACDgBMS2iFKAC2j7emDp"
    "4AYrP0HD95I6nAxS+pwMvKIs4JwOaoS9uvmD4TMwN38Ethm+/SQuokH3UIOOCQb1o4ER+wIY"
    "sU/uA+vpb5A6+9VHSHfpN4GI/oOUe39iqaMHbUQTkwAAAojuCQAEQItGPaFs0NQ1MFJmqUN3"
    "EcFC+j+OMQVcOfo7tI4G1scvgDn6ESiiv0Ny9JsPkIhGz9Gw5gl6RP9BiugfQzmiiUkAAAE0"
    "IAkABJqQQrsD2ERQYWBIMwMmAnkGBi3QOCA2PaCL575B+tGgxhc4RwNb3c+Akf4W1PJ+jxj/"
    "JpSjfyNF7LfhGtHEJACAABqwBCAPnRr8BkkAIMABxJoiDAwGYgwMqtIMDHLAhMALci8wcr88"
    "BUb4R0hEv30HGd9GjrQ/SB3DP1gi+ieWOvovwwgFyHEOEEAsA+UI0DYy0GI1RYQQKIKuALPw"
    "ayA+dw3SG2NFGq75izYihi+ikRPHiI1oYgBAAA1YCQADoDUDWG64AZUG/NCxF+i2GvgACqyu"
    "Hs3RVCgBAAIIJQGMgpEHAAKIaTQIRjYACKDRBDDCAUAAjSaAEQ4AAmg0AYxwABBAowlghAOA"
    "ABpNACMcAATQaAIY4QAgwACBIr9a1KJoNgAAAABJRU5ErkJggg==")
index.append('taskcoach128x128')
catalog['taskcoach128x128'] = taskcoach128x128

#----------------------------------------------------------------------
paste16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMhSURBVHjaAEEAvv8B////AAAu"
    "AQP88AAL+P8A/gP7APdrOEBQBVNYnh0R9/7s/OgAyefWBgKmvmWZz/Wp8P4ACfv8AAIBDgD1"
    "QOz//QKIhQEEuNk6GaX5NNkY/v/+/eX3bxF1AVPtECk+OU5WBkYGXQb1TzoMm07e7mb99jRQ"
    "mION9cdfZpYfbz7cA+rMAwggRpAL/sXI/2cKkGRgeHaJYcc7AYYmXT2GX6rfGP5//83AzMjI"
    "wMTNynDvwGuGruvPGeI0fzAwShsy3FzyikFjzR1GgACCuICd6e939u/MO/+EMmwR4WJ4+fIY"
    "w5+73xkYfrIxMPxlBrqCmYHtJyfDBmZfBu63/xn8lC4zMHMy/QVpBQggsAFMDL/+3f/KzPxY"
    "KZ+h2lmPoeTDTQam/0DNf5gZGP4zAQ1hYvjP+I+BWVCWYe2miww3XqUxcP/7+R+kFyCAwAb8"
    "YWJk+M/GxiDA9pNBhoeFgZVXmwEb+P2PgUGYF2goqyLD/3+vwWIAAQQ2AOhKBkYuSaAgOwMo"
    "UoBuBoO/QA1/gfw/QPz7L8QxH38BXSYgDRYHAYAAghgAdAEjiwAD4y9WsCBIDqQAZMA/kM1/"
    "GBi+/gb7hOH7T6DI/z9g34EAQACBDfgHMgColOnfP7AgiPr3H6L5J1Dz918MwFCCGMwIlACG"
    "CBBCnAkQQFAvADlMTBCh/whnf/8DMQDkd7AXgAogXmQEugBiAEAAQQwACjBwcDH8+8YCtuXH"
    "H4iNv0Ca/yMwyGX/gQQj0MV/oeEEEEDwQGRgYmb4C9QOUvgTGnj/oK6BYZABjGAISvMQEwAC"
    "CO6Ff///Mvz4yczwB+jUf0A5oOUMvxgRtv+Ehsn3n0wM/4Ch+xcaiAABBDbg938WBikBVoZ/"
    "568wbNwCdCIjMJSBVoIwKCb+/PkHxCBvMTNwfbjBICvOxnCJERJjAAEEMYCVhVVAU4ohnO8m"
    "w7N3N4BZDZoY/kHDGkr/Axoo4czEIKAgxfCTmRmsFyCAwMS7y8/uMc9fLMTw498/fmBIwPz8"
    "H0iAoxSM/4Pxg99MDPcZmZgenX3yCaQXIMAA5StYSgFpgrUAAAAASUVORK5CYII=")
index.append('paste16x16')
catalog['paste16x16'] = paste16x16

#----------------------------------------------------------------------
activatepreviousviewer16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9cKGBQVDU7ZcOMAAAMS"
    "SURBVDjLXVPNa5x1EH5m5vd+7Gc222wl2bVJoGCbprTFfqBQFaQWIu2hVA8tVbx48yDoRQ9e"
    "BNEG/4GCeOyhpVDECFGpgiBC1RiNaWNLEkPX7G6z3a+8X/v+fh7eJBUfGJiBmeeZGWYIpysA"
    "CJAIZCIgah7Olgc/Da3glyAM3kUUaTASWATYBjsxAMHefEJAGgjDSy88//TnF89OHfzjp6Vm"
    "92FrHYy9MBhDDB8KXagkfRuEl8qAAUSZ96bOHvzwlTcmCR7j+6/Xol6/6yubSWuY5bW1jV/v"
    "LH7VC3vvw8HDxwTPlQE/Pjp2tPTl6x/tL0V6E1EA7CoWICQwxsCAQZ6F6lIbV699c7taa5yD"
    "YDUZYXQQYF5vt4JKo9s60S/52Gj6+HFmGbd/XsFvc3/jz7urqFMdUtLY88Tukb8WHuT7XnwT"
    "BpAX951AVrmmHYQz1fmWt1rtHLNd213+ojO7Mld/q3av9Vl1oTnfaHrP1DMNx+RioM1DzfXu"
    "bNZk68ohhT2qiKd2jWBx88EnvXl/dnGpOe1z9MOTVJlBoAGOv13//Z/hR+3Oa8vpRxF3UCuG"
    "RXO8exxKwwgBFzSbqcOD46EY7jb8TsOj4GR2U17e9PzvcgP5d3y171C8Et8zMCAbUTbMvB3G"
    "4WUVxxgfHR25Mjk56di2jVQqhUIuj1w2j2s3ro8vLN6ZPn/x1Q+mTp1G4AcaIEqnUrh7f+nk"
    "x9OXC0rEYRHL0VojDEM4joOgH2Eo48JNZbIwVqFSrqAwNAj854TKUQXp1EBRDQ+PXMnlCmi3"
    "PRAxer0QSrVRqzXhupnCxMTEmyICYzSCIAQzw7ZtiDByuYH96siRQ8+eOXMKQRCBiMBMYE6E"
    "XNe1lbLGlBJ4ng8RAQBorVEqlXDgwMSQsiwnzmTSynUNiAgiDGYGs2z5SZExBv+HbTuxYlbo"
    "92NonRAYY8CcmDECZgMRBREGEe0YADDbUI7jiGUx4hg77W8ri8hW/LgbIt75JtdNKbWx4ddv"
    "3ZpLa20MEUDEW7tgMBOMIYhsKybjiBBEFNXrrd6/lH1C9uCr1lAAAAAASUVORK5CYII=")
index.append('activatepreviousviewer16x16')
catalog['activatepreviousviewer16x16'] = activatepreviousviewer16x16

#----------------------------------------------------------------------
checkmark_green_icon_multiple16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAAoLQ9TAAACMVBMVEX///8LgRkKfhkOdyET"
    "cioYajMSazAMcCgMciIIdx0CfhcBgBAChQsAiwYChAsIdhwUcioSbDACigkbYT0eYT4fWEkc"
    "ZDonUlAqT1ctTV0tTVwAiQAQogwQpw4AhgEnrBogrBwAfw8Ahgs6sysAiAYKgRgJfRgAehUL"
    "kBgAeAkAmgAclSUWkCMJcx8LcCEAcxgenCYAbw01uSIqmDAUhCUPbigKbScIeCEBcg9byDUe"
    "ohwQdCcQeCgKhRsKZyYXhCsGhRshmSEOaiwykT5zx14pmy0klSsajyUAAAAHnwcXZTcRaTFF"
    "qj8hji4AAABTxDA+vC0TWTw8lEMmijYAVgVdwD4eVEohZ0QxjD0RfyUAAAAiTVIADBEqS1kw"
    "ZlUzb1A5g0oALzYwT2AwY1otR2IvXFgkSlkoYk8MN1AvRWYkOWoxPGgkRGJp3URHvi+k9Vtu"
    "1EK9+2RGtzSW6FnS/2lJuDglshxi0ERGtDqz8m7Q/3JTuz9s1UJ60Fp/1V9sylDG+YHK/npW"
    "tD5810aR41PJ/JgtoS2T3GnM+ZHB93xNpTmc516o612CzmzR+qFOrUR61FpbrlK48nw8nDe/"
    "+Xqu72m17omOxnJ/x2yd4HshhzA7oznR/5Cr7G+F20t6z1ul4HhMn0qz6ouy7YVgwEpLqUK3"
    "9XNjvUZFmkqS6VhGm0Nsu1rB+IhYrEm+9pE9nj/W/5VmvE9axD+A40xFlEldrVXJ+JFqvlE6"
    "gE5oyUf///+C9aJeAAAAbXRSTlMAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAApxe9g4+0V"
    "m/1OIRc21W849s8bBWr2lcbpxBIoq7j0kYLAVlbi4KMn9/7887QCdAOa/tEd6+RI/tpt/Qe8"
    "68gIcSAT4PrqPIb6d5Zwn1wVNBZiNX0MAwAAAAFiS0dEAIgFHUgAAAAJcEhZcwAACxMAAAsT"
    "AQCanBgAAADrSURBVBjTY2BAB4xMzCysbOwcnFzcPLzSMrLIAnxy8rkKSAKKSnn5yioMqmoQ"
    "AX51jYLCIk0tBm0dXQGggJ6+QXFJqaFRGYNxuYkpq6CZeUVlVbWFZY0Vg3VtnY2tnX19Q2OT"
    "Q3OLoxCDk3OrS5tre0dnl1t3j7uHJ4OXd29f/4SJkyb7TJnq6+cfwCAcGDRt+oyZs2bPmRsc"
    "Mi9UhEE0LHz+goWLFi9ZGhG5bHmUGIO4ePSKlatWr1m7bv2GjTFiYgwSErFxm+ITNm/Zum17"
    "ohhQQFJSKik5JXVHWvrODDGQAARkZjFk54BZAPaJRPVDRVk1AAAAJXRFWHRkYXRlOmNyZWF0"
    "ZQAyMDEzLTA2LTAyVDE2OjAxOjIyKzAyOjAw9OZXtQAAACV0RVh0ZGF0ZTptb2RpZnkAMjAx"
    "My0wMi0xN1QxNDo0NjoyNyswMTowMOP4qUcAAAAASUVORK5CYII=")
index.append('checkmark_green_icon_multiple16x16')
catalog['checkmark_green_icon_multiple16x16'] = checkmark_green_icon_multiple16x16

#----------------------------------------------------------------------
treeview16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMdSURBVHjaYlRVr7yuZ6qk8vP3"
    "//9//jIw/P0PxP8gNIj/B8r/8w8qDpX7y8jE+OrG3ccAAcRi42Eo41AUyvLwGUTDLyD++YeB"
    "4fcfCPvXHwgfWezHbwYGNm4GBsaVG0UAAlA0xigAgDAMjLaDi///jOCLVBAKLrZGSHJTEp3b"
    "vbeBvQ5EAOdDFkUkDrIRfHP6k4IyKmlSYHbjCSCWj5//MsRrMTA4aAkzAPUAASPDh7evwJqE"
    "xSSAfCagZkYGJqa/DP///QWrYWT8z7D98m+G9M3/GAACiOU3IxMDJwcrgyAfBwMMHDl4CWgb"
    "E4OvkizDh2OHGX58fc3Aq2HIwCWryPD7128GNrb/DOzs/xj+AfUCBBDL3/9MYGcjAzY2dobD"
    "J88x/Ln/gsH4wwQGNpkrDM9fdjHIRxUz/AQGChvQBf+BIfkPaAlAADH9AZryD+RBJPD//z8G"
    "Lm5uBlExQQbWB08Y/p/9x8D6n4vhD9D9//7+ZfgHjJJ/QFv/MTMxAAQQyz8GZrB/kcEfoCJt"
    "dRUGGx93hs+qCgw/Pn1l4ATyf3/7wfAXaDNI/u9fRqBFTAwAAQQOA3QDQOH9++dPMIvD2ICB"
    "HUj//PWP4denL8B0ADSA6R8wXTABQ5OZASCAWP4CXfD/H7oBjAycnJBAfXf6EsPXj18YuFXl"
    "GZh4+Rn+/PoJNgAYIcCoZmIACCCWP0DFMBf8B4YFIyMj2H9Xbt5l4Pv+n0HqYjsDk+BVhtcv"
    "WhhEfVMZ/vz5CjQA5A2gF4CWAwQQ01+gM6AJAA7UVVUYOLnZGa6eP8fA/fkig8ivNwzMP75D"
    "Ag4ciH8gKQvoAoAAAnoB6BeQAMNfcAIBxamahjIQqwM9/pXhywZxhu8f3jOIGdsyMP/7yMAC"
    "DHZu1r8MnIz/gOqZGAACiIWTh5Np/30mhttv3gPTO8j5/8EBBcZAw/9xxjP8FQfS1z4x/Dn7"
    "DZiRWBhYgBrvfGBl+M3OzgQQQCwvHj9/tXHnf47P3/78ByXZv9C0/w+aC0HhC05o/xghmQGE"
    "QXwWJkaGby/fAQQQ4///GHFIEgAIMACRCWrbkqKM1wAAAABJRU5ErkJggg==")
index.append('treeview16x16')
catalog['treeview16x16'] = treeview16x16

#----------------------------------------------------------------------
arrows_looped_green_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQBSURBVHjaYmRIZ2Bg+MfAwAyE"
    "/37/Y/jP+J8BDH4B8V8gZmcVYWFmW8fw/df1Px9+tzIwsDxiYBUCyjMDJZkYAAKIiQEZ/INq"
    "+gOU+cPACsSMDN/+M8uyiot0R/elmWlY72L49TeC4T9IISNYC0AAMYJdALL0NwMnEIdICEiG"
    "iPCISbMzcDD9+8fw7++vf/8U+BR05hRM4/zC8pahd8FshqmbZ3QzMPFUMjCx/QUIIEaGDKDm"
    "HwwGSiIqszOc0kw0NZQY/nJ/ATrmD9j8f0Db2Bg5Gb6xvGN4zHqRwY4thGHf+hsMNfM6+/7+"
    "/l0CEEBMQFtVtKV0Ns5Nm27Ca/KL4Tr3EYbvrB+BXvzJ8JPlG8Nvth8MP1m/Mjz6/Jjh2OOz"
    "DK/+PWNQFFVkYP3PKsPw+w87QAAAQQC+/wEOZw6pl12YVswTvAAF+wQANvZGALnIvvL09fTu"
    "BAMFAvr5+/8eDR4aXlZWBeoC4wDMvt8AhKKMtfP39WUABQjnAohFR0LXR1FageHuZ2kGkx9W"
    "DMlTUq49f/00gYGd4QHDJ2C4fGfgUZBWWW+ga6y24+CuL4v3r2wDhvZEhv9s3xiYWBkAAgBB"
    "AL7/ARxxIO1EJEMIDAQP/83f0ALH2MF4AAf+kgAICxcA+QDqAPILFQgGDr0F+wDy8/XseQAC"
    "AdIA9foMAP0DBQAJEfsCiJEhjlmW4ctfFmBMAOOG6QPD33+KnEI8DcAEoPr9608nhs+8r0Bi"
    "DJ+/P2LgYAzQUzOe56Rpx/T990+GXedO/AAIIBYGRsbHQOdCovXbv2RfU//mGNd4yaY5DXev"
    "3r3Ey8DyBRQhb4BekeVjkczvT2vkFTL+xCDDoM/wNO8dO0AAsYDTABMjN8t/5knlYZVJ4b6+"
    "DFLMcgxn9Xzl5IVVtnOwcjH9+fX/rwAHP6+Pk4f4Q/kTDJeBjtG8zslw8ty5YwABxMgQzsDM"
    "yMQydWr81HQ5Mz6GmY+6GPQ5LRnkOJUZfv//A01twKTOwsTw/P8DBk5WNgb77+EMZTVT3h46"
    "d8QHIICALmD59/8v0+dPHz8z/PzCzfDx8y+GH7//Mvz+9Q/oo59AzUDtjP8YmP4zMyj+MWH4"
    "fJuVIW152/1L18+nM3BznQAIIEaGaG5g2v/LBFTdmWqbURLg5cEgwynN0DNz+q8rT+/e5GTl"
    "YWT8x8z05zcDw8t3H14/eXR3x5+/f+YwsHO8YWBlZgAIIEaGKC5IXvjFysDw9nOUuapjY7h9"
    "uMq09dPv3bl92ZaBhecDOMr+sf5lYOX+yvDtx38GNqgeoLcAAghiAChz/QFGxaefoJhQZGUX"
    "rWVg5Nb+/eeXL8P3L8BoBGbaf0ALWICu/fINSLPDDQAIMAB4ZXTygcHxYAAAAABJRU5ErkJg"
    "gg==")
index.append('arrows_looped_green_icon16x16')
catalog['arrows_looped_green_icon16x16'] = arrows_looped_green_icon16x16

#----------------------------------------------------------------------
viewalltasks16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAALqSURBVHjaYmRgaGJgY2dh+PWP"
    "iUHg910GN4bZDH8ZGBgYgZgFiH8B8TaGIoYfDKJA1l897wDddX6e8vLTZq6+cPHchDCAAGJi"
    "IABABskwnAeS/xgY+KQ6pL2dlBXd9VnE3TNNGNhU2wACiAWfZqAWhj9ALMWwn+EegzgDMwcz"
    "87OvrAx7HjEwfPn7j4GR6Z8MQADhNOA/yMEMDEK8XNyRrIyMzMo/1y67/fpo+a2Vf3T+3dGT"
    "uLdj5bf/P64uAQggFlTn/mNghvob6DdVC3OTba5VtSp//vxmmNdYm/Pg0nWfW8fbLG4dZ/Zl"
    "YPh8E6hsL0AAwQ1gZvjN8IVBguERgyaDMsN1Bi4Ghs5gKWEVJQUxoD/+MvBLCKsevcQw6xLD"
    "XwcGBs5pDGCr/jIABBALwva/QCO4GV4wqjPoMNxhkPz/R1123zEGhvf5QHX/GKQuXGeQZWDU"
    "usTwU4yN4d+rX1B9AAEEj4U/v0FB9pHhLqMqww4WJYbTDP933+fhZ2DgF2dgEBBjuMzFx3CN"
    "geExUNGX30CrwCEEBAABxAIKrt8/f5tqakvVK8gLKt689fjWvTt/G58wMFT8+PhBL/zpM9s/"
    "//4xz/388fl9hv9NQD3f/iOFG0AAAZ3e5u8fv+/T9Xuf/3/+/Pv/6asf/zv4zvnAwCDoBgkb"
    "hhBgWsgDMjWwxRZAADEwSq65Ez/7z/9TT///v/f6//8jj/7/D+j9/5+BP+MBUJ6LUEIDCCAW"
    "Tl01+VMvmRmYdzIwCPMwMLz4xMBw+SsDA6uKtezvszO9gG5YA/Hvf6wGAAQQkyTP73+qkgwM"
    "34Fh+OIbA8NvoHs1pYCGcX0HyXMjEjR2ABBATJ8urj7H9/sPg7oaA4MOEKuqMDBwfPjC8P32"
    "utdAW49BEvM/nAYABBDzt/cPzn55/s5HUFiSj4uVieHumRsMRxY2fX/9YA0oxLcSCgOAAAIi"
    "CSBmV+TkUpsuJGFxiI1dfhNQIJSBSAAQYADMQ/VyzP+PVwAAAABJRU5ErkJggg==")
index.append('viewalltasks16x16')
catalog['viewalltasks16x16'] = viewalltasks16x16

#----------------------------------------------------------------------
folder_green_open_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAIBSURBVHjaYmRgYGBS7xP4y4AG"
    "bhZ90AZSt4H4NwMeABBALCDNq7J3YEiEMXhcBRriDGSeAOJfQPwfiP9BaTgACCAWEHH39zUM"
    "A0CGAg3ZS8hVAAEENuD7/69gzn8gZARCEH351ykGdJehueo7SAtAAIEN+PjvHcMfoKEsDKxg"
    "hft/bGa4tPQOVj8DvQx21ZfLv4uezv86EyCAwAa8//cGruDxn3tgzXMylzPwMQniDLywqR59"
    "QOoaQACBDfj87wPCn78vgukbvy8w/P7/Cxhq/xj+/P8NdOEfhj3f1zHYc3gjm6MHEEBgAz79"
    "hxjw9M8DhmcrvjBkJ+cADf0I1PoXbMhfEPz/h+Hn/x8Mn4CWrVu4ERSgG0AOBgggFBe8+fCa"
    "AcKHaP4F1Ayy/T+YB4S//jJ8ZfsMs/0REB8CCCB4IH748Y7hzbYfDEHx/mAD/0E1gVzAADbi"
    "H8N/YCrYuWgfzPbjQPwKIIDABrx9/5bhzfYfYGO//v8EVgwCf4DOhhjzB8L++R/Z9sMgJQAB"
    "BDYApFnMj5PBUMACaMBnhn///0EN+Q8NRIhBIHVQ20HpgBukFyCAGIFYCxi3VxmIBEAD+oFU"
    "LxA/A9kAEEAgA0CpxxCIQSlMiID+L0B8AIiPgpwPEgAIIEaoBIhmhxqGD/yFZqw/MAGAAGL8"
    "//8/AyUAIMAAofEBNHbKjNcAAAAASUVORK5CYII=")
index.append('folder_green_open_icon16x16')
catalog['folder_green_open_icon16x16'] = folder_green_open_icon16x16

#----------------------------------------------------------------------
star_yellow_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOYSURBVHjaYjwrKcHwT0KS4cn9"
    "ewwfP3xkYGFkZPjPwMDw//9/AXkLi4vLX716O/PePQt2BoZfIPGfQGwmKMCQIifPsBmoByCA"
    "mIB8hn///jOIykgz8AnwM/wDCQANYWBiCtKtKpHziAw24GRgcGFkgABrPl6GRBkZhr///4P5"
    "AAHEBBH+D9TDyMDNzweymeHPv38MfIrSQXwWBgxGDiaMssxMwd+hthsBbWdjYoRYBAQAAQQ1"
    "AOSKvwwcvNwMApLiIEk+UQtNM2ZRCQYpLVUGUzVpez4GBrY4oHcNeHkYvvz9C9PGABBATP//"
    "/mP49/0Pw78fvxn+fP3FwMrOxcDCwGAvbqclwsDwm4FZnIfBUl8B6GMGfSduXoYvX38y/Aaq"
    "//3jD9i1AAHEwifGd/y3vrT8j0/ff//9/5eB/feff3xGksISTspAdz4GhsV3BtcoM1bWV192"
    "8nByfGJnYGL8y8rCyM3Bysy9/+VegABiXGOt+iZ4gYswg4oiA8M3fob/QK8wMgGd+JcN6C+g"
    "w4GGMjB/ABoEjJ1/TAyMQGEG9jcM9zZfY0jN2PkCIICYHzx+t/v2rmuOBmKfhLnVgZq/fwMa"
    "9BkYYm+BEfcE6IvnQBrI//2BgZHlNQPDj1sM2/r2MGRUHHhy/NW3IoAAYjQHGngOiINYGJfk"
    "5ktHmsYKMLGxACPtLyh8YZEHjDK2fwzvX/5lmN398n/H1rd73zMwJAAlngIEEIM5UA0wkTBY"
    "ARVbMzAUHktX+v3/otT//0cEgFgIgo/z/v97XOV/qYnEL6DSFuTYAwgglhSg4T+ADE7G/wz/"
    "/jMeEuQT/8fADjTyJ1AtN5D+A4zUXz+BfudjYOV/AlT6Yioo1mEGAAQQkzzQZk0glgYapCbC"
    "FyNlJ8/GwKXLwCBhyPDzPiPD3y/AgBQzY2AUUmPwtJfgEWJgjGJAAgABxARy0w9oKuNUFXbh"
    "NQOGCr80w4P1ZxnS0w59bs7Z/+nLlYcMDFLaDNrWxoyaglwByAYABBDDDiABwtsZGQ0ft4T8"
    "/v++9//+ZNX/JkwMt0BBo8bAIB8hxHjm2gTn//8/TPnf6qX/GRhs8jD9AAHEsABIzAPi5Rws"
    "hY9Ljf9PcxD+K8bAsA4oJAZMkQzGDJBABnpq8tZ0td8rw7T+szMwpsP0AwQQQwWQqALiUiZG"
    "RRsWhtXAdFIPk2MFYhMgFgRibiDWY2CIV2Vj2AZMTvYwNQABBgBj3B9qhU7FqwAAAABJRU5E"
    "rkJggg==")
index.append('star_yellow_icon16x16')
catalog['star_yellow_icon16x16'] = star_yellow_icon16x16

#----------------------------------------------------------------------
life_ring_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAPlSURBVHjaYvz//z8DJQAggBhj"
    "tLUZ/gEZ7EDM9v69GCc3d6GGkVGwspaWHL+4OOvPL1/+3bt168PmQ4eOvbhzp+8hB8fBDz9/"
    "MHz/+5fhz7//DAABxAAyIAKIE+Tk/KcqKLy6l5v7//+HD//RwZe5c/+v0NL6ZcHBsYSNgYGD"
    "mYkR7AKAAABBAL7/Ay0XFwAE/f1ZQgkJpFMpKQ4Y19cA/fLyAOw2NgDs5+cA9AYGAAXIyAAK"
    "hIQALCIiAy4NDi7U8vE54fj5pgAAAP4CAEEAvv8DLgkHKjQ9Qq5Xj48MBePiAP6trQDu5uYA"
    "s8rKALSamvzbubkAEM7OARfLywD8ExIAG4+UAA8dIiLGwr4T8/39tgKIIVPfcNn+Ndv//4E6"
    "9f3atf/rZWT+8jMw7ALa4A90bngIO/vNWy0t//9///7/F1DNlXN3/sdbOXxmYmAwBAggJjEF"
    "ZUc+RWUGYJiAwe1fvxgWvX596SMDgxeQu/EXA8PKDT9/ep16/vw7A9A7DH//MzBz8zDomVnz"
    "CDIzxwEEIAjeUgAEgQCKXsukBxj5MbiF9tH+19AShD5EnYHO8XuSZG5CRyfMAauVofoK6OM9"
    "V4y0Ur6lNcM5rCt9KKdkjnW7fwHExMzCwvTjLyPDTwYmhn9fvzKofv7MEMLA4CTOwOCq/OcP"
    "hyYjo4DBv3/ztH785Pn//h3D9/+MDL+AaplYmBmABnICBABBAL7/BPXy7vb46ekO1bSoAAy1"
    "rgAfLzAAdGhoDjPr67Xu3d3lERoaDy0WFhEjDg4A4UhIAOsmKwD4VGIA8/DtAPb59/QCiCFY"
    "R/fsbG39/7MZGP6vAeLzmpr//9y6CYn7d+/+//j79///P3/+P46K/H8YKL8OiJfJyP9PMjT+"
    "x8rK0g4QQEznnz+bce/N838sQP+9B4baR2UVhj9i4uAA5RYUZGBnYmL4y8zM8FFVk+EFUOwb"
    "ED/+8pHh6IO7b37/+bsYIAAAQQC+/wMBAwPO8w4OOBgCASUUFRkADniFAPsSFwD48/MA9/j6"
    "APDw9ADm7PUA7xchAPXc2QDr6uYAuRMT/cvt7oz//f2sAgBBAL7/Aw4ICADv+vq14wECPhgx"
    "NB0YLjUA+Pv/APP2/QDw8PcA7/L4APXv9QD68/IA4uzoAK3h3P3C6emf/Pz8eQAAAP0CiFEc"
    "GDiswOh69+07By8LU7eRhGScnq4Rn6SyOgM7Jw/Dnz+/gCnzEcPVC2f+nXpw98WDz1/6Gf79"
    "6wMlY5A3AQIIbsDnHz8Zvn3/xsD2/78qME35c7MwmzAzMQv+///v//fff4CeYDgOzPjAMGR4"
    "jpydAQIMAC7Qs7eQ85s3AAAAAElFTkSuQmCC")
index.append('life_ring_icon16x16')
catalog['life_ring_icon16x16'] = life_ring_icon16x16

#----------------------------------------------------------------------
weather_umbrella_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAPqSURBVHjaAEEAvv8BWVpxAP//"
    "/U82NVefDQwiDPr6//3z8+Xw6e3e3eL3Edrq7eLH7uTRr/32z+wEE/kABfQCAPX/9ADbyNcA"
    "AgACAAKI5eObZwzvvn9+pKwqxf/+vQTTx/cMrBzMYhLCvEYSfLx/rO/w/kh490up+cf9+4Xc"
    "fx4vZ/jPwYAMAAKIRUZGhUGaU6tD4v8rJrF/qxnEhK8zCDG/YuBiZmTgYuNj+M8qzHhHR158"
    "i4nJwu1n39j/f/KnjOE/wyeYAQABxKLJw7TWSfyYmzp/JQO7wCMGBlag6D8g/gHEb4H4tzCD"
    "tJ0ag6RmAOtbu6r02wf7fT+t/Bjwn+HfaQYGRgaAAGLW4/9bGyj7RIyT5yuQ/xcUHmDN/98A"
    "8V8hBgbXAAZGCzsGJrYvDI/f+jGwqovzvvx/LPjbmR/b/v9legUQQMxPvv5Zvv8Rq5aDlJQ6"
    "HydQ87f/DP9fMwKt5mJg8tRmYDS2ZmBgZmFg/nOU4eJrGQa+P0ADNb9wvn520eL79V9zAAKI"
    "WVJY+MeJu29WXHzNoOGtKKHD/eUfw7/fQA3OwMDSYgK66g8Dw8/bDEz/rjHc+WzC8OmnAYMg"
    "hwbDM/YzYm8P374IEEBMTEDLGNmYGPbdeB7ZfvDXRQZmcQZmK6AfpN8BXfOZgeH7DQaGP/eA"
    "vv0LDNif4ODh+CPMICITwswuwx4CEEBMP/78Z2BlYWZg4WBhWHvjffLslx8/3PkO1PjuPwPD"
    "X6DyP0AX/GcGamNi+PWXheEXkAs0C+grM4b/vGLyAAEAQQC+/wP7AwkABQMBAAUEAgDz+fjn"
    "wtvwusPKFSHk4BYh/vXx/yEc/gEADAQA/gH/AL/kTQDk8CkA/f7+AOb1BgDlxLPlAojFytIV"
    "niiAdjLwczKeZ/z4RnTihrMX+tZ97so9xdpRXvpFgVtVlOHUKx2Gl8DYYWb7w/DqGzAdfmP8"
    "BRBALHwCwmDNjODoZ2IQZf9Z+/DJWdX3X18GAX27e8PB/+tfvP2xPbhAyenA7hsM/+4t/cnI"
    "+O8Pq6As+9+Pr98BBBALy/8/YAP+Aa1nY/glzvT+Vvbpa3cOAYV2s7EwMvBzM/w6foXV+0DG"
    "2Qmvnu7+zvz703p2Tp5PTKw8ql8+v3kOEEAsrMxMUAMYmTj+fVt++e4DrteffxZDPPWLgRkY"
    "PuxsTD/u3n2S8fsvkM/JBcR8DD9//L7w+/cvBoAAYvr+/QvDNyD+++eby4vP3+zO3H+3B6jz"
    "Bkg7OLDefGVgBPqPjZ0VGlKMEJIJYjFAALE8fXafAaTi/7//L+7evT7r25dPU2CB+hfor4cv"
    "vjIY8PMx4AIAAQYAt4+PZvkJ5icAAAAASUVORK5CYII=")
index.append('weather_umbrella_icon16x16')
catalog['weather_umbrella_icon16x16'] = weather_umbrella_icon16x16

#----------------------------------------------------------------------
cross_red_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAlASURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsMEajvDzD3z9/GBgYGRmAocLFyMRUwycgEPD98+djf3/9KgEq/MAEDC2mf/8YQK5m"
    "BLKZoWwG9FAE8oFmcDMyMrby8PD4f/v8+QBQXxELI+N7sH4gTvnyBawUIIBY0F30798/pn/M"
    "zN0O2tpZJh8/Mtzi49Pc+eSJxL9fv+KAlr0jxlf//v9nBzppsrWubqKGjAzDvW/fEo7u38/3"
    "9/fvOCZm5q/IagECCCUK/v39y/SbkbHHVl8/ywXoQj5gqJj8+MHgJirqzcjCMv/v//98RFjO"
    "+vffv8nmSkqJRpKSDNx6egyGSkoMVq6uQUDxWUBHcIBCGQYAAogJEWr/uf+xsEyxt7QsdH3/"
    "nuG/uzsDy4oVDP+KihgM3r5l8BAU9GNhZV39j4FBEo/lfECfz7WSl081FBZm+B8YyMBaWsrw"
    "z8WFwVBRkcEmJCSKgYlp2b8/f8RgegACCO4AoMHV9ra2mV5An7M4OjIwFxQwMDx5wsDm58fA"
    "WFPDoP3uHYMnB4cbMwvLXGBICKBbDvQAy39Gxsk2YmKx+pycDIxhYQzsbm4MDE+fMjCbmzMw"
    "WlszGGloMFjHxgYyMjN3wPQBBBDcAbyioqFWoKBhAgrl5zMwPHvGwPDpE9gADi8vBsbKSgZl"
    "IN+ZmdkTGI9LgL4VQnI8x19GxmkWfHxxmiwsDIyRkQxcIMtfvmRg+PwZTDOamDAwCAkx6AId"
    "IaKh4Q7TCxBAcAf8ePfu0FWQ5cCc8H/+fHBuYPj+HWIAMCQ4gQYyAR2h9PMng9OfP94sTExL"
    "gBYLATEowU0zZ2FJ1QBazhAby8ANjD6G168her99Y2D4+5fh/+nTYA/dB3ro0/37Z2D2AgQQ"
    "PBf8+/GjdMfhw2IMVlY+5qtWMfwBWs7i48PA8BWaaIEGcQOjhuH3bwal5mYGNgYGz/3s7Bv/"
    "/P//zfbfPzcVYNZiBVrOB/I5yHIgHww4OBj+XrzIwHThAsMdoBmHliw59ffbtwKYvQABxAgr"
    "CZukpRn+/f4tzMTGttDLzMzb5M4dhj/e3gwsnp4QR4AMBIUKDw/Dp/37Gb53dTE8BvrsFzMz"
    "gxJQP3tmJoOgry9ELchMkFp2doY/9+8zMJ0/z/AQmJv2r1596s+3b6Es7OyPkqAeAwgglGzI"
    "xMT0luHHj5jdZ85svyQnx8C4cSPDr82bIYaBghJUeADjk8/SkoEDmEglgBbJAUOKNTGRQRCY"
    "0hnevIGogQb776tXGf4fOcLwGBgVB9evP/fn69cwFja2R8gFF0AAoYQASBPYRX/+SLJyci5w"
    "UVNz0wEmRkZgsLJZWQETyg+E77i4GD4cOsQANJRBBBTnMDkQBvr81+PHDH/PnGF4BkxThw8c"
    "OP/r8+dwoM9vwyxPAjkSCAACCKsDmEFF6Z8/fEysrItdlJT8dJ4/Z/gP9CEnyBEgjTBHsLFB"
    "cg3Mcmic/3z0iOHPuXMMz3/9Yjh0/PhJYHoKAwU7ss9hDgAIIHgi/I9EgzAzE9MnYPGbeODe"
    "vaVssrIeStu3M3wDGshlbAyxEJowGZDrAqCDfty8yfALmOLfANPM4QsXTv/9+TMSPdiRAUAA"
    "wR3AA6qIoIpAFQ24sGRmfgesiCL3Pny46Y+oqK3Crl0MjMBQ4gQWrwxAx8ANBYUGMDH+vHeP"
    "4duJEwxvgGYdu3Pn0p8fP/yBBddzfEU3QACmyiUHQBgEotDIqve/Tq9kdNFE0LYzxBhXBBbk"
    "8Z0PYENCjkPfP42vlhCI9Tuiu5lMnODeWtqCHVDePdUTPkDlQPWGrjy1SrhfOsZZEE8FpJKS"
    "lXX9AJYAgqeBCYKCKA4AGQyu1ZiYptsqKydqvXrF8PbhQ4bfoDQKchgwoQHrDrBDmUAYmBZY"
    "gEUwMxALSUkxgLx99PTprYzAGhDozHfoDgiD2gsQQDgbJMBqmRVYyEw2V1dP1AVWy5+Alv+H"
    "GgCWB5aIv4A54Dcw2wHLDwZGUGIEYhD9CVh5yQDLC0tra28gf/4/PLUoQAAx4bCcD+jzudZG"
    "RqnGwDz86dYtFMsZkTAo7oGVCwOsigWLAx3x+cULBkVgqFq6u/sxs7HhrEUBAgjDAf///WP5"
    "x8Q02dbSMtYcWHZ/u3KFAT39giz5C8SgKlEYUhlhAJAjvgDrECVgtWwVGOgGzNJz/0G0oACA"
    "AGJCs5wDGOzTrGxt40yA1e+3s2dRfA4DYMuB8f8bmOe/ArOeMDAE/sFyDtwFQB5QzVdg1MkD"
    "Q8IsNNQTmGiXAM0TQlYGEEDIDRIeYFNsqoWbW6oFMM6/A7PTX2zRA7IcaPEvYBwfY2K6fJSB"
    "4dRHoG+FgWLYQgIURd/u3mVQkZBgMImN9QbWNSBHSMCkAQKICSnIWqzd3ZNsgZb/ARax/7D4"
    "HCTGD7T4C9BHh3//Pv3z/38voJjr8e/ft7wGhoAgMGv+/YfmDGhI/Lx9m0Ed2MQzTU/3ZOLk"
    "7INJAwQQ3AHcYmI+enzAxLpnD1afg6KCHyj/WVSU4cinT6d+/f0bAsxeT4AGfAKGXsLpt2+3"
    "vgYWUoJANcCoxEg3/4Eh8RdYOSkDGyQCKir2MHGAAII74PurV3tvgapIW1usPucFJqbPwBbu"
    "kbdvz/368ycM2MR+hPAk41tgzok5++TJ9rfAUpAf2PL5jx4SQHFmVVWGR8DS8uP9+6dhwgAB"
    "BHcA09+/tUe2b19/DliIsJmZMSCnTl5g/L2VlWU48PTp+Z8/f0awMjE9xEjNTEwfgP2K5JM3"
    "b+56ASwN+YAOBrchQAUOsJxgB7aMHwCj9+Ts2cf+fPlSDNMHEECINMDM/Irx16+oI1u2LLsI"
    "dAQnsNIBSfIAff0CqPng3bsnf37/HgBsit3GWagwMT0HOiL0zKVLm54CQxPkcGBxzMANbBE/"
    "AmbpE7Nn7/v76VMQsNS4C9MDEEAo2ZCZlfUHy///aSf27Vt3VUSEgcPGhuEZMOEcunr19B9Q"
    "rcbM/AgjbYDqAqRaFFgsfwJGR+KFM2d2PANaygkM9qfA0vLs6tVHgRVYLNDyl8j6AQIIXhdM"
    "BcYbrNIAxpcg0KBeEUlJp49v3lwANqNygS3hx9h8zQmtFeE1KJQGmiLCxMIySUBS0vrTq1dn"
    "gB4oBvr2AUxfKNRegABiHOjeMUAADXjvGCDAAO/S/p9cVw0HAAAAAElFTkSuQmCC")
index.append('cross_red_icon32x32')
catalog['cross_red_icon32x32'] = cross_red_icon32x32

#----------------------------------------------------------------------
envelope_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANdSURBVHjaYvz//z8DJQAggBh9"
    "fecyfPr8i4GPl5vh+/cfDL9//WVgYeVk+P37H8O/v/8Zvv34K8TMzJKiqa5YdOfO0e/Hjjc7"
    "MDD8fMjAwAw2ACCAWHCZ/PPnLx4mJtYKJSXJJG8fTUlTY2WGbbsVGS5fXVH1+dOVdJg6gADC"
    "MODvv39Mf3/8rlNSls60d9AUs3OQZZASY2M4d+UPg42zPIPZ1tCQvXuu1DEw/HvJwMDEABBA"
    "YAP+/fvP8Of3X67//xlrlBTl0mzstYStrIEaJVkZ+HkZGC5e+M7w5z8zg7QkA4Ovf4jQoYOL"
    "in//vlsGtI4BIIBABohxcbHnSUqJpFvZaolYWMozSIozM7CxMTBwsTMwvHv7h+H1278M8koc"
    "DMx/GBicnbQYDAwDok+f6m0B6v0EEEAsgoI8J+3t1RRs7LQY5GTZGNhYgQ5jQnjp7q3vDKLC"
    "QJfwMDIwACNMUQHkijDJs2eWZ//796wdIICYjYx8W/TMNNmevXgHVPye4c2r7wzfvv8FxgAj"
    "w7Onvxg+fvzLoKLKzcDCwsjADAx4Lk4GBgEBccb9B+4ovH1zdhFAADH//cuu5OHjb6SgLs7A"
    "zsHNAAwHhk8ffjM8evid4d27nwzqmnwMfHwsYFexAD3MDKRlZJgYXr8XEzx0YMdLgABifvfu"
    "7W1RQc14Oztddh4OVgYxYU4GWVluBkVFHgZFJW4GTk5IRDEDKS4uiCHPnjEwnDrzkvH0qV2M"
    "AAEEdBYvg65u3Kqjxz79f/3m///nz/7/fwOkP378///r1///v3/////v3/9g8BYoPnvWnf9m"
    "Zg1/2NnVzzMyMvsDBBDQVDEGDg4l47rabd8/f/7///2H//8/APGXL/////kD0QgycNbMG/9t"
    "bet+s7GpngM6KB6IOUAuAwggBi5uaWDSFWIwMkzfc+Xyj/+/fyM0vn717//c2Tf/W1tV/2Zm"
    "VjwFVB+LnvgAAoiBg1OCgY1dmIGH18i+r+f4b5DGly9BNl7+b21d8YuFRek0UF0UQgdIPyOc"
    "BxBALHx80sD4/c/w7z/bwc1b1p9mZWO13LBh7e8DB5ed/fv3YR9QzXog/oMrzwAEECMnpxDY"
    "RGBQgaJQ6+8/jsI/v58fAgoux64R5IK/DOBUBQQAAQYAP5FRv1nW25oAAAAASUVORK5CYII=")
index.append('envelope_icon16x16')
catalog['envelope_icon16x16'] = envelope_icon16x16

#----------------------------------------------------------------------
taskcoach48x48 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAYAAABXAvmHAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAABjeSURBVHjaYvz//z/DUAYAAcTE"
    "MMQBQAANeQ8ABBALiGBkZGSoE2FgCOZlYGBihGIGBBukKMQug+En00eGX/+/MHz++UnwP/un"
    "WAUl0SAFaWnVf0y/GO+/vH3/4qUbW349ZZnLsPf9q7OazAxGnH8Y/gL1MgPNwJdQPwEVCVwg"
    "3fGg5A8QQGAP6LIzMIQBHf+XoA4mhr///seoG/JX+bj7aWopaDBwsHMw/P3/j+HH7++SNx/f"
    "tpo7f1XC9ePnszj+/90LChugy3nA+B+DEJAWAor9AobIDaDMR5JdbAPEZlD2MwgFEEBgD8wT"
    "Z2D4CQwiFkbcen8yszL8Yf45wdiJPT/UO5jhJ9BFz349Z/j98zfQA38Z/gE9wS3JxZBUFqg2"
    "m4tty95l509r/fki+I+RQfA/BysfExcXNyMvNxPD799Ay18/ZfjNsJqBg6Ef6JFHMDuc+RgY"
    "9n7CYrkIIwOjHzAsHP4xAK2FAAMIBRBAYA/8ATqeDab4H4M00C1S//4xiAFDTASIJf/+ZxCr"
    "P7xcZpK1WugHUaCa3ywMlz6dZWAGJjSQ48EQRH/5y8DExMRgl6TO0fTupa0kuxFDSIgVw39W"
    "VgZGLg4GBhD+84eB4dZtaYZ1Wwr+HzkXCYyNTDYmhvXglMDJwGAIxP2vgKkBmuaYjJgZGOOZ"
    "Gf7z/mX4/x3TbwABBPYAVC3fn+8ME1i4mILYpUT5Wfl4GdgE+RjYhAUZWIUEGEJEBBkufGBh"
    "6Lu8ksFJ8SbDNyB8/u0Z0AtATzD8ASejf0BP/AFiFlYmBgFgiGUtuclgz+vLICoLTJ8/gQ7/"
    "B7TpHzPDPyNtBiYTDQbGzXvF/09ZtHrdq98uQPsPTHjJwFAOTA2CQGVv+ZgYmN3YGBhdger/"
    "AmPtBygzIbkcGuIAAcQC4//+ztDNb22SKF9XxMAuJcHAxMnK8Ps/M8PXP4wMP4Hp4CETM4PG"
    "sbsMf/bsYZhzdg5DrHkSw7U/1xk+/ADG+X9GoNv+Q9wHwsDge/vxFcN75m8MybMfMRi4+wJT"
    "DjAEgcH9D1hgMPz9w8D4/xNDlbc7A//TR8y8E3d3AZ1gBUoMncDQZ1BhZeDIAGYdLmDS/PEH"
    "4nAmcEizAXEhkC3OcJ2hESjyESCAwB4AJhcZYP4Mla+sYODUVwf69g3DjP3/GJae42BQk+Jk"
    "YGZmYmBlYWFg5zBjcBeOZNhytoVh3o85DJryOgw/f/xjeP/tG8MvYCb68xsYE7+/Mfz4/p7h"
    "94dvDAyPJRhMfWQZIk20GDiANrEDHcIKLJLYWRkY1t/8zVB18AjDVG8bBp8tR0wdPn33PfCd"
    "YT0jByMDaxw/MLkBE/v3/5AiEOTGP3+NGdiYJ1noqlu9AyaVW1Pvf2LoZGgACCCwNDAfBgvw"
    "Mwhy/nkK5KgBy4kPDD++MjO8+SHCcChRiwGct99D8FvXIoao/scMu04uZnh45zEDI7AE+w8q"
    "vv6AyxcGYMpiYHjDysD8VJEhRDOAIdnXlkFKEJgB//5j+POXAZzZ/wKTQ7AiC8OR+9oMq769"
    "ZgjzNGIounu0BOiBTcAQ/guMenhmBRaVzH9//SvhERWqT7Iy4uTh5WSYuf3Ef4Z34ETFABBA"
    "YA/8+suQLCQJZFzZBMzd7kDHMDNE6v1kaN7/juHS428MejJcDH9YgA449Z+Bz1+AYV5GP0PH"
    "KnWGzVc3MXz89wIYrN+BdckfBu4fogwCP+QZVHi1GJzCLRkcrCwZ/rOLMTx58ZOBkRGpjgES"
    "HGz/GFqcxBhi1uszWFk6MHhsOWll+fGPx3EGhq2wZP33zz/dP0xMk/S0VRzKrIwY3v77xdC4"
    "6QjDu87XMxmeMMwHqQEIIEZQZbBVhPGvmT0Dk6gqFwNDLlA/FyjDvWFwmcrPYKSpy1DtLglM"
    "68CMdBSYHMUYGDh0gGkUmCfevHvN8OnLGwYm5n8MbCysDNzsXAw83DwMvLw8DEyskLj/+xeY"
    "jv9B88c/EP8/mP4DpJmBvtr37B/DhqvHGRaeK2ZYOfvcwYhPjA5sZYIMf/n/FXIJcDQGW6nz"
    "ZmtpMpz99I6hatMxhnddzxcwXP5bDjT6FcjtAAEEi4HDr18x2ItKAOP/5hEGBgsPYFJ6zRCh"
    "942h/dhbhvZgKQZ+PjaGv+bAWDj7n4GJDxjPv/4wSEqIMIj/EwU6ClgIAB30F5hMvoPwe0aG"
    "V59fgjO3OK8ow48/PyAxAMyILMA8AK7dQZUO43+GQE02hoMvDBlmKYYxpElftO8VY8w4zc3i"
    "pa0m4ptqpcngISIOrNhfMzRuPcvwrvfVMqDjq0GOh8USQABBilFGhpnPnjDYa2gCLbkMjAFz"
    "P6CLGBn8NH4wFG39wHD8zhcGFUEOhm///zD8efWPgfUqsIgDVi5M/38DMzgjAzMTxEHA4p6B"
    "l5uV4dGHRwyxe2PAReoc50kMliqmwFLoNzgm/oA9CjQeWPn8BOaZZy++M9QY8zPkvQ5meGCz"
    "mOGt7NvpwS7qDInaygxyHGwMO9+8ZOjacZXhdf/ztQwXflUg6mAIAAggJmgLYeu7Dwz33rwD"
    "ch6fBfoPmJmZuBjEhH4xmEl9Zlhy/B2DhCgrg4I0K4OiMSuD9HtmBjkJFgZJURYGYQEmBlAF"
    "C0oxoOTxHZi1Lj14wPCB9SWDqvF/hoiNKQxrDh1mePeOleHZq98Mb97/Yfj0FZhP//wDxgYD"
    "g5gAF8O7L3cZnn1sZpgNLEgTYqwZivQMGcRYOBn2vXrN0L/9NsPziU83/z/3vRTkOvSKDCCA"
    "IB74z/AJWAJufQSq1L8D0/r1g8AyTwhctMTof2HYfPYdw5fvfxhevPrD8IT7D8PtKz8Y7tz7"
    "xXDv6S+GR89+Mzx//Zvh3Yc/DN++/2P49es3g5WSIYP0T3UGJW4+BjcnIYb8o7kM118dZVCU"
    "5WaQEgN6mp+JQYiXg4GLnY1h4ZnlDO4rwhnYFK8wTPe2ZXAT1gRVEwzHgCE/Zed9hidTH+/8"
    "d+ZzMdAx97E1cQACCN6c/sfEsObxE2DU/gRyrm4HCrCBi0YvjW8M7z9+YDh88zMDLxczg5As"
    "E4OEDDODxBdGBkV5FgZFGRZgzLAwyABjRFyEGZiBge0OYJkcIhfGsP3EM4ZoLTsGTxcphuhN"
    "2QxL9uxmePOajeHTR3aG0zceMvjNyGKov9jAEOohxVBj5c0gwSzN8A2YX86+f8owdddjhkcz"
    "Hu//c/J9IdBFt3G10QACCF4TAwuJE+8/Mtx6/opBTY7vCgPDC2BsCfAyiAm+ZbCS+cyw7Ph7"
    "Bk8DAYbPP4BtHkUGhtcXfjOwSv9n+P3tHzhNA/MuuHkLapqzAYtcL3VPhgVXFjPsu/OAoco4"
    "EVhQzGIo31HJMIuHCViA/2KoPdjBIKzxhaHV0plBnVsZGPG/GH4yfmS48ukRw6wdjxj+Lbj7"
    "9NexNyDHX8fXQAUIIBZoEgIXRsBkuf7eQ4ZyOVmga67uZ2BwCmRg+PqMIdboE0PRzncML99J"
    "M3z6BIwWfmDzApiOeV8zMXCJM4NbsaxAApSRmYFFDKjEYWMVZYhUC2VYcmIKg5eCLUOJQSIw"
    "VhcwpGytZODg/weMFXGGQFUXBi4GAYaPPz8D8+EXhmuf7jMs2PGA4f1JRQanL9zC3Iz7mK/9"
    "h9SRuABAAIGT0Mff8Ob+qkdPGP59B+m4sxsYrGzgpqqX2meGL18+MBy785lBVYGTQV6WjUFW"
    "j52B9zkjA48AC7AeYGL4DixR3n78x/AEmE/uPfnNcPP+DwYrWU+GP4/EGTbf38PA+p+HIUnL"
    "hyE4QJwhL8iAIVLVjYHpDx/D+18fgRXvF4YrH+8yzNn8iIHzjDTD1IAshm9Z0zhUZJXK4EnE"
    "rx2rBwACCOwBYMhDmguMDBc+fmK4+OgFkPcRmOxePATWXAIMogJfGGzlPzEsPvIO3Jy/9+g3"
    "w0PufwzXLv5guHXzF8PD538YXr3/y/D1x39Q946Bi5uZgU+AgcFSX5YhUNmXYefJuwx3vl9i"
    "YPnPxxCs5MqgK6DL8PHHD2DJ8Z7h1z9QyN9hWLT1KQP3OSmGTLt0Bgcba4bSAHWG217NgQLM"
    "zPqQ9n8wVg8ABBATrD/w5z+0zfSfYfWt24yQXHEbmIw4gW2Mf98Y4ow+Muy48I7h+ftfwGYA"
    "M4OgAhuDhCQLg9h3RgYFRXYGOWk2BikJNgYRYRYGHh5mcP3w4es/BncdH4aPN4UYjj+9xPD5"
    "72uGDz+/MLz/+Z7h8++PDN//fWa48fkOw/JtLxm4zkkyVPqlMgSFuDIwArsN5oJ/GcLjAthe"
    "m6WWQnpUXxiw9bcAAgjsAVA1D0tGwKbruoeP/v3+/A0o9RhYnP4FttaAcl5qHxj+AJvOB69/"
    "YlAAOpSbk5mBQ4uN4fO1X2CHPn/9l+H+098Mtx8Ci9mHvxnuP/kDjJkfDDLi8gwOEi4MB08/"
    "ZXj+8yHQ4Z8YPgGTzbd/HxnufL7HsHbHWwbmkxIMqQ6xDNrGTgwPnv9muPvwJ8N5YDGdqc/B"
    "4JBWEcEgb2sHSh5gx/IBOwws7HAPAAQQWOzLP0g7hRFSK9/89JXh+O37wFrmB7DofQXErMIM"
    "InwfGBwUPjIsPvwWXN7fuP2T4RHTf4YHwHrgxe3fDMDCiYEV2LUSEGBmkBRnZZCVYWOQA2Jx"
    "YCaPdQ5geH9NgOES0KzPf94zfPkLrDW/PmTYsvszw5/D4gyhpqEM+ubuDL+B/Q42YG3OzwNq"
    "7P1huP/gA0O4LBszR2j3VGBG4wO72AXYDIpdAFQkBeYCBBDYA+f+QlLMj7/wInXl9augchGI"
    "Hxxg+M8pA4yJTwwxhu8Y9l0EJqMPPxhkgbEgA0w6SsBQEn7/j0ES6FghQRYGbmD6B2VqYBOd"
    "4f3nf8BY+QmsG5QYNFisGc5f/Mzw7u8zhsffnzLs2QPsVR8RZ8jwCGGIjgtg4GQHqn34hmH/"
    "mRcMG469ZNh8/gPDoce/GYRFuBi01YAdj1MrI8GO2wj0ALDyY2CCZG+AAAJ74CjQnZ+Brv7y"
    "Gx4vmx49/Pv1zTtgVL0+ysD4lwPcNvJUB7Y8f39g2AXMC+ICoLYDsNhUY2d4C4yNl8//MjwC"
    "ZuY7D0HJ6BfDA2Byeg3M2D9+/WfgApb9kS6+DG/PCzDcALZgTxwE9g8OSzEkuvoxePj5AVMo"
    "C8Pzl98Z1p14z8AqJsygriPDYG8my+BlKs6gIc3P8PrdFwbF6+tswF0EUPfyMrDZ/x4yFgAQ"
    "QGAPvAA6/iFQluk/PBk9AXpm/7VroO4fsF309h6waSHBIMr7hsFJ+QPD4oOvgT0xYFq/D3To"
    "t/8ML4A5//VVYIuTnYmBH+gxKUlg8pFlZ5ABZmxxMVYGTmBH3cJch0GP25zh5GI+hi+7xBlc"
    "DZwZNE28Ge4Dm2Z3Hnxn4ODlZjBW5AV3+sX4WcC9SGCKZLjx5hfDS2AbJ/3vE19RJgYl9EwM"
    "EEDwpsS1f5AK7Rui1lhx9TywBfkHqOQZMDNzAqtfYPqNNnzLcPzyO4ZHr74yKAIdCErrKpY8"
    "DPxv/wBLIFZwEgJ16v8Bg+LHL0gyevIS6NFnfxg87WMZHNj9GYIcvBncAwIZhES4GcREgQ1D"
    "GXYGSSl2BntzMYZnT94z/Ad6QgDoaUEgvgBMgiKsjxnsZL7ye7Aw5KJ7ACCA4B64BcrMQA/8"
    "hOYDYNjvePLk77unT4G56tMpoO+AyegvM4OH6ksG1n8fGLadec0gDmyU/QEmLVYlDoav74Et"
    "TWDSefnuL8MDYEV2+wGwsffoF8Ort3+BVTxQDbBONDKWYyivyWaIiA8HFsG8DBxAI0Ge/Qu0"
    "7QuwSfLzPyuDCAcnw6GzLxlEeSB96OMPvjEY8N9l0AwWYPDgYEjgZWSQQ/YAQADBPfAe2txj"
    "ZYB74C0wAHdeOQvy0WsGhg93gHEqxyDK94zBRQ1YGu1/Bao1GH4AO/N3X/5heAqM7/snvjD8"
    "AJYkHJxM4KQjJ8cOjCF2YCizMvDysYA7ML//gUZYgM3fL3+BnvsD9OwvYBL6CcwzvxjeAFu0"
    "utrCDL+ByfPW02/g4Z6Ljz4xmHHeYhCwFWJQ1+YUcGViyEb2AEAAoQzuXoU4HGwJtKG0/PrZ"
    "X8DyHxgUbw4zMHCrAltMbxmijV4zXL76luHGg08M/JyMDMKCrAxqtgIMIt9+M4gCSyJeflZw"
    "kcrACIwhYIYClUifvoDqij/AMv4XOHaevPgDrrk5uVgYJCRAeYYDTPMBk6GptgjDiavvGC4D"
    "65HXwBamqRTQVVzAfoivKIMHG0MyNyODJMzNAAGE4oG7QPwZaaQL6IH9r179ffrgLtADP88D"
    "RTjA7SM35acMrEyfGNYffcEAbHGBG3CsEqwM/4CZ+Mud7wzAgofhAyjtvwBWSkDH3gE6+sVr"
    "UCcGWA8BY0JGmh0cO+LibAxcvMzgvibISqBfGESBVpiqcDKIA80tWfOMge//BwZtBQVg/hNi"
    "ELXgYdDR5hB2YGTIgLkZIIBQPPAViB8gJSNQtgAmoM1XjgETE7Cpy/AF2D7iVAWWRg8YXDU/"
    "Mizb/4LhN7ADA+rrfgCG8BsBdoZLB4GV1GNQz+svuF0kBAxRUIkkC2wAigCTEjsXE7h1CRpi"
    "AaVxcW4GBjm+fww8wEb2zccfGCbufM7gPfUhQ8aWdwynrj1h8BO5wiApCCzOmSUZmDh+MMj7"
    "CDPYsTLEAbUKgxwIEEAs6Ln6MhDrIPGBnllx8/TPjB+fORk4PgM7/OJhDAwvDzDEGL9kiJop"
    "ynD57jsGGQVxhudv/zPwqHEzMN79zCAJDD12IVZQFgH3E34BMTMwiDmBhglxAWmgrT+ARdSD"
    "V98ZNt79zrD/1neGcw++AHt1rxhEGR8yGPPfZqgVv81gqf2MQUv8FwPrV0GG/yw8DIwSQQws"
    "ip+BWWkWF7AmAdauDG8BAgjDA8CsygDqGvMzQIbbQQXBu/d/b9+9zKCqLX0N0g75x8vgqnQf"
    "WO3LM6w7+IyhRk2cgQ+YFFhAY7cyHAzvrn5mELYWZGADJl1gl5eBG1gC/fv1h+HZ2+8MR+9/"
    "ZThwHdhWATbNn716w8D++yGDNs8thniJOwx2au8YDORYGWQkxRhYhDQZGHg8IeM4wPYYI6i1"
    "+fkuw5NV3Qznf/0//wfiTAaAAMLwwG9oXjBngM8XgJL0ussHfpRrewLTyo+bQBfpMYgwHmdw"
    "1TJkWLv/KUNpjDZ4tPUfMKUpOoow8DD+Y2Dj+gesQb8znLzxieEwsAF46MZnhttPP4GHLeV5"
    "XzE4yLxhsLf/zmCmzMmgLCPLwCtkAkxT/JC2/VdgmfgRWNM+ARbfH4FlI7CXBhrm+fvhE8Pd"
    "M38YzjIwHIANrQAEEAu2NjYwnBlMUZPR6junv5d+ecPHxMNzDBg9UUBDNzNEGz9l2DpFhOHi"
    "7XcMdiYSDI+ff2e4/fgjwxFgCXLo0geGiw+/AUuw38CM+5/BVP4PQ5zzV2CH/y+DtrQwsA4R"
    "BWbeHwwM34Hu+AjsNV7bBgxhYDf2O7DI/gXMjf9/Q4fxmEFjNuAMc/cwG8O113/eAb10HNTA"
    "BrkNIICweuARpOQHT6f8heT088Bi8OKtk38MjZSBVR4HMKQEAxnc5I8zcAEbeo0zLjOIid1l"
    "OAp09OcPP4Ht+T8MOjK/GNKNvzPYqXxhMJT+zCAv8BHYNACG7K+XwMgH4idAG4BdSWC1C87s"
    "4DEWNqBzgBUbwy9uhj/AQuErsFL8/OIvwwdgkfv69neGK2d//dn4n2HBb6ROPkAAMcI64ujA"
    "GYgdGSDD8gyQcdtKLSvutvgFIkAfGTMwSDSACimGzsWvGJpn3WXg4f3CYAPsM9irvWGwUHjN"
    "oCb6HlhHfIGM9v75CWmEgQpLZqBDGZnB/VeQoX8+/WP4/AboSGBD8D2w9Hr36A/D26dANrDY"
    "fffh3++3P/6/ewtsrgHj6f5FBoZ9wKbTHmhW/Q1yO0AA4fSANBCnMEAGif9DaHU2dsbL+duV"
    "WAXkgbzfgsCYUAdKfGG4ffMeAzvQM7IiwEYB53/IGCIDFANrZgZghfUb6NAvwP7yB2Ar9R2w"
    "xfrmPhADu6ZvXv5leP327683v8EOfQnEj4G58wEwrh4Cc8xjYBAAvQNOEB+hJf0XWE0FcjtA"
    "AOH0AMj6dCCWgGZsBkhsHPavELOxqQWKvgWFKjAIQeOK7CwQhwJbpn+A5f9nYNPi4+NfwEYs"
    "sHlw9xfDa5CDQc3rN39/vf39//0biEMfAfHDD0DHAl32COiyZ39RHfoTavV/fLOUAAHEgksS"
    "FPIngDgY6pmfkFmd3mPT3trwAJsLCnbcDP9/MTN8ANayb+98AfcJ3gAd/BpUib358/vNL7BD"
    "n0ND9CEoVD8hHAqUYvgADc1fhByKDwAEEM4YgAFlIAa2ahkCoPydDAwJWgwM1bLsjLKgUXOg"
    "Qz9AHfoE6MhH7xEOfQ4N0Q/QaY+flDgUVwwABBBBD8BADZRugTT4pIDlkBEwltiBDgVmN3CZ"
    "/IHYqKemBwACiHGoL/YACKAhv1YCIICGvAcAAgwAKWyhZrLOSpsAAAAASUVORK5CYII=")
index.append('taskcoach48x48')
catalog['taskcoach48x48'] = taskcoach48x48

#----------------------------------------------------------------------
led_yellow_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANhSURBVHjaYtxQw8DAzMzAwMXB"
    "wMDJycD+5SuD3aePDKm/GYTtmFhF+RiZGBm/fX79+ev7Nyf+/mWYzcDIsAeo/jsrOwPD//8M"
    "DAABxMIABUCOzP9/DF2ichqRaqpRDIIKngwcPDJAGUaG71+ecby4tcf3xpHFvg+vXF73j5Gh"
    "lJGR4R7IAIAAYoFqlmNkYlgpa+ZlIWVcycDMYQIU5YCZzcDGLc7AL67NoGRix3BlT1fQiQ3r"
    "NH5+Z/BjYWO4CxBAzOG2DByM/xkmqFnpeinYZzEwsYoAtfwA4n8go4H4FxB/BOKXDMysPxkk"
    "1WUY2Jkfiz64+ESJkZlhC0AAsQD9ZSMqzhAjpw5y7VsgAfQcwx+o5q9gL0AMfA82hIHhLYOO"
    "6T+Gx6cZfO4/ZPAACCAWJibmDFlVVgaWf7eB6s4AXW6NpJkDasBPIP4Awb/OMzD9vsKgaczO"
    "8Ojp3yyAAGLh4WO3EZFnZWBgAjrz4wZglAA1shoBFYtCXcMA9QbQdb/PA9XsApr5jUFChZeB"
    "T+CfIUAAsXDzMfOx8f6DWMb8Ceh6oCFMp4FsZaCACNQAoOZ/94FyjxkYWIDeYwIp/8sA1MsO"
    "EEAsbOxMDIzsQAM4gepAccL2F6jgGdTPSC5g+gbUBdEMDiIQACYSgABiYfrP8OUPAzsnM8dn"
    "oK1AWUZ+oIwgEHMjReVPSJgwAg1lA3qV5S/Dz2/sDN++/PsFEEBM3z59P/HhNdBqFmBgMYI0"
    "CUP9LwnEUlAMYotBxBl5gRYxMrx6xM7w6c3PywABxPTl0+9Zd0//Yfj7G2YrzAUgDRJImoWA"
    "GKiZgYvh318BhmtH/jH8/vlzBkAAMTGz/N9/++zHtdf2CUIVsYMVMTAIQDVLQg3kgnpJkOHG"
    "YWGGu+c+7mNh/78ZIICYvnxj+Prn7++yI6teXLuwgw3oEgGoS0Chygd1ESdY7N9ffoar+9kZ"
    "jix/eZ+Z6WcOMyvDB4AAYnbUAAYsM8P73z9/73h46Z3Sm4d/1ZlZeRk4uPkZmFlYGP7/+8Xw"
    "4/NXhifXnzOcWH2L4fy227v+/v4Wy87JcAOUmQACiAWUlSExAsxd/35F3r9w2/Px1cdZfKJC"
    "Rtz8wEwODNtvH7///PTm7ZXfP35MB+rZyMLC+Pk/SDcQAAQYAHVMFBibV7XKAAAAAElFTkSu"
    "QmCC")
index.append('led_yellow_icon16x16')
catalog['led_yellow_icon16x16'] = led_yellow_icon16x16

#----------------------------------------------------------------------
trashcan_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMDSURBVHjaYvz//z8DJQAggFhg"
    "DEaOtwwMP/8rc/MyxEiqMIXxiDPIsbD/4/j37y/Dj69/fnx5+/vJm4cf13378HExA+OvG///"
    "uYD1AQQQI8wFzGxvA/Vt2abq+rJI/uD8zvD29SeGL2+/MPz8+I3h/7ffDGzMLAzMrEIMz2/8"
    "e/f4zNOCv3+cFoP0AQQQ3AAt289PbfOYpc7cfsfw4uYHht9vPjP8+vCF4ff7Twy/3n1m+PPu"
    "IwMX118GFVdLhtd3Od89O2cgDNIHEEBMMC9o6zEKfvn5i+HOhQ9AxV8YGL5+Z2D89p2B4dsP"
    "hv9ffzAw/P7J8O3je4YPrz4ysPCxc8H0AQQQ3AARtj9/wq34GZxMRRjYWJgYfnz+AXTBV4Y/"
    "QMzy/xeDgIwwg7qPJwOPmTPDq5e//8H0AQQQPBB//f3DpCjGyFAbJ8HwzFuE4fT1LwwXb3xj"
    "+PL5L8N/djaG/9wCDB9/sDM8fAgM6+8/4bEAEEBwA/4z/GX4/ZuBQZCdgUFWhYXhzS8Bhud/"
    "BRi4fzEwfPnGwPDuHZAGYgag3UzMcAcwAAQQ3AuM//8xMDEygPEjYIxefcTA8Beo+T/Qsr9A"
    "/P8vUA6omhGs+i/cAIAAQhgAFGRmhCh6B7Tx5x+gsr//Gf78/g9yHlgcrJsR1QCAAIJ7gYHx"
    "z39mZmB6AOITp38xrNv4h4EFKMvKxsjAwsYETCdAbf9YIXb+RxgAEEBwA9jYGJju3PnPIMzy"
    "l+HFy38MT5//ZZCUYAYmHpBLgF7/8R9oOTBBMbIzMDL/Z4TpAwgguBcuXfj458HDfwzsnCwM"
    "PDyMDBzcTAxcQMzNy8LAK8jMICDCxsAvxs7AxgqMlW8f/sD0AQQQ3AVH972s/PX3d7dinTIX"
    "Fzcfg7AEKwOXIBMDDz8D0DCgq4Ex9Pz2K4aXB078/PfsVhVMH0AAwZMyI+MuIPlVT1iYIU1Q"
    "mjfgFyenCCMHMwsjMN5+f//x5+ubt+8+P32y6e/PLzOAIXPh//8KsD6AAGKkNDsDBBgAu+ch"
    "8Aa5E70AAAAASUVORK5CYII=")
index.append('trashcan_icon16x16')
catalog['trashcan_icon16x16'] = trashcan_icon16x16

#----------------------------------------------------------------------
clock_resume_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9sCDBAEL1eg8bkAAApW"
    "SURBVFjDpZdrcFXXdcd/+zzuuU/0ulcPBAiwBOIhhGyMMZZRgx0nxdgxBjsytR1jJyFmaKdu"
    "Wmfyoc20/dCZZtJOnczEybgenLouNhhoGDvBCGSpCClCGMRLEtK9AiGht+6VdN/nsftBFIem"
    "rZn0zOw5X9as9dt7/fdaewnu9tt23MCS1VhyE4qo0Ax1qaGJAiEh48iYmbau4dCLSgua+hlH"
    "Ho3fjVvxhRZbjvnJ2vWKV39l2QJv9QMrCzw1lfkUF3jwGAoCQdp0GI+m6eyJ0npp0uoejF8w"
    "49l30ZV/4eOvTvz+AHVHn/f69b9+9MGSpc9vWczSRfOQwHTUZHIsjmUJhBAIHPILveTlG6ga"
    "DN5M8P4n1/m4aWgoNpX5O9zKzzj+uHX3AHW/9JG1f3TfmtDuV19YxeoVufT3ZwlfGoR4HNVM"
    "49UEU5M3MLMZSkrLiWcklm4gvX4WVZZQWeklHJnlZ/96heYzIx+iKn9M85PDXwzw4KEgpvP+"
    "zu3LN7+4cxWTM9DR0EMgOcn6Kj/Ll+Xj8xvk5OZw4sQxYrEo27fXMzMzSzKRJhKO8ZvOGcbk"
    "PGoeqWRhicbho328ue/iBVuwg7ane387nHpH8PsO+Elbh3Y+U7n5689V0dE5S/hEF4HZ06yq"
    "TPHYVzeh6TqqpmG4PUQiEVLpLKurqrEdB7fHYMnSQmyzm6HLpxkf9HIt4eHhulKCAb2oo/1m"
    "rSx59iDDB1K/C7D2fZVk9o0d21fseLK+msamSeSVS7xaX8K6B8ppbPyUyspVBAI+VFVBCCgs"
    "DLJ48RIMw0BRBJqmk8lk+Oijj3j261t5+N4QnU09dPTr1D5aRp5bLTnXNrCKhfUfMnLQvhPA"
    "s7W+qrrob5/79oPKqbMJ/OGL7HmxjPxQHm63j+rqtfj9PjRNRVVVVFXg9/vx+/2AvA0lBKxe"
    "XUVeXgGa4aJmVYBwR5j2mz421C1iajC6bLBnIk7sSMvnAEv2+dwu5fCuP9mUF5n1Yp67zOsv"
    "LyA3mAuArisEAn40TUXXFXR9DkJRBEKAoghUVbn1V3G7PTiOREqJoumsXeHnbPMAEUI8tKGI"
    "8y2RDSnX1l8wfWRWAWA2U79hY9kiURSkq7mXUuU3TM1MAQKv14OqaoC8vXsh5gLfVrKYu46q"
    "qqJpKkIwZ6upqKqC1ATleRfoP9XJhOrjD75c4SeR2TN3AkU/d+ma8pOnvlO34Ey/xlrzKvdX"
    "Sdo7+hgZGaegIEAwmHvL8RfXLSEEiiIAgaoo2LbFvn37CIZcBD1BmoeCVFUHudDcXZrVtvyb"
    "inhs3aKl+d+b/8g6va9jgL2bYfNXNhIsyGFgYIQzZy4RjycIBnPweNx3VbU/h5DYtqSiYhk1"
    "9z2AlxQHT0bxVszHjtzIHemdbFcxv/xczSPLt4yFyghN3uDlLflI1UVJcSH33rsCj0fn/Ple"
    "zp3rBiShUC6apt0lhIKU4HZ7SKdN8ucpfHZhih5KWBwwRfd/9A0rSHtZfvl8Bm/aLPKl8fgN"
    "kGBZJrbtsH59Na+88hSVlWU0NZ3lvfd+RXd3+M5ojSfhQicAjmMTjU5z8eJVGhpacBwbKR1A"
    "orsNyvNMhm4kMYpCuHRxj+Y2xBLbE2AkHGfxckFgXg42Oo5joSgCKSWBgJ8tW+pYt241DQ2t"
    "HDrUSEVFLyvXrkS7FiHwwQck9+yl79gphm9OEI3OApLcXC/r11u4XAY+nwtNsVle5me6L0Vi"
    "gRu/T52vGYZRkJYa6bQgFr1Bw4lhLEdj/fr7CYUK7hBeYWEBO3dupasrzInGdqJNp7l+9SpK"
    "UQWFrVcwNEFxcT41NRUsXFhCfn4uUgqmp2dpbW0FJ8PEuI1mlzJtqWgul19DuISDgirAzKaJ"
    "RbOYjo5t2/9rr1qx4h68Pg/7f/EBB7rKCOg23ygr5oUXHsTj1u+wlXKuHsRiMRw7TTqtoyrg"
    "SAWJCy1jqVHVcQh4JaWlFezYsQZL6khpYlk2uv4/XD/bpmT/u2x/eCNNwzP8w/dr+fn+S9T/"
    "eQPPPbaUZx6vmLv/Emzbxu328PTTz6AKi8P/3oMacfAqDqapJpW0qV5z4ikWlfqJJuRcV0sm"
    "sG0HKeXvbl9K7DffxKyooLyulkRWUjR/Ht//043s3f0AxzvH2bLnI46eCGOaFlKC4zgkk0ni"
    "8TjXh+KEgi5cpkkyI0YUpN43FZmguFhnNO0mlUwDEiEEjiNxnDshnAMHSKkqnm3bAMjYcPEm"
    "NPRAypvPS9+q5fldtfzz8SF2/Nkn/PJEmNmEiZSSbDrLRMLF/IV+kiMxslklomAYpwa7prJF"
    "PpjERyQcwzAMMpkslmXjOA7/dRCyqYnkjRu4d+9GuQVk2jAwA+EonL0ODV0woeXx5POb2LB1"
    "A281jvONv/qU46eHuBaJEZl1Mz+kMHB+GFSjRcPj/mx0ON1tTyXX5C8poaPzPEnzJOfOdbJr"
    "1zcxXDlYAvTublJtbRh796L9liYyFlyfhkgUHHuuG/bHQBUQ8Oez7olaYqNR/vHYBTKDNylZ"
    "vQYjmmKwPz6Ix2jWGH8i7uR+/M7FxoEfPfVqJYMj85j6tIWdLz6J2/CQRmCMjJI8cgTlpZfQ"
    "fb47UpK2ITIF4SnAmQOYK8IgYyAd8HjzWPqVOuIjE/S0X6Gj7SrpTLKTBdOjczXV436nq33i"
    "e2u/lCxcv7GS1JkMHncutiIQM3HM/ftxtm3HW1yMbTu3m40QYEuIpmA2A+IWwH9/9MVjMBoD"
    "jydI2SObCE3G0BZcfmzs8pVjc6kc3jxppeUPGj64RgKN4JpyPjwUJjUVRT+4n/jadThLlpCO"
    "p8hmLbJZG8uysG0HR87p4P96YgsxlxLThlgWUkJHNXQdIdzKbSu/5+3h3sSRg++OoBXnULy6"
    "gsNvn2V0/iLMqlW89cYbxGLTSAmmaWFZNjMzCQQgFNBV0HXQtVtL/Xx5POAPgBOPMdjcxsV3"
    "D7QPNZ36Iyeb/dLnbS18f5byc9/p+GSs4se6vuovXyumqOAhDrUPYr11mFBoHnl5uaRTGSTg"
    "9/tobf0UBYcc71wqcLgtAKGA7gZFhemxWSKtFxnsGMias4m/QI2/Q893pwHu7Kt9NaMsv7Tt"
    "zK/H3/uBJda9/moBX9vh5+RREzNr0dLST8WyXDxuF7btxrLSKKpCnh9UbU5wAC4DNA3Gh6fp"
    "aetmpHuUibBnDLFsG1fvO/3Fg8nK3iBZ8Xb+Ev8T33wlyMt/qOJLpmhr7ic6nSSTMQmFfAzd"
    "7OOnZ3Sefe0JTBs0FTQDxofinD15iei1ccbGipgZ9nWgGfV0VYTvfjSrvmaQFa9h6K+vrPHl"
    "7XzKz9O1CiU+ydTQNKlkBkUobP/7dvb8zeMIAYP9M7Sf7OHG5WGy9kIGBoJJmeKfcMkf0lkW"
    "/f2G07XDC8mK3RjqMzlF+rKq1W5WlLsoKxbkB+Ct/b9m13frOH7kMmeaRpiZLiKRXnidtDiK"
    "Ln9CZ0nP/286vj01TRRh8xA2tQhxD4ooRSGQm3N8qUt3WWODBb0oi9tQvY1otHA2OHA3bv8T"
    "SUx7OtoeqPoAAAAASUVORK5CYII=")
index.append('clock_resume_icon32x32')
catalog['clock_resume_icon32x32'] = clock_resume_icon32x32

#----------------------------------------------------------------------
down16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAK6SURBVHjaYvz//z8DJQAggJgY"
    "KAQAAcQCItjNZh/nYmAw+AN0zLe/zO/+cXGHMLCxH2f494+BgV+agYERqOjvPw2G/0zbWdg5"
    "JP6zsDL8Y2a79W+Zij5AAIENUJLitegqtWL4wczEsP34S6n5i25OYGBldGFgYfsM1v33Hzfz"
    "33/T1XSkFP7zCjN8+MLI8Pr1Bz2QXoAAAnvh1Yuv97mZmBhCLWQY0lIMGQxMZMwYPv+vZOAU"
    "BOoHGvD1T46RqoCDiZEsAwMXD8MvBmaGv38ZX4L0AgQQ2IB3b38UpTUfZ7j1/CsDFyvQoGgD"
    "Bn4x3lyGn3+MGP78UxEW4ShVN1JguPScieHVmz8M71++Z2D4/78EpBcggCCBKMSz4e69z9Nq"
    "p15h4Pj5n0FSio/B3EmNh+E/43Rmhv9ztAykhe9/52Z4+paB4dO7Twz///1bxsDEvASkFSCA"
    "IAawMDMwCHI2rt797Mr6vU8ZpIEhqqguwyAuL2rGJyJg/19QguHBawaGzx+/Mfz5/uMeAzNL"
    "PdAJYK0AAQQx4N9foF//v/rPzlLdvfDun1cvvjHICjEyKOopM0hrKDI8fc/I8O7DH4afn4Fh"
    "yszcCNRxBxI1DAwAAQQxgI0biIHWsrFtevvx7/Spqx4xSPH9Y+DhYmP48ped4e2n/wzfP34G"
    "qVzMwMi0CIgZgF4AawUIIIgB/DIMDHziwBDmZWDgZS8/cfHjpYOn3jHoyzMwvP4EjIRP34FR"
    "+fsuUFMeekICCCCIAf+BXvj/DxSyoGj7zsDGkr1699sPf379ZNCW+svw99PXHwxMTCDNH9AN"
    "AAggqAH/ERiU+liYjnz78X/m5oMfGb59+gJSMQ9o8nZsSRkggBhBmYkx+i66MMglnMBEvo+R"
    "lZ3zPxOjPVDoI7rm/3OEGAACiJHS3AgQQBTnRoAAAwBe7eXig4nCewAAAABJRU5ErkJggg==")
index.append('down16x16')
catalog['down16x16'] = down16x16

#----------------------------------------------------------------------
pencil_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAakSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAD7gCAAGKBMRgZGSkzSVacgUFKgoHhHxMEg8BfxgCGX4wLGZgY3jKw/q9iYPq/Aqb8"
    "/5kzYBoggKgTAtLCDAxyEqhiv5ii2Ln+LpIIfsPHavNOkYH532KGP4y16FoBAoiFYsvVgBZL"
    "iTIw/PyH8M9PpmIuod89AfkPGF4pMDGcfcnE8EPlI8v3ddJNDN9YfzAw/++GaQcIIMpCwECa"
    "gUFGiIHhz1+E2A+mLgHZbz0h1XcZZHW+Msj//8fAx8zBwKn5h4HD7xnQjf/bGf4z2sCUAwQQ"
    "eQ5gAqYXYykGBhEeoOX/YKJsDD8YF0pofi4NrbrLICb3g+HbV1YGbqZvDPYiXAzczIIM7No/"
    "GVgNPjAz/GZshWkCCCDSo4CDmYHBVJKBgZOTgeE3yHJmkKgA0PLlimbvPbzSnjJwcAOTwA8W"
    "ht//fzKIs6szBImlMbiI/meY+HQawx217wwfzzBawIwDCCDSHCDABrQcmNqZWaCWAwPwP4Mc"
    "w0/GNZpOr03d414wMDEzMfz+CfTkv58MMpwGDNYiqQySHNoMP1n3Mhj8YGC4dQyYYN+9hucG"
    "gAAi3gFCQMstxYBZDBj8f6Gl538GPYa/DOt0vF4qO4e8AUYv0PLfjAx///1iUOC2AlqezCDC"
    "psRw+ecqhtUnNjBsmSjC8PX824kMb5+UwowFCCDiHCDDAYxzYGL7DbYUAv4xODD8/79KL+CF"
    "qIXbBwamf6BA+c/w7/8fBjVeZwZL4UQGPlZxhjPf5zOs3LuLYcdUcYbfV17WMHx80YpsNEAA"
    "EXaAJhcDgxYfMFKR6oy/jKFMrH8XaPu+5tI2/8rAAswEf5kgbtPh92EwE4xnYGfhZjj6ZSrD"
    "si3HGfbPFvv5/8bjHIbP7+agGw8QQPgdYApMTUpAB/wAGQ0tKf8y5DBz/Z6k5vOGUUH7BwMr"
    "0HImFlBJysJgyB/EYCIYBcpqDAc+9DIsXn2Z4eRikfcMtx8kMHz9vAmbFQABhNsBFkCLFYFB"
    "/xPZ5wzNLAK/auQ93jKIK/wG+5wVmDTYmNkZjAQiGQwEQhl+MXxi2Pt6AsOiJfcZLq8WeMRw"
    "514kw48fx3BZAxBA2B1gyM7AoMwGtZwRou4PwzRm0R+p4k7vGQQk/jKwAuOcFRjsfKw8DJZC"
    "cQzafP4MXxieMOx42s+wYP5rhntbuC8y3L0fyfDr93V8gQwQQJgO4ALmaz2gA77Dfc4L9Pli"
    "Rqnv/rzWHxm4Bf8zsAGlWICWi3EKMTiIJTGo87ozvP13nWHzPaDP535leLaLdR/D/YcxwELq"
    "OaEkBhBAqA4QBFpuwwlNcGCfSwItX80g/82a1fAzAycwVthBRQ/QchluKQYPqRRgirdnePb3"
    "JMPaa1MYls7+y/DuAMMqhgdPU4A55DMxGQwggBAOkGYFJjqg5QLAvPYTLKwBNGQdg+o3TQaN"
    "bwzsQGkORojlKnxKDP6yaUDLTRke/N7DsPzcLIZVc1kYvh79M5nh0dsioN4/xBYvAAGEcIAd"
    "MMGB8xNU/N/fFQwq3zUZ1H4BKy+g5UCLmYBYV0ibIVQhnUGZV5vh5s91DIuOLWLYuICT4deJ"
    "7zUMzz61klqyAwQQwgFMoIj9BXQ7UOj/X18ufjb9X4rfwBUdKzPE5xZipgwRimkMcjzyDJe+"
    "L2BYsG8Nw/alPD//n/ySw/D62xxy6jWAAELUhszwUAOWpd9Laqx8GFzE9MCByQZ0gLO0PUOM"
    "ci6DJLcMw4mP0xgmb17BsG0+1/v/Rz+Fk2s5CAAEECIE2H9CfP/vl5O0mIRVqqkHgxOjNcOx"
    "d8UM3vJ2DIlqqQzcbCwMu172MCzZfpDhyibeRwynP0YyfPlzjJImBUAAYSkH/pYW2ngzCAlJ"
    "ACsSHYbFVhMYNr45xPD+32uGHY8WMazcepLh8U5gHj8DtPzHv+uUNqgAAgjVAUx/TRn/sLqJ"
    "M6kyMPEDaz5mAQY/4QhgvudmKFxXz/Dw7gOG76c49jGc+RwLrOyfUaM5CRBAqA5g/lf4/7U0"
    "Y37/TgZhCTmGL5yfGZYe38hw9NK532/uvb/A8JBpE8ODrxOBNeFnBioBgABihPWMGDMYDRh+"
    "s51ieCbHysD1HpgmvvxgePXzPMN9ht0Mrxh2AcvZC0BlX6llMcxegABChAAzQwbD99/AlHjn"
    "IMNNoKWPGPYB/XkF1MykZccEIIDALgG7RoUhlIGHwQQoxkqPHhHMXoAAYhzozilAAA143xAg"
    "gAbcAQABBgBR5RNw1IyEcwAAAABJRU5ErkJggg==")
index.append('pencil_icon32x32')
catalog['pencil_icon32x32'] = pencil_icon32x32

#----------------------------------------------------------------------
exportasvcal16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMpSURBVHjaYqyqqjrt4eFhzMDA"
    "8B8IGBjADAYGRkYwk4GJiQkq9h/OZ2NjYzxw4MD3lpaWAoAAYtixY8eP/3jAzx8//n///v3/"
    "v3///v/+/fv/z58/weKXL1/+Ly8vfwgggFiAEv9gNoAs+fr1G8ObN+8Znj55ycDM9JeBn5eZ"
    "4eOnrwyKKmoMnJycDL9+/WLg5eVl+PPnD1gPQACxQPUzfPnyheH69btAja8YeLhYGCTEBRl4"
    "BMUZvrBLMPz8eYvh1YtnDMpAQ35BvQHSB3IJQACxMAI9C9J87tx5oCZRBgUbQ4bfrPwMT3+y"
    "MTz6zsDw+CkDgySTJAPr50sM3398B2v++/cvzAAGgABiAREgARbGvwzvORUYTj/jZPgA1Mj+"
    "4xODEe8HBj0ZYQZhIV6GcyeZGF68eMEgIyMLdjHMAIAAYgH5hZ+fn4GRiZnhy5uXDGoSCgwS"
    "L28yiD2+yMDx4BPD/7O/Gf5GRDEIS0kxvHj0iEFcXAIUC/BYAQggJpDtIMDDJ8DA+fU+g6Ho"
    "NwYRYVaGXxZmDN8EhBn+nzrD8OPDBwZhSUmG79++MXwAsmFhAAIAAcQEM0kSqOD12w8Mz5+/"
    "ZvguLMbwg5ef4Q9Q4R8BAQbWvj4GTqBmPhERhpdAbyB7ASCAwAaAODw8PAxc3NwM9x88YGAG"
    "SX7+zPDD2Jjhj7Q0A+u5cwyM378ziAMtefLkCTCqvzKAAh+kFyCAWCDxDzFEVlaW4cSJEwwq"
    "BgYMQkCD2GfMYPjEwcFwKz2d4d6VKwyPb90CpwOQWkZoUgUIIBaYF4CpDOwNFhYWhrMHDzLw"
    "AjXeBrrgJTCQ/718ySD8/j2DnJwcg4qKCgMrKyvYIBAACCAWmF9ABnADvaCpqcmwYsUKsHfk"
    "FRUZbICapIDeEACGBSjwfvz4AY52EAbpAwggFmZmZkaQrVxcXGABdXV1hvz8fHCyBRkIcx0I"
    "g6IcqB4sB8JAbzABBBDLuXPnvltYWHDAFID8BjIQFF1v376FhzYiv0BcC8xMDJ8/f/4FEECM"
    "QFsSpaSk0oDyrEBJSORCASyk0cWAmOX169df379/PwkgwABf6awALrY5vAAAAABJRU5ErkJg"
    "gg==")
index.append('exportasvcal16x16')
catalog['exportasvcal16x16'] = exportasvcal16x16

#----------------------------------------------------------------------
folder_red_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAGtSURBVHjaYmRgYGBazMf3lwEN"
    "xH76pA2kbgPxPyDGkIcBgABiAGr+/+/zZwwMEgfKOwExFxCzQDEzEDMi6wcIIJAgw59z5zAM"
    "jn76lIFBWnovIZcBBBDYgP/v34Ml///7x8DIxASmf+3eDTEEHUhLXwUa4ghkHQUZABBAYAP+"
    "vXnD8Pf2bYb106czEAOA3tsPMwQggCAGvH4N1ozVRlxAWhpkiAVAAEG8AHQBCPw+cgQi+RcY"
    "6H/+MPz/9Qso+Jvh/48fDP9//gTa9I/h55IlDOyhoTBjXAECCO4FsEEfPgAjlQlDM8hAOP/r"
    "V4b/Hz/CDBAGCCCIAW/fgnn/QIEJtAWMYa4A2QzUDKZBfKCB/6CBDopSgACCeOHdO4gLQCaD"
    "NP//D9EE1MAA1Ai2Hc0VMAAQQKjRCPQKKArBAKQZpBiqGWw71AXIBgAEEMSAz58ZkL0CdgFU"
    "A9ggkAFQg/8DDWX4/h2m/wdAAIGSpRYwXq8ykAiAUbgKSK0HCCCQAaxAbAjEzkAsRKR+kBMu"
    "APFegACCZQwQzQ41jBgAyp2/QCEFEECM/0H+pQAABBgAaE8F4JYoHyAAAAAASUVORK5CYII=")
index.append('folder_red_icon16x16')
catalog['folder_red_icon16x16'] = folder_red_icon16x16

#----------------------------------------------------------------------
person_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAP8SURBVHjaYvz//z/D8TxmBjBg"
    "ZGD4++UfGyuPRKeIiW8kr6K+MCvTT6Y3d89/27Ht0JlNJx4VvvjGcOHKKwYGqA4GgABiABlw"
    "LIcRjI9kMjBc7jI++vXxif9g8O/L//9fb/3///7Q//+PZv9f32D3WVuYwQ2kjwmKAQIIhBmY"
    "+YUYmHj5GbjEZRuVguqsuGR0GBh+f2Rg+P4OiIH0lw8MDP+4GAISw3nCvTQXALXwsQBdC8IA"
    "AQQ2gJGNneE/ExODoKpRNKeAMAPD2+tATQ+BmoFu/QXU/Ocrw59vnxj+/2JkcHfUllAWZojV"
    "kGZlAGGAAGIBGfDs83+GXz//Mwj8Y2Rn/PkCGBBAcxlZgbb+AbK/Mvz7/ZXh7/8/DD+/fmb4"
    "9esrIysbswwLKyQUAAIIbMDTZ88Zfv9mYPh24thhBX3TSA5uSaCmPwz/Gf4x/P/3l+HP3z8M"
    "3358YfgFdNnhk5f+3H7x9wIn21+wAQABBDbgIGMJAy8/I8O1+79KWfbd8w+2+cbFwMrL8AcY"
    "1r9//2L4+eUtw9cX1xiuPPjKMHXrl2N//zOs/PKLHehKZgaAAGIExQJ/LAPQAAZxZ0OG1fJ/"
    "b9gEKO9n/PPyJAPbnw/AmP3B8Or1F4YPAvoM577HMTx98+PD1lWzE989uriB4ftdBoAAAkej"
    "cORXrtQZv65cuv3p/5U7v/5fevP//6r9d/8fcDT8f1yO9f/8mev/z7j6//+Ba///33j0/3/v"
    "ylu/mAVtQkGuBwggsAH6ed87Nl34+f/cvc//P/789//9j///33wHGhSf/P+wuu7/O1du/X/3"
    "+///P3/+/7/z4sf/M8/+//dMmwVKTuIAAQSORjXZ/7H/2RgYPv7nYLj/gZHhLTDw3334AI5s"
    "eUkhhi8vXjG8B8b5keevGe58YmR4/Y2BwdTZW5SVRzkVIIDAgcjOzSj2+icLw4//TAyfgZo5"
    "gJjp+RcGxXfvGNg+fWR4cesJwytjBoYrrz8wfHrPw6AoBgw/HgEGdh4RY4AAAhvw+jMrw7s/"
    "TAyffjIwcAJFmIG28d+8y6B59wYD693bDB+uXWf48R5o0T9Vhu+/gK4DRvmLL4zAGPrPBhBA"
    "YAOuHD1yT9HYXk2QC5g8gc5j+g9MiKzSDM+FVBief2FiuC2jz/D/LQPDf6AFHPxAzUAXnjl6"
    "m+Hnh/v3AAIIHI2MvO5uss4la3S8nHiFJZkZuDiABnEDDQKmrj+//jKw8HMw/PsKTNk/gOEE"
    "zB63jlxkuL6i9NbfD7t9AQIIYgCPPQPD18carMIa9TxyFhZcosK8bFwczIzsLExMzEwM/3/8"
    "ZvgNjJVv7z78+vL0yvsfz/btZfj3vB3o+McAAQYAN1i/XKmb8j4AAAAASUVORK5CYII=")
index.append('person_icon16x16')
catalog['person_icon16x16'] = person_icon16x16

#----------------------------------------------------------------------
led_blue_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANPSURBVHjaYvz//z8DJQAggFhA"
    "BKPOEgYGQR4GBmEgfvuVgeHF1whxXtY4PRluI0E+Nt43b75/vHTx5Zk3L97OB6pez8DADNT1"
    "DYjrGAACiAVuFCMjA8P33/KiMsLTizKNPcOdxBkUJFgZgKIM/xgYuO49/uG7dNVd34ldx9e/"
    "f/U6C2jIC5A2gABiAHtBezEDg9lqOeWYE3c3nfv9/933//+//Pz//9ff/////oPQX379///2"
    "9///qw5/+y+rvuQKA0OOOEg/QAAxQe1nZvjJNTvIyUhJXJiF4elbBoaPQBd+/QHEPxkYvgDp"
    "D18YGJ4B7VRQ5GTwzQrVZmBhngFyN0AAQcLgP0Pwf24Vt+vPWBjefGJg4GVjYAAF7fffDAws"
    "QCv+AP3wDWjQe6Chn4H0zfdABcpxAYw3N7sDBBDYAHYe/pQ/ktIMux/+Z9C6xchgqcDA8PMP"
    "AwMHK9BpQAP+Ag34/gvoEqDmg3eA+PF/BlYNTQamp86JAAEEMUBUxOiPGDcDCycDw5bLQIVA"
    "m9SAPhTgAroAGOC//wJtB0bODaAXDt4FqudnZPjFxsnAJqpvBBBAYAOYBbl5+PgYGGS4GRhE"
    "gIbceQ30L9ArPBwMDKxQA0DhAPKGFDCm2YBiT9mBscMrwAcQQGADWNj/fxLnYRAVBWrgBmJ2"
    "oBfZgM5nAjkfGI9MQA3cQINZgap//oLEOCgpPPv5/hNAAIEN+PHt02lRjr9eHOzMDOxAk7mA"
    "mIMNogFkyD9gGPwChgkzMyTKgA5iEPnxl+Hui3NnAQIIHI2fbzyd+evNUwYuoBfYWSGauYAu"
    "4eGEYJDtMENB8ly8wEC9eY3h28fdCwECCGzA/5dfN1/bcnQj269fDOxAxWwsEG+ADOHlgngL"
    "pBksDrSE+eNfhmvbJm1gYHiyAyCAYAnp/7urj9KPTl15heX7XwZOYICysELCgZMDQrOADASK"
    "M378w3BkSvfVT+8WZoL0AQQQEyJfMb18cGWm67qqwrV3d11j+Pv5D9j/oMwACrQ/H38z3Nx6"
    "kWFddda6J7frXIFxA84LAAHECMoLjNJzgUH6DshdCcRngVjZU1jUOUZKSc+Yi5ef7+un95+e"
    "3r1w9v3bPUsZGB5tQ87OAAEGAOfDHTQvRWbvAAAAAElFTkSuQmCC")
index.append('led_blue_icon16x16')
catalog['led_blue_icon16x16'] = led_blue_icon16x16

#----------------------------------------------------------------------
timelineviewer16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9kDBQ8sOeTzyT4AAAG1"
    "SURBVDjLnZO/alRREMZ/M+ckm002ESGllShCCsHCziLpLG2s7XwAOx9JEDRYCcK2voGCikjA"
    "aDAxJvsne++cGYt7d7PRSNQPPoaZYc43M+ccuX7j8Zubt69em9QRVqAEFG+sFbDWN2/jba6I"
    "yt7bDzv5zt1bVzYf3c+fPjcF1ZR2yknL2prcSQ2LKyBPttfzwZF7/9U+R4cnJAV3iICgZTT0"
    "NpYDVgmGqctwWCL/OC482BA2N9aJAJFABAghaCDMI7i02uHp62MevnCyibLW63J5rUNTeT7C"
    "/Yy/vLJESEINxYrzr7C6YCjZUGqzViZm5tdmpm5EICJYXRFJyC4J0XTa2r3+herDZ5uIJiIU"
    "NUm/relvECGgiWyhTKp6lhg936KuCwsL6Q+FzZij0ZhAm1tQTbMEQM4KQK+/fO4hg60RIokg"
    "oQUlpbNqIhePlJKACLmgVOMxInNq1RhUGGwNIBzMcCuYGVYH/n0XrUAkkbu9rvY/Ku++7VMV"
    "KCUoEbhDKVA85j5T4B5kcd4fdqkXO5q/7Ozubb+MpaORBSGU6buPpsgD3GXucwAOZBVGXw9k"
    "fnn/g59CuwCnhtEDcQAAAABJRU5ErkJggg==")
index.append('timelineviewer16x16')
catalog['timelineviewer16x16'] = timelineviewer16x16

#----------------------------------------------------------------------
person_id_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAL8SURBVHjaYvz//z8DJQAggJgY"
    "KAQAAcQSFFh7xchYXfkv0Cn/GFgY/jMwMvxjBJnLDGT/Y2BmYQGSTAwgh/4DyQExSM3ff4yM"
    "xy88fQgQQCwmphpyVVXRHBDz/jAwfH7CwPD7CwMDnwzDf2YBhtev3zEwMTECMRMDEyMjAyMQ"
    "s7IyM3BxczHU9u2VBAgglp+//vwB6/33l+HHqysMzKxsDMxfHzL8fXGa4QWnA8Pew7cZWJn/"
    "M7CzswE1sjKwsDAyCArwMVhZaTP8/PPvL0AAsfz885cBFIz/frxnYPh6n4FVWouBgYWH4dvL"
    "U0CNDxj8/G0YGP7+AdrMBMQMYMwC9BaI8evXXwaAAGL5++c/AwgzsXIwMP15y/D42CqGr1+/"
    "MnCzvGH4yvCN4fbdq8DQ+M3AxsYK1sjExMzAz8/FoK0lz/Dn738GgABi+fXnH9D1vxkY2HkY"
    "Ln7UZHhxdg/D2xePGTjUPRgM1S0YlLk/M/z9+xeomRnsClB4cHEBgwwY5L+BrgcIIKY/f/+B"
    "Fdx8/I/hv7Q1g6CIDwPnZz0GGddqhjc/uBn+/PsP1MwEtpmZGYSZGJiBhvwFBt3v338ZAAKI"
    "5R/QGZ++/GG48pKNQUGNgUFBkI9BQYCf4bXQP4ZTl/4x/Ln3gkFVHBhOjOxAb7CAY0NAgJuB"
    "h4sV7AWAAGL5++8fw49vfxiuvmZmeMLPwGBx9xUDz6WrDAfv/WG48Z6dwV1OnsFMl43h2y9G"
    "aCAygsPjz78/DL9+/2MACCCWbz//MfDz/mdQAuKt5xkYnnPbMjBbKzG8vM/BwPr+FcP799cZ"
    "jn5lByYoVmA0soCxAD8vg4wUP9Dr/xkAAgjoAgaWD+8/MTjL/mLg/vKL4QU3H8M/JSEG9R/3"
    "GDR1/jCI8MoxAL0KDjxGUHoBuoCdnYXhy8cPDP/+/GEGCCBGE9fm6yrywgrMjIz/geHD8Pc/"
    "Izjpgpz7jxEUgaxgMRD4D03SDCCj/v9nvPzw5yOAAGKkNDcCBBDFuREgwABaMwTCL7KYQgAA"
    "AABJRU5ErkJggg==")
index.append('person_id_icon16x16')
catalog['person_id_icon16x16'] = person_id_icon16x16

#----------------------------------------------------------------------
calendar_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAY9SURBVHjaYvz//z9DeXk5w5/f"
    "Pxm+fPnHwMjEx/AbyGZkZGT49esHAxsbO8Pfv//VODg41H/9enlbQIBPXkhQ0IeXj891xYoV"
    "U44fPz6FAQsACCAWdAGQRUDMxsDAqMzMzGwqJCQUxcT0T0NMTEyejU32lZ6+poCGhiYbSO35"
    "CxdSgQZvBzLvopsDEEAoBgON5GFjYaniYGcxFpMQsWJi+s8pJS3OLCIqzqCiIs/AwcolJi8v"
    "Drb89dt3DMYmpprr1661/PjpE4bBAAHEAnUl++9fvw3Y2FgylFSkEoTFJBgUlWQZeLl5GZSV"
    "ZBi+fvnOwM3JyvDj50+Gx08eMwgLCTNwcXExmFtbs2poaHicPHVqFdCYX8gGAwQQ2GB2Do5V"
    "hmYWXhraOizycjIMPz5/ZxAWFmR4/+4tw/cvnxnY2VgYvnz7ygAMDoZ/f/8ysLKyMjAC9cnJ"
    "KjAYmZk7Ag3WAnIvIBsMEEBMIIKLl9/MxcOLRVtJjoGTCRhpP78yfP36iYGbm4uBi4eHQVBQ"
    "iEFSQoKBgYmJgRlo6N//IF8yAF3Ox2BiaSMlISbmgB4UAAEENvjPj2+v3n78wvAaiP/++88g"
    "LSPNwMfPz8DDw83AzcrC8O/efQaGM2cYmE+cZGC4fYuB4e9vhr9Q7+qamjNo6uj4AJlCyAYD"
    "BBDY4Pt37x578fINA7+YCNiVDEzMDP+Bye3vi5cMTIcOMzBfv8YATIsMjM+fMTCfPMXAeO0G"
    "UB5igLycLIOuoYkeJzubEbLBAAEENvjM2dN779+6xfDmK8SLYAwUZ/z8meEfBwfDb1tbhl8O"
    "Dgx/DA0ZGIARyPTyBThl/AEqEmBnYjCycxGVkJDwQDYYIIDABv/79+/s9Uvnbz17+4PhJ8hA"
    "kMFAv/5RUWH4bW3N8I+PD+h9oMCHjwzA3MPwV0qS4R/QR8BQY2AFqlfX0WXQ0DEAhbMUzGCA"
    "AAIb/OXzl/uXz504/ODeQ4YPf4AhwQh19V8o/ecvA8vevQws+/aCXcx89hwD87kLDP9AKR+U"
    "OqQkGAxsHA24OdmtYAYDBBDYYDZ2NoZPH19vv3Hh7K9HQEf9hWcYqMHAMP+rpMzwy9ub4beV"
    "NQPD69cMrJs3MjABI/I3UI0ABwODlqkFs7SMvCfIOJBegAACGwzMugwvX7w6dP/iiZt3n35j"
    "+PoPKAYz9B+E/qOizPBHXZ3hl6kJw18tbQaGd+8YGB8+BjuCHRQc2voMGoYmziAmyEyAAIIE"
    "BTDGv33//vrZvYvbb125yfAQGImMTFBng4MF6Olfv4GW/GdgevKUgfnyZaC72IC+UAIGByT4"
    "5cU5GbQt7OVFBPlcQdoAAgic89iAif4f0NWPHt3Z+PjyyaLrJoYsGvygWAUioArWQ0cZ2I4e"
    "YfgPNIzp6TOG/1wcDD+joxn+qSgxsAENZQZaLgLEevauDCrrDILeHDm0HCCAwAb/ArkGVKox"
    "/j3/4vqRoxevB9iby0swyAKj/AfISUDn/+XmYWAE5sI/lpYMDDraDMx8/Ay/gYa+ApYQjz//"
    "Z3j0hZHh2R8xBhlFdV2Ok0dNAQIIUroB0xcw9TD8/PPv+5M7Z9bxX7pgf87IgwGYARmYgJr/"
    "2lozMAIxCHwD4pfAoLr3BFhWvv3P8OQdI8OrN4wMnz4A1X76xvDz2292JiZGQYAAAhv87v1H"
    "sKa/wMBi/v9n15vLu++dvOWiZC/FwiAMVPEKmLiffmFguPUeZBgDw6PXDAwvgPgLkP//0zsG"
    "1neXGJjeHWf4/vzkl2cPrlz48fvfI4AAAhssAizJYODPn983Ptw7su/WuZtKW1W0GRiBXr30"
    "FJjtXwJT2RsGhq9v/zAwfXzKwPHhLAPT25N/f7w8/frd86tX373/cO7D51/HgAnoEtCYpwAB"
    "xAgKWx1NZeQqhIGF8a+HgEneht+2xewfgF78CXQy+8ebDJwfTzMwvz/558fLMw8+vLp3+d3H"
    "j6e+/mA4BTQMWJgwvGZAZAEGgAACGyzAz4NS5DH+/82ub+69/7l4niXTx1sMnG+3f/r17sqj"
    "92+fXnzz/vuRX38ZzgKV3QHij5C0gwkAAghsMB8vL2q9B8T8/LzuwEoj5dOHN8/ff/59DJgC"
    "LgKFH4OSPQMRACDAABLoZ3R+p3OCAAAAAElFTkSuQmCC")
index.append('calendar_icon22x22')
catalog['calendar_icon22x22'] = calendar_icon22x22

#----------------------------------------------------------------------
paperclip_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOUSURBVHjaYvz//z8DJQAggFgY"
    "GVuA1HcGNj4hBhYubgaGP38Y/vz6x/D75x9WMRnBRVyCnPz3z1ybYOhl3v/y8Ztbz27cT2bk"
    "ZXnHyPqXgYmPjQEggJgZGJyABvxhYGbnZGBiZWNg+Pef4cfHH9wSisLz0iaGRH76zawqpSYX"
    "65brLKpioqnx7PYz44/PXh5g5GT+xMjOzAAQQBgG/Pr6i1tcQWhh5tTQsGOHnzEw/vrNYORv"
    "wDCvbPWF129+sLumuGq/fPDc6uOzFzuZeNk/AQQQ3AAWDk6GP38ZuWTUxNYk9QT4Hd7zkIHx"
    "528GdSsVhqV1a/a9vXTR+9Wtezvf/WGLsIv0UXr//JXDh7evtgMEENSA3wx/fzMLKxoqrE/s"
    "9HPbte4GAycrI4OqpTLDkpoVWz5cvx7GwMbynV9FvuLf39/at649/Wsd7C338d1HN4AAYmJg"
    "AMXCfymgTVsS2n2cdqy6xiAowMGgbKrEsKRy2apPt65FADX/F1KWWhSYH5khoazIw/D6/n8G"
    "RiYGJgFpFYAAYgFp1nO32hZR66G/aeEFBgkZfgZRZXGGhSWL5n59fDudgYObQ1hdYZlvTrTP"
    "qSNX/39+9IjBLTWJ58jeY++fXNocAxAAQQC+/wQlOy6vVXpHL6+dzQAgJxYADBYOAKNznpb4"
    "6eNsAAoMERcrJK1Ja1dAxrfM//fs7v86VFIDvrS45+nY0UwABf3OAgBBAL7/BCI5IynX1e0A"
    "FhwTAAgOCAClc5+F+vHofAAJCx4aMSqpRGRUOL+uwQAI//0BN09JAbqrrtbv3tRcAAoQzwD+"
    "AQACiOXXr7//mdm5GISUeBi2d67Z/uvLpxoGLj5FKQO19c6pwfr71x5m4ONkBcaGDcPO5ctO"
    "fn54I4qBS/AeAzQFAwQQCyMLE/M/IOfn1z8MTOzMIkz8PKVyproJ1iGOWofWHmEQE+NnkFRX"
    "Y9i+cP6W789vJwKd+YYBKfkDBBCjc9nlb2wCgpxM//8BvczB8P3bb6BBnAyndp5hUNGUZuAW"
    "l2DYt3j+0t8fH2YxsPB8YmACplYmDgYGZiD9/wkDQAAxsXFzsErLcDO8f/WZ4cuX3wy//jIz"
    "PL/3gkHLRIOBmVeAYc+CWRN+f7yXxsDK8QlbZgIIIJbrJ2+/YGZjlWblZPn7/vXH/7///Gdg"
    "YmZmevb85b/bh7c3/P36ugOo+R8DjlwLEECMlGZngAADAIRtVCEzSnoaAAAAAElFTkSuQmCC")
index.append('paperclip_icon16x16')
catalog['paperclip_icon16x16'] = paperclip_icon16x16

#----------------------------------------------------------------------
wrench_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAcFSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsMIaXlxeKxL9/f6X+/v0bzsLCbsbFxaPCwPD///fv3+78+vXj+P///zaysrI9Aoox"
    "MDIyATEjyRZv27YNTAMEENwByFEBtICBg4NnrpWVq4eqqgqDoKAgUJSR4f37d6a3b9+KPHfu"
    "eM6rV09aWVlZF5FhNwoACCAWBPM/A8gNMAOBvmcUE5NkMDY2Znj58i3Dnz9/GWRl+RlUVdUZ"
    "rKxs1TZuXLXg/PnjykxMTPUgx5ELAAIIJQ2AQgEWEj9+fMleu3bexd279zBwcXEwfP/+neHD"
    "h48M7969B4YIP0NsbAqjvr5Fza9fP4MoCQGAAGJCslyGk5NnEwsLx6Y/f/4oMzMz3f3+/XPA"
    "unWLj5w7d5ZBVFSYgZOTjQEY7Azfvn1n4OZmZ/DzC2ISFBRvBKoXJ9cBAAEEdwAwiMPs7Nx9"
    "o6JSfYHxvxpoqDwTE/MDYEhErlo1/+K5c2cYBAT4GdjZWYHpgw0YRX8Y5OXlGPT0THWAbAdQ"
    "FJIDAAII7gBWVnZTWVkFBicnB4bQ0ARDNjbuZb9//5IBpvAnwJAIW7164YmrV68w8PJyMbCw"
    "MAPVMwMdwsKgpqYO0mv369cvht+/fxONYQAggOCJkIODU4WXl5fh0aNnDBYW5gxMTIxWy5bN"
    "2fj164cIoPStL18+z79585qFqakp0AFMDGxsLGAsIiIGpDlUPn36wMbCwvqL1BAACCDkKIDT"
    "z569ZjAwMAKFhBEnJ+/Kr1+/ZIuKSuVqaxswMDP/B/oYYjkoJH7//gPU8/sbsNz4C8QM2PD/"
    "/5gYBgACCB4CX79+uvP+/XsTQUFhUBZk+PjxEzALmgAtYzO8c+fGFF1dQ2B8awEdwAi0GOQA"
    "VlAoMbx9+xqYO96BMvBfkONBehH4DxD/w+CDHAUDAAHEhHDA56N37twE+wpkOCiYQRpMTIwY"
    "4uISGExNDYC5gBXse3Z2iJovX74zGBkZMJiZ2fn8/PmzBJaNQfjfv39wDEzQYAwKLfQ0ABBA"
    "SNnw7+YTJ/bfe/v2LQMfHw84u4ESGiPjf6BP/0F9Dkl8nJwcwKz4ExhKXxikpUUZysrKWK2s"
    "nDu/fftWCizEMeIZVFSjYxgACCAmhCLmh0+ePGhct245sBD6DcznXMCEyQrGIN+C4hxUDnBy"
    "soN9/ubNewYeHk5QrcEgJSXEUFtbw+To6NkFLCOqSMmSAAHEgsJhYV109OgeFWBw1QQHRzIq"
    "KysBxdjADgD5/N8/RobXrz8CHfANGErcYAdwcbGD9UpICDJUV1eD6pGGnTs3/mFjY+8ixgEA"
    "AcQIK3rNzc3A8QXKzz9+/AgRF5euA8atrrq6FoOkpBS4rnj58iWDtrY2g4qKDDAdsIALJFCa"
    "QAZv3nxiaG5uZti6dV0LMBpr//37D45/UCKE0aBE+PLlK7B6gADCcAAogYCEfv78IQrU4MDB"
    "wWXLzc2jBHTAz48f3zPZ2rr6FxVVMKqpyQDTBXZfgRzR1NTMsGHDihJg+dILywXYHAAQQDgd"
    "AFIMAsD6H+QYNgYGpn+ghAlMyQUODp6dNTX1TGpqUjiD9sOHbwyVlVUMmzevqQfqawba8x+b"
    "AwACiIVgScUESvlsv0BVLjMzCwj37NmziRGYI7oqK2uB1bMkhp5fv/6AK6umpiZQim9cu3bZ"
    "X3Z29lZs5gMEEAupRScoCwGDtXvbtrUsQJ+01tU1MyopicHlv3//yfD58w9w9PDycjO0tLQC"
    "2UxNy5cv/A90RBu6eQABRFabEOQIdnaO9s2bV1e2tjYy3L37Eiz+7dsPYPx/hsb1f2Dh9g3o"
    "CA5QomQKDY1u/fbtKzhRIgOAAGIhtx4HOYKTk7N7/fplvMDora6tbQRnVWC9AJRjBdcpIDVf"
    "vnwFh0RbWzuoGK5ftmz+Ny4uzl6YOQABxEJJawbYIP0HzO81q1YtfA+0uLOqqoGZi4sNHAKg"
    "+gJU9L58+ZHh3/+XDBpqigw9PV3MwDKlZ+XKRcBEzdAOMgMggChulgPbhKCQ6F2zZkljR0cj"
    "MMf8BcY5GzgEQLnqP7DwunnjMcMDYDXPx8cJLDHrGZSUVFNh+gECiCr9ApAjgGmieenSedXt"
    "7fX/fv36DRb7C3TEH2CFxs3LCSw1uRk+ffwJDIVuBmDLejFML0AAsTBQCUBDom3BglmMf/78"
    "awHmDnBpCao/9HRUGDjYORnqG+oZJkzobAGqa4TpAwggqjkA5ghgFm1bvHgOMzCLNjY3dwBb"
    "0ELgqGhqamCYNKm7F6imDojhWQEggKjqAKgjgPmdtWnp0vlfQQmuqqqJYfLkXpDlLUDpWlDi"
    "RAYAAUR1B0BzB6jt0Ltw4Wy2EyeOxt69e3sNULgB1IJCBwABxDjQvWOAABrw3jFAgAEAEUBC"
    "K+qIWOAAAAAASUVORK5CYII=")
index.append('wrench_icon32x32')
catalog['wrench_icon32x32'] = wrench_icon32x32

#----------------------------------------------------------------------
led_blue_questionmark_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQ1SURBVHjaYmTQncwABv8YGRi+"
    "/+JkYGZ05BMRzJGRF7cUFOTl/vv3P8O7N1++P7/37PLnF6+mMvz5t5WRifnT/x+/GBgYGRkA"
    "AogFrBnIYPj5W45TkK/f1csoyNlejYGLh5vh89d/DN9/MDD8/sfE+uLVV+sLZ+5YXz54cu/X"
    "py9ygJpugLQCBBAzg6gXyGY5EVnxtbn5ns4hPioM3778ZXj58BmDDBdQ9/ePDLfuvmJgZGFl"
    "kJSXZuCVUVZ6++y19483b/cxMDO/AggAQQC+/wMAEUcICQ4MkGxvSUg+NBcA3eH2APfs6QBb"
    "PhEA+Pr6ACodCQA4JwcAws7pAOvu9wAGCQkABholEe3g3i387/KwAgBBAL7/AgABAjZBRjBc"
    "OjEXAO7vAADl5/QAMyQLABEPBgChvvEAiq/vABsXDQBbQxMA7PD2AO7q6gACBAIAFTE8PAAE"
    "BVICAEEAvv8EBwwLaisxJQD//AAA8vX8AAYHCADB1e4Ay9r0AAkMDwDq8PoA1eD1ADMmDwAJ"
    "BusA6/H7APv96gAGFBQABxEQawKIRVRa1O78S16Gn39/MmiK/gOGJyPDy+/cDK+f/mR4/+U3"
    "AyfTdwZVGTGGd1/+MHz49BcYqCwMHz8Dncr5lYFDREgNIIBYePm4+V5++M9w5OwHhggLNgY5"
    "IUYGEGRn+c3gZ8zC4GksxSAhxM4wb/kjhpcfGBj+/vnO8Ob9OwaufzwMLJxcbAABAEEAvv8E"
    "//3/AAQC/QD49/oAosL3ADEcAgAQCQAALBsBAGNIFQDy9v0AhKfjAJey7QDz9vkA9ffyAP78"
    "+AAA/QoA//0AAAKI5dvHz1/4/v/mZmD6x/Dv3y+Gs3eAgfHtF4OIHRfD5+9/GWyKrjLcefiV"
    "gZH1L1DDT2CC+wvGTECv/vn05Q9AADG9f/rqOMuvzwzcHEwg9zEwsAAVABVeAmpacPAjw51H"
    "XxkYmH8z/AcaDrSBgeHPb2DEAZMRUNmv16/vAAQQ05d3n6Z9enCPQVqUDZjkgIp+AW1h+cfg"
    "bynI4K7PxRDmLA5MTL8hhv8Fyv/5ycDOzcvw980nht8f3s8ACCCm/3//Hr1/4vw6/n9fGMTE"
    "uIGKfzKw/v/HYK3Dw6CnwMlgpsHHwABK90CbGX4D5Ti5GNgYuRi+37h5juHfr1UAAQBBAL7/"
    "AwANRfv78vTM9ensRh0kHRUDDAsA9fHyAM7a8wDR3/gAoLPdAPT28wACBQEACA4RAAcFDwD4"
    "ztHm/OzoWgAABN0CiJmB3QnoR6b3P9992vHp0TMlfm4ONT4JIYYXH/8znLjygeHPf0YGTgEB"
    "oEZuBsa3wGR+/uLJ32+eRgI1nweGBANAADEyiLZBMiQDE8P/7z+5GYHe5xAUyGcXFdZmBMYz"
    "KJp/ffr6+9erN/f+fngz8/+/v8sYGP6/YwCqB2GAAAMAS8yiWnSbjn8AAAAASUVORK5CYII=")
index.append('led_blue_questionmark_icon16x16')
catalog['led_blue_questionmark_icon16x16'] = led_blue_questionmark_icon16x16

#----------------------------------------------------------------------
arrow_down_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMiSURBVHjaYvz//z8DJQAggFhA"
    "BKP+dCDBCBZgYmKoNdIUa5IW5fz7789fkCxQjJHh1cefzCevvJ7+79+/LLBCoL3/L2QwAAQQ"
    "2ACGX/8YGH7+ARvCys6s15xlwuBhLcf85fd/hn9//zHwcTAznL72isExer3m1+9/oVZDLAQI"
    "IIgLuJgZ/oNs+wcW+8nF/J/h+EsGhoVnGBi+f/nB4KXGzKDJAbKA4RfDf6j1bMxgxQABxARz"
    "DgMTI8xbf0ACbz4zMFx58p3hysNvDI9ef2cAOh2oH2QFyO3/IXqAACCAwC74/++/JMP3PyIM"
    "f/7//PnzD++v30B1//4yMP78zvD/13egkewMf4Bi37784mT49kcN6FU2Bk6Wj0CtjwECCGwA"
    "Hy/7Nl0dMVmgou+MP/4Ii/CzMdz89pPhx9fvDD+B+PvX/ww8XKwMNkYSFl/+/NvLwcbMeePZ"
    "59dArZoAAQQ2gOM/o1BbkbWwrqogw6/ffxjEuBkZjh7+wvDr+08g/sHw/sM/BklBMYYZzdbs"
    "nGyMMm8//2EIKdoPDgSAAAKHwauXn5OnLb30W5CXjWHjtX8M0Us+MWy79IWBEeh8lj8/GQ5e"
    "fsfg1XCJoXPlXQZhXhaG2atuMjx48DETpBcggCCByMZ8YM366/OWbbnJ4KjGCgy4Lwy3n35l"
    "+PfzB8PfXz8Z7j/9xHDy8huGYCtRhkPn3zIsWntrLQMz03qQVoAAYoKmnj9/GZkaCjuP32H/"
    "9YWhwYOb4fvn7wxfgPjrl28MH999YaiJlGMw1RBgKOo5+/T7rz+VDOxMP0FaAQIIEY1cLC9e"
    "PftcmN547JefCS9DohU3w7OXXxiePfvEEGYtzFASpsRQ3Hee4dbNNw0MnKy3YdEIEEAs8EQN"
    "EmBn2bJj74OOkp4zdX3FxgxvgTa/fcvBMKtQh6F3xS2GZZvuTmUQYJ/DgJR9AAIIYgAsDTED"
    "GZws9RNnX5JiY2VKaYzXBEr9Z5i4+g5Dw8Rzaxk4WHIZ0PIeQACxYGQvUIrkYMrqnnP5xZGL"
    "r0rYWZlZDxx/NgVoSyVQDiPrAgQQI6XZGSDAAAEPO32oRnsoAAAAAElFTkSuQmCC")
index.append('arrow_down_icon16x16')
catalog['arrow_down_icon16x16'] = arrow_down_icon16x16

#----------------------------------------------------------------------
new16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAK3SURBVHjaYvz//z+DYcKEzaFe"
    "1jaMDAz/gFys4D+UYGJmYvz55zfj8nX7N966disLIIBYQOJejib2VWGmvAwkgAdP3nrcuv9K"
    "DiCAwAb8//f/B5DCagDIRf8YQE4DYiCDmRnIYQSJ///JwMz0DyCAwAbgcvc/mOZ/UDZIGZDN"
    "yAw2AGw6QACx4HLiPyj+/w9KI7F/AvEnkJt/MzAABBALrgCD2Qq1iOEvEP/8A5RgYmD4BqS+"
    "gkwB8gECiAWns2FOBoK/fxkYvgPxL1ZgGAAN+PcDFAxAScb/DAABxIKuGebU/9AABFn6ix1I"
    "A537/cZlBhZmVob/KhoMP1h5/jOw8PwHCCAWjBCH4r9QzV+/fmP4dOciAwO/AgPT+TkMDFwc"
    "DF+F6hkMXp5i0360nQUggMAGMDIi+R1I/AbiX0Dd/4A2f339hOH/teUMv35JMnAyvmP4/52T"
    "geN8C0OBzGLBDx+fsAEEENiAr78hBvwG+ROIfwO98BcUiMCA+s8uxPD5nwKD4K+TDLwSvxjY"
    "OJ4x/Ph5kWHBcZ0ven8+cAMEENiAt9+BGkCBCtUMDgugzOenjxi+nlrBwMv5i4GDj4dB3E8V"
    "GIoPGF5tvsJg6/NZREyOpwcggJgYkALrzz8I/g3F/5jZGP5ycDH8+PGD4d/3b0DnA236/pPh"
    "L1DR57ccv/9++PcFIIDALmBlYWZkZ4D4mekvxPkgg9ilJRjYeGIZfh9ZyMD+5SnD0/VnGTg5"
    "vjNwCbAwvHjA8OHnim/FAAEENuDlu89MF179B4bBf4ZfQPeDXQTyEwczMAZeMPA+fsBwlcGK"
    "QUXwAtAWLoYHb2UZTIV6+L6yfmECCCCwAQd37272u3rXH5g6/v6HZoz/4JgBJpd/QJPecf3/"
    "w/L1f7/yLeXfTGzMpU+0btleUF/l+ufzTYAAAwBq3EnkqUfaHgAAAABJRU5ErkJggg==")
index.append('new16x16')
catalog['new16x16'] = new16x16

#----------------------------------------------------------------------
folder_grey_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABmJLR0QA/wD/AP+gvaeTAAAA"
    "CXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH1AICAgsuqRRvGAAAAWxJREFUeF6lkzFrg0AU"
    "x+9MhoJLoIVA65QvIBTSb1G6SvwEHd27dcpeyCdw6tLRxSVu1S4Zs7gFxA5ZEmOI2vc/8lKr"
    "lVB68Dy9e+/3/r53J6uqEv8Z0rKsB03T3pqQJEkefd9/pfU1WdGZZDKZkIj2wPp4PH6mwGuy"
    "XhdAwnE6nbb2DcMQtm231pvK+vDIskw5oh5SSjUvl0vhum4LQNAZKbsJw3BGm4kCbDYbkaap"
    "mM/nZ+s5Go3EcDh8oiS9KIpeBH4hCILKcZxfa9G1CH/Kdn9SoOu6iONYKSjLUllRFMoOh4My"
    "/BplFaZpstLbE4BrgRo0gxmEeb/fi91ux4BBC4AsMDgDhMysgr+56KAowHa7VURsdAHqqqCC"
    "x482oht8tOsBXBNW0ALkea6ArASQehC/c4EB4Jg+TtZisZihC9TOs+cABVytVsLzvA9yXkt6"
    "XJLdHW1wlvDtgEv2DgAuCgKvyC7+AEAvP78A1jxoc0ieV/AAAAAASUVORK5CYII=")
index.append('folder_grey_icon16x16')
catalog['folder_grey_icon16x16'] = folder_grey_icon16x16

#----------------------------------------------------------------------
led_green_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQaSURBVHjaAEEAvv8BAEsbAAAE"
    "+AAACPEAAAr8BwDxAD4A7w9lFgUiPhYKGQ4BAgEB6+7v8+rq78j+/ACbAAEDvAAFA/cACAUA"
    "AP//AAKIEW4A+wcZBo6/XbqyWpHRFokMnroeDJKCkgyMDAwMLz+8Zthz5gjDog2bGM6duL2O"
    "gYWtlIGH7R7Dv/8MAAHEADbAV0iOMZLleNBC7//H3xz5//3/1//o4CdQ7Oz7k/9jJmT9ZzHQ"
    "u8qgY6jMYGTEABAAQQC+/wMAJQ4IAAD3kDc6LUpiS2QAEgUUAPj6+QD+/v4A/wD/APr++gDx"
    "+vEA6PbrAPb+9wAbKQ0A6fnkFeDQ7CIAAgS0AogFqMaGhZ8l5jfnL4bXX94ysP3jYPjD+Qes"
    "+evfrwwg8PPfT4b3P94zvPz+kuH5z3cMvyRfMrDK/PL5/YrLAyCAmFk1BLr4FBi0njE9ZhDi"
    "E2YQ4RRh+PP3D9jmb3++MXz+85nh3Y93DK+/v2Z4+/Mtw9kX5xj23NnJwMXKwfD/uYAoQACx"
    "cPMx27DIMDL8YPnFsOXZJoafjD8YdAT1GETYhBnYWTjALvj15xfD299vGa5+uMyw++lOhq/c"
    "Pxk4lb4zcF1nNAQIIBYmfga+3/w/GNhYmRj+sfxkOPphD8OjPzcZpDnkGHhZ+MEGfAG64tnP"
    "RwyPvzxi+M38g4Gdj5nh1++fDCw8/9kBAoiFkeM3AwsnKwMPEweDIAcfAxc7K8NP1o8MTxiv"
    "MbAysIAN+Mv0Byj2h4GPmwvoKjaGDz8/M/xnYWb4x8TECBBAQBXfvwiyCnB+Byrg4WBn4OXg"
    "ZOAG0hxs7AwszEwQA/7+Y/jx+xfDV6ALPzN+Z/jFBHTxDwGG99+YfgEEEMvnDz9OSH5i92VT"
    "4GHgYmZn4OHiZOBlZ2fg4uAA2gZMbaCo/PuX4dtPFgZgomT4C0x9fzh+MjA/FmN4+urPZYAA"
    "Yvr1gWXWp4ssDELsggzsnMzA0AUawsnJwM/FxSAEdLIQDw8DP5DPw87BwAl0FTsHE4MwmxDD"
    "l1NcDN9+/JwBEEBMDMz/9z8//XPt7+PiDIJ8PAxsXP8ZOFhYGLiBGvi5uRkEuXmAXuMEeomV"
    "gY2DkUFQgIuB4Ywsw9Mz3/cxsPzbDBBAzAxyYr//fWI78+4yi6sYr5iopDYwQAUYGbi52ICB"
    "xsnAxsbE8JvxF8N/tj8MjP/YGF7v5mc4s+Dj/a9/v8cwsDM/AQggZgZeNWBmYn3/+9uPHU/O"
    "/FT6do9bnYedn0FQiBsoDIzav4wMP9+zMTw9ycxwds4vhtOr3+76+vNrLAM38w2G/4wMAAHE"
    "yGBiyADOct+B+PdfbqCgJwcHR5aIFLcRjxAwuQEj4tu7Pz9fP/ty5fu3H9OBWXgj0ObPDEDv"
    "gAwACDAAvIZw301II8cAAAAASUVORK5CYII=")
index.append('led_green_icon16x16')
catalog['led_green_icon16x16'] = led_green_icon16x16

#----------------------------------------------------------------------
chat_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAPzSURBVHjaYvz//z8DJQAggBit"
    "HeYy/PkDNOQHiwALM0uBvLR4qLy0oIKQAA/77z9//z998erzlWtnL919fGU6L7fQyncfTzG8"
    "fHUEqJUJpJ0BIIBY/jP8Y/j/m8lOW0FxhY+3jqSmvhADryBQCigPctzf31qC71442B/cd91u"
    "w5adWU9e7QkE6nwHcwFAALH8/8tobqGpvyUs0oiXX4KR4ct3BoYPTxgY/v0HmQ+0h5mBgZ2L"
    "gcEjUJNRXFjarnf6o31nP16yARr/BaQCIICYLXWzNlgpWSn+Z2NiePeZgeHLJyD+ysDwDSj9"
    "FYg/A/lvXzIwPLkPNO0XO4OUkKHEhVu7eL9/f7kdaAgDQAAxKQgq6P35yMzw8OgfhmdA/OLK"
    "X4Y39/8xvHn2j+HzBwaG79+ABgFd9RPopD8cDAyKWmIM6ip2/iDXg7wAEEAsXMw8LKDwYGL5"
    "x/DvFyPD3x+MDP85GBkExBkZPv/8xnDr/nuG3///MTD+/8vw/y8LgzAPNwMT0z9OWBgABBDT"
    "yy/3PzFBApSBmZ+JgUWYmUFAjpHhzssXDFtOP2BgFmZg0DPlYzC1F2JQ0ONieMX4m+H+Z0U+"
    "BgZVKwZmUQaAAGL+/Oejkra8uzEf0OR/PAwMPJKMDA/fvmJYcfAdg4OnCoOikiDD518cDN//"
    "szPwCHIyiEpyM4ipmLFcuc4W8Ov9/o0AAcT89ef7Ay8/PXCSkdKVFpAQYuSVYGBYtfcZw+m7"
    "wgyPPjADNTMwcHMxMxy/+o9BHih379l/hj2n/zO8+a7O+ef9UXWAAGLS1w3/8pP5t+W1l8c+"
    "CIgAow/om0efWBhYBNgZPv75y/D4yx+GJz//MtwG0pef/2Xg5//HcO8VMFR5+BkY+fX0AAKI"
    "RUnREZgW/m8V5OIT3LZ36b/L17a8f8bkzydlGMEqKPKDQUGehUEamD6kGVgZhKUYGPjY/zMo"
    "anIyfHz1n+Hplae/AAKI5eDBRpbfv3/s/vzlxe2fPz5dZmfhXv5X/H8Hh5V7pqiiIIOEDAOD"
    "nBgwRQLD3RLIPvkQaAgwnN7e2Mnw99Wu/QABxMzFJfznz58fx779+LDt399fZzk4BX/++se0"
    "8/PLqzZCSqqKKqoiDNLAmDACav4HTJVvmBgZnj75xHBpYdZVhm/3vAACAEEAvv8DDAwWAP7+"
    "/AD5+vQA//78AAYGDAD9CR3++vDgouXFoQTk09cg6eLkBtPM0/+9vc388PH7A/cACQHv+xQN"
    "+Oze4AKIUVxcm+H37+8MHz8/B2ac7ww83BIMP7h1GIDJhuH/q/3A9CHMxcDOpsTw44UQMOl+"
    "Z2ATNGbjErH4zy0i8/vT4yMAAQYA6Kd9HG3Y0SUAAAAASUVORK5CYII=")
index.append('chat_icon16x16')
catalog['chat_icon16x16'] = chat_icon16x16

#----------------------------------------------------------------------
note_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANMSURBVHjaYvz//z8DJQAgAABB"
    "AL7/BHIDA5tDDQ1hIhYYAqnk5972NgpxbD0Q1Pzs5vYH9fP13dr/9PTqAAAX8gAACD8EAAUh"
    "/wAE1v0A8ugBAG/a/wACAEEAvv8ERwwMXzYICAMKCAcAaw0LI90SHLAEOD/3C/Pg+ALx5PYY"
    "Hufq9+Xs5wrq9er+Dfzg++0A4gHwAOz88v/j//4A0gIAQQC+/wQgHh4CBAcGAPMREgD02NcA"
    "A/wwAg9LgAsLAPgLAADlExEA5x0A9uEA/fXhAPrw5QD77+r+AvDx/g7Y4/sKiuSVAojlyOu3"
    "z8M52GV+XLvO8OT3bwYmQy0G9n8/GBiYPgN9yAbEkgwMXIIMzFyaDJyi3xk4Gb4yMTB8Z/qv"
    "sJblz8THpgABxPJe6efOF4o6yc9uCDAw2JsxaOanMLAwAT3M8AWIgQYxfAPi71D8DYo/MXx9"
    "9pbh40eG1wABxJJiK8rF0zSFQZxJloGb9TsDE8NHsAIGhl9oGoH0f6Chfz8wMLC8Y/j88iPD"
    "p28MbwECiOUfK5MAC9AmXtY/QEVvoRp+QTX9QBgC0/znHQMD81uGty+////8g+EFQACxMHNz"
    "8P1n+cvAyPAeqvgnks0w26GafwE1/wNihncM757/+Pf1P8NDgABi+c/OxMfI/A+q8QeSrTD/"
    "fwXaCtQMjD+GP0BL/gPp3x8Z3j/99Qeo4jlAALH8Z2PiAcqgaYYF3FeIJpDm30Cb/wLZjED6"
    "y2eGN69+/wZ69CVAALEwsANjBuzn/0gagfT/rxANMM0gv38HsjnegpW/+MwIcvI7gABiYRJg"
    "ZAdGOpD9lwHikt8Ql/wFxsZvkAFA5///wPDj3XsGlsvPGVh+P2Z4cZmV4ezt389BoQ4QQCz8"
    "It+AKe4uKFtAwwFkADARsQANYAFGJxuQ/vuWgY3lGwPTXyaG86t+MeRv/HHz8B+GPFDoAgQQ"
    "4xR/tktcCnwq3DzsTLy8bMwC3MzMvFxsjNycrAyc3MwMbJyMDBw8/xj+P/vLsGzas79N+95s"
    "fsLAkAHU/BKUGwECiNGUk4Hv3HcGOWYGYDpiYOAHYklWZgZRTmYGEU4WBn52FmZ+HhZm3t9/"
    "GLgufPq1Dei+XuTsDBBgAKZFV0+MM47wAAAAAElFTkSuQmCC")
index.append('note_icon16x16')
catalog['note_icon16x16'] = note_icon16x16

#----------------------------------------------------------------------
arrow_down_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAZmSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAD7gCAABpwBwAEEAsyh9FkFi51qgz/GVYy/P6rAmT/AouAkg4jDjNZmN4xMDEmAdkH"
    "sCn4fyYNzgYIIBainPnzj4OiDL9hZ7ElAxcHM9Ad/yEOYPgPRZCEzAh0EAszE0PbvAv8x089"
    "82DgYD5AyGiAACLOAX/+MwrxsDIEOioCLUD29n9UDMpRjCwMS7bdYTj++99/BnZmTLMYUYMN"
    "IICIcwAjw/9/QPO+fP/FIMDFynDsyX+G99//MzACLfz75y/DbyBmYfjHYCbNwCDGz8bw6/c/"
    "sB7MFIcZZwABxEJSigXq/we0dO1NJoarrxkZWP79Zvj96x/D9+9/GHiY/jBo+bIySDD+x60Z"
    "CwAIIJIcAArhf//+MbACvcf8/y8DM9AB///9YmD5/wfI/wN0HBM4PWB4HoflIAAQQEQ74D/c"
    "EaBg/83w9+dfoGt+Mfz59QeIgXzmPwz//7ACHfQf1Xo8loMAQAAR64A/sDwHSgt/fv8FWgrM"
    "jf8gloMx819gCKCkm7+ELAcBgABCdwAvw99/ugx//7Mgefo7w6+/6iBLQXEAjAGgA0BxDwr+"
    "P2D6908gZv3L8PfvX3AU/fkDVPTrrywQGwP1c6GEBgvTTSDrFUwIIIDQHPB/hiAfexQrkPX3"
    "H6Kk+cPGzCDIy8bAxPQfGgJ/wBgcAkDHgPH/fwz//v4Du1qQm5WBX4AznpWHNR7FdKBxb7/+"
    "BpUNjjAxgABCdcD3v6JmFuIMUypsgFnrD9gykIn/gIazszIzsAId8BMY93//QIIe5IC/v0AY"
    "mAaAPgep+/79N0NVsi5DZpg6MMsDHQwMuL9Ah4mLcDJ0L7nOMHv5NRFkKwECCNUBHMzN+48+"
    "srrz8CO3h7Us0Pzf4CBnBhoELAiAlv9mYAIlwt+QoGeARQEIAy0HFTu/gGqARQGDgCgbw/cf"
    "kGjh5eZgePnpD8Om3ff/An3Ri2wlQACxoGWXw79+/O3PbztUs3e+P4O0MBvD43e/Gead/cPw"
    "69dfBkaQhUDLH735xcD09zckKkAOAYbIV2BINK19ysDK+BccSl+//mJIdRNnMFbhZfjPxMKQ"
    "13OS4eWrb+sYeFiXIlsJEEBMGHmNk6Xj1q23JxpnnAE6iIVBguc/Az8wgW27+oNh700gvv6N"
    "4f2nnwz/oaEAyg3/gGngO7CU3HfpHcPW028Y1h18BvT5HwZ1aS4GIUEuhnlb7jEcOPT4KQMX"
    "SxnQjt/IVgIEELbq+CsDH3v2vJVX3yzdepuBk4eDId6QmcFCGhj/QEvYgTny/9/fUMt/g8uB"
    "378hocHO+A/MV5ZgZ6gJl2eQBMb7uTufGDpnX/wDtDwPmCgeoFsGEEBMOIrNc/8YGXtKuo8x"
    "XLv/iUFUkI0h25qdQYjjH8Pnr9DCB5b6oRgUCt++/WRgAkZTNdByDRluhm9/mRgKe08zfPz4"
    "ayEw7tdhswoggJhwFnscLBNfvvy6PqXuAMO7L38Z1CVYGcqcuYHF7x+gRb/AvoclQBD7x4/f"
    "YPGyIFkGZ30hBlYuDoaqKecZLlx4dR4Y71UMOKoIgADC1yL6wcDDnnX8+JNLNVPPMrABDbRT"
    "5WDIdeIDx/cPYHb7Cy2GQbnj3cfvDCmukgzRDhIMAoLcDLM33WNYsu7WG6DlScgFDzoACCAC"
    "TbL/L4DpIX76kitvJq64zsDFy8kQaMTLkO0ixPDl60+wQ0Cl4Lv33xli7MUYcn1lgImOm2HP"
    "udcMjZPP/wYm6AxgvF/AZwNAADERUQdfYGBjLizvPsGwfM8jBkEhboYEOyGGEl8JcNx/AuaI"
    "ZDdJhqowBQYJUW6Giw8+M6Q3HmX4/utPK7DYXUvIeIAAIq4yYmNa8ufnX+G0mkN9v+utmSJd"
    "ZBniHUQZFIRZwInSUV+QQUyEm+HkjY8MybVHGN68+zEJmOqbGIjocgAEEAvRdTE788RvP/6y"
    "ZTUd7fr8zZQh1kOBwduMBVJcszAz7D3/hiGr9QTDq5dfZwDjvYThPwNRPR6AAGJE7hnhaRXD"
    "qlgGht//Qhl+/ZvkaisjkRSkysAJrKhW7XrAsGzb3a/AVlkt0KH9BP2D1CoGCCDSHIAAWgzf"
    "fucA49iRkYmR9f/Pv6eAQT4DmOAOERWgSA4ACCDGge4bAgTQgPeMAAJowB0AEEAD7gCAAAMA"
    "p0HuRcSb+YYAAAAASUVORK5CYII=")
index.append('arrow_down_icon32x32')
catalog['arrow_down_icon32x32'] = arrow_down_icon32x32

#----------------------------------------------------------------------
edit22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANtSURBVHjaYvz//z8DLQBAALHA"
    "GLx2xYucnG0jeLjZmf79xW8ZsltATCYmRoYfv/78P3Tg2Pa3e9r9QOIAAcQAcjEIu5Qt//H9"
    "Pyr4hwXDxP+iqfsJxGEd237DzAMIICaYzdxcHIxsUPb9x68YTl++y8AIZKNjmDNBGi/ffMJw"
    "+fZTsDgryAxOdrhPAAIIHhR///77/xfkLaCufaduMFy+85TBVFcZJQj+/Yd4HRRSbEDTth+9"
    "zPDz9x8GTSUphr/MjAx//v6DBxJAAMFdzMTEBNbUPGMjw8LNxxhU5cQwDP0Ho/9BxORlRRm2"
    "HrnCUNi9kuHLzz8M7Kxw4xgAAogFofE/2EuJgbYMUT7WDNKi/CiR9R/qWpih3/8wMHjaGTKY"
    "aCsyAAOZgYmZmeHNl39wPQABxIIe1TLiwpguhRoMCwoYzczEzCAmJgyW+/CbgeHZZ4Q+gABi"
    "IZSs/iElK5ihIPAHGCE/gPg3UAHIkH/AUGBiROgFCCDsBsNcBmX/Q3ItCPwCGfoHEolgyxkR"
    "bBgACCAWHObCI+s/clBADf0FMvQfVB5I/GWEYGQAEEBYDcYwFJrEwIb+hboOGpEgNjMDwjcw"
    "ABBA2A1Giv1/4DTOwPATGp4wy0DyIBqW+/+iGQ4QQHCDGRkRBoG8B1L0B0j//guh/yGFNcyl"
    "YN8A2X+YIAYjA4AAghv87TfERaA0/h+If/+GhiUDUsaAGYoUXIzMiOBDdjJAAMENfvmFgeEL"
    "0LDvP74yfAVGMzcHJ1AjIzxYYN5GtgAk9/n7d4b/wLBi5uFm+I8UgQABxIQICkawhomLdzNE"
    "F09jWLntKLBggoQv3FCoBaCgYedgYDh46hJDZs1UhskLtjB8/faPgYUJkaUBAgiRpYE6QOmx"
    "IN6dgZ+Hi+Hq7UeQjAD14l/koIDGwb37TxlMdBQZshM8gL5kBhVCcIMBAogFqRBiBBZQDEK8"
    "nAy25toM8q/eM/wCBg1I7B/Ua/+hkQxy2B+gnL6OKgMjkMPLxc3w8zsDiA0PDIAAghv86/ev"
    "/5+BJrABI0xRVZpBUV2a4cN3SET+RUqzsIgDZWddAxWw2OtvwMgHGvkT5BIoAAgguMH3L5/a"
    "Ht0s5MvHw8kILlf/I3IhMvs/UpaHYWagQ7///MV468yRowwM7mC1AAHESKvKFCDAAAtr+oNS"
    "uN0AAAAAAElFTkSuQmCC")
index.append('edit22x22')
catalog['edit22x22'] = edit22x22

#----------------------------------------------------------------------
clock_resume_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9sCDBADALMwWicAAAWh"
    "SURBVDjLhZVbTJv3FcB/39VXMAaDzS2OR0sgKWmahkKakAS6ZG3I6DKqdavUblMrbZq6iyZ1"
    "06Rpe5gqrQ97zNpK2wtatbXqsippozQBsiSUQdImaXC5RDAId2wM2PiC7e/7f3sgTTQt0Y50"
    "Xo50fueic5F4kLSdCaJLP/AW2o5Wlruqiwp1lwTEk/n03EJqPraWPU/efJue9tH7uUv3hR48"
    "3dX4WGln+6EtztotLtyqimRagISlQNo0GZ9Lc/byzEbfwOIZLOs7XOrIPRjc+PdQUYm994ff"
    "bdja/KiftZk1SK5iB6KL05hGnkBVDRsWWE4P3mAJ10eivNk1tBiZTx3j087PvkQpd6ENf6ut"
    "8Ds/+9Uv9ge8TjvXTndT50vTti/ErgY/wlzEU2jS/vQegn6F1MIU3R8OUvnwNlpbt7pHw4sv"
    "rirHzhB9f/EeuP4dtbhAu/rqawf9a5EsofwSrY06Jglqt23HFIKCgkJKy8rRNBsOp4ulpXEe"
    "rfMgrwvCMxb7Dj+kha/c7kw5O95k+WRusxVVf+p66eeHXvQGAuzVpmk/UoEl25AlCVWVURQZ"
    "Vd3MwTBMhBAYhkAIC0nk6Lk4R/dqBblMgj//vue8MfX9IzLFbwXrdwSecweDRAd62VK6giXp"
    "OB02HA4dm027CwVQVQVd17DbdRRFxpJtbAvlyIZ7kHwBdjVWt1L0x8cVSfnaL5/obGpNb8i8"
    "0rTBwnKUcHgCr9eN1+t54DRKkoSiyAAMj41QV13IP8fcuMvc8kTfLZvqcWrtSVcxtdYKLfu2"
    "Yem7GRub4Ny5f+HzFXPgwG58Pu89ojBB3qxAliXAYu+TrUi5JKeHZ7lpq8RX4mhV/MFvvW7W"
    "NzgOlSY42OTHFDLV1X7q6kJEIiv0918nk9mgqMSLOjqCOTTEksvD+Pg0/Z9cJ5/PUV5ehq5r"
    "zNyOc2GhCG1+WlV1l9O1ti6RiU8zOLhEqKYeu70El8vJ4cN7iURqudh3g7F/fEz6iyEK/ZUo"
    "0Txuh41gMEAoVMX6+jpjozfJJXMkM1XY7HZdFpINSQZh5EinUghh/lcvy8pK6DzShC+d4EK8"
    "gjMzfior6jl+/Kvs2dOA1+vBMEzSqQyGkUWWQGBDzqRJl7gsyisfoq3tMB6PF9MU98i5HOLU"
    "KZ7Zt5udOxv4yx+OMr0B3/jpWaZur2GaApfLRVvbU/jKaih0CIwMeTkRzS0UaQbzKYn1eBxV"
    "VRHiDtiyMD76CKO6Gtsjj6CpKhenJYzSr/Dqa0/zxgdTvPybC2RzFqn1JLeXDXxuiXg0u6Im"
    "U5wTsdT2RGkh165NgJamsbEJTdcQfX0YLhe2lpYv43AzApNR+DwC9rpdbH3C4vjv+gmJeYp3"
    "tEA8RXzF6FNR9Lcnriz+qHn/Tv3jy5fpOBLAtNnJDoWxVlawPdtx91JZEtxYgKllkO8YxRzo"
    "jz3Jupql99RFMpNCCEs9oZD96/J66tu7PVWe+uaWh3EYEkFzGWP0FvIzR5GQwNqc2UvXFhiX"
    "y1lNQ97cVENIbBgSsbRCaUMNFNul5fmB2c3VcTpf6H5vNnJr1UHMcjDYPcYtv5+BqwMIA7JZ"
    "k1hsjUw6g5BAlUFV7qgMNhXsusHoh72Mdp38NeGf/HYTPNO8kctYHW+9MZ0ZS5bB/gNcGlwi"
    "GYNkMoWqyoyMhFlcmCdQBFUlUFkM1T4IVcHK51e4ceKsiH3ibePfP3v9fw99zfBO1ek8/8L3"
    "/GWvfF3CEY+xHImhKRrR2Cwn+2PUvPQ8mYyF0w1Xe4aZvDLL7HgokU2Y32Rye8+DX9P2STc5"
    "9f2qHQWtzx1z6vsbJArkDAUO6Dr1BY0vN3O1b4bed4dIJevNuXG9H9V4npGtC///5wHULT4O"
    "yo+lAvVgwK+VFhZIeiIxLJf5E/LQQFFMiC3dYJ5gNNB3P/f/AEFBW9C6MxLvAAAAAElFTkSu"
    "QmCC")
index.append('clock_resume_icon22x22')
catalog['clock_resume_icon22x22'] = clock_resume_icon22x22

#----------------------------------------------------------------------
activatenextviewer16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOKSURBVHjaYmTIYECAf0D8i5GB"
    "4fd/BJ+VlYmdjb2b7Te74Zen74sYWAUv/GdkZWD4C8QM/xkAAoiZwQTJAJC+30D8k0ESqNmc"
    "4Q+DKsPHfwpirMK+eYnRbv8Zmfwf3HvxioGZ9RLDf2awFoAAYmFgRDLgD4MwNxd3q4GhhoeC"
    "jIwQExMD459f//5zs/BwKBqyMChaWUjyivIs3Lbpsuzff//aQHoBAoiRIRWq+S+DnKSYyLqI"
    "EGdjSVU+hv+cv4Hy/xgYGRkZ/v7/y/D23QcGVnagj5i4GBZWXn/94MxrLwYO5jMAAcQCdvJf"
    "BgZObrY6R28d4/cirxhu3LrB8PLmV4bf34C++sfIwMHEwiBjBDSU/Q/D3d0/GB7d+ryCgY31"
    "PAPQCoAAAEEAvv8EDP0MAPQA9CsaIxJF4wfnAObY7gDtw/zz8NP0gQD/BM8A+QMHEw0Scf8K"
    "BUXXyOvw98wAhwgFDIokCRQAFgINAAIAQQC+/wQ87jwAygzK/+Lf7erg7O0E/fb/BQbiB6Id"
    "+h6NAP4DAAABAgDm9vE22KPyAxf4GHQLDgjPKAoaAPf5+AD///8AAgBBAL7/BAT9BCZA90Bx"
    "9Or19//kAOIBAAEAAwUDLwj8BoDz9PKMCgkIPyUI/T0J/wfsFA8NZP0B/gz2+/gG/gD/tv8A"
    "/6YCiIWTj7u1vKQ0RE1J9f+3799BcfmfnYOdadvunS4zZ856qaIkWxISGKwENIzhw+dPDN+B"
    "an79+sVw5cqVwIcPnx4HCCAWLk5+IWlZGQZeYT5GXgY+UHyAI1ZGWgZoFqsAByc3Dzs3B8Ob"
    "D28Z/v37B9YMopmZWdmZmdmZAAKIhZeXX5OZmQkckzBJdnY2oAJmBi0trTQODm6B8+evAG3+"
    "wfDnz28g/gt04z8GXl5BBklJxtkAAcQ4ffr8XxkZCawgjX/+/AEHzN+/fxl+/PgBVgzSBGKD"
    "UzZQzb9//8Fq2NlZGTZv3v0HIIBY2NjY/4JSPANyigYrYAdikKa/DFxcXGAaZMDfv//A8szM"
    "jMBswv4XIIBYmJjYwJpAKQ6EQYAJmIZBCv/+/QPV9BdqOwSDDPj/nxGojoUBIIBYODg4WWAG"
    "sLKygP0H9D5QEzAJ/2WAGsAIdgGEZgR7A6QG6EpmgABief3646vdu09wA237//fvf6hf/0Ky"
    "B5DPyPgfrAFhM4gGuZKR8d27H98AAgwAdZ6I0i5UVxsAAAAASUVORK5CYII=")
index.append('activatenextviewer16x16')
catalog['activatenextviewer16x16'] = activatenextviewer16x16

#----------------------------------------------------------------------
listview32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAdNSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsIIKRMTmNh4upjoOdifvf//9/QWLggGEEMcCIASaGHF7/GdDkYfqQ5TH0MTL+/fOP"
    "4c/Pv5P+/5/XCBBAjKAo4ONNe9rYEy/FKy/F8PXbf4a//xgYgGoY/gJ1/QU65w+I/gfFUPYf"
    "qByyWhD9D5kPlAcywTRIPUiOgYWR4cPT1wwX5s579evrDHGAAAKHADcXK5eEjjLDD3EJBvZv"
    "EENZkAxi+YswHOYAmOV/kSz/i4RBan+D6P+oav8DbfzNIcjAxM7BCrIbIIDADvj3j+nvly9/"
    "GB4D2R8/QcIOZOB/mK/+o1r+D2YwTA4phGDs/1D1cL1QzMwBNP/Hb6DZTKDwYAAIIEgaYGJi"
    "+PWHkWHlmgcM3z9+Z2BgwoxfbOA/ckQjMxkZMDSCuExA8X/MbAzSsnxgO0EAIIBYIBqYGT58"
    "/cvgLPWBoTBXFujS/wyQNMiIasR/JAuIACi6gRw2YPwv3P2SYcn138CkyAwWBwggFojRTAy/"
    "galEWISHQVVJmMoZ7T84Lv8B44GJmYVBUvwnw58roOCAhABAAEEcwMgESaV//8G1vd65i+H1"
    "yTMM/9lZGH5++8AgqGfIoBgUitX3kMIMGGpAbzIBw/kfOP38A6ehf/8gZv769ZeBh+c/MEf8"
    "BaYNZrgDAAIIEgVMzMBgZ4QH8X9gFLzZtImBT+wEA5vYR4bvvx4xPNpqySDt4s/Axs8GNvTb"
    "t+/wPM/JyQE0jwmY2v8yfP78Ayj/H+4oEBuknhnoex4eLrB6UML8zwSJAoAAgoYAIzgEWGBx"
    "B/QJBy8rg5jMCwY2+d8Mf3n4Gd7f4mL49wfimydPnjIUF1cBffWHgY2NjaG1tZZBTU2F4dGj"
    "5wyVlfUMP3/+BDsIBr5+/cYQHR3BEBMTCgkVUIJghAQlQABBHAByPVCQBSlYmX99YPgPtIiB"
    "kYPhD9cfBqavP4EpF6JJQECAISEhGuizv2CLhIWFwD7l4+MFWhTC8AdU8jAwgsVARdHv338Y"
    "VFVVgczf4BgEZ1OoWQABBI0CJnB+Z4fbDkyxfOIMT7fKMDAIMjN8B5YozNqKDCw8EOUgi7y9"
    "PeCOBVkEspSHh4fBx8cLSZwBmg7+M3z/BoyaP7/hUcDADDELIICgIcAIcRVS3hUrqmb4GZEG"
    "TqB/gYZzSAkzMLMyM8BqT3D5/v8/PL5h7O/ffwLTwj9w/INKQIj8X6AD/zFwsbNC1EGqBLA5"
    "AAEELQdYgGmAEaXsYAL6khOIMVM7A9RCBmgQIxzwH5rdIJajOgyGGaBRAIxPMBsggJDSAJ6c"
    "jFTkQXyOyHp//8Ic8Q8uhwh6SDaEWQ52ENgMRrCdIAAQQHAH/PmHpfyEV6AM0ET1D8XnyDQs"
    "z8PkQQ4CRQUsBP4CE+zfv8xgl4HdDM2GAAGEkgj/IxUy3x8/Zvh47Q7DPyZWcALjU5Zh4FNV"
    "ApYR/5B8/g/qW1hQw9LFX6TQgPJBjoMW8WAjoLkAIIDgJSEoGyI74MHUaQw/7u9mYBb/wfDt"
    "y2OG+2xuDFZ9yxkY2ZnBVR0HBws8pYMS3p8/f4AhxcTAzs6BFOz/kNg/gKHwD14O/IemAYAA"
    "gpjCzASugBigZcd/YKnE8ucHg4bPZwZOlc8MP4FV98UFHxh+f/3NwMHFwvDy1TuG9es3gy1l"
    "YWFmcHd3ZZCWlmR48eI1w5Yt28HikDIbYuHPn78ZdPV0GRxtjCBR8A8U6JAoAAggRBr4z4hU"
    "2zEysDF+Y2B+AyyIuFgZmHn+MrD9hYQQyMBfv34w3LlzC2jRb6ADWBns7GzAvv/16yfD/ft3"
    "gAUPqLZjhCfeHz9+MsjISIHj+D+0HIBVxwABBI0CZgakeggcEky/fzF8OvCVgUUcaCGw7fL/"
    "CwM4+H8BSzVRUTGGzs4WaCqHWPDp02cGQUEhhpqaGniwg9IJJCH+BUbTL2D98Q2pcQBxAEAA"
    "oSZCpOalUHQ2w+cz1gw/WdgYfgHLdiktNaADmBj+AWu1f+CC5S9S/oal/r9A3/8FV0qgxAhy"
    "AEgO5IjfIPX/oS0dpGwIEEAoiRAWLKBsx21qAsbILZpfQJ8yMv9nYPwHLecZgWwmIIY1VoD8"
    "/4xQPgOI/gtmMzJA2mjAFjAwuTFCWqrQRAgQQPAQADng+5cfDF8+f2f4DS1cYDYzgguQ/9Bs"
    "9B/c3oPz/yMKoX//oWKgUAHnQ0aIpUAfsQDtYAYmdlBoMvznhIcAQADB6gImXl52hvP32Bki"
    "eu+DHQOyGFY6/vuP1EYEWcQAkQdbCPIjtGj+D01kyP2Af0glJzPTN4YXn4GZjo+T4S8jpHgD"
    "CCCwA37/Y/zz7s1bBm55CYZ3X/6CNcFaxfBWMAO0eY3UGobx/zEgtYyRWtDoZvwH9wuYGJh+"
    "vIdHLUAAgR3w6x/DxGMbD+YzgRpt////w9a6xegRYeP/xy8PNY7x339QIvqzECQGEECMA905"
    "BQigAe+cAgTQgDsAIMAAhUZ/m0vuW5MAAAAASUVORK5CYII=")
index.append('listview32x32')
catalog['listview32x32'] = listview32x32

#----------------------------------------------------------------------
undo22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAO3SURBVHjaYmYgAJw0NPwdRUVP"
    "CAoJSd9/+3Y7A5EAIICY8El6KCn1e2tobJhcUiLA9POnFQMJACCAsCIrY2N+P2np8zvr6///"
    "//37//8tW/67yMufIcUMgADCZqh9ho7O+0cbN/7///o1BK9e/d9XQuKpo5ZWPMhSYswBCCBm"
    "ZI6rmlq9n7j4gupJkzhE5OUZGL5/h2BRUQYbU1NeEwGBgP9Pn1bwMDAoiKqoXHj8/PlHXAYD"
    "BBAjzOtCr15tSHZ1dQjIy2Ng+PcPFf/9i8K/eewYw7RZsz5c4eRM2HfjxkZsBgMEECPQUHn1"
    "V68OlOflKahbWiIMQsdo4t8+f2ZYNGsWw6qvXxP2X7u2EN1ggABiAWIBHR0dBVkNDQaGly8Z"
    "GNjYGBj+/EEYAmLD+Eg0F5DOCA5mYFixYgGDlhYDuuEAAcQMDKeXv9nYHtw5ftxZ3cCAnf/j"
    "R4gFnz4xMHz9CqF//ICwQRjoUoY3bxgY3r9nYHj+nEGLmZnhzv37zl9lZde+eP36LcxggAAC"
    "R96D168v/peU3HZmzx47ZR0dUQlQxD16xMAAsgSEf/5kYJCQYGD49Qti6JMnDAzv3oEtZf39"
    "m0GdlZX9wJs3xnc+fZoHMxgggOCpAuRyXgWFpQcOH5Zj/fpVVzcggIHhLdAB376Bvd+6dSvD"
    "G05Ohp+ysgwS3t4QXwBdDAp7fhYWhq/v3sk+kZE5C3T1LZB5AAGEktyAhv+89/79ulc/fz6/"
    "efCgrwHQcC6gJoZXrxiaz5y5ue3Ro5QDp0593bt5swSPigqvMiiML10C+0SMiYnh4MuX7Le/"
    "fl0FMgsggHAiM3194xhl5Uc3Zs78/3/dOvScx+agrNzZ6+n5/+uyZf//e3v//29r+79ITu6/"
    "goyMDEgBQAAx4zL46cuXz5kkJRcd3rdPh/nbN9VzDx7cuv/pEyzm/z54/34PKNJfXrkSYJef"
    "z8AATNvfgWF+kpf3EjA4LgIEEDM+VwMVfL/78eOy+3//3n3y7dv6j58/P0KWB0X6P17eD+zv"
    "3nnoWlgwMN68ybDn48fHDz5/3g4QQAzUAD6ysvtfr1z5/yswOLzFxUFBxgwQQEzUMPidmFjC"
    "yg0bGLisrRkEGBklgUKcAAFEFYOPnT37cM+BAwfeyMkxKLOySgGF2AECiCoGg8ArKakF5+7f"
    "Z2DmB5eqbAABRDWDgWDD7oMHGRiEhcEcgABippapoMwl+euXAy8np8IHXt7NAAHETEUXM/Cp"
    "qT1+9O7dz3P37q0CCDAABUu8rTcDKQ0AAAAASUVORK5CYII=")
index.append('undo22x22')
catalog['undo22x22'] = undo22x22

#----------------------------------------------------------------------
arrows_looped_blue_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAO+SURBVHjaYvz//z8Do8ksBgYg"
    "zfDrLwMDBzMDw89/DAz//jOw8rIuUpcTdLt15UXPr++/FzKws75mYAKq+wPUw8nK8O9CJgNA"
    "ADExIAOgHMNfoOa/f1kYfv9l+P/rn0pts4t4TpVLt7S6xH6G33/yGP784wUZDgMAAcQCZ/39"
    "x8Tw7084DztHAq84jygzIyMDCzOT6oOPbAyylhoMCQry2mcO3pl4Yf/ltFePXvUAXb4SqOs7"
    "QAAxgr1gNIOd4c/fqRpm6snatjoMnEKCDMwsTGAXgRzz9jsjAyc7AwM7B1DLr98Mry9dZTi+"
    "7ez5L0eTjQACCOKCn3/ylSz1kjUC7Bh+AH3w9Mlnht8/fzEwMzMyiAgJMHz7y8jwjxFoHVDu"
    "w8uvDG+efmX4+5/hHkgrQACBDRAQ4y8KSTZmEJZiYNg46+z/c7vPX/n3//9jFlYWy9z2cEFe"
    "MS6Gx3c+MJzZd/XPtWPXD/788K2ZkZv9IEgvQACBDeDgZBb/9uQZw9ubvxnO7z73/teXH1bA"
    "APjCyMxwSJT9i+3ZHdcYdqy7vOfj0/czGDhY1gLlGBgYIUEHEECwMChk+PqbD+hnJgZe9tvA"
    "GFgC9BYDAydrh4Q4t/OL228mADWtZeBm+8Hw4w8kprjZGP6fz2AACCAWaPT1A6OIgYEFyP0H"
    "lPz+k5Xh1z97IQkewxfPvxxi4OJYysAGDFQmRj+gvBrQelA8fgHimQABBI1GoHs4gc5iBir6"
    "9M2OnZ8rw8lbK9jaRZWtpXK74I/P38CJjJWVaXJAprWciDgfw6bl50AaZwIEEAs8Bf36p8/M"
    "y1qp6qznZe6qx6uuK8bw7cNPBsZ/DKrAwFwH9KiIjq2WtImvMcNPoDd+Lz73DaQTIIDABrCz"
    "MU+RsNZNFtHV4ZBQEmd4CzRvL9ACDkZWBp1wb4E/v/8HsrCxMojICTOcus3AcH3XBYZXj9/M"
    "AekFCCCwAUxMjPaqqsIcYhqiDH+AvvjxA+RdBgZeTiYGcaBL/oKT+H+Gr+/fM9w5fI3h5oFL"
    "O4DerQXpBQggSCzoz+Bh+v0zRlxeLN/AUV/DwFoJmDY4GP58+MGwsnfr99+//9wH5Z9Xb75+"
    "/Pz2y6p/LCxTgbHy6//ZdAaAAGKBBuGXf38ZZjy/93LV8zu7k68clkwJiTJQszaTYrh+5+2t"
    "36+/WgCdxMrAxw4MeWB2BKUDKAAIILALmPRnAHPeb6AcKDxBfvgpxcbJlqhtJF1w7d67Yz+/"
    "/fZn+AnK6kD7fgGjmx0a+0AXAAQYAAWyaQBW5RX6AAAAAElFTkSuQmCC")
index.append('arrows_looped_blue_icon16x16')
catalog['arrows_looped_blue_icon16x16'] = arrows_looped_blue_icon16x16

#----------------------------------------------------------------------
newtmpl16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAEZSURBVHjaYvz//z8DJQAggJgY"
    "KAQAAcSyPIjxmbIbg8DfXwz/GP4CRb4B8Rck/BkqBqK/QsW+MjAyf2JgfPyN4RNAALEoOjGI"
    "mGUwsDL8AEp8AOK3UPwaif0Oif8GavV3BgZORgZWgABi+f8XbC8rw3OoApAh76Hs91D8EckF"
    "P4H4FxD/hSCAAEKEATPIQ1AaHTMhYUbUkAMIIIoDESCAKDYAIIAoNgAggCg2ACCAKDYAIIAQ"
    "BvzBgkER/A8J/8fEAAHEwsgMjigGBnEgZgOnCEh0MkIxSONvIP4BTQNfEdHNzMjADBBALE9P"
    "Mby9soaB/89PoHn/oAo+IeHPaPQniEEs/xgYH/5n+AwQQIyU5kaAAKI4EAECDAAirl/guYDp"
    "zQAAAABJRU5ErkJggg==")
index.append('newtmpl16x16')
catalog['newtmpl16x16'] = newtmpl16x16

#----------------------------------------------------------------------
markcompleted22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAZ5SURBVHjaYvz//z8DLQBAALGA"
    "iAuKjAxszAwMLIwMDHw/GRjMXAI4NXQ4Ot1dzKN//v/+b96ctTN+TTnbf1+NgYeZmUEY6JYn"
    "DJwMr7/8YmAQvsnA8LuGg4FBGqjxH8SR/9P+MwAEEAuExcDw9yuDPCsrgwIbE4O4Oe+7GMNI"
    "D19Grn8MzP8ZGeyzLGpEfv6MZ1bT4GaQEOBmvHHzBcPOo80//v6bi+FUJggFEEBgg/8wsoWL"
    "50TO+amuy/OEVYjB8M19hg3ndjCIq/IxvPr8keE3yw+Ge/qfZLU1Exjk1dQZftp8l7cU7ZhT"
    "XL3sClD7SYjjgK5jZNBh/cmiAuRtAAggsMFsUjxlkjlRPH2nRRk0ZHUYfP7+Yjh99BHD9ivr"
    "GH7+/cbA8JaNweCvHYORuxmDhKAow6e/DAzHTZMYbFU3Fi299DWc4e9fBnYmzigXU43ZZ+c+"
    "ugEyGCCAwAZz/H+nz3D3NIMalzkwwL4zaHLyMczwmM5w4G4Yw68/HxnE2eQYDJVNGPiE2Bm+"
    "/frFIMbMxPBc2Y5B1dbPX+X1cqP3iuIxRX5GhTvW3HnyYubbOIaJDAwAAQBZAKb/AP///wDC"
    "IADE/5xK/+Xftf+Fsdf/9OX9/+/Y+v/5+///+/f//+3x+f9ouXL/wdnN//bz///u7/j/+Pj/"
    "/4mopP9YmCb0p/xg/1KrOPEACQA5AAAAAP///wACCBzUz94yLPlw5gyDFMcHht9fnjO8ZvzL"
    "8PvTb4YvX74zfP7xk+HPz78MPxlfMMy818Bw+ektBhYGdob5p5cxPFbcwpCY7sW9dceDL+vS"
    "TycwfGc4DotDgAACB8WnXwybH1161Sbw+ragEj87w8UXXxnsOdgYfn74x8DE/5+Bi52LwZ4n"
    "hOEJ31aGxXcnMTBeFWDgV3jEEK6nzrD04GWGHXkXlzK8+7seOXEABBDYxX8YGJ7ee8BwhOH6"
    "eQYz6a8MF+++ZGAQYmH4/uY/wy9g+vny+xeDsbgNw/sn3AwOxmIMGnqfGVw1tRiWHTzHwHPf"
    "kSE8uM0alAYYwmbBDQYIICYY48kbho0fL11iUBP8xPDp7SOG1yxMDL++MzD8Aqb7b3/+MYjw"
    "izLw/tBmePjuGYOciADDihPHGH5c0Gco8cpgiM5L0+Ez8gtlkDeFGwwQQHCDP/xi2H370pPv"
    "jL+fM8hxPgWG5XcGRh5mhh/v/jP8ZWJi+P7/H4OltCfD+RuvGLbfOM7w8aQqQ5JjAcO5x98Y"
    "vr3/wmCX3t7AcHEDB8w8gACCGwwMjkfXrvw+wfDsNoO59BuGS7eeMzALszF8evWX4cdfRoYv"
    "wAhUkpNl4H1jznBjnRRDinMZwx92boYnn/8yiMvxMsj/eK0itqsR7mSAAAIb/OoXhHP/EcOG"
    "T9dvMhhKf2Z4+fgWw08uZoZ//xgYfgLzyPffjAwffvxhCLVIZ6iO6mIQlRdn4ONnBRYeQEcB"
    "5fgZrjOkyjFkwgwGCCCwwb+hBdyXvwzbbpx59IuN+yODBNMthhtPvjH852Vl+Pn+LwMzJzPD"
    "X0YmBg5+AQY2fjGGt0CXMgBLJA1FfoYDV38zCAq9Z0gK4PEXZWDQBJkFEEBggxmhtgDpO9dP"
    "fTzL8PUlg7nUE4ZzV54y8EqzM7x68ovh3af/DD/+MDGwswMTjAADAzvQoitPvjKsPHaHYev+"
    "rQymwu8ZlLz0udyEWNNAZgEEEAvM6f+g9MOb/zd+vffY0kqZm2HZ5ssMf71UGcRluBg4eJgY"
    "Xnz8w3Dy+keGm/efM/z4fJ9BkfMeg53UK4YKz+8MAgLMDD/PSTG8+s4ADliAAGKBuRhW3H/5"
    "ybDl5qkHrUZZhswCv28wbNr9nuHzt08M9zYBi2DGlwwGEk8ZMpUfA9P1KwZG5jfA3PUOnFb/"
    "nX3LsG/Vu9/HvzOAMwpAADGC8nUFIyMwkwIjCBoi+vasZ6M3hxteuqzDMH3dAwZrjecMjqpv"
    "GKT5PgIN+szw6+EXhke3vzPcu/Prz90H/17dfvb/wb23DDdPf/2//RkDwxqgmf8BAghssB/Q"
    "YGOgiSA//AZiXm6m8tQFkh2iSuwMn+5+AabAnwzXr/x+f/XW72c3H/+9c/ftv+tP/jJce8vA"
    "cAuYfx4BtbyGpFgGBlghBBBAYIM5gAaLQwXDgXgVAwOXAhtjPT8nk/zdj38vA11x5T0wYkEZ"
    "FIg/EqrvQGYCBBAjrSpTgAADANIrjEmyTHLrAAAAAElFTkSuQmCC")
index.append('markcompleted22x22')
catalog['markcompleted22x22'] = markcompleted22x22

#----------------------------------------------------------------------
fileopen_red16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9kJEAgRKMyTCPsAAAIH"
    "SURBVDjLlZLLShxBFIa/nimNsRcZL+BIjMG9wdmYh1Awr5BdfIVA3GTjLouATCCrIO4TfIkx"
    "IHjbhIFInAEv0z10vExPV52qyqLHCV4I5sChNuf/6j+XwInwr5hWavs1jMdwo7AcBIOfvX/L"
    "FLx6Dv52lmAZGNuoVJwT8Vkce50kXieJz+LYOxE/Cr9UEb4e3uNiRqmqhqdRo5Eme3vD7d1d"
    "CgMDAEinw9TCAg4yBfBzff0O4FCEGaXeteKY+sYGnSiioBQA2fk5ABYCBXDZbPaFQaGAd469"
    "1VUORfgwMkI4OUkniggKBQCKg4M8npgAQAGcHx1xsr/PZq1208bKCi/CkLG5ObxzfYCkKaOV"
    "CkDu6XejwWatxpd7ZvFpeprS7Czm4oKgWATAGcNQuQzwSAFcnJ0B8KNazXvLMpwxWK0ZCkM6"
    "zSZTS0t34BbCvIVWC4Cr4+P8BxFslmG15n29zsr8fF90CYS9twNxDmi3c8DpaU7WGieCdLus"
    "LS5i0hRrDKI1a1tbvKlU+LizwxZ8UwDJ1VVOPznJHVibA7IMawym28UagxVBA1mvvr+F1rW9"
    "KOL6tJ21iNZYY3DWYkWwIhigexvQBoaByzjGO4f3vi/y3iNa45zLVwhkafr3bkqw/ASqZR4W"
    "deAZcADbFjYDYAx42csSD48E+B4AxZ5wHBj6D0AXiP4AYdojwWu+hAUAAAAASUVORK5CYII=")
index.append('fileopen_red16x16')
catalog['fileopen_red16x16'] = fileopen_red16x16

#----------------------------------------------------------------------
arrows_looped_blue_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAWmSURBVHjaYvz//z8DCDCazALT"
    "DH+BfCZGIAay/wHZjEA2kGJhYTzGwcRk8OXD9wkMrEx1QDV/wBJMzBC1P/4yAMUZ/p/PABsD"
    "EEBMDPgAyM4//4D4LxsbO4t6d5cbZ1CITiXQ8icMP/8kgi3GAQACCLvBjGDMBNQoDjRYluHP"
    "f/lf3/785xPlZ0gudWCo6QoSV9GTm8fw+/8lhl+/bcC+YkQ1AiCAWLA4kwXorWpgOCRw8bIJ"
    "c3KwMv8DBg87Kwvnref/Gf59Y2DgUpBkiCv3YTh35L7uie1nD7548PIAw3/GFAZmpvswUwAC"
    "iBEljP8ziDH8+HlQVUdKw8xVl4FPXJSBhZUVHCIgrzGxcTG8/MYECXNWBgZOLmCUfP3J8PLi"
    "DYYLBy//fPryy6x/5zLyQOYBBBALiut//jyoZq2roe1lzfDqNyvDo09Aw6CBBfItL5B++x0S"
    "t1D3MDCzszOIGOoz6HJwsL9ZcTAdKAQ2GCCAEAb//NUkpCADNNSO4ccfJgYOYGT/+PiO4fP7"
    "z+DwYwaaJqIkxcDHBdQCjE82DqBL2IAWPXnPcHjzeYbXN+68YGT+XwwzDiCAEAYzMkVauOkx"
    "aMoD4wyo8fiWiwzHNxz/9efv31dArzOxsLNImNSHMSlJCALDm4Hh/ZtvDKf2XGG4eODy15+f"
    "vkxnYGUt/c+ESAsAAQQ3mJeXQ8rCRIBBWYaB4d6ddwynt5769+fPPx8GDtbdwOTFxsjC8kRN"
    "nlVUUOQfw86N1xl2rTn/5+2Tt6uBthQwsLO9QiQnCAAIILjB33///Tunaw8DGxvL/y9ffzJ+"
    "+/3vFQM7y25wzAHTKysbE/PDa48Z5m27+f/yiUengHIpDFzsV8Deg2UqpCQHEEBwg//8++f5"
    "6MYrMYbf//4ysAHjnJv1AVghKEexMP369v334sntB+z//vjTwcDDthJsCsjnf4H033+QzISU"
    "FAACCDlVHATGECQZsDBBov0fNFv/+mPI8ItB4y87y0EGbvaVDIzQrM7AoAZ0sTzQ4D8M/5lY"
    "gIa/A4qdBUkABBALSvZlZYYwmKCu+PVHgOHXv8kyqiIRrr4aLKtXXbH58v57C9DiN2A9//5v"
    "5hflVuHmZv//5+8/xndvv31jgKRKBoAAYkHJxyAe2JUMHAw/fley83MV23tqcwfFGIKDcvnS"
    "S/8YmIGyoCD4/VeS9e8/ueQiLyYDU1mGZ48/MPSUb+aEmQYQQKhZGuztX0lMLCzN6vbaUjbe"
    "RgyyioIMT78wMHx8/onh749f3Aw/f58A+ug30HhZXTd9Di4pWYZ7rxkYbt3+zvD+47ePMKMA"
    "AgjJ4P9WwEJlOp+8vJ62kwmDrKY0wwegK1/cgHiGi5GTwSDAienPn/+qoGKAk5+HQQKYYS4+"
    "YGDgAGbt41uvMPz9+XsvzDSAAIIbzMb8f4dNhCUvt54Bw0dgZnvyEhrujBCPCHECE4qqKjjo"
    "wZ77B8negtz/GW4fuMDw5MrtN8A0nQ8zDyCA4Ab//s246t7JG3GGXCysqhrqDP85WBm+fQUa"
    "8BcS7BJcwLTM+gto8H9wgvj35z/D59dvGS5vv8Jw6djdtwwszD5Aiecw8wACCFG6Gc4Eptnf"
    "WgyM/2bLqEpZWHiaMumYyQLzMhPD1x8MDOzfvzCsnbQFWPj9/sUETJZfv//5//zV1y//f/3d"
    "xcDBlsXAzPgB5MP/Z9LA5gEEEGrksTJdA2Y96yd3XritmbRl2hUjBeWwWFMGCzMxhs/AoLn1"
    "6PPX3x9+eADD5g0w7BiAmegRAzPzN2x1BUAAMWFURSAfsLPsAiYrlRtn7+U1l6x9t3zyEYb/"
    "n78wcHADSx8ulhsMHEDMzAiK1m+4qiaAAEINij9/GcAuAcUWKHYYwRmFC1g1tQmIcGd8/f33"
    "ye/ff7WAEr/AsQjLoWAnQipdWFAABBgAhGIfg4WD5MwAAAAASUVORK5CYII=")
index.append('arrows_looped_blue_icon22x22')
catalog['arrows_looped_blue_icon22x22'] = arrows_looped_blue_icon22x22

#----------------------------------------------------------------------
new32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAbTSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsyBxGvURVfjGpqZJySuaMDP//AIWoEzyMIMTI+PffP6Ynd2/t+vrhTR4DE9NLkBRA"
    "AKE4gEtApLG1Jts1yEqK4ecvMmz/D9HzH8GFiP2HOIIFaNv8HTfD2jsmP/nz7XMxAzMzA0AA"
    "oThAUEJSx0lXikGSDchho25QgxzBCHSEnZ4MwwQ+Qc3P378yMDCzMAAEEIoD/v/7/+/XX6hz"
    "SQT/YL5HppFCBMTmYmVg+PX7H5D//y/YCiAGCCAmzEAkz3cwS9ANQhb/D3UoAxIGCCAWSoL1"
    "H9Tgf8jx/x9h4T+oGpBn//3D7juAACI7G2IkNDTf/kNmoyVOZAAQQExkW/4fM3j//8O0HDlE"
    "YA5ghMY/CAMEEAs5lv9HS3AMWCxHj44/QIlffxkYHnxgYPjyE2EYQACxUOLz/3h8Dsv/v4GW"
    "AhM+wx+QODMDw6efCPUgABBALJT4HD3BIYuDfPwTWJaCcjVyImVkQM3lAAHEQg3L/6PF+S+g"
    "xb/+oSVEaE5ABwABxEKJ5f/R5YEW/IAGOSwK/v1HOOTff0TWhQGAAGIiK86xFDIg332HWQ7L"
    "/1xAzA+xBTnBIgOAAGLCW7oB8V90n/+D+gapIAJZ+h0a32A5kCSw2H2zYzfDs/6JDL/efmD4"
    "z4xWGDEygzFAAJEWAlgKHFDW+vEbYiEzN9CznEA2OwPD+/PXGX6vz2WQY+hm+LByKsN/YOX2"
    "HxjhPzmBCpnY/0n/eM8gA8QAAcSEq4T7+x9/nIPkQan8J9ABjEALP9y8xXC+JJfh0ao1DMxA"
    "/s+njxn41IQYJGJdGVh+PGX4C7T38+M3DJrz2xiCnhzk9Pxwl8Hr3S0GgABiIZgAsVQmoCwG"
    "yt/gIhaYpf4B2XdmNjKYB/ExPNzex/Dgz28GdjFlSHb79ZOBkVuY4cfDbwy/WuIYgrIvMXir"
    "sNjdLn9kwcjIeAIggJgIWg5NVH+ghQrI17/+IlL0/7+QfM0qJMvw7TMTg1F3KwPvnakMj+bM"
    "AbZAOBgYONgZvly+z/C9OZpBq/AxA4euNMPj879e3mPgfn6fhZsBIICY0Gu3P38RbJDPwEEN"
    "tRiU2P79RyRCWNZiBAavRmULw42Dnxmeb1nGoF2bzyDJsIPh/5e3QFWcDNwvdzJo579h4NKX"
    "YDhSysgQdcLuwk8Wloeyf74xAAQQShS8ATZSHn1iYDCRhDjkLzTV/4MFPzQV/0OuXln+M7w6"
    "f4/hz/cvDLyeJQwHWnMYLJ6/YDCbX8jw5/FpsBr9uU4Mfxn/M9wIuMywib2Z4T3nWV6+P58V"
    "ORj+PgcIIBQHwHwLT4Qwy5Cy4j9otIAwKJ/fXb2M4fupRQyCkhLAVtxvBhU7EYavd+4y/Pv8"
    "koFFyxao+hkwhv4zfNp4koFJmZ0hRWoyQ4rmaxtuFonLnzd82QYQQCxojVcGRqTU/g9aDjBA"
    "Q+EvUo4A8RmBxPvLexkso80YRE39gdXcfWCI/AAmEmCV9+cjKAVCiprffxgEHGQZBHy/AzU/"
    "B+ZXdmCkcbA+/8vmCBBALLgS379/qC0euM//I/iMQBfJ+BcznFrVxsC+up6BjYOP4fPlewxi"
    "zDcYTGdmA1XwAF0NdAQrM8PHPfcZ7kz5yPBcx5Hh072Lr+0+3jv/6trPNQABxIKtBISX3wxI"
    "bLQSEFwWAKtWAV1tBj7NpQz/gAXN61O3GJjvJDFo1ZUwfH/2ieHn0Y0M/BaqDF/OnWPgD9Zi"
    "UHh6jWHNYQOGjveax7oeLw7w/HCVASCAcDoARiMXSP+wVS4/oAXRpZsM3xfFMzh1xwAlfzDs"
    "8uhhUA5RZxCwY2Y4X3OPQeXxTwapPAmGXI6pDO+bhTn537xm+AbUDxBAKNmQkZGJiYWZkYEF"
    "1FyGYhagCqAQBDNBMZI8sG8BdtGrhbUMlqUxDEy/OBhONW5iYAztZ2AVAJbNv74wMLvHMty6"
    "HcXwdt5tBqk0UYbWrBd2ej9emYIaRgABhJoLfnx4fe3lV4bHitwM334iZTskXyNHETyNAJPU"
    "FwkThocbVjLcv8nN8D9rOQPjkwcM/2/9BCv6CwyhL4WVDMe72RgMu2YwfLry6xmw+ngNSvQA"
    "AYQaBR8fT5g4banGkq060sAi7j+hFjGiTmUCJkgLBp7zXxi+yxsw/Nz+iUHixBmGRbb/GBk4"
    "WRh2n37KMLvr4n9OBltG1mWPbljf3ZeUyvD8AShXAQQQWhpg3PLx5lGrjzeOKDPgbknj6IAC"
    "/cMBTPUXtzIwnNvyj+HdM9b7b27Ust99oPfytFDDy4fTzzD8/8rGwCd2w5Cf/yXTF0i2Bggw"
    "AJIalOSPBHBZAAAAAElFTkSuQmCC")
index.append('new32x32')
catalog['new32x32'] = new32x32

#----------------------------------------------------------------------
clock_menu_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9gGARIFCGlLrU4AAAoZ"
    "SURBVFjDnZdbbF1Vesd/a1/P8Tknjo/vMXawYyd2HNs4ZAyBhEwzMOoAEwYSSiYaZjSFDgIx"
    "D33pU6Wq6gOVqkqdUatCKyEypCiTgZApwwyXBMdpSEImTC72xI5jO45vcYzt48u57LP3Xmv1"
    "wSYNbSCon7S1tdbWXt9vrfX//lpLsPEAN8IUYFvgWGAZS49tgtLgGC6hbiPUD2CIBss161xL"
    "FAsNeaXnAi8cRnEZk4+xzD+gdRqlIZAQKpAK8uFSW6obKS2+KgQgdRxf7jZM45m11bG2e9YX"
    "R9sbk1QUR4m6BgKBFyg+S3mcv5TiZM9M2DeWvhBkgn3YxusIpr86xZetgG2Ar35QELf/9sHN"
    "lXU/ePhO6mpWoIH5VMDMVJowFAghECiSZQUUJV1MC8YmMvzyg6v8tmt8fG42/xIR4xVCFd5q"
    "BW4NYJsxfPmPd7eWPvf8081saFrJlSs+gz1jkE5jBh4FlmB2ZpTAz1NZVU86rwltF10Qp6ax"
    "ksbGAgaHFnnlPy5y7PeTb2EaP0XKa/i3A7DMEgL1yz07123/4Z5mZhbgzOFLJLIzdLTEWbc2"
    "SSzuUriykCNH3mduLsXOnbtZWFgkm/EYGpzjk/MLTOkVtH+rkepKi7ffGeDl17ovSMEupLp8"
    "M4DJqieX9nqJJ44XHtzzZOP2p77fwpnziwwe6SWxeILmxhzf/tMHsGwb07JwI1GGhobIeT4b"
    "WtqQShGJutTWlSGDPsb/eILPxgoYzkTZuq2KkoRdfub0xBateRPIIZY0ZlK6c0nlYJL1f75r"
    "Z9OuHbvb6OyaQV/s4fndlWy6p57OzqM0NjaTSMQwTQMhoKyshDvvrMV1XQxDYFk2+Xyed999"
    "lz976lG2bizlfNclzlyx2fLgaooiZuXZUyPNWOZbSC1RGpPCx5bKJOvvbmkr/7vv/2SzcfzT"
    "DPHBbl744WqSpUVEIjHa2u4iHo9hWSamaWKagng8TjweB/QNKCFgw4YWioqKsVyH9uYEg2cG"
    "OT0R495tNcyOpdaOXZpOo/XHhAoLqUDpWMQx/n7H7o3m2asCebGfnz5bS2FJIVqDZQlisaLl"
    "RALDMACBWN462zYBlvu/PPYBL32Qpr978q9nZrx9WMa4QSBhMb/73vtW14jyEnqPXabK+ITZ"
    "hVlAUFAQxTQtQN+YvRD/kxxYbgu01l8JsO353zNtxvjmQw1xMvkXCCQG+dCxbeMvNj64gRPn"
    "AtpjU7S1lfLe+5/wzjsfMTk5STRq4zgWhiG4XXwZxC9e+3ceqg85/alH3b1NxAud3eT8UoNM"
    "vr2qakXrfEGS6f4xHrkvwcM7HuWRR7aSzwfs3/8e7733X6RSc3zd+N8Qo6PXeHzX0/zJ3YXM"
    "XRpiQkepX1Ncy2L+AYtMsLW2qTJ6aRrusBZpWpcklwtYu3YN69c3cPZsDydOdNPbO8zmza20"
    "tzcSiUS+FoQQglzORylNLufTUL+SDclx+kY1tRtrxbkjA1sNtFybrF/F2ISkJuYRjbugIQwD"
    "pFR0dLTxzDPfo7FxNV1dn/LGG7+jr2/wi9k6P4IL5wFQSpJKzdPd3c+HHx5HKYnWCtDYEZf6"
    "ooDx0SxueSmOLdZYEVfUymiCycE0d64TJFYUIrFRKsQwloSVSMR5+OFtbNq0gcOHT3LwYCcN"
    "DZdZf9d6rOEhEgcOkH3hRQbeP861iWlSqUVAs3JlAR0dIY7jEos5WIZk3eo48wM5MndEiMfM"
    "VZbrusWetvA8wVxqlMNHrhEqi46Ob1BaWoy4Se5lZcXs2fMovb2DHOk8TarrBFf7+zHKGyg7"
    "eRHXElRUJGlvb6C6upJkciVaC+bnFzl58iSoPNOfSSxZxXxoYjlO3EI4QmFgCgh8j7mUT6Bs"
    "pJTc5NFfiKamNRTEouz/xQGSlRVMfZbioe3trFtX/wXgz7WgtWZubg4lPTzPxjRAaQONg5UP"
    "zZSpFIkCTVVVA7t2tRJqG60DwlBi2+b/GRQpqdy/j51b78OvqmLu+jR1datvkRyklEQiUZ54"
    "4klMEfL2ry9hDikKDEUQmFnLC8xhlc5RU1VGKqNZWFgE08VxLEzTuJW8kS+/TNDQQP22LUt9"
    "9bVfVgtoDUopPC+L0D5Xx9OUljg4QUA2LyYNtD0wOzRNRYXNdS9CLusBSyWklEapL9a0+tWv"
    "yJkm0ccfv20pLv2vbrik7/lMZxxWVcfJTs7h+8aQgeseH+ud9ctjMEOMocE5XNcln/cJQ4lS"
    "is99RXd1kR0dJfLccxhfw5CUUkip8Lw8kajLyPA8Q4sRVpUajJy7Bqb7sUU08ofr17w+OZtt"
    "TdZWcub8ObLBR5w9e54f//hZXKeQUIDd10fu1CncF1/EEuI2JgRSLoEHQcjeva+ypq6a2Zlq"
    "ihubyGdCxq6kx4i6xwwsN63y7O3uHKGipoCxYAXHj/bw2GM7iLhRPARq8jrZQ4cwfvQj7Fjs"
    "tjOXUhKGEt8PAYMdOx7jYs8o50YkFXUJLp+awJtXv8aJjBmYNkQje3tPT08N92Wpu6+RwrLt"
    "RCMrkYZAL6QJ9u8nfHwnVkXF8sw0N9u91kvlJqUiCEKU0gRBSBBIpJTEonEKCrdSd/9dTI37"
    "nDs6kcN1f4ZpsXSKsIyZ0NN/c/jAMBksSlrreevgILnZFPab+0nftQlVW4uXzuH7Ib4vCcMQ"
    "KRVSasJwKdnSt5BcziMMFUpJwiDPoUP9RKrvgMIov3vjKpnZ8GdEnMsYDiYlz4Npg+tcSE/5"
    "LXOe2bj5WyUkIi5n//MMVeuK4Bt38/orL1Nfv5ZoNLpsUppMxsPzPEzTJAyXViafz/Paa69S"
    "U1MN2uTtg/1ESitJtlbz+r4Zzh2+dpqo9SzC9DFMTIpfAGGCEBLbOjpxOfudsQWr7NuPlFFR"
    "U8LRXjh/qpOSEodNmzrIez6hVLhuhGPHuujp6Wb9+lY8L08YSqLRCLOz1/nwg9NcHVlJbctq"
    "Slqr+adX5/nozbERLPEoqCnQy6fi5E9uOtoYGWzrw4n+3Oa+SWPVxnuL2d5RxOQoyKCExXSO"
    "WMLAMMBxHAYGeslkMjQ3N+N5Htlsjp6eSSYmHBx3Fdt3tDBbUMRL/7pA58HJIQz9FEL/ES1v"
    "uhfUf3oTgAmWDcIpwRevJmvj3332mRL+/DsmsWyOU8eukJrPks8HlJbGGJ8YIO/lWLOmlamp"
    "NLZlsSIR5Z77a7DKV7D3iOZf/m2a630LJ7H00yh/EBWA9PlcxbcGMB0wbRdf/CWu/Vfr22NF"
    "e74X54ktBpUxzez4PLlsHkMYIEAqhes6lKxaQUqa/OY07D2U4dypxYzKBj/H0f9AGKQI83x9"
    "AMNe1oZVjS+ewzWfLCy317ZsiNBU77C6QpBMgCEglYGR65q+AZ8L3XlmJoNhcuFvsPU/I+Ql"
    "tIQw4P8HYC6/EeVI7keyBSHWYIgqDBIs3U3TKD2J0oMYnMDiY1AjKAU65KsA/huQJeRcEsh6"
    "QgAAAABJRU5ErkJggg==")
index.append('clock_menu_icon32x32')
catalog['clock_menu_icon32x32'] = clock_menu_icon32x32

#----------------------------------------------------------------------
clock_icon128x128 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAIAAAACACAYAAADDPmHLAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAFMdSURBVHjaYvz//z/DKBi5ACCA"
    "mEaDYGQDgAAaTQAjHAAE0GgCGOEAIIBGE8AIBwABNJoARjgACKDRBDDCAUAAjSaAEQ4AAoiF"
    "kAJGRsah5SOdZcwMDL+EGBj/qwN5mgxMDAoMDMzSDExMUkDPCDMwMvECaU4GJhZ+kPcYQN5j"
    "AhH/3gHV/gTq+wRkf2Rg/PecgeHPM4b/fx4z/P12k+HHq+sM7y68Yni46RdQ8ZAbPME13gMQ"
    "QIyEBoIGfQJQW8zCwPgHGMkMdsDIM2VgZAZiFlWGf/95GP4Bo/Qv0H9swIKOnZWBCUizsDIx"
    "MDEzMbABaXDEg/wHRL+A6v4Blf75B4z+v0DiN4gByiJAdayM/xiY/39j+P/rPsP/n+cZ/v04"
    "x/D9xSGGh1tuMrw88X0oJAhc8QwQgJqySUEQjKLoscj+lCgKJ2LUrAW0hWbtsVXUzEm0iqAg"
    "pBAqB6Z+ambPRm3BBxcu3OE5vHoKMNvoaPlSAK4pGyshuSCn89uMFn1DZ+6YTG0TxzawJj2s"
    "cZfhoI0pWwV/JP3/nmFGlhWE0ZsgTPEDyUPh3RPOt5jT9UWkClBiRVOrpEgpkyOfeE/ib7ns"
    "DniufA+KOgnwFUBDKwHIztIF5ukIoKv8Gf4CI/3XfyYGLhYGUUluBgNtEQYDPVEGXU1hBiVF"
    "fnBkg3I7GzCe/4Ey9B9g7gZGDShzg7z8By2aWJghhQGwcIBkehZIAfH7N0jvf4b3n34y3Hvy"
    "heHy/Y8MF24B8d2PDK+BiYThF9BAdmAJwfDzPsOfTzsZPt9fzXCy4RTQyEFVMuCKZ4AAGvwJ"
    "QHI2MzC3+wKL9mSGf8Dc/usfGwMnK4OiigCDvbU0g7WlFIOujiiDgAA7AysrMNSBNfT3n4jI"
    "Bkc4uLKHl/YIGhY4MPwflQbJgxIEOFEAEwgnGwPIanBi+vD5N8Plu+8Zjl5+x3Dw0huG+8++"
    "QqoNNqY/DH8/HGf49nwxw4WJaxk+PQCVCn8GawIACKDBmwBEZ3IwMP6NAjbe0hh+M5ozAHMh"
    "nxQ3g5ODLIO3lxKDkaEkA78AC8MPYNB+/gbN4f8gMQmLNA5gpLGBMAukGQASY2WGRCwolyMn"
    "gD/QhPIbmnBAGfvXH1DbAFjqIyUmkCYWqDm8nEA7gGZ//PKX4dytdwxbj79g2HfxLcOndz+A"
    "CoCK/n+5zvAdmBBuLFvI8PzYq4FMCLjiGSCABmcCEJocDmylVwGLeD1QrCgAi/XQMA0GD08l"
    "Bhlgnf4TGCEfv0IiCFS8gyIWFNG8wBzKBcyh3KyQnMoKjXQmqBdAjTyQd//9xyybYZ0BkHeR"
    "1cPag9+BVcFXIP4GxMDmAjhhgOSYmCAJjB+YGNiBbnjy+jvDjlMvGVYffMrw4AnQkcwgw7/f"
    "Z/hwewrD4dIFQN6AlAi44hkggAZXAuCfaA9sotcDG3SOoNa7hqEEQ2yiLoOrhzIDLz8LsHEG"
    "jIAfkNwKilhQJAtwQDCwbQeOAJBz/0Aj7QcwskAlBCg3g3I2CIMi9fc/7AkA1jEA5W4QBpUa"
    "oBzOAU1MoJwPCi5QAvwCTAQffkAwKHGAEgNIngvY5hDkBpVKfxl2n3nJsHj3I4Yb9z9BGo5/"
    "P1xgeHu1h+FE42agdV8YIP2MAU0AAAE0OBIAT58ksI5vANbxycDQZZbVEGZIyTJi8PZXZ2Dj"
    "ZmZ4+xFSr4NcCirWhbiANQQQ87FDcj6opweKBGA7jeELKKeC2gF/EDn9PyOigcfEDMm16N4G"
    "yf8DdwERDUXG/4iSgROYELjZwJ0MsL2gxMfMCCkJQPa+BlZD775BqguQHlB7QZgXKA9MfZuP"
    "PWOYt/0hw2NQO4GV6R/Dt0fbGB7saGK4vfoKUOkPejQWccUzQADiyx6HgCiKwofCPJP5ySBj"
    "ARbAXtSimnWoFSxBInYheq01UA1CXkzmLxOc62kUSjH1FPfd851z7/0/APZ8yFZOkd17VqAw"
    "ivoYRwN47Sa3bLPUvRhhQ7sOEDom4kV0Xmu4cNfWbOHt7UKIU2XmN4youYhCGvK0gqZC13OK"
    "RBeo1T/f9SAtDi+HoGPDJ2GK14Wi6so2MJSso6qMZwUklzX4TJ4Wo9+1DAwyIo70dZyYhJBP"
    "QAk9GVkllus9VpsDCtaCehbjtJthO1nwN/3rsfBN56cAGrgEwDaBB5jFOhj+MmaDmuymTgoM"
    "pfU2DBr6YgxvPgAj9CskW/ABA1maDxL57CyQ3A3KbSD86QckITCDimkOSO79/PEPw7MHHxhu"
    "XX7BcPvKK4bnDz8wvHv1Cdgw+wa07j+we8fIwMXFhFTwM8Az4Ldv/8Bdvv9AP/MBE4EQMOYk"
    "5QUYVHXEGFR1JRikFQTAVRGoFPkBsvsPJOJBbhSFlkqgUuLnH0giePoJ4kaQLaAGowjQHzce"
    "fWLoXn2H4fSlt5B65fPdLQxX5lQwvDx9B6jsJ70TAEAAYsxdBWEgCqIDAUniE1TwHcVSEESE"
    "dH6HtZUfIPgR9jb29nbaiJ2NjWCpxPgsggoagwaJ97K1pVhsu7vszJy57H8MIHXLxOE+lXJN"
    "CSlotXU0mhW8KL6ceu5wTnk2AmQi4p045fubSBjjXSIRZEWIfqJhazEzMZ8aWC2PsC824lEf"
    "tKyMfCEITQsglVYRi8lQaWM2AeNBksRP+JuVJBO4rocHxdiynjgQrjfGHQYtc+vAOrtQwiqK"
    "pSSq9TzKeg6JjF+YweE9RE0wodJBQQeeNXZk5u1V0IFnhHiIa8vDYGKiN1zDsels77rGZtwh"
    "I4zoIrdfVMI3nT8CiP4JgLEjDBil0xj+/xFW1ZJkqO13ZdA1k2B4+grSwAMFkjQ/A4OSECQ3"
    "vQUGLjATgyMe1JgDFe3swCL3/evfDGcP3WM4tPUmw+3zTxj4gQ0vbS0+BgtLMQYdXUEGMTEu"
    "Bj4+NqB6ZmgAMELrfUZIewAY+efOnQTLGRmZQxMBJKDA4wTABgATsJr4AWxFfgY2+1+/+s5w"
    "6dI7hhPHXzFcvfYJ3AtRNZRhsPNWZzC2U2IQFGVl+PkTUlWAGo+ghAAsPBiANRm41Lr3DpiA"
    "P0ISN6ihKA30H2hQqXnxDYbb94ASrH+/AruK3QynWqaDBiapXSXgimeAAKJzAuiqAJrYCswv"
    "TJ7Begxl7Y4MLDwcDM9fQwJGEBhY6qLA4p4XUoeCAu3xR0hDiwMYaCzAUuHJvc8Me9ddYTi6"
    "9So465mbCTO4e0gzaOsIMQgLcwDrf2ZgZDICI48JGpFM4IgE+QMZs7KyMixZMhfsqpiYZGD9"
    "/hvYCPwPDihkjBD7B248/gFWV++ADY+rV94z7NzxhOHEKWBRzs7JYO2tzeAcpMMgo8TL8AfU"
    "+/gJaaDKQhMzqA3zEpi3bwL9+v47JKFLCoIGrH4zdK24xbD96DPQ8PJ/hjfnFzEcLqsDOus5"
    "aFiC1gkAIIDolwCYevuAzfJCUOmWVWHDkFBowfAGWEe++whplSsAA0NNHFLcgyL+zltIzmFl"
    "g9TvD29+YNg4/wzDqV03GJRkWBj8/OUZHBykGCSluIG5mQkYUZCIZgbWDSA+JNKRI58BhWZh"
    "ASWA+WCnxcYmAov/39CIhrQJ/v1jQEkE/4ACf4GtTBANwkxM/4H8vwzPn39lOLD/GcPGDQ8Z"
    "7j35zWDmpsngn2jCIK8uAG4n/P4FKclUhCEJAZTQbwJLuwfvIL0UIR5I22DBzvsM0zbcg7Q4"
    "P17fynAgLx/okMdA/IuWCQAggGifAFimsjH8/z0HGFqxbMBOdW2vK4N7uCbDI2D6/voD0krW"
    "lgAWl4KQFv0loPiLL5BEwQVsVL179ZNh8/xTDIfWXWRQlGFliI1TYbCzk2TgA7bYIZHOBI10"
    "SMQzMzNizfGwrh6kCgCVACwMCxbMAYsnJKQAE8AfaCD9h3cRsZUGf//CEsNfYGnwDyj+F5wY"
    "PgH7gocOPWdYuvgOw52HvxjsgvQZfBPNgA1JdmDjEtLFlABGtp4kpOfwENjWufYCMrDEDSzd"
    "5MQYGHaefsHQvPA6eFKK4eu9owwnGjIZvj6/RY3GIa54Bggg2iYAtulsDP9+LwQm+wheAXaG"
    "tpleDMaOcsCcAvQR0OPCwAgGVqMMglyQovHqS0h/npMT0nI+suUGw/rpR4AB9oshNVWTwdZO"
    "goGXFxTxkAYcqLhnYYElAlixjxzxsAj/j9Lqh5QAqAngz58/SIH0H95ugCUI9MQAKg1AiQGS"
    "EP6CaUbGfwxfgcXWwYPPGWbPug5M0GwMgZk2DDY+GmATv3+HJnhxSFX3AZgwzgHD4u03SA9H"
    "CSh+9vZbhqrZVxg+gwY0vt0/xXCsLp3h24sb0PECqicAgACicQKYshjYcovhFQRG/iwvBj0b"
    "GYb7TyDj9qCGnokcJKjPAMXuv4PmemBj7sXDTwyL2vczPL50jyExXpkhNFyZgYeHCZhrOYFq"
    "mIE0MzjyIUU9opiH0YipHOx+gJUA8+bNBvOTklKRSgCUYEMaMIKYCSkJGMBtAkRCgGCQGf+A"
    "3ZJfv34AE8JfhlUr7jEsWHiHQUZPiSGu0pFBQp4P2MuAlAaKwOrARBpSKp15BGkggkYfFYGl"
    "4SVgHVg16zIkEXy9e4Jhb0YK0HKKuom44hkggGi3JIx9ej/DL0Zg5HMytAJzvraVDMPth5Cc"
    "Lwcs7i0VIA293bcguZ8RNMECrOsPb77N0Bq/lIHj62uGmTOtGLJy9IE55wPDwoVzGD5+fAds"
    "2XMDewGg1j0rOBIhCQGSGBB1PKIkgCQKBpREgtouYMAQh+lBLU2Y4GKg0gZmL8gN7OwQzMvL"
    "zfDhwzugW2cD3fyeITtXj2HWLGuwX0B+AvmNjQPiV5Cfd9+GhAEoLEBhAho/uP0UWEIoCDG0"
    "puoy8IKGHXmULRjs+iaAVkGAshS1owkggGiTADhnlDP8Zixg42JlaJrhwaAFjXzQ5I0KsOgz"
    "l4cMkuwERv5zYMsYNGf/B1icLus+zLCobhNDXLgUwwxgwOnpiYKLeykpKXAEHz68FxzgbGws"
    "8IYe9shCjlQmeNWA3D4AiYGKbhCGyUMakQh5WOmCmmgQCQemHpIQIAny0KF9YH1SUtLAUoGJ"
    "QRfoh5mzgY3eCCmGJQ2bGZYA/QjyK8jPIL+DwgAUFqAwAYUNqPoHJQItYCJoStaFrFwS0nNh"
    "sGxqBK2IAE1ZUDOqAAKI+gmAa2YIw1/mFkZg3VzR48Sgb43I+WpADxoD6/xbryE5/yOwTmQH"
    "5oiP734wTMxbz3BjzzmGiRNNGTIytYHtAA5gwLKCc7qwsBCDt3cAw5UrFxmuX78CloO18mGN"
    "OliEwCIOuU2AGnmQSAUV4fb2DmAM6eIxYik5GNGqGCa4HZBqBTxTAGZzALsqN29eY7h69SKD"
    "r28A0M3C4FIB5AeQezOytBkmAP12c+85sF9Bfgb7/TskLEBhAgobUBjBSgJ9ZSGGihgtSGUm"
    "bhnCoJ+dA1ohAWpaUyu6AAKIugmAe642w3/W2cB6nyW52IzBwV+Z4cYDYOsFWMwpALtBxsA6"
    "/zqwC7TvDqT1CyoOnwPr+wkZqxg4vrxmmDHThsHeQRrsPzY2NmCgsgIDEZTg/zLo6+sDSwQj"
    "hq1bNwBb1d/AuQ45VyMiCrm4R7AhOZsJLAbigyJdV1cXjEFsmHqQGkQpwAhXj9qVZISrg3U9"
    "v3//xrB58zqgOw2B7tQH9w5AiRfkBxD9/z8LMLFJA6s1W7BfQX4G+R2UCEBhAQoTUNiAwggU"
    "VqCZzBvATqCDoQRDsq8ysIj8z8Sg4J3JoODpD5owR2nkUAAAAoh6CYBnMbCTw7SQ4ftfAecw"
    "DYaIDH1wzgd19UDDuVYKEA/uvQ1J4aD6/tn9TwzT89cA+/9/GGbMtmZQUhYEBxQHByTyQcUq"
    "rJUPwt7evgyvXr1iOHPmJDiBYMuliFzMBM/tsNyNWiowgVv+IIwshkgojEilAvK4AhNGewHU"
    "Jjl79iTD69evGLy8fOFqQI1VSBsBlBDYwX4D+XHmHBuwn0F+f3rvIwMrO2SKGRQ2oDAChRUo"
    "zL4Cm3y3gQ3kCCcFBmdLKWAiYGZn0IypZOCVMwFNL1Aj2gACiHq9AN6FUxi+/ctW1BZi6F/q"
    "wfDuBxt4XB80E+ahBen3br8BGfzgBHb7Xj/6xDCnaA2DvTEbQ1WtEbABxQHu2oECCxRokD49"
    "E1LjCzJ8e+PGNQYREREGMTFxcM5FrgJgw7wIJyNyLXh6l5EBaTwAW0sZe8sZQaOKg8YFwLkI"
    "GNkvX75kePPmDYOGhhZ4WBl5QAk0bgBpb/xh+AlM/SD5L19+MLQ1nWfYc/wrQ9qkcAYxBT6G"
    "718hU8+eGpBxkR3XGBheAdsHosAekxDvb4bCKecY7j8GCny4eIzhcGk20ILrxPYMcMUzQABR"
    "JwHwLw5k+MWwlpOLmbF7gSuDqJIIuK/PBWyzeutCWrobrkDaAaDp1XdPPzPMKwZGvhEbQ32T"
    "MTh3gPrloMiH9O2ZobmIASWCQTQ7Ozu4zw3qbqEXzTD3Ig/8QCKeEYmNvgyEAWXuBaYGufuH"
    "OTDEgDQ+ABGHDUj9/PkTrgY5ASF3FX/9+g1ODD+AdWNj3VmGA2d/MiT1hjAISfOCp69BtV6A"
    "DmSF01ZguH0BRrEysHv4+uMnhtKp5xi+g4rQO6umMlyZ3Qs0/hEDESuRccUzQABRXgXwLRRl"
    "+M8M7PL9Z4zNMWCQ1xYB9/VBYWqpCFGy+SoDw2dgVcDMBmrw/WRYUL6eQU/hPzDnG4KLT1jk"
    "I7p1kCIU1sBCLtphQ7bIrXtkGr0dgOgeIvcC0Fv4iGIeW/2PbCZyQxCmBtKo/A+eT0BtjzDC"
    "EzDIHSC/waoEkJ9BVV11nSE4LEBhAgobUBiBwgoUZqAos1CAJNP7L4ClggQfQ6yHEmQ8WcEj"
    "lkFI0wYoJUBJewAggChPAExszQzf/8nrAhs43pHq4Hof1IDRBlZZ0sBibBMwBb/+Apm+/fnr"
    "H8OKxm0MMjzfGNq7zcDFPqieR0Q+EzzCYDkYOZAxu2QMWId9UdsBiEiF1efow8WIiGfCWv8j"
    "m4XcC8BUj4oRDUgGeAKFlHCIRMDDw8HQ0WPOIMv7HRg2W8FhBAorUJiBwg4UhtqSkDAFtQe8"
    "LWUYdDVFQHUuH4N2Uh6QoQwadSE3+gACiLIEILjcmuEvUxIbPztDRqkRw5uvTAzvv0DqfSM5"
    "SKPm9hvIChpQ73X3rGMMf54+YmjrMmUQFORCqfNhxT7yYA5mw44RrZvGhJH7UbttDEi5H7NL"
    "COnDM2Lt+iFHImqCQFeLLQHiSgQMGIkAFAZCQlzgMPnz9DHDLmAYgcIKFGagsAOFISgsQWH6"
    "HthGePOJiSHDT5WBjQvYExQGdouU/YNAnURy4xIggMhPAMLrgaHB0s7w8z9rQKw6g6SyAHiC"
    "B7RCxloZ0q89dh+60haYPi/uvsNwedNJhoYmQwZloFpGRkSdD+vSoeYWBowZPNTIwBy5Qx/t"
    "Q+0dIHcZMUsAGEZOEIheAxPWEggbjWo2ExqNPRGAlg6rqAgyNLUYM1zbcorh4q474DADhR0o"
    "DEFhCQpTUNg+AvYSJIV5GQJsZUGTxUwMSv7RDEysGqB+GDnRCBBAFJQAv0MYfjHaiqrwM/hE"
    "qDHcewpZvwcq+kETHtuuQ9fpAxPqm8dfGXZP3c2QkaHCYGcvBawvmcGDJIg6H31Ejwll0AZb"
    "EYsrMpCLW0SkMuHNpdgwcomBwAxog06YpQf2koMBbTgZOREwgxMBaGbT1k6KIT1dBRxWoDAD"
    "hR0oDLdeh6xOBoUtKIzvAdsDPtZyDKLi3MDAlpUBdg2jgCEtRc4AEUAAkZcARDcDe/Es9aAF"
    "kmEJGgyMnOwMr4DdPAFgC98AmDB33oAsfgB5FLRUatfUvQzGaiwM8YlqYI+CBkZgw7mQgGZA"
    "G2RBBB6iZc+AtSpAr5fR++moU8FE9o0ZGVB6FOijiKhVBCPWYWjM6gr7UDKkYcgMDo///5nA"
    "YWSiyQoOM1DYgdS9Ag0Z34SErQAnkP8BJM7GEOYkD1kJK+fuw8AhpAtqkpMalQABRGYJ8CcK"
    "mPu15HSFGKzc5BjuPoFsxQKNYj18x8Bw8hG06Ae2aK/svcXw+cZdhqJyPWCDhxOIecCtX0Tk"
    "MyJ19ZAjDblYZ8Ka83EV5dRexoY8t0DILag5ngFjUgrZb8gNQ1ACAFUJXFzsDGWV+gxfbt4F"
    "hx0oDEFheeoRJGyN5SFhffcZA4OVrhSDnBwwzpmFhBjUwiKABsuQWgoABBDpCUBsK2jbZDbI"
    "E/4RKgzf/jKBN2xI8DMwyApBBnvAGy9ACznf/GQ4ufQQQ3KKMoOxsRTDt29A/slTDF++fAYm"
    "BC5w3Y8tp6EWt0xoRS4u9Yx02cOArfjH1gNBbdBCIh1XaYHcRWRiYmXQ0xcHhpkKOOxAYQgK"
    "S1CYgsJWTggS1qDG9refTAz+NnLQ1SaWtgwsnKC2AD8p/gEIINITAONvL4bfTEYSKnwMxjZS"
    "DA+eQew3AhZP54HtgHtvIZ1SYDXPcHbtGQZFwV8MYRHKDM+ePWc4cuQoMAGcYViwYBnDnj0H"
    "wCNpoEEgJsiID8YULXLkYwY0A8aKH3oC/C1/Rng9j6jSGDCqBOSSAJQZQJni378/DI8ePWYI"
    "j1QGh93ZtafBYQlSDQrbc08hYQ0ahHwAbHQba4ozSEgC239sEqLAHoEXUJkEKaUAQACRUQUw"
    "p4HGndz85Rl+MTEzvP8ELBR4Iduh9tyCDF4Aq3mGNw8+Mtw7cIEhNVMDvJhjw4bVDJcvX2ew"
    "t3dhsLCwZDh37grD5MmzGC5evAJuE4AGhDD78QxYpmEZMQZ9BgoglwbYSifYMDFm2wW1zcPG"
    "xg6e27h27TLDvHlTGbZsWQfeu5CWpclwd+8Fhtf3P4LDFBS2oDAGhTVo4SyoFPj1h5nB1VQK"
    "UkRIWNtD1w1wE+sHgAAiLQFIbNdi+Mfizi3OxWAGbLGCun2g+kgPlPuBJcHzTxBl/4Ceu7Dh"
    "NIOeEiuDvYMU0HNcDLy8YsDczg9M3S8Yvn//x2Bn5wDs+qgwrF+/jWHmzAUMT58+B3qaA1wM"
    "Ys9hyK37wXVuAfZ+PyOOBakMKANMoJz/7NkTYMTPZJg7dzqDqKg4Q1hYNDAcuBgcHKWAOZyd"
    "4eLG0+AwBQFQGIPCWlcGEvaPXjIwmOtIMnCDDrzgUZZjEDezBHXSiR0dBAgg0hIA4z9g44+B"
    "xcQaGJkCHOAdPKBdsSLA1HjwDsRGkEPfP/nC8OL0DYboOGUGbm528ApcQ0N9hl+/PjNcv36V"
    "4d69xwy3bz8BygkyuLq6Az3LwTBjxgKGFSvWA9sHX8EJAdY7QG2EMaGM+w8mgFzko45h4Kq+"
    "GMClHmhyq6urleHlyxcMqalZDElJaQzy8gpg/3NzswHDUIXh5Zkb4DD9Bz3XABTWoDAHhf2b"
    "jwzgtQgmWiKQcQFpJydoY5CDGHcDBBDxCUB6HzBrsgSCtuBaOkgwvARG/jdgn1RdAjJi9eQj"
    "dGIEWF/d2neZQVuJBdivlQSv6AF51srKnCEvL5lBWVkMmACuMdy5c4fh7t3HwBLhJYOCgiqD"
    "o6MTw4MHTxl6eqYy7Nt3GLrIgg1tvH9wn1mEnruxFfeIhSQM4Ekt0MxmSEg4Q3FxBYO+vhF0"
    "2flfcKMQ1GW2spZk0FJkAYcpKGxBYQwKa1CYg8IeFAcvgV1wSz0JyGYDQW0NBmYOZWK7hAAB"
    "REIJ8MuU4S+ThiRoq5W6IMPzV5B98bLAwuYIaDn7P0ju//ruN8PTE1cZAgLlGXj5OOAtXNAU"
    "qLS0NENKShxDcnI4sF3wH5j6rwATwwNgafCQ4e3bLwwmJuYMRkYmDHv3HmXo7p4CbDPcgC8A"
    "HeyRj5oI0McumNDGOCDyoMgWE5MAJn5ncDvo16+f0GlvZvgIKWj5e2CQPMPT41fBYQsKY1BY"
    "g8IcFPagOHgOTAzyUkIMkmLAqp9FmJ9B0toC6BQRYuIXIICYSPCZH2jznqGpMMMfoMNAW6Mk"
    "gMXQR9CihVfQuh+YQp9eeMAgwvqdwQ5Yf4FyP6hvC5vdA83kgebFDQz0GAoLMxkCA52BXcM3"
    "4IRw//5jhlu3HgH1sDA4ODgxiItLA3sLK4FVwyKGFy9eDanj6jDrfwasJQFkm/o/YMT/gk9D"
    "I49EgibHQGFo7yTNIMr+HRy2/yA9Z3CYg8IeHAffQAuGmBkMNYRBjUFGBlFjY+jSMU5CbgUI"
    "IOISgNxhZmD54wra/ahlKAwe9QPN7SsA09jl55AlTaBduqADlR6fuM5gZyXMICHBBR/kgC3f"
    "hkXiz5+/wAnD3d2Jobw8m8HaWpfh+fN7DDdv3gCWCE+AVcMTBkFBcQYXFzdgAvkNLA2mM6xb"
    "txU81z50ACPWbirmJhUGFD6s5wBbagYqPSQkuBnsbUTAYfsbGtagMAeFPSgOQHEBihMtZRHo"
    "USmqcgyMLDLErBoCCCDiEsD/P4oM/1l1+UU5GaSU+BhevIFsZABtyLzwFHq4AlDZ55dfGX49"
    "esLg5CYNL8bQB2hgOQGU8kELIoSEBBliYsIYSkrSGRQVhYHVwVVg++AuMBE8Ynjy5A2DqqoW"
    "sMdgz3Dp0i2GpqZ+huPHzwyRUoABY9gXfdEK6kom5NVMiOFt2Cypo4s0OGw/v/oKDmuQOlDY"
    "g+IAFBcv3jIwSIkLMPCDTq9glxBmENLShg4K4S06AQKIuATA9M8G2MJkU9HgBZ+f9ukrZDcP"
    "aCvX0w+QgaC/QJNeXXsE3vWqpSMELrpAmziQZ9fQh33Bg8rAKgFUIigpKQKrhQyGzMxIYOPv"
    "D7C3cAlYGjwEtw8+fPjBYGFhxaClpQssCXYCW81TgYnk/hBKBJjjGPhKAVjbATYbCdqhBApT"
    "aWAJ//r6I3BYg8IcFPagOADFxadvIK2sDCqg3ah/gZqF9HSh7QC8ewkAAojIEoDRAuQwZQ0+"
    "8PKub8CSWEoQ0hKFF//APunrqw8YDPT4gLmaAxr5iJU32CdbEIEEah+AGkXm5kYMdXVFDMHB"
    "LsAu4QtwtxHSPngMzA2cwGrBlYGPT4hh2rSFDPPnr2B48+bdoK8KEJkQNZejlgKYiQHWFgCF"
    "pbAwJ4MhMGxfXX4AOckMWg2A4gAUF6A4+fKDgUFZjh+SOngV5EAD94TaAQABRDgBKJ4AVios"
    "JoxcrOCtz28/QLoivMBe5s2XkKVfoAmp75/+Mvx5/oLBzFwMXu8jcj8DlrqOEcv6vf/gNXOg"
    "dX8+Pm4MDQ1FDJaWmgyPH98CJoCb4G7jvXtPgQ1EGXBCePXqI0NHx1SGbdv2ghPQYC0F0JeW"
    "Y85OonYPEb0FJngmAlWppsCwBYUxKKz/Qo/EA8UBKC5AcfL2EwODjCQ/AyPotCwuWTEGJlZx"
    "QqOCAAFEeMz4/w9gkmJTZedmZhCR5GK49hRyFBvomJtHH6Bn7IG6f8DOKMfvrwxaupDin5WV"
    "Ca3+xzYUil70McK7RyC9oA0hSUlRDDY2ZgwbNmwHD5WKiEgBS4ZvwFKGn0FHRx9YfXwFtgsu"
    "MJw8eY7B19cN2JXUH5S9AkhxzgA+hQTRDvqPEQ6wDanIbQVQIgCdT6StKwwM41vgsOaWEwGr"
    "AsUBKC5AcQLKnFryvAzswL7hj18ifAzswlIM319wQ1MW1lWhAAFERBXAqAnsfPKIS3IyMAIN"
    "BoY9eLUv+Ii0b9ADFIGqPj16ySArwQo+hgV95gtbOwQ50pGLPtQcAQFqaioMZWW5DBkZEcDA"
    "+AbeHXT//iNgqfCQ4evXP8BehC2Dioo6w/LlGxkmTJjN8ODBY8pjrbGRAVjHQE+GIg+AEvKb"
    "N28ZDh06wTBv3gqgu2+BxzWQi3hkv2Jb0o488ygKbITLSbKCwxoU5qCwB8UBKC5AcfIF2B5g"
    "ZGZjEAPtP/8PbKzxqylBB4SYcbkRIIAIlwCM/4EJgIlJQpaT4ddfRvAuH24ByL420OpkUAoC"
    "DUx8fviSwVKeg4GXjx2+jAq1fkP1NHJ9iNwWgCQe7E6xsDBh0NPTYti16wAQHwFWAS8ZZGTk"
    "GD5+/MIgJiYEHla+c+cWw6RJcxmMjHTAJQI/Px/pMTd7NgOw/mH4zcvL8M3cnIFfS4torZ8+"
    "fQE2XO8C3fEAWHU9A5ZW34FtFn5gxLMzHDt2BpiYlZD2KfxHWboOE8M+z8AEDFs2BiVgGN8G"
    "hrWQlTb4fCRQHIDighuY7158AnUJGRkkRLkZHt17x8jAJSMJ7QmAGoJYUzJAABGRABgUQclN"
    "RJQDvHsF2GAHr1u/9Qa6NAREgA5vfPeBQd4UWPywM6OUAKjbtBmwrs5BnRrF7xwuLi6GgAAv"
    "cGNx48adDKdOXQHmMiFgQEszCAryM8jKKjIoKCgAu42XGNraJgMThS2Dk5MtRkMUJ9i9m4Eh"
    "P5/hL7DOfVRTwyCopIRX+ffvPxgePXoKLI3uMjx8+ATolvcMnJxcwIQnCOzZaIDdC5r2/vz5"
    "G7DUuglutIqJiUK3ozOiJQIGuBhkKzoDyrgAKGzlFfgY/tz4wPAL6bSAx+8ZGHRFIHED2lwq"
    "Cjp4AVQ8sAP72JAEAFo1/A2b+wECiIh5YxYZ0LktfIJsDF+/Q2YdQStWX3yGNDbB5+sDUwbT"
    "t88MMvKS0NE/XF0fRqyNQcyeAeHIkpSUAFYJ8Qx2djeBXcNtDDduXAYGrDQwoL+C2wcGBibA"
    "6uEjw969x8E5LzDQk0FXVxO/oVeugE6LAJ/k8LSwkIE3L49BiAN1TgUUcaCRyevXb4Mj/Pnz"
    "V+CVvXx8AkD7pYBVkRY4Er9//wV2y8uXH4Dsn+CE8vTpK2Cp8IRBQkIMrc6H5HxYhENKg//Q"
    "xIEIS9DZR7JyPMCwfsrw6zswzNmZwXEAigsDMUjcgLbi8fFwQI88FQANBIEWi+LcUQwQQEQk"
    "ACZJ0OgSDz8buI4BmQs6uwe0WwV8oiYwxf358ZuB+cdXBglgIxF5Jw60FYkS8SB52I4f9NIA"
    "V3sBH9DSUmfQ1FQD1rPHGDZt2gOMmFcM0tJywKL4K7DOFGRwcHAEBvwThgULVjMoKsoAE4IX"
    "UF4CXG28evUGXE+LAvlCr1+ATotiYHj2jOFVZCQDZ1Mjgygw8kER8fbtO4abN++Cu6NPnjwD"
    "b+/i5xcANlJFGIyNFcEJ4OfP3+CZzIfA4hkS4d+BCfAr0B2fGH79+gY05zewfcQHbeT+R9l9"
    "hNz4Q92VxIiSCMAz8qAzkX58A5YAv6ETRpC4AMUJKG5AbTQeUH0APjEbdDohuBuIcywAIICI"
    "qAKYBZlYmBg4gb2AN98hp1iAj2T9Bcn9oHPyfgNtZWH8xyAizIFWbyEiGhbxMM/C2OjDoOQM"
    "+YPssre3BkaGPsPWrXsY9u07AW4ffPkiy/D+/WdgjhMGVgVu4KnXGTOWMKiqqgLlP0D36/1m"
    "+AUMLOVHNxn8bgIbaa4uDK+BRf+fB08ZDgOL9QcPngAj8RsDNzc3sGQRBiY4ffACDtD5QKDe"
    "yPPnb8ER/u3bD/Cu5U+fPjL8AGaGv39/MQgIcAKrAVEg1gOWDArA7qs4Aw8PLzjRwcIEnk3+"
    "M8Ab66i7uJD3NzICExE7MKz/g8OcAZjTwQngFyROQHEDLGgYRDhZwVXGP2YuDmgCwBnPAAFE"
    "RAnAzMcMTACgA55+fIQcnAzqdnyBnukHcsBPYP3Gx8HEwMXNwoC+WRP7wAhtAGjBaXh4AIOV"
    "lQnD+vU7GC5cuA4smkWBOfEbsH3AB2wbqAGL8NvA3HyPQUNDCbwQ89OnzwyvX71lOMEpyHA/"
    "MJ6BRwFYjazfw8DJzArM5fwM8vIqwMjnAedaUCSDZi2/fXsNjfTvwGL+E9D8z8AI/wmso0EH"
    "Q4gC7VFkUFdXAbIlwV1Z0HoIRM/gPzQBIM4uAnFhCR+ZjT5BBOKDwpiLg5HhEzDMGYENX1AB"
    "C4oLUJyA4ga0g4iNnxVYAAATABPoVEXwugCc8QwQQESUAIwcjKDRPPDMFKRkAR1qALIIZCHI"
    "Ab+//gTWO5DZK1wLISCNMEakPjADvLUPKxGoBWRlZRjy8lKADcGrDGvXQtoHkpKyDO/evQNG"
    "4AsGDw9bcIJ4+fI1MEH8YODhZWcQ+sbCcJsDmMu//2PwtDEBL84E1dugevzVq+fAXP0DnAC+"
    "fPkCFPsI1PcdWPT/B5YuQsAqSAncupeTkwFWOyLgRiDWIZX//1G644i6ngGpwccETiCou5kR"
    "YQgq9kFh/RYY5izg9XeQuAAdScsFzZBMoEkkyK0IzNDiH2c3ECCAiEgArIIgSzm4mMGtTJAG"
    "0BHsoK4f+NoVcD8QcQonqD6E7OtnBfd5UadGmcAuRt+pSquZXj09bXAbYd++Qwz7958GtgXe"
    "MZiZaTPo6KiDu2iQ4WpmMA1KvJyg497evQG2BW8BI5EPmLO/wOvxnz9BjejfwITDAzRTAlis"
    "ywGLdSVwi56Pj4/IqooBvqMYteiHhQFs7cRflEMrQeEKug3lz58/CD3AMAeH/19IXIC6g6Cj"
    "asE7sDnYwKXzL0bQ6AA4ynBmL4AAImb1KDjbwh0KXZwIsvgPI9Ro6PQUyPHnz58ENpbuoizy"
    "hDUGQSkbtBYQdCrHH/gACyNa45C6ADTt7ObmBIwsNYYJE+aAF6ZcunQd2Db4AGwIfgZ3y0D4"
    "w4eP4EUZfHw8wD78bWCxzwvM8aDl6+zAIl0MiLWAuVwZWI+LgYt1SGImf3gYfbc2KNNcAfZC"
    "Dh7cD10lzQgvNUAYtGZAQUGZQU5WGz79CksADNC5AdgBaf9hbW8IwYYvAQAEEBElAKK1Brb3"
    "P6TIYYAJM0LO4x/sALRuTlJSjOHMmYvAQD4ObgDCAhe0EJWPjxfYO5AEJ9ijR48zaGtrMERF"
    "BTHIyEiBi3XkenywzDHBw58JEif/WBGJAH7xEewgIxwAIICIaQOAOifAiGcEpwXQgc6P3jOA"
    "iyTw7h/QadlMEAtBgWpoaM5gbe0APuIFspQLddctZAXMb5SSAdY2wDzAgXoAdLyclLQ0Azcw"
    "R38C5nxQ/Q46sAHkRm5uTnAxLiUlDqwangIbkZbAtsJ7YI/wNYOpqTHV3YLtrAbQZJaamgaw"
    "PaGNUQVApsz/gLuWL56/hxaXkLUf4KV4TJA4EQDGJicH9KKMf/+xW4QGAAKIiCrg3zuge3i+"
    "fvnDwMbKBj4EGbQAgRV6Vj4oAfxkRJywCSpiQZ5hZoadnIF9Jy2hAKE24ObhZFAEFuHv3gHr"
    "a/BBjt/BVRKoDQDanCIkJAAuhk+evMigrKwObOz9BLfu0Q+dpEbEY44BwDDoBBHYecSoCQDU"
    "XQUlDIS+//AEACr+QXECMgtYiwHd/ZPhL/jenK+wW05xXk0DEEDEDAX/BOX+P7//gxsWDEh3"
    "6jBDi38hQXbwWn/I2bnox6oiujeQRAFb9oTczfkPHQKl7ay84c71DKdY+Rl4PV0YuNmYwCUA"
    "qFTi4uIED+SsXbsF2O9/BiyhgA1Czj8MLi52VO+2ojYA/2OIQXoAsDOLMU8tB4UxKKxBYf4P"
    "mgGBbVdwfMBuNPsDSkSgUbp/oLFB8GHTOBMAQAARUQL8/wSKvB9AS1mg17CAtiqDb+QC7f4F"
    "YlExLobnwAQCmplDOJwBR+75z0DLsQCcYPp0Bt4JvQy6mroMR9VVGfiFhYHdKU7w+b5Pnz4A"
    "dvVeA+t9FXCVABonAJ31BzqIihYAvReAXArA7i5CnEuE2n0EhfFPYFhLAsP8NewepP+QOAHp"
    "BTVVQEvt/oFKgL+gk5cZ/uBLAAABREQJ8O/tP2Asf/v6j4GdGxJ1oPke0L08oIs3QIUCJx8X"
    "uI0AumlDVY0B7dRt1Doekvv/w4eMEYGAmAChekmwaRMDQ1sbwx9BIYZvGSkMDp72DB9evmV4"
    "+fIVuHUNGqI1NNQERrgY2O1PnjwFr1WkLfiPUwzbIVWQ4eP/wDD+Dg5rUJiDq2EmyOVW7ND9"
    "g6CJuq8ffgAz/1/IAA2kBMB5iBRAABFRAvx9DUpDXz79ZZCQgTiRHekiRiZIHwZ8x87zZ1/h"
    "Cx3+/WNA6+b9x3paF2oO+I9nBJFMcOYMA0NFBdjsx3l5DHwRkQyCbGwMgopcDIqKsliHlUED"
    "SbTL+f8xhnsR4ujtIdSTykFh8+zpN3BYg8Kc9T9khIfpHyROQFpBY39vQadMgS82/PwdMleL"
    "+/YRgAAiojP75xko/Xz69AfcwAA3NJihDUEmSEIAVUC8wnzg+3Ug9fl/rEeqEVMHIotTDB49"
    "YmAoKWEAtvwYnsbEMHCmpDAIs7ExDBzAFfH/sYYLsjpYAnj48As4rEFhDgp7UByAMyQzohH4"
    "BTRUCxok+PUeNHr1HV8CAAggwgng/+9HoJbF29d/wXUN7PpVNmhDEOQA0OVNPGICDA8ffAZf"
    "doA6ivUf7YDF/yijYYj67T/GKBlF4ONHBoaiIgaGa9cYXvn4MPzNz2eQ4OEZuKj/j3yuIANG"
    "2CBnBESuR721BNQVfHDvMwOXqAA4zFnR4gLWHnjzGno8y6+3n6EJAOetIwABRDgB/PtyC9ja"
    "+/fm1R9IccMMGWEGpToQDdrMC+pyiquIA6uAH8CS4he4O4NIBKQVg9j0kAxAC0RrahgYjhxh"
    "eG9jw/C5vJxBVlJyQMdtUCP9P9YIR5ZDTjiQnsE/hs/AsH3+/DuDBDCsQV0/VhbUuADFDTPQ"
    "jDeg83mYgIwfT0BLpr8w4Ll7CCCACCeAX0+uAyuZb+/eAfv3P/6CixiQJg6opaDUB9qwI6ok"
    "zvD27S+Gl6++Izn6P0oLFl+Eo5cKFFUDPT0MDBs2MHzR02N4U1nJIKeqysDEwDDACeA/UmSj"
    "lgTIYYBeXUBmDyFh+Or1d2Bt9hsc1r+gl1KB4gAUFyD/geLm948fDO9AJcB/YFH87R5o3Tbo"
    "UkqcW6oAAohwuHzY8JLh/8/7IAvfvf7NwMMJaaOxQ+/WBTcGQZaLCDJwC/AwXLn0DtzHRx3N"
    "go9PYg0ERImAva1AEli0iIFh3jyGnzIyDM+BVYCMqSnDQA/iQsIBuaVPKGOgZgRQWILC9PLF"
    "dwwcoAszRAUhYQ4Nf+gqPHDcvHv5keEXaOHmr9dfGf58fAdNADirAIAAIiIBbPnNwPjj/P+f"
    "/xlePoMkgP/QiUbQzlRQMQS6z/crIzODkpYEw/Fjr6B36EAuWEKv95HrfGx1H3LgkJwI9u1j"
    "YOjvZ/jLzc3wJCeHQczNjfDuSLo2ADFzOqK3hNl4hlxSBalOQWF6DBi2oDAGhTUHtPplg5bE"
    "ICN5gJ2Dl0/eMvwH7Rj5+fg9w/+/bxkgt5Xj7AYCBBATUS7/+/kMKAk+efgT3PdnZkY0BmGN"
    "EJCdmuaKDDdufALfqwdpB/xD6xJCr2//h5rCkYt89AYQ0eDSJfBKXlB99DgxkYE/OJiBn4lp"
    "4KP9/38cvZ3/eBvAsKFh2AKSd+9+MNy4/pFBy1wBctcCM9KILBMkTkB9+if3X0MM+P4AtGcb"
    "lAC+MuC5iRQggIgLoV+PjzCw/P/99Mkfhr8//4HrGnD3H6kbAlqhJKohC24PXL78HjzCBiq6"
    "QCkYdsESIuf/xzh6HTX3424UYQXPnjEw1NUxMLx+zfA0JISBLS6OQYSDYxBEPiJBw6pDBM2A"
    "te2DnAkgJQDkWrpLl95C2loacuCwhnXDWaExCIqTvz9/MTx98AbYIAAG+teroCO7X0MTAE4A"
    "EEDEJYA3i24Ck9StL5/+Mbx+/osBWA2BxxaZkEoBUPfjDw83g66xDMOO7U+gZ+T/g1cD2Bo/"
    "sPP2kYs8THUE2gNfvkAi/+ZNhlfOzgx/MjIYpAQFB0eh/x+9V4Nc9f3H0iZAJHzYRZWQttQ/"
    "ht27njLoGMmAwxg29g8OdyZIXIDi5PXj1wxfQAcK/37xieHnI1ACeAXtBuIEAAFEXAL4uOs7"
    "w98Ph0BNiQd3foL3ojFDB+tgDgG1RJ8BG5+2XpoMp06+Bd+oCcr5MA+gzg/8x9JC/o/U7fmP"
    "tXrAHKQEVm1dXQwMx48zfDA2Zvicm8sgIyU1yCL/P5beDvYEj9wugN0xAFIDCssTx94Aw1YD"
    "HMbguRhouIMAKC5A638f3HgKWRjw/c5rYP3/ApR1GQjcPAoQQMRWkv8Zfj/ZwsDK+O/ubWA/"
    "/9d/8HqAf/8RI4Og9YEfgI6T1ldg4AG6Zu8e0JIr2E0Z/6ALIRFzAKBj0WBLpJG7Q6g9AgKJ"
    "YOZMBobNmxm+qagwvAY2+uTU1XEvfhvQQR9UDItgyDJwJvAqY9j6QFjdDwozUNgxM/9j2LPn"
    "KQMXsPUtbaAIDmNQWMNG/kBxAIqL/z/+MNy9+hQ0Hfif4et50N75Z9AeAN46FCCAiG8lvZh4"
    "lIHx24P3r/8yvHj8G3wwAcjyf9ABCXBxBG51sDL4hmgzrFv3EHydKmh9AOy2DJDnQBEPmoDZ"
    "v38v+GQsyCIRBgIYSz9540Zwl++3qCjD08xMBmlzcwbWQRH5DPDLpmFtH8yEDLtqhpnh1asX"
    "wLDYAw4TUEKAdftgxf/Hjz8Z1q55AA5TUNgywUpdFkT48/MAo+f+C4b3oMUif998Zfh+BZQA"
    "gKkBfBU9XgAQQMQngE+HPjP8eb0DNP9748p38G3YoNYnaM4B5ijQDlXQ0IN9gC7QY38YDhx4"
    "Dm7AQC5mgpx0cfnyRYa+vi6GNWtWAttsr8ELMrAFEgYGLZIA0SC3HD3KwDBxIsNfdnaGRwkJ"
    "DKIuLgxcg+AMIUTEIzfy/uOoDiC3noDuGQKFRW9vB8PFi+eg5ymwgHcggVYdg66hBYUlKExB"
    "YQsKY1hmA4U9KA5AcXHjzD3IytDvV54z/P8DivznhOp/EAAIIFL6Sf8Yvl9eCkyEf+4Bq4Hv"
    "wAYhFztkc8ivf4gJCdCKoW88PAwBwRoM8+bcBrbRfoGLs4cP7zPMmzcLWGpPBc+zl5dXM2hr"
    "60DP/SGUAKDLnEBXs9y4wcDQ3Q3u7j0JC2PgCwhgEGBhGUQ5HzPiket65NY/6GQUTU0thrKy"
    "agZxcQmG2bOngcMIFFagKvPL158MCxfcYfAHhiUoTGFX0oPC+hd0Wx5o68f3d18Z7l1+xADe"
    "nfPlKLDBzvAQWv//I+RugAAiraN8L+08w783x39+/s9w++p3BgFuyJpRUL+UCdoWAKXQa8Da"
    "JzzNlOH1mz8M+/Y9B6+hX716GXi3TnJyBkNSUjqDpKQ0/MJm7MU9cuABcz8o2b94ycAEavS9"
    "esXwzNOTgSUigkGUi2sQ5XxsjVj8VQKonpeSkgGHSUpKFrBUfMmwatVScJjt2/uc4fGTnwzh"
    "qaYM159BwhYUxqCwBoU5KOxBcXD7zB2Gn2+BvaHfd94y/Lp3F+ikB9ABIIIAIIBIzTo/GH7e"
    "X8jAIWZ75eJPBk1DLmCDhBG8Fv07sPQRhC5I/AhyixI/Q3yKAcPUKRcZbG3FGby9g8BXwIJu"
    "1ATVd8gXPSM38GCNIPDWpn8Qz/5jAdZ9n78wMPX2gLt7ry0sGH7ExDIoAs0aqAVGmA2+//DG"
    "HXIrH3sPABWDzgcEqdXU1AFvMH327BnDt2//GKZMvsaQmGrA8EeIn+HDE0jOB4XHe+h0P3hj"
    "DrDxd+XYDUhX4OtRWOQ/YSDytnGAACJ1qOw/w+P6DcAovvXx1V+G+zd+gi+JAHkTdGYdvBQA"
    "OvQ80BlR6SbgSwOWLr3LICcnC74mBrTRApTqYbkEvcuEGCWEBiqojQAsKVimTWVgOnmC4YMW"
    "sDGUkMQgIy8PXvn6jxqzh2QW+bCJGhhGLglgvR5cbQFE4kB0+UBb2ED1v7y8LMPixXeAsmwM"
    "0Rmm4LDkYkHkflBYg3QLABt/98/dZfj4+B1Q4OUnhu8nrzJAbhl/S6j1DwMAAUT6WOnXcx8Z"
    "ft5bCBqBOH/mBwPTn/8M7MBGCGj12adfkFQKXpYEbH48/8fOUNNkxzB3zh2G8+dfAD36B7xe"
    "ADJX8A/rSBkKBi0RA9rDunQpA/Pu3Qzf5OQYnsXFMchoaoIHuxBDzf+oM41M0uQOZjsFs/pi"
    "wFjUibkWABL5oDD59Qt0/fxvhgsXXjLMnXsHHHagMASFJShMQWELCmNQWIPCnOnnX4bzey9D"
    "4vr7sXvAnAIqAe4TGv1DBgABRM5g+R+G5/0LGBg/Pnr3/C/D7Ss/GQShM4SvvkGWKIMmKDiB"
    "DrwIdIqtjxqDs6syQ2vLJfCFEaDlzaAWLrYBIuSiE5Sz/wFb+Sw7tjOwrl3N8EtAgOF+eCSD"
    "uLEJ+LQD2Gwj8hAret1Li+IeYd8/rAkBNnaPnusRCecfvFEIm+yBXSgJahN9+/aLobn5AoOT"
    "ixI47EBhCApLUJiCwhYUxqAugCAw998+foPhHehCQab3Xxi+HbwIlLkF6rAz4FkBhA4AAoi8"
    "2ZIPO14xfLswETTqcu7UD4bfX/+Dz6gB9UJefoUUVWAHA3PA2ccMDG19zgxPnv1mmDP7Jnhw"
    "CLQx5Pfvv/BZQ8zGEijyORhYTp5k4Fi0ENwGuOvnzyBgY8fAw8wCbTwyoOR+1MDF7HOTU8Qj"
    "RzpsVA55nQNywkMkQOw9APRSCrnoh90mChr0mTPnJrDh95uhvc8FHHagMGSDFv+gsAWFMSis"
    "fwMbAud2nIOMBn3bdYPh/w9Q0X8bOvhDNAAIIHKny/4w3MtexPDv9eUvb/8yXD7znQF0XD2o"
    "Twpy5OdfkOFKUKv14Wugo3m5GSZOdWWYOesOw4EDkBFC0Dp8UEQi1g0gVQds7AzMt28zcM2d"
    "BVqPznDXyYWBzdWNQYiDE95zQI4I5MhHzpnY6+j/GBNTuLqhhCP9H4Hu6z+MyEce60e+TxgU"
    "+aCwmTX7DjisQGEGCjtQGILCEhSmoLAFhTE/sLF9efd5hi9PQdP9T98z/DhyAci4Am38kXSq"
    "FUAAkT9f+vvFB4ZPh9oYmP/9vXLuB8O7538YeNghxdSDDxAloDoL5PiTwHTp6KfCkF9kzlBd"
    "fY7h9u0P4ClqkMchN3gjqoN/rGwMTMDuIs/cmQyMb18zPDQ1Z/jl7ccgxifA8Ad8mCSibkUu"
    "StHrYOSSAZYosFUZ2MSQZzDRzUSPeOxmMKC4E316G2YHbMsXKCxAYVJZeZYhN98MHFagMAOF"
    "HTu0nwYKU1DYghd93H3DcGU3sMRnBfb7v28GNgL+gfr+N4jt+iEDgACiZMIcWAqkb2P4dWfb"
    "35//GY4f/MbABroDiAVybNmDj4gEADrNCjQ8UdloxWBoIsdQXHSK4f37b+Bh4h8/fiMSAWhq"
    "68tXBp5F8xmY791heAHsFr0PAHYfxcQY/gHV/MPRmkankXMoInIQJQNssQoiwhGLV2D1Nyr/"
    "H0rEIvcAsLkBvRRAjP0zwGdJITn/NzgM3r37xlBUeAoYNrLgMAKFFSjMYAkAFJagMOUAtQWA"
    "VefxlYcZ/oIOA/p78SnD7yuguv8SdOyf5DPtAAKI0hUTXxheza4HdtLfvHjwm+Hq2Z8MvOyQ"
    "FSoPgY5+8QVyiSSoEQNapnb9NRPDwmVeDMxsXAwFBScZPn/+Dg4AUED8BteFfxl4165gYLt0"
    "geGdgiLDE98ABilZYHfvz1+0XIcasejFMoKN3mJHbSOgVw2IkuQfPKEgcjO2aocBZ9sDuRcA"
    "q2JA1R165IOukS8oOMHAxMoJDBtvcBiBwgoUZqCwAxX7oLAEhSkobK/uusjw4hKwZcgK7Bv8"
    "WH8aWvTfZIAs/iQZAAQQpQngH8Or+dcZPh3uBDUIL5z8xvDm0R/w2DTI21deQ9sDrJCGy+3n"
    "wOIL2K3ZsCWQ4TWwp1pbcw48HPoHGDC/gAHCu30zA+exwwzfREQZ7nj4MkioawBrGMgNGrAA"
    "Bm3ThpyigVzv/0cpshER9Q+pPidcb0MSD/IaBlzF/j+MnA8auAJtLsVshyCqK1Dkgxp8sPYP"
    "aAtXXe05YFgwgsMEFDagMAKFFSjMQGf/XHkFXe4Fuh7m+kuGCxuOQydhtl5l+P8eFPmg+v8l"
    "McO+2ABAAFFjzdRPhjtx8xl+XNny58d/hqN7vzL8+/IfcloFsBgDdv/BdResOLv4EEiL8DLs"
    "2B3CcBdYalSUn2b4+PUXA/OblwwCJ44w/AY29C47uDAI6hmAD7eB7YiFnUBy6RIwB7x4AU0E"
    "DCg5G72xBUsYkDr9P1IV8A8rRlXzn0A7AGEvyC0gN50/fx48vYuo7//B7UM0+H6BaVDOr6o6"
    "w3Drzk9gWASDwwQcNtBqExRmZ4Fh9/0vpDT49/4nw9H5uxj+gAYFGM4/YvhzHJT7gd0AhrsM"
    "eFb9EgIAAUSNBADy6QeGJ00VwIL+/oeXfxhOHPzKwP4P0nV5DXTv6WeQlSugIg1UlJ0Gdlj4"
    "pfgYtu0KZrhz7xdDftYRhuf/ORgeeQcyXLCwZ2DWM2LgAgbkr99/kHI+C3iIdNasaQy3bt0A"
    "H0WDrdGFiJh/8PoduWuGnLsxW/3Y+ejtANRZv39gt4DcNHv2dKAbn4LdirykCzLI8xvc3gGV"
    "ZqD2T07OMYbbd34wbANGPigsQGECChtQGIHCChRmoFPZwBkHaOeJhfsYPjwECrK8/cTwe81x"
    "aORfAoc9uX1dIAAIIGqtmvzL8HHfHYbXi0sYmH99f3DtJ8Olk98ZuKFLlh8B26annkLWr3FC"
    "t5edBN1/B/T4jj0hDN9+sDAkxh9iOMcqw8Du4MDAxcQKHjRCDBhBctHWrRsZREXFGIyNzcGz"
    "iOj1LnKko9b//6BiqO0FUHUCwugNOuTcj94OwNb7ALkF5CbQLCfIjTB3gCIeVuSDIp+R8Q/D"
    "nTvvGZISDzN8/c7CsH1XCDgMQGEBChNQ2IDCCBRWoDBjhw6rX1p7kuHBiUugXaDAAFlxCtjx"
    "BxX7Z6ANv7+URBxAAFFz2exPhqdtexg+bG5nYPn/D5QA7lz+CT65ClQS3H4HKdLAnmKDjBye"
    "ARZe/BJ8DPv2hTHIK4oxpCfuYzi6H+Sn3+AJI1CggQIPtJYAtI7gCrDB6+sbCD5+FVI1IHfb"
    "0Ovnf2g5+R9Ktww0EXX58mUwRqzGQa42kBuOyGMMmF1KkFtAbgK5DeRGUDUFcjNoaBfkB5Bf"
    "GBl/A/35lCEl5TCDjLwo2M/8knzgMACFBShMQGEDCiNQWMFmVu/svcFwacNhBvAe4P8bLzH8"
    "f3AWyDkJmpulpOiHAYAAova66c/AruFMhk8HF4M4p/Z/ZXh6+xcDJyOkeLsAbKqceApdPALd"
    "owlavPSHg4Nhy7ZAhpg4I4bc3BMMU6ZcgR7h8hs8dwA6xGnLlg0M2tp6DBoaWuAj2zAHg/5h"
    "jMWjRiRqewBUV4MOZAJhSL2N2f9Hrz4QuR6zywdyE8htIDeC3Ao6XRQytv8HfKbg5MlXgX47"
    "BfSjIdivID+fh156AgoLUJiAwgYURrCq4OmpRwynFu6ETnceBpYTJ0BF/wkgvkrMah9iAEAA"
    "UTsBgOqidwy3QuoYvp3aBuzlMBzb9YXhGVIiADUK992HzF5ys0HWs4PuIHj6iZGht9uWYcky"
    "P4YNG58xJCcfAeak1+ARMlDdDzlhzBk6bPoHbbk5A0qRD0sMmEU56ng96Gg4yM1c/7BUAdgb"
    "kKijfLAxBEjrHoTt7d3AakFtAZDbL19+zZCcdBjop6dAv/mC/QjyK8jPIL+Dw4AREiagsAFH"
    "PmiB7anHDMdmbWP4Czrkg/HcAwaGLaBiAFb3v6Ok3kcGAAHESGjdPZnXtYFmquUYtA8sYmDT"
    "tWIFNuctXXkYpNTYGH4xQuay1YQYGDxUgN0bVuhW9v+QO4i1JBkY7t//xJCXt5/h0IF7DHFx"
    "ygzR0aDTOpnBN4xCrlNjQbqNDPlad+QTt7G7H3b+AMiM+fNng8USE1Ohx9b9x7lVHXaIBeoM"
    "5j/4sS2gagA0jgESAy3m+Pr1L8PyZXcZ5i+4w2DroMQweZIjg6IiH8M1YDfv1WdIpIOK+S+/"
    "GRh2ABuAt6DFPmir17PTTxiOz9zG8PvrN2CL8Ppzhn+zQMUAKAEcBOJHDHg2e+KbyMIGAAKI"
    "VgkABNgY2BXUGVQWzmFg1zFjZWdgsHDhYZBWBSYCZkgXURoY4T6qDAwyfJDDJ8FLnIA5QkWc"
    "gYEPSC9eeoOhsf4IMLp+MaSnazI4OEgx8PCwAT3DBL+QEnLQI+J6GuQbu6E+gJ8+guwnUAJY"
    "sGAOmJ+QkAKfY0AuzDB36qB2/2DdR8ik1l/wHAdoCdz+/c8ZZs68Bl4LUddgwxAbrQGexr3z"
    "EnLTJ+gcH1Dr/gmwobflNrD0+wxdUge04ykw8k/M2cHwG3QyN8uNFwx/Zu8FxvcRoBMOQOv9"
    "X+TOZGIDAAFEywQAng5gYJfTZFBeMJ2BQ98C2FtiMLLlZlDUYWf4B0wQP/9BSgBPZQYGfQnI"
    "zBdorRtoabcYMFHICQJzy9ufwGLzNMO8eRcZREVYGBLiVRnsHSQZ+Pg4oDkdcp8O5I4ixCVV"
    "mNe0wRID/gSAeaDFf4wxBFj3DjalDVq8CVq9C5rMWbDgNngpXFKSPkNxqSmDmDA7+Ai3V58g"
    "zXXYySoXgcX99ruQEoAdtK8fmAHuH7zDcG7pfoa/v0Br7K48Y/gzZz8w8o9Bc/4dShp9uOIZ"
    "IIBonQBAADSeo8ygsamfgdvaFSSgaczJoGXCycDEDfQedOOwhTQDg5MiZIUraH4ENO4PagiB"
    "bsQCzTTevPWBoaf3DMP6tTcZRESYGQID5RmcnKQYpKR4wC1uyHnDjEi3laHf4I16dwEoASxe"
    "PB/swNjYRKQEgH1iCdZu+AtdiAJa7QxKAKBNG/v2PWNYt+4BsLH6lyEwWJ2hpNiEQV1NALxy"
    "59l7yAIO0EoeUC7/CmTvvQdp8IFPXAMtefsKbAdtucBwfetpyLlvDCceMPxZcADa2j8MHez5"
    "QVHjDEc8AwQQPRIApCQA3WuvuqSJgd8jhOEPA6OMGjuDvhUXA5cIE8NfZkg7QA6Y672AVYK6"
    "MKQ0+A0d3ASNgUsIQHLKvQefGebNvcKwYvlVhk+fvjOYmgozeHnJMOjoCAETBif0rF0GeJsA"
    "USWgYtDpoIsXz4UmgBTwQhXYMi3M+QGIOGSJ+z9gC/8beP/jtm2PGU6ffsvAy8fJEBmpzZCU"
    "rMOgpMALLtlefAA206H5lRWa62++ZWDYdhvSxwfV98ygq99e/GS4uOYkw5MTN0G7O4GW7brN"
    "8HvNIWiD7xh0hQ/F3T1c8QwQQPRKAJA2AQODLIN8Zy6DcGQaw18uTj5RZgZ9ax4GCUWgz4GR"
    "/Ps/5OQxCxkGBmdgaSDCBUkYoONuQDkIdBGGELDdwM4I2oX0m2H71nsMa9bcYjhy+AkDG7AC"
    "1dTgZ7CyFmXQ0xNiEBfnYuDlZWPg4GBBarghqgRQSXHu3EkwHzSIAxtXQGzagDQYQdO1oFNP"
    "Xr36DuyVvGM4ehR0W8hHYE+EkcHGVoYhJESNwdNbiUGAl5XhJ1Dru8+Q49tBaQl2gMYbYFtu"
    "LzAaTzwB3fELyfWMwJLgxZU3DBdXH2P49BB0Ezew3P+z+jLDnwPHoZEP6u49JrfOJzYBAATg"
    "7QpWCIii6BENz2QhDTNMegtZsvEBks0s/YEvsPQ9/sJmkp3NZGElSRZEJkrKoBl513uWdvIN"
    "53bPPfe9c+4/C+CjDiwY3Q6sXh/xcjGuxVCtM1RqDMmsPD9PoOcY0ORCPQhqEHQvC0H9EqYB"
    "irxwurICbfcBRu4aw+EanreDf6CzMRpsm4nJOwPOhQKx0iiYKTChsTRNGsiIBgjvMJKvqLRz"
    "CIIQvk/nXa7v0KvV6oLNhpI5HjDyOhoNC47D0WpzlEyZPkBtnfK4boq6Emr5RRHuE9Hqx0LE"
    "HQNl56Jk79MTy/EcC3eGiLJ3E7sz7gMP0cJTwE+VseNnxxC/4fwSQPROALCl6EIMnGomDHKd"
    "1Qw8dlagPqCQNCuwXcDFICbPwsDECTmAEhTh8nyQhGAM7B6CrsUFH38HrRpAAQ2aKAFtUGGF"
    "OvPLtz8Mt4HthbNnXwDxK4Y7dz4wPAM2s18Ds+Ff8F47RmAiYEY7jg5S33//DlmixgKsHkDV"
    "iZQUL4OKigCwhAANP0swqALrdR4uyAoNUGkFuq3zO/TmFFhRD7tR5exzSMQ//ARxJyjJ/QMm"
    "glc33zNc23ae4d3NJ5Ai4u/RRww/Fx1n+P/1ArTOvwrt5/+hZqDjimeAABqIBAALedCFRkoM"
    "MvWpDCIR0QyMovxMwCaygjo7g5IOBwOfGLBVzwHJAqB2ETAuGGxkGRhMpYA9BB5ISf3rH+KG"
    "DWZorgPtk2dH2yH69dtf8C3kL19+A9bfP8CHLaB7CxQMoCtvhYU5GMSA1Qc3MKK5uVC3TYC6"
    "rqAGOnjc4h/iiHs26BG6r75AJnGOPIbslAZt3AAlzL/ABPHp6Q+Ge0duMzw4cYvhHyjlsLz/"
    "xvBz7TWGX/tBuf4sdGz/HnSEj+pLWnHFM0AADVQCQG4cSjHwWtszSBVnM/DYGDP8ZWLk5Gdm"
    "UNTkYJADJgYuIWB9zcEAvygRtPnEQBySENSEGeBrD0Alwz/oCelM0DqcGQ0TO+z5D2oXMgaP"
    "NEJTLhP0vGQQGzRnfweYX08CI/7CC8imDWZYjgfds/z6L8Ojs48Y7h+9yfD91UdQJQhsSQJb"
    "A9+Xn2H49/Iy0qzeM2o09khNAAABNNAJAFYl8IO7iuIZoQyi0eEMbJqyoNDmFWYGJgIOBmll"
    "dgZuYUZwiQA6Gx9UBYCKW2APkEEPmBh0xYBdDAFIFQHK/aAi+Q9sHx6Wogd81j6WMWz4ZQto"
    "6kGJiQV6EDMo94OK+HvAVv5lYNvt0ktgZf0FUhqxQo9t/QtMBF+BEf/00lOGR6fuMXx++hZ6"
    "swowqXxfd5Xh9ynQMq7L0MUcd6Eref/QMpBxxTNAAA2GBAALZ9B4Aeh0ZnUGmZooBqFgbwYW"
    "WWFQLHIBSwQpJXYg5mDgF2NmYANGPBMrpFT4A113ACoZVICNSFUhYLsBmJyk+SAJAhQpLMzY"
    "l30jH9mOzZt/oO0NUIQ//QRZmgWK+FvA+HwH3Z4Fr9+B0fcLmBA+PvvO8Pzqc4anFx8xfHsF"
    "XaHN+Pwzw/dNNxl+bD8PXb4FW8b1Etq/p/mWFlzxDBBAgyUBIE9O8YCrBVYxXQaJ3HAGAVcb"
    "BhZlcVA5zMLFyCAiycYgoQCsq6XYGLgFgf15YFeRkQVSMvyB1sugXgKwnccgzg3sSgKxGFAN"
    "aPMKaNRRgAMS2bxskIQDbv3/hSxdA+kFLb4Ejc69/w7ZhPHmK2RdHmhlDqiVzwgtCRihdyb+"
    "Bqr7+uYvw9sH7xheXH/B8Obua4Y/n79Bc/zDDww/9j9g+L7tIsP/77eRIv4ZdA3fP3oFLK54"
    "BgigwZYAkKsF0E1M0sAGgDqDeKYvg4CbHQOHniz4Tlem/wxcfMwMQhJswITAziAgzgbkMzFw"
    "AJMOMys0QUCu1QG3C8D36UB7DbBDlVgYUQ+s/gMNht/QbdcgLmjChgmKYRH+F5g4fgCj7tv7"
    "Pwwfnn5keHv/HcO7h28ZvoFOZQUVGSygqcGrbxi+77vH8B20b+vPfWikgxIAaPzvE62Le1IS"
    "AEAADdYEgN4+EAf3GPjdrBiEAhwZuE3VGZilBEAjiiAVrJxMDPxCrAx8wkAswsbAI8jKwMEL"
    "uuuQkYENmPOZWKBXH8HuYmKEsv8jKqD//xjg+0T+Q0sScLH+DXTByH+GH59+M3x5+43hE7DC"
    "//TiE8PHl58hEza//kBG8P4BJX4C+57f991m+HUZFNkPoOP396BF/ceBiHhCCQAggAZ7AkBO"
    "CMDCnEEEXCow86swCLhbMPDaGzFw6sozsMoKMvwD1vT/ITNJzMDszcHDzMAFLPM5eZkZ2IHd"
    "OdC1d2zAhMLCDpknYONkQW7+Mfz6/gcY4cAGOrCvB2L/ADbvf379xfD94w+Gbx+/AxPAD2B3"
    "7jckwkF6mECrPB9/Yfh15RXDj2MPGX6cvcvw/+dT6HQtKPJBu3RAhzR8HciIJ5QAAAJoqCQA"
    "9MYiqFQQBo8qMrHLMXAZawK7knoM7KoyQCzOwCrBx/CfnQVYB0AOGQA35SH1AOTsyP8MkMvE"
    "oWU9dNkXA3jCB9rJ/wOlwSUGqBIBpow/z74y/L79juHnndcMP848Zvh95ynD/z8voKN2T6A0"
    "7HTO7/Ro3FGaAAACaKglAGTADE0MoAElAWiCEGdgZBNjYJOVYeDUVmRgV5ZkYBEVYGAR4mFg"
    "FuRmYAL2Cxg5IOuQGHnZGKBLtyHl/adfDP9+AlPC199A/Ivhz5vvQPyV4e+rzwy/7gEj/eYr"
    "hr+v3wIj/C00kt9Ai3YQ+wN0AAfUov87GAMLVzwDBNBQTgDoJQNosokTWlVwQxuRAlCaByoH"
    "w6zQaoUNZTwYMvHyBzoG/wOai79BW+yfoBH9CVqsf2VAnMX/f7AHEK54Bgig4ZIAsCUI2L25"
    "7NAIh12jygKVY0LCsAFAGP4LTQh/oBH8GzpKB7t/5/9QCxBc8QwQQIz///9nGAUjFwAEENNo"
    "EIxsABBAowlghAOAABpNACMcAATQaAIY4QAggEYTwAgHAAEGAKfkWThjEHi3AAAAAElFTkSu"
    "QmCC")
index.append('clock_icon128x128')
catalog['clock_icon128x128'] = clock_icon128x128

#----------------------------------------------------------------------
led_blue_light_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOTSURBVHjaYmRQXcTAwMLEwMDH"
    "xcAgyMnO8O2HHePPr6niPIx2YnwsfIxA8Obdt88vXn898ff339kMjAx7GFiZvzNwsDIw/P/P"
    "ABBALAww8O+/DMPvf13GemKRcc4SDM4GIgzighwMjIwMDK8/fOc4ePaF74J1N3xPnHy4juHv"
    "v1KgxD2QAQABxAh2AROjHLMg98qEeB2LiihFBhU+hLnI4NGX3wy9C64wzJxz4trPP//8GNhY"
    "7gIEEDODUAAHw3/GCV6hel4l8eoMXMxMDN9+A1X/BzoKaPtfIP0dyH/3k4Hh939mBi09SYa3"
    "f9hFr5x9oAS0eAtAALEw/vprw6IuGfPPWIXh/RegJqDl/OwgLzEwsP5hAHqZgeEXkP31FwPD"
    "J6Ahn4Fif410GNiOPvb5dfu+B0AAMbOJBHaJu+tpvRcTZZADhqUwBwPDH6Ctf/9BNH7/A9H8"
    "GeiKb0ADrr1lYNj0ipGBjY2F4efNe6IAAcTCLcZjI64hwQBUw3DoJSg4GBgUeBkYuNmALmCC"
    "+P030KBvQIMef2JgOPCCgYEDaAGPsgTDDxE+Q4AAYuEQ5+Xj42VnYAc6nQuo+TZQ0VegbSJA"
    "l7AzQgz4CdTw7gcDw9PvQDFmBgZpoBe/sXAwsPJxswMEEAsTFxuDIBsjAxfQAFEgFgDazA2y"
    "+S/QZqgBoADlAmqUBBrKCaRZgRZ8BrqKmZGBESCAWH7/Z/jCx/Cbk4+DhUEIFAZA0zmAihiZ"
    "IBisH6iYA4hBljCDYgZIswID5O/Xb78AAojp3edvJ76+eMcgzg1MiEDbgeYwMAExC0gRFLNA"
    "xdhALgQmQAkeoOuev2J4/+LTZYAAYvrz5fus6yduM/Ay/mHgYofYysIM0cgGxWBDmCEBDEzs"
    "DALM/xiuHb7G8OP77xkAAcQElNl/9czdtecPXmXgA4Y+OxtEIRPUIJDtwLQFFmMFyvHzMTBc"
    "OXaD4dKJu/uAJm8GCCBmBtmA3wzff565dfGRKx83i6i6ligDNw8TRDPQucwg5wPZnNwg//9j"
    "OLz7GsOCOUfvf//LEAOMkicAAcTMIOwGdCfz+9/ffu84d+ah0oP7b9S52IApHJiRWFmZwTnu"
    "57cfDFcvPGFYsuAEw7rV53d9//E3loGb/QZIDiCAGBlMZjOAs9x3UGL/ww3MF54cHCxZEpJ8"
    "RoJC3BwgqQ8fvv18/uzTle/ffk8HatoITDSfYdkZIMAAgGUdRF8J2cgAAAAASUVORK5CYII=")
index.append('led_blue_light_icon16x16')
catalog['led_blue_light_icon16x16'] = led_blue_light_icon16x16

#----------------------------------------------------------------------
cogwheel_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAq3SURBVHjaYvz//z/DQAKAAGJE"
    "F+joWMDAxMTIwMjICJZmZGRiALmRkfEfw79//xj+/PkNZDMzsLAwBwFF+37//nPt37+/J4DK"
    "TwP5V5mZGR9VVCQx6OmZAfX/Z/DwCAbruXfvJsOGDUtQ7AJ5HiCAWJAFNDUN0N3D9Pfv3yyg"
    "Y8L+///XDOTvhkn8/fsvQ19fW15YmE/+xYu3nu/ff2T4+PHT248f388ESleHhCQwMDMzgT3w"
    "+/cvBiMjK4arV88z3L59FcUCgABigjHU1HQYEhML4RJA35oCLd8nIyM12dzc0JaDg3ML0LdJ"
    "EMt/m3NxcTnr6CgzmJqqMfj6WjJERrozWFgYCz9//gQYMgwcf//+Aar7Cw41WMhFRaUz8PHx"
    "ozgAIICYgIDBzy+aISOjiuHXr5/gaAFqaOLh4Tlibm5k7+FhxWBlpc3g5GTJxs/PNxfoiO1A"
    "X60COoxJTAxhGCsrIwM3NwcDOzsHN5DLhi2+QeanpBSjiAEEEIuxsQ3QAmeGb98+g+McGJ8u"
    "QFfW6upqAYNNFRyMf//+Z1BXl2Hg4LBlePnynYeYmCCDoCAPAxsbE6phLExAR/CJqKvrOgC5"
    "m/78+aP679/vUGBsawPTVR5Q7C07OzuDi4svw+nTR8F6AAKIxdnZl+Hnz+9gDjMzKwtQYYWo"
    "qCjD58/fGN6//8wgJMQLToT//zMxyMmJMygqigMTKfYULS0txODu7sQpKiqy5uXLF+f4+QX0"
    "VFWVOL9//8lw5colPqASP2Do/hcRkWAIDo4D6wEIIBZOTi6kVPlPioODw4GLixMYZ38YHj16"
    "BYwzLgZQNAETHVQNJIcwMzNgOISbm53BxkabQUFBkvXp01fmSkoyDKKiPECPfGf48OGjz4MH"
    "d5qAHqwFpQ9paTmwHoAAYgIlDkTK/vvo06ePO758+QK2FOggoEP+gbMLiA1yBCRLMgDFGYCp"
    "+z8DtmJERkaIwdxcA2w5CAgKcjJ4eTkC+WI1QG4YKysb2AwQAAggJmh+FAb6aicwb58COkLr"
    "1auX4LgH+gTsW5DFwJADa/r8+SvDzZuPGS5cuMlw7twthosX7wFD6jXDjx9/8BY44uICDMrK"
    "asCQeFd7+vRh5tWrF4LFAQKIBeLzP+lSUopu2tpqwFTMCs42vLzcYDYoK4EKJiYmFmACfA8s"
    "UJ4CU/NvoAOZwRgUvx8/fmF48eI9g5KSJIOICC+G5Z8//2A4evQqsBy49HfHjrVzHz68x25q"
    "av0NJAcQQCzAoBXj5OTOEBQUBAaRANBiLnghCbMcmDwZPn36CkxI94D0Z6Cl30C5hQGUfoBZ"
    "E4h5wY6+d+8ZMJSkGYSFeZDSFQPD+fN3gaF19s/atQsmAi1fChT+bmvrBJYHCCAWoO8DgKlV"
    "loODFWzZr19/wEENSgOM0IgCJbinT18Dg/v87xMn9u168OD2qd+/f38XE5NQ0dY2ctfXN5VX"
    "VlYC5hg+hufP3wI9wQnMoswQbwCNAJaWwNB78u3ixTNzWVhYX8vJKQJzHrjMYQAIIGAIMOiw"
    "sbGDOd++/QSmZCZQOQ/2EcgR//4xghPihw+fGS5fPn1+69bVqaD8DCpXbt26ynDixEFZMzPb"
    "Sh+fiEwTE1MGMTEBYJR8A4YmIiokJYWBaUCaz9bWJVdBQWmiiIjYzV+/foHlAAKIhZHxvzo7"
    "Oxu4gnn8+DWwsGEDFzKCgggDQCEBctCPH99ABcZLUEkNkwNm18fHju0H1hcMP7m5eQr4+EzA"
    "oYgM+Pk5GYKCvBlUVJQzgekg8d27V9uBnqsHSl0GCCAmVlZWfVBcgrIGKFH9+PGL4fXrj8As"
    "9gec/UAWQwzhBWYvRU2gRsGwsHgGfX0ToG8lGHh4IA5VVFStePDg1p2XL99AQw7hAGANyaCh"
    "IcMQFubCkJ6ezKGmpht45cp5cL0CEEAsr1+/OH/nznUPoOHAuONjAIUGyGJQdLCxsYLZoIJD"
    "XFwIWFvqidnZuRVIScnVysjIA336AxyX7969AYaY0M/v3z9vffXqVT7QT+DEB0zD4PSDGhpc"
    "4ELu3r3boGhkAAggljlzJvgICYn4qKtrR5qY2HiYmtryCwkJgrMaKP+Dgh+UBjg52YCFiwkw"
    "Lbytfvz47ndmZpY+oEd/gEJOSkoaVJyrMDKyOIiICAHTESdaCCBnye8Mb9++Zvj69fM9EB8g"
    "gFiAdfXfly+fbQT6diMwKqYBq+VMeXlpYBHMDQ5+WMPk929QWS/K4OnpwXj48JHWu3dvRv/8"
    "+W03UM0LoAIVHh6BQEtLGyEbG2NwQwTkaFDQg0rM58/fgz0DCtGnT98As/L7H1+/fnkKcgBA"
    "AIELIj09Y2B16ynHxsYZCKqIQAkQEvT/wCUiKDhBjgGlC0lJEYbAQF+GJ09MtJ49e6b148cP"
    "YNTxAtOAAoO8vCS4Rvzz5y84HYDSFCj3gCwFNUxApenNm3eBhdajZz9//ngMshsggMAOcHb2"
    "BMVlpaqqrgQbGxvDly9fgYmLC5oI/0MbDoxgNsgRIEepqMgBGzGK0IqJEZzfIXXHf2jh9R8c"
    "3DduPAJXYMDEDsyenxmuXz/LcOTIvslAs0AhxwAQQCwqKurAku2rgqCgWKq4uCTQkt/AQuM1"
    "0Ccg37AAc8V3YIL8ziAgwAsunmEh8///b6BFf6E1JSO0uIa1JUGF2H9wefLixSuGM2eOghz4"
    "FyjPfPDgjvXPnz9bBSp2QA4ACCAWOzsXkM++fPz49i6wJlQTEREGF8Fv374HxhkH0Me/GUDB"
    "/P37d2DC4QGWagLguIQksn/QcoIJKdUzQkMNEueGhpoMp04d+L9+/ZLOd+/enfz06dMjoORr"
    "WKIECCCmJUvmgOg3a9Ysibxw4dhXUEME1GphZmYD+vwHuPYD+QTkO1A9AConYOUDyCJQfH/8"
    "+JXhzZtPwILsJZB+D62+IXJCQgLAMsOcUVZWwfDjx4+bgOIXgfbB2wAAAcQCbQcyvHr14tzm"
    "zSuzBASEF2prmwIte8Zw//4NcDtOS8sIWLSKQNMFI7RqZgRb8vPnb3DLCcQHZi0gG5LiQSH1"
    "7dtXhmfP3oLMBlVgP6FtxV/I5QJAAIETIawuuH//3qLt29foPX36IPbGjcvHbt26thlo8B8T"
    "E8tyF5dgLWNjE3AhAiojQKECKr5BIQaqkkHphJ2dCZxAQWno16+/4Ci5e/c+w/Hju5+fP3+2"
    "B7kIhwGAAAI74OzZE3CB06dPlJw7d3ohMB2ASqp3QPwDmD4+fPjwaiOkZfsH7GtWVmZwAjxz"
    "5jwwZZ8Dh5SBgRUwdyiBC6JPn74z3Lp1E2jevj9Hj+7vBDr8PKjqQHcAQACBHXD8+EEUQaDl"
    "l5H5CgqKz0G5AZTQgCUg2COghPfkyUtgDXny/6pV8wqA8f3i5cvHwJ5SoLSEhDTDpUunQD6/"
    "ePbsqQnv33/YBUv16AAggFiI6b8BXf/m8+d3f4EhxczPLwQszwXBldDt29eBluzb9ObNu+Wg"
    "lL19++bLwCy9EtjqVTx+/MDsu3fvLQCKg4rcL7jMBgggRmIcEBERzXjo0AE3YNB6CAuLqouI"
    "iCuLi8tIv3v34v3WrRujgWniCCx+gWlAFFh+6P78+esqtN2As7EISsQAAcRApANQWt+gWOHg"
    "YHcCZlc3IFuY3J4xyAEAAcQ40N1zgAADAI0/xrdshJopAAAAAElFTkSuQmCC")
index.append('cogwheel_icon32x32')
catalog['cogwheel_icon32x32'] = cogwheel_icon32x32

#----------------------------------------------------------------------
lock_unlocked_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANeSURBVHjaYvz//z+Dv38CAwz8"
    "+PGdkZubPdLAQK9QXl5JjYWFhfHhwzv3jh07NPXt25dzmZmZ/8HUHjlyhAEggJgYkADIMFFR"
    "kbqQkNClpqY2Jl++fOf99Okzt729u35eXsksaWnZ3r9//yJrYQAIILABzCysYMzKwhxhaWlZ"
    "8/8/G8OixfO3Hz6813Dv3p26fX3tK9hYuRmio5NzOTg4o/7/BxnyH2wAQACBDfjPzMHAwMTC"
    "wMbBJXbl+u2DW3dsmfn23buU3//ZLn7+9efanQf3UpavWHBMSVGdWV1dO+nPnz8Mf/4zgw0A"
    "CCAWEGHJeorhI7s0w9VvSpM+PH016e+/PwwGYt8ZpDmuMnzn+MFwm4n967X7T/a//fjFSlpO"
    "Ve7ImbOyTqIfHoP0AgQQ2ACx3/dEFXjfCqsw3f7znwcUEsxM3Cy/GNmZ/jEwsbEyqHF9+cT8"
    "hIHx9fMXDAy/f3H7Sr/R0uT++gWo9T1AAIENEJKX3a5qoKTz78/v32D/AY34z8jEwMIG8uE/"
    "hr8/f/5TNmFhY2FexqCl+l2SRUl54/3rr24BJfUAAghsgKKOrLFmSAADw7dv7Ax/3gL1fAYa"
    "8pbh/f2bDP/+fGEQluZlYPgMVPrjIQMDIxDyC7AzsUrqgvQCBBDYgP+/vv9l+PScmeHHNwaG"
    "P+8YGNg/MVzYcoghofEew6/fDAyzSngYbGykgBYwAg3/w8Dw6wXD/5+i4PgECCCIAf9+MjD8"
    "fAqU+MHA8Pc9MEbeM7y4+5bhwzOgeUD557e+MzCYPAe6AGgAKB0wsTH8/8MPjgWAAAIbwPAP"
    "mLj+vAFioAG/gQYwf2GwseRgcBL6yPAV6AIHI2Cc/wOGGcjOf/8hrvj/B6wVIIBY4KmB8SvE"
    "74yfgJKfGD5c/MIgDhT6BTT7000GBlEjTqDBDBAXcLMB2ZBEDBBAEC+A2UC//wcFHtCmr18Z"
    "hOUYGfKauIGxwcjAJwB0OhM7AwMfI8IAJkhCAgggiAuAUcXABIxjlu8Q01iYGDgdhRg42Rgh"
    "0kBvMPwE5SEQH2gAFzvcBQABBDaAGRjDDJICwFBmA6VriEJGBlTwH8b4B3YBEycb2AkAAQQ2"
    "4OGND1c4t4ko/PnN9QehEBf4zwBKUg8uvX+uHszAABBgAAXnNhsIHBAWAAAAAElFTkSuQmCC")
index.append('lock_unlocked_icon16x16')
catalog['lock_unlocked_icon16x16'] = lock_unlocked_icon16x16

#----------------------------------------------------------------------
folder_orange_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAGISURBVHjaYmRgYGDaEMLwlwEN"
    "BKxh0AZSt4H4HxBjyMMAQAAxADX////jIwYGiQPlnYCYC4hZoJgZiBmR9QMEEAuYfHwKw2D/"
    "JR8ZNjDw7yXkMoAAghjw/R1E9j9QjJEJQt/cBjYEw8UM/FeBhjgCmUdBBgAEEMSAL68YGF7f"
    "YNi4eCoDMQDovf0wQwACCGLA11dgzdhsxGkIAz/IEAuAAEK4AATuHYDQ//5A8N9fEPz7BwPD"
    "nx8Qr52ew8BgGAMzxxUggOAugIcFKAxAmv9ANYM0Ihv26wtQ3QeYAcIAAQQ14A2E++0dxBYQ"
    "hrkCZMAfJINANCzQgdYBBBDEgG9QA358QBgA0gTT8PcXpiugACCAWOA2w8ICpBkWDjANYDaS"
    "i34iDAAIIIgBPz8xoLgEZAhMAwzDDP77Gxio32D6fwAEEChZagHj9SoDiQAYhauA1HqAAAIZ"
    "wArEhkDsDMRCROr/DsQXgHgvQADBMgaIZocaRgwA5U5g4DD8AQggxv///zNQAgACDACcYNXd"
    "LndF8wAAAABJRU5ErkJggg==")
index.append('folder_orange_icon16x16')
catalog['folder_orange_icon16x16'] = folder_orange_icon16x16

#----------------------------------------------------------------------
mac16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAAoLQ9TAAACQ1BMVEUAAAAAAAAAAAAAAAAA"
    "AAAAAGt3g8yWn9iVnteiq+eXnsSbpNuZo9iZo9mapNl9ic4AAHU5S6xBVLJHXbNMYLZGX7hM"
    "ZbpGYrxPasA+YL9ObsU/ZcVOcco+Z8lNdc46as1OedM6bdNPftg/dtZEetgAAABDc8NTgctX"
    "hc1ZiM5cis5ijc5sk9FPbJouO051mtZuks5rkc5qkM5OfMYAAAsAAAAAAAAAAAALDiLa4fzC"
    "zPTDzPTEzvbDzPWco8RIS0/v+f/l7v/m8P/i7P/g6//w+f+ruPGdrOyfru57h8uZqOervf4T"
    "FiHV4vzX5P/F0uyTnbHV4//O3v7V5P+Im+p6kOaFnO4wOIt5j9yDmuEzNjrZ6f/Q4P+jr8sb"
    "HSHT5f/F1/7K3P9ogeVif+RqiOdvjeh3mPA6THqNmK/O3v/G2P+2zf2nwfqevf+XuP6au/84"
    "XN5PdORXfuhgietsmPoLFSnY6//I3f+0zP+wyf+rxf+jwP6cu/4mU98uYOJEdupQhO5fmf8A"
    "AAAwMTYqLDWyxO270f+vyf+kwP6hwP8yYeM8ceo4YsxWkPVjof5vsf92u/8KGi7d7v/R5f+/"
    "1P+Bkrivyv+oxP87bOdIf/A5X78qQpwpQZoxTKQ+YLoJESQxNTwXGR0jJi1wfZi30P+tx/9S"
    "iPBjnvlysv97vP99uf+Cuv8VIzba5f7j8v/D1/+2zf6wyv9qlPF2pfeEtf2Owv+Vxv+kzv+5"
    "3f9SYXKzusfj7v/Y5v/Q3//H2f/F2f/////KAhpNAAAAOHRSTlMAVWAB+Rbq/fv7+Pr7+/3u"
    "HY+djJ6MnoyejJ6MnoyejJ6MnoyaF9Lg39/f39/k9tzf3+DVG/tUTk2P59UAAAABYktHRMAT"
    "Yd/4AAAACXBIWXMAAAsTAAALEwEAmpwYAAAA7UlEQVQY02NggAFGJgYUwMwCZbCysXMAAacF"
    "FzcPLx+/AIOgpZW1ja2Nnb2Do5Ozi6sQg7Cbu4enl7ePr59/QGBQsAiDaEhoWHhEZFR0TGxc"
    "fEKiGIN4UnJKalp6RmZWdk5uXr4Eg2RBYVFxSWlZeUVlVXVNvhSDdG1dfUNjU3NLa1tFe0en"
    "DINsV3dPb1//hImTJk+ZOm26HIP8jJmzZs+ZO2/+goWLFi9ZqsCgWL9s+YqVq1avWbsufv2G"
    "jUoMyps2b9m6bfuOnbt279m7b78Kg6qauoamlraOrp6+gaGRsQnCd6YMaMDMHEwBABg5SbqP"
    "M/4bAAAAJXRFWHRkYXRlOmNyZWF0ZQAyMDEzLTA2LTAyVDE2OjAxOjIyKzAyOjAw9OZXtQAA"
    "ACV0RVh0ZGF0ZTptb2RpZnkAMjAxMy0wMy0wNFQxNDo0MjoxMCswMTowMNi3z7kAAAAASUVO"
    "RK5CYII=")
index.append('mac16x16')
catalog['mac16x16'] = mac16x16

#----------------------------------------------------------------------
incpriority16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAKfSURBVHjaYvz//z8DJQAggJgY"
    "KAQAAcQCIhhNZ2HK/P/PyfDj7y5GRkbO/+zMdgxMzN8YGFHt+38ykQEggKAijKj4H5B6873S"
    "y0bWJsJT2Zjh3bc2BhZORgY+CQYGPkkEBgKAAMLuhR9/zISFOTO7Sq0YAqItGXjE+FMYfv12"
    "YQCFFzIGAoAAwjTg3382hu+/pzcWmokwiQszXPrCy6DjZsXNyMA4GSjHC9b47y8EAwFAAEEN"
    "+A/Bf4Bu//yzzd9dycg7QJdh4zUGhgcvGBiYJVUZ2OVU1Rn+/etj4OBhYBCUZWAQVgTrBAgg"
    "VBf8/usuJcaVU5ZvxXDsBQvD+dvfGW7f/8pw5zUTwy8JPQYGLt5kBiaWCAZ2bgYGdl6wFoAA"
    "ghjACAy4v/+FmH//7ajIs2T/wCfMsPfGP4aDG44x3D16goGH6R/DPxZuBiYpTaBCxiag8+UY"
    "/v8DawUIIIgBf4HOf/+9LshbzUDDUoNh700GhvNH7zC8fPDi/K+PH48rMzxiEAFayCIozsAu"
    "LKUKdGkDAxNEK0AAQcjPP7wUlARzgmMtGU48Y2K4cv0jw7VjV78zsLJmff7Dlnzl8pMPSkLf"
    "GPiALpdVk2Lg5OdOZPjPFg7SChBAYAP4+Tn7s/IdmN/x8DFcuv+f4cze8ww/v36bxsDKfIKB"
    "+f/1Z69/9b2+/4RBXvQfg4YKK4OVlTgDGydLL0gvQACBUyK/GI/ak68sDDe2v2C4dfUlw7un"
    "L88y8HA3gdMVnxhIWf/DJz9c//O8teVl42NgYWZm4OTjkgbpBQggRlBmYjWbfYaZkVX7DzM7"
    "w18mlo8MLKyRQD/uB8e5sBIDOMCY2HQZ2bi2srJziDKwcjL8YWG//3cquxZAADFSmhsBAoji"
    "3AgQYAAsQtEvrVL0fwAAAABJRU5ErkJggg==")
index.append('incpriority16x16')
catalog['incpriority16x16'] = incpriority16x16

#----------------------------------------------------------------------
folder_red_open_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAIASURBVHjaYmRgYGCaxMPzlwEN"
    "5H35og2kbgPxbwY8ACCAWECaM3btwpRxc7sKNMQZyDoBxL+A+D8Q/4PScAAQQCwg4u+1axj6"
    "wYa6ue0l5CqAAAIb8P/LFwjvP9BwRkYw/efkSQYMl6G66jtIB0AAQQx4947h/69fDIxsbGB1"
    "PzdsYFhy7x5WPwO9DHbVpT9/iub8+DETIIDABvx78wbiAJB37twBa05fuZKBUVAQZ+DNcHPr"
    "A1LXAAII4oL37+ESfy5cgNDnzwOD7hfD/3/AcPsN9PKfPww/V69mYPP1RTZHDyCAIC6AGvD3"
    "/n2GZd++MSTl5DD8//iR4T9QE8gQhr9/wez/P34w/P/wgWHBxo2gAN0A1PIYIIAgLgAKgsDH"
    "168hXvn0CWwjKFzAtgNdATLgJ9Ag1s+fYbY/AuJDAAEEccHbtww/gAG5Baghwd8f7CWw00Eu"
    "ABoAZoMMAapduG8fzPbjQPwKIIDABrwHGrANZBvIMKDTQYohAfEH7nwQ+8f//8i2HwapAAgg"
    "sAEgzQHs7Ax8lpYM/0FOhNoIThdQl4AMAamD2g5KB9wgvQABBEw1DFrAuL3KQCQAGtAPpHqB"
    "+BkouAACCGQAKxAbAjEohQkR0A9KsgeA+CjI+SABgABihEqAaHaoYfjAX2jG+gMTAAggxv//"
    "/zNQAgACDACjHgKOxdwECgAAAABJRU5ErkJggg==")
index.append('folder_red_open_icon16x16')
catalog['folder_red_open_icon16x16'] = folder_red_open_icon16x16

#----------------------------------------------------------------------
computer_handheld_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQ0SURBVHjaYvz//z8DLQBAADEx"
    "0AgABBALMqeioqLV09Mzk4eH59e/fyCvYPcNI5zByMDEyMT47fs39gULFmwB4uI/f/68AkkB"
    "BBCKwXp6eqE2NraCe/fsY3j77h0DCwsLSC/cMLA1/yEcEP/v378M/PwCDG7urgzXr18PWLhw"
    "4Qag8FqQMoAAQjEYGN7/Xzx7wXDvyTMGCSVlhj//mBl+/f3P8OMPA5BmYPgJo0H4HyPDk69/"
    "GdjuXGMwNXrF8O/fP5Bd3DCzAAIIxeBfv//8B7lCRkGJQd/SiuH7b0aGH78ZwAaD8Hcg+zvQ"
    "UJDYNyD/71cGhn/XfjH8//eX4e+fvyB3/YOZBRBAKAb//fMHZDMDUA3Dh08Qw379gbgU5Mof"
    "MDYQfwHiT0CDuYCB8u/vP4bfQL3IKQwggFBSxe/ff8CK/v4HuhRo0O+/EK///gfBf0AYqPf3"
    "f4jBX4F8kNq/ID1AnyIDgADCMBikAKgO4n1oeIIM//UPgkGGfwXKffjFwPDxF0QMpOcPUALZ"
    "xQABhGLwnz+/Id4CKv6OFAS/oEECMhQUtu+ABr4H4g+/oQb/Abn4D4qLAQII1eDfINdCguL7"
    "H0SkwYLlG9CgN0ADX/9kYHj7C8L+AUwM/8Eu/oviYoAAQok8cAQAXfzrLyPDJ6Cm/3+hYfoX"
    "4v3XQLFXPxgYXgINfv6dgeEeMPKU/wAjD5Qq0MIYIIBYUIPiL9DQfwxcjN8ZJFk/A10ODHOg"
    "K34CffEVSPMBVUuw/2P4+O8Pw1dmRgYLXi4GnkcfgMlQBJTcUAwGCCAW9DD+BXSiIMt3Bk2u"
    "dwxf/n9l+P7tG9gn//7+hroMqOb/TwY+fl4GEUEBhqOv3jB8+aUMVINqMEAAoeW8fwysrMDc"
    "Boy9D99+Mfz8+R+cKX4D09cfoIX//jMBDWYBhjswTIFB8uvzX2BkMjKwsjAzoJeSAAGEYjCw"
    "4GHg5uZiuH3nDsOFixcZWNlYGUDqmZhYgOK84ILi85dPQNdDIuo30IfcnJwMXJwcQDWoBRVA"
    "AKFwQZKMjP+AGhkY1FV1GdhZuBk4WLkZNNR0GF48e8Lw6sULBm0NfQYeTj4GQT5hBl1NQ2B2"
    "BpVIf4H6GFEMBgggFvTyWUxMhEFbW4NBUFCQgZ2DAZzFhYS4GGRkpMCaQey/fyXBvhERFmHg"
    "5GJmAOlBBwABhGLw7du3GV+8eMegrq4HDN+fDOLiMpAI/fWLwcJCCOx9NjYWoEHSDMzMrEBx"
    "kBo5hpcv3zLcu3eXCTkEAAIIxeAdO7atunnzZgYwPFlYWdmYODm5mN6+fc38/v17RnZ2NnAp"
    "DDKMl5fvv6io+N8fP378+/3759+vX78yPXx47y7QiBcwswACiBE5NoFe5QRSZkAsArWUFYqZ"
    "0eIDVDyC8vBvKAaltZdAfBFo3meQAoAAYqRVZQoQYABNejJuj5JELwAAAABJRU5ErkJggg==")
index.append('computer_handheld_icon22x22')
catalog['computer_handheld_icon22x22'] = computer_handheld_icon22x22

#----------------------------------------------------------------------
prev22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANhSURBVHjaYvz//z8DLQBAADEx"
    "0AgABBALugCjySwkDhD/R6JhYiDw828tCxNjGTcvB+fH34yH/5+Id0Q2ByCAyHExD8PnX3sF"
    "2JmbJjY48aTEmjAz/GCwQ1cEEECkGfyfwZHh3fd7elpiTktnBDL4+qsx/GXjYGBgZsVQChBA"
    "LEQb+utvDcPPP3UhwTqsJblWDN9ZOBmuvAYKMzKBDGZEVw4QQIQNZmTkY/jycyM3F5tDaaEN"
    "g5+vNsOll0wM7/8wMHAw/GY4c+EtkMHxA10bQACxEPC6HcPH76tU1cXE68vsGGTUpBhWX2Rg"
    "+A6MyO/vPzDsXH+B4cGDjx8ZeDhS0bUCBBB2gxnBsV7F8PN3g4e7GmtpgR3D459cDBP2MDAw"
    "AYPz/f2HDCf2Xmb4/u3vFaChHkDVT9GNAAggbAYLMHz9tZ6Dg80hO8uSwc9fn2HNOUaG3VcZ"
    "GDiZfzG8uXaZ4fGNh38Z2NimM3Cx5TKAMhgjRhAzAAQQpsG//1xWUReXKcu3Z+CXFmMoW8nA"
    "cPYRA4Mw+1eGR5eOMnz78PkNAydXGNCw/dA4wGowQABhGvyPgZOXl53h5T9OhgUHGBjOPQEJ"
    "MTB8/PqfgfEfKAGw//rLzPEaaDiQw87AAEwdDKwcGMYABBBmOmZnMTx//snlxupNDC9u32PQ"
    "UgC6lpeB4QcjD4OCtT2Dl4OCFAfD/7MMv/9XgiODCbuLAQIIWwZ5zMDBrvfn75+p9w+f+Pfy"
    "/DkGcYE/DJIiDAy337EzPOHRYfDw1mYT5mNtA6btg0D1fNjiHyCAmHAkM6D3WHP+s7GHv7j1"
    "6OPD42cYeBg/MUiIMjBcAIb3iTfiDLqWGgwysnx2DH/+3QCqtkI3AiCA8GdpRqY1wMSv/fn9"
    "50sPTl8GBvRrBjlJBoa3nxkYTj3mZuCSVWTgEhIAijBuR9cKEECEy4r//58ysLHq//7LOP3p"
    "tQf/vjx9yiAp9I/hPzBY779hZmDk5APFKC+6NoAAIr4QYmbK+s/KFvP2+fsvbx89ZRDk/A2M"
    "CgaGb7+ANjCzYdQWAAFEWunGyLCcgZVN7evXvxdfPXrDwPbvJwMnBzPIYIxkARBA5JTHzxmY"
    "mQ3+/Gee+u7N9x8/f/75B4zoC+iKAAKIkVZ1HkAA0azOAwgwAN92AsAHZtPtAAAAAElFTkSu"
    "QmCC")
index.append('prev22x22')
catalog['prev22x22'] = prev22x22

#----------------------------------------------------------------------
hearts_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANtSURBVHjaYvz//z8DJQAggBhD"
    "NDTAjH9ALMDMzMD+9y/Dz1+/GD4xMsobiosvM7a0NP3748ffuVu2bN/4+HGMECfntzhVVYbL"
    "nz8z7Lp7lwEggFhQTAPiH79/M3x7/TrAVl9/YnJnpxy3pibD/9mzWXV+/Qq8z8y8WFpJKViA"
    "nZ3h38ePYD0AAcTyn5ERrBEEvv36ZWAiK7vSysNDSdfSkoVbTIzhz+LFDC8aGxkEfvxg8OXj"
    "cz/+40eloIqK9veXL9cBtawDCCAW9u/fGUCh8OfXLx4rc/P1KbW1ChwCAgwM9+8zMBw/zvB7"
    "wQIGxm/fGN4DLTISFub2a2lp1WRmZlQ6fz4smZExFSCAWDiAmv8C/S3AwRHmERCgwMHKysBw"
    "4gQDw6dPDAynTzP8ungRHD4/gOJaEREMqrKyjN+yshj0Hj9mVePmTgAIIBbunz/BBgixs+uI"
    "A9lv9u1jeHHtGgPQGAb+Q4cYmP/9Y/gDcqGiIoOoggLD/3nzGP5euMDwlo+P4fnv398AAojZ"
    "UEKC4TcTE8PzX7++cj99Gs8hKsok4O7OwMrGxvAQaMA3YGCxcnAwsBgYMEgBo/z96tUMv4De"
    "3sbL+3fFly+9AAHEwghUCIwKe1kOjhpWISFG9chIBmagn9+0tTGIPH/O8FpQkIGdk5Ph5Zs3"
    "DCy7djEwff3KwCIqyrDn589Xf/79Ww8QQCwM//9H+gcHL3Dx92djBTqXAajp16JFDN8PHmRg"
    "B0YXm7Aww6Ffv/63nTu3H5hOHshzcpqK/vkjefTz51VAn70BCCCGedHRj//fufP//5kz//+v"
    "WvX/f3v7/y+iov+fMjD8fyUo+D9DWPgrMKAnAV0pwssIi3AGISDmAjEAAohFT05OnOHBAwYG"
    "YMAxfPnC8HvvXobvr18zMAEVPwf6fdu7dyd+MDDkgRR/RiT7dzAGQACxfLhy5TeDhAQrw8uX"
    "DH/v3GF4D4w6YAQy8HFzM5xkZv736NevVfjyAkAAMX969UrP4vNnHQFg2v4PjJ7vHz4w/AXG"
    "CivQ75O+fXtz/ft3kO1fcRkAEECMnExMPBqMjHPteHhcFdjYeORYWVmU2NkZXwMNiX34cM3L"
    "P39C8bkAIIAYeVhYGL78+cMgysgoCfSYMtBAPWBo6/3/90/g6Z8/E4BqTuAzACDAAHTRQbrd"
    "G9xMAAAAAElFTkSuQmCC")
index.append('hearts_icon16x16')
catalog['hearts_icon16x16'] = hearts_icon16x16

#----------------------------------------------------------------------
folder_orange_open_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAHSSURBVHjaYmRgYGCa583wlwEN"
    "JG1l0AZSt4H4NwMeABBALCDNiZMPYpGyvwo0xBnIOAHEv4D4PxD/g9JwABBALGDyxSUM7RBD"
    "7fcSchVAAEEM+PUFwvsPtICRCUI/PMKA6TIUV30H6QAIIIgBX98wMPwFupKZDaLu8iqG+Ycf"
    "Y/Uz0MtgV517wVA05SzDTIAAghrwCqHizS2w5sSu9QwMnEK4Qy/Xvg9IXgMIIIgB394hJJ6e"
    "gdBPTjEw/PkF8Q7Idf/+MDBcWMLAoBOCbIweQABBDPgONeDtHYb5F/4yJKbnAsU+QDTBNIPw"
    "nx9gy+av2gwK0A1AHY8BAgjFBR/fvYYY9ANJM8wVQP7P338Z2H9+gtn+CIgPAQQQ1IA3DD8/"
    "v2VYdxMYfWG+EAOhmsCGgNhA/A+YAuZvPAiz/TgQvwIIILAB79+9ZdhwiwFhO0gDCMCcDsU/"
    "/jAg234YiP8ABBDYAJDmME0GBm5VWwYGkBOhNkJc8Q9uAEgd1HZQOuAG6QUIIEYg1gLG7VUG"
    "IgHQgH4g1QvEz0AJCSCAQAawArEhEINSmBAB/aAkewCIj4KcDxIACCBGqASIZocahg/8hWYs"
    "eGgABBDj////GSgBAAEGAAz00JvD7EHuAAAAAElFTkSuQmCC")
index.append('folder_orange_open_icon16x16')
catalog['folder_orange_open_icon16x16'] = folder_orange_open_icon16x16

#----------------------------------------------------------------------
books_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAAoLQ9TAAACH1BMVEX///9gAABgAABnCQm1"
    "Y2PnhoblgYGsV1dUBwdgAABgAABgAACnXV3gkJDgeXmjTk5OAAByDw88AACdJSYnAAAAMkR7"
    "OD86AAAGP1+fLCwtAAAPUokeAgIAAAAAAAAQV4YAGSEDNlAVYqNKfkoYTjUAAAAALjAqT0sA"
    "AAADNTQzUVAAMzIVTz0AAAAAJiUSRzQ9ilZys40eaEYAKCIADg4AJSUYUTlEk1xUmXEpdFAA"
    "LSQAEhEAAAAAGRkADg4AAAAAAAD9kpL/gYH/eXn/cnL/amr9c3PBYmPniYr/kZH/iYn/goL/"
    "enr/c3P/a2v/ZmbzZGTKYGD/UFDwW1zLVlfPb2/ufX7/e3v4amrdY2Pjjo764uLhxcX/hoj/"
    "cnP/XV7/UlLqWVq9TU/fgYH31dX////lwcFBhbZbe6mhOj3caGn/mZv/hIb/cHHIRET43d3b"
    "hoYgeOAdcsgseblEaJJiO1SoPUDkb3HNaWr65OTgjo6rP0B2Oz8rlPcnie8jf+YfeN8bb8Iq"
    "c65nc5yOTFSSSEi0oqLo6OiuwdElieMvnv8qk/YmiO4qb7Hr8vyYt+M5bZotf0sUV2oYZ6sn"
    "jeouf8Hv9PyevedJe6wbWlA6gEBep3CmzbWD6Yd97X1k12gtgUsVV2sYVGY+f0VbpGqYz6vt"
    "9/BUp2ia+puX+peH8oh36nho4miHuZr9/v2/4stzz36h/6KL8I7F2s/M6dYG0gF5AAAAQXRS"
    "TlMAAwRM0f791lsFAT2//PzMT4x20k4t516c7l7kdxQC1UR4+/ydA06/DVWuI9cJN6by/uOR"
    "NEWx9f7rnD0OAhwPAcGjvgsAAADxSURBVBjTY2CAAkZGBiTAxMzCysbOwQnhcXHz8Do6Obu4"
    "uvHxC3ABBQTdPTy9vH18/fz9AwKFgALCQcEhoWHhvhGRUdExIgwMomKxcfEJiUnJKampqWni"
    "DBLpGZlZ2Tm5eUBufoGkFIN0YVFxSWlZeUVlVXWNjKwcg3xtXX1DY1NzS2tbe4cC0EhFpc6u"
    "7p5eoIa+/gnKKqoMDGoTJ02eMnXa9BkzZ82eo67BwKA5d978BQsXLV6ydFlqqhZQj/byFStX"
    "rV6zFqhp3XodXaCInr7Bho2bNqduMTQyNlGFeMfUzNzC0sraBsm/tnb2DmAGAEiLRfXko8Ge"
    "AAAAAElFTkSuQmCC")
index.append('books_icon16x16')
catalog['books_icon16x16'] = books_icon16x16

#----------------------------------------------------------------------
clock_stopwatch_icon128x128 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAIAAAACACAYAAADDPmHLAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAEmzSURBVHjaYvz//z/DKBi5ACCA"
    "mEaDYGQDgAAaTQAjHAAE0GgCGOEAIIBGE8AIBwABNJoARjgACCAWcjQxMjIOSs8kMDCw/WNg"
    "mAfEwX8YGH4zgZwKci80pcNoGJsRhxguNiuQ+sjAMHMaA0MnkPseiH8PxnAgpWcHEEAswyk1"
    "/2VgMAd6PVpJTo5BRVGRg+HPH4a/QMzEygqOQGZgwPz9+5eBkZmZgRmYNECR+wcoD0rQLEAx"
    "kJr/QHlQ8MH4zP/+MfwFYiYWFoZbly8z/Pz4MVmKgeH8MwaG3UDpVyAtQznMAAJwU8YoEAJB"
    "EKwzWhBRQbjFDY39go/1Q35BEBH3olNuwQXBvmdoMNDQwyRVzKMEiJD/oTUx0m0bPk1xzrGP"
    "I4fAfo3BWktYV34hECTGu645lb33nIJeqc90Y59nXokUKUuaoiCbJnrJsoDRd2m1Mmg+dxfg"
    "EkDDKgH8gxTTDN9evGB4CsQ3hYUZ+NnZGV4Cc+7n378ZHgMTBCcw0t9eu8bwBphAXgMjnB2Y"
    "s/++f89w49YtUAJi0NDRYWACij8A6gGWKAw/ZGQYxJWUGFgvXWL4CzQDaAcjUJwHKMUBrSH+"
    "DeUwAwigYZUAGKDZEVZ3s7CyglsBTMCinwlWrANzNRMQM4P4QBpUHYCrAJBekFpY0Q81hwWY"
    "QBiYmOBmI2V3xuEQXgABNNoLIKOUGU4AIIBGEwCJkf9/qFf6aAAggEYTAInVy3CbOgMIoNEE"
    "QEYiGE4AIIBGEwAZVcBwAgABNJoASMz9wy0RAAQQy7DOscBuHTOoGwcdugbzQaOCUP5/mDys"
    "mwfqLgK7gUwweZAeUFcRqIYRSWw4AYAAGm4JgAnWQecEYonPnxm+3brFwPT3L3iASPjHD4bP"
    "d+4w/P7+Hdzv5wWKv7t/n+HPt29gPqjv//XZM4a30IABmSHw7h0wFfwDmwGrBhiHyRgACAAE"
    "0HAbCfwG8tAdIP4Myq1fvjCcBWJ46/3nT4ZLDx/CczFo3P/648cog0cPXrwAJwR2IGYDJYIP"
    "HxhuAzEoMbyAavs3jAoCgAAaVgngFgPDGT0GhkOvGBhsnwIjiRUS8f8hxT2wFGcClu7QyIZk"
    "Y0hGRmRnRjgNgkx//vwDifyFJiBggmB8wMDw6ANkEujHcKgRAAKIkZxFoYN1OhhUirMwMisK"
    "MjD4/WZkVPvFxs/BzMzOCvKjjISCipqmjtnP/5wMfxnZGJiZgXU7IzN4WBjkHyYmZgY2YNbn"
    "Zv3PwAvELAz//i9Zv+zU129vH3H8+faF9f8/pj////99z/AfVBCcBeJjQPxyMCYCUuIUIICG"
    "VQJgDD/GyPhorwzTn2+WDGy8Fn945FQYWHllGJi5xIDSQsDCm5PhPzPaeC4jaguC+T+Y5mb9"
    "x/D9w4tX/358vM387dlNho/3rv97eebu/493nwBVAQsYhjcM4AnIQdhbISFOAQJo6CeA4L1c"
    "DH9+WAHzoRsw5qwZ/rPoALM4L2jCn4mdkYGTnYlBTICdQYyfnUEUiHm5mBkEeVkZWIE5/z9S"
    "EvgNbOi9//Sb4dP3PwyvP/5iePH+F8PbTz8Yfvz8x/D3J1DlX2BAMf//xvD/202GP19PMvx8"
    "t5fh6aHDDLdWAVuJDH+GagIACKChmQAC97Az/P1hD4zwMGCW9QQGvyQowjl5WBk0ZHkYdBT4"
    "GFSkuBnUpHkYZEQ4GPi4WMFFPgsw8wMpMI0N/AFW9n//QWkg4+uPPwyPXn9nuPnkC8OdZ18Z"
    "rjz4xHDj8ReG719+QxIE0+83DL8/HWT4/nItw73Nexie7PswGBIDKXEKEEBDKwH4bpEB2h4P"
    "bNHFMvxhUmdgYWQQE+FisNUVYrDQFGQwUhFgEBPkZGBnAwUCA8N3YDz9+gOJ0H//IRiW49G9"
    "8P8/ojIHtRRBGJRQ2IDNZE7IrDLDT2CB/+r9d4Zzdz4wnLj+nuHw5XcMr958A1oAqjZ+PGT4"
    "+Xo9w+sLCxguTb0BrR7+D/YEABBAQyMB+GzTAnorj+EPczgwFgV4BNkZHPRFGNxMxRlM1YUY"
    "BHhYwNX6Z2C7/OcfSC4GRyAQgxp27EDMygxhg0oAeARDx0H//EMkEJDeX39R8R+oHEgvOzBB"
    "8HJAmgsfvvxhOH3zHcOu0y8ZDlx8w/Dl/U9g4Pz7wvDn9U6Gd1enMpzvPglU9p3eCYGUOAUI"
    "oMGdALy2KAFtqwXW6WHAGOaSARbvIXbSDN6WUgzSohzAepuB4eM3SKSDXMQBasUDcz8PEANL"
    "fTAGRToTI2ZOB9H/kNp+jIyYJcNfoJrfwATw7TcEfwHm6a9A/OMvxAxQYuDnAiYuoAFPX/9g"
    "2Hr8GcOaQ08ZngCrCWBC+Mnw580ehhfHOhiuzTpHz4RASpwCBNDgTACe2/mBwV8KLObzgFmS"
    "V0mZnyHWTZ7Bw1ySgZebheEDMNK//oQoBRXPgsAcKQDEfOyQCP8PzdXA9hw44kAJBBSRoATz"
    "GzqKA8vt4L4jtFQAr/xlgmJmSARzQqsAkBhIHlQifALa/QFY2rz/AalmQIAbaLcAMDF8/vqH"
    "YcfJ5wyLdz1kuHf3I9Dg/98Zfj5ayXBvQyfD0933QcNRgykBAATQ4EsAbpsDgXV8N8Ov/8oi"
    "klwMKX7KDAG2MgzswHL8HTBjff8FiWQBTgYGMW5g5ANpNmDk/PoHiZjPPyE59dsfSA4GjeQw"
    "ghp+wIhkZoYM+//5BR7dRR1DBqlhg4iDRn3//IEM+TGBin6gGVwskJKFlx2a0KB2ApsEDK++"
    "AhPEd0ji4ASqEeIBDTr+Zdhw+AnDnE13Gd48B6ZY5h8vGD5c6WY417gQVHtAx5cGPAEABNDg"
    "SQBOa0QYWNn6GH4yxrICy/IQVzmGJB8VBhFgff/6EyTiQblRFBi4UnyQiADl6nffIRiUI0ER"
    "AirP2Tkgkff1E7AlD0wNT++/Z3j28APD6+dfGD4AFX/9+B3cyod5AxQEoF4CNz8ng4AQJ4Oo"
    "JA+DlLwAg7SiIAM3MNa5+ZjAiejnD8h4MyjyQSUOUCkYg0oLUMJ7BnTn6y+QkgeUEESB7nwD"
    "bBfM23KHYc3uRwy/QXXH32f7GO6tKQWWBtdpNZpISpwCBNDgSADO6x2B/ffZwBhU1tAUYiiL"
    "1WYwArbqQRH/6TukKJbiZWCQE4DU65+AieHlZ0jOA+V0JqA8BzBCvn35z/Di0SeGC8cfM9w4"
    "/4zhyb03DJ/efGbg4mBkEBZiY+DnZWbgA2JePjYGXl5Q1xAyEv7v3x+GT59+M3wGjQN8/sPw"
    "8fNfhrfvfjF8+/GfgU+El0FGSYRBw1CKwcBSlkFCjo+Bi4eR4Qcw6oDawCUDqCQSB7oPaCy4"
    "ygGNFT/7DKl6+DghCeEcsNfQtfgqw43r70ClwSuG18fqGK5NXAW0/iMDlZcakhKnAAE08AnA"
    "cUMZsJHXwszCxBrpo8SQHqwOjFFmhpcfIHWuBDBglYUgLW9QTn/8ERLxoJY5ByekOL9//S3D"
    "8T13GS4cvc/w8cV7BjlpdgYNNV4GdU1+BnUNAQYRYQ4GTmDxwQHCHMxQ9zMxvHoFmd4RE5MA"
    "xwEoLH4Ac+kPYP//OzBlvX33AxhhHxhu3vjIcPPmZ4ZHz34y8IsLMuhbKzJYuigzKGoKg6uN"
    "H98hPQ5xYEKQAbZehDggJcJdYFy/+AzJ4uICICv+Msxce5Nh+ZZ7DH9/Ayuar1fnM5yrboQO"
    "Kf8eiAQAEEADlwAcNnEyMP6dCiwEE4XFOBmq0vUZbI0kGIDxx/DlB6SOVxMFJgBgkf8RGJj3"
    "gYH59DOkTucCNrY+vfvDcGLPHYZ9a68wvH/yhkFTjYvBxlaMwdRMlEFSgpuBh5cNHMn//jFC"
    "x/oh00Dg1j5oGTgLK8Pq1UvATgkNjQHvEPr//x8DJDj+A/VB5hAZGSH0F2BV8uL5V4azZ14z"
    "HDz4kuH6rW8MgjIiDE7BOgwWLioMfEIsDN++QdoMoNJKCZho+YHV1AtglXDrNaSNwANMGBKC"
    "DAyHz71gaJt5keHtK6Dg30cHGK5PzmP4fOcWtRqIpMQpQAANTAKwXinOwMy2hOHHPxctbRGG"
    "5lxjBjFRboYnbyH1sQIwkDTEIDnn9htg5AMTxW8ghwuYwz6++cWwa9UVhv1rLgCL3N8MXp6S"
    "DC5uMgxycrzAhiILOMKZoGv/ERiSCCCYATwtyMrKwrBkyTywc2Jikhh+/0YkAFCYwPA/0NYw"
    "YHsBlCBA8kxM/4ANvD8Mjx59Ztiz8wnD1u3PgVUSK4NjiAGDW5gOA78IG8M3YAnFCrQHNCul"
    "KgIpyW68YmB48B7SzZQRBpZir78y1E4+y3DtKmhK4e01hvuLsxleHz4N5HyjtF1ASpwCBBD9"
    "E4DVSilgpb2F4cd/QxsbGYaGbCNgDcDK8BwYOFzATKstDqzrgQH3FFjUX3kJadyBIv7Xt78M"
    "e9deY9ix+DSDlOBfhqhoJQZbW0kGAWAf8P9/WEQzA3M2M7hBB4p01IiHTv0ilQALF84Bi8XH"
    "pwBLgN/QSEcEIiIRQDAoIYD2FoJoUGIA9vUZPgK7AYcPP2dYuuQew7P3zAwesaYMzsFaDGxc"
    "zOCEAGos6gD9JA2sGh4B/XgV6KdvwGpLUhA07/SboWHqOYYjR54AOZ8eM9ydl87w5sgRoPVf"
    "KEkEpMQpQADRNwFYrJQB1u+bGH7+N/QHtvALE/UY3n1lYngPLNpFgEW9kQwkEVx6Acn5oJY3"
    "qJ6/dOwZw4q+gwxs3z8ypKarM7i4SAITBRvQo7AIh2EmIEaNdEbo5D+kKIexGcE7fhYsgCSA"
    "hIQUaBWAWPj9/z94qyiWEgGREP78gWBQqfDt6y+GPbufMcyadZPhFwc/Q0SRPYOelRS4fQBK"
    "K6CSQE8CEvnngPH9BhjFgsCqQoj7H0P//EsMG4E9BQamj08Y7i/IBiaCAwzQNS20TgAAAUS/"
    "BSFmy2UZ/jNtYfj+Ty8M2NDLidNjeAas1z8BCzwZYG4wlYOMsu29Dak3Qbn+O7BFvqTzKMPF"
    "3ZcZIsNlGCKjtYGB/QOYKDjBLXhQ5EMSACNKUY+IcEakBIsoAWCJADlBI/jI6higJQIjPFBB"
    "NCihgRLfnz+Q6gGUeDg4mRisbbkZLK2tGVYsvccwu3Qdg56rLkNonjUDJy8Lw+XnwC4iMErN"
    "gP60UmRgOP2IgeEJsET49ZuJoTjZgIGdlYlh1dqbwLosYTrQGlAi2EdJIiAWAAQQfVYFGy8G"
    "NomYV4EiPyRAlSEzVo/hAbBOBA3sKAPrQwt5SB96x01IQ48DWBrcufSGoTVhBcOnm7cYZs4w"
    "Y8gvNAAG/F+GpUsXMjx79oiBjw9U57MysLGxgHMzpNhngjf4QPU8go3AID4owYAwpM7/B+dj"
    "lh7I+pjgNHhmEZjwQHaD2h0gt4DctGTJAqD4X4aCYkOGWTPNGD7fvM3QEr8C7BdOHojfQH4E"
    "+RXkZ5DfQWEACgtQmIDChuEPrxSDTPQEBl4tcwbIJlSaDrsCBBDtE4DJajYGRra5DN/+Wri5"
    "KjBkxOkzPHwFGcNXAbbyDWUhDaQ9wJz/FdgRYgcW+XtXXmeYlL2Swd2Kg2HaTBsGHV1RYNHL"
    "xCAjI8ugoaHFsGfPVmDE/QFGACs858MiCBkjlwyICGaCrA0DinMA6xcQhjUWQeIwdTAzYWxs"
    "iQGUEFhZWcHjCLt3b2XQ1NQGulEGWCowAt0sxjB9lg2Djz0n2C+7V1wH+w3kR5Bfr72E+B0U"
    "BqCwAIUJKGxAYcTwR0CeQS65l4FNSJsBsjaVZgAggGifAP7/bQdGfoCZuRRDYboxw9M3jAwf"
    "gY0jNWAr3xhY558B1ocH7kFX2zIzMKyaeJxh19TdDC31mgwlZQYMvLycwFzFBuzHswMjnJnB"
    "29uP4dOnTwwnThwBRh47vFEHomGRBotwSEQzIokzwdX+/fuHISwsHIxBbFhVgNxzQE5YyGbD"
    "7AQBkBtAbgG5ydvbH9y74OBgAzcyQW4vLjVgaG3QYtg1bRfDyv7jYD+C/HroHsTvoDAAhQUo"
    "TEBhAwojUFgx/JfUZVDMbQMqVWSArFGlCQAIINomAP2lccBiv0hWUYChLNcU2PBhBo/uKQKL"
    "PmNg6j/1mIHh+EOIK/4A69aFjbsZ7uw+yzB9pjmDh5cCMJBZgUUsOzBAIUU9KMzFxMQZ3Nw8"
    "GbZt28zw4sUzcA5Ej3hYFYDoCTChFP+wHM0JbEuAMDY5mH5YTkckKoRdoBII5AaQW0BuEhMT"
    "Bbc9QG4FJQJ20MIEBlYGT28FYOPQkuHu3rMMCxt2g/0KWnoM8jsoDEBhAQoTUNiAwggUVqAw"
    "Y2BWd2SQTSkAGiJNq/YaQAAx0TDyQUuzJoAGZMrzzRj+MHMyPAX286WA3SFzYP13AtgIOvoA"
    "0tL/C2xZL27cxfD30T2GmXOsGHSBRT7Iv6Bcz8HBAs5VoDoXFCm/fv1ksLCwYrCzcwD30RGR"
    "gj2ikYtxRI6GYMREEBNGzsdWgqC3BUDiIDeA3AJy069fv8DisMYpqI0CSgj//7OAq7HZc60Z"
    "/j2+D/Er0M8gJ4DCABQWoDABhQ0ojEBhBQoz8GAWr00Mg5CjH9CZwrRoDwAEEG26gXrLuRj+"
    "/d3J8JvBJr/AjMHJWZXhGtCjglwMDF7akJGxvXdAS3hBU7H/GVa17mJgffmAoXeiBYOUFB8w"
    "EFnAgQeLeOQ6GDx9C617QV0xyOAME1JrngFlxA+15Y+6LgDVT+hy6GMCDCjdQsgcwn94wvn9"
    "+ze4R4A8hgDpMv4Fyv1l+PnzN7it8Pz5J4aivBMMv8TlGcIq3Rj+AruboBrIWQUy8rntKgPD"
    "e2CbQAvYFNgH7BJNnHAKaOCrxwwPulIZfr04Bu0ZUK0bCBBAtCkB/v4qY/j2z8bRUYHBBRj5"
    "tx5Byi8bZcjY+O5bkPl4JmAxuHHiIYZvt+8wdPebwyMflGsgrXtmlHobMaTLAK+3UVv+qEU3"
    "ct2NymeEjhkgMPrAEXo7ALlBiWwnKFEg3ILeCIWUBqCEDErQIL9JSvKBE/qPO3fAfgeFASgs"
    "QGECTBsMNkqQsAKFGSjsQGHI8EdElkEiohAorEDt9gBAAFE/AWjMN2b4zVgKmlKNB7ZqHwFz"
    "+9cfwHpOHjLnvhGYwn/+BXoS6I0Dy88zvDh5mWHCJHNg65kfGvmInA9Zu8+AErHI/X2QeegR"
    "jxrByC121MYgslp0cfS6Hr0biZyQ0BMasjikZEEkApDfQH6UluZj6AcmghenLoHDABQWoDDZ"
    "dBUSI6CwAoUZKOxAYQgKSwZ2fWcGAccAoApRasYbQABRPwH8Z2xi+P2PKxbYr2Xj5WF49gaY"
    "bIG1l4IIMPKvAPu9wOKNlYOB4cqRhwwX1hxmaG7WZ1DXEAIGFHKxz4RSrGMb1kWONNQ6HtEW"
    "QNaL3kbAhhEJCTUiURMeE1piwu0+WCKFjDwyg8crQH4EbUhR1xBmaGkxZLiw+jA4LFihs52b"
    "gYlAUQQSZqCwA4UhKCwZ/gADSMA5HhgkaqAFSNSKLoAAom4CUJ0byvD9v5eRuTSDBbAsu/MY"
    "Mn9vDuzIHAZ2e+4APQQ6t+n1ky8MuybvZshIVmawtpUGN5JgLX1YnY8c+dgjkxGjakDtsmH2"
    "BNAbg7gaf6iRi5kYsA8WMWEtFZBLKuTqAJSJre2kGTJTlcFhAQoTUNjcBYbRwXuQMAOFHSgM"
    "QWEJClOGf9LKDCIhMUDNktTqFQAEEPUSgMoi0J6rKkZgXz0sQpfh1UdGhs/fIcXZ668QT4EC"
    "ArSKZ/e0fQwORuwMkTGqwIYRM3g0jZUVMp4PaZ0zYo187IHNhCNnY+ZUREMSWyMWWwnDiNbF"
    "RK5+mAhUB5j2QRIBE7gkAHUh//1lZoiKVWdwNGYHhslecNiAvA8aIwCFGSjsQGEICktQmDKB"
    "1sJxWfgwsAjqA43jo0a0AQQQ9RLAv9+gPr+BA7DRIqMkzvDoJQODBLBbIyPEwLD1OjDiQSt3"
    "gL2aCzuuMrC8fsRQVKYHzPUc4AYfKFeAxvaRB1lwRT72wGbAiBjkeQEKOklYcz2kjcBA1LAz"
    "Mg0r1UCJAFTagTCoN1Ncps/AAmzog8IGFEagsAKFGSjsQGEICktZZXEGR2dgsfBbQJRBwDsK"
    "aJQMA+RYRIoAQABRJwEoLGBj+P0/l42XncEnUIvhyWuIJ4zlGBjOAIuwe28gXb53z74wnF91"
    "lCErQ4NBQUGI4ePHT8Au0i8Gbm4ucKAginQGtEDEXx2gF+O0WLOIWbVgr/+xtQewlWCg0g6U"
    "8EFT2LJyggw52VrgsHkLDCNQWIHC7NQjSBiCSoZHr4BdaD9NBnbQYkg2AzsGJm5tapQCAAFE"
    "nQTw70cQw49/2pbWsgxCksIMz15DGjHAzM2w7w50sAcYJ2dWn2Kw0mFjcPWQAfaHXzFs2rSV"
    "YfHiFQwnT54BBxBo1A8WuJChXQb4Ag7MEoEJawDTGmCPUEY89T8jRsKGqQW1BXh5uRm+fPnO"
    "4OQqyWCly8ZwFhhGf6ErmfcDww40mAgaJQSFqZCUMIMFMIwZfvMJMfC5+gOdI0VpWwAggKiT"
    "AP4wpTByMDO4eKkzPH8LWVatIw0Z5Xr/FbJo89n1lwwfLl1nyMjWZXj69CHDnDmzGUREpBiM"
    "jU0ZTp8+zzBz5kKG69dvgsfWQTkDUbczEawOmJjou1MJW8MTs8RCx4iEDRuGBg1iHTq0j2HW"
    "rAngMMnK0QOHESisQGEGWkZ2DBiG2lLg5YTgsAWFMSisGdj0rYFOUYLOGJINAAKI8gQgOcMG"
    "mPvt9fUlGcQVJBievoLUXaAUfPwBxPOgIuzKljMMHo7CDHoGIsCUzwsMDG6G79//Mnz69BOY"
    "CECDQLIMGzduZ1i4cDnD69dvGLi4OKDDv7gjH3Uen/4Ad7HPhBThyBEPGbgCjdRdvHieob+/"
    "C5wAHBxcgFWiGoOWjiCDJzCMQGEFbhAyQMOQCRKmoLAFhTEorBn+SEgxcNm5ApWIURKPAAFE"
    "eQL4+w+0bYvF0lEJnNu//2Rg0AS678RDyJAmsPfK8ArYt/n3+CFDcJgSeChVWlqKwchIl+He"
    "vRsMV65cZ7h16xGwQcTDYGVlB15oMXPmIob167cy/PjxE5gQ2HE2rgbDNnXYLCT2Rh9mm4aN"
    "jZ3h7NnTDPPnz2ZQVVVnKCmpZHB2dgO3g0CJJSRcBRhWjxheAfvMoLADhSEoLEFhCgpbUBiD"
    "wprhH1Axmw6oFADWCQxc5LofIIAoSwDis4Bdv/8+gpI8DGoGcgxPgClUkBtYOgHrrVMPIckS"
    "tHz79t5LDFZGfAxq6oLgpVY8PNwMERGBDAkJwcCi8C/DtWsXgQnhJsP9+88ZxMVlGSwtbYDs"
    "p8AcMgOYQ06AAw7UVURtFzAMKoC+DA3hVtTECxo2VldXZygoKGEID49m4OcXADeEIaOeTAxq"
    "GoIM1iZ8DLf3XYJsOmaAhCUoTEFhCwpjUFiDwpyBUV6FgYkXNDDET667AQKIsgQA2qP/45+i"
    "nrEkAxMXJ8P7TwwMyqKQBR6vvjAw/Aea/vHVd4ZP1+8x+AUogPu/kIEeyESKrq4OQ15eKkNg"
    "oDMwt78FJoJLDFev3mZ48eIDg4aGLoOOjj7Dvn3HgAlhJrB9cAdlbmCwAfRIxz5gxACu94WF"
    "RYBFvhJ4ZvPfv79IQ9eQ1U2gsAKFGSjsQGEICktQmILCFhTGoLAGhTnDLy4eBnZja+hMIVmN"
    "QYAAojAB/PcC7bHWM5dneP0RklpBu2BA693AnQNgW+Xx6TsMGlL/wXU/SAUiEhnA06cgDzs6"
    "2gOLwgwGa2sdYOTfY7h8+TIwwu8yfP36l8HU1BJYKkgDewtrgA3HZQyvXr1hGKwA+1IyzGoA"
    "stT8D7wUg41fgLrCoBXOoLAChRko7P5BD7MAhakYH6RdAAprUJiD97czq+pC1wuQtXIIIIDI"
    "TwCC09mAdb8TrxAXg7SKJLibIgwslT7/ZmC4/xaykhG0l+7N5VvARo4YAw8vO7jPC1u7Bxug"
    "AZUEP3/+ZODj4wNWC8EMxcXpDGpq4gx37lwDlgjXgO2Dh/D2AajR2Nc3m2Hz5l0M3779GAKJ"
    "AFUMuU2ArhY0uASbA+EFhpWToxg47EBhCApL0LjAh18MDCK8kC4hKMxBYc/AKCPHwMgJTA3g"
    "i05IBgABRH4C+PtTi+HHfxUVdWC5xMHF8AlYTEnyQ4qqbz8hy54+vfzEwPL+DYO5pQS47ked"
    "4WNAqS9BuQKUEOTkZBiyspIYsrNB9SMjsH1wCZgQboDbB5KScsD2gTWwmrjL0NU1leHo0VNg"
    "fYO1OsBWBSAPOaOuQ2CELyYBlQKWNlIMrMCwA4UhyIegzbG3gGErCSwFQGENCnNw2P/i5WVg"
    "UdUBZUly4hMggMhPAP/+WQJbKeyKWuLgAxNAGyX5gAkS2IWF7L8HFl0vrzxi0JBlZlBS4Qdb"
    "hbxyF7k/jbwUG7TEGrTW3tBQl6GiIochMtIDWFW8AyaCi/D2gaamLoOWlg7Drl1HwO2DW7fu"
    "DeqqANmv+PwOawuAxBSU+Bk05FjAYfgPeuYBKGx5uSBhDQpzUNgz/AGddaeoCZQWIWetAEAA"
    "UVAC/DUEnb0iqyrG8PIdsB8CtBpYQoPXuoNS7E/QPv37T4ANPQFgF4cNbVEm/sABeffXrz/g"
    "RpGzsx1DQ0MRg4uLMcPLlw+g7YN7wPbBP3D7QEhIgmHevFUMc+cuY3jz5t1gSgZ4u4TYuoiI"
    "9YdMwJ4SK4OeLj8wDB+DwxKUAB6/h+yGBoU1KMxBYQ8+/4ZBVBK6ToDkdgBAAJGXALgmMwP7"
    "oQacXGwMwjLCDO+AjRJeTshqH1C/FVQq//rxn+H3s5cMRsawxh8TfBIF26gaZu5hgC+r4uHh"
    "YQgN9Weorc0G5n5Jhrt3rwETwjWGmzcfggeVbGxg7YOZDBs37gCPH1C2puE/5IQIKlUDsIYg"
    "cm5H5qMnGsjGEyZg2IkCw/AVw6/v/8GnmYBGBkGzhKCwBoU5KOxBccDwXxQY+Syi5IwHAAQQ"
    "eQng7zdehl8MiqJSfAz/WDnAq1dA27fvvoUsb/oD9NdnYFEtyP6LQV6BF16/wcbvcRWByA0n"
    "RF2JNOgoKQlsGyQB+9DxwJzPBCwJEOMHEhJyDObmVkCx+wzt7ZMZTpw4w0DOekcwWLaMgSE4"
    "GNjaekZGwfiX4enT5wyHDh0HlkwrGFas2ABup6AuJ8fuZ+Sl7aBMI6/IBw7Dzy8/gMMUFLag"
    "NRWgsAaFOSjsQXHA8JuHl4GRVwJaApDURwYIIPImEv4xyjP8/8snLs3H8Os/MwMow4GOU3v2"
    "EXLuDujsk89P3zAoASNJUJgTo1UMi2D0nIJaCuDKJQwM2tqaDBoaagxHjpwA9gj2AkuDVwzv"
    "3skCE4EYsITQA5YAXxi2bz8ElD/N4OfnDuxVKBHvtxMnGBhychh+fvrE8DElhUFMSopAYfGf"
    "4f37Dwy3b99nuHfvIbCN8hq8e5gHGCeg/v6bN6+BPZr7wDaLGjBx/AYvG4f5F5FAIVvZQFzk"
    "RCEEDDtpYSaGu8CwZJcWBJesoDBWE2AAhzko7EFx8OgSEysDo4QMsPzlgWZqoo+fAQgg8hLA"
    "/9+qwA4om6AoL/hUA9A2PFDdBCv+QefyfXv9nkFShBnYkmfHUhfiSwyMaEUidieAplPt7a0Z"
    "jI31gZG9l2H//lMMr1+/BCcESUlRcPvg9esXwFy4kkFVVQGYENwYREWF8fvrzh0Ghrg4hr8f"
    "PjA87+lhEPDywqrsy5evwMh+AMSPGZ48ecbw+fM3Bi4ubqBfBRkUFTXAYxu/fv0Fd1Xfv//J"
    "8ODBI3ACgFRrkIWkyBEPE0dd+MrIwMfPxiAhwsJwGRiWsEMsQUvqQG0tUJiDwh4UB+DZAkZR"
    "YFfrOh90jQDRCQAggMhLAH/+SYF2NnALcjF8+Q4ZjwBt4wY5DtwDAB2k+fETg5QhN5blW7jr"
    "f9zVAW4Aax/Y2JgzrFu3jeHChWvAXCcKvjFUQkIEKG4PrCLuAnsLs4FVhAGDu7sDeCEK2Bt/"
    "IOeAg+8GBN0LEB/PAMzKDC9KSxkECgoYBJghozA/f/5iePz4CbC3cZ/h0aOnwEj9DJ665uXl"
    "Z5CSUmBgYwPdUgs6ROIbw/Pn78D058+fgfQnhk+fPgDdwQG2C+LX/1C//ceSIFDXHoAys6wc"
    "N8O/M58Yfv6GqAJlso8/IGEOCntQHEBKZR5+6FgAKAEQPUgCEEBkziX/lwI5jluQm+HLN8gR"
    "a6Bj274AMQfQ+v+/IGf1i4lxgj2Be66eEW9CIGlSUlIC3D64evU6w9q128HjB2/eSAMjSAI8"
    "fiArKwdsH1wDtxlcXOwZ3r79DMy9L4E56w+DoKQIg8XmVQyKx44xvEtOYmAor2D4BuxRnLt6"
    "Axjxz4HF+hvwIBYPDx+wWJdgkJFRATdOQTn8zZsvwIh+BYzwr0D8EVj9fAXK/QKWBlwMCgoi"
    "wNJHn0FdXRXcDkA0brE3GmHiiHYCI7DU4mT4BwzL378gIj9+Q8IaFOagsAfFAaTU5+aBtgHY"
    "SAk3gAAiMwEwgfYtMbAB+yOgLd0gd4OOz/0DO4cPdHYCsO8iIsqDNhJGXMuZEgBqH4AC/MiR"
    "kwxbt+5juHTpFTCyIe0DbW19YA59xrBmzW5gwhAHRhIPuMdw8dRFhtN/uBks3f0Z3uuZM3xc"
    "vI7hLzCkQfW4oKAgsPiWAgfyjx+/gDn6K9CMp8BI/wLeD/jt2xdg3Q6atWQDJjRhBiUlVQYV"
    "FUVgCSTJICAgAE44sNLm37//aIn8Pzx8UNsDDNBdR8CWvggHMCy/gFdYgaaFQWH8BVoa/AKG"
    "PTeoTwi+4gJcrHGSGqcAAURmAmDhZ2ZhZeAC9kc+Qk/NBB2KBDto+R/IlT9/MPDysSL1ANCX"
    "ZTNh7MlHDgBcDUCiXAcs0h0crBlMTAwYtm3bzXDgwGlw++DTJwWGjx/fMOjqKgEjVQVY6r9n"
    "ePUKNC7/g+HxbxGG1SzMDMa/mBh0NDXBu5F//foNztkPH74EF+ugCP/69TNQ/Buw2GcEJioh"
    "oFmqwAapCrBUkGIQEREGbwrFHDP7h3cqGSSN2ODKALYbFj58oKPHgGH5+zfoBnMmcBiDzkME"
    "ZS1QWIPiABQXf/+ADqYD535mUsIKIIDIrQL+M4LGrdlYwKkQVJOCDj/4A71Z+R+wPmT884tB"
    "gJ8d3lKGbKuCba1CbF/CVzJQWhqApp3DwgLA7YMNG3YCG21PgAHKweDsbAWsxz+iLNpgZfrP"
    "wAYstV6/eMXwmEcEWqR/Bkc6KMJBp4CIifEzKCvLAEsYJQZ5eVlg8SwKb08QGhTCrOcZME4f"
    "gWHksOIXYAOH5W9gmIKKfVAYg8KakxtyEDYoDkBxAZ47htT/JHXtAQKIvATABMmi/6B3qIHG"
    "In59YgAflA4aPvn/F+QKRvBI3pUr5xnOnz8LzDFsKOPhoDN55OTkgQ24cJpP70pJSTJkZSUw"
    "zJy5Erz06uTJ88CS4DO4NQ8aPQSVBF+/fmMQFxcG0p+A3cpLwJzMyMDNzQrsQooBI10OWKwr"
    "AeVFwdUCpSOEsIYgCKxZswrYsHwILTkQp5CAZkoNDU0YWJhlwHpAy+z+MUHCGLRaCHQ0LfgQ"
    "639Ided/0otMgAAiLwGANrkD8X9IBwR8rBv4zFzombv/oM74B577FgUWt7rg5d+wWUBInfeX"
    "QUhIGD70S4+LuKSlJRju3r0F7DbuA7fsYQ0zLi5OYP0tDmwoSjHcuHEHWDq8YoiIAJUcZuAI"
    "p0UChZWAKiqqwHAQApdCsGYAbGIMFHbvXv9DVBegcw2YIGENanuBCp//8PhgIuswGYAAIjMB"
    "gJr1kNEqRmixBHYQK8Qd/6B9VNDRazIy8sBcpAU9NIEZZe8eKBBAu2phAyA0TwBy0gwaOloM"
    "P4DdM1DfHTRqB3ITaDmWiIgQuDh/+/YjMFLUGM6evczg6GhDg8hHjSUzM3PoSiHYUXT/wWcP"
    "gRLot29fGV49fwxWBxpoAy0UBS0QAfW2QMU/JxN0MJeRieyTuwACiNwE8B90Ht8PYKxzAHuf"
    "nxghp2n/Z4a5hYmBg4+d4c3rn+BA/vEDcsgucgJA3WGLe0iemuGvIsLL8IKZlYHTQJfhz9ev"
    "wHr1LzghgtoKvLy8DDt3HgTW+b/Aq5NUVMTB7qVFpMP8xsAAWRQDOYYOMwGAVgu9ffsT2JiG"
    "nE4NinzQsAErNJxBy+5/fPwFPhsRckQl6QAggMhtBL4DLVP+9vkXA6Mg9KJGVog3QaU8M7D+"
    "ZOJhY/gKOjgX5c5tSFEPa/wgz/4hqoD/4MERSORTt2rgndjLYLx9D8O54ioGMSNdBh52yO1h"
    "3759Bxb9txiePXsC7BV8BVYHggxubqFYW/TkzCvhk0OXRz6pFBRGoATJBQzLf8BeB+z0c/AN"
    "JtDqFxQHoLhgYAStwiD9BHKAACKzEcjwEWQVqAQA9VJA8Qgq/sGLGBkhR7P/4eJmeP3qB4pn"
    "iAksmlUHU6YwMMyazcBtbMwgr6HA8AWY++8/ewQ91YMB2J3TYPDxcWE4ffoieMuWpKQU1XI/"
    "IqL/Yy0VkA+ohPUAYPKgMGQEhiVogwhoghLUEOSAXmEDEvsEqnvBDUHQhTbgmpekRAAQQGQm"
    "gH+vQafafv3wnUGUDZJJOYHF0i/oil1Q0fRfkJvhBbBLBcrFuBMA/hKBaolhwwYGhvZ2hq+q"
    "qgyvWlsZVM1NwKMloCIWclQ8oqi3s7OkUQsEswuMLyOAwgwUdi9efGPgERRjYAQV9/8gcqCw"
    "ZoAmAFAcgLterJ9BR8ySXAoABBCZC0L+gJZ9/v/89ht4TBoUR6Adz6wsEMwGrLJ4xAQYnj39"
    "Bsxh/+Dz+ug5ATMwcIlTAI4fZ2CoqGD4KSDA8LKxkUHS3Bw+VAZZo8jMQEuAyy/YSwVE5EO6"
    "gn/BYQgKS1CYwsKXHVoFgMIeFAeQ+1HfgTri3xlIPHUcIIDITABf7jKwMP589/obeOAHVOQj"
    "X8wE6rPyiAsyfP/+D9il+oE0sPEfrUgklDv+M1CUDkCze0VFDH+Bxfyz8nIGMU9PBg4GeoP/"
    "KBGOzX+I84f+o9CgmURQGILCEhSmbNAwBoU1+FJ00IUZwDhgYAH2qf+DzlkHHzRNUgIACCDy"
    "EsD/N0+Aye/Dm6dfgI74A76KBeQ4TmhCYAI6TERehOHdp//AOuw7jnbAf5w5AXnEkOyTUl+9"
    "YmAoKWH4//Qpw7PsbAaBiAjKNtGR2ddH9Qfukg8xBvAfPnT8Chh2bz/+B4clKEzZoWEMvv0M"
    "1BMAhj0oDhiYgX3pf69A6+E+k5oAAAKIvATwY8Z7Bua/dz6++8Hw68sPBlY2SIpkg1YDIAey"
    "8PEwMAP717dufQB3AZG7OaiBgMpHbwyRVQKAup3V1QwMZ84wPI+NZWBPT2cQZGOjf97/z4BR"
    "6uHzM4wNmTT6x3D75ntwGILCEpz7oWEMCmtQmIPCHhQHDExvvgArZLISAEAAkbso9B8D0+/L"
    "f378ZXh57y0DDxe0HcAMqZdA9FdgIlbSkWI4e/oNA+QkrX8Y7QBsxSJiTByVTxIANvgYdu1i"
    "eOXtzcCQn88gxsMzAJGPLcejR/x/rBNHkOPm/jGcAYYdKAxBYYkctqCwBoU5KOxBccAAPmyf"
    "AYS/MJB4/QxAAFGwM+j7eVBae/HgA3iVKgM0AbBDGymgPquMtgzD3TufGV6//g7eEgXyHHKO"
    "wBUgmJMkJDhr1iwGhiVLGN5bWjJ8B9b7kmJiDAMBkBM3Nn9hJnjEBRWgsAKF2R1g2IHCEBSW"
    "oDBlh1axIAAKc1DYg/P7P9B5YgzvoI1AkgBAAFGwL+DZIQZWxq/P7gCLqT//INeyMSFu6vwF"
    "7JAIKMsyfPnGyHDj+ntwwoTMif/DWg2gF5FklQKg7t6kSQyfNTQYPgAjX1pJiYFxQCIfNfGi"
    "JmgGHMU/qPH8H577rwPD7MtXSBiCwpId6eZTcFgDwxwU9gwsoI0Ul0C9sjfQRiBJACCAyE8A"
    "P5Y9YGD9ff3Fo88M399/BfdJQQDWEwDfq8fBzqBuIM2wf98zsKcgFy38wwgQ5FYw9kD7j3XU"
    "DAWAFnN2dDD8EBJieF1czCBlaMjAwsAwYLkfNeHi8idqSQDKHKBhYEbGfwz79z4Dhx0oDNmY"
    "ED0AcEkLDGtQmIPCnoHl5SeGv89A19m/IqcEAAggCjaGPP8FtO/w7y+/GJ5cf83Az4sYCmaF"
    "3rYJulfZ2EmD4eLFd8AW7TfwwAukLfAP69UsmKUCcmL4h7sUuHmTgaGhATwe8jwvj0HCyYl2"
    "x2sTnfv/4U3IyBdQwDAkbP6CwwoUZqCwA4UhJ7RxzQSd8AOFNSjMQWHP8Be0Yez/K2gVQPLN"
    "YwABRMnuYKCrX24HjQo9uPyagZ0RcvQryF+wUgB0i5aoljwDMysnw5HDz4Ee+A8+NxdSzCEW"
    "hIAOTQANyMB6CdgiHmfD6eVLBobmZmAQvGJ4mpTEIBgQQP5pCVSIfJgf/v1jwEgIsEQNmggD"
    "LSqFzXPAIv8PeHLqP/gOIiYWDnDYgcIQlvv/Q4/XBYU1KMzBqwF+n3kASvcMkPsHSe4zAQQQ"
    "ZdvDvy0/wcD2+87DW+8Zvr3+ysDFAZkPAF21CmuwvP7DzOAWoM2wdu0jhq9ff4K3RUOuWvnP"
    "wArsy4BmC/fs2clw+/Zt6MZI3BGPHMDw7l5bGwPD1asMT8PCGLiioxkEWFgGOOf/x5LjEQkB"
    "tEjmzp3bDLt37wT7HRQGID9B7h/6AwyjXwxr1zxkcPXXBocdrGENClNQ2ILCGBTWoDBnYAF2"
    "/35dBm2MfArtAZAMAAKIsgTw+/pXhv8fN//+/Jvh/oUXDPzc0F1V/yDz16CTLp8CCyZjdy1g"
    "q/Yvw/HjL8GlADglA3PBpUsXGHp7Oxh27NgKTOnfoOP//1ByECJHISUC0GokkCH9/QzA7MLw"
    "0t2dgSklhUGUm3vA6vz/0NU52Iv9/yhVG8ivID+D/A46Kwg0JA0Jk3/gMHr16i+DiYcWOOzA"
    "N6CzQCIfvEQM6EVQWIPCHJj7gRH/5ym0BCBrvzxAAFGaXYCV1r01DCyS2TdPv+BSt1MAn/j5"
    "A1gZCzBC+q2gQ5DfM3Mx+AZrMixccJvB2lqK4d275wz79+8Ar9cHnRLm7u4NXhUDmplDXiGL"
    "3EIGJQ5QADMxQ0/dnD2HgWH9eoZ35uYMvzIyGGRERAYw5zOgJtB//1ESBHIiBu0O0tXVZygv"
    "r2XYuRN0TN48BiUlFQYnJw8GLi4+hgXzbjH4BmmAw4wBOvoHinzQKiDQUbKMwAAFhjVo9O8P"
    "w7fDoMsmQVduvGUg8/pZgACi/JCoD12XGJg+HXz9+DPD06uvGPh5IJc4gxIBaNQKtF7wLrCa"
    "9oszZnj5ClTcg3bSvAWvFkpOzmIID49h4Ofnhy+MwGwDMCDVp8BABB0oCYx4xiWLGT6pqTF8"
    "zMpikFJQGNDuHmbk/8fbFQT5FeTn8PBYhtTUHHDR/+nTW4Y9u58CmzR/GfziTcBhBgo7UBiC"
    "whK03F4AdPEUMIxBYc3w/8orht+37wCdAWoDfCLXDwABRI1zAn8y/HuyANS+uXLkGQP7///g"
    "fuon6AZdUP31B5h6XwNTdGq2McPMGdcYeHgkGSIiEsBHw4EWY4KWXyP3i9GLf3jggo5bP3KM"
    "gXnWLIbvIqIML1NTGSR1dRmYCXURaQBgl0nCbhTFnhD+IWGE38C15+/f4KoAFAaRkYkMfHzS"
    "DNOnX2VIyTEGhxUozNih5TMoLMHDv0AzLx96CloW9J/h++7bQKn70Pqf7O3QAAFEjQTwn+Ft"
    "5XYGls9nn9z+yPDy5lvwQRGgEcrPvyBdGNAK1htPGBicQgwYRMX4GRbMvwkMjN/gyMccHPqH"
    "pdEHbDQCW81Ml64ysE6eyPAb2GN4HB/PIGZlzcAGGjz5h2oGrXM9oiuLerMoZuQjVwPQ20YZ"
    "YF0+SI8IFAagsFi48CYwbPjAYQQKK1CYgcIOFIagsASF6atbbxme3gU29hlvv2P4cQR0u8Bt"
    "aPFPtscBAohah0V/Zfj7cCZoNejZ3Y/ApQCovgJNVYMahBwskN1CN98zMVS3ODJs2vSM4cSJ"
    "5+ABD9BBEMiJANYDgAUYmA1sKTM+fsLADoz8/x8+MDwICWPgd3YBdveYwFu7kEsM2JwDLSZ1"
    "UCOaASPiUUsBBhxV2n/oiN9fcCkACoPjx18wrF/3BBg2TuAw+g8NM1DYgcIQFJagMAWFLbim"
    "/7YZVPffgxb/XynxG0AAUSsB/GN4U7qagen1ked3PjI8uvCKQQCYYn8CHfviK2QQAzR2Dbob"
    "T1hDgiE+2RDYdQft3fsKrv9AVQBs6xRKYAH1/wOty/vwkYFz2mQGxkcPGB54eDGw+/gyCLCx"
    "Qa97ZcBRBP8jbyIJx6AO+sJN5KIdUw5Xu4ABmkgRfn4LjOHmpgvgMAGFDSiMuKCLP0BhBwpD"
    "UFiCwhQUtgz/L75k+LbrItB51xkg185TdEgSQABR88KIL8BuYT8wU/4+u+cxw/8vfxhAm5VA"
    "6xVAO4dB89igVH0SWGgl5lswyCkIM3R2XARvEAGVAqBGIShCETOGwIBiAvYIgAHFtWAuA/Ol"
    "iwyPrWwYfgeFMIhw8wDryD/wiIHkKOSI+g8XxxY5iJk5BMa8LPofWj2PWtxDMAPWEgBb5MOq"
    "N9glUiD/gnoEHe0XGGSBYQEKE1DYgMIIFFagMAOFHScwMfwDdvnOgHI/E9DGz6tARf8NaPH/"
    "hdJIAwggaiYAYHJu2s3A+HTTh2ffGC7ufQw+yQLkddCk1V/YDiJgi/bqGyaG/mkeDBcufWWY"
    "D2wPMDJCbtUCLYGCVweg7iCwf8y1ZjkD65FDDC+AXadPIeEMEsDu3h/Q+gd4BPzDEjmIHIsr"
    "QvFjRIKBVSmQeQz0RPYPZ0JBj3zIOP9fcGIH+RXk5/nzbjKcv/gFHBagMAGFDSiMQGEFCjOQ"
    "TtDp8Jf2PWH4+Pw7sOV48DHDj5OXgcJXgPgFAxmrgNEBQABR+86grwxftrUzsP58dRVYr72+"
    "/QnceAHtZr33HjKkyQP0IOg+gR+8vAyzF3gxzJt3j2Hz5gfQRPALUhKAAhvY7OXavYOBc8d2"
    "hnfAbt7L4DAGCVk5hn+//+DIaf+wdsGQxWERhpmTccn/w9vNQy4F0HsD2CIf5DdQ5DMx/WXY"
    "vOkBw5y5d8FhAAoLUJiAwgYURqCwAoUZKOxAYQgKSwaW998YPsw9B438O5TW/TAAEEDUTgD/"
    "GD4tv8rw+2Iv6Ciz40BPMnz5C64KnnyGYFDLFsS/COzAqJpIM0yY6srQ0nKFYf/+J+CAAW3B"
    "/snIxMB+7CgDz/q1DF+FhBke+ocwiKqpMzCCqwhEIw+9yEeOcNSc+Q9LDseGEdUGzGz0NgBy"
    "qYA6iYXZLYRM8SIi/8eP3+ALsPcBc3QTsA0E8jsoDEBhAQoTUNjAwwk0uwoMO1AYgsKS4dPC"
    "awx/noFy/0Vq1P0wABBAtLg38CfDy9K5wCbfzvdPvzOc2/mYgYsJMolxDZjK34BOtWCDTG4c"
    "u8nA4BmqwVDfZM9QU3OB4ejRZwz/WRgZWK9cYRBau4LhN7AUuOnpyyCkpw9uBcMiCgQgDcD/"
    "aBGDWv9jFtf/seZ+bKUJtvYDakJDJI4/4BPFGNEmf/6jFPugyAeVckeOPGOorr4A9LMD2O+g"
    "MACFBShMQGEDCiNQWIHCDBR2oDAEFv1PGD6vPwO05By09f+TWpEFEEC0SAD/wTNTHxZVMbB8"
    "fHTz9BuG2yffgDeQ/AbKnH0OuUGbFzpfCwqAlDx9hqo6O4acnNMMW7c8YPjPxQLM+SIMlyxt"
    "GTgMTRg4wLuJ/8InU0C7eSdPnsDw6NFj8Cwiat/7P0brHLUd8I9A3Q+r61FLDsg6BuQE8Q98"
    "/dvjx48Ypk6dyPDhw3vomD5CPazIB5VqoNJtC9Bv2dmngX61AfsZ5HcGaD0PChNQ2IDCCBRW"
    "oDADhR0DE7A8eNt3AqjsAhBfJnfWDxcACCBa3R38h+HL9msMX/fUAMvtn2d3PWF4desLAw87"
    "5CzhY4+BjcG/EI+DGj5HgW3aNGCA9E10Y2hpuMAwY+dXhtvR8Qz/zKwZ2EB7EIEBCEoAoEAF"
    "JYBdu7aDj2MRFhZG6jngbshh5uJ/0KPbmMEYOWGgdyOx1e2IRPGHQQiYUD9+/MCwY8c2sNsQ"
    "ffw/0IbtL3BwLFhwk6Gp6QrQjy5AvxqA/QzyOzgM/kLCBBQ2oDAChRUozBgYgSre9p5j+PMa"
    "FPmgEuAZA2QbCNUAQADR8vbwnwxvujcz/Dw+8ffP/wxHNjxi+Pr0J9jDb4FdnIOPIOPboH2P"
    "oHNvTtxiYIhN1WJYusKfYc6M2wwdXdcZGP8zASP4J/gYF0jLmYnh9u2bDGfPnmQIDAxl4OXl"
    "g48fwOplXDkYM+czMNy/fw+MkRty+HoMyCUM7F5g0LlBILeA3HT79i1wggK59fv3X+Bu3tev"
    "PxgaG88xzJ59D+g3P6AftcF+BfkZ5HdQGIDCAhQm4JIAGEagsAKFGcPHGdcYvh0HRfxJIL5L"
    "zaIfBgACiJYJ4D94kuJ50QSG32eXf3v/i+HQ2ocM3178Ak9xPgP2YPfcg0wc8QG7i6BTsI4D"
    "A8bWTY5hz4Fwhhs3fjCkJB9iuHbtLTjRgwZOPn36wrBx4zoGNTXQOYHawED+gbUPD+lv4xoP"
    "gEQeqOrYtWsHGMMWo2DvJcDMQjUTpubnzx9gt4C2wG/cuBZ88ATIrYyMf4Buf8OQknKE4cbN"
    "n2A/gfwG8iPIryA/g/wOCgNQWIDCBBQ24DAChhXDlxW3Gd4vBl0WDSr+r0InfKg+1g0QQLRM"
    "AAzQluprhiepVQz/r+/5+OIHw7GNTxh+v/vLwA1KBMCOzNbbkMkOfk7IuPkZYDqXUBRi2H8w"
    "nMHUQokhNeU4w/z5N8BdxPfv34CLfAcHV3Augwwe/cMxA4dcd6MPGP1Hm3P4hzRE+x8pwv+h"
    "jU5img1yA6iRB3ITyG0fPrwBuw3k5mSg200tFBn2HwoH+wnkN5AfQX4FjfFvuwMJA1BYgMIE"
    "FDagMGL4ufsxw+uuw8CwOw4t+l9Tq9WPDgACiDbXx2MCVgY2OVUGqQlzGJhULUUUeBis/WUY"
    "2EVZGL4B6z8hYG7wVmFgUBCATH2CnCQjzMCgJMTAsH7jfYbS4v0MvDz/GQoLtRlMTETAq2gg"
    "x88jLpnGvMELsbUc/cApkDzoZrJ582aD+UlJqeCIxLctDfnKeOQFnLCRPZB9f/78Yjhz5jVD"
    "f99Vhs9fGBm6eh0ZAv0VGe69A3bt3kI3zrIA+0cfIQkfVOxzAVv8P1//YTgKjPw3D4BFwa/D"
    "zxmeFu5h+P/zENC6g9DxfpLW+pESpwABRK8EAJ4ZZmARV2eQnTqDgUnTkl+Ck8HCT5pBQJaN"
    "4fs/yI5XFyUGBiNJ6HFzfyGHIqsD+Z/e/mRoaT3FsHzJZQZzcyGG+HgVBk1NQXCjC3YLCagY"
    "R9wYDjuRjAHlxk5YxEJu72QF5tI5YJHExBTwkDQkLBhx7llArQ7+Qg+ahLBBy7gXLrzDcOLE"
    "W4bIGB2GmmpzBj5hdoabzyHXv7JCN3acew4p9kGnfYJO+Pjw+BfDiU1PgTkfqOjngWcMT4v2"
    "ASP/GDTyyar3SYlTgACiZwIAAQ5IIpgylYFZ15pbiI3B1EuSQVyVg+EXI6R4NJWCJARQvxjU"
    "UAItAJIUBFYLPMB+0KV3DB3tJxl27rjLYG0lzBAVrcSgoyMEvoARdLQaJAEwQXf9MqLc04d+"
    "YQOo9Fi0aC7YUXFxyfBxBdRJIAaUngNkOPgvtAfxH1zUX7nyjmHp0nsMR4++YXD3UGaoqDRn"
    "MNATYnjxBbJfBzSsC9rPD9rGD4r4088gq3vZgOIvb/9gOL3tOcPXd6A5391PGZ6WgCIfVOwf"
    "gkY+Wcu8SIlTgACidwKAJALQhYfyczsZOGx9WDmYGfTsRRmUjHkZ/rFCZr/k+BgYPJUZGNRE"
    "IGfhgVrKoJExKWBC4AFm+uOnXjFMm3qeYffOewxKSpwMnp4yDNbW4gwSElzg0gCUGBAXTDNh"
    "vcAJVAUsWTIf7KCYmER4FYBrUgjUTgBFOijXv3z5leHIkZcM27c/Ybh3/weDq5sSQ1a2AYOl"
    "mRjDF2AV9uw95IYP0LE5oFNmbgG789uB0fkI2IxjBw2KgYbGz35muHTwNcNv0GT/p2V3GZ43"
    "HIS29o9AB3vIvhOHlDgFCKCBSACQ6oCBQZ5Bur2EgTcggYGRlVXZUIBB2wqYm4UZGUA9INBm"
    "CCsZBgYnRQYGAXbIoghQCQFaKCHODwnIuw8+MyxedJ1h44bbDG/ffGZQVwfdHSDGYGwswiAu"
    "zgU+ZBFy+hbssAnkixlYgK32VWDH+PuHgfv0qDOFDOAIByUA0DEtL19+A9bvbxiOHXsF7KF8"
    "ZhAW4WXwD1BliI3TZFBW4AUn3Jcfoce4gk7xAEb8ByB7331gH/8JpMUPWs798+1/hqvH3jHc"
    "PQ+a7fn5l+FN/1WGN3Ngrf3j0DV+FHX3SIlTgAAaqAQAAqDRbikG0bQoBuHUEob/QoIC0pwM"
    "ujbCDBIqoDNxIIEmwc3AALo020QSMk368w8kekDFqgBQjosFlNv+MRw/9pxh86a7wAh6yvDs"
    "6ScGMTE2BikpDmAXjR9YSvAxyMlxM3AB+1rs7MzgyytANKifDvI+qD3w8+df6OFMoFO+/zA8"
    "evSF4d69Tww3b3xkePrsB8OrV78YpKT5GKyspBl8/ZQZLK0kgaUSE/iU9A9fIdUVI3QJ3Heg"
    "2BlgXb/3PmROH5SYmYAlwos7vxguH3nL8AE0vMvw4hvDs+qzDJ8PnIZG/mnoQM8vaqxaIhYA"
    "BNBAJgAQALXiRBm4zW0YJKtrGVh1dZmAWVvZgJ9BzZiPgQtYGvxmguR8JWAPwVGBgcFAAtJt"
    "Aq02Bs0JgfbKgRZQgAZRQH3aX3/+MzwElgzHjj1jOHv2JcOdO+8ZHj/+BCwhQEe6MzPw8rKA"
    "D4HkBLLZoHutQK34b9/+gk/8/vz5D5gtLMLFICvLx6CiIggsUcSBES8FvvyCDagX1B8DHY37"
    "DXo8LuhcJPCOaGAiuPCCgWE/sN1+7wN0lxRQ/hsw1986+4nh7oWPDP9ARcWPQy8YnlSdYPj9"
    "9AI011+GdvWoMspHSpwCBNBAJwDYWAToqHNVBummbAb+wBDQRCgvMAerGwswSKtzMLDxIvpB"
    "SsB2gDWwajAAlgjCnJBW+m/oecWgxADaRQsqHVigTgRFEOg6mQ8ffwBLhq/gM3dAmy9ARTry"
    "cbWgKgN0txGoHSElzc0gwM/BwAnsmjBDR0pAS7NBuRw0iAMyE2Q8+Lg2IOMtMENfAOb4o08g"
    "U7mQfi8wMX5mYHh68wfDzbMfGD6/Ap2q/f4nw5upNxhezz4FndU7DV3Y8ZGa/XxS4hQggAZD"
    "AoB10kEnXUsz8No7MkgU5DJwmOiAQldUjpNBWZeHQVyRjYGVF3RROSTSRbmAiUAc2GsAJgRF"
    "YKLgZYMeUPkPsUKYCdoIY4HuqiX2NKC/DIiDr//8RazkZYSehwjK8aCBnPvAyD4NjPgLLyGr"
    "d0DyLKAuLDDiX97/xXD38heG14++Qxz0dd9zhmfNZxh+3LkMndW7xABZ0fud2iN8pMQpQAAN"
    "lgSAXCUIAbEKg1hGGINwVBgDs7IkE7ASFZXlYJDT4GIQV2ADH04JOpQSNHMGKnpB1+jqiAGx"
    "KDAxCEDG2MELUaERCcLoh9MwYhm3Rr+xAJxooIdggQaoQCOW94FF+xVgYX3lFbCb9wVSFbEy"
    "Qs7r+QHMxy8f/GJ4dOMbw+vHPxj+gRoxf659YHgx6SrDh80XoUO6F6ALOt5Re2KHnAQAEECD"
    "LQEglwYSDIxsGsDSIJJBOMyLgUlaiBHYtxcQZ2OQUeEEJgR2Bh4hYH+eC3J23m/oSlrQCfXy"
    "wEQgD0wkMsDupCSw1BDnhrTKWZgQkYo150MTC2ggCtTrAPb2GJ6DFmh8gozePQRG/qffkMQA"
    "LglAaoE5/8u7/8CI/8nw5M53hg8vfzH8B3X+/97/wvBy5g2Gt0suAgVuQev5m9ClXFTP9eQm"
    "AIAAGowJAAZAJTYPuKfAIqDNIJrkzyAY6MzAqioBOquYnYeZQVSanUFcno1BUIKFgYsf2Lfn"
    "hOj6Cy3GQZEEapWDIh/UlQRW6+AGJA+O44JAu61BDTnQlSygLhwoEfyE7sphhjoIZPBvYPR9"
    "+/Cf4f3LPwwvH/5ieP30J8PPL6CGCLAS+nXlA8PrhbcY3q25yvDvO2gw5xo04kEt/C8MVFjH"
    "R80EABBAgzkBIFcLoMuQJBkYmVUZhMOdGQT9HBk4jZQYGAU5Qa09DmBiEBRlZRAUY2UQEGNm"
    "4BFgZmAH5npWYIQzskCbmYyQVhb4+CUsB1D+h55yygibIYOc08TwH5QAQPMzwNLgywdgY/LV"
    "X4b3r34zvH/9m+EHKNLBuzbf/WT4euQVw5uVtxk+7rkB1Agav78FLeqfQ2fy/tArwEiJU4AA"
    "GgoJADkhgEoEMfAgEoeyDoNQoC0Dr40eA4eWFDgxgIZ+WUFdPCZwIuDmY2bgAmJObiYGTh4m"
    "BjZORgbQ9AF4IIiVAfl4Yoa/v6FL0YHRBLqo8fuXfwzfv/5j+PbpL8NXIAZF/vdv/xj+gxoe"
    "oBT0/+1Phm8X3jJ83P2Y4f2WOwy/X4AGcB5BR/FA7FfQHP+H3gFFSpwCBNBQSgDI3UbQGRDA"
    "mp4B2A9gkGFgk1RhEPAwYuAyVmfg1JBkYFMQZmDkY2P4D8zToP4gM7RHAGwEsLAxgg+zZmFF"
    "9cOf36DI/8/w59d/6BQztLAG5XBG0Jjwx98Mv25/Yvh+9S3Dl9PPGD7ue8jw580zaA4HnekO"
    "7ASCF2uCzsX7RqvpW2onAIAAGooJALmxCCoVuKHjCELQBCEGTBAyDFwGKgxcWrIMrDIiDGxS"
    "fAwsQtwMzMCWACMHKwMzPztGEwxk2l9gE+/fFyD++JPhD7B3/+vpF4afjz4yfL/yhuHr5ZfA"
    "CAcdxPQGOmjzChrh76D9+K/Q3P5/oAOGlDgFCKChnACwVRGc0NIBligEoJgXWn1wQiej2KFD"
    "0dh6g7+g+DsUg4rxz9Cc/QEpsr9B5f8MtoAgJU4BAmg4JQBsvQg2aGTDIpwVKg5rGqJ3CP9B"
    "8R9oBfAbmhh+QvEverTi6ZkAAAKI8f///wyjYOQCgABiGg2CkQ0AAmg0AYxwABBAowlghAOA"
    "AAMAPQCOI49Aor0AAAAASUVORK5CYII=")
index.append('clock_stopwatch_icon128x128')
catalog['clock_stopwatch_icon128x128'] = clock_stopwatch_icon128x128

#----------------------------------------------------------------------
copy32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAATDSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEECMDEapmKL//7MxMTKWiMkpZ7Cxs7EBQ+kfWaYDA5cR00YGJkZmlo9vXz94//pZJUAA"
    "sTBgi4Jfv8K9gvxauwqCGbiYGRj+/SfZXgj+j6D//UeIs7EwMJy//1G0qGFyP0AAsWCYDlLN"
    "zKJsZqjLoClEnWAG2QCy5i/UIaxAT7Fr8DMoKCnKAgQQCwMTMBmgh/B/hr+/f/8hyzKQJTAf"
    "/0Py+T+gFf+g9B+gA779BKr9++8vQAAxMTAyIpxJoS/RLf+P5Pt/SFGC7F2AAGJBCXpGRvIs"
    "/49qCVafQx0Hl/8HSaAAAcSCYhIiPfxlZWEmPisxQgwDGfof5hB0n/9HOILxPzxDMAAEENAB"
    "/5kZfv3WYvjzTxAo8geo6htQt+LlG7cZ9p4GxtfvX1hjhxHJEmA8MpjqKDAI8PMw/PgN9Qua"
    "z5HTBBMQ/wFK/vrznwEggFgYGRjrjcyNa2WlxRj+/PkLt+Drxw8MPatPYU0cyDmXEej9d5+/"
    "M/AyH2BY0BDLIC7Mz/D9F1IIYEkXoHT/4gsDw703vxgAAohFREgoY0ZtDIOJohBK4vhPKFsx"
    "IIIa5Gy//HkMOw5fZkgJtGH4ipT/UXzPAMlwIAxLsAABxMLGxsrMzcZKVLkMy0YwA0EW/wMS"
    "LOwMDLx8nAy/oGnoP1rQoxdI8LQAjEeAAGL5BypnCVRIsIQFire/0IT29z9StvsDcsh/YEpg"
    "RC350KMALVpAigECiIWY/P0H6nOw5dAg/4vuGyaIBX+RLGH4jxZd/5HUQM0GCCAWQj7/i5Rv"
    "4TQDgv0XSn/6/hesAWueh2VRGA0NRVBOAgggFnzxDfPN33+oZfk/BkQUgMBvoMLXn/+A4xQl"
    "/zMgORatJISVkgABhNUB/7D5/D9qeQ7jg9WDq1hGjNLwP7LP0bMjVC9AALFgtZwB4Wu4Y/4h"
    "hQhavoalDWQfYy0JGVCzJAgABBALuuXoGNkhGCEBMxhaLP7/hxrnuEpCWKiAAEAAoTjgD1Lw"
    "IvsSm8//ItV+f9FrPixZ7h9yWYBUmgIEEGoI/EXz5X88fGTD/6E2PGC+Ry8J/6OFIAgABBAL"
    "EyNmY+Lvf0ReRfb5X6QcAOODi1UmRBSipAMsPkfmgwBAALF8+/GH4ePPv/AaDL1RgZ4Y/6OV"
    "A2C1f6Fp4D+ipMTqcwbU0AQBgABi+fr9D8P773/hIYCS8tHiHbkMQHbsX7SGKHo6QC8P/iFV"
    "dgABxAJrBP1jIBDn6CXgP9S8jlFy/kfLfkiNFZjDQQAggFhQ8j6az2FBipwe4MUvcpphRAoB"
    "BrQmGQNmGxEWSiAAEIDJMlgBAAZBaCnr/394MN0KdusgCSLyDCQsIlbz+v7Wzgb8y2aSUeHQ"
    "GnQKpCEjg3pfugtvGybF2Yr5ezUyTCaPAGL59fvHh0uPPwhISQgxfPr6G7PQwBYFSKEEKoI/"
    "/wIm5M9fGN7852a4/JaB4csXaLrA1jH5D7H81N0vDF/ePH8FEECMDIYp8RzcQg08fIJ8wJbB"
    "H7AqRkRVysCIp55mhDTJfv74yfDt82sGPmlNBmZWDgZ4T44RR/OKiZnly+vHr3+9f1QLEECM"
    "A907BgigAe8dAwQYANZrLXiMIQtIAAAAAElFTkSuQmCC")
index.append('copy32x32')
catalog['copy32x32'] = copy32x32

#----------------------------------------------------------------------
magnifier_glass_dropdown_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABmJLR0QA/wD/AP+gvaeTAAAA"
    "CXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH1wEEFSMYxsW2fAAAAnRJREFUeNqlk0tIlHEU"
    "xX//7/tmxtfovNRQR81M08TQUElXLSokbRFJRWCBQStDElpEmxaFhEVtplwkbYMCLR8huMse"
    "RJD0QIO0Ml/pzDSjozPzzcy/xegg7bIDB+7q3HMP9wgpJduFEALR2XkJTTNgt1kbCwoKL1vt"
    "jloh0Pw+34TXs+wKrAV7JicncblcNDY2Mjo6SjgcRkqJEAJNSkl2lqO9ovrArR2FJYY8hxlF"
    "Ecy6A5U/p7/eX5j+VOP3+y8A0fr6eoQQjIyMJFxo5rTUQ7sr6+4mFdaJFJtGTAMJpGSYMBfb"
    "0BVTW2R4aALo7u7uxmg0oqpqQkBx5hdcDVnLRFBoeFdhcYPuAKxHIGQtp67hYIfRYEjz+Xxy"
    "aWlJhkIhKYSQgNRSLVn7vbF05DrEIhDQ48qhKKyGwRvRsOQU52Y6bCXL84siFDeYyFEDqRhU"
    "8KzDWhhMG+7CUQhGwGAAVZEApgfAeRBBkFJKIYRAWXEvjGcbV7GkgB4DfyjOcAzMyZCTEsU9"
    "88U7u+yZygFuAkkgEhkMDQ526bPj5KdDkQOcFsizwE57nHLhAwP9fb0Nur6oA5nAlS03KH1P"
    "n/Xfu9N13TcxhlUEyM2AvAywa0Hcn9+wuvKL5oaqih67clgCEcCyNYTNYW95WevRpuaLxaVl"
    "exRV1Wa+Tc09Hx562OIwF52+duSs2TMQfdf2/syPGf8jD9Cx8Uh/QzUZDbuSk0zlm4taFIom"
    "WrV5+bpWrrw8pj9xmk/eBv6pAkVQ9fGEaU6+qpG/XzTpvaXWc//cIadg39vjxu9yrEY+bi+f"
    "2lYJ8xWqb1QZh0vT1FObAuJ/6/wHDeP8af1Kjs8AAAAASUVORK5CYII=")
index.append('magnifier_glass_dropdown_icon16x16')
catalog['magnifier_glass_dropdown_icon16x16'] = magnifier_glass_dropdown_icon16x16

#----------------------------------------------------------------------
progress32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAeRSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAD7gCAAGKBMRhNZkEZQPwTGC2//kPYKOB/KsPXvzlA8dUMzEyTGP7+/8TwDyj8F6j2"
    "HxD/ZYDQoFj9BpT4+Q+bnaFArP//f1ENiAMQQJghANLMArSZCYv43//phgZieuLCnM0Mn37u"
    "AFpmj9VbILXsjNhkmIE4EohzYAIAAYQ9CkDKOJkgIQBPo/+ZGb79ZkoK12S4uDuKwddX1ZLp"
    "++/dDD//9DEwMgpgNYedCZfzfsE4AAHEhNMHIMs5mJBDAiT67+fPvwzAEGBYO9uLYUq/C6sg"
    "D1shw4cfB4GhYY0RZcyEUxlAAOGXBhnIyoiSFmC5lonxP0NmrC7D/s2hDB4+KnoMn35tZ/j1"
    "rwyolhXuXCagRmZGvFYABBB+B8BCghnsCFCKgqeqLz/+MXz/+ZtBX1uEYdMyX4amVjteYIB1"
    "Mnz9vRKoVgIlPeFxA0AAsSD5VhHogwCG33/ZgHH6DxLg/6EpHMj7+48RmAYkf//+Czb5/of/"
    "DKtusjIk6v1hUBVhYqgtNmNwsJFhSE3fFXjz4itNBj72FKDCo2CzWYH+/IU1RzAABBDCAf/+"
    "zxIX4XSRFGCH5KT/8JTPAHQOw3+g4I8vfxgkhTghAQFUcOXlf4a6QywMCXp/GdxV/jLYmksx"
    "bN8UxJCYtkvj4M57mxi42KKAineCQ4AJOfwQACCAEA74+kcpOVmDoTJBh+Hr1z8MzKAE9J+R"
    "gYkJGoSMjED3MDKwszCBUiLDf2BAsIO0/WJjmHaOmeHe+38MMbp/GRTl+Bg2rvZjKCrZLzRv"
    "1oVVDOxsycC0sAYcjf8wi32AAGJBivA/nKzMDMz//jL8/fmLgRkYbIxAyyGQiYGJGYiZ/jP8"
    "+/OP4R/IMUDf/AYGKxPQBFZmNoYtd5kYHn1kYMgz/8sgycvKMHuGO8Ov3//4lsy/NBfoiA9A"
    "C/ZgiwKAAGJBTvJ//v0D5pz/DLeff2fo3/GGgREYDOxsQEexsTIws7AwsLCygNnsQIf++M/C"
    "8OsXMwMLKHUCHcoDlLvylomh4xgzQ4nFXwZZfmaGaZNcGN68/s63Y8utiQws7A5Apa8Z0AIB"
    "IIBYMLIFMKQ+ffvNcOIG0NEsrAycHKwMLGxsDKzsIAewgh3AAnIEOysDFycLPG+CalUuoLp7"
    "HxgZmg4zM1Ra/2VQEmRlmDnNlcHhxjut+3c+AItwpnr0lAAQQKjZ8D8osYNS4D+gz/4ysDH8"
    "AWbqv0D8h4HlPwSzAiOf9f9vBqa/vxn+/PoDwb9BbBD+xcDF/I/h1TcGhimnmRi+AAstOVle"
    "htAQNaDhfzyAoSwMYiBbCRBALJh5HxjPwJQPMpAR6BhmJliRAPElBLCCc8l/eGn1D8z5DxNh"
    "ZmX48JOR4QewQuMBplRublDZ9J8dah9KJAAEEAuq3UDLgZb++fMXmBN+MTCy/APzWYFuZv7z"
    "Hxj0QDbQccy/gTQHsLpggJUxbOAcAwKf/zAycAMlUnQZGUR4mRm+fP3NsGXrPVBpdhyo6CXY"
    "9UgAIIAwQgBkIQswtUsBExEDCzARAisUFjZGUHIAY2Ygmw3okH+Mfxm+/P6PVGD+Z/gOdKQA"
    "DyNDoSkjg5UMM9hh1TUHGU6fePICaMgkcKGGVjsABBCKA0DZ7tvPPwxKEhwMi0u1gNmOCYoZ"
    "wTQzEP8HZkEeYJl76clvhsYt7+BV/hdgG0JaiIGh3IadwUYRaAcLB0N710mGSRPOglzeALT2"
    "OjA5MaN7GCCAkB3ADMoBvLwcwBhlAmY/Jkj5AyoLGCGOANrN8BdUMv79y8DDBk14QLGvQFdY"
    "qLAwFDlzMOjJsTD8Y2Fj6O0/w1BVeQSYmpkbgUE6k+H3PwYGLM1PgABCOICD5cPSHfcYzt14"
    "x/AHZAmsKIa2cEBF8a/vfxhSge0Bf3sJYAIDRgEwnfwFltNR1kIMOU68DFIiHMCGEDNDWfFB"
    "hmkTgT5nZSkB5ut+SEsJe2UEEEAIB7Ay5ty68zHo1rW3TNCajwHe3PoLdMIfYGX0+XectYmk"
    "eIiTJLAm/MMgys3AkO8lyuCmw8MgDOTcvPeFITNvP8P+7fffM3Cz5QKDbClY/38GrL4HAYAA"
    "YkGqek8Aay0gRkqkCAdAy3FGezY2JvGvwKpYVpCFYUKsNIOmLDcDnwA3w7rtjxjyiw8xPLn9"
    "/hIDP3sy0LwzcF//+YezOgYIIBaimq4QHzBB2jjAqPj9h0GYh5lBSoKH4f3X/wxNzacYJk8+"
    "z/D757+lDILspUAHP4e3XP78x+l7EAAIIMIO+IMafKC0wAqsEcXFeRiOX3rPUFR9jOHiqedv"
    "geVwBQMXyxxoVkM0n/7i7/gABBALzqbYP2jz/B8DokUDTJvcXKwMbJxsDE2TLjJMmHGJ4du7"
    "H/sYhDhKgD49j5LQQHrwJD4YAAgg7A74C7X8P5LlIAYHC/PeY88Ylq69w3D66NPXwMZBIwMf"
    "21SsvgQJYfYLYE0TNpgAQACxYPX9HzSfQ8T/ARPp5s07HsgBLdwOTGiNwIR5C+xYbOAn1rgH"
    "RegOIH4LEwAIIMaB7pwCBNCA9w0BAmjAHQAQYABMAqbWGPAKOwAAAABJRU5ErkJggg==")
index.append('progress32x32')
catalog['progress32x32'] = progress32x32

#----------------------------------------------------------------------
delete16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAALuSURBVHjaYvz//z+DWmjHngBv"
    "G3vG/////fvHgBUAlTH8Z4SwmZmZmbbtPXnmyuJCS4AAYgAZUDBj/7//JILa5adBFANAALGA"
    "TWf4/xtIsWHYCrX5L5BmBNr+E6jqD5DDy8HAAHTqH5AagABiAhH//v2Ha/rz5w/Ds5MnGX59"
    "+8YAcvFfIPHn10+GZydOMfz885PhDyPEQJDtIAAQQEzINv759Yvh4ezZDB9aWxkezJ7F8BsY"
    "ICDFj5csZfg5dzbDu0XzGf79+cEAsvrdN4gegABCMeDzmzcMv2fNYtBKTWXgvn6D4ebEiQx3"
    "gHzOkycYVKMiGdj2HWL4/OAh2Gtvv0P0AAQQigF8EhIMnIWFDE+amhik+fkZeDdsYOBatJBB"
    "VlSY4VX/RIavXh4M7IqKDL+AMcUEjRGAAGJBNoCJiYlBJi6O4S4jE8PDmhoGeXFRUPAxPNu4"
    "heF1fgGDcFwsw78vjAx//kECFwQAAgjFgL9AwX9Ak5mFhBg+vXvH8OgR0LmsrAzfxcQYmHh4"
    "gQHKyPDvLyjQITEEAgABBDaAEeqcf0Dh5+vWMzAmpzCIf/nM8Ajokr+MzAzyLEwMv3s6GT4B"
    "1bH5BDH8+c8CNwAggMBh8Bua+j68es3wOz2DQe7De4aPoBTX1c3A3tfP8PHbDwYxLk4GwWlT"
    "Gb7de8Dwlx3oAqgegAACG/DyC4TDJSjA8D87i+Eh0Ek/a+sYREuKGASzMxh+llUyvHj+nOG9"
    "kysDs5Q0AzAm4S4ACCCwF/5A/fQPmBj5KmsY3jm5MfAb6DP8+wrxL0dKFsMbPWMGFn0zBiYG"
    "DoY/vxEGAAQQEyQMIIHwF5Rq/rAwCNtaAcW4Gf7+AhrwCyTGzsBlYweMOg6GfzDNUD0AAcQC"
    "jT9WEBfoTQYmoI1/QUkNmN6Z/0FiBpT+QWLMHJDoY+MCZ0mwXoAAAhMnjp86Hj+b34wRmJv/"
    "gHSAovM/LGagWfk/wmZGZiamk4ePXWPIMWYACDAAuRJIdjvS144AAAAASUVORK5CYII=")
index.append('delete16x16')
catalog['delete16x16'] = delete16x16

#----------------------------------------------------------------------
arrow_up_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMcSURBVHjaYvz//z8DJQAggFhA"
    "BCNjF5rwf0YGhn8lJsZSdWxsTCzHjj/pAapqYmBg+o2i6n8ZA0AAMUGMAVKsTCCTgJx/QPy7"
    "Oy/XpOvgvjCe/fvDOaqqrGqA4tMhcmCtUMzAABBAYBcwsDMDNYP0/mVg+Pu3OC3VpHjiBGeG"
    "VXeZGD79YGBobbVn+PuHIbmz6+hLoCuqGThYgZYygrUCBBAL3EAQ+PvH3sFZuWn6FBeGVbeZ"
    "GOad/c/w/esPhu/v/zC0tVsx3Ln7vmzt2usXGFjZVjOwQgwACCCIF5iBnF9/BARFeKbMmuzC"
    "deEDC8OSC/8ZOP79YuD8/4Nh4dFPDOvOfGSYPtmORVZBqJ3h+29JWDgABBDUAAZGpv//qrq6"
    "7HR4lUQYWg8xMPz48Zvhz48fDH++f2f4+/MHQ+XypwwPvv5nmDzBTpmdlbmF4ddfsOsBAghi"
    "wPc/pl7+GrnJCXoMe+8wMAiy/GYQYf7O8PXrd4Zvn3+C+dK8DAyLdz1j8HSTZgiLVI8HGuAK"
    "0goQAEEAvv8A////ABcmmNPHzOf/9/n8//T3+///+/f/lLzs/zCB3v8nc9P/cZzY//Hs7P/h"
    "5Ov/7+/t/9PX5f8gLZzh////AAII7Iw/nCwc3TPOff3z++8Php9/+TpqzVm5OdghAQjEbMws"
    "DK/e/GQoazvz5/vvvx85OJk5frJBHA8QQGAD3n744bZuxVUhhl/AUGNirMvP0AthAGr6+eUb"
    "w8+v3xj+/2FneP3xJ8P6LXcPMHz7U8TAzMTGIMTxEaQXIIAgAcHEeP8fN9t9Bi5gyHMyf2QF"
    "RtFXYAL4+PErw/dP3xi+fOYEquFh4BHj/PTl06/LDH+Aepgh0QgQQCyIRA0RAEYpCxOQyQFM"
    "dQKsfxi42P4wcDL/Aybu/6DYYmFgYwTR8MQDEEAsECYwibJCswALI9svYLKMdRBhsFZhZ/jz"
    "+w+DqCAbw71nQK+wAFUBXcfIhkh8AAEEMYAX4ZBfTAx3m+ddZFCW5P777x8k7QO1MDx58535"
    "BwvTQwZuFqScwMAAEECMlGZngAADAMelMK5IzYNnAAAAAElFTkSuQmCC")
index.append('arrow_up_icon16x16')
catalog['arrow_up_icon16x16'] = arrow_up_icon16x16

#----------------------------------------------------------------------
paste22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAASMSURBVHjaYvz//z8DGBgzMvz9"
    "wcDw5y8Dw2+gEM9vBoZ6ZT2WrfoOZ42VRXQ//H36/92fFwwf/7xjPHfj+r2Ic990mp9++/Ea"
    "qPYPCP8DmsHOwGD3DGIeQACxMMDAfwYNht8MNgz/GP4ygOR+M3xh+/nD2cZJXS/Py43h67+v"
    "jH/+/WNgYWFhqF7ap/zv+LI5/38zbPj3n4EXiBn+/WVgYWJjOA3UeQFkHEAAwQ3+z8W+l8lB"
    "WZKZBagZqOr71z//v/LIM7HrfmBY/mEZw/8/DAyMQMjBzsEgZsLP8FRXLprN/F+kIjcz48//"
    "zAz//7EyvDhw5y3QKFGQeQABBDf4nxCfGLO7IiPLy+MMDN/fMez/IMF4SJWb4eOrbQz/Hn9n"
    "YGRkBJr8n+E/EDNysDA8NmNj2PXsOVOS9nsGBlZ+BgZZR4aPN98KwswDCCC4wX///P/H/PML"
    "w8tPbAzrXmYwbPz/nOH89SMM3899ZmD4CVT2mxloOxD/BWKgJf8//2WY9EGf4eNPFYYwpS0M"
    "0qKfwWbAzAMIIISLGZgYGNh+MOz8asTwx2UyQ6f6C4ZHT88xMDMAlfxhBWKQwUxgg///A0b0"
    "/78MEnIqDEduiDGsPfiCIU//LTACGeFRBhBACBeDxJiBXgVGjiLfHwZ9CRkwJgRevP3BcJcF"
    "mByYGBn+/UeIAwQQqsFsrAyMLMwM/4CRhw7AMc8Ajn0GkOxfIIebDRSEvxmYhbSA6ZOJ4d+f"
    "W3D1AAGEZDBQORsfMC1KAePoP4qh/2EG/4cY+h/K/glkf/z+n+EPEy8DAwsn0DJ4EDMABBCS"
    "wUCVrFwMjEwCKAaDWH//Q1z4H2roL6Dp34DJjx0Y5F9/gXzxBxxLf5kQYQwQQIjIA4mBktR/"
    "TO///Yuw4BfQjO9A/BsowfyPAewIsENAYYzkS4AAQhgMsg2UkoCByAg1HeZ9mKE/gQb++ANx"
    "PQiDsjLITEaQIiYGFIMBAggpKBjBtgJzAEgNxOB/EMUg14EM/P0XYgGoXPgDdTE4JYAwExNK"
    "UAAEEFI6BsqyAZMNOxcDLIhBBoIwKExBBoEMhRv+H2Lwf6Sg+IswlwEggFCTGyipAd37G6gT"
    "FB2gsASnBGjkgUo9kPgfKB9kIUieEezi/wxI+YMBIIAQBoP8z8wI9jvI9WBDkNIszKVg/BfC"
    "B2VKsGNBORkYlMhBARBAaGEMCVc2YO7jBHL/cEO9D0qJsAj7h0gR7Jyg0GNm+P4bEj+gYgQG"
    "AAIIKbmBSi8GBmGOjwxHjj9k+PFNiuHH7z9Q1/4DuxLsA6BNoKAC8UG59PKZ5wxuIp/AMfgf"
    "ycUAAYSa3BjFGTxNWBm4r21geHyLk4EVmBtZ/oOLELCfGaE55j84hzEy/AF6JVrzF4OjKdDp"
    "jJIMf1mY4AYDBBCSwcBQlvkDDDYBBietb0CFPxjgueU/IyQwYUmL4T+CBsXmXy5gTH4F+vo3"
    "3GSAAIIb/P7qp6dXimbLAT30/99vkKtA4fsfHBSQquc/JF1D2aCw/w+jGcERyvj40q/XMPMA"
    "AojxP1qBQy0AEGAAzscjB1XlEX8AAAAASUVORK5CYII=")
index.append('paste22x22')
catalog['paste22x22'] = paste22x22

#----------------------------------------------------------------------
led_purple_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAPzSURBVHjaAEEAvv8BSUl8AAIF"
    "AwAHCQgACggKB/Px8z7w8fNlBgQAPgoKBw4CAgEB7u/y8+vr78j7/fubAgIAvAQDBfcJBwcA"
    "////AAKIhQEK/v//J/P7378uTUXFSEtzNQZdUzkGIWEeBkZGRoZ3775wnD1+23fjhsO+xy9f"
    "Xsf4718pUPweyACAAAIb8O//PzkmFtaVvrrmFv4Oxgw8amwMDHxACUaI4Xx8nAwKYqIM7hrG"
    "DItX7wuasXutxrdf3/3YmNnuAgQQs4qqLcc/JoYJnhIWXjEaVgysvIwMf//9A7oI5Cwg+vWf"
    "4e/Hfwy/X/1lYPnMxGDOrc7w8xOj6MmnV5RYmJi2AAQQy1+Wfzbiv0Vj1M+IMvxS/MXwn5uR"
    "gYWRmYEZpPkrI9gV/34ADXn/j+HPy78Mv5//ZjC/o8Kwj0XR5x7zIw+AAGJhZmPO0H4pz/Dt"
    "3E+G12qfGIR4eRh+//vP8O8rMwMjB9QbP/8z/AG6ggFoyJsr3xg+nf7OYK6ow/BY8EUWQACx"
    "CLDy2Sh9kGJg/cPJ8GTjVwbGb6wM3EZMDMyiTAyM7EyQQPj1j+Hvm/8MXy/8Y3i86ysDww9W"
    "Bo3P8gzCogKGAAHEwsfEwyfwkZeBk4GbgeUTO8OnDawM/06zMbCp/GNgEvkH8cJbFoZfd5gZ"
    "vjz5DbSIC6gS6MUvLAz8f7nYAQKQQMY4AIMwEDNRqiTw/18yVyxtCGL1ycNZ/TG89KDjMrAM"
    "ZAq54It7IV9KgPFb9VHWAWdq0rYAYvnH+/8LowArJ89LPgYWZnYGVmFgIIoDNQgAFXBCfPAP"
    "aNj/DwwMTC+BXnrDycD6m4XhJ/dPhs/MX38BBBDT+z+fT7yQesfAC3QHGx9QsxQDA6s0EMsz"
    "MLApQbECAwOLLBCDxIUYgWrZGB6LvGB4+efNZYAAYvr4/uOsI4LnGP6L/WZg5wHaLMwAdgFI"
    "A5si1AAZiBgTUI6DF0iL/WXYy3eM4cfPXzMAAoiJ7TfL/hNfzqzdrHeAgUcUqADobCag81nE"
    "gLYpQGxnlgCK8QPluBkY+AUZGHZoHWE48ufMPrY/rJsBAohZTtbi95cPX86c/nvJlVOSXdSA"
    "X56BT5iZgRVoAMg7zEBNrMCY4wAG4v+ffxnWsB9imPhl+f3/3/7HcDCzPQEIAEEAvv8DDw0n"
    "AAX/A+/1+fi4xvbRItcE4ycFDAQADw8MAAUDBAD99/4A8PD0AMjp1gCR56sAoua2uvr1/F0M"
    "AwnY/wH/AAKI0dQ4HRhTjAxf/31n+PPnD/c/RkZPDja2LGkBUSMRbn4OUEJ8++3jz6fvgWnw"
    "14/pwPyxkY2F9TMnIzA7Aw0ACDAAaS1I9aB1XNgAAAAASUVORK5CYII=")
index.append('led_purple_icon16x16')
catalog['led_purple_icon16x16'] = led_purple_icon16x16

#----------------------------------------------------------------------
cat_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQ5SURBVHjaYs62MMmLsRDdf//V"
    "D/63337uZgCCYDVtBnNJaQYTKWk7JQmhilsv3l6wlZT5HGhqwKDMyRegJS+edefV28OsLCy/"
    "AAIAQQC+/wTK5OkDW1YtCTzn8mTZ7PGGAAAA8Pr+/gD1+v0ELCERmV4rKyj+/fdCBgMKtv7/"
    "+wE7NBZjJCMaDMrh5Arf7O9kAojl7tMPG568ZW6REGHnBWpWBOLzHOyMDOxMjBZ6UqwqtnpK"
    "DEpebmq/BFXUfv38xCD46iID78NnDJaKIoFAAyoAAojJTkFenI/tH+urDz/+ATX/5WZjZfj7"
    "g5GBi5XRX06Xg+WHHhcDv/gvBr5/ZxlE2e8wcKnwM7AYcDJYaAsLMDEw2wEEAEEAvv8EOToc"
    "jw8QAQXH1u3ZIg0O0jIcEgAF+/4AICQPWCE2LgD0+PkA793FAAT/9gD9/wEAAwsWABgcGgA/"
    "RiwCK7nbagIAQQC+/wI4NhA2s8Tk/tTr85k5FhNXGAwIkAYI/K8pKQwbDAgDAOzv+QDo/A0A"
    "HTkyAA4nKQDd6/0A+/j8ACEcDAA2KRUxAgBBAL7/BOjk7vsaBPEFbEQ0d1ddVxUVICIC/vju"
    "AAz5xAAA8+MAHQsEABYRAwD4uggACBH3ADAm9wAK+eAABAgAAAcJBQYCAEEAvv8Ek7HYsI5l"
    "J1djTnEXCCZAAAD99QD94scA7s71AAEdLgALCgoAC//9APbrCwAGFQEADiEFAAAFBgDf0+wA"
    "xNDq6QIAQQC+/wLv6/Ss7vEH/wgO/wAA5LkAAOK3APPauwDjwb4A27a/APkCCgAADhcACkT4"
    "AAYwBgAAEhkA6uf+AHp/ufvS5emMAojZR1ne3cqY3UNSQYCBiQloCAhz8DAw8QsxvLt2keH1"
    "6VMMjH9+MHAyMDK8f/yOYfuq8wy1Ves/bTx+vQUYaxMAAoiFgZmR4+t/ZoZ/7CwM376+Y/jy"
    "4S2DoJwKg6CZM8Mfxv8MDxfNY3j69hUDh589w7k9FxhqZr59duPTFz+g5rOgVAsQAEEAvv8C"
    "CAkFKAcRGADz5+wAmpKyAIR+rPzgzMwACgH1ABAdJQAECwIAFS0oAPj+BQAK/uUASTEHEQQB"
    "CvUDAQMABQIDAAIAQQC+/wM8JRlyREMpH52iyP+BocWxxt/ue9XN4iTo3eEXHCkzAh4sLAC/"
    "xuH/srbb/yQT9QE3I/0F6vEOTtjp+acFAwP1AojBVU2mpiTQ/IeOjOhtoIvYQM4SYAFTodPq"
    "sn/8///w/+xCz79sDIyzY0xFH8+LEf7vqSn0mYGB2YKFiYUBIIAYdMWFQIrtgdgc7CdGRgYJ"
    "Lk4Qk8HPVGvb1cNT/69vi/zPxcxYw8XKKSvJJ9DHwMDoD5RmAakBCCAGfQlhBnZGYKpmQAAB"
    "dlYGXlYWBkE2Fml1Mb5VqhKCh5kZGYNEeTgZRLg5GZABQIABAHyzVcIgx9zFAAAAAElFTkSu"
    "QmCC")
index.append('cat_icon16x16')
catalog['cat_icon16x16'] = cat_icon16x16

#----------------------------------------------------------------------
palette_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAApFSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsIKJcjBFV9D8SBST+/QNS/yH4z1+I+O+fDEK/fzNo/mJgMAYKKTEzMCiwsDNKMTH/"
    "Zweq//HnJ8OTX/8ZbgF9eIKNgeEQ0Ib3aLYwzAcaCBBALKS49t9fBmagwTZAi0M4uBksJFUF"
    "dGU0+dkVtbkZhCSYGDg5/jIwMf1mYPj7jeHP989m969wMJw+9I/h7rW3p4AOncTBwLAU3UyA"
    "AGIEpQG8IQAEf34z8P/6yuDDxMKQLqkuaqLhoMRp4CzLICnHzsDB8QdoITAMfoJUA9kM3xkY"
    "gF7/+Y2Z4e9/Fgbmv78Z7px/zbBoyu1/9x9/nsrFwFDECFEIDgGAAMLrAGDIs/7+ypDAws6Q"
    "pO1iaGHoq8egYy7BwMIB9OX7DwwMP34CLQOy/wBVMv9h+PCRieHMK3WGY68NGa7+VGNgY/3P"
    "oMr1kCFYbieDNMc9hgVtlxhOHnu+kJ2BIRNo43eQAwACCHsUgJz4k8Ho7y+Gdk1HbTfn/BAG"
    "JSNhBqZvTxmu3rjBsOTVPYZPHz8zGP/iZ4hQUWLgEmVhePeKkaH2WhHDeQYzhn+MrAysggwM"
    "QIczPOI0YLhyS4Fhpm4zQ06bKcPXwuPxV86/fg6MjkqQVQABxILN8t/fGAI4udlnutbEiVkn"
    "uTKwfr3JwHD/DMOJp68YcpkfM7yR/8zw/90LhvVvHjBsvPSIYZqqEQPLf0GGY08FGH4qsjLI"
    "Ak1nA6ZKFk6gcfwMDExP/jP8ePuTQVCCjSE0U5vhXsGRrD/f/qwD2nYaIIAwsuHPzwzponLC"
    "y5KWTxBzyPFmYH10gIHh8RUGhm//GGqf3md4LsfKIMLCzMAHdKigDAvDYdVPDBvO3mcQl2Jg"
    "qBPuYBC/sprhwe2nDG+B8qwCDAyfgLEj+O0ug5jwFwYGIFLTFGAwsJfgA0ZcCMg+gABCccDv"
    "7wzh0hriM1JWL+NU0hRhYLi4FhgX3xgYuIHeEORmCP7Cw8C46zHDi7tvGD6++sXw7eEfBqPL"
    "zAwmwuIMDOxcDIEeXxl2+ExnmMaXyiB7dTnDi1sPGD6evMJg+HcfAzMPMDj+ARMXEyODhiEf"
    "AwsTsxnIToAAgkcBMKUr8gjx9PvW1jL8eXWZ4fGJtQyM/7gZ2Hl5gbHCyMAhwsOQ4WLCoHec"
    "n+HslWcMzEz/GWT4eBgMFMUY5NSAjv0Lyg3sDOxS4gzO0r8ZTO7OZrj7Zg0Dl9xXBg1FYDAw"
    "8AITN6hAYWLgF2ZmYOZklADZCxBALLB4//ebocbQw0GSn+sWg4DsNgYWNV6Gf88ZGf6++8vw"
    "/ysrw/ub7xn+q8syWLnoMFh9U4f6BoiZQVkQlPeBhjOCEhCQZmFl4NeRZDACJiYGJm6gHCsD"
    "w+df0Bz2n4GZ5T8DEzMTO4gLEEBgBwAtNxBXEAg2DIln+Pl8KgOnFTAVfQWWX3xAjQxAdQI8"
    "DN+/vGN4/+gzA58UMGJBukAOAFkKshCcbZkh+ZebjeHTy68Mt+bfZmC8/5GBi5OFgc1cjEHR"
    "ToaBCVReMPxl+Pz5N8P/PwyfQLoAAgjsgD8/GDx13R35hVUNGJ5efwtMPUBhRiBmAjoAmEpe"
    "n3/McP8OE4OWvwbEIkagRYygvP8PZB5YDZjBzsrw9OobhlMtxxj4b79nUAaKigLxk+33Gc7n"
    "/GAwjtFk+PflFcPblz+Bdv57BNIFEEBgBwBDzEDVzoaBleMjw9dHbxi+XWRh4OL+zfD71UeG"
    "J6e/Mbz9KsOgHmzCwCPMBgluZmZwtAGLOqDlTJBKAhgdoJg+O/kcgwzQciUgWwCavgSB6m4v"
    "usHw0kiCgVPyE8OjW7+BgffnMkgOIIDADuDgZFDiElNg4BD4ziCqY8xwY81DBg72X8DykpOB"
    "RVKPQS9Yl4GNA2jpD2A8sgC1/GVElJggzAS0+h8jOAn8AkYLKHUJg7I0EAPDk+E9EH9+/4Ph"
    "0rYbDPIBAgz3L3z6AXT2DpARAAEECQFmBh4mVqDvgLWMnLkcg5S5DsNfkEe52RlYgBgS10Dj"
    "mEG+BTrk/2+o5f/hKRtUcDMCpYyzjBku1BxkeA1MB3+hDngMzHq/ZPkZtIyBRfW+DwwfXn45"
    "wMrIcBxkN0AAgR0ANP/t388vgHGvD05ULOxAzMMGTQtAU9mBbB5ZYOn9GYifA33MDKmjGZkg"
    "KR8cDUD8A5iXjcUZuKe4MDzY+5Dh/f0PDGw8rAz2ZlIMUlosDPdfP2G4NP0FMBX9nwjUBq6Q"
    "AAIIkgh/Mdz++PCqtay9MyThgQwFhggwVBme337G8PTmIwY+aWMGNadYBqY3m4COeAYuUMAK"
    "GJCiA8T+8ZdBTJGfQSxNl+Hvrz8MzIwQW169uMpwbMs3ho+PP68F+msHrKYFCCAWqN4ztw4e"
    "StCJKWYAphKgOe8ZXj97wXDl8FWGV+evMbB/e/vlLcv638JCgoKiulpABzxFWMyIVG+DOCA+"
    "0BHgxgMDpOT7+PYew5E9nxnuHHr9konlXzUoDcMAQACBMxArJ8POOyfPv/z68CwDg7Qdw50r"
    "1xj2zt3K8Gz7wZ/cH19P5Of4Z8D1j6H7682jkBoGFNz//8ILFrCl4Kz5H8kxINNZGL5/f8Jw"
    "8vRrhoub3gCL+h+FzKwMN0EBzAh1P0AAgUMAGCR3Pr/9v/30iikJ0g5ODEcXnWD4//TtDW5u"
    "hnxgmtv1D9IU+/DvP7Sw+fcXkQAZoG21/8gh8R+ct39+e8Jw4vxzhotbPzN8f/u5iY2TYTl6"
    "5QcQQGAHgFwDlJx0Zu3+sBsHj3L9e/19G9DybGAB8wAk/+sXuLTU4ZRUBpdawMgF2sEIzQH/"
    "kfA/qKnMDF+/Ai0/9Zjh0taPDJ8evpnFzM7Q/PcvZtMDIIDgdQHQEed/fPgb9+/zd21ePoZu"
    "oNh3kMOAlRQoB7ICE5Mqp4QiMHN/ZQDnUQZoTvj3H+EQUJEILB3fvLnLcPzoc4Zbe98xfHny"
    "YRIwXRcw/GbA2v4HCCCUBgkwx60FKl4LcxTIHhAGRjcLKwcDNxMzUPm/PxAMzr+QxAZ2BChL"
    "svxgePz4FsPRfa8Ynh1/x/Dzzad2JjaG2n84LAcBgADC2yr+/Qceun/+fGX4/vcbsFhhUYO0"
    "zUHZ8D+0jQ4M8n+/3jBcvnaX4eyBtwwfLr75/vvz90pg9pvI8A9/SxsggHC2CUHV+5//8Nz2"
    "G1iznnl2Yq+zoKEOsGDiAWY1YKEEChHGrwwvXzxnOHfqCcPdE+8Yfj16f+3Pn99FQMt3Mvwl"
    "3NQHCCCcIfAXFq2w9gIjw7ozG3blS5jIcAhJ/2L49/M1w/uXvxiuXn3CcOvse4ZPN9//Z/j8"
    "dQkwGZQCC8qXf/8S19cACCAWLJ4Hp60fPzDUnnr79Pum1Q2LwqQMJRl+/vjD8Obxd4avT74w"
    "MH76evn/339dwGSwBFw1k9DdBAggknpGwHIl5wOwxHhz54s9E7D8Z2H8dxPYsdgBzBCbgPgF"
    "OX1DgABiHOjeMUAADXjvGCDAAP0a4eB7YO5SAAAAAElFTkSuQmCC")
index.append('palette_icon32x32')
catalog['palette_icon32x32'] = palette_icon32x32

#----------------------------------------------------------------------
markuncompleted32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAn0SURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsIOKMAiMDG9ApLEAMchEzIwMDzz8GBhe7WIYvv1+zcIh+z7a21ElUkpFVePHpyYdt"
    "uw5uuL/+XsMj1q8fZPmAGoBq//+FeocZYTjHWQaGX3gsB4U+QACxYJeBGPrn/39RJX3uZUFh"
    "ni5SElIMf4G2KP+X41cyVMqf9WeF+cud53bKMv4T/8/JKsbIwyX2/9uP9wxvv69lYGdYysjC"
    "8AdklL8AA8OeTwwMX/9hdwRAAIEdwAj08f8/DIJ/fjJ4MrMwyDNzMEiwcLOKF7+7qrtCUUiL"
    "RZCJ4dDLIwyMQBf//v+bgREYREZFahYl7L8sNjqGMPAriTD8Z2VjYPz7jYFh5yHf/0s3Jrz7"
    "+jMMaPTrM18ZGBYpMjAkP2Bg+PAX0wEAAQR2wN9fDGIskmK7JFOj9bkUZBj+c3ExPP7FzaDz"
    "jY3h5qlehgNK+xiY+DkYnn55DgyF/wx///1j+Mf8h+GCyj2G4G1fGIy13Rj+/PzDIMb7hSE9"
    "QopBQF7UYU/21E6gfUlPfzMwFD9mYPiJI60DBBDYAd8/M8TKlUfqi6ZnMnz79IwhbxkvgxCf"
    "GIMwFztDsPwfhulb0hhU9eUZPjD8YPj89RfD758/GX5+eM/AcI6BwdxEmSEv2pSBg52BYf/F"
    "Xwzdi/cwtCYaMHjYqUQqLrvTfpeB4fYDPAkBIIDADmDiYkgQ+H0NmGI+M3x78ZzhzXs+hpkZ"
    "MgyQBBLNIL/pO0P34Q6G9/9eARPObwbu70IMmn/tGALNghgK48IYeIEJ8R/QhwFmbAwHT2ow"
    "nLv6gsEowo4je9edgsI3DNmgdPkXhwMAAgicEo+oMPz5W8j9///jbf//f1z7P7t/6/991379"
    "RwavXr/5f/HG5f+Xbl7+//Dho//fPv0Ai/8D4p/AePnx+/9/IPX/xO2//wub1gPNav3/MkDm"
    "swwDgywzjvwOshsggMDibz8y7Pr0CJha7p5kYGDnZXBQfM+w89xrFMWiIsIMeuo6DLpqOgxy"
    "crIMnLzswPQAzCmgLPgfknGAyYDBWIWJ4SO7PsPRK+wMYuGmPIl8DLl/8WRFgAACO+Drd4Y5"
    "T54AGdf2AE1hZXBS+8Fw5/5jhq844g4U3CDL/0Hxfyj9A+iAb8BEF+CuwDBjJ9DvRvIM8eYi"
    "yaIMDOK4Sj2AAAKL/WVk2PXoOcO9/7eAqerNKwYhcSZgtnjKcOzmTyzBBi0moBbDfQ/05m8g"
    "/gbUYq3DyPBfxJxh7wUuBuVwHaFIToZ0UDHAiMUBAAEEcRQjw5dX7xi2vr3/nYHh3hkGBlZu"
    "Bmfltww7T7/EVj7BLf4Htfw31HJYyPwBhlxUgALDrH3qDAyGkgyJRnyZwHQqyIDFEQABBA+V"
    "778Y1twFRcOtg0DvsDE4qn9muHP3EcOXXwjLUSyGWv4X6KJffxHRAcLfgHqMVIFFsbwjw+bz"
    "QgwGYYoSgWwMSdhCASCAmGCu+sfIcPLeM4a7fx9dZmB4945BSIyBQYr9EcORaz/gQf8PzXIQ"
    "GxT0f/6jpQugTT+B2iICZRnmHzFl+KcvypCozZHLCaxi0NMCQACB2X8gBv588YZhw/PHwGh4"
    "DnQECzeDi8pLhp0nniFCAC0NgFI9LBf8Q3PED2Bi1JIDhrumG8Pqc1IMdsFi8u7MDDH/0UIB"
    "IIAg2RCa1r7/Zlh9A1h0MTw9ATSdncFR7QPDndsPGN7/gNQXsDj/A03xv/8hLEX2PYz/A6gv"
    "JECSYelZJ4Y/egIMCaoshazAmEGOAoAAAjvg1z+Iq4B6zt66w3D195PrDAxfvjAIiv1hkGZ/"
    "wHD48jdwsP2DxjfM53Af/8PMmiAxUCgoSTIwSBp5MSy9oMrgEcijZsfAEIocCgABBHYAyCc/"
    "IdXlnxdvGdY9vPORgeHjLWDdzsXgrv6UYcOBJ+C6FeRQkNq//1AT3T8sUfAPGk3fgRVkgJ8o"
    "w+prAQx/dEUY4hSZSkBND1gIAAQQpBwAqn7/GyIAjI21ly4B9b8Flgm/uYG54RXDo7v3GB68"
    "R0TDPxxB/g9LqPwCulxamIFB0dKLYeEVEwZfXx49IwYGX1jlCBBAkDQAZTBC6Eu3rjGe//EU"
    "GAK/fzMICP9gUOK6x3Do/BcGYFsBu2/R+LCEygjyKiuE7RskxLDhpg8Dg74YQ7QkYwkjNBYA"
    "AgjsgIvQltQfiCP+v//8f83dKx+Azn8Iriq9NB4w7D/6CFRiYvgeljCZgHKsQMvYgEmMBYh/"
    "AflPgD47cu4Hw7ylLxkmdVxjeP4AGMwK4gzuznxW+gwM3iC7AQIIHBfAipjBGohZESXe+sun"
    "/jVpR11mZWBzZHDUuMUw+dgdhmfvtBgEOaFNP2gbEpQjPgNzLjALM9x7+I3h1u2PDPduvWF4"
    "9fgNw79PbxiE2N4zKIt/ZghU/Mrg0PafQUCUn4HDQ4ghaNvXEqAxWwACiAUWBUC/MmhCQwEY"
    "GjfvXv536uvju9bcoi4M/EKfGTR5bzIs3/aWITVMmOHRy//AVsZXhhu3PjLcufWW4dWT1wx/"
    "v7xhEGZ7w6Ai+o7BT+4jg47pFwYlmR8MQsLAYOIEmsgMdPlfbqDrbRgYxdWBBd8MdpDdAAEE"
    "T43AjMegDY0TUPX58cv/VbfPvLc2MHsBFOBlqAm+wpCzYDVD/HYFhj/f3zEIszxnUBV9xeAv"
    "+55B2xhomdRvBn5hoHFcQIuYgSU/I7AC/Ae08C8wXH8Cg/47MEq/vALyrzE8W36GYc/b7+vr"
    "gfYABBAjqFHACEzeoNZ1FhCzMUCa0kB3K+hYsV6PXWjLwfDPHphNgFU1Az/Dg2dMDMKCzAy8"
    "IkCLuPiB8QCsYxhBTSJ2SOEAbFUx/AC2JX4AS9Dvz4E00NKfQMu/A0ulr38ZvgFDb8Wc318K"
    "Hv43+vT//22AAIKHALDlzHAfiHWgDgAG2oP75/8c+XT/pgufoScwGJOAueIlg4IMOySp/gE2"
    "YP68A9Y8wMj7DbToF9CiXyCLgAni8x+G72//M3wEVqZvn/xnePnoL8PTZ3++P33599n9N39v"
    "7frJMAnozNsgewECCKVfAEqMetAcASpIPn//v+rWid8uJiYLgHGjAnQEqLMANPU30KIfwBLm"
    "0x+GH2//MXx69o/h9YN/DC/u/2F49OD3z0dP/rx4+ObPgyff/98ARuA1YPq8CSza7gF1gCqW"
    "r8h2AgQQPApAABioDGmg5hdUFTCBi8mosF8KnSQlzi38n+HL418Mb+79ZXhx+xfDk7u/fzx6"
    "+Ovlw5d/Hj758u8WMAyuvYZYdBdq0WdC3TKQ3QABhOIABqjlwOIbnElBCXI7A4OZGhNDtTAv"
    "s+Tjj3/vP2VguAwM7OvAgvEW0JHAAGZ4T26/EGQ3QAAxDnTvGCCABrx3DBBgAPQVjAlRMxeD"
    "AAAAAElFTkSuQmCC")
index.append('markuncompleted32x32')
catalog['markuncompleted32x32'] = markuncompleted32x32

#----------------------------------------------------------------------
mergedisk16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANtSURBVHjaYvz//z8DI2PfOQYG"
    "BnUg/gXEjAxgwPiXgeH/FyDjPxDzADELA1QCyn74/3+RFkAAwQSLgVgaiH9D+UDNv78CzbMF"
    "qmdlYGA7wMDAxAlkg9T/A2I2IH4FUggQQDAD9jPAAVAvwx+Quk4vL40oNlZGlg0bb3IALS0A"
    "6v8Ldg8SAAggJjiLDchkB7nuDzvQ1Wvra+3KVq30l1m6zE+it8cph5n5/w6G/7+5GFiBJrAy"
    "wrUBBBDEBXzMEJ99B7nuf09psUVQXZMtw4zzDAxff/5nKCyyYPj5869LVfXhaUAFiQzsQHew"
    "QQwBCCCIASBnMQEFfv0O8fJUy+rssGNYdJmBYcO1/ww/v35n+PP5D0NllRnDhQuv41atvnGE"
    "gYNjDsM/iF8AAgjiBZDTv/5Ul5PnnzxzhhvT4ZcsDMsv/Wfg/P+TgePfT4Zlxz8yLDrykWHy"
    "ZAdGdQ2RbobvP/UY2CFaAQKICRoxLEATq1ra7CT+i/Ix9B4FBuGfXwx/v/9g+PvjOwPTn58M"
    "9aueMtx595uhs8tagJmFuQXobE6QVoAAghjw/bdzZIx2XGyUFsOZpwwMany/GSRZfjB8+fKN"
    "4cvn7wySXH8YDKWZGHacfsPg6yHDkJSs48v4658/SCtAAIENEBLjnpWWrs/w9sMPBivBbwzd"
    "Ln8YrKT+MHz49JPh3YfvDKYyTAzLMqQZEmyFGF68/cGQnKzJIC7HOxWkFyCAwIH4i5XxU2Xb"
    "sW//fv/7yvLjN/+MCQ5snOzcDN+BLvj+5SsDOwsbw93HXxiyio/8+vr3/0dWDmau74wMb0B6"
    "AQIIbMCXH7+dThx+zMvw6+8fYHqY+ubTL58/jBwMv77+YPj1DRgOv38zvP/EwnDowqvdDF/+"
    "ZDGwMDEzCLJ/A+kFCCCwAYyMjK//c7K8ZuBgZuDgZP7JCkxUX9/9ZHjz7jPD909fGT595GZg"
    "YuJg4BJg/f2NhfERw19GkB5w8AEEEAtKkgIJMjMyg7ginP8ZNEQZGX5ysjCI8TIx/AepZ2Zk"
    "AicgUBL4D0kHAAEENuA/4z948vzPysT+FZgdQqyEGGyUORh+//vDIMTHynDu5keG/yyMbHB1"
    "UAMAAgjiAl5WuEN+MzFeqJpyzlVKhOMvw79/YGUgh718/5PlJwvTTQZuVpTMBBBgAK0sJue3"
    "FWqzAAAAAElFTkSuQmCC")
index.append('mergedisk16x16')
catalog['mergedisk16x16'] = mergedisk16x16

#----------------------------------------------------------------------
die_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOzSURBVHjaYvz//z+Du7s7w/fv"
    "3xn+/fvHwMHBwfD+/XtWXl5eZV5eHhc9Pf1oIFtr3759ZX///p3JzMzMwMjIyAACO3fuZAAI"
    "IBYQA2TI79+/GYAKhJiYmPw1NTXtTExM/PT09IS4uLgY5OTkGCQkJKYuWbKEGah5GgMSAAgg"
    "sAFAQTtpaelYQ0NDW3NzczWgRkYxMTGGs2fPMxQVlTBERIQzpKYmMbOysk5ZtGgRL9CyTjY2"
    "NrABAAEENsDKympbZmYmt6ioKNx5799/YLh06RJDeHg4g6enO8OHD+8ZfHx8GNnZ2TvmzJnD"
    "CXRpI8jxAAHEBFIM9CM3CwsLWDPIKyDw8+dPhoMHjzI8f/6CQVRUBCjHxPDt2zcGV1c3hvT0"
    "jHoODs42kDqAAAK7ABiAfy5evMiiqqrKICMjA1T4nWHfvr0Mbm5ODL6+PkBD/wDDiZGBmZkV"
    "SP9j8PX3ZHj7/nUFUGslQACBXQB0zl8gZrh//z7D58+fwTGyd+8hhpMnzjJwcnIxsLOzATEj"
    "AzfPf4aff94wfPn+ioGLk/0/SC9AAIENAEbfP5Dzubm5GZ48eQLWMG3aZIaenjYGoM8YGJk+"
    "MDCxfGA4c+YMQ0xkNsPjh09BXvwF0gsQQEzQWPgHcsHevfsZXr9+xfDu3Xugxm8Mf/8/Z1i/"
    "fhHDq9dPGbg5BYEB+5VBU0sTHCZfvn79A9ILEEBgA5iZmRi4uDgZDh06wXDnzn2glxiBhn5l"
    "2LvnBENd3RSGVy8+MixdtoZBz0CFoXdCLdA1bAxfv/4AhzZAAIEN+PTp86dfv34zFBZmMejq"
    "6jL8+vWL4eevPwxq6nIMGzbOYLh95xlDV9cshjdvPzAsXryO4c3r9wxMjEw/QXoBAghswI0b"
    "N6qBsfBHQkIcGA4c4Gj88eMXg6KSFIOYmAiDhpYCw4pV0xgunLvDMGXifKD3WMHpF6QOIIDA"
    "Bvz582f+nj17qi9fvgzMC1zgpM3JxQEMtJsMXl6JDEAfMqipKTGYWegzTJjczvDwwWMgfnQe"
    "pBcggMDpAJQsgYZ0bdq0SUhQULBcT1cPaMEfhu9Ab3p4OTOIiokzXLx4k+H6tVvvb167c+b6"
    "jRurgZlqFUgvQACxwDIFMJ0zfPnypWL58uWCPLy8aSYiKgxamkoM7GycDIsWrL1y+/b9ja9f"
    "vtwEjLFTTEAngVIuCAAEENwAkLNBWfXt27cFq1auFL53z9Dr8qWre+7ff7QCGOf7BAT4XgDz"
    "ATjLw/ILCAAEGABjwFqAmv9BvAAAAABJRU5ErkJggg==")
index.append('die_icon16x16')
catalog['die_icon16x16'] = die_icon16x16

#----------------------------------------------------------------------
listview22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAS2SURBVHjaYvz//z8DLQBAALHw"
    "82df09KV0mBkYv7/5+9/hn9Awf9AAkT/+w/B/6H0XxgbJPcPSR6mloGR4d/ff4zvHzx5DBBA"
    "LLqGMuql0/MYP/3nZPz14x/DH6CGX38ZGH4DMYj9B8oG80H0PzQ+TOwP0GJGJoafv38znOqf"
    "IgMQQCwsLKwMt9+xMpx+9I/hL9BEmMv+/kNgsNhfhPgfmPg/iNg/qAP+M/1nEORjYWBgYWMA"
    "CCCWf4ysDPeuvmTg+vSTQZCXGewdBjCG+hfkd0aQJqgoM8TrUBVwmgko/xPo9HOvOBl+/WFm"
    "AAgglm+/GBlEuf8xpAVKMUiLcgLD4RdQBVArK9AEDmYGVGMYsPBhYgwMX7/+Ykib/4nhNNBM"
    "gAACupiJ4fd/RgZGdk6Gf0+fMjxYOI3hL/Mjhu+/GRgUQmsZ+NTVsRjCiJkMgD4DBxPQLJDz"
    "AQKI5S8jMzj8GJkYGX69/MDA9XU7g4TrdYZHVzkYvjxMAhu8c9deht279jNoaKgxxMdHMbx+"
    "/YZh3rzFDL+BMcbMzMTw/ft3BgN9HQYPL2+gBcCUATQTIIDALv4LtZSZ9R8D0+M3DD8P/GT4"
    "94GdgUWTGSwuJyvDYG1tyiAmJgbmc3BwMBgb6wMdBIwRRlDo/WaQlpQAxgkk8v8BHQkQQCyg"
    "JAJy8X+g7awKcgws9vUM796+YmCVEmTg19EAG6SpqQ7GoHj8A4x+bm4eBk9PN7AcSC84Tf/5"
    "yfDt23dgcAA5TMwMAAHE8h8UFKBE/gcYqIICDELJKUihCYT/IfjvXxj7H5j95ctvoIEQ9l+g"
    "6UyMQDmgc//9h6QVgABi+QsM6L+4cjUsp/2DcEAGgQyHGQh2Kciwf2AngBX/A0Ue0LEAAcTy"
    "DxZ5wCBh+Peb4f35awzf339hYObmYBDU02Rg5uQEp1EmJhagmv8MP378BFvGysoKDmOQJSC9"
    "///+AgcTyBKQBoAAgoQxiM3BzvDz5k2GF3NKGThkrzG8eQ3MLH+XM0jaWAFTwXuGT58+MbCx"
    "sTPw8/ODI+vt25dwg0FBwc7GysDPxwvkQ1wMEECQVPEP6vMfvxnERR4yCFk/ZeC8Icjw9+sP"
    "sPiRI8cYduzYxaCursaQkpLE8P79e4a5c+eCXQhKbqBI09XWYggI8IcECQMTA0AAgdMxOMBB"
    "3uNhZfj5mJfh3Qo+hi//JRh49bgZfgKl7B1sGcwtzIBRwgR2LS8vH0NGRjo4Bfz9A4o8YHoG"
    "JrGfP38BI5AZHBQAAQR2Mbjo+/WDgVlVhUG0bi3Dr08/GKQ52RjY5CWAwf6LgYeTnYEbGFQg"
    "F/758wdoPLCc4eECBwUYg8IWGBwswDhgAuVKYHIDCCAWRmYWhh+//jM8e/6JgfE/N8MvfgkG"
    "RgFGcPH1980PYPr8By2PgfS/f5BUAIr9v5DUARH/Dy7EgaHC8P0nxGCAAGJhZmZk/MbBx1C6"
    "+RtQ7gOkqPyPKMT/QfM/xEBGoPx/cARBikug+H9oPvgHKSs+AB3HwMnECBBAjGwW8+4LS4vJ"
    "//3PAq5BQMkQVIPAaoX/SDXIPwYoH6mGgdds0IwBtJLx/6eXLwECiJFWdR5AADEx0AgABBgA"
    "eN992ho3vXMAAAAASUVORK5CYII=")
index.append('listview22x22')
catalog['listview22x22'] = listview22x22

#----------------------------------------------------------------------
arrow_up_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAbLSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAD7gCAABpwBwAEEAsyh5FnInZVf/9D8D8g+x+Q/s8gw8DwOx3ICwLqYgMKbmdgYJ0B"
    "ZF9jYGJkYAB5iwVKM4NoRhTj/n/Ig7MBAoiFZCf//+cMtHyis7OydmysNgM7OwvDunU3c1ev"
    "vhYAtK0SaPNSUowDCCDSHPD3TxQzC+P02loHvrJSUwZOToj2kBBVBicnednSsgMLv3z+KcXA"
    "xNpNrJEAAUR8GvjzN4aDg3HutOnufPV1lmDLb7z+y3D26R8GFua/DBkZ+gyrVvkyi4pxdTH8"
    "+VNJrLEAAUScA37/dWP4/39W7wQ3jrQUXVBCYNh//x9D61FmhrbjLAwrL/1n+PvjO4OnhxLD"
    "vPleDHz8bC0MP/4kMTAyEjQaIIBQHfAfCSMSoAqQmNHabseZla4Hltxxh5Fh0hkmhq+/gU75"
    "84dhztl/QPyH4duXrww+XooM02e4MnFyMvUx/P7nQMgBAAGE6gAORghmZ4SkYmZGboYfP2bH"
    "xOkpVpSZgX1+5NE/hpnnGcGJm4XhLwMj0AFs/38zLDn7m2H1hV8Mf75+YYiKUGcoKjPjZ/j1"
    "aw5QkzQ+BwAEEKoDWBgRmJmBkeHLr3YDYymH3k57BiZgVrrx5j/D7PNMDKCAZf73l+H3z98M"
    "v3/9AobCbwYWoCPmHPvKsO3qD4ZfX78xVJQaMXj7qyozfP41DaiBA6wJhpEAQABhjwIQ+P43"
    "il+II3vadDcGMVFOhjdf/zBMPcvE8OEnIwMr0Oc/f4Esh+A/QAzMIQw/gA7q2f2B4cy9bwxc"
    "LP8YJvTbMyhqCPkxfPtdwsCK5DkkABBAqA6AufD/fyWGf//a2jvsmCxNJRh+//7DMOMcM8P9"
    "94zA2PnL8Ato0V+oxX9gDgGKsf7/w/Dm40+Gti1vGO48+cqgKM3BMHmiPQMbN2sFMD1YYwsB"
    "gABCdQAoYhmBJdvHXxOiY7XlMpOBie7/X4YV1xgZ9j0Ayv39y/Dp60+gA4Bx/RvmgD9gDIqK"
    "j19+Ah32h+HYzc8MXVtfMTx/8YXBw1GKoaTAkBuYYicDS0RBBjZUFwAEEGY2BGYfKWUB3/pK"
    "C3B8vP78n+HxJ0YGS8k/DAaivxn0Rf4ysDH8YfgJCgWQI4D4FygKgIlRX4qZwUyelcFZg4Ph"
    "zYefDDeefmN4++YLQ2ayFoOhlaQh0BE16NYBBBAjcnuAUWaKNuPX37sXzvaSjA1RBxoO9Bmw"
    "DmAEOoQRGBL/gfXAHyC/ZPNXhguPfwIT3h9wKHz//puBh/Uvw8xYMQZJfmZgWvjH8Ado7q/f"
    "QD1//zEICXIwXAaGinf4th/f///3/v88dx/MToAAQg2B73+q/IPVwZb/BWoEOY0ZGC3g8oSJ"
    "GRQ9DH/+AQ3/8xeY5v6AHQjBvyH0n39A9UwMXLwcDNw8HAwCfJwM/AJcQE8wMthbSjCkpehw"
    "MPz6hxIKAAGEWhdwsXBcuv3uv7n/6v+/fv2FJcx/37/+YdLVFGFa2OsAFgDFMyjR/YeGABgD"
    "ffwPiDk42Biq+s4zbNnz8B8XN+s/mCeB7mL49uMPEyMvC4qdAAGE6gBWptx79z6suPfr7z+k"
    "DPmX4dsfJ05Gxrw//yH59M8fiMUMIAcAHQLCf4FyoCgCyd958JHh9rmXyxi4WVej2MHMxMrA"
    "wXwO2UqAAEKvDZ8Bc8JqBjZmRLnAygTKuxys3Mx5IIF/QLeBgvwv1AG/gWwwZvwPjjZQKLBw"
    "APXzsJ5n4GDZBAwaRNbDUjcABBDu6hhmOahY/g0syaDgHyghghIn0AH/kKMAJAdKN/+hFjIx"
    "sIMLH3BN+h8j/8MAQACx4LQclF85mSBsZkYIBoUAMBH+/Q1NAwx/EGUBVA4UDf8ZGRF6QCUf"
    "MFcw/MLuCIAAwnQAKPbZgRZzMyFSATMUQ+P585dfDO8//QCWfH/BBdC3b8ACCFiJ/YUlHSYk"
    "PeDEzQQR/43ZBQAIIBYMy0E+52ZGKy0gZSgo+EGWuGhxM6iIMDEw//8HTpCggogd6Ft2YJD/"
    "AmZFSJHLiIhzEMUNJL79g9iBBAACCNUBIMt5sLRRgO5hBhrOycbCwAhsCWV5iIMLp7+g4AYG"
    "O9hhQIO///jNwAZMN6xsTJC0jx6+vKC8iBoKAAGEqoSPGXtKYWVi+gSM8yNX3gAzCDANgHwJ"
    "csJ/UKJjgGAGCIPj/R+GVx9+AC1nZkVvDYMBD6oYQAAR1yjlZDl/8/mXu6H5u0SBofobnjYY"
    "kRItLKr+/2f+/PvfDwZullPEGA0QQIwD3TcECKAB7xkBBNCAOwAggAbcAQABBgDRZ+/wS9rd"
    "MgAAAABJRU5ErkJggg==")
index.append('arrow_up_icon32x32')
catalog['arrow_up_icon32x32'] = arrow_up_icon32x32

#----------------------------------------------------------------------
charts_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQgSURBVHjaYtwfw8DAzsigx/id"
    "wZiRkYGB4T/DPwYGMM3AxcbA8Po7+7eWtx2eL2W8/FnuHV8/TT5pHx/zP64vTAzn/7EznAUI"
    "IMZ9kQxa0gqsh0VNDYQYmPiB2v+A9TMwMTG8ffGd4cgjGwa2wDLGR7wiDCePf2RQvDD3f47B"
    "Kkbmpxe+PLrx0xUggFgY/jKoCanxCwoGdgB16QHd8AViAAMzw8GDHxkuSekweKkwMDy7CxTi"
    "52dYxFfEaK1vxxAk7c/z4NIzA4AAYvnFyPD7z5+//xl+3mUEO/7fdwYwzfufgfkPJ8PF55oM"
    "v88wM3z8xMCw7zgDw7d3DAx/v75h+Pv753+gW38BBBDLH6Cd//7/BjKvMTD8Bsr+/wV0/n+G"
    "z+/fMVx4KsrAxmTP8OEHA8PpOwwMnz4DDWYFKmF5ysDI9IXhLyMDE0AAsfwFhddfoKaf14Em"
    "vWf4/+8Xw933QNt+/2G4xCTAwMrCxABEDExAjQxAyxRfX2IQOLqf4bXwTwagAz8ABBDICwz/"
    "fgId8/IsAwOnBMPD7z8ZVn+2Y1AztmOQefad4c6z/yCbGP78+cbg/nYHQ5fgfga166wM1z8Y"
    "MT54cFsXIIBY/gJNZ/gNNP7jH4Y/r14zvPr4n+EU51mG/5/fM/z9bwV0KtD/QCXBj1cxlEid"
    "ZBCWlWVgePWFQYeJl+Hl6c8VAAEAQQC+/wQAAwb67vQP//DOOAD16wYA7hQGADYD+gDe7OoA"
    "2fHwABQQ/wD7AwEAHfjqABLd0wAAAAYA+ff4AOy/vgAD490FAgBBAL7/BJrnlA14FNYCK+b9"
    "ACfy0AAGVv0ASgIBAGmtcwDs8zYA/ffrABETFgD7BQcA/fb6AATW1gDn19gA8OTkAP0AAP8C"
    "iOkxh08tC5sa49cfLxi+P3/L8OPAHYbOa14M9/h8GfRUpRkYvzAzvHnykwHoEwZGUGgyMDH8"
    "/wnkQ1MLQACxaFm81z1kEMXw5IYTA5vgXQbW8w8ZHhqGMYgJsjD8+MbAIMDHz3Dn5Q+Grz+B"
    "cQZ0GSgmGJmZQRTIEEaAAGJhYeZgklbiZ2Dg5mf4oKvO8AUY16LAeP8GxP+B+eEfMxODzLtT"
    "DGKfLzN8/snEwMfBzAA1h+ErUAlAADGdPfzsw7PXwIQHzDi8wNjkAxrLAnQuG5DPwcPA8OjJ"
    "W4ZwoU0MSZEXGd7p/2X48PMbA/Ov/wxPhZkZToizrgIIAEEAvv8D8Oo15ubqAk0HFxQCAggD"
    "APz+AAD5/wAA8vb1AM7N4AANzaD+Yha2//X7///1C/z/1Ovp/9n6+gD0Ng7w4f4AVQKIkYF3"
    "CihtunLwcTcJqkvpcvBzcfz4+eP/x0ev3397+Hp7uu6i9epCD/5//MHJyAxMLoxswMzDzMIi"
    "yMVw7eHVl9cAAgwAgzKFwAt/FHAAAAAASUVORK5CYII=")
index.append('charts_icon16x16')
catalog['charts_icon16x16'] = charts_icon16x16

#----------------------------------------------------------------------
cut32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAbLSURBVHjaYvz//z/DQAKAAGKk"
    "haHGxsbysrKyAUpKSgIHDx5ccPbs2Ye41AIEENVRdXV1/b59+/6/f//+f09Pz1OgECc+9QAB"
    "RFU0efLk+V++fPn//fv3/w8fPvwfGRk5EyjMhk8PQABRDXV2dsItB/l+9erV/zU0NGwI6QMI"
    "IKqgtLS0/NevX4MtB9Fnzpz5X15evh8oxU9IL0AAMVFquY+Pj3x4eHgDDw8PAzAEGJ4+fcrw"
    "6NEjhqNHj64FSn8kpB8ggKjhgAYrKysBoO8ZgL5nANG3bt16duTIkbXE6AcIICYKLbe3sLBI"
    "+PHjB8PHjx8Zfv36BQ6FCxcubAFKvyXGDIAAosgBzs7ODfLy8mBfgzDIAbdv3wY5YDFQ+hcx"
    "ZgAEEAslcW9ra+sAs/z3798MX79+Zbh3796BGzduXMZRQOlzc3MbuLm5OQgJCTnMnz+/HCCA"
    "yHaApqZmA7C0Y/j27RvY53/+/GF49eoVw4kTJ+CJz9ra2l5cXNzBxsbGQEBAwEFERERAQUEB"
    "rAdYOn6+c+fOTYAAYsHiSn6gNwIYODgUGH782HD2/fuL2NR4eXkFgAwCWQzCIPbVq1cZ5OTk"
    "jKdNm7afk5PTQVFRkQGWO/7+/QvWC0qooPQCLCdmAcuLuwABxIJsqOC9ew0ebGwFPjExDFIC"
    "Agx7T5xomHz+/IJ9b94kIqsD+jwBGPcC//79AxsMCn6Qoezs7AzZ2dkJnz9/BqsFJU4QRgag"
    "UNqyZcvmAwcOTAByvwAEENgBxsACQ/3BgwPd6ekGYkD+9xs3GHg/fWIIEBdn4NXVTXh77tzx"
    "ix8/zjLm4bF3v3x5g7KDgwCoFgVhkO9//vzJcPfuXQZBQUGGd+/e4Yy2ixcvft62bVvDsWPH"
    "loDcAhIDCCCwA9TFxQ/MLiw0YH7xgqFx3rzP+75925wkIBCV7OjIYM/Ly2DNxlbPxMfHk6ut"
    "3WsJDOr3np4MDx48YNi1axeDqqoqKNjBPtPT0wOnB2QA4gPrhc+HDx9eDiwbZgFD6hrIjzB5"
    "gABiMRYUjK+wtTXgYmZmWHbwIMMSLq6CB58+Lfn59++rsJcvC3iBcRgkKSkVJy3dq87GxnD8"
    "yhUGDSUlBmBxCzYAGI8MbEBxUHSAQgQUHSAAciCwRLx5+fLl5cC0sRYYMg9AQY4eKgABxKL5"
    "40eApp0dAzAcGZ4DfffgxYtd4DzMwfEBnt+VlRn+v33L8PrxYwYBW1uwRVpaWuCgByUyYL4H"
    "x//p06c/v3jx4uz169f3AukDz549uw7UDjLnL65oAQggFq7v3wVYgL4HAUk+PgYNERHjG2/e"
    "PFF7/dqBzcIC7DBgxDJcB/qo4e3bm8oeHs8zWVgcmJiYQFkRXO4DfbkZmP26gBbeAhrzDZtP"
    "cQGAAGJ5wcn54SfQd+ycnAxhQUEMH9av3/CMh+dBupmZAjsokQGz0EZgouz7/n3Zsa9fq+zm"
    "zFmcCkxwIEdzAx33FuiAe79+HQZafoSc8gQggBiNlZT0k1hZL2SlpDAAizKIKChE7txhePPs"
    "GUMXMFjX//tXeweYLpxERHp64+MTDERFgQH7gWHnpk0MTU+eLDv26VM5UNcTchwAEECQvC0p"
    "Gd8pI/P/aXj4/9+5uf8/BQT836Ol9T+Ym/sMsEI3AzWrgJbPP19S8h/Yzvr/v7b2/1Idnf8a"
    "3Nw9QDlhSuoTgABiRHKE/N/Pn9OUmZndn3/9evMFL++ue+/fgxLkc5DlvWFhCQbAxAgOJWC0"
    "vLx1i2HmgQMfNrGwJJx982YjuQ4ACCCCrWIXQcH+jsDAAmMDA7DFYAdAHfEHGA0L9+1jmMTK"
    "6n7p5ctd5DgAIICYCUSNfaGKygxnHx9g2v7GcHLnToaVJ08yXLl/n0FeWJiBB5jn9YCl38Hb"
    "t9Wu/fmzCF92wwUAAggv8uPkXP9j0qT//ydM+H8jJOS/FR/fUqAwsAXAIJkkKnr/d1zc//8O"
    "Dv97pKVBvRsxcuwACCC8DRIJYBnBzs8PKaTevGF4w8kJymoPgb2M5+8/fDgAroaBIXP8zZuz"
    "5HoSIIDwtwdYgNKgohXoABtgGoi7ebNtPTe3mtyvXwKFenoJXMBieC+wHLjAzr4MqOYDOQ4A"
    "CCC8iRBY+9Wvj4hokJWQgCfAz/fuMbABCy12YCvoMTAs0oBF7o4vX6JAuYUcBwAEEP42obr6"
    "hIr16y+8AdXvoFoOGBK8wHYCO7AKvgYsmgsfPz57nJm5hFzLQQAggAhmQ1BbQZaLa7K+hEQs"
    "LygUgA2QN4yMzzb9/r3l2sePzRSVgEAAEECk9I5BnUxeaNb9C63lflGa0wACDADFuDA+qnY6"
    "IAAAAABJRU5ErkJggg==")
index.append('cut32x32')
catalog['cut32x32'] = cut32x32

#----------------------------------------------------------------------
exit16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAN1SURBVHjaYvz//z8DJQAggFhA"
    "xEpGRgaQMX+B+A8DgwYLM1M+r4K8P4+8vMi/P38Y3j18+O7Fs+fbvv7+0w9Uc/kHVG070HKA"
    "AGJBNg0omC5jZtynnpfHJeHkzPDmxQuGf6/eMIj+/yP+9MD+xFPrN8Reu3W3Gqi0D2IXAwNA"
    "AIEN+A/hpWuEBMwwmj6TgUVEjOHTo0cM9y9dZfj76ycDCysrg4K5FYMUvyAL7+pVnYfOXwJp"
    "6wIRAAHECAqDJYyMWpLmpmftN2/mYBEVZ3h38SLD9wcPGP6LioIN/7Z+PQM3UB3vv/8MXB/f"
    "MSzdvv3vpecvzfr+/z8HEEBMYFMYGXM0iwrAmu+sXcvw4OxZhqenTjEwAv3P9OsXw+unTxne"
    "6Osz3Ll3l+HvyzcMVlLSzMBwqgTpBQggsAG8MuKB4r9eM7zfv5PhHxMzg4qLCwPD0ycMDL9/"
    "MzD+/cvw/8kTBgUbGwbB1DSGe1cuMbD8+8EgwMPjCtILEEDgMOCSkhT+t20Nw4v7nxkkNuxm"
    "eLNmFYO0sCgDCwcHAwMzM4Pkl68MLzs7GGS6uhmOCwgxcD2+x8DLysID0gsQQGAD/gH99vPL"
    "dwZGDjaG/0An/z19goHp/z+Gn9dvgoP4/7dPDD9OnGD4+/0bA5uCAsPPW8DA5eACxxxAAIEN"
    "+PDuw/ufirJizE9fMzALCjD8efOO4dHObQx/d29k+PObgeHr268MQta2DCw8fAz/X75k+M/I"
    "xPDh1+9vIL0AAQQOg9dPX2x7w8TDwPL2FcOvI8cYxFs7GFjFpRl+PP/K8O3NVwZuIREGpe5u"
    "hp83rjF8u3iO4RsnO8OzHz8PgvQCBBA4GrsZGfUUtNXPWiuLsny59YxBch3Qdl4uhq/rNzP8"
    "BwYin58PAysbG8MFLy+Gnw9uM5xkYPp38fNX22X//x8DCCAmqD8u3bh6s+Lssy8MrCLcDC/8"
    "3Rl+LVvOwG1syMBrYcbwfetmhmsOdgy/795iuMXEwnDh89c2oJ7jIL0AAQR2wQRgXvgKNAfo"
    "qRIlJfk2NR42Rj5gEmb5+4/h35+/DJ+/fmd4DUyNN4B+v/DlWwcwcTWzA9PXQqBegACCGwAK"
    "ERD+zsBgwszOXirEwe7OzvCP5y/QCx/+/v/26vfv/R///O0GOvkII1AdKxCDDAAIIEZKszNA"
    "gAEA1sFjF+2KokIAAAAASUVORK5CYII=")
index.append('exit16x16')
catalog['exit16x16'] = exit16x16

#----------------------------------------------------------------------
save16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMUSURBVHjaYvz//z8Dj9T81VqO"
    "mgGSXH9+MTEwMPwDiv37z8DwHwgZgTQEQBh///5hevSDm+X+iXsHvj6McgUIIAaQARbpF/4v"
    "f/j//+df3/+DwC8g/grEn/78///tH4T/9z8EvPv68X/R5uf/lRIvg7gMAAEoHIMUgEEYCA5V"
    "qyf9/zdbKsTE0HgddpbJ57Xc2eXVa3eF0bAFtkENxCHFqkRaq8E/ZT4SLPlxfwEENuD/v7//"
    "//5jZGBgYgQ79D8jxNEg6jdQ4DfQwN9MEIHfQAFePkEGLg6wfgaAAGJhgANGBkagAYxARWzM"
    "QB5Inhmq6Q8DOExArgKGCjiMQM4HAYAAAhsA1AeWYGRigggyQ2xnYIIYBDL0N1DzH6ij/zMy"
    "MTAyQ6wFCCC4C5iAqtiYQab+BfOZgQo4gbr/ATX9AXkFGB3//4G89xNoLicDE1QfQABBAhHI"
    "+/qbkeHYA2aGP///AG0DRiNQ59+/f4H0f4a/IC8AIQvzX4Y/P/4zfPrDycACjV+AAAIbwM7y"
    "n+HFdwaGM1dZGf58+wV285+/QMOAgfD3LyQF/ABa8O4TMwOHADMDOz8wRlggBgAEENgAZqAf"
    "vwFjkPPnN4YMy68M4uJMDF+/MTL8/AUJCxYORobbD34z5EziYBAz5mVg5mdmgAYXA0AAQQIR"
    "6N+fQGeysvxlkBL5xzB37V8GB1MmBnN9NobvQJdxcjIyANMYg6wUMAZYGMGxwgaNaoAAYoHF"
    "919QNAHj+MXr7wzTVnEycLH/YtBQ+gd0NgMDNzsjw8dPvxjERHkYXgEt+wo0gB1qAEAAQbwA"
    "NPUX0At/gcnu08fvDBoKfxlYgQH2+hULw+fvjAzfWf8zfP3yl0FUiJHh1S9GsKsEmSAmAAQQ"
    "xAWMjMxsfAwM39mYGJiZ/jC0ZYPji+HLd5DrgNEHdB4zMNCUVbkY7j5hZPgLDBsWVkhMAgQQ"
    "2IAL2y8dNuXjsPz/8fP/fa+eM/BwswJdBIxGaCJiAkbZV6DN74AWcHxhZpD7x8h4b++lqwwN"
    "ygwAAcQIS5LkAoAAAwBDND5TZ7iMJQAAAABJRU5ErkJggg==")
index.append('save16x16')
catalog['save16x16'] = save16x16

#----------------------------------------------------------------------
calculator_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOiSURBVHjaYvz//z9DbGziIidn"
    "h3ARQRFGpr+s/xn+/2P4z8DAAJRiYARioAADI5jPyPCH8S/jj78/GffvO/Bo8eL5/gABxAIU"
    "ZzAzMwlLTIhnu3LrKsO5R+cYWFlYGf7+BWoDGQSk/4JoIP758yeDDK8UQ4SDP4OwkIDS2rUr"
    "ywACCGwANzcPE4hee3gtwyrmXgZ+AQGGLz8YGL7/+c/w/TcDBP9lZPj++SOD3VsjBm9zTwZO"
    "Lg4GJiYmIYAAAhvw9+9fEMUgISDKIPNNnYHjDy+DEPM/hh9/gBqBzv/L9p/h23cGhvecPxnk"
    "pDQZ/gP99ec3WM9fgAACG8DIyAR04l8GfRkbBvmP+gwSYnwMjx6/ZxAQZWPg4mJlePr0I4OU"
    "liDDn9cvGH6/ecXw6ddnhn8MEAAQQGADWFiYGD58+MDw9AkfAxurAAMz01eGH980GH5zfmX4"
    "/fcLw48fugw/WX8yCN/ay8Bw7BTDKzUVBkZOLgYmoMUAAQT1wn8GDg4OBmWln0AH/2Pg5eNk"
    "UFD8DfQnIwMbGxcwJn4w8PEyMjDr6TD8FeRjEJCRYXh67zHI6QwAAcQCcQgjMECYGZhZXgLF"
    "GIGGiQPpB8DY4GPgYOdiYGB6wMDFLMHwSVad4Qm3NIMYNzcDw58/YJ0AAQQ2gJmZBeiFjwxf"
    "v/4CR9fXr4+BzgOJfWZ4/eY9Ayc7O8PbVy8Ybn4SZnj8RYBBVuAT0HAGsGUAAQSOvn///gMD"
    "ixNoEBMwPFgYxMREGX79/gX2loiwCMNXYBTw8IM0MjLIcb5lEORlY/jzlxHsAoAAgrqAkeHb"
    "t2/g6Pz37x/D+/fvQXEMTjg/fvwAhgMrw9fPQNd84Wd49YuX4dPXH0AXgnQyMQAEENgAUBIF"
    "2cwOdCooaYNsBhkEEoMZxMHJxsD/lxmYNv4AY4oTksyBhgAEENgAUACCbP/69SvYWWxsbAyf"
    "gTZycnIysLKyMnz6BPQzMy/Dz19MDO8+A5P1PxZw2gEFPkAAgQ1gZWVjZAWGiqioCNjmv3//"
    "MUhLSwED+g/D79+/GWRlZRj+/voGNJmPgUeEH5hwvgJdywq0mJEFIIDABpw8efKphoay/M+f"
    "P4Ca/gK9wMnw5s0HsO3MzMxA9ksGbmAg8/55wcD6+wnD57d8DIePngO67MMzgABiBPmZl1fQ"
    "XEJCsoaNjV0QGI1/QLECihGQHCh8QOEAchkT0NOgAP/7n5H1+bOnj96/f9kMEGAA2EdRiOuD"
    "HvwAAAAASUVORK5CYII=")
index.append('calculator_icon16x16')
catalog['calculator_icon16x16'] = calculator_icon16x16

#----------------------------------------------------------------------
edit16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJwSURBVHjaYvz//z+DWmjHngBv"
    "G3smBoZ/DAyMDH//Aik0AFTGwMzMyPAfKM/AyMi0be/JM1cWF1oCBBADyICCGfv//YeBf3//"
    "4wYIZfXLT4MoBoAAYgKZ/u//v98g+sqtJwzTVuzHsBnmoMWbjzMcPHMLzP73798fEA0QQGAD"
    "2NnYGB6+eMPQPGsTw7cfv+Ca/4E0A+k//yH83//+MbTP3cZw4c5TBg52VrAYQACBvZA/Y//P"
    "f0Cn33n88v+3Hz/hzvwD9M0vIP7+G4I/Aombj179//Tz9/+cuSd/g/QCBBAL1DmgcGFQlhGD"
    "GAoWg7gAhP9DXfOfkYVBXEKUgRlo+csvEGcBBBALqoehTofh/wj6J9DHP/5A2DzMDAxMUC0A"
    "AcSCHl1/YZr+IcIApPnnbwgbJPnnH9wuBoAAQjEA2WaQAlDo/wDq+vUX1dB/SAYABBDYAEZG"
    "RJTBFP3+B7H5NzQc/sBc9A8iBjMAIIDABvyBxjPIoD/IGpE0/YUaxMQEdSFUD0AAgQ14+ZUR"
    "qOE/w8v3Xxg4OLiBCpggGv9D0sBfqGF/gIKfvn5l4GPgYvjPBHE2QACBA5OZmYXh7pO3DPnN"
    "SxgWrt/PwMMFdMFfJI1ANjsHA8P2g6cZyjvmMdy8+5yBjRmSkAACCGzAj1+/GTRkRRgKY1wZ"
    "fgCDGxYWIGf+hzoXhH98+8EQH2TPYG0izfDjNzj1MwAEECQQmSHGmRipMChpyjKApLh4EH5n"
    "hoaFj6clAwsLC8NPiB6wXoAAAhMnjp86Hj+b34yJEWjPf0aGX0D3/0dKVLCUyczCDHEdIwPT"
    "yaPHrjHkGjMABBgACVaMb7rUskwAAAAASUVORK5CYII=")
index.append('edit16x16')
catalog['edit16x16'] = edit16x16

#----------------------------------------------------------------------
led_red_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAN2SURBVHjaYsxhYGD4B8TMQMwO"
    "RN8ZGOx+MTCkcgoL23GLivIxMDIyfnr9+vO7N29O/GFgmA1UtoeVgeE7UC3DfyAGCCAWBigA"
    "GiLzl4GhS11DI9I4KopB2dOTgVdGBqSf4fOzZxw39+zxPbR4se/Fy5fXAdWVMjIw3AMZABBA"
    "DCAXZDAwyBUwMBzf6eX1//vhw///f//+HwP8/Pn/x4kT/zcHBf2PZWC4GsHAoBwH1AsQQMwm"
    "DAwcQBMneOnqenlkZTGwiIgwMPz4AXQS0E3/gXb8Anro40cGhpcvGVh+/mRQA7qK/fFj0QtP"
    "nigBXbEFIIBYgP6ykWNgiDECOeftW2AoAH335w9E89evDEA/QAx8/x5sCMObNwwWQMNPMTD4"
    "3GRg8AAIIGYzZuYuGw4OLQmgZjZBQYgL/gLdBLSN4ds3BoZPnyCaX79mYHj3juHnuXMMX/bs"
    "YWBmZma4zsgoChBALPwcHDYqrKwMTB8+MHzesIGBGWgrqxHQPWJiDAxsbJAQ/v0bbPOf8+cZ"
    "Pu/axfAfaLAKLy+DMAuLIUAAsfAxM/PxAm0EKWUC2vYHaAjT6dMMzIqKDAwg14C8AHTBv7t3"
    "GX4/ecLAAvQeUISBEagHqJcdIIBYWIAK2IH+5YDGKRtQgvHRIwaGFy9QXMAEDEyQOiYg9w/I"
    "AJA4ExMjQACxAMP4y192dk7Oz5/BtjFycTEw8PAA4wZoJDMzNJH8Awck05cvDGxA57MB+T+A"
    "8l///fsFEEAsH378OPGYi8tXAuRUkCag38CYkxMSIyAAikqQa5iYGBhBhn3/zvAAKPfyw4fL"
    "AAHE9P3Xr1lHgf76xc/PwAAMTLBGbm4GBhBfQICBARgzYDZIDCQHNOgvHx/DQaBXf/74MQMg"
    "gJhY///ff+7jx7V7hYQgGkCGgGwGaYAZAPMSCwvYdQeFhRlOf/q0j+3//80AAcRsDAyin3//"
    "nrn865crh4SEqAJQMTPIAFBYAG0CawKlCaAr/wLDZD8QL3727D4wjcQAPfUEIICYtYFeBNr5"
    "/sfv3zsufPyo9ICFRZ0VqJEHaDsL0M//gCH/GYivAv29Ehgzmx8+3PX727dYoPE3QJkJIIAY"
    "E6FR8h0YiH/+/we6m8GTjYMjS1xQ0IifjY0DJPfh16+frz58uPLj+/fpQO5GVkbGzxxAQ0EG"
    "AAQYALoATpZS+Ly6AAAAAElFTkSuQmCC")
index.append('led_red_icon16x16')
catalog['led_red_icon16x16'] = led_red_icon16x16

#----------------------------------------------------------------------
folder_blue_open_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAHpSURBVHjaYmRgYGBicNv6lwEd"
    "7PLWBpK3gfg3Ax4AEEAMQM3/sQGQOFDeCYi5gJgFiJmBmBFdO0AAgSQYzj3HNPf/Ti8GRvet"
    "ewm5CiCAwAZ8/AHh/AXaycwIoffcgxiCDIAGXgUa4gxkngDi7yB7AAIIbMCrLwwMP/4wMHCw"
    "QBTOPc7AsHvtNlxehrjq1fEihgstMwECCKzlxSeE/MVnEM1vNngxCHPiDjpGd4Y+IHUNIIDA"
    "Brz8gpA4eQ9CH7wP9Cgwbn7/g9JAvHn2Mwa3BClkc/QAAgjihc8Q3s0XQCP3bWNY0OHF8P47"
    "VCPQaz+B9B8g/v6LgeHNVwaGpp5toADdANTyGCCAwAa8gbrg6mMI/e4rwvZffyA0yIDPP4Fy"
    "3+C2PwLiQwABBDbgNdAFj98yMLy/sI2hrsSL4TXQgH8gTUD8E2jAv/8Q9l8gnjINbvtxkOMB"
    "AghswDWgzR8uQkId5ESQBhAGueIP1HYQ/f03iu2HgfgPQACBDQBpFjfxYgg2BbriG8IAkEaY"
    "7SB87esFmO2gdMAN0gsQQKCkqQWM26sMxIJd3v1AsheIn4ESEkAAgQxgBWJDIAalMCEC2kHB"
    "fQCIj4KcDxIACCBY5gDR7FDD8AFQrv0F0wwCAAHECMp4lACAAAMApK3hVGhigEMAAAAASUVO"
    "RK5CYII=")
index.append('folder_blue_open_icon16x16')
catalog['folder_blue_open_icon16x16'] = folder_blue_open_icon16x16

#----------------------------------------------------------------------
restore16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJZSURBVHjaYkxNTX2qoaYs+uvX"
    "j39///xl+Pv3H8Pff0AMYv/7y/APyP4DZP8Div8G0n/+/Gb69u0Lw87d2ze+evUxFCCAWLS0"
    "dCWVNXwYnz77zMDE9J/hP1DD////UOl//xmABNDwvww8PBwM9x4+ZWDc+9CNgeGsLEAAsdx9"
    "/OP/td8CjP/ZBcE2/Qa54M9/hl9A/Aeo4c9fkAv+A23/B3bd779MDIw/+BnYDXr4mA7k1QME"
    "EMvbD/8YVC35GARVmBk+fGYAamRg+AHEP39D6F8wNgj/BTrk/Q8Gb5HnDOvPqTM8Pi5kAhBA"
    "LIyM/xm+f/7OIPb6KwPfL5AtjAxAC4E0EP9jYABaDKaBFjP8ZQTKcf9g+Pz+EwPTb5Ci378B"
    "AojlF1AF89/fDP7azAycLMxgjQyMYAQHjEgcZiYehl8sIgz3p71jOP7jz3+AAGL58/sPUAEz"
    "g5iEIAMbEwPRgJX1EwMDMHABAojpHzCq/gAZv/8Srxmk9M9fSMwABBATKH5BBpAK/oGj9j8D"
    "QAAxgaPpLyjOidf8H2wAhA0QQEygFAdyzn/SnQAiGAACiImJiRFoOyNJLgC7Auzt/wwAAcT0"
    "/u3L/79//2TgYCNeMwsIM0IMAAgglnPnrh9Q8vzv/PoXAwMbJFzA3oHRIOIflA93PTBdfP3H"
    "BUogzAABxPLnj+qUi2efOaY0/Wb6DUyv/4DhAcmRwHABBe4/BnCOhLAhUcfAxMJw985boAE/"
    "/gIEECMDRygfw99bTQx/WYyAkr+BVsHcwACxmwGJDRMHJVRQ4n6yGiDAAOAmVPNfysXbAAAA"
    "AElFTkSuQmCC")
index.append('restore16x16')
catalog['restore16x16'] = restore16x16

#----------------------------------------------------------------------
fileopen22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOiSURBVHjaYvz//z8DLQBAADEx"
    "0AgABBALKYpDQv3PKilpSv/+9fMvuhwjELKwszF3d3YeBIZCOEAAMYKCgtFti5+2Au9GXAZe"
    "PX46ieFK6dqJkyZ9zMvNZfgDFPuPYTADA8iVqqpyD2/ffqQAEEAQFzMybbwyyx6roXN3v2ZI"
    "ufR+HoNKscjXn38YPgHd+vEDUOIfpt8lBRkYfv/99xPEBQggeFDc+wShX71HqP0FNCTeVZRB"
    "jMeCwa+QoevT5wcMd18ANX9nYGBmBroa6FsmJkYGUPz/BnrjB1DP398QvQABBI+8j98h+Nsv"
    "BoY/QNf8/Q/x3qmHDAwaKmIMH3a6M/wCir98x8Dw5DUDw7M3DAzP3zIyPAWyQRjkoHdAx/2B"
    "hhFAAMFd/OYrhN577hnDjgdSWBIPJ4ORuBfDUaBF339ALEUG9goMDIJciCACCCC4waeBGj69"
    "fcbQueIVw//1UljD+/R5BgZ5NaCXv0Ps+vXjP8PP738ZuPlZGCZvfPTVUEOOGxRhILUAAQQ3"
    "+MpzoEug4XsMaAkz0Ekgr/+F4t9A/OEtUBIYcV+/AbUDdX54/Zfh3fOfDNLqLAyc4nLcfJxA"
    "B/8De48BIIDgBv/6A4pRaHgDvcoClAalVlB4//kLicjffxnBPoUliJ+//jN8+fqP4TcwXJmA"
    "mIcZnAzBoQQQQHCDPwMN+/kDwv72G2Lwb6jBv6EGs7NwgE3lZAMqAhrCDXQhPw8jAzc70FAO"
    "iInMTCzsIDMAAghu8Bdg6vv1C8L+9B2S4kFB8AcaDH+APvrJLMtwbv8dhjdP7oDT7ZtHPxle"
    "3f/OoGQuwHDoGgNDf1z22xcvHuwHmQEQQCgu/v0TERQgg/8ghS8sOJiEVRiEBFQY/rMCw5rp"
    "GwPz+48M/FqSDN8/Ah304l4tUOcukBkAAYTi4r9QF78FRQ7UxaD0DHb1H4jBMD7DX0iaB8XD"
    "19+YWRwggOAGfwNq/PcHGhS/IOH1D9ngv4iMA3I9SB4WuSD6PzOqwQABBDf4+5e/DP+/AlX8"
    "eM3w6C1EDGTwv/8Q+g+UDwseJmAUffrKzPDtJzPDa2AS/PPh1T/ksgkggCAGf7yV9Omu1Dww"
    "m0OUYcOB74TLUFZgsnkJVPfkJ8P9o08YGC60rAWFKCjlgqQBAghSbDIy8gHZGkCsDsQ8ZJbt"
    "n4H4FhDfAJr5CSCAYAYzQQ3kB2I2Mg0GufQjyNVAM/8BBBAjreo8gACiWZ0HEGAAanGSGnXl"
    "Z9oAAAAASUVORK5CYII=")
index.append('fileopen22x22')
catalog['fileopen22x22'] = fileopen22x22

#----------------------------------------------------------------------
key_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAHMSURBVHjaYvz//z8DJQAggJgY"
    "KAQAAUSxAQABRLEBAAFEsQEAAcQCIgIZWbR4BXk9Nczk/RT1pHWe3X1++/6VFyee3nrRzMvw"
    "/y0zDs3zgREAEEBgA/wjtLaGtLop8MhKMDCw8jEw/Ptp/vr4TvNSmx13fjD82wkzABhf/yDU"
    "/49sDAxvQWIAAQQ2QF7+p/Drq6d/MzAIs/AosTAyMLExCGu8ZShdrjL550+mP4z//jP8/8PI"
    "8O8PSDPj//snP7/aueplGlDrdoAAYgClg83p4r+z2BiX7m1Sf/T/P8//f3+4/3+9yf3/7z2e"
    "//9fAPET7v//nwPxQyh9kev/DDuuFyC9AAEEdoGg1L/fJUt5QkVkXjAzvPzCwPiHgYGLnwHs"
    "4LdnGRjun2D4y8nFwPziDsPbP78Yfvz/xcBx4ghDZDpQCUAAgQ348/MXp6IRUNfP7wwML4EC"
    "rECN1xgYNs9juHh6L8Pitz8ZdrIzMPD/ZWB4DsTvgGEiBDT7HkgvQAAxgpyRw8+cIa/O7Khh"
    "/M+Si/sv77N7DA9un2O4dP0hQzYnA8MXUGIHBSQjUgz8AuKlQL0AAcRIaV4ACCCKExJAAFFs"
    "AEAAUWwAQIABAP22rxeX/gvnAAAAAElFTkSuQmCC")
index.append('key_icon16x16')
catalog['key_icon16x16'] = key_icon16x16

#----------------------------------------------------------------------
paperclip_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAUaSURBVHjaYvz//z8DLQBAADEx"
    "0AgABBALiGBk7ASSv4H4H1SYEYj/Q2hWLqDUTwZ2fr61PMK8Mm/v3XVl4uDJUTJSzbh36e6y"
    "f1/eVDBwAdWx/QWqBernZmX4f7+dASCAWAha/fs3A4cA7/L03oCgbwwsDLPzV13xznKQconU"
    "Y96/7mbZxv613P9/vstlYEP1PEAAEQiK3wys/NyrU3sCIwSUpBh2r77CEFzsIavrqse8aP5N"
    "BjkjdcaA0vBsBh7BRQy/fqHoBAggJnyGsvMKr8zoCwgR1xRnmNG4h0FLS4RB10WNYfmk/T/O"
    "Tlo6edXUXZ+lDZQZfXPCYpm4hBYw/EEYDhBAOAz+xcAlKrE5fVJomIiKGMPkqn0MllbSDCb+"
    "BgwL2vf+uL95bxwDL0PeixNHXZf0bPoipKbK4J4eG8/EKbwMZjhAADFhM5RTTHJH1uQwH0EF"
    "EYbJlXsYbBxkGXQ8tBnmt+769mDb3jAGnl+rGViBkcXL6/v16b2va6av/8oppchgFxUTycIt"
    "uhpkCkAAoRn8m4lLXHpP5oQQd15JXoYZdXsZbJ3kGbSdNRkWAw19vGtvFAPXn80MzH+Aatn7"
    "3KK8qjwzE8W/3jz7+dCGA/9UTZQZxLWdgkAmAQQQksG/2HilZPblTgt15hbjArp0H4ONiyKD"
    "rpsOw7zGHV8e7twXzMDxbyPYUGb+Ke4poQUq5lqMZ3Yd/Sssryjk5a3NdP7MG4bnD85fApkG"
    "EEDQ5PaHQ0Bebl/WlDBLBlZGhmnVBxlcA9UZVCxUGGbXbvn4/OBhL6ChxxjYgIayCs/2SA9P"
    "kdNVZZjfu+4f1++P//3To9mu3vnMcGbL4hMMf+56g0wECCCwwdziYutSugMtmTlYgIbuZXAN"
    "1mJQtVRkmAc29JAXAzvDMQam3yBDF7mlh8cqGqkwLOpe+5/z3zeG4NwElrOXXzKc3Tj/IAPD"
    "YwcGFnawUwECCGywqrWqpYiKMMP0yr0MLsHaDMpmcgxzKje/e370uD8DG+MxBmZgTHOLLndP"
    "j46Q0ZJlWNK5lkEImCHdE2OYjpx4wHBtz6qtQEN9GJjZoLmWgQEggMAGs3CwMj17/ItBRkOS"
    "QclYkWFu1ca3L44ft2LgYLnFwPwN5KW1HtnRQVIqsgxLe1YySIlyMDjH+DEe2HeD4fb+5YsZ"
    "mH/GMTBzAk36C48xgACClBUsjEz//v5j4BTgYXh0/z3DiwvXVzBwct5iYP/LwMgqvM41MzJQ"
    "QkWCYVnfKgZJST4G52hPhv3bLv+/c3j5bAamz+kMTALQsgYBAAIIbDATKzPzH6DBf//8Y/j9"
    "C1j4cLAZMfz/l8vIIxTonh7kKC4vyrCydwODgpIog5W/I8PO9Wf+Pjq9upmB5WsjAyMH1iwG"
    "EEAQFzMzMf/59Y/h188/DHLy/Ax+JWGWf3//suATE2b8B4yMlf2bGDT1ZRkMXC0Ytq88/Pv5"
    "hQ0lDGw/JwFjE2eBABBA0KBgAqZnJoZb5x8xsLExAn3AxvDnHwfju/sfGS4cvsigYajAoOdo"
    "yrBt2d5fL69sy2Ng/zkTGDOQkhUHAAggsMF//jEzy8hzMTj6aDB8//GX4f9/JmDJzMjAzsLM"
    "4B5iASyMRBm2LNj6/e2tfTEMXH/XMfwHupRAzQMQQGCDb5+5fncnJ6sSExPT39+//zH8+fP/"
    "/x9geP/99x+EGR/d2vv628MzCQzsrLsZGP8TNBQEAAKIkVZ1HkAA0azOAwgwABYtsChYo0qc"
    "AAAAAElFTkSuQmCC")
index.append('paperclip_icon22x22')
catalog['paperclip_icon22x22'] = paperclip_icon22x22

#----------------------------------------------------------------------
undo32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAXaSURBVHjaYvz//z8DIyMjA7HA"
    "ytiYn/379w1M//4pfOfmDjh29uxFBgoAQAAxkaIYaLm94OvXD6K5uR3s37xR+P7jRxoDhQAg"
    "gIh2gL2OTr30gwcHChMSBJKXLmXgYmdnAIYeC6UOAAggFmKCnOPbtwV6X74EFC1YwCDr6cnA"
    "cPcuA7UAQACxELBcn//t2w1e8vIKSVOmMHDJyTEwfP3KwPD9O9UcABBALHgsz5d4/HhCQkAA"
    "g29VFQMDGxvEcmCihTlA4f37NAklJbVPgoIbgNwFwAT5kVQHAAQQI3ouAAU5y69fEzRfvUoo"
    "bG5mUHd2hliKBT++do3h1smTDKf27mU49ewZwzsRkQV/2NgagA55SKwDAAIIxQGgIOf98GGB"
    "Ew+PQU5fHwOXmBjCwn//8LLf3LjBsBGYRrbevPnhpYwMyBETiXEAQADBHQC0PF78yZMJic7O"
    "Ar55eQwMTEwIi5BpfGwgvrl/P8OMZcsYLgkIbPjBxZVAKFoAAgjsAHs9vflqL18mZGdkMBi4"
    "uUEM/fsXpyWE8LeXLxm6OjoYjgoKXvjGw+OAzxEAAQR2gJ+s7P+KhAQGKwsLiMUcHKgOAGFk"
    "Pi45ZPGfPxmmTJzIsJGT88KemzdNgHb9xeYAgAACF0Qf+fiK5wDj79jp0xDL799nYHjxAijx"
    "EZLyv3xB0DD8+TMDw6dPCAxSC6Oh+pKCghisX740sNHRWYQrBAACCJ4GTPX1wyRev54TZGTE"
    "m1BRwcBw9SoDw5s3EFXMzJA0AUqsoOiAAWSfA30MxiA2yIE/foAd8eb1a4bKK1cYzsvIRJ29"
    "fHk5ugMAAgglF5jo6Snz//ix3oqNTTevrY1B5Ns3BoZbtxgY/vyBYJDhMDYuMWQ+lH3syROG"
    "1i9fPt9nY1O5fufOK2QHAAQQI5bakNlKS2ua+qdPaXkFBQwGhoYMDKCoAQUvsuGxsQgdQF8y"
    "AC0BO/bBA6yO6bp4kWGDqGjv8Rs3SpAtAwggRlzVsZm+fprUixc9EU5OvOGgbHnmDMQSqIG9"
    "a9YwnAQGMz+whNTX1GTQNDBgsPT1ZeACRcmmTQwM9+6hOODm27cMpa9ff77w/7/u4+fP4QUV"
    "QADhrA1PXbw465mEhOPckyevdeXnM3zT0QHGETAxs7LC1dyTktp8lJs7as7z512tmzffTAgJ"
    "YVg5Zw7DN2DxzRAeDknQUKDOz89g8P07r7CgYDCyPQABhLc6Bjri7FdBQauNX7+uzI+KYrj5"
    "7h0Dg4cHA4OgIDQN/n0OjNPlF69fL99/+7bhWW5u82nnz58EqwXlmsREFEd4iIgwiH/5AmpD"
    "cMLEAAIIVKcTVWYDE2hmkJjY/01paf//nznzv0dX97+BltZMLEqZQWqDRUQ+Ha2v//9/06b/"
    "/729//+3s/v/1dz8f6iw8H8lOTkzmGKAACK6QXLm0qXpD8XFzaft2vV4SlkZwzdQlsMO/oLU"
    "PpCUdOmcM+cDsBRkYAgNBUtwsbAwGALTGx8Pjz9MMUAAkdQkA+bjUy+4uAzXPHmyct/fv2ff"
    "f/x4AJ/aNxISAROA7YjHoHRjZwcW1wFGicCPH1ZAJhuIDxBAREcBejADMbCqZOAnokVVX6Cs"
    "/P//4cP//7u4/L+hpfXfS0bmKVBKGCQPEEBMZDZkQOU6qEAh2AABRkHjdQaGC5sXLwY2LO0Z"
    "ZLm4GDh+/ZISExYGeYABIICYGOgAPgsIFKzduZPhm7ExAxcPD4PQr18MwOxoAJIDCCC6OAAY"
    "CgefsbMf2LsB2HLT02NQAtYtzMzMyiA5gACiiwNA4Csv74Q9B4Bp1tGRgR9YQrIwM8uCxAEC"
    "iG4OAIbCxrs/fny4AGzSywoLw4TZAAKIbg4ApwV+/g0XQKEArDfY/v7lATkAIIDo6gBgi3kD"
    "sNgGZkBhBr4fP9RBYgABRFcHgKLhAbCN8RbYhIcBgACiqwNA4Asf34XTx48zfGdlBbsCIIBY"
    "6O0AYCs54MC/f75PX7y4AOT+AggwALLu/e/RHM9cAAAAAElFTkSuQmCC")
index.append('undo32x32')
catalog['undo32x32'] = undo32x32

#----------------------------------------------------------------------
minpriority16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAO5SURBVHjaAEEAvv8B////ABFT"
    "xNMoHw8m8fwD/evyAPXs8QF5AAcLnAAJDQAABQcAAP0BAAADAVYCBgSS/wIDEQwIAQH4+wDg"
    "+o8AJgKIhQEI2FnYeAvC5NjUdPkZNpz+yTlv7auZT178FwFKdTD8Z2BgZGTIlBJh7vv6j4WD"
    "j4eBgYeTgeHuUwY+kF6AAGICES8//A0snfT40rkznxhs1NkZ3JykmCRkeNoZ/v1vYmFmqNBT"
    "YJ3Czc3KwQd0ECcbA8O9pwx3fv5iCAbpBQggRpAXGL1OAJ3JrsIlwLeyMlrcSF6Fh2Hu4X8M"
    "p67+ZGBjZfkvKsjK+O0f0DdA656+Zrj25zdDFAMzw8X/kxkYAAII7AKG/3+ARrPf+faf2blx"
    "5ZeDV67/YAg0ZGIQFeVk+PiLlfEzUBqk8MlLhtN//jJ4MLAwXGT49wesFSCAIAZ8e8vA8PYx"
    "0KBfH/5wcPt1rf+1/sqdvwxJ1gwMKtIMDP/+gm3e9ZeFwYfh/7/HDF+A6t8/BmsFCCCIAf9+"
    "A42/DPTcGQaGx+c/MXBwRc7Zy7TgygMGBilgUL1++X/NfyaGIIY3j14xvLkDtOwhxNVAABBA"
    "4FgAhjMDAwsHA8PH1wwMvz4BvcP1k0HeJH3NWYYHwEDkYOBkbGJ4c+87w8fnQL+wAqOUC6IH"
    "CAACCGwACzNjFisvu83vP////WFhfcLw4vokBg7uZwwCco1//gAVfnvDwPDuiRADC3sBAzOr"
    "EhMLM9N/JsaTQK0TAQIIHAsSHiv+F2cZMvxl42Q4cvo9w+6dt0/9+sccw8DLd5uBGRhvvxnk"
    "uPjY5v8TlHFiBcalGDA6nz75xPB9rgwjQACBw+Dds/e1F8++ZPB2lWGITdJncAywMGPn4tjG"
    "8J9JHehmKREhtm1KmspOfHLSDPJyfAzfv/1m+PULmMiAACCAwF74zcjUsnTF7e8PX/3pKa+0"
    "YHBwlmF49oVd5crpWzt4OFh+KeuqqT1mEGTgZ/3D8OzOG4Z3b380M7Cw1oP0AgQQJBCZgf7k"
    "Yus9sv/F5y9/zk6sKDHkcLITZXj0iV3h+w8Ghjt/+Bj42X4xvHrw5vfHd7/KgClqAjQMGQAC"
    "CBKN0IgASsy6cP5jYkPblbe6Qp8ZPG35GFj5+Bi4GX8wvLz36svH97+ygOomMCABgABiYkAH"
    "rEwrbtz8EdbQc++RqdhnBmf1Hwwvbr58/fXT7xhgcpyDrhwggFgYsAEOxn1PXv8Nbpv7Yik7"
    "Jzv7rz//44HePIhNKUCAAQCvzEM7EINWiAAAAABJRU5ErkJggg==")
index.append('minpriority16x16')
catalog['minpriority16x16'] = minpriority16x16

#----------------------------------------------------------------------
taskcoach64x64 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAYAAACqaXHeAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAACdKSURBVHjaYvz//z/DSAYAAcTE"
    "MMIBQACN+AAACKARHwAAAcQCYzAyMmKEjCobVA6ILbggNAg7czMw9HmlMfz5/4PhN+M3hp+M"
    "nxl+/fvG8O3nF+O/LF/DRER47Pj4uCR+/f/+5/XnZ09ePnt+gOE952qG30LXGA6/YmDg4WL4"
    "L/2RgYGVgeHPX5DdQPsYCTu2+AkDQ/9L6ngcVvYBBBALuoQ80FEJ/AwMehwMDApsCE8LMEMc"
    "CWIDmQx9SHr+/mLUZWT/W2Fho+LjaG7JpyarwsDNycPw9/9fho9fP6rcenLTYdPOnbknd99Y"
    "wPCfoQGo5Qs5ju6TZWC4/YOBYctH6qUAgABihIUEKAX48DAwNIpAPPuXAYJhAcDAiBoAIQ6Q"
    "FPDr388yXpX39T7+ZlxmmpYMjP9ZGH78BqaMf78Z/v3/BzablZWV4TfDT4Y12zcyrGo5cETk"
    "7peU17KfvwOVCv79yyAKtFqM6T+DGDBwpBj+MUiCrWBmuArEO4HsM8gOfvCTgUH5CjAGSfYp"
    "cvQjUgBAAKGkgG5RBoafQPEv/yCehXmYEUth8Y2FjYHl759JzDoXcjNDkxh4hKUZHn54Ao51"
    "kMfB8D8Eg8RAhlg7mTJ85vhtc6vl1Nmbb37+V//9i+cvyB5gSvvPyc7AxM3NwMgLzF/MwCB+"
    "/oKB4d2PZmA22QnEHUArD4LsBbmNBaiHnRHCJgpYArEHEr8RwQQIIHgAyLFADGRmhMQwkjwv"
    "KAcAMT8w5MSBWBQYeGJl57fJrlDRS7si8ozh/bdPDMw8wgyPvj0C6mfGGghg+O0/g46hHMPD"
    "3Gfc/hNZGQ6GOzOIacsxMHBxg8sFRi5OBgZODkih8OoVw/8jxxkZNu7wYHj8xp2Bk6ELKFwF"
    "dN+/38BIuqnDwOB9m4Hh+g/8sc4Yy8jw3wWo4Q+SOA+CCRBA8ABYIoGiNezvD4aYv38YNIAx"
    "zws0h5+JmYETFFPMoHKBk5Mh5N87hmu//zCcfCXIsPPRNoZ0yXKG5z+egwMA2dPwwIDyGYAO"
    "VjUQYNjofIch88EvhjUJRgxM/4Cu+/MbmOeA8v9AGQ9ISwkzMMYFMTC42zH8n7+KkXHTnvJf"
    "LAyib/4wJIMc2AUsDCcCy4S4BwwML35j8bs4IwNTOtAtSkCzvuPOMAABBA8AaKwz/fvHMOnv"
    "L4ZsHh0lBm4dTQZWESEGNmEhIC3IwCoowMAqxA+MbWCMCfMz+Fx8xrB0yxyGg0LHGNxV7jEI"
    "c4gwnH93joENGFL/YB5nBOJ/oICABMB/ULnAxMTAq/ONYf38vQy7t6szuHvoMfwFhcF/JmBE"
    "MTP8ALJ/fvnC8PPnVwZpAS4GtrIUhv+CPAwsCzYkLXvLsA/ozqUzgJXJhvfAVPsXM9aZTFkY"
    "mGJYGP5zACW/I+Xf/5h5GSCA4AEAyk7ACPD5/5sxWzYngUEqPw2YJLkgMkxA/Psfw59ff4BV"
    "3V+Gn7/+Mfz49o9BUFSKQfavOsOzl2cY+k/0M5Q4lDLcYbvDcOfDfaA7mIEFIlA3ML/8B9PA"
    "pAjCQJkvH98zfP34lYFBlIUhe80bBv1f8gy/gZ4HqWFkZAYHEAswrbMBHfv260eGKRbXGTQi"
    "PBgYD59giPnyomrOW4YNf/8zfEWPeUYxoL5kYBaSB9oDql9/wz3MAvRGGdBJYcAAWQ90RDPU"
    "ywwAAYRSCP7+yRDDo6HIIF1UCgytDwwMnz+AY7F4IwvDsfvAUGUBmsYEpIEOZGAGWsbGycAn"
    "4sog+eQCw2PWcww9u3oYXHRdGKTYZBkefXoJDKw/DH9AMfsHmAKA2eUfsHb4+fUDw9evryF5"
    "8ikvg7OFKEOxuyoDJzsLAwfQ7xysjAyswNIXZBUIz7/8gyFp/zeGQ2G3GFi8HBhsbq7QcuVl"
    "CNnxiWEhstuZzTgYWEM5GP5x/mH4/+MfJEkDU8P/f/91gdXZFEFxYTtNRTmG8xdu6H/n/n4R"
    "KLsBpA8ggFgQDQMGKaBj7QRFgcHGCLQZVNL8/Q4MyW8MGy8JMPDzCzNsyFJhYAXGDAcrEwPn"
    "T6Aj/wFDWkCVoXD6A4bZ194wPP75iGH+y/kMgrwCDP+A9dqPX7+BqeY/w19gXQf0PdgScN36"
    "EViQnBViUGXVZ4h0NmVQE2XDmUcTdTkYtt5SY2g58pqhwdWAgWHdLobCb+/ydn5iWAE07ScT"
    "GzDAgngZmB3YGf59/Q2JdWiV9+/Pv1xgHdyiribHF2+gy3D1wwdgncoEintFmPkAAQQPAGAk"
    "BTCwMYgLsjxmYLh/joFBUZeB4RewdGf7zZBo/IOh7cBXBj5g3SPIDdXyCWj9LiCOY2LoSS1g"
    "+DvrO8PSi8sYvgu+ZHj//hMkhf2H5jtQbANTPMMHYCn/hpeB7YsEg42SEUN6QACDlpYBw7fv"
    "wOwFzH+/gYH+799/SFkIKriB2Y4RmBr6nYUYnBZrMvipfWQw8rZkcLi31ciWh8H30BeGNVyg"
    "1potF8NfUMHxH97KU/j76/8kdmF+X1dDdYY0TXWGPc+fM2w4d4nh17rvhxheMZyC+RsggOAB"
    "AMzeyfxCwCIfVOmd3wYsqq0gIfmXmSFI6ztD3WZgoXXuHUOSrRg4W/wXBxVaQMfe/M/ArMTB"
    "0J1Yx6C9U5th6aG1DPff3Gb4ywRs7LECi3ymXwxsf7gY+H9LMwiyyDAoa6swGKvrMhhq6jCI"
    "S0ozfP3NyfD1zS9wzccCrOCZmcBFAAMbMAlzczKD+TzczAz1TnIM6QfeMBy3N2FgW3eQIe/7"
    "lwJgAKwHufD/j//wUvz/n/8x/5iZe2RUJcXjjbUZwqRkGBY/ecAw98RFhq/LP55m2PavHajs"
    "NMzfAAEEbwluEWL8I6fKwKwHajTwAuvE4t1A04BF7e+PwHz0g0G7V5RBSlyGYWuuGsP9J0CP"
    "AfMqwyWg3mdAh/sCs8NfJgZuDnaG798+M7x4+4Th168vQA8BlQHLDE52TqAcJ9AjPAw8vFwM"
    "7EB1YF/Ci1+MljowJQB9Bizp/vz5z/AbmG242JkZQta/YdDlOsbQfL2f4dOMQ/+dnjP43JRl"
    "3/Y/HZTl/oj/+fGnm0WQPdYc2NbI0NFgMOMSYJj17D7DjKMXGT6tfnPp35of9UCjt4HiG+Zv"
    "gABC1AJMDNc/fWDQATfa3gBbYffPMjCo6wGVfmQAtvMZQnV/MLTu/cjwGpjP1JQ4QU5kYBIG"
    "OnLZfwYWfmBgcDFCPcPLICeniTU//wZ5Bpisv/9gArewvgGbtV9+/gImf4hHQcke5HHkQRpm"
    "aHNUiPcfwzQvIQbL+ToMvtouDGbSJxhzP/zKy/37fxswoLz+/WacIqokohhkqsIQoSDLIMbI"
    "zjD/+UOGuceuMXze8OHGv3U/2oCO3gHyPLKbAAKIBan+nPfuHUPfh7cMDIKgdt+lLQwM2raQ"
    "bPCHiSFA8xtD0+YfDFsuvGPIcJIExvB/hp/swIKOF1jfnwIWhkZAD34HFXhA+u9/cOyBPAQs"
    "hcE0uJEBSubAZu6bb88YNl7fxGAqZ8DgpGINrGF/glMLKLkzAwtZVnhWYASXx8h91EY7SYaM"
    "E64MJ212MVg/OWLPZ861+Q0Hm4+BjjhDvIESg6mAALCpzMiw4vljhjlHbzJ82PL+/t/VXzuA"
    "hS/QQwwY7UaAAEJuB6wCNsbKnz0GFoTA/M1w8wAweoCFGTMwuf76waAv9YNBWeoXw7KT7xiS"
    "bcQZbgN7JaCWIYM00IOn/jFwqbKCqy1QM56DA5glgD4AsUH1OchT4CY2UAxUqFWtqGTY/GYL"
    "A89DQYYpDF0M4aahoMwLTgmgAvAXqM3xF5pigG2OP6BmyG9IKolV52LY/0SfIeG9FcOjwCsc"
    "Pw3EfcIMpRnCleUZRFnZgfp+Mex5/Q7o+bsM73e8f/Zn5acOYI22HloMYwCAAEKuBp8C2yH7"
    "nz5hiNDUB4b19zcMDHdOAlMBMGp/vGdgAmaDcL1vDB37PjE8/vCTQVuNA+hYYDYANuV/3PzD"
    "wP6fmYFJkhGtxwUpzf+BkvhfkMf+M7z/8pnh3JPbDHa22gx/fvxlSN5WwvD6zW8GP60ohs8/"
    "v4CTPyjAmKCBCUoNoEDk5AG2HoFN8N+MPxgk/05h6Py5gUHdWZGhSF+BwU5UBpjSGBl+/v3G"
    "cAKYjGcffcDwZvf71z+Xv+0EtglWg+osXNUsQACxoI2CrH3/gSHiLbCdIioCygZbGRj0nRjA"
    "TTqgB4K1vzC0beUHZoP3DHluksDGzV+G36zAoAWmgg/HfjMweTAx/PryH5wFQDEGrv7/Qwo0"
    "8KAHMPY5gI0nY0EThgt39zMUuGuDq7rSfTUM3GwsDIlOYcBY/wVVizRIAs4HLAwXH19mKNzQ"
    "wHDq12mGAA85hnB1NQYFDnGG77+BdjN9ZTj7/i0wzz9heLXv/Ycfy1/1/P/2bzlQ43t8HUWA"
    "AEIJAKC79wLNevP4IYMIsJXLwHD7EDDsPgKjAtj2//2eQV/yO4O85E+GZcffMqTbizPcvPeT"
    "4T/IoWLA2D3zm4FXn42BjQfYhGUBde6YgY0mSNUGrt6gnmIBJtN82zgGr5V7GW68/MJQYK7L"
    "0PX7EkPRvjoGZpb/DHF24QyfvwBbjMCA+QksZ5iBjZPvwDp6zZUFDH2nJjOwKv5iyLLWY3CX"
    "VAd2EPmA1Siwsc/0g+HSh1cM848/Y3hx4MOX7yue9//7/GcR0AevCfWUAQKIBbXyAYYWE8PO"
    "x08YovWAZSXLd6Dnbx9nYNA1BvbNXzMwc/5jCAV2Ynr2f2Z48PY7gw4wG/wGpm1mYAPp2zVg"
    "4fWViYFdnRmUndEqNQZwAweUDX7++smgK2HEYMZtzrDz/DkGXzlLhiorCYbmv9sZ8nc0Mjx5"
    "/psh3CwS2Mb4ziDAzc1w5/UdhrqtHQzHfxxksLAWYYjQ02fQ5VUFpjJmhq//voI9f/3TC4Z5"
    "x18yPD34+evXlU+m/H3/ax7QyhfEDBUABBC8jP0KdTSwHFj5HujvVy+gwQOqDViEoNngP0Ow"
    "zmdgpv/JsOkcMKyAmRRWnTPqMDJ8PveH4TOwVffu41+G569/Mzx89ovhzsOfDNfv/mS4dgdC"
    "3374g+EVsHiJ0A1jeHnrL8OOR+cZjHjsGIos7BlULbkYes92MRx7sJZBXZmbYd+99QxJq5MZ"
    "bvCdYIgNVGYoNLVi0OU2ZPgBzHffgf11YJeM4ebnpwyLTrxg+HjgHcOPlQ+W/3n9YzrQOU+I"
    "HSgCCCBEUxjamAJm1/3AfsuzBw8ZpKSABRzDnaPAtjswG7ECm4g/3zEYy3xlkJX6ybDixFuG"
    "Qg8phruPfgHb/MD8Dmz7f3n0m4Hn3D8GNmBfnBWYb9mA5QM3NxODILDRxMbKCG/psTD/Z4iQ"
    "dGRYcEqXYe+lWwxuUucZbPiCGH6YfWeY+uccQ/XOiQxrTu9mOPrpLIOSMQtDsrEug4WgHgPT"
    "f1CS/wIs1IF9FGBheO/rY4Ylp54zXDv4n8H8JBuD2dtvv4El3iNSRsoAAogJKf9D6m2gP4DZ"
    "YOujJ6CqB5hxvwGT2S1gIHDJgEY/GVg5fwOzwReGc9e+MFx/+pVBUpSdQVqclUFVi5NB05Cd"
    "QeEDM4OaCgeDggwbg6Q4G4OwIAsDLw8zAzuwJQeqBkEF2i9gXcfFzsUQbhjE8PDKX4b9z08x"
    "fP77jsGMz5MhBVgmiBv9YzjFeJbBxUuAocDGjMFG0AZYoPIAU+lnYEn/A+jWn0DPP2BYfuoF"
    "w5NDfxl82EwYbhjWMFjp6IQA27CyHCQEAEAAwQMA5Plvf+Ciq4G1AcOzF6AWHpB/Ddh6ZBGG"
    "ZANglzFA6xMoMwOzwTsGfj5WBk5OJnAhx2HIyvDzLrA+/wrq/wAbSsCa4Mu3/wxvP/5jePrq"
    "D8O9J78ZrgMLzqu3fzGcvvqVQUfcgUH2tzLDkWvPGe58OwcMmP8MRrx2DDFm6gxJvvIMcRp2"
    "DDKsOgxffv8Cuu0Lw4+/P8EDsQ++32dYffYVw919fxks/uoz5LtHMJSmeTPMNqsRdeFnTWPE"
    "OhKKHQAEEBNSOwCcCqCBcQRYYD24cw8oAPQcw2Ng5+njO2APBVgW/PzNYCr7hUFS4ifD8uPv"
    "gDHzD1QkAOtyYL5n/sdw/+tfhosHvjFcufeb4eodYBnw+DfDc6DcV2CHhRmYBYQEWYFZiI1B"
    "UZ6FwcZQnCHM0Jvh9oXfDKdeX2X48vc10KN/GPT5zRmcJRwYWP8LMXwG9kW+//0K9Px3cNJ/"
    "BPT8+nOvGO7s/cdg+FOTwcskhEFIxYChwACY4hydGR6aJCfzMDCIQdrRrAQDACCA4AHwA9oF"
    "hTZbvzOyMGx++BDY1P0OLBl+A6P07hFg3aYAjNqfDBxcPxmCgIXh1ZtfGS49/MLw6fN/YKH3"
    "l+EnsATl1mFh4Hn0n0FBlg1YkLEzaACxqiI7OEuIi7Iy8PECawpgS5EJaOwPYMETYuvFIPxZ"
    "iuHEjVcMz37eBnr0G8MXYBL6Amx9fvkD8fz3P98Yfv37zvD0x32GLRdfM9wE9tNM/qoxZPhF"
    "MPgH2zOIi7ExfAO2GCe78DDcdsiV5JLVSYT70KsRbwAABBA8AM4BqygmUDb4C68N1nwA1gaP"
    "HgM53KCm8XZgNhABigOTFbCxEqINzCM/fjGsP/2WQUkW4lFZcRYGGQtOBo4vwH78h//g8Wsg"
    "CxyosDLm9x9Ghi/APsObD/+AqeM7w8/v4gwmYvYMdy7+Zbj07g7D938fwcn9KxB/AwbEN6Dn"
    "fwI9/+LnQ4adV94wXN/FzKD/W5EhzjWIwdLeloERWBP9ASbX98CsJgfsPle5yzA8dG7KZmDj"
    "EgUna5NIvAEAEEDwADgOzP/f/0EGZhkh44MngPjWrZvAEGEHNvpfnANmA2D9xSEKrgYt5D4x"
    "SAL7ByuPvwcWB3/A+e3fX2DMCoGaxMwMny/+AsYiA8PLt38Z7j/9A8z7vxgu3wLmf2B1eA+Y"
    "LV6/+wtu4wN7yAzxnr4MrE+FGM7ffc/w+tcTYIx/hwYCJOm/+vWYYc/1tww3drExaH6XYQiy"
    "9mVQN7BneAZs5ly7DcxuwDLl1v1fDFef/GHIMORi8PSwlWWwKawGewTYvGYQlAWWZUJYAwAg"
    "gODV4F2gY54BsSxQ0y9IF/0XsF7c8PD+/7KvX5gZuNmB2eD+YQYGLSuG/5/uMnAACz9fjc8M"
    "sw7xgLOBsjgPw62nv8HV6Vdgjfnl6FcGYSlgcmdjBNYAjMAeJgsDB5BmZ4V0jmBTkaC2v4Gm"
    "GoONvBXD0fM7Ge4oPGZQAFadv0BDaMD08/X/G4YTtz8z3NzJxSACDCRLU1cGGTUHhk/fWYFm"
    "/2MQFmYDmwsaQPkLTAnPX/xgKFb5x3DYMjzri6zWGgZWjiNAxzIwRM1mYLgKTMUHJ0EKPCgA"
    "CCBENQjE1/9BBlGRaoO1Hz79/38PFDrAqozh3k4GRlZRSJsWWCIHab8HZ4N1J98ysLNAeoDC"
    "/CwMalbcDKrCLAxKLMwMysAqUQpYHQrwARu1bEzgLPEHnNKg2QPaUvSx9mT4fYeP4cbjbwwf"
    "fr8C5/t3v58znLzzieHGDl4GqdciDNGu7gwRsT4M2poCDOJC/xh4WP8AE+VnhitXXjNsP/SU"
    "YcnupwzTDrxkOPnkB4OlEjCGJPQmMTw6w8Hw/CqwSf+SgUFKh4GBTwolBQAEEEpT+CLQNXaM"
    "EAdBs8FZoNDl61f+6OlaA2vXt5eA2eAVsGYAGvL9AYO14gcGEfHvDKuBfYPGMHkGRRlgCw1Y"
    "9TECY4RFhY3h49lvDMwqbGADYbPPoHGCHz//M3wD9gRBY4E/foCayD8ZeHnVGHT4DRgeXDoO"
    "7G+8YeABdiiuAbPKze3CDGLP+RkCHB0YXL29GfgE+BiePvrIcP7KW4Zbb38x/ALN2PCyM4gI"
    "cTPIy3AySAuxMyiLsDI83P8WmK9v6DAsS1UBWnuFYXE8UscKAQACCCUAngAd+gCIQUOmvyGp"
    "5C/Q3esf3v2r9+kdIwMfHzAbPAJmA3Vrhv8frzHwCHwFdmM/Mcw7yMVw8tZHBmN1IWDqASZv"
    "YOxy6nIwvFj2nuE/sO7/Dm5PQTo3oAAAjfKAsgWo/SAqysTACawVBIARFuzmwdCy8grDI6NX"
    "wJ7gb4bbOyQZhJ/wMfg5WjG4+/gy8AsJMHwHBhwLsEy68/wnwy8BXoZQJxlw85UDPPwGwb+A"
    "Zc/hxz8Y5D7cZOX889PoJigA4PU96hAcQABhLJC4+g/S3vmG6Bus+/id4e+tG8BMwsMODIBd"
    "kGzADLQRWEgF674FN4o2nHzNwAkUevj0F8OV698Zrr0DNnyAaf3O0S/AUhyoFZiFZKWBVaMq"
    "B4MGsBOlpMDOIAFsQYJaiUzArPPl2z8GCzNjBlVOVYZ7x/kY7u6SZOC7J8Bgb2zCoG/mxfDm"
    "Cy8w1r8yXLkJrBGATRILHWBV//krEP1h+PkHnBPB3W5QQABbxAx33vxm0Hh9hSGOiyEV2zIA"
    "GAAIIIwAuAGaHYYOZjBCGkWXgcxzV88BCyVmYDYAxjzDJ2A24AK2Cb59YrCW/8AgKPaDYc2J"
    "N8D2wC8GTmAUCAuxMigrczDoOfExyABbgXLAho+wMCsDBwcTeGYINMgJqjhA4wWgahU0TvAX"
    "NBAC7Cr7uIYyMB6TYWC5wM+gqaAJbF57MrBwiQHV/GOQBpqjpsLJoAqscg0MBRn0ZbkYjl98"
    "ziAFLOM4gW0eUATwAePo9usfDL+/fGNQY7vLYMPOYKPFyuCOKwAAAggjZF6BJmyAPpZDZANg"
    "ImBY++DeH9N3L/8zCAE9C+x3MjAq2TL8f3+WgV/gE4OXxieGpYc5GM7e/cBgpSfO8PAjyJtA"
    "XcDYfn/oE8NnYPOXU4EDXBaAys9//yCjQ6Ay4Cuw5fgVSP8Clgsgi1TUDRiaa6sYPn94xiCv"
    "pMwgISUOrin+/WOCDpiC9DKAW5XGeqIMt/Y8Zrj3HNisluUGy3MDi5xzT78BI+c9g4P9Rwa+"
    "R0wM/r//5V/7zbAd2xA0QABhXSN0EzpZ+uMvvEW9/stvht/XLwJt5uEE1pd7gc1icUiK+POJ"
    "IUQPkg3WHnvFwMH8DzKkDcSsoKpPiYPh04VvwGrrP8PTF78ZbgDbAZdvAPvwt38wPHsJNBTY"
    "dhAENo+VgOo01TkZ5GWZGTR1FBjMbK2ABaw4MGn/A/cpQDUHaIiIiRlSc3wGlikffzAziLNz"
    "Mmw69AxYC/1n4IK2fE8++snA//8pg6HmVwZ+RyEGCwYGZ0UWBntsfgUIIKwBcBU6goi0TuAW"
    "kH3iyhlgRvsPDOKvt4AueA4MDBVwSNsrv2UQEP/JsPHEa4b3n36CHfL563+GW7eBjR5uZoYL"
    "Z74w3LvxneHrT9DECwuDvBw72LPqqpwMssAmsgCw6gTFKHjA5DeoloAUmKDhNJCHQa3Iz8B8"
    "+eTZb2DD5wc4AO8++MXwGVhuGBqJMkhwMTHsuvSOAdgUYXgP6os8+8WgxX6PQYLjJ4OUuygD"
    "vyATkz8bQz42vwIEENYAAJYxDA+h+QM2pA1kr3ly/w/DyyfA8GcHpoSXwGwArLb+//zKIMj7"
    "gcETWBs8vfOF4fj1DwzCXJB8zgKMMUUjHgZVYPUoD/SMkgo7sCxgAbcH/gLTOygpA/tWkNhl"
    "YAQXhiD8G1iLvAeWHQ+ANcjVGz+BheoPcOEKGncQApYvKsDyRVODExyQohLsDNb6ogwnLr9j"
    "eAP0/KP3Pxm+vPvJYCjwloEL1AiTAdppJwBKBV5SzAzm6H4FCCCcy+RuQCX/ITqWm4BZ4sd1"
    "UCoAZYNXByDZgAXYUfj1jsFf+y04c6458hyYDf4z8PEwMUhIsDFwAxtAnFrcDO/OAfvyPxnA"
    "AQPudYIKP2CsMzJDAuLNh78M9x7+Yrh07TvQ09/B2QWkTkyMlUENmFI01DgZ5IAe5gemFlAg"
    "/YKW/N+BWESCl0FPnJNhy9k3DKcfAdPujy8MBtLAfjyPBNhN8l4iDEK8jKzeWFIBQADhDIA7"
    "0GyApOABMBscuXbyJ8P/P8Bs8OMuMPk/BRa7WkCFbxiclN8w8Ir+Yth68hXQMz+AJfJ/YPUE"
    "GtdnAAfAn4/A/P7qF7AKZQTH5Ku3fxhuAQvHy0APX7v1g+Hlqz/g5C4tBexYqXMxqAOrShlg"
    "tcnDC1meBq7qfkNqjz/QpTwCoKUAfP8Z1EX+M7gZ8TFcuPOBoWfPawYWtm8M2qJARXx6wBT6"
    "l4FXmZNB2EaAweo/Q4AosC+F7E+AAMJZP4JWooE6gmpIc0lAxauePfjt8vzeHwYpdaAFr4HZ"
    "QMIJmBp2M4gKv2Zw1fzEsO4QK8Phy28ZnC1kGd7+YATPDP/iYGZ4z83C8Hz3ewZ2E36Gf8AS"
    "nx1YJfICywcRYKuNE8gGzQz9h06N/YOW9P8YENUxaNKFh/U/Az/Q09wsoEmSPwyP3v5g2Hv/"
    "K8MBYIfo6MOfDE+AgQpayGEh+IhBUQiokRvoelZgG+jvBwZFLz6Gl4fec9p+Ykhe95chD+ZP"
    "gABiwddVBGUDDdTVJ1uBLdfPV0/84JUyAib9d8BWoXwEw39WAQbG368ZQvTfMKzby8+w4egz"
    "YNteBryW4MXrvwwv3v5mYFIEqj/znkHWS5SBi5cJPOYP8zCoxvjzCzqbDi1zQDHMCfIwsMXI"
    "xwasVv//ZXj98SfDsXtfGQ7e+sJw6C6wMHz2B9JiY/rKIM/9giFI4D6DucA9BkuxpwzCoGmr"
    "F9/Aw/mgGTE+HTMGVomvDCIf3mhBF359BtkDEEB4A+A6EDswQEbFoGXBM6CGA9eP/fB1ShYA"
    "1hLAovLHEwYGYGHI8G4Xg7PKSwZeMVmG7SdeAj39HZgNuBh4gGWApjCwtQfsFD2+9p7hD7Cz"
    "81ebB5w1QJ6FLSEAeZidGdKQAcUyO7Dh8/nrL4Ybj74xHAP2Bg/c/Mpw5tEvhq8fQLMt34G9"
    "0+cMVtx3GMykrzGYCt5h0AamQFmh38AmNQcDE6jryyrC8J9TmYFBwoOBQUif4fPZ2wyfnm4C"
    "pWrYCAc4AAACCG8AgMqAB0AMyjQ/EYupVr149Nv36c1fDHKGoCrjAAOjsCswtDcwiIm8YHBQ"
    "/cyw+SgwG1x6zeBho8Dw6itocBUYSUCPsSjzMLy98JGBTZ0HHKKgLiwPGygv/2fgZgU1JX4z"
    "PHr1g2HX3U/AUP7EcPzuT4bnr0ErpoC2M79lUOa8w2AseInBXPAGg6HYWwZVSTYGUTFJBnZh"
    "XWCoqQJjClgts0sD1QowgGZsGH8A/fgFWF3f3MXweNp0hpcff/4+95/hOANkHQkYAAQQC6Ex"
    "s+vQAEDKBjuBut9fOfhdUM4KaNGno8BuZjSweSDGwPjrOUOQ/muGzQf4GDYceswQYC8PjFlG"
    "cIOKCdiAFDATZGAHlmLiwPAHduDAS+JevvvOcOjGF4bD1z4yHLn5meHqU9CcOdDTjF+AAfOM"
    "wZ7/LoOZ/AMGE6mvDNryvAwyMgoMfOLODIwCwNhlA/ZJ/rNABj2+vACWRcCC+f1BBoYPQPoT"
    "MOq+ALvAvz6Dl+c8vfyX4S4jw52Xf8Fe+gzzD0AAEQyA+9CZRU5ENngN1LT75rHvYW5fhBhY"
    "WIEh/OsRA4OgJbAuW8/gpv6cgUtEjmH/6RcMT19/YxDj5gY3qYWABrCxMDN8BHru2u1XDMeA"
    "2eHItQ8M54CF2Nf3fyH5gesHg4bgawYj+TcM5rLAulyJl0FF3ohBWNyfgY1bCLLW5tsriOdu"
    "bwaW1LeBXgFmw69Aj/78BB6vBJeg0LlEBtCKMwEuhnf3fjO8fvKZ4eJ/hpMM4CUdiDUCAAFE"
    "MAC+QbOBHlI2ANUGr57+Dnt05SeDkg3Qsk/7GRhF/IFGr2CQEnrE4KCuybDtCBPDoQsvGcLd"
    "FBku3XrHsOP6W2C2eMNw8uZHhhcvfoLWaABDlRk8CWuh/Y/BDNgHN5H/x6AtLcIgLSLFwANa"
    "4/cX2Lb4Aoyw+0DPfroHdAwwln8C66e/P6AtNCZIpgR5lAO0gpMdIsQInZADxtjnF38Zjq7+"
    "zgDsH/268I/hKGilErL/AAIIZbE0LqANxBFIAQBaOgt0wk3bED4xvzZgMvwGTNMqKxj+P9/E"
    "wPhuMcO8/RoMyfO8GRR1+YHdXXaGS3c+Q7qYwLqOVZCZQV3iD4Ox3A8Gc3lgg0XmG4OS8BcG"
    "Yc6PwIAF9SmAnvwGTFXf34JWXED67/+hHgV7lgk61cyA8CxoHO7HP4ZvwNbjJ2Ct8x5YO7x7"
    "+pfhA5B+c+s3w7sP/3+vZGSYdfIvA2jC9ALyEhmAACKYAkAAmNAY3kAXDEP7Rx+AZdaOW4e/"
    "xf36wMjAxglsPL9byMCoUMXAIOnNECD2hGHCua8Mly8CYwsYk5JiPxn0VD8C8/JrBlO5lwya"
    "wAJMku8jsOcGzLv/v4EXYDB8+w01HJp8QVPLbLwQfzMhzd4Ae47fPgA9+gri0TePgR589Ifh"
    "0/M/DF/e/mP48fkfw3fIAtGvwGB/+4KR4fZxRoYdj/6CC79H6EtkAAKIqBQAAj6gTROoa0y8"
    "gU7ekjhDhkHdE1iqfwZayWkCLN2ANQKHALBfcIlhzdazwFT+jsFA7hODkvh3BkHuP9C5Ckbw"
    "oiSGf9B16IzQ9TOwGAV5FNjg+PHxL8Onl38Y3j4BevTRb4a3D4D4MdCzwJrh2/t/4FWwQPd8"
    "BwbjO2AJ8AJY4z8GlhCP3zIyPPrwn+EZUOz1f0jX5hM0Dt+jb5gACCCiAwBY5jLEQ+sPaFuF"
    "B5glrpsH8ssET5cBFkiQ+pkBtPD5H7TpBkqufxgheRS8bJMRsfaeCeHRX5/+Mnx+CYnNN/eB"
    "nrz3i+EdkP3xBShW/zJ8+weO0V9Aj4Imrp8DffEE6JvHQPoRED8DxvSLf5C8/RlabP2A5thf"
    "UCf/xrVjBCCAiMoCDNBm8Tvwmnl4NvgC1Lzt1sEvaT+ADuWQAvWBuaCh8x+x7h465QwaF/v9"
    "EeLRd8Be3tu7vxheA7vLbx/+BjZQfjN8eQP0KDAvg9puQI9+AHkKaN8zoEcfInkUqBseo+ge"
    "/QVdkknSXgqAACI6BYAAaETBE2ojNEiNgIFxWsuJh8mzVYpBWIEVUvh+AeZRYG/u3WNgTN4B"
    "evQO0KPAjs8HYBL+Akq+v8AeBS1vgHn0ORA/AuKHwKLvKTAAXgJ98xbq0a9Qj/6AWvsLNSFS"
    "tmcIIIBICgBQcgENroG2FshCgxvowhSg+BQJLiZ2US0OBhYORoYvz4DJF1gogYa8vkGKuY9A"
    "j74CxSjUo4+BvgPWzAzPf0I8+hHJoz+RYvUPjpWUVNs0BRBAJAUAMsgAYtA6qltAfADYZdBh"
    "YEgXZ2DQBC0LAgbKe2hsgmMUFMvfIYXQR+iGqe9QDyJ79C89t8vB/A0QQGQHAEh1NXTcYAVE"
    "CLQuQQHa0/oNzaPf0GL0N709SigAAAKIcaTvHQYIoBG/cxQggEZ8AAAE0IgPAIAAAwASEsbR"
    "c+IbhwAAAABJRU5ErkJggg==")
index.append('taskcoach64x64')
catalog['taskcoach64x64'] = taskcoach64x64

#----------------------------------------------------------------------
cut22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAUiSURBVHjaYvz//z8DLQBAADES"
    "q1BMTIxBWFgYzP716xeDoaFhclBQUNnKlSs3XL16tZyFhYWBiYmJ4d27dwwvXrxgAAggogyV"
    "k5Nj0NLSYtDU1GRQVVVlyM/P73z//v3/69ev/wda5o6uHmQJQACxEDJUSkoK7NqvX78y/P37"
    "l8HKyqqos7OzDBSE8+fPv/D27dv9nJycDGxsbAy8vLwMfHx8YDZAAMENZgR6g5ODA8wGGfDz"
    "508GHh4eBmlpabChIAB0nV5VVVU7SO7atWsMmzdvXgoKGZBv/v37x8DMzMzAyMjIALIUIIDA"
    "BguKiDBIA13GAbQJBP4BJR49ecIgKiQEtgQEQIqjoqK6FRUV2e7fv89w9uzZjzdv3lzNAXUM"
    "yGAQAIUzCAAEEIuEjAyDBNBVIBv+//kDl5SUlQW79P+3b2C+jIyMk7+/vxsoYn78+MGwc+fO"
    "bUDDHoJcCzP4D1A/yAFAyxgBAohFCBh+IFcxA73AwMgIdx1I7BvQy+zs7GC+l5dXCSgMX758"
    "yXDv3r1/+/btmy4pKQm2BBgE3ECgBowPU6BFDkDfPAcIIJZ/IEOBYfMLaODvb98Emf7942Pi"
    "4noICitGqGUCAgLq9vb2rh8/fgRbePDgwTv8/Pwczs7OZcCgsVZQUNAHWiIPiuiHDx8yzJkz"
    "JxoggFhAGn98/86jx8BQG6eunizAzs4z/+rVdYf+/En6x8j44/fPnyIm+vp58vLyLKA0+gQY"
    "9kBDFIEpYhcoNTBCfQlyHNClf9ra2iqAwbUMIIBY/v78KRDMx7chQ0vLfsulSy81hYTYJ2pp"
    "RZpv3Xr+Fz//oQR+/hVWTk4KN27fZmAEhuPjx48ZREVFWUHBA4oDUPi/fv2aYc+ePUfXrl3b"
    "/Pnz550giwACiMWAgaGzxc3NvmDx4lNTHjzwCJCRqV2vq1tYoaZWJ8rFxaIoIsLxFZgptm7c"
    "yCAOjA9QBIHCFhrWr06fPn3g5MmTy4DpeTso6cGSL0AAsQTIy8cyCwgwXHnz5jYw4b7/ycHx"
    "6vfv3wzxCgo8ty9dYlgnLPw2RE5O2DM4mOH21asMyxctOg4MkuN37tw5AnTpKaAZT7FlLIAA"
    "Yvn86dNPYPrgzHJwcP9x6VJHqopKPCsXF8O606d/Zj1+XMD+/fs/r0mTZioA0+vagwefb92+"
    "3QOo7xOhHAsQQAzWSkrJxwMC/v/v7Pz/v7r6/4/w8P+loqI3gZFiJSQkpL7Iw+PBz4KC/9MU"
    "FZ/wMjAEEVtoAQQQg7qJCYO5ikogMNoP1ikrXzJkY5vIwMoqLCMhob7Yy+vF/0mT/r+Oifnv"
    "wMGxXUxRkZlYcwECiFELaDA7kHH+zBlw0ABzxB9NOTmVMhWVIwne3uK/bt1iYANmlO/PnzOU"
    "X7y4a/LDhwFAdd8JGQwQQOCMzYQo7P+oq6gweQsKzkzw8xNfv2HDe+Np07pDtm5d+oWd/V+F"
    "kJCbABOTOzEuBgggcCEES+RgS/7+lXDX0LAGFm8Mc06dOnFFQKDsGTDzn3392lcHmCuBpQkT"
    "MQYDBBDLwxs3GNTV1ODF3X9Gxm9v3r//CixV2OstLByl7t1bLsfOLuf4/z9f1v37p7/8+7eH"
    "GIMBAoj5N7CaAeUeYJ5nEAEWn/+YmX88evWKzZqFxUlbSorFj4tLx/b/f9nWa9fuTXj/Phao"
    "5yExBgMEECMsKEC5CVS2snJzMzCzsjKwPHiQZ8vFFc7z7x/vkQ8fTh399q0bqPQmsakCIMAA"
    "2+QIbdVT8iMAAAAASUVORK5CYII=")
index.append('cut22x22')
catalog['cut22x22'] = cut22x22

#----------------------------------------------------------------------
persons_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQeSURBVHjaYpzozMDw/z8Dwx9W"
    "bgt56+A5apYO6oy/Xv/fsmbj9YVbT2Ty/uI9tl6Cm0GY+Q3DoZfMDPM+/2c4+vMHw1MGCAAI"
    "IBZOLg6GPz9+Khr6RG+ySusXZWFjZGD4doNBV19a7x/H/00rF13U/f/r5/P/jL8Y/v1iZvj3"
    "j4UBGQAEEJME018GFXnxXBOfYFEWpm8MDJ+BZn94y8DAwsoQF2YlbCvCuvqPnslB9rD4J/wG"
    "hkeZf/9y+4NkAEAAsXz5/JtBRE1Qh4sNqPndZaBffjMw/HrH8PvPewbGJ18Z6lq6rIX9oxmY"
    "2f8zmD29J102Y/bGm/0z4l7+/rMaZABAAABBAL7/AzYSAADv9vwnKDAlPgcJAwD/9+8A/v72"
    "APDm4QD/8hQAGFZrAAIKBwD/9PIAAwL/ABk1LwT53Nk5vNjeoQYAAPcCiPmVEyPDfuEfr4T/"
    "/03R4uFh/sb4g+HjpxcMz86dYBCRsWLgFJdl+P8Z6KXvnxn+AQ1hYmNj+P/xNc+6fUcPC7Cz"
    "3AYIAEEAvv8E0hD/DEBDT6MAHkUGxe/fAAmysgD0/fkA3TUVABoM/wAA8vAA/vz3AAD+/QD7"
    "/wAAA/fyAO/t6v76/gAFAP/+AAKIRUVYrb/eKMPAUN6a4dGvDwwcO38z8PPxMPAEBzN8PvqU"
    "QeDZAwZmUQGGf0DNrEDN3x89Ypiz7ez9z/+Z9jP8YmcACABBAL7/BDUnMVELD/4A7vXfABsG"
    "GAD+enQABgcCAAjrBQDA7RUANf3YABns3gD89vUA/vv7AND6KAD9OnYSBeKPHvvx29QCAEEA"
    "vv8EEREFEej73QDx+e8ACwTfAAkLDQDz9PkA2tEeACklJwALERcA1t3lAOD26wASHhoAER0b"
    "AIRHARP8+e6utp9wNgKIWdpLtFZCgYHp47fPDJ//fGT4+Pspw6XLPxlY/lgz6KhzMQjyMTNI"
    "ygoyKGpKMyjI8jGI8TIy/GcFxjSLpPDhXdu/AgQQ88d/jJH8+n9FmQU/M3xj/Mjwhfkpw90r"
    "igwigjYMbDwMDF+A4fftFwPDTyB+D2S/+MzA8OoHA8PbH+wMp/buYgIIIJbn+9/Grnz+bamS"
    "o4CqgBgnExPzV4bvj3UZpKUYGO59YGDgBAYeGzD1MgFTOJDJ8BOYzn4DXfAamKx/AcMbIIBY"
    "GJgZz74/+Vrn7MlXPgwM7MbACGfkEDpgL6lcaPPLjJuBHZhuWX8jDPgFJIBRyHDm+C2GTy9v"
    "PQMIIEagW4FJ+DM4noEGgOwAYh4eAaXoI5q+MXqiygqMnNycDExMDAx///5n+Pb5K8PzazcZ"
    "rm/oe/nt9Y44gACCGPAeaMDff1ADfkPprwIMDBK57AKSFsxswKhhZAJmxb///vz8/PPXp8d3"
    "gJrmAhWdBggwAPgNjACx1hWqAAAAAElFTkSuQmCC")
index.append('persons_icon16x16')
catalog['persons_icon16x16'] = persons_icon16x16

#----------------------------------------------------------------------
markcompleted32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAv3SURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsIIKRkRGnAplYV34h6X8ldpaGEeIiwuJ3Xtx8vmXdweVvl97vAEr/AuJ/lDgAIIAY"
    "QVGAywFqcaEaqhYsy/2D7A0EBYQY/vz9y/CP6S/D9cd3GFbkL969996DqxL8TGKMHByiDFwc"
    "An/evLv35uPfRXK3GXYDtf8BYoLxCxBAcAdckGGQ+PubwY+FlUGBjZNRUoCDU2yRmqb+gTRJ"
    "aU9zF4Zb728Dg4qB4fe/Pwws7EwM954/YRBb8ZFhQVQ4A4MAOwMDOxsDw4tnDAwrNzE833Js"
    "utQ1hjKg+d8IhRBAAIGj4IQkgzSjvOwBhcx4FXYpSYZPzJwM37l4GWw+/WPoOdLMICx1guEz"
    "20+G19/eMvz795/hz79/DD8YPjNs53zMoHU1mUFZX4/h77u/QC9rMcinGDBYszdnOt8+ennv"
    "b4YlQOO/YjhiBkM44yvG6//r/l8FCCCwA75+YUhWiotQEYqOYVh6+DXDkScyDOZKnAwc7KwM"
    "0YpvGKasL2FQ1JVhePf7G8PXr78Zfv34wfDv1ScGtpsyDMr22gymCgYMLMDQYQYm6WXX3zGI"
    "2vgwFCqfzN174892aDr5Cba4i0FJSIJ/kpGRkvfhtmsHfjL8DAYIILADWHkYYvm/XgDG2mcG"
    "CYYXDCz/ZBgSzMUZGK4yMAR5ZDJw7/nLMPPAFIafTN8Z2Bn/M4h+lWFQZTJmiA+OYnAz1QNG"
    "4R9wyPwH+jNCVYhh3jdvhgrP1ZraN875AI1YCnZED0esmZZcT6i1jujiDRe//9z2cx9QXBAg"
    "gBhAaeCICsPPf4U8//8/2fb/5/O1//17jvy///rX//d3gfjE7/+/fv////jxm//Xb935f/vu"
    "/f9Pgeyfn4E6//z//xlIv33/A4h/gvHnT7/+73r09//SeRP+r5VkOM2gwGzBMUV0WfpF+/9b"
    "/if+N12u9YuBj6ETaLkFEMsBBBC4HHj+nmHXh4dfGBjunmJgE+RmUON7zXD41hcGZjFGhh8f"
    "fzN8fP+TgYdPgEFKQolBTFSegV9ImGH/42MM4WszGR6/e8zAycYOjeb/DD///GWwE2Fi+K4T"
    "z3DBXd9Qt1B6W3eYY2SknipD86pz/05nX5/D8IlhE1DxKyB+CxBAYAe8/MGw8MkTIOPaHmA0"
    "sDJ4q35i2H3pFQMHBxMDqxAw8b/6x/Cf+TfD7z/AWPv1k+HPn/8M117dYLjNs42h5Gguw80n"
    "jxnYWdiBofmPgZudg+Ht5/cMZ5/3MRyOFWLuTPcV1BYWYqhec5HhZM61xQzv/i8HWQl1wHeA"
    "AAI74Oxfht33njHc+3/zLFD4DYOF0l+Gl29eMDx4/YvhvwQwAp/+BXruPzCR/WdgY/3PwML0"
    "h8FTwY1B7ocug5kpC0PF8XyGu88fM3CxcDJsvnCEIW5nAgOH3DmGdhtbBsZ/jAwNay/+OZp/"
    "eRXD69/zoJa/hGVRgAACJ8L5wDxl8p5hx+v737PE7p1lYDczYNDkf8lwCBgNkea8DK/+/Gb4"
    "8wIYErygUGZkYGL8xSAjKMOgzmTDwP37CYOPLStD6Z5SBgNBU4az/7YwRDjpMOgLKzI8//GI"
    "YcLuqwwPa289YHj2YwbU1y+QsyZAAMHqgn9vfjGsvfcYyLp9CBgNbAy+qm8Ytp97xcDMzMzA"
    "Kc/MwPqGgYGLj4WBHVgIMQHz2/c/fxg85IIZdp2+x2AjY8BgayHA8FxoH0OJuxuDGq8yw8Nv"
    "jxkmbTnHIHzDmsHFpUGGh4EBmK0YPqCXCwABBK+MTv5nOHHrGcPdv48uA5PGRwZr5R8Mb948"
    "Z7j78jfDF15GhpdP/jC8e/uP4QswR/8GhgIwTBhM1VUZFP6ZMay/sp/BWdGYIdbIieEPMNc/"
    "/nmPYe6uCwxC140Z+mIKGeJrcjh4DX1jgNYwM4DLUwQACCC4A7b9Yfj+5A3D5mcPgQ58doWB"
    "Q5CdQUvgGcPeq58ZpOU4GPjEmBlYPjIwMLExMfz4zcDw8ct/hhcf/jK4KgQznLjwguHet9sM"
    "r76+YXj+5wHD0j1XGZhP6zJURlQw/OcVYtDg+s8Q0zjBlYFH0hRoFSeyIwACCLk6/v/2D8Oa"
    "G3eAeenZSQaGvxwMfmrPGHaff8nw4f1fhrc8TAyvbv9i+PmXkYGZlYmBm4eFgYPnP4ONvi6D"
    "0EcThpMPrjK8/v+IYeOhuwz/jmsyFIeWMLDwCDPcffiO4fSNtwymYgJs4pmLahi4hARhVQAI"
    "AAQQSntgFwPD6Wt3GK7/fnKNgeHzNwY75c8Mr14+YXjz9Q+DlDonAzsTI8MfoM8///jP8Aro"
    "+2ev/jC8/fqfwU01kOHKpW8Me04+Yfh7WJOhJqGUQUpBjmHL0ScMtz4xMnxn52KQleVnkONl"
    "N2L+/lkfORQAAgjFAVeA0fv4HcPGB7eBaeXDHQYuARYGHYGHDNvPvAO65z/Dew5Ghvd3fjKw"
    "cDIz8PKyMAgJszJwAkPBwtiAQeiRE8PHLZoMsZ5ZDBzCCkBH/mGQleRmkBDiYtCT5wKmPGYG"
    "7nvnGV0YfwcDreKBpgcGgABCbxH9B0bzmksXGf4yvDsPVuev9pBh95lnDKJ8TAxi2twMTJ//"
    "Mfz6xcjw7hMwpl7/ZXj8/DfDO6DjCqKKGNpLWxhMbLSA0fMf6DAWBiUZfob7994yAJMTw81n"
    "fxhUZe8zFOgwBgBtlgcaDio+GQACCKNJtoWB4fL1a4yXfzy5BazD/jI4qbxjeP3iEcO9p98Y"
    "vgDbDa9+/GN4/+gXAzMHMwMfMIRExdgY+PmBISLMw8DGJ8rwFJhrnrz4A46ef8wsDOIiXEC9"
    "XxluPPnIYCT7lMEhQJDfjIEhGmgVNygUAAIIwwHAUuLPq8//19+9/B5UBDJwC4Ki4R7DjlOv"
    "GSSEmRgkdXkYWN//ZmAE5gZQTnj6EhgKQAufv/7D8Pn7X2BjhYVBABg1MpKsDLLizAyWOgIM"
    "p+7/ZNhy4SWDiYwoA4e9GEO8CnMIMBTkQKEAEEDYGqX/PzMwrL90Ctg++gqsTJkFGIK17jDs"
    "PAEspYDZ7wPQggcPfzK8eAxsFwBLaDZgehAQZGWQEGdhkJVgZpAWBuYSYLF9/v4Phv4tLxiS"
    "p59jmLdjD4MTwzIGHWFg6SsuyeDgriQOjAM/UCgABBALtmbSAmC1pHDp39mvj29bcIvaMTir"
    "vGSo3PmA4e4jQwYeUR6GP4ocDFxA20WAvmQDtvp+AqPl4fMfDGfufGQ4ffMFw5NnDxn4/t1l"
    "MBF7ypCv/J3B3A4UFfzAQkQHiPUZnj+b9/8DpL3ICRBALDiaav/efPm/9vaZdxYGpm+B8cvM"
    "oC94g2HXyRcMeXEqDG8MeYHp4gvD3iMvGY5f/whspH5k+PfjDYO68BsGL6VPDPYWPxg0xH4z"
    "sLAKMDD8AFrx+TUDw/2bQPZzYDH5jGHP/ncv3wHTGsgigADC5YD/wNbBxssHfzUbJF/jYBD0"
    "YIg0uMSQs+Uew/fv/xmOXX7J8PbDWwYJnvfAIvsTQ5LtJwZTqdcMgtxvgQUY0GigYxheANnf"
    "gXkKWHb/+/SX4RtQ+ONrRoYrh38zrPnAAKxwGB6CqmOAAMLXLGeu4WTcW7ZF2p5XLxrYFtBj"
    "yO2/wfDqxQMGH+P3DA4a7xiUJIHOZAXG6+8fQF8CK4F3wIT4EphLngGL6Uf/GJ49/MPw6PHf"
    "Hw9e//nw4OO/18Dq5Nk9BgZgucgAbHgw3AC1hQACCG+/IIuBITupUWKKcREfsPZmhbQvvwFr"
    "I2Dq//3sL8Ob+38Znt/9zfD0zm+GB/d//7r77M+H++/+Ai36/wKYm4ARw/DkO7Bm+QepgoH2"
    "ApvSkAbqV2jN+AUggPA6AFhkiSkJsuwOSBfW4wTWiM9u/mR4eOfXz7uPfn28/+rPm8e/wBY9"
    "AZr0FGgRqNkCamgAwx/Y6IJY9BvaKkam/yDRfwECCK8DQAUVMJEoSDEwZABpaaAXXgCdDgxJ"
    "BmBqYngN9cVPLJbA8F9o3f8X3mhE6y0BBBAhB4Bb7UAMagvxQxPtX6hFyD76Q8giXAAggIhx"
    "AAO04mCBFt3/kSz6R6xFuABAgAEAFFPKTSk5QagAAAAASUVORK5CYII=")
index.append('markcompleted32x32')
catalog['markcompleted32x32'] = markcompleted32x32

#----------------------------------------------------------------------
weather_sunny_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANNSURBVHjaYvz//z8DJQAggFhA"
    "BCMjI5jzP5OR4f8nZqAAA8O/9yxsDP8YOP69Zvn2/wej489v/3f//v9P/8fPfxd///rH8OPb"
    "X4Z/QMsBAghsgI4AIwMvG8JUoHn8/38wfPz/i7GLQYLxJZMgazjrp3/n/t36/RwoysTIxPAD"
    "qOw6SC1AAIENMAIasNACaDPDH7AB/74xWv7hYIhhSWb1YXFn4GcQ/MnA8ptJn/0a20+mHsaM"
    "jzv+2DP8/ysPtOgxQAAxgV0gAnTSl99APzAwM7L/s//whkmNtYglmiWSk59BgAdoLj8DAwsL"
    "A6PBF3beWb+FOF0ZtjH+Z9L9/4/xH0AAgQ3IlmZneMSrCnL839d3mD+wOf7tYraSYGB4K87A"
    "8FyQgeEVEL+VZWD4DFTDxc4gWPBDllGMSefPz3/GAAEE9kKMxBaGlwLKDEe/KMv9ZfwXza4O"
    "9M5noDcfPGdg+PANaDuQzyfEwCAMNIhJgYFJ9T4Do8xXN+ZHjAsAAghswAtRTXDIM/z6/+TN"
    "y79Hv3xSLP10TY3h8ytZhresZgzy71Yx6LPuZGBR+MjAwKYENFCEgZH1i+A/BkYdgAACG8Dy"
    "9yfDX2YWhtvcBsIvC3NS2ay8GX7zSjBI6zIwaAG9v+NyAsP55ycYLD5OYND5vBfoOiaGt4/+"
    "7+dhYrwDEEBgA37/ZWD4+/+/RLv+/uTeBAHvT0BX8zKBo5Ph4lMGhpc/mRj8bKwYHryxYnhy"
    "r4xBfcPEt/+ec/7lY/z+HiCAwAaoiDJVx3uKFcmJsgvde8nA8OUnA4OZFJAGRsxFIP/dZwYG"
    "UVagxdxAtkIXw1HbqF8rv705aXZq+S+AAAIbIPDjszM/n4zQFzYmBiF2BgYhYOp+/B2oCZi4"
    "vIAB//ADUPMvBgZxTgYGQSA+p24gacD/d/WCXxIlAAHECMoLwrprGTnYmcKtnJXypGR4+Vh4"
    "uGRefvrPEeEryf4G6B0GoHf0gbEKTMEM808wMGxZcf0C++P7C799/rkVIIDABkgar2d4++Q9"
    "A+PvP0x//gIT07//skz/fouIW2g2/PjFwOZtL6L8h4WNi0mAW2Dd0pu7v529HMkrzv+ZU4SP"
    "ASCAGCnNjQABxMRAIQAIMABmRxzkGQZ+PQAAAABJRU5ErkJggg==")
index.append('weather_sunny_icon16x16')
catalog['weather_sunny_icon16x16'] = weather_sunny_icon16x16

#----------------------------------------------------------------------
lamp_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAbuSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsuCSaGBkZQK77A8R/gfgHA4PuPwYGa2ZGBhMuYQYNTn4GGTYWBj4moKK/fxn+//zF"
    "8OnrO4bHnz4x3PnIwHDuAgPDMSB94SEDwz9kc9FDHCCAGHFFAcwBvxgY5H4zMDTI6jOEqnow"
    "8MhZCDAIKmsxcAjIMjCzCTIwMrEw/Pvzk+Hvz3cM39/eZ3hz/TrD4+PfGe5uY/hy8gHD6gMM"
    "DO1PGBhu43IAQACBBbDhBqBkPQODYRs3w91TExj+//1lCBTuAuJbQPwJiH8C8W8g/gPFP4D4"
    "PRCfA+Kq/z/eq/4/UMDwv5KZ4a40A4MtLvsAAginA5oZGLhbGRkOX13KAuTmA/ENoH1P/v//"
    "cv3//w+XgW4A8r8/Bbrjzf//X+/////xKhADxb8BHfgHKP7/DBAH/j/ZxvA/lYHhJDCuVbE5"
    "ACCA8CVCP4MYBhutqAigrjAGhm8PGBg+nWNg+AwMzZ8vgBH/nYGBiZWBgZkdFJPAxPIJqAYY"
    "2B9vANWcBSaa90DxPAaTIgcGe3MGMzUGhmigADu6JQABhDMRMjMw2Ku4swFZOkCDrwAxMCV8"
    "A6VIoBY2AYjlP9gg9J8vDAy/gUnu1ysg+ysQ/4Ok3v/cDEycpgyqTkcYJE7+sbzGwCABFH2I"
    "bA9AAOF0ACc7gwav3H+I+m9A3335C3EAI9AT/4G+/w+04e8XiBGg0Pj9Dhgyr4H0ZyD9B5J1"
    "mIBqOZ8xSOj+ZwA6WQwoIozuAIAAwukAdnYGPlYOoK9/gYJTE2g5MD98B8kAff0H6NvfQEf9"
    "5IGE1b+fkCj4/R7iAEagA5iA0cICNP7XGQZukb8MXECnMEAwCgAIIJwOYPrH8Pc/yMffLwMx"
    "0Ds/gEH9HWjof6ADfgEtZuUF2s0JccD/39Cg/wSJDmaQA0ChBRTju8vw7zMky0MUowKAAMLp"
    "AKA5z369ZoAE97fzQAx0wBegqb+BWliAFrNwQaKDEeQooAP/Ai37/QUSGux/IcUX83+wQz49"
    "AAfeD2iZhgIAAginA4Cqr3y8yuAn7QEJDgamnxBfgcq1H0AvMQE9w8gKVQ0U/Af09f9/4CQB"
    "xswI09+eAmIGhncM0EhEBgABhDMbHmJg2PlkLcOPPyBtXJCoZ+CAYpDhYF//hPj432+II1ih"
    "8uxQ9fzAgLvDwHDzIMPfqwwMwPzJ8B7dHoAAwumAYwwMR65eZtj2YDI07cIM5oAmJZAFrMAg"
    "ZoFidiQHguRA6RPooHttDAybXjDcAsbmCaDIK3R7AAIIZxS8AXppPgNDo9gEBiMuGQYFqUig"
    "4HNoUgLp+o0Uo7DkxQrF3EDMB7S8hYFh5SqGt1sZGLZD/MTwFd0egADCWRTDgAEDg+dKNoYX"
    "LzqAws+A+AoQnwbiE0B8DIiPQmkQ/wwQ3wDixwz/H2Qy/G8CxjswIGYBjbGGeRbdHoAAIugA"
    "EACWAp4LGRgePCwBSj0F4utAfBGILwDxeSgN4t8CJsN7DP9vhzP8r2FgeAmMqelQy1lxeRgg"
    "gIhyAAhoMDA4zmNguPWsFSj9EohvQyz8fxOK70IcdycBWAMCIwuYbicBtVkgW47NAQABRLQD"
    "QMCHgSHpmjNQ+gM0Oh5BMTDI/78BFs5A3+9SZfgrysAADDAGU2yJHN0egABiIaX5BPTVCy42"
    "YDXDBYzP/2iJEJQzgDmSGWjlf0h5f4sBrTWEDQAEEE4HrEJ3OTQX/mKBBqoAluIbWBBysoK9"
    "/ZcYy0EAIIBwOoCVkwuzeP7NwPT6Mysj9342YNnzH+IqaGyBCsV/HxkZ3v/+BRT6DHTDP0Zi"
    "HAAQQDgdsMDGCkPs95/f/3Q0jP4HfI1k+PPnN2r74Rczwx/G3wxrDGYwvL+3EqiYqABgAAgg"
    "3A0SAUHU1iuw0vnx5ctvdgmxfzyK0gy/gT5FCX4mcPXHwCUu/v//v/9ERwFAAOF0wN9fvzAV"
    "MzL4crKys71/+x7DAYyMTOCKkZeLh1lVVcX0xo2boML4CyEHAAQQTgf8/PkDI/sAQ4EJZBHI"
    "s4xoMczI+B8cCqyswPTx7x+wGgK3gF4QcgBAAOFpEXFgiP3795eXi4uHgZkZXB2CowXhAGDC"
    "ZWVl4AQmXhYWFjZ8ZiMDgADCqej+/XuoUQLs/ggLCz988uQeg4qKBgMvrwBYDG4QsPn18uUL"
    "hgcPbjN8//79CyJ/4AcAAYTTAZcuXcYQ4+PjncTLyxeor2+sIyQkAIyWP4hsy8rM8O7dE4ZT"
    "p449un//wQ5igh8EAAIIZ9eMkRF7NtbW1k4TEhLKBsYzRhwB0wDb7du3z78A1Z0MDKew6Ue3"
    "DyCASHYAtOBVZ4C08f9hkfsOLYY/EOMAgAADAK5Mb3uOL29WAAAAAElFTkSuQmCC")
index.append('lamp_icon32x32')
catalog['lamp_icon32x32'] = lamp_icon32x32

#----------------------------------------------------------------------
folder_purple_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAG9SURBVHjaYmRgYGCKE5v+lwEN"
    "LHqVqQ2kbgPxPyDGkIcBgABiAGr+//n5HwwMEgfKOwExFxCzQDEzEDMi6wcIIJAgw/NLPzEM"
    "nnoxlYFBn2EvIZcBBBDYgJ8f/4El/wMpRiYIff/gd4gh6ECf4SrQEEcg6yjIAIAAAhvw9c1f"
    "hvf3fzMsX7iGgRgA9N5+mCEAAQQ24Pv7f2DNWG3EBfQZQIZYAAQQ2IBvbyGB/PjUDzD9789/"
    "hv9AoT+//jP8+/2f4c9PEA0U//ef4cqqLwya/twwY1wBAghiwBuIAT8+/AOHAciAv78hNEgz"
    "yLC/QIP+Ag38/e0/w49P/2AGCAMEENiAH9BA/AmUANkCCkSQJpgBYM0gV4BcBaRBFkEBI0AA"
    "QVzwDuoCoAH/oQaANP2HagDZDHfFb4grYAAggFhgToeFxX+o4RBvQP0ODZO/IBf9/svw6yvc"
    "BQwAAQQ24NcXiInfoS6BeQGkgeE/lIYZ/P8P2DtQ8AMggEDJUgsYr1cZSATAKFwFpNYDBBDI"
    "AFYgNgRiZyAWIlL/dyC+AMR7AQIIljFANDvUMGIAyK+/gPgPQAAx/v//n4ESABBgAImqDvrw"
    "nHjcAAAAAElFTkSuQmCC")
index.append('folder_purple_icon16x16')
catalog['folder_purple_icon16x16'] = folder_purple_icon16x16

#----------------------------------------------------------------------
fileopen16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJQSURBVHjaYvz//z8DPmBvZ3XW"
    "yclN5Ov373+Qxfl4+dh279paCRBAjAwuG/0ZmFg2YOh8dzGT4UzV6qXLVryOigxn/PqXgYEJ"
    "KvUPiLmZGRhMTAweAQQQC0jz/51eGPoZ3RmmM2hmST9/++v7gy8MXB9eAw2AmvATaIKSDAPD"
    "778MPwECiAUkcOIhptNBhgINqXny/j3DhVsMDN++MjAwwwz4AeQDXfDzDwMjQACBhd59heC3"
    "IPobA8MbIL3lGsQQQR5mBj5uoOb/EC+AMAtQMz870JVANkAAgV3w6gsDw/WXDAwLT2C6JEOQ"
    "iUFCGKjpL8ILv38zMEgKAS34/58FIIDABrz8BNH8ohXTgMkzmBhk+IGafwFtZISI/QUaJsIK"
    "5P9nYAcIIIgBnyES++9CbQAG0h8g/gVUyMXBzvD7y3sGDSlBDMP//v7HDRBAEC9ADXj3HeIv"
    "kGZgCIMNmL50KcPMeXPAvgelmK8f/zDw8LEwvP74m+HB7StvAQIIbMDrL1ADvkLi+B/IdpAh"
    "wKQTWLkCFNoMP/9C+EeXPWMwi5BimLIeaNtlro0AAQQ24A3UBW+BMQCy5e8/iDdArgBpBrnk"
    "918E//NPhDcAAghsACjqYLEBStn/od74BdX89z9EM0gMZMAXJAMAAghswCegAaxsUBcAFf/7"
    "D3HFn3+I8ADR4AD+i2oAQACxgNL8u0vAZCvpyLDoKQNh8OgPw929TxgYDiWeBfI+AAQQKNCB"
    "yYTBDIoFGIgHH4D4FEAAgQxghmoUAWIOEgz4AQp/gAADAI479CxVBabfAAAAAElFTkSuQmCC")
index.append('fileopen16x16')
catalog['fileopen16x16'] = fileopen16x16

#----------------------------------------------------------------------
progress16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANaSURBVHjaYvz//z8DJQAggBgZ"
    "DKczMHz6xcDAzMzAwMjIwPD/vwAXB+tGzr8MOu+efsn4//PfZqDEDwZmJgaGr7+B8iwwvYZA"
    "PA0ggJggxgA1/vnHwPD9DwPD2x/cKhI8SstnegolxOisYmBiWM7w548ews6/MIYkEGsABBAT"
    "qnuA3vn7/+/fn3+/qSkKMsyb7MawZplfgIqK8EGGLz/TgSqYGBiBBjAzwkz6DRBATEDLa9jY"
    "mLaxszGvZ2dnWcvAzjSd6ecfaUaGfwzb7wPd6ajGcPRAhEBopOYMhv//lgE1cTMwI+wECCAW"
    "FSm++tZ8YxZRblaGf//+M/z985+Bj4uVQU6chWHhod8MU04wM+RZ8jCsWubH0KIlEt7QcFTy"
    "76+/XkDH/AYZABBAzFqmkVWNmYYsN17+ZbjzgYHhFzsvw3smDoYzD38zXH3+h+H9D0aGY0+Z"
    "GT7/ZWQoCJdjEORjk9+1+54cMPLOAw3xBwggFiDj/9t33xgmrH/IcPL+TwYxUV4GNi4OBg5u"
    "LgYBfg4GHp7/DCws/xk2XOMARgIjQ16mEcOpMy+Dly299BoYGN8AAnBEJikAgEAMK+OG/v+h"
    "IjooOGL11ksOTX4TY4E1FNomNAm2Ge8cREeYvgL3koPaM4QCS/GBPeNjrwACyjMy8HIzM4Ra"
    "izAYq/1m4OPjYGBhZQUGMTPDuWd/GJ59+Mbw69MfBi99JoZae26GVcuvMixdcmUH0NgtQP3+"
    "AAHE8vbLL9Yztz4yWKjzM5irMYADkpeThUFbgZchbcFzhnP3vjFUBEgw5HvwMcybc4EhJ3/v"
    "9e/f/6YBDQCqZmAHCCBGJqOZE9j//zdkYmT4BUwD/79/+MVkqCRotXK2C+eyEx8YjNUFGOw0"
    "eBgqqk8wTJ1yfj/Df8ZgBkam9wz/GN2BBiwGCCCWf///F3z/AUyBwITIALSd4esfia9//x34"
    "z/hfvSZMhuHwyVcM9h57v507+rQPqKKFgZXlJ8MfRP4BCCB4wmYApclf/0EGMbOxM3O+B/q7"
    "ov0cw5QpF099e/+zhIGN5TDDL2gy/s8I08sKEEAIA0CCLECuIPP3B69/vAzJ2Cv99Mb7sr8/"
    "/s5n4Gd7z/DxD0gRMM/AdbwC4kcAAcRIaXYGCDAAPQYuSZDebHgAAAAASUVORK5CYII=")
index.append('progress16x16')
catalog['progress16x16'] = progress16x16

#----------------------------------------------------------------------
newtmpl22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAFgSURBVHjaYvz//z8DLQBAADEx"
    "0AgABBDNDAYIIJoZDBBALCBitjXja1UPBqG/vxn+MTACBf4B8U8g/g7EP6D4OxSji0MwI0iM"
    "+TsD46OfDJ/i/v0XBAggsMGKjgwCDrUMTGAfgDR9BuKPUPweiD9AMbrYRyQaJPaHgeHkLwZe"
    "kJkAAQQ2+P9fBkjSeI2k8BPUgk9Q/mckGoa/AvE3qKt/AfFfMAKbBRBAqGHMCBUBYWYkNjpm"
    "xIGRAEAA0SzyAAKIZgYDBBDNDAYIIJoZDBBANDMYIIBoZjBAANHMYIAAQjX4HySR48T/kNT8"
    "x4GhACCAwDmPkRmavEWAmAuIeaG5jBuIOYGYHYpZoBmHAWrIX2jZAcKsEDlmaFYBCCCwwU9P"
    "MXw+PYuBH1gI/QcL/4cWNF+h9Dcs+DtalgYazgIswh78Z/hqDuQCBBAjrWoQgACiWeQBBBDN"
    "DAYIIJoZDBBgACIZdUY3viQmAAAAAElFTkSuQmCC")
index.append('newtmpl22x22')
catalog['newtmpl22x22'] = newtmpl22x22

#----------------------------------------------------------------------
viewnewviewer16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAN2SURBVHjaYpwgyMaADJhYGLlY"
    "mP7L/nr566aep9RObm4WrgdrHjlISbErMH39+/Lfj39fGBgR6gECiNlfhJmBBWgGCDOz/Gfg"
    "FOIK1It22P7x4YfvCi4SnsIaosLvzj9jUcuWWfT92rd3fz7/Pc3IBDIBggECiNn4+1+GL18h"
    "+PPXfwx/Pv9gljJRC9ILMQtmY3rLx87PyCPmzOfI+OP/t7eb305m//3vEcuf/wwwDBBAzG5s"
    "jEBnszAwsrIwMLGyuKq6GKX/ffOZSdZcQF4+1IyJz4CHieHzB+ZXW55cEdDkUGR8/ZeBhYv5"
    "BhCDaAaAAGK5bWbOwP/+NYP862dAPzCbKrsYx7D+/f6Tmek3AwOPADBQvgENZmAQtOXRZef+"
    "Z/Lx8rcXP/4zrYcFA0AAAEEAvv8DEiwyAOYBBgDWCRAABvX3xuqtwa7//P4lHzMnJfjh1QD8"
    "5uUAAPb0AOmv7Mj1jOdWyQ0SpIgRJP7KDx4AvQkYAAIAQQC+/wMmIiIA9/j3BfX9/QvaDRP7"
    "FQP4PE9sPHrKptCoAvHnD/zs/PjKoNi9LGMRRrXTBMyCECTPAP79Cf4LDfz1AwUAAgBBAL7/"
    "BAMDA0UpKCh+CwsLB/r+//sD/jAXsM74AM/oDgVpG/X//xIEC7QDQx3hvAkA20ANe0gCB18H"
    "//4J5ufnu+3t74ECiCEyMmb7nTt3/4PA92/f/n/++u3/l9///y/fte9/uYzE0u64qI/Xnz7/"
    "f+/5u/+v3r77/wOoBgT6+yf8t7V1UAMIICY2Ni4OTk5OcICwsbMzcLKzMXCzMDCIsTIznJHR"
    "OHeHW/QL959fDFyMvxiY/v9j+P33H1gtIyMTA1AvG0AAsYiKiutycEAM+PPnDwPQcKAkMAK4"
    "uBisvDzLeNjZhA8dPsEASjyMUI1Ab4PCn0FFRXUpQAAxzpq16Fdqaizrv3//4AaA6L9Am5iY"
    "IIZ+/fqdAST/799foPhfMM3NzcWwYcPOPwABxMLExPIXaBwrcn5gBDqBCWgjxDVMDFxcnFAD"
    "IBgkzgROzsx/AQKIhYWFDa4JhJHZMMXImmFi4ETEzMIAEEAs7OycLDBNLCzMUFtBmhgw6L9/"
    "ITaD2CDvsbGxMwMEEMvLlx9f7thxnPvv39///4I8w/AfaAjIpv9Qm/5D2RCbYRhoIeO7d9+/"
    "AQQYAGNclQeXlEoyAAAAAElFTkSuQmCC")
index.append('viewnewviewer16x16')
catalog['viewnewviewer16x16'] = viewnewviewer16x16

#----------------------------------------------------------------------
pencil_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJbSURBVHjaYvz//z8DJQAggFhA"
    "BCMjI6YMNxsDg6EiA8NvoJI/rAwMP4GY7X82m+z3ll+POFoZ/jP2/D9/mgEggJiwGssH1Gwi"
    "jeD/A1rA9C/VMfp5n3bhEwEGu1edDP8ZYkBSAAHEgqFZAGiThTgDwy+gpr9QzSx/851Tnkww"
    "cH7N8P+PNgNjECvT+SdfJwFllwAEEKoLxIGa7YQh7P8QzYzsf8vd0h5O0ANq5vmrzhCjWMzg"
    "xqbCwPzsyx2QMoAAQhggCXSMHR+qZo7fDbbxjzu0bD4yCPzTYPBVKGfYfm0Pw5T8o3v+HH/u"
    "CVIKEEBQLwCd6cANDKj/CM2cvzt0/V+Xq5p+ZxBl0GDwUChj2HJpO0Nb9r5tP8++Dwcq+gLS"
    "CRBAEANMgTb/hUbnPyDm+j1B2uVdvrTOTwYFNk0GL/kShrXnNjJ05u5f9/v8Z2Dg/f8OczhA"
    "AEEMUAca9gvogv/A+OT8PZ3L6lO6oNJvBh0eHQZfxQKGFafWMHTnHVz29+L3BKCi38jBBhBA"
    "DOCEFAVkhQFdEcU9i6Gb+b/gcob/SUf1/5/9Oud/yT7n/4y67HOAKpjRIwykFyCAEAaE85iI"
    "NQn9Z5jH8N9/j9H/ox/n/M/fbfufQYt5Mq5UCNILEEBwA5TyZefuOtH+33On3f9Nr6f9T9xi"
    "8R8Ydt34kjFIL0AAQYgYBgWPGueP/789+L/91rL/JhM1QJobCeUDkF6AAIIQacwzWf0U/1vm"
    "Wd8QtBbqZ+BmsCYmI4H0AgQQhLBm7GCQZggEivHAZRmJMwAggBgpzc4AAQYABEbgQL928iEA"
    "AAAASUVORK5CYII=")
index.append('pencil_icon16x16')
catalog['pencil_icon16x16'] = pencil_icon16x16

#----------------------------------------------------------------------
progress22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAASzSURBVHjaYvz//z8DLQBAADEx"
    "0AgABBALiGA0mcXA8OMPA8M/IOc/I0KWkaGG9df/6t+ffh1mYGOOZ/jD8BwkyPDzLwMDE5D+"
    "A/TtZyAb4ekQIK75/7/IACCAsLj4P0Lh1z+2mfG6HEU5xq4sv/7eYvj+uxgsyYjToQZArANi"
    "AAQQ9qCAhfvvv78UpHgZeutsGTatCODR0hbpYXj/4yzD3/+KYMNBmA3FFqC3wf5mAAggJpgj"
    "Ifg/BP8Deu83UP7v/78/f4HVMXg6KTAc3R7GkFdgasT0++8lhq+/YyGxxIjVBwABBDGY8X8B"
    "w5+/e4GG7QTiHWD84886hs8/Lf///sPwE2jXjNN/GZ794mCY2OnAsHa1P4+0OPdCho8/FzCA"
    "TGDGNBgggMCRx8vG0p6cqsUhLczJ8AcYIYygYATiH9//MjjbSDP8//2L4eJzRobt95gYMkwY"
    "GAK8VRhUlQQZk1N2xJ889liGgZ3dC+i7X0iRyAAQQGCD+dmYmVOD1Rk0pHkYvnz/w8DEzMTA"
    "xMTMwMzCxPD371+Gj59+MbD+Y2T48YOBYeppVoY7b/8zpJsKM2zZGMgQFrHJef/eBzsYmNmd"
    "QEEHMxgggMAGgxLMv5+/Gface8NQsfwZAy8PGwMbJwcDKzsrAxsHO5BmY/jGABRj+8/ACnTV"
    "trusDG++/2eosudiWLnMl8HLd53jmVPPshkYWL/BDAYIIHjkMQL9/vX7L4Z7Tz8zPHj+jeHh"
    "i68MD18C6ZffGR69+s7w7csPht/ffzL8ATqbm/E3w+lnjAwtB/4ziIpxM1SUmwN9xwxKipzQ"
    "ZMAAEEAs0IwAdDLQ5b//Mvz5+QOIGRmAoQC2jBko9xuo9ieYD0o0kCT0l+k/w8fvLEDfMjJw"
    "crIwsDAzsP39g0gfAAHEAnPx33//gIb/ZWD7/4eBBYiZ//9mYPnLyMAMDFvW/8CwBlr2E5oi"
    "P/34z6ApzchQacvE8OvrH4YpU84x/Pz5dxE0KMCGAwQQ2GAmJkZGJqATHfWFGA52GzOwsrAA"
    "IxBoINAZf4AR8PnbX4bmbe8Zbrz+xsDE/o/BVo2RocGdlUEEGOaxibsYtm+7c52Bia0K6MAa"
    "WMoACCCwwV9//WXYeOgpg4woF8P/v/9hocPwG5g5LPVEGORFORm+fP4JTB3/GLJdeBjK/QUZ"
    "3n34zRAatYNhw9qb1xmYWR0h5QwDCywoAAIIzPjw48/E2glnHYH+/A22EWT47/8/GD781G9v"
    "shFMDldjkOD5x5DsIMwQ4STOcOzEK4a0zP0MVy+82M/AxuoGzMh/GNBKX4AAgtlQCjbs3z9o"
    "jgeymYHZiYNlPVBHACiLd0bLMggJcjL0Tr3MUN948s/Xdz86GXjYaxh+gfRhlukAAQRPFZA8"
    "zwhJLKxAQ0HJ4RsjCxsLI4OoOBfD6QtvGRLyDzPs2fbgGTBvhjMIsB1h+PkPWuRgAoAAYkEv"
    "MRkYmSDJGxT9LEysH4CxPnH2dYa6zjO/Pz3/spCBhy2f4ce/b/CC689/dPPAeQMggFhQTQVR"
    "0JIUVOBzsV6ZsvCa65e3P278//M/nYGb9QjYZ7jBDSB+CGIABBAjreo8gACiWZ0HEGAAo73H"
    "xZi3HZQAAAAASUVORK5CYII=")
index.append('progress22x22')
catalog['progress22x22'] = progress22x22

#----------------------------------------------------------------------
mergedisk22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAR5SURBVHjaYvz//z8DLQBAALHA"
    "GIyMfZeBlBYQ/8WijhGI/2NhgwATlA+iH/z/X6QMEgQIIBYkBdOB2ASIf2Ex+DMDwz+ghYxM"
    "EHMZeKEWIFvCBsRXYRoAAgjZ4Gm43fnTHshpgGhm7GBgYN2M4W40ABBATNhFgbpYGUHhAzJ0"
    "nrKy8P4ZMzwd5s31ttLTE93IwPBjA9BgJgYWoDwzI1YjAAKIBcV1YA8xQRSDXPP720ITE8m4"
    "pYv9GNQ0hMDSHu7yjNFxW/z373u4mYGV05uBDaj2L1Dx938oPgAIIISLOZkhGOQKkIu//2jR"
    "1hGLW7kykEEFaGjXsf8M9Qf+MghJ8TKsWRXAYGkl68Xw8+c0cJSBDOcD6mVHuB4ggBAGgyTZ"
    "oF77/ttZWISrfO4cLwYlJX6GHqCh+x8yMhx78JehevNXBnZuVoYF890ZpKT50hm+/I5EOA5h"
    "HEAAIVis0HBl+M/H8PfPir4+ZxZzc0mGuReAhj5gZOBh+s3ABUwwR+7+YGjf9o5BTY2fYcoU"
    "RyYmBsaZDL//KTKwgoIQYRxAACFY/6H4089ViWkGInGxWgwHHv1n2HiLEeiQ3ww/v/9k+AHE"
    "HP9/MWy8+JVh5s7XDIEBSgxFpca8DF9/rwVrRopHgABCcjGQ+e13rLKOiFtbkw3D3Q8MDFNP"
    "MwKD8Q/Dn58/GX7/+AWm//z8BWT/ZOjZ9ophz5nXDFWlBgzG1lKGDJ9/l0N8DAEAAYSU3P7z"
    "MDH+7+9qs2OUEOdm2HrzHwM7wx8GaQ6ggUBDf/74AXY1yHA53n8M/Kx/GRbsecnwD5hv2pst"
    "GNi4WGoZ/v2XhZkGEECI5Pb196zUdAPhIF8VMDfekIkhRucPw8dPfxlKNv1iePcN6NLvv4CW"
    "/WJoDBJikBQQY/gHSpF//jG4OkgyFBYYcHf2nZ8HFHIF6QcIILjBbLysgWKSPAzLN94Ce5/h"
    "z18GK2NxBmlpHrD3f/2ABAEj0Bfi/KxAsT8Mh06+ZPj17z8DJzsTA68gGwO3ELsDzDyAAIIb"
    "/JuNaW7zhJN2QJV/GP7//8nw4admV58Tf066PsMvkMHffwAN/gE2mBmY1vccfM6Qnrn3EwMX"
    "6zVgDmUHxhEzIzvzWZh5AAEENxhYfOYwsICSDDOIAxLawMLO7P8XmKt+gyMMhH8D885vhn9A"
    "7zOzANXxsR1iYGH2BWcoBpg2CAAIIBasGR2kgoWRjRGo99+/f0DXQiLvDzQo/gCD6T8jSA0w"
    "yP+DQpoZwwiAAMI0mAmoEKTpPyM4xfwDGvLh03eG9++/gcOYnfEv1GCQecBS6h8o/YLKCdTy"
    "DCCAUAshJmjJBvUSEzAsuYCFUoWPKMPnrwJAA/8A7fzHIMjDwsAEUgcqV0AOZgJqAFmAFBQA"
    "AYQwmJ0RUcSBA+sf24dvfxjef//D4KDDD5b5B4R/gUHzFxhG77//BgcXpO5gRMq+EAAQQCwo"
    "pRtKODMcmrL6us2KXfcY//799x+mDRwCQIPeffkF9A7raVwFPUAAMdKqMgUIMADYxpmmVpIy"
    "XgAAAABJRU5ErkJggg==")
index.append('mergedisk22x22')
catalog['mergedisk22x22'] = mergedisk22x22

#----------------------------------------------------------------------
clock_menu_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9gGARIDDtZyr/0AAAVz"
    "SURBVDjLdZXbb1xXGcV/+5x95szMmRlf5+JL7LgOTtImaULrligOrV1sCC5E4FYIqlIeEK0Q"
    "fUECif+A8oSEqtKHIhTEAyI1IjShseOqIS7BcRJXiSF2sNPExo7t8e3M1T6XfXhwXBqarsdP"
    "2j99S2vtvQUH/wAAUoOIAT5ghcCnmZB4uSphfrWhztpRmQhZArALbmnubnF+ZX1zENd/E5gg"
    "CMDxoOyCr7Zw/L88BY5/ov1Qsq/36aZoW5NFTEqEHwCCQMcs+X7V1FzpkXcvzL4y/I+FMwTB"
    "twHnkxjx8caGDoiWyprwe6+8tH/nFx5Nsz67DoU1wkB2YQbfc8k0trIRQBCtoKq5hrEbWd44"
    "cX1hab74LARX8HwAdDLPgwAc1Vafjl752U87MlXRMFf/co49tSW6jrRwcH8a5S9QkfDp/crj"
    "NKd1indvc+6dERo+t5vOzp2xifGFF9cWS2eQYgEBOqk+cHxZHTdGf/STp9LrS5u0uIt0tofw"
    "ydG2+2F8pYjHEyRTdRiGSSRqsbg4xaN7KtDyivHZgCPdu4zxS3f6irbzBkI4gpbfguud+O6P"
    "n36xKpPhsDFDb089gWaiCYGUGrquIaW+FYHno5TC8xRKBQjlMHR+jnNr9TjlHG/9fGjQU6JH"
    "Ym80721vfC7W3Ez2/F9p+mY1gdhJNGIiBGiauC9bKXWE+HTm23r8+XOdlwdvPqYJx335ofa2"
    "yPxHNi/1JLk9v0x//wCzs3Ofgm4rCIIHzr/xiyXqD7ZKqdSrWkXU6C1Y1SSDVY4e2c3x48c4"
    "cKCNgYGL9PcPsby8dv9p5T8QvpGzadSy2GYVtTWRTq2iNtq44oRprQwIx2MIoXHo0F5eeKGX"
    "eDzK228PcuHCFexCGe/aNdyhIeYXlhn7cIKTJwcAcF2FGbXYVydZzksSyXiNDFlRaz0vKNsz"
    "jIws0tK6l3C4BsuK0t19mKWlNs4Pf8jkn85S+ud1EukG9KxLLGLS3JxhdXWdfD7P5MQ1nIJD"
    "odyIGQ6HpBImQgPlOZSKHuqe1W2lUjX09TzJqd/3M5mpY8+BPXQefQLD+F+A2ewapWIZz/PQ"
    "BChMZLlEKWUFZl3DLrq6Wilt+vi++rheOA7q1CmOHfk8zzzUiinlfVDfV1iWRVfXM8yvT5HI"
    "KuwyrpbLOncrDY/5oiBv20gpUUptx493+jTejh2Y+/YRj0YIhYz7HPm+QhoGxXyBO8setTGB"
    "nd1clYUiA2ql+HAumeDq1WkwSrS3P4kRMlDDw3iWhXn06GfUbqsdly59gO6FWJVNYBexV71h"
    "DT305vSlBSdcW8PZC3cwpY5vhtm8Po6zukqop5sHtVkpxeami+cFxCJh3n3/FuFMNbcvZ30V"
    "yNc1otbE3L/s0+NX19nZ0U2unEabmsT59xTBl4/hewr/3hu7bd1xPFx361r7/iZ5O0HDE19i"
    "YrzE9OXlvxOJDeskfwi68ec7N0s/2HW43qoMO6yPjlPYlWJ6boYddU24niKXy1MsltA0iabp"
    "jI5eZG11jdmPHObLklteihO/nM57ruogHCpqaBqY+oZTDr7+69dmypOFFHR8kb+NLFJYgUKh"
    "iJQaN26MMzZ2GcOQ5PNFyqUQ7793E5WoYs5o4levzbrlnHecsFxC0xC0jm551CQo84CMRge/"
    "87106vtfE0TsFZaXVjB0g+zKf/Bch7q6VhzHoTpVSVCT5DdnBb97a9HeWCv2oW8Oodx7P8g2"
    "WEgwIiDMGI482fhIvPO5Z6Ohjv2CuFYmHgEhIFeCogozMgF/PLPh3RrLXUR63yLYuItbhuCz"
    "wDIMgQRPPgb6qyIun8qkjWQiLkIIKBQCd2HRXfFt7wPwX0f6wwgP3DKfBP8X/saHLvK6UQgA"
    "AAAASUVORK5CYII=")
index.append('clock_menu_icon22x22')
catalog['clock_menu_icon22x22'] = clock_menu_icon22x22

#----------------------------------------------------------------------
clock_stop_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9sCDBQXNlUtsDcAAAUV"
    "SURBVDjLjZXbb1THHcc/5743r714b8Y26xWpMSROUhLHUMzFSECIiXkwqpRI9CJFShS1L5Ha"
    "qk99itT+AyWRkhekSFGUiwoFNfjSAobYJjEp3mAvsQteg722WV/3nPWuzznTBxMjUoM60mhG"
    "M5qPvvP9/eY3Eo9rB88n0KU3Q0Hjleoqf21FUPdLwGJ+1bo3ZU7mFoqdrDrv0902stFxaUPo"
    "/rOnm34a6Wg7sMVXv8VPQFWRHAFICAUsx2H0nsU/Lk+s9PZlzyPEa1xqLz0e3PRZsqLS0/PW"
    "Lxvrdj0XY2FiAfLzeIDZbAbHXiVes5UVAcJXTihRyfXhWU6dHsrOTJrH+Lrjmx9Qyjq08eP6"
    "zTHfN3/8fUs85PMweLaLhrDFwT1Jnm+M4TpZyoMObS+/SCKmYE7doevv/VT/ZButrXWBkVT2"
    "5Lxy7Dyzn2Yfgrd/pG4q06795nf7YwszRZKr07Q26TgsUb9tB47rUlYWJBKtQtMMvD4/09Oj"
    "PNdQjrzskpoQ7Dn0lJYaGO8wfe2nuP95ac2Kmg9O/+KdAydD8Ti7tQxthzcjZANZklBVGUWR"
    "UdU1Dbbt4Loutu3iugLJLdF98R5d85spFZb48M/dnfadXx+W2fReYvvT8ROBRILZvh62ROYQ"
    "ko7Pa+D16hiGtg4FUFUFXdfweHQURUbIBtuSJYqpbqRwnOebalup+OsLiqQc+cNLHc2t1orM"
    "G80rTN2fJZUaIxQKEAqVPzYbJUlCUWQAbqaHaagN8q90gEA0II/13jLUcp/Wlvdvol7MsXfP"
    "NoS+k3R6jAsXviIc3sS+fTsJh0MPia4D8toNZFkCBLt/1opUynP25l1uGNWEK72tSizx83ed"
    "7Y3eA5El9jfHcFyZ2toYDQ1JZmbmuHr1OoXCChWVIdSRYZyhIab95YyOZrh65TqrqyWqqqLo"
    "usbE+CL/nKpAm8yoqu73+ReWJQqLGfr7p0lu3Y7HU4nf7+PQod3MzNRzsfdb0l98ifXdEMFY"
    "NcrsKgGvQSIRJ5msYXl5mfTIDUr5EvlCDYbHo6uuZCDJ4NolLNPGdZ1HvIxGK+k43MyZjz7n"
    "38UV7G+vUFdXg6Qo3B2Cu4BlFpjOTjKdc0kudLOUD0tqwcKK+oVRVf0UBw9uxSo6OI77MBNK"
    "JdwzZzi6ZyelzC2OLoFmmv8bzUgEIuBa8xz/27hQl2ZLU/WaHZo0JZYXFzECFQ9UKyAE9rlz"
    "2LW1eJ55Bo+hY2WzkM+DtEGZEQKlWGTV1Apq3uSCmzN3LEWCDA6OgWbR1NSMpmu4vb3Yfj/G"
    "3r3rhWV5ZARX1zesXgLQHAfZqhtXUfT3xwayb+9qeVb/8vJl2g/HcQwPxaEUYm4O43j7OkQC"
    "zNu3sYXYWDGgCUFUdgZkFl8duXdz8VxqcIG6lkMsFWLIo2lK348ijhzFsV0cx33kuuLB+OMu"
    "hFiz2/1PZu3p+Hyvd31yd+bWvJec8NLfleZWLEbftT5cG4pFh1xuAdO0+H/bGnhi10qpINrf"
    "+0umkM5HoWUfl/qnyecgnzdRVZnh4RRTU5NP+B0eXVfXZ5md/aZ2c9e7f5rqvP2rWPSNV4/i"
    "XcxxbWAcTdGYza2QX3YJAu4TlGoP9tVHVsd23LB33N56+tT9T3sulbWeOBbRWxqjlMkFEtXV"
    "5K70EHmC4h8scB8bWoCG7Aug/FYqU/fHY1okWCbpwft9Utl4pzDzRUu3cploYXAgXExlfmSH"
    "5ELnfwEbrTB4GQV/HAAAAABJRU5ErkJggg==")
index.append('clock_stop_icon22x22')
catalog['clock_stop_icon22x22'] = clock_stop_icon22x22

#----------------------------------------------------------------------
person_talking_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAO+SURBVHjaYqzz5GcAAXZOAYZv"
    "X34zfPzwmeHH738ML75+ZTjzOpbh5c8QBhHxvw3eAVKpBgbCoh8//Pm7fdv5uyf3rKv7/3fX"
    "OoAAYmGAgr9//gDJ/16Kmlrh/xkYON6cubRNinXxwv+K6bXRBep1SUEijNxAFf8YGVg1vTS0"
    "W+r5l15aeSICIIBY/v37x/Dv7x8GLj7uBvfUsipVIytW5r/PGBxP7QxuaZ3nwqz23kLJQoTx"
    "6E0GBhZGoAH/GBhAJrlEe3Lc3adfABBALOwcHAy/f/2wcQpPqjL1SWZlYPjGwPD5J4ORmz1z"
    "1Y+30dXrTnxh/uTGICnExsDOBHQC0BBWoAHXz71j+PPj1T+AAGJSN9Bl0DQyiNIw0GJl+HGH"
    "geEDEH95ycDw9g2DibEKo+D7vb8OrLv9kw+oifkv0JO/GRgO7rzLsLKj8s3PD2f7AAIAQQC+"
    "/wQPDAcA/f8DDQL77AD76dcA/Pr4AP/68wAA/fgAAvzzAAoMAgWxubvn3dCrCvTq5fD8/f7/"
    "4t3L2Pry04wAAADEAohl87mLDH8//tqnoqgUxa3iy8QjbcPAysLG8Pz2BYb3Z7cw3Lhx6eG7"
    "Oxei3v3/9paBQQSIfwL1ff4FC3yAAGKZ8vUZA8NLhkNiF8/+jOMU4vzFygl0KyvDu6srGTY8"
    "uMJw/NWXIwz/Ga5AlHMAMRcQfwXFB1gEIICYGZSZRPwDg3dZ+sZJ/WB4z/B+URfD251LGS6b"
    "/GNgdfBnkBDXVLt09Pwxhj//H4IDmIGZAeIKiAEAAcSgkKY288CD7f+/AOG1v2/+326v+X87"
    "N+P/0d8X/r/5//L/lU9n/5sFm92EWg2JBjCGAIAAYnZJsJ9nb6HH/f/PXwZhZg4GDgMTBjYH"
    "GwZJdhGGHz8/MnxjfcPw6vtrwdPbzl/7/+//FQY0ABBALALibIKvfz5i+PzvKwP3j0cMrOw8"
    "DEzMTAy/P3xn+P7vG8Nvtk8MXJIMTCw8rFa/Pvxcjm4AQACxvP32lvHFv9sMzN9eM3AwczGw"
    "fmcHOpARGLV/GH78/c7AwPSV4fXPF8Ck/o+XAQsACCCWczsePlEyZZYXFeBlYP3DzcD6nw2Y"
    "WoAGMP1h+MP2neHTl28Mp3bc/v/3y5+L2AwACCBGYEoJkPMQna/hJiggKMHOwM7BAhb+/fsP"
    "w6dXvxhuHfrw786aJyf+f/3hD5R4g24AQAAxMrADA/fnLx0GLtZkDl5mVSZWJlCmY/z35//f"
    "n1/+fvz/5cdRYJTNB4q9w+YCgAADAKHTfmLbzkoGAAAAAElFTkSuQmCC")
index.append('person_talking_icon16x16')
catalog['person_talking_icon16x16'] = person_talking_icon16x16

#----------------------------------------------------------------------
book_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANNSURBVHjaYvz//z8DCAj6rmXg"
    "4ORgeHH1EQOfrBADN5D969cfZTklsTpJob+eR+YVTvj05nkbAxMzA8P/v2A9/7+9YAAIIBYG"
    "ZPAPaNivvwzfP/8yVlaVqLEwlnTz9VDhYvn1nuHufuu4T3z8FxieLN/D8OvLLwZGZrAWgABi"
    "hLmA220lAzcft7MQL2uDr6+mubWRBKugABvDr98MDF8+/WI4du4Fw7qjt//cm5Nax/D1fjsD"
    "EwvQIb8ZAAII7gIxWaEjEUG6VtZG4ozcHIwMX74yMLx4C3QQ0Pzf/9gYzNWEGb7/+M4ydZW8"
    "K9iAf3/A+gACiAlmgJaBtJ6NmQTjr1+MDI/fMDC8+cbA8AlkOxB/Bqp99/Mfg6woLwOnjIYq"
    "ULkQAwPECwABBDfg3onDd+68Amr6xcDwFajh828IBvFBBnz4y8wgwcvKIK2gLALUrMXAwAHW"
    "BxBAcANePXx54cX7Xwxf/gE1/ATiXxDbQfgrEL/4xcwACi01FRU2BgYBfaAoWB9AAMEN+Pry"
    "3bHHD5/8AxsAtPELkitAhn38zwoUY2JQlpdgYuCXtoTpAwgguAF/fr0/8+L+i28wP4P8/wmq"
    "GeSiT3+ZGJ59Y2GQkRRjYBZX0AVqYQPpAwgguAH/BRVvvXv57sXn70DXANPJl19Q26Eu+Qp0"
    "2ZMfwOjm4GCQktGQBWpRAOkDCCC4Af8eXPv2/t7tW1++/GT48hcaBtBwABn4GWjA7V/swGhl"
    "YVBTVeVlYOAxBukDCCC4AQwftzO8u33s/Ke3X/+DXQDEXxmB0Qey+dM/hvcvvjDwf3vLIMn1"
    "i0FBU42FgUPCAaQNIIDgCYlN0pXh2w/u019fvPzDKCXE+uwj0IDX7xlk/7xnMGD5AYz4Xwxc"
    "LP8Y9l3+w7D3/GMGBlZmcZA+gACCG/CHS5bh34+vB6+fv/mZk0VESOHCLgbvIxMZ9up4Mpzl"
    "kv337tGdn5+e3n798+mtOwzfHp8FemotSB9AADGA8gI4PwgDXSRkz8Agbl/AYJjyKFXH4cc3"
    "aYb/rmx8zxkYuGIYGJgMGMCpB+RrZrhegABCGCDixMAg6srAwC7KwMDCzWDFzJBax83wWIWJ"
    "YRkDAyMDBEMdzcgGNwAgwADqaV1C3COCHgAAAABJRU5ErkJggg==")
index.append('book_icon16x16')
catalog['book_icon16x16'] = book_icon16x16

#----------------------------------------------------------------------
clock_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAZQSURBVHjaYvz//z8DLQBAALGA"
    "CEbGOVDuJyD+xsDFz8fwm5FJSFRB2FXXRNxFy0DEQFScV4aRmZnp7afvL2/ceXH59NGjR169"
    "eLiF4dfnxwyvTzH8//GVgeEPUP+/b2CTAAKIEeRiVIM/MfCJiAaFp5lUBISrmYoK8jD8+PKD"
    "4dePX2AVrGwsDBw83AxfGJgZNu8+e2vh0u3T3x+aPAXI/cPw9yvcYIAAQjP4HZO8qlB9w1Sf"
    "KnkVIZYnN14yMHz+yCAjzsnw9/cXht9//jBw8wgyPH7+neEPKxeDtIY8w7s/fxmaWmauuLqq"
    "OZeB4cMbhv8QBwAEENTg2UDmXwZZFb7GzqV+dT9+/GH4cvs+g478bwZjEw0GVnZWho8fPjL8"
    "+fuXQURYhOEv0ILz568zXL7DwMAkqcAgpibCUFU1efONVSVxDAy/PoAMBgggiMFsKxm4eP8F"
    "1s9xX83Eycos/uERg4+LBMMfYLxycHAxMDMzM7CwMIGUM/wFGv7373+G79+/MnCyMjLsPPCU"
    "4foPUQZBJXGG2rT4nvcXF5WCDAYIICawu/8x8btGGdT+5OFl/nL9GkOAmyQDN58AAx8Qc3Gx"
    "M3BysjGwAcOWjY0ZaBEbkM/KwA+M4P/MHAyezrIMgt+eMbx4/pPBO7Uii4FD3AlkJEAAgQ3m"
    "kxXyUrCS139y8xWDluhrhm27TzA8ffqCgZ2dGehaJmBQIZIRiA0SY2VlAcqzMfz49Y/BQvsv"
    "w9t7TxhE5TS5RPT9EkGpDSCAwAYrGoo7P/7CyqTA9oEhINCOQVVNkeHw4TMM27YdYnj37iPW"
    "dAqyAOwDTi4GPT11BgOJXwwPn/9nUDHwsAWaqw4QQGCD2cUF9J+8+MZgpMIBFGNnMDTQZAgJ"
    "cWPg5uZi2LBhL8PxY+cZfgDDleH2LYY/Z04zvH7/ieH06csMu3YdY/jx4wcDMzsng54CO9CX"
    "XxnYhOWEGFh5VQACCJxBfrBxy/788puB+f8XhvfvPzAICAoDw5aTwd7ehEFLS4nh2MlLDEuW"
    "b2L4c+0yA7+ULAPzww8MbMDgUFKSBqsD+Yqd5SvD95+/GH4ycHOwsnMKAQQQ2OA//5kZ///7"
    "y/D/LzCNAzEjUqCKigox+LtYMOxcsobhsrQsg66RLoOpsQ4wGNjA8qAU8uvXb2AS/A3m//wN"
    "DiVWgAACG/z/9+8XgiIsYkCLGISFhcFJ6v9/ZkikAdPs782bGZwsDRnsVdWALgOKMzHBLf79"
    "+zeDiIgQw6vnPxh4gKnly7evf35///YLIIDAKn68/XCVX4Cb4fajb1BX/2P49/8fWOOf/fsZ"
    "/klLM7BqazNwsLGiGPrv338w/g90yP1nPxn4hLgYvj+78/X/38+vAQIIrOrlxfVH2IG2PfnB"
    "yXDl4g2G78AC5S/QgD9nzoBpNisrjFQBylh/gNkZFHnXr1xjuPmKhYGTm5nh5YWdD4EBdAcg"
    "gMAGf7u3cNPDy8/u/RCRYrh1HxhAnJwMv6/eYPj15i0Di6MjAyOaoSBX/v79j+EnMEBZgTny"
    "wWOgGcLiDC8uXvj/6e7mnUAl9wECCJrz3jy5vm3t5IcfOBi+iSoxnN9xmeHPgzsMX42MGX7+"
    "ArnqN9B1/8D4B7CUA+GvX78ALfjDcOHMU4YPnNIMz7+xM5xf3nSJgeHteqCJvwACiAlaFDH8"
    "vts6dcv0NStvMQgxfBGUYDj0RYzh+u0nDL9//gDK/2N4+/Ydw6tXr8AR+u/fP4br1+4wHN53"
    "i+EDCx/DIw4Jhg2dXW9/vF4/DWjaOZCRAAEEKYSYxRnAxd1/IVEes7a5CdXhvn7q3xmYnzxl"
    "YPr/m4GLm53hN7CsBSVFbk4Bhs+ffjD8Z2Bh+CcpwbD3PhfDzKq+d+8u1/UBk8FEoJlfQAYD"
    "BBDCYFBIMgJz3t9PfAxSSXWWMUVpQT5SvCYSwBLu5ycG1n+QcvYPI7B8YOdhuPaOiWH1mmsM"
    "Bxe0Xfj3dtl0oKuWA6U/w+IBIICQDAYVSfxAna+BDFCRqunEIO+ToGjsZaOhJSYiKs7BAVTM"
    "+Ob11z+3Lz/6cufMjof/nqwFRtSLDUDFZ0ExhRzBAAGEZjAf0MXvgZaDfAN0PcNHZgYGLk0G"
    "Bn4lYIkiDJQAZqjvQKe/fwPMGndAsQ+KKGyFFECAAQBCzni+4gHlCwAAAABJRU5ErkJggg==")
index.append('clock_icon22x22')
catalog['clock_icon22x22'] = clock_icon22x22

#----------------------------------------------------------------------
folder_purple_open_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAH+SURBVHjaYmRgYGAKEWn/y4AG"
    "1ryp1AZSt4H4NwMeABBALCDNXavzMWVCGa4CDXEGsk4A8S8g/g/E/6A0HAAEEAuIeHX9F4Z+"
    "sKGhDHsJuQoggMAG/PkBMfQ/0HxGJgj99OxPBgyXobrqO0gLQACBDfj27i/DP6CZTKwQdbe2"
    "f2M48HApVj8DvQx21dNfV4uOf1oyEyCAwAZ8f/sPruD9g99gzY2zcxk4BZhwBl5Z6MQ+IHUN"
    "IIAgBnxARMKL698g9KWfDH9//wd75x+Q/gdUcn3jVwY1Dy5kc/QAAghswM9PEBd8ePiH4cyX"
    "NQwpyalgMZAmkOa/f4AGAdl//v5m+PHhH8OqnfNBAboBqOUxQACBDQAJgsC7T28YYAb+A2oC"
    "uQAUNv/+QVzy5/9Phl9fuWG2PwLiQwABBPHC+38MX75+Zrj6bRdDmHsi2Ev/oTH+9xdQ839I"
    "zPwDEhuOLITZfhyUAgACCGzAu8+vGK593wOx/cs/iGZQqoE6He4FoAuQbD8MSgEAAQQ2AKRZ"
    "l8uTQclQjOHn539gm8E2Ap0OSnvgsAAaAlIHtR2UDsB+AQggRiDWAsbtVQYiAdCAfiDVC8TP"
    "QJ4ECCCQAaDkYwjEoBQmRED/FyA+AMRHQc4HCQAEECNUAkSzQw3DB/5CM9YfmABAADH+//+f"
    "gRIAEGAAEvwAiCqSpnQAAAAASUVORK5CYII=")
index.append('folder_purple_open_icon16x16')
catalog['folder_purple_open_icon16x16'] = folder_purple_open_icon16x16

#----------------------------------------------------------------------
person_talking_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAXgSURBVHjaYvz//z8DLQBAADFW"
    "u/NCGIxMDMzMnAx/fjEwMLFxMXz+9IXhHwMjw7c//xmO3X3F8PmfPcPj390MjD/vMHMLMnmp"
    "a/FF6pmIGrKys3I9uPX5+Zljh/a/e3JnAcO/ZTcZGP4wAAQQC4otQPjv32+Gf79+Mv75/Yv3"
    "LwMTw+/f/z//+c3wn4GFGaiekUFaS6SztEE/P9hNlIWPnRGogoHhx18GudO3bcyrmk9Gn19x"
    "NZfh36mNAAGEYvAfoHOZmFldlfVNU2RU1DT//fnOeOPShevXXxye9/X7xx280v/DfUssi3jU"
    "eRgvPAFq+MfAAArJ30DmJyB2zjCXffu6pvPR7sibAAEENxjoQgYBYbEs78yGfh07fzYGpu8M"
    "DN+fMbh/eqjj4mIYlF8xOfuB6Ek1JTVTxldvgK5kZWBgYwH5EqgXaMHvvwwMcgIMDFpWqsqP"
    "dotYAAQQ2OB///4ysLGxWXun5PfoOISzMfwHqvz2AYh/AnX9Z9C2t2RuKX/alDDl0lmmbwy/"
    "VKQY2P7+YGBgZwaHH8M/oKuZQCHFBnT5kzdAmQ9/AQKIiZn5D1DuJ4OCumq0tokJJ8OX+wwM"
    "X58BnQD03D+gq38D1X38wmBubSpmLrKfc0HbgkO/PjL8V5RgYFAQZ2BQAmIQm5ebgWHHhhcM"
    "J9e1HQZqOAUQQCw///wBhu1fBgk5aVVGBqBB34CGMgE98g+YPH4DXf3vC8Pfvz+BrmJkUJf7"
    "L7hk06U9124yOH76+Iv5+ZNPDLw8HAxPX7z7d2jXoefnd/bv/P/93HSgP24CBBDLf4a/DKCU"
    "/Ovn598MP14xfP/4kwFoDzBWgOK/vwOTH9DFTMD08ecFw+ffHFwMfExsMzqKV3y4d+T13y8v"
    "7gADBBgYQE0Mr28BNV0E6vwACl6AAGJ5/uE/AzDeGE6dvnHCz+ue5/3XYgyCqo4M3OzAhARM"
    "239+MTM8OL2Ggf/rNYbL1x48//egt/8tA8NzkNVA/BNXBgEIIOZHQAfdeM/AcPHB57saCn/C"
    "ZYUE+H6ySDMISKgwsLDzMDy/fYXh243lDLvPnPo5b/eH6UDfbAMlBCD+iy/nAQQQywdbcHpk"
    "EpZTT9nBJyIs/ukOA8ujBoZn16SB8faV4e+TSwzvxf8ynDTTZBd8xKv5fMddDlCeIJSlAQKI"
    "kcGdgUFTVbtycuGEtn9srAysXH8Zrm2bx2C2cCMo1zEczHRh4HRVYBD/aMHA+5ePITctb8L9"
    "/XdLCLkYIIAYGHw5zCbu63tz89O5/99+f/7/6Mfz/5d/PP3/IiTs/8OIxP8nf975f/375f9f"
    "f37+f/fblf8927u/M/AzhBNyMUAAMWu46ZSEeTs6sQHLAhZgMuNkZGUQZxdg+GVhxsDi5cgg"
    "zy/OwPWfk+Hr/w8Mr749YeAUZmY5efYyz/u7bzeBEhMugwECiEVWWtz+D/tHhq9/uRiefr/H"
    "wMnMxcD6k52BRYILnF3ff3nD8Pv/L4af/74xfP/3ieE32zsGJV0Zzbs7b6kApc/jMhgggFhY"
    "ef/Lfv77iuEvMLNzMPMwcLBwMrAwsjMw/2IBG/wPmL1/AzPLz/8/GH79+8rwmektAyPvf36g"
    "lAS+oAAIIJY3bz/+eP/3BcOX398Y2Jm4Gdj/cDCwMgLLIEaEwX/+/wYa+gPs6l8cXxjefXgP"
    "0suEz2CAAGK5e/r90fu+j8LFBHkZGH9wM7AxAg1mYgXqghR8/xghBv8GGszA8YPh+avPDHeO"
    "vwaWbwyv8RkMEEDM35/+fPTyG7MXr9IfPgaerww/Wb4y/GL+wvCD6RPDd6aPDN+YPzB8Y3zH"
    "8Onve4ZHzz4w7Jj45O/L3U+XALP8Gnw5DyCAgJUG0MNM7JYcasLl4rrcpoKyLDxcQsxsTMxM"
    "TOAi8c+//9/e/v397uHvHy8vf37y89bbjcDSaQFQ7wN8LgYIIEZwlfMXXBVwMTAwawBtkQYW"
    "EqCKkA1S2oLrCaDLgE5m+PcIyL+Hz6UwABBgACT1Ydu5ezpnAAAAAElFTkSuQmCC")
index.append('person_talking_icon22x22')
catalog['person_talking_icon22x22'] = person_talking_icon22x22

#----------------------------------------------------------------------
house_green_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAP3SURBVHjaYmTwk2IAg/+MLAx/"
    "mZkY/jP9Znj/7j/Dp1+KSprma1gZWDlvXjgTxsDKfoWBhZWBgZGRgYGNnYHhx08GBiZmBoAA"
    "YoLoZpAH4olAPB9okhrDj69yKup667Z3LTLa3bdMU8/QbDXDj2/GQDkGdAAQQMwMGjxAUxmM"
    "gXgKAxOTLsPHt6yGWuY5GzpXmnySusHwhuM+Q45zocjJ21e9Ht+7fRrokkcMzMwMDH/+AvUx"
    "MQAEEDODNjfQKf9lgU4LYvj8jsNMw9xwbdMyuTfiVxn67tcx7Hu9g0FGWIoh37mQ7+ydW94P"
    "7t48wcAONOQvxACAAAIaAPTP/39iDJ/fxFhr23GuaVjG9FT4PMPEe40Mf/8wADEjw54XWxnE"
    "BUQZit2KeC7duxdw7+7100ALH4DCACCAmBk0vzMwfPzuZGPsHLC0fB7rXd6TDNMetII1/gPi"
    "X7//MvwBGrTj6VYGXi4uhgqfYs7rDx9737l26RowMG8DBBAwDBj8vSx8Jy0pXij4hPcyw5SH"
    "QJv/MoE1//j9C4x///3P8Avo5+1PtjJI8okz1PpWct199dLn+u1bjwACiMVa327mnNzZ4hIC"
    "Ygx3PzGANX/7+Z3h959/QM2/Gb7/+sXwHUT//MHw7x8DA/MPDgYhPj6GWQ0dfB9+fp8EEABB"
    "AL7/BN/u4c5RiFIqOyI9GGBaUwcTARgAYxtfABMHFwD8/vwA/f7+AP7+/wDo8ugAitaJABwO"
    "HwCElpD6sse46Of1+NsCAEEAvv8E5u3jSubk4RMC/AS2Xv1UNg4EEgAsBzYAuMK/AOny6gAB"
    "/wIAEgkSAEs9RQAWCKEA/v8BAMfVydjLxbtHABYg6QKIBZgCGViEWN/+Y/37ifE3EwMLExvD"
    "yy8vGeqW125//uLRNIZ/jP95eMSS9HS0gnhYeBn4/vIwfPv/+zODANsLYPQzAAQQCxMLM8M/"
    "3n8M736+/fHn1z8Gtv9c4ATHy8/z6MV7pi0MwBTLx8fn+B8oyPGfk4H1LyvD5x8ffjIw/wel"
    "HwaAABTSQQrAIAxE0UmaUpfF+59RLYgEweh0O7v5PIuYVAW0Ub7hDkPi5/iNPFtYjdjlQgrC"
    "UY7OqHX2rvnFbQtHALHAkvevP7+ABnxj4OMUYPgPEmNiBGsGG8D0D6jsH4MImzDDz5+/GT5/"
    "ff/x388vDD+BrgEIIFhmYvjx5/vrv8C4TlHMZpDll2X4x/SHCZg/wAb8BbIlBWQYqk1rGNiB"
    "YfTt19vPDP+BfgNigAACuRjs55+MP4907+r+uvvSHrbXX98wvvjx6jsDH1D3T1aGD3++fq9d"
    "Xv9Hml/y/+VHt/5+Y2E4xMDLC87aAAEGAApOlafzPRriAAAAAElFTkSuQmCC")
index.append('house_green_icon16x16')
catalog['house_green_icon16x16'] = house_green_icon16x16

#----------------------------------------------------------------------
cogwheels_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAM7SURBVHjaYvz//z8DJQAggFhA"
    "BCMjI1bJxsaZTGJiokVbt659aGVlmaCvbyQ3YUJP0O7d6x4BpX+C1AAEEAM+F0REpFfevfv6"
    "//XrD3+/ffvp7+/ff/43N0/eXFPT3W9qaisGcgBAAOE0ICmpSLu5ecr9Bw/e/n/37vv/+/ff"
    "/P/06cd/EHj8+NX/iIiUSqAyEYAAYkLX2No6Hyz2+fP7jtjYWAVOTnaG/v7J993c7LdPmzb/"
    "079/DAzPn7/4eefONZD3hQECCG5ATc1kpkmTlldqaiqf9fAIqdDXN3G7cuUGw7dvPxhYWVm+"
    "3r59dUlXV928Z8/eMZia6rKXlVWXArXxAQQQC8yADRsWuwNtaDMx0WBwcbE0ePnyCcOZMzcY"
    "fv+WZwgODtERExNr5eBgY+Lm5mS4ceMJw61btz4BtQkCBBDYgISEfBkmJsYSUGT8/PmT4e9f"
    "RoYHD14xfPr0meHy5ZsMkpKiDAEBAQpMTEwMzMysDNu2bf3c398yB6j1L0AAgQ148OBOxZQp"
    "s52EhbkZXr58x1BXV3v99euXvz08/HT5+NgYHzz4xvDly9d/hw8f+GpsbMa7efOaI2/fvr0C"
    "1PoUIIDABqioaFg9e/aagYnpH9DP3xjOnz+55fbtaycfPrwbtHnzziggG2jwQ4YtW1Yf7+mp"
    "v/r9+7fjIHuB+AVAADE3NDQwxMaGPWBiYjU1NTUXERDgZRAXl1X8+/ffc1EgCA0N1TYy0mMQ"
    "EZFk3L171/GbN69uBGq8DcSPQZEFEEBgF3z58vmKtLSMIDBNAqPvG4Ozs5OUjY1VjaioMAM3"
    "NwfDyZNXGFauXPH28uWz54DKXwLxE5A2IP4HEECwpPzr5s3Lj86eVRb79+/PfwMDXUYhIQGG"
    "168/MPz6xQ301leG1avnL3rx4jnI5jdA/B2kGaQXIIBgKZGRmZnZkI2NPdXExPLA+fOP/69Z"
    "s+dvfHzmi6VLt/13dfU5BZSPA6rTA2Ju5IQHEECMIAOgmYkDFK+cnFyuNjbOmV+/fvxw/Pjh"
    "bVxc3Ipfv345CpR7DsS3gPgd3HYgAAggZANAgBlqkDgU/wUFFBD/AmJQwvkAxL+RXQAQYAAJ"
    "WlogZK/PkAAAAABJRU5ErkJggg==")
index.append('cogwheels_icon16x16')
catalog['cogwheels_icon16x16'] = cogwheels_icon16x16

#----------------------------------------------------------------------
windows22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQ/SURBVHjaYvz//z8DLQBAADEx"
    "0AgABBALiHD26t9mZK9j8fnLv5+//zEw/PnLwACiYexfQPwXiP8A+b+gYn+ZmJnfPX3z/e7O"
    "itz/fx9uApnDwcHJYGtrw3D48GEGgAACG2xgq2so7Oki+PXxfwZ2YMgwAjX+B+J/QEP+/2Zg"
    "YAaxgZjhL1TuD5DNysgg9OIZw4P9HM5/vjOADVZWVmZgZIQEAkAAgQ1++vLXD6bLbxm4fn1n"
    "YGJkBEvAgh7Eg8cCiMMM5DGDXMzKcPDNR4Y/v//8EeTnYjA2tWYQFBRi+PDhHVgpQACBDf75"
    "+y9DqhMPg5qkELIxaOA/xLb//8Dsf/+YGDxvv2NgZefkNzLUV+Xl5WH4+fM78+/fv74CFTwG"
    "CCCwwX8YmYFhxwR1EiOmkUADQeb9BeE/EP5vYDj9/sfIEJOYGmyixmr37ScjAysrG/PNm5df"
    "Hj9+IhMggCAGMzAx4Ep2IPF/QEP+/fvP8OcPhP0fGODgCARqERNXFJCR4hH4+P0vAw83D8OX"
    "L9+UgNqMAQIIYvB/RgZs5kIM/Q9MERD6H8hAoIkgg8GpBKiPl4+NQViMg4H7FwNY3efPn34x"
    "MTExAAQQ1GBUF8PYMJeC6L9/EfgfEP8BGvIXKGdsqsdgbSkJVv/p0zeG/fv3g/T/BwggpKBA"
    "dS3EwP9Qw0DB8A/u4n9A54LC+x8wPn79+g1SzfD9x1+GHz9+geMIZBZAAMFd/O8/qvdhQQDB"
    "f8EYZCiM/RcYcX//QfUBE/1fkIVQQ0ApFiCAwAb/BqUEqMGQ8IR4+Q/I20A2zMA/QPz7N9QQ"
    "YPiCEt5fUC5CshSW4AECCOJioM0cLJBkxsICTXJgXUDXAlPCHyYozQhMYkCJP8As+B+Yw5gY"
    "GKFJ8C8k7EGWQJMsQACBDWbi4GY7cP0Hw60n34EOB6XZ/8Cw+wN0xR9wBP1FCm94ZAId8/wT"
    "I1QcGvZAcWjGZQAIILDB718/ezltB6fUty8/geHNAPYmIzMLAzMzKzgswWL/IGUFOJLBZQgj"
    "AwvjdwYhHl6GHz//QYIIlLChGQwggMAGP9iQmwE0yQmonBkY5T+5ebilSosqs929nNi4uDnB"
    "yQuSpYEu/QML938MrMxsDBLCXAzvP/+EuxyWVAECCGwww/eHp4DkKViZw80toq0owZ5hpC7E"
    "wAYsClFzzT9IcgNiUHB9+QYM9X9/gEHwH+hDZmiqYGQACCAWlFKfiem/iYktg6amHtOf/yzs"
    "V68/BOZ/Fkg2/s8A9So0OYIt+A9PnpCkCgweFg5mYNHJBBBAKAbLyCgySErKMrx79/brjRvX"
    "H929e1/m16+ff2HhBi6M/kMMBxkCoSHpFiTOysrO9PLl0x9AH/wGCCBGkAAjNCp1dIzB7F+/"
    "fjE9fnzPnImJWQdeTmItoBjgqQCafpn+/v39+8ePb8cBAoiRVpUpQIABACBymTxM7pXbAAAA"
    "AElFTkSuQmCC")
index.append('windows22x22')
catalog['windows22x22'] = windows22x22

#----------------------------------------------------------------------
clock_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQaSURBVHjaYmRg6GRgYPjPICLB"
    "y/Dl2x9xOU2xcitXJW91HWEZfkEu9h+///19+PTth6NHj5++duvRxG/nV+1m+PkcqOcXEDMy"
    "AAQQIwNDO5Dxj4FPgNvNN9ZwnmewprQINyMDx//fDL9+fmNgYWFj+MfCxfCZiYPh+LW7Pxb3"
    "ds5+fmFNHgPDX6A+ZgaAAGJmYIph4ORmd4vMsVjjGaolwv7lFYM0918GOXFOBk7WXwxigpwM"
    "3Mx/GN4/ecogJi7HombnZnb9+h2Vz8+vrgcZABBAzEyMEeLO0brb7fw1RBRZPjOYqPMwSEkJ"
    "MAgI8TEICvEz8PFzM/DxcTBIiLAxMH/9xPDxpwAjv7qZzpXTx3/++fL4KEAAMUmoiZar2qtI"
    "fb5/nUGY/S2DsLgIAxcPNwMzMxPQ+cxgmo2NjYFfUJhBR0uEQeLvfQZePgUmXdeYYqALpAEC"
    "iEnCWN73z58/DLYavAwfv3xh2LfvBMPTpy8ZkAEjMKSYmRkZ/jOxMpjqCDDw/v7IwKfkIsIh"
    "IBMDEABBAL7/BPoRDhsF/fcALSkXAAkNIgDe3uQA0NDYAOHm6wAmGRIAA+PiAO4SFQDm6/EA"
    "GxoWACIgFAAbDe8A8f0VAQkTHjwCAEEAvv8E8/8IBvz58QD08vgADBAPACcnIQAeHhkA1Nbe"
    "AKimsAAKBQQAJjUzAA8ODQAUFBAA9/PzAAH+8wDx+BIA/wQMEQKIhfE/418ONnZmTk42hv//"
    "/jEwMDEzcHKyM5gwfGfgtTBgYFVSYlCWkYB758+3vwysLJwMQI0Mf37+/A8QQCwsPz9//MvE"
    "Ifrl6x8GHqG/DEzAqPl36TLDv9+/GTTsrFDC4vfvPwx/fv1j+PyHi4Hj3zuG3++fvAIIIKZ3"
    "l9ee+sAowHDhxhuG7z9/MPy5f5/h3zdgAjIyQtH89+8/hr9/fjNcu/mU4dFXDoYv9479/fnx"
    "4V6AAGJ6f3nTxFu33/x6yqrI8OriA4Zfz58y/NXWYfgPjDoQ+PfvP8MvoM0/f/5k+PDmE8PT"
    "n2IMD15+ZHh8cM49BoafKwACAEEAvv8DAPzy3uri6hYNLUc2GxEDAfX79wACDAkAAwEBAO7u"
    "7wD+/f0A+Pb4AN7Z4QDn9QsAx+sRAKuSf/jx3qt0AAAAyQKImYGBk+Hvp6vbb1x6ovJDzkJH"
    "XFWDkQWYB75//Mbw8cMnhq9f/jB8+MrM8PyPGMPO678Z5nU1/Xh5fkITUPM0kAsBAghoAC+Q"
    "Ymf49/Hi+mtnXv68fP+D/ksGKa4P//gY3/7lZrj1lpXh0PXvDEvX7f27dnLV3c/3lhcBNUyH"
    "hQ1AAAHTmBiQYgVGHwsDA585A8OHA7IMrAJRHCKK1hwcnEJ/fv/49+Xtk9cM32/vBfkZqPgd"
    "cuACBBgABlp1RgH8JYEAAAAASUVORK5CYII=")
index.append('clock_icon16x16')
catalog['clock_icon16x16'] = clock_icon16x16

#----------------------------------------------------------------------
box_in_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAGGSURBVHjaYvz//z8DJQAggFiw"
    "CYpWiBq/EXyzGVlM5L2I7+uO12fR1QIEEAPIBeiYoYPh2dv/x/+9/H/674O/F/7d+3P7P0M2"
    "w2tsagECiAWf83YzxP649UuU5cZXPjZcagACiImBQgAQQEzofmfsZHyGSzFjDuNrkBpkMYAA"
    "QvECKOA+lx+QvMDg+e8Nw1FGZDn9Eg+Rx1+/MryZfngHyC6YOEAAMaGHNG+nw3Ndhp1Mtxh6"
    "MVzwbvrhNyI8Ih7IYgABhGIAKJpAhgh02j23YliHkkAu9uwAa0aPSoAAwhqNIuUixqCoBEVj"
    "3Xfbn6AoBIlhUwsQQIy4UiJyYsKZiIAAIIDABsxKN4GbkjbzDCO+aOsMU/7/9+9fBhCu3fCY"
    "ESCA4LHgl5nNsGn6VLBhIEP//fuHgkEaQDQIeHhKMmzd8gTMBggglGj0Tk5k+Pv7K8O/X1+A"
    "9Dcw/gfFf//8YPgPxP/+/mT49+cXXA9AAMENANmOHLC4XABzPgwABBAjpdkZIMAAN4MMGTCJ"
    "TEQAAAAASUVORK5CYII=")
index.append('box_in_icon16x16')
catalog['box_in_icon16x16'] = box_in_icon16x16

#----------------------------------------------------------------------
mergedisk32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAeHSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsyBxGxr4aIJUFxL+BmNKgAelnA2JWIP4GFQPxV/7/X1QIUwQQQCxomj4D8VMg/kek"
    "A/7hCUlGLGaA7PuALAAQQOgOmAjEc5B8gM9334GYHWgP0PL/UDYDMxFR/hVZACCAWLAo+opV"
    "KyM4joB+hrnrXxgwptyBlgODmG0/0Oyl4KBmhHqcyAgECCAWolQxM0IcAAlwdQaGX52Cglz+"
    "rq7qDOzszAy7dz+IffHiYyIwEEqA6k6B1YM0/PlH0CEAAcRCMMCYGJED3piB4ecKPT0plRkz"
    "XBksLaXAwteuvWXIyd1ju3/fvU0M/9mBDmHcDtbLBiT+/IeEGg6HAAQQE04fcwGlOIGYlREW"
    "AroM/3+sdnNXUdm+PQRs+d+/fxl+/frNoKUlxLB+XQBDTJyeOMP/X0sY/v93RHgRqJcVag4W"
    "ABBAqCEAVswIoZHV//+vzPD7x2ofPw3FZUt9GHh5WBnefP3LMP0sE8OX34wM6fq/GJREmBgW"
    "L/Ri4OBgEZoz6/wKBiYON2AoXESYwwgJzb+oQQEQQKghAPI1CyMinUPUcjL8+DlHz0BKfdZM"
    "d4jl3/4ytB9jYjj5nJHh6isGhuaDjAy3nwOLjn+/GHp7HBic3VXEGH79nA80SwzsIRjmAGJu"
    "VCsBAogJI3Ohp/yff1okZPgdFgJ9JynBxfDxx1+G/hPMDDfeMjJws/xn4ACWWY/f/WZo3f+b"
    "4dbTnwx8PAwM8+e6MqhoiBgyfP09FWgGC4r5aHYABBATRtwj47//IpkY/hfOnO7GYKAnyvDz"
    "91+GqWcYGS4Afc0DtPwvMP5/AzE7wx+G2y9/MrTt/cbw+Pk3BllJdoYZ050ZeHhZQxh+/i0E"
    "Bz2yuUgAIIAwwwOGGRlUgD7ozC4wYfTzUQIK/GVYfZ2R4egTJgYe1n8Mf39DLP/96w8Q/wKG"
    "7l+G849+MEw99JXhw9vvDM4OUgx1TVbAEPxXC0xD1mBHwDASAAggzBCAZD12hs+/Jls4y8u2"
    "NliDw+3IIwaGdTeZGDhBPv/9B5z6fwMd8QfIhjjmFwMn0BEbz39iWHbqE8Pnd18Z8jN1GIIi"
    "1XkZPv+cDvSQILZcABBAqA5ghWaZP3/zeIQ4PHpa7Rh4uVkZHr//yzD3IjMwRv4z/IP6/A+Q"
    "/gv0/d+fvxj+gEPhL1iM+d8fhul73zKcuvMVmHF+MjRUmzLIqAjoMvz40wxO4CyoIQAQQJiJ"
    "8N9/TWDCq6+ttWSwNpdk+AE0fNp5ZoYnHyGWf/sOdMDP3+D4/wO0/DfMQT9/Mnz/9gss9+r9"
    "D4bmdc8Y7j39yqCuxM3Q3mrNwMwErGX//XNGr7YAAgi1HGBiZGX48qvNzkWBOztZH6j+H8O3"
    "n/8YXOT+M7jIgEo0oE+B+XjZ2b8MDz79AfNBvv/x4zeDAPt/hmxbfgZ2pv8MP36DxP4yfAPi"
    "Z88/M/i7yTCERKgzrlxyvY1BkMMduUYECCBUB/z9F8cvxh0wZ5IrAzcnREqIl43Bmfc/pCIA"
    "+g5oNsPGC38YfgJ9z/z/DzgN/Pzxi4GVg4HBWYuTQZCLmYGVlYWBBVgvfv7+FxiYfxkY/wOz"
    "bpMFw7HTL8weP/uaDzSsEWYlQAChOyBLS0uY4fqd9wyXrr8BlYAM/0F1CjD7iQhwMFibSACj"
    "5BcwAUIshoQAMDqA8r9/MjB8+f6PgY+bneHUxXfA7PiVgQUa3/+BaYeXl5XBQF+E4fGTz3HI"
    "DgAIIFQHcLNsPHP1lVRQwmZgUoA1Fhn//v/yi8/EWIJv76YgcC4CWQ6Ka4b/v8EOgCRKRpBN"
    "QEsZGLqmXGDYuf3eJyZetk/IbQQWdmZmBm7WLchWAgQQWm3I2PT73/9pwOwIbDoxQVs7QFtY"
    "mXL/sTDVg1sB//+DfQzKegzQKADnCKADQG4GuoHhH6ROmfyPmakf2iQDG/7rLzAumBhfIdsI"
    "EECY1TEj4xuU0ooJmGzZ/31iZoE0NP4Bg/PPH0jQMwDNAyVCMP7NxPAPmEBBjmBiAephY/4E"
    "zHJvCbUHAAKIhWBDBOR+YHqDBSQoZ4DyPygKgE4BO+QPuCxgBsr9BYcQuA4B6QDp/YPUcsQC"
    "AAKIBWeLD+RjNkZY9oRgYPiCQuAXMMhBJeG//3/A8f8T6JhfPyFyoAQHb8iAMBu0jf3nP2oV"
    "DwUAAcSC1XJQicjOiFpcwZp6wBAAWQgqdFgZIA74/v0Xww8uFmAmgjamkesUBmhzlQEaGmiO"
    "AAggFgzLQRZzMjFgqyX//AUGP9AnKfZCDIEGPJAQAbaKfgMTJScwtFiAPv71+x+k8Qqr+f5D"
    "LeUCEsBsCg4NJAAQQKgO4GfGqC6hUcAISoS8PGwM7Mz/GfxMhYB2MILTAyjR/QM65C8wAX79"
    "8QfYHmBlYAE344DWMqG1LUCNkb+oRgMEEAtGZYStYcLKxPj60y+G5bseABM3JCSgTTWo0v+I"
    "ljswFB6/AXaEWIGlP1rVC3YQWs8BIIBYcPYBkAEH8867r74Gx1ful4TGJP62NAvTZwYOlgPE"
    "tPgBAohxoHvHAAE04L1jgAADADcpEtPUL0/aAAAAAElFTkSuQmCC")
index.append('mergedisk32x32')
catalog['mergedisk32x32'] = mergedisk32x32

#----------------------------------------------------------------------
taskcoach16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAPDSURBVHjaYvz//z8DJQAggBiR"
    "OYrBNnrWEVpTjY00TQ6cOX33XsHGqXsl/jBxcDBy3Hr6Y5fJI4YbQGV/gBhuK0AAMZ5Q4HAV"
    "NNfL/c8vKrFQkkfpsvcvYVZhJob3vz8zfNpyh2GyQjaDkIIsg+C2lW87+9cm979n2APU941h"
    "OgMT409GTYAAAEEAvv8EFTIcnQeMhh2u5C0AVxAFANnFcQDD7rsA/fb/AC4DPQAJMWIACRUQ"
    "ANoVPRMQtcyDAMnh1wFwBAwzKx2W4abv/gIAQQC+/wDWOgPw/c6H/5K5zv/x3f//7t78/9Tb"
    "3v/b2+b/ttLE/+Pq8f/18///7vf//6WSlf4wUQ26buVB+jV5IdcAAQBNAgBBAL7/Aznz/yMv"
    "LwYy4hMqAPEBGAAZ/QAAEBwSAA4KDgCV2YgAxei3ACgIOgDW6NMAyRjAAUFhFiOvgdXbzo/g"
    "gwAAAOwCAEEAvv8DIfoA7ysK8V0bQjADwegVACwPEAD2/foAAAYDAA38GADkAcQAzOjGAO8J"
    "zgAdHPMAv64DAMWEzaz8/vvSAAAA3wKIqfcfw6njp16fl/11heHLhwcMz5mZGL68ZmD49Pc3"
    "g5qIEQPzazUGXT1GBk6eb2DNJbYTGALzOlwZGBm/Ag14BxBAzEDin9JnBj4nK1a3N7yyQBvV"
    "GMQZWBl+cjAxMLOzMvz7xs+w4vw8hteXRBlCNOoZfjILMEjpmwptPXj69b83d88CBBDIAAaB"
    "fwyvNPm+p+g5ybFuuyXJoKEgx/Dl4x+GX5xMDEK8Egzf7wkzuOhGM4jLyjN8+PqLgZWXieHr"
    "igbxW28+7gQIICaQAVsZGB4eOPTuoAjbfQbmD+cZXjIxAr3zj+Hrb0aGr38ZGbzcAhmklWQZ"
    "WLgYGJg4uBm2n7jJ0JrIoaLGzhAOEEDM0AT1T/YTA6OzI2fQL3Y2his/DBmUJISBSY6R4Qcz"
    "I8Pd538YLtx5yXDp8iWGN4/3MVjxHmFwNOVgOL/tHidAALHAkuTVfwzbj25/9NKhWF98+Yrz"
    "DD/1/jG8fX2fgZvlKYMi9x0GD967DFoSTxg4f75hYHj7heHK5A8MWx59vwwQQMh5gblRimlm"
    "3RG75BNX+Rie3XrEoML5noH10+f/zx58/37h1q/PFx/+fXvrNcOrp18Znr/+y/D4JwPDaYAA"
    "QslMygwMOnaazDM5uP4LXnr07+X99wxP3/5heA5U+BwoDYxcho9A/B2If0DpjwABxIiWO1mB"
    "WByI5YAYGGQMP6EKYRjEB6Yyht/QXPkLIMAAglZq9MCCdyoAAAAASUVORK5CYII=")
index.append('taskcoach16x16')
catalog['taskcoach16x16'] = taskcoach16x16

#----------------------------------------------------------------------
print32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAlaSURBVHjaYgwJSWXAB/79+6eo"
    "paW2WklJjh/I/sdABGBiYmJ8/vzVrwsXriQxMjKewqcWIIBYfv78idewP3/+Cltbmxh6eDgx"
    "MZAAzp27zHD69HlxoGPwqgMIIBagC/EqAMr//f37728gkx27iv9Q/A+K/wIxJ8OvX39AnD+E"
    "HAoQQCxQa6CGkAr+oVmOjAkBiMcBAogFoZgRGHesDP///0UPAQbsoYTN8v/QEPiPUy8TEzPD"
    "379/gGnrOxB/ZQAIIJY/f94CLf0PxIwMbGw8DKysfGA2PID/MxEI9v9Y+OCEyMDMzMaAmgb+"
    "A6PmE8OPH+8ZQO5iYWFmAAggFpDPQZy/wGj+/v0N0FW/GTg5JcFiMIcxMzOjWf4XzULMKGBk"
    "BDmABewAkK/////H8OnTE4afPz+CQ4WFhQ0c6gABxIIcJ0xMLEAXfgalfAYhIUWgQpDFv5CC"
    "Ed3S/2jBjxAH6WFlZQNaxAr08Ueg5S8Yfv/+ArYD5BgYAAggFsw8zAx25cePD4A+YGdgZxf8"
    "Bkwb/xHx/g9P/CPYQIv+A0P1++fPT8AO+PXrJzCKMTMSQAAxYc96zEANX4Aa37J8/vzKhwmU"
    "OlESHi6fI/hAPczfv7/1/vr1LS8oOkAYGwAIIBZEYvsPjP+/QJpB6vfvX3p//zJLc3Fx2uvp"
    "acWysLCgpfy/eKIBEQVaWholV6580f706dMmYMp/BpS4CnTIXWBBCXcAQACxQC1nApayIXx8"
    "ArEyMrLGKipqwqKiEmwyMlIM9+7eY/j3+wcDanmBXG78x8r+D0zUUpKSDF5etZ7Pnj3zfPPm"
    "1Z+7d+98fPTowcX379+uBNq3AJi4fwEEENQB/8r8/YPbHRycGYSFRRk4ONjBlrCyMDGcOn2O"
    "4QeLOIPY468Mv/98B4r+Z/gLDClGRtTsBwpBJiZI6IBS/81nPxneffzCoKGhxqCtrQeyg+X3"
    "79/C79+/czpx4qjTmjXLNYFipQABBBJU1dHRLY6KioUmwO/AuP8GNuTHtx8MTJxcDDyyKgzv"
    "vv6BZxpGRlZwNmWEZmEmmDP+AEmgw/7/YmRgF5NkYBcQZnjy6AkDEws3WAUHKzvQc9wMbm4+"
    "DPfv3887ceLwBoAAAkYvk5ejo5MIyNdfvnxmAMUPKBGysbEwvP/8nYGNX4xBW1cWqB9cMIAd"
    "wAyylBFi6Q9gcvgNLPFBlcWvv/8ZfgOTwF8gZpUWYGB7qgIsW74z8PHzMnz78YPhwOtLDLwM"
    "XAz2MvoMxkbmTOfPn/YBCCAWbm5uHVlZWXDeBJVMMMDMzAQMjR8Mv/ilGa69ZwLVywwsIF8D"
    "8RegbW+BlejHH/8ZPoMs/wd1G6jggiaVf0DM/FeEwYrlCwMXJyvD11/fGH4Bo05KQISBGZin"
    "JIDpg4+PXxMggFh4eflUhYWFwQaDLAXp/gv0wq/vPxi+/PjN8JJJkOHTFwYGNmAIvAamxVvv"
    "GRiefvvP8ANoKdN/SBQgqhYEB1QNsn7iZQj+85lBjo2RQZifjyFMwAaYWv4CHfcPmNaEGMTE"
    "xGUAAohFQEBAgpeXBxz0oGITWI0Cg+0nAzswMD7+/M/w9h8nAxPQARdeMDJcewv0/R8GiFpG"
    "AvUkKJq+cTDcf/+HwUQFWK6AQglURAMhM9AednYuBgEBfgGAAGIBlvXcnz59ZxAX5wQb/Pfv"
    "X2BUABWwMjG8AYb1uy+cDCfuMTA8+wAtthiJrHBB6n5wM9x7Byr3gSHIxgr3JAiAPPrvHyMr"
    "QAABW0S/ga2X28DgEAQGCy8wXrjA2ZCb4x/DvQ8cDFtOcDP8AhUDrNAQZkSpzlHrKAa0yvEn"
    "G8NdQQGgb5kZfjOB6oRfwDrhC8Pr1x8ZPnz4Dkz0PxgAAogF6CJQAmZ49eojEL8HViCsDDw8"
    "HAziotwMahL8DCkGXxk+/WVj+PATmCu+MzJ8/P6f4RtQx++/jAz/oLkC5BZgpmEApjUGXmAR"
    "IsTFwCDA/pdBgPkXg5k0D8PdRx8YXr35ArYQ5GFQmQGqF4Dp5w9AACLLJQVAGAaiA/0Qz+DK"
    "tSfz4KVLEcRibNFJu3CdEPKdF09g7CG4RURIwdqNpShSBma5sa0HPJ2dtZAINYlVDngk8Jdv"
    "OxPdOySdcVpVVP6bF68l5QZ9tHtOU+xcMM2h9J+fAAJGwY+nwBLMiJOTA1hd/mYAlfufPr1n"
    "OHBgF5j/9y+kjoCVAeCCh5ERWuoxIrWeGcAh8v8/vEZjYATGNwuwLQGKexMTSwYJCRmwmaD2"
    "BbC+AaaD7+8AAgjYIvr98P9/UCOEHWg5sARj5wBGxSOGgwd3AOPsBwM3Nxe4VPz/n4Q2I7SU"
    "BCVoUEEESrLKykoMwDoGaOZ3cGPk48dvDN++fXkOEEBAB/y5D3QJ0CJOoCSw6cvOBnTdD2A6"
    "4GGQBBYWkZFRDIKCQsDo+U1ScxXkS5AHVqxYzgCsjMAtLnZ2FnDjmp2dneHt2x+gNsJDgAAC"
    "hcDZN29e/gImPDZQdgFpFBcXA5bXnsCEwslgamoCdNBfcOEEzsn//jNAuif/waUniA8KHEh1"
    "DqmYwNEBZCsr8zM4O3sAHfCUQUlJAZi7WIGJnAlIcwJD4D3IAecBAgjkpLO3bl275OfnbyIg"
    "wAe07A+DkZExMDuKARUzM0hLy4AdALMA1DlCWPgPKgbDf+FyoGYdHx83g6OjC8O9ey8YjI01"
    "GL5+/Qb2ICgB3rhxBRj1DIcAAojZxMTk1+fPn94Dg8uehYWd89ev30wghTdu3AMmHA1w/IPi"
    "EoRhFoJCA2QZhIbhv3AHwjDQLAZRUX6Ga9fuQNPDV2D6evX3+PHD786ePdUGdMxegAAC94yA"
    "ZcGavXt3ngcK6gIbklJ//7Lo+fkFxPHycnOCimWIbxmQQuE/PASQxWDioKgB0T9/ArMjMAPJ"
    "yor9nz17zhompu8HgI5+8fbtmxtsbGzXQHYDBBC8ScbKynIXGAJ33717D4x7sQgNDaVkSPb6"
    "jxLcoCBGtfgfFgcxwMXfvQNWRnKSjKysPHdevHg6TVRUBFzYwVraAAHEAqFBwcoOLCL/AF3N"
    "xmVmpp2noCADdhw3NwfOlA6JbwZ4FkWmkeU4OFiAacEibe7c+3OfPPl6F5Qb2NkhtQlAAAEL"
    "Ilaor5jBzSlgWSD0/fsXvs2bdz36C076jHg6roT7frAy6fHjZ6zAXCAPNPLunz+s8OoMIMAA"
    "Gd6hlHDPSl0AAAAASUVORK5CYII=")
index.append('print32x32')
catalog['print32x32'] = print32x32

#----------------------------------------------------------------------
checkmark_green_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMFSURBVHjaYvz//z8DJQAgAINx"
    "cAQgCANA8MgQlRkefrUDLZM6LBQSAd3XxlxOZofYNtQTWhMWOqYVxHjVsdWY0gj/x+Iw7NqP"
    "/KByfwKIiSTrQI79/TtBXVlqV6S1hZUAK98/gAAi2gCgV0X/MTHMdrfQm1/r6Shz8drzfw/u"
    "vEoECCCW/wz/FYHB8A2o5iUOnQz//vwz4xbjnJ/soqUVoKHM0LH7PMOx0w8yGbmZNgEEEIuQ"
    "CO/VX7//3fz8+I8PGwPDUxS9//6z/vj7J1VOTbCnwseY00ZcgKF853mGXQcf1DOxs8xiYPzH"
    "ABBATHzM3J99HBQNuAXZVv7+808IrJMRGE5//2v85WBY5uaoNHVqmAenlbgkQ+Oeawy79j7v"
    "Z/vP0sTABIk9gABienbn+0wJBjGG7EAra0YuhnX//jEw/vn9z5VXnGVXarBBSIOrO4M4Dw9D"
    "78FLDNu2v1zF/pu5nAEp5AACAEEAvv8DBjAZ5u70+2RxSUoEBw3+H8fX4mUiKg1hdlc6FEoi"
    "MwD4/vMAn8nQAK6t6bgA+v1dAAf89/wE+gD9BvkA/wf8AAIAQQC+/wMQMCHw7e73HRUcCDNX"
    "MDwBCBn3DlM2LgUqDyMA7vzpALHU0wCxquzRAfIBWQUFAuwD/QQABf0GAAb7CQAG+QoAAgBB"
    "AL7/AxAvIv/99AX78gHvWy8nFwEwFiQAA/8DAO/96gC/3dYArqXx4P/tAU0GBwHgBfwGAAb8"
    "CAAI+QwACPYPAAn0EAACiFlIQ5uB9Q/H4y+f/1nxq35V4mFnYfj0/xXD6Sf3GLYu/f7m91OW"
    "ACaO31f+M/+DeJoRGPEgzPQPjAECiInjFzsD5z+2n4zP+LaePv2e4TnjHYaLr+4z7Fz668uv"
    "21wxzMwMp/ElMIAAYmL+y84Awhz/2Be9PsP98MTt5wxHVzH/+n5ZIBuoeSehFAoQQMwSakbA"
    "KGVmYGJk/M70jUPszW0eix/XeesYmP9M+8/2h+Ef82+G/yzADAT2AiOGAQABxEhpdgYIMABQ"
    "rSEzlY7wIwAAAABJRU5ErkJggg==")
index.append('checkmark_green_icon16x16')
catalog['checkmark_green_icon16x16'] = checkmark_green_icon16x16

#----------------------------------------------------------------------
music_piano_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAKGSURBVHjaYvz//z8DJQAggJgY"
    "KAQAAcQiJia2R1JS0urKlSu/DQwMGB4/fszAyMjIICMjw3Dp0iUGbW1tMPvq1asML1++ZDA0"
    "NGS4cOECM1AP87dv3y4ABKAIjokABmEAitZABiTkLnqYwEBMoAEvrAwIiIxIob/X957e+7v3"
    "RkQ451BrpbVGRFBKYa3Fb4yBqpKZmBlzTtz9fgLQOCYpAIAwDAypdKP/f62CWvEUyAQmFJEZ"
    "EW11d6gqToeqehfH6LyMJDITZvZ3awvA4RjbAACEIJDiN7Bg/y0xavJKCeHIve42HBGoKqyJ"
    "gevXADPjfNBtJIGkzzITXwCxgBSAXKCsrMzwHwhFRUXBkiAAdCrQFgibn1+AQVFRiQEUa0pK"
    "Sgx8fHwM//79ZwAIwPEYnAAAwkDsBLEP9dMBerj/ilo8OkBC0jMTNgyMQErm7rVw7xNwhLYS"
    "7DVBRpWR1OKU4OELIBaQiSysrAwCAoJgf/PycgNdwMLAycHOwMfPCw8LLi5OoCE8QL+zgtkg"
    "A0B6AQJwKAcpAIJQAES7XW6r4xuJQQXhJv5XSalJWszqTWdMXzQKzjlSUo59+7uzMltLbMb7"
    "EMLJ0p5aM6v3iFxM41A+AQQORE52NgYFBXmG/3+/M3BKijIwgEz+94dBWQXohX8/gcwvDMJC"
    "fMBAFGL4/+cHg6ycDDgN/v79iwEggFgY//+GOIXhLwMjM4j+BfE2M4j4AVTHCPYSMF7AmBEc"
    "qyA2G9CivwwAAcTy6/sn5s9vbjCwAeP5z5+f8EDDDf4DA5kNzPr1/SMzQAAxCvJzHTfUVtAH"
    "WvUXFCPEAEZGJqCrGJgvXHtwCyCAGCnNjQABRHFuBAgwAAMX8o1Bs3g7AAAAAElFTkSuQmCC")
index.append('music_piano_icon16x16')
catalog['music_piano_icon16x16'] = music_piano_icon16x16

#----------------------------------------------------------------------
listview16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANTSURBVHjaYlRVr7yuZ6qk8vP3"
    "//9//jIw/P0PxP8gNIj/B8r/8w8qDpX7y8jE+OrG3ccAAcRi42Eo41AUyvLwGUTDLxD+g0QD"
    "8U8o/g0V//GbgYGNm4GBceVGEYAAYnn36d+//bvfMnz6+JOBmYWJ4T/Q9H9Aa/4DGUAmhP8f"
    "QoP4LECCF8j6yszJ8PXr3/8AAcTy8fNfhnhtRgZHTSGGf19/MvwHOpGZh42BkZmJgYWFBaj5"
    "H9DAv2A2IyMjAwPY0L8MWy98Z0jf/I8BIAAAQQC+/wL7AAEACxEOABDu5gD2pmsA7qtTAAHj"
    "1gAKCAMA7erlAPv48wAPDQcA8vDrAOvp5QDr6OQA/PjzAAgIBgD+AQMAAojlLwMzwz8GJobf"
    "r18xCH5awcAl+Jfhz9tXDK/ff2FYvGQlg4GBHoOrqxPDpk07GF69es3w588fBidrQwYmMSew"
    "AQABxPIHqPnf798MLDKSDJ8YExl+fPjNICCtyCAgws+QlZXCwMrKBnQyI0NERBDDX2DY/AVG"
    "AS8nE8O2q78Y/jExMgAEEAvIlD8/fjAwSskzCNXMAvuRgRUYwsDAEhAQBGv6DbSAjY2T4R8w"
    "NP8Ao4qF6Q9YHGQwQAASySAFAAAEYQX9/8VZORK8Kkyr88kjefQV8CC/AbOZb0UEYPkUzTQu"
    "PwEE9gIDKzvDvxfPGJ6smszw7d87BknPQgZ+dQ2GXz9/AW38DzbkHygBgV3zh4GF9Q8wZoAW"
    "Aw0ACCCwAf+ZgSH69BmDwLepDBwCnxn+vgxmeM4vxjB/zhwGXV0dBnt7O4Zly1YyvH79FhyI"
    "1ibaDMzCjsDUwMQAEEDAWAAa8A/oLwEeBsbXGgz/3/9kYDIXZBAUFWCIi4tiYGFmBdr8BxgT"
    "zkAX/Qa7QJiXhWH/w19ALzIxAAQQy1+gM/59/cTAoK/KwF9/gIH/N9Ct/MxA9/5gkBMTAtoI"
    "0cQuxAP2+1+gC7hZ/zJwMALTNNAAgABi4eThZNp/j5Hh9usPDL9Z2CEh/eMbUOM/cKYBBxoo"
    "UwE5f0GBCAp5oMY7H1gZfrOzMwEEEMuLx89fbdz5n+Pztz///0HTPUjRP2gu/AdMqUAfQjIC"
    "DP8DZQpgIvj28h1AADH+h8UhmQAgwABtaJg6e9y6kwAAAABJRU5ErkJggg==")
index.append('listview16x16')
catalog['listview16x16'] = listview16x16

#----------------------------------------------------------------------
cross_red_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAW2SURBVHjaYvz//z8DLQBAALGA"
    "iHoZGYa/f/4w/GViCpIXEprD8PfvvycfP8ax//u3jRloMRMQM//7x8AMVMsIpGFO+fv3r5W4"
    "oOAGdnZ27hfPnhWx/v07E6Q+/ft3BoAAYgKrAHK+//vnoSopuTRaUlIwQkBAWImLa+1PRkYv"
    "XC76+eePhQQv73Yfc3NRf29vLhU1tSk/f/6MZICGAEAAgV38h5ExVEtJaVGokBAHb0EByCkM"
    "/hUVHNt+/lx79/fvOI5//1YjG/rr/397OS6uzS7q6rxCGRkMDJKSDC58fCxMLCwL758/LwJU"
    "MhkggMAulpGRmR0pIcHBm5zMwGBqysBgYcEgWFXF4M3BwaH279+iX0gu/8XAYC7LwrLFVVaW"
    "VyQ9nYFBV5eBQUCAgd3Hh8ElLIxV3sioE6QOIIDABgPD7d9fRkYGhs+fGRjevGFgePCAgcnY"
    "mIGvspLBiZOTQwXo8j9MTB5AQ02l/vzZZS8szCMINJTJwICB4dEjiB6g3n/AYACZBTITIIDA"
    "Bj95+DBx9dOnPz4vWcLAsGUL0Fm/wBpYDA0ZRIqLGRxZWDgUv33bKPXz5yEHDg4+sdRUBlYj"
    "IwbGp08ZGL59Y2C4fp3hx86dDPtXrPj96MyZQpCZAAEENpiVkXHjnbt3gze+efPj65o1DAx7"
    "9zIwgCwGupxFR4dBoKiIweL/fzbrnz85hNLSGNhgLv3xg+H/rVsMPw8eZDi8c+efu2fPpjMx"
    "Mc0GmQkQQIygdNwoIQGMQWBy+/8/VFNcfJEHFxeHoKsrAwPIAKBmBh4ehq/HjzP8+/mTgdfe"
    "noHh61cGBhYWhv8vXzJ8OXOG4fjFi39uX7uWD3TgNEZmZobU378ZAAKIBR7VwDBm+/9/9c3n"
    "z7/+FxJa67FxIwffp08MLHp6DAzv3zNwq6pC1IG8z8rK8OfFC4avZ88yHL9+/c/d+/fTWJiY"
    "5iOnHIAAAhvMiGw4I+O2++/e+ezk5d3quGMHuzAwvFk1NMCGgwHQRb+BwfDxxAmG0w8e/Ln/"
    "4kUkKwvLGlD6ZQRhqFEAAQQ2mAeYU0AS/4EGMwLxz///X/9hYfn/Byj+duNGBnagQYzs7GCL"
    "/wOD7Pvjx2ADgDn1PzAVvGKF5kpQhDFDEgUDQAChuBhE//j710JWUnKnGzAl/AUa8Alo0O9d"
    "u0DpFwzYODgYWPn4GLiFhBgs1NRYgZlix5MHD0KABm9jRDILIICY0LKpvZy8/C43Li4+1tu3"
    "GX4DDYW7BISZmMAuBxrG8AeYO/9//MhgaWDAqaCuvvY3A0MoslkAAQQ3+Ofv3+ayKipbfPj4"
    "eLlA6RJoKMj2/1BX8APDlh8Yaczw6GAEW8z49i2DtZUVh7yOziKg4fAcChBAYIOBtttLa2js"
    "9BYS4mEHxvQPYITBDAUZxMvFxXCDg+PPZWbm39xAw1lAYQ2KKKAPfgNdzgBMIZaOjhwy+vpr"
    "/0JdDhBAYIPFlZRW+0lI8PMDY/oXkqEgSU5BQYarHBy/7/77F//w71/vyz9//mAHhjMoWP4D"
    "Iwpk+B9gumUDGm7j58chYWAAziAAAQQ2GFjWcHEBDfwLzAAMSC7lBmacK+zsf25//ZoO5C8D"
    "JsXdj759C77y4cMPTmCmYQYGD7iiAPoApJcdyGfn4eEEmQEQQGCDn1+/Xrnrx4/ff8zMwN4E"
    "JRVWRUWGs2xsf269f58PNHQ+LLaBuWvbw7dv4y48efKDhZeXgRVoGMgQRnl5htNHjvx7fPx4"
    "F0gdQACBDQZqnHzz7Nn4A3///gGVWKzq6gynmJj+3Hr+PA1YI0xjRK92GBlXP3z5MvjCnTs/"
    "mEBFpqwsw8Xz5//d3LOnDFiW14LUAAQQCyzHsTMxLb939SrPPzm5fmAm+Pf40aN8oAHzwZkC"
    "zWCQRazAdPvi6dOIM4yMC9h4eTkeX7/eARTrhakBCCBGWlWmAAEGAAbpLTSsJROfAAAAAElF"
    "TkSuQmCC")
index.append('cross_red_icon22x22')
catalog['cross_red_icon22x22'] = cross_red_icon22x22

#----------------------------------------------------------------------
folder_blue_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAGkSURBVHjaYmRgYGBicNv6lwEd"
    "7PLWBpK3gfgfEGPKQwFAADEANf/HBkDiQHknIOYCYhYoZgZiRmTtAAEEEmQ48RDT3P87vRgY"
    "3bfuJeQygAACG/DuK1QTyHig+f+AjC3XIIagA6ChV4GGOAKZR0EGAAQQ2IBXXxgYrr9kYOia"
    "so2BKOC2dT/MEIAAAodBx26In0kB0DAyBwggsAtefoYYvP8uhP4N9N0fIP4FDPtffxgYfgLp"
    "H0D6L1Bs06xnDK6JUjC3uAIEEMQLUAPefYcEMUjz778QA77/gbL/QPhffzMwfPgO94wwQACB"
    "DXj9BWrAV0jQ/gPZDjIEavtPKA3i/wAa8O4bIkwBAghswBuoC95+g8QEyKm/oa74CbX5918E"
    "//NPRHgCBBAkGqEmgmIDGDxgQ8BhANX89z9EM0gMZMAXJAMAAghswKdvSC74D0kHf6EBCQsP"
    "EA0O4L8oBvwACCBQmGkBo+QqA6lgl/cqILkeIIBABrACsSEQOwOxEJHaQfFwAYj3AgQQLGOA"
    "aHaoYcQAUO78BQoqgABiBCUqSgBAgAEADRcEmdMu8V4AAAAASUVORK5CYII=")
index.append('folder_blue_icon16x16')
catalog['folder_blue_icon16x16'] = folder_blue_icon16x16

#----------------------------------------------------------------------
bug_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANxSURBVHjaYoxhYGBgBeI/QPyX"
    "kZHhBxCz//vHYCwpyfCXhYVh4ePHDNcYGDiA0j4cnJw3GP7+ffTj1y9+IP87EL8BCCAWBiTA"
    "CML//4MME/n+9aunoJSUfqiqqtDVnz+Z7v38af9XTo7v86dPXx7fvbv5958/NUDlTAABBDeA"
    "GYj/MzDofOTgqLfQ0nIO9/Li2/fxI/On378ZkkxNGVRfvWI4fOAAwy4lJaHfv3/HPb53bwZQ"
    "yxWAAGJigDr/GQND1wF29pOvtbVDHL28BFW1tZnNmZkZ+O7dY7h+9SqDKNCQRH19BvkjRxhe"
    "vHrFDdRiDNILEEAsQFvZlBgYVohycAQIyskxhioqMti+f8/AMGcOg8H16wwG374xfH/5kuEf"
    "OzvDX15eBi9OTobZT59+e8fAcBdkAEAAsYgzMJTFMTAEKAoIMDLIyDAwff7MwHANGGw3bzIw"
    "AAOTQV6egVFQkKFk+3YGMX5+hgSgASpAzacYGY+AwgwggFi0GRjilIFh9/vrV4Z/QBvZvn9n"
    "YPz4kYGBh4eBwcCAgUFJieH0hw8MW16/ZlAGGu7+5AkDNxMTKwMLSyjD79/7AQKIZSsDgxDQ"
    "GwzuX74wsAMVCAOj7hHQqezGxgwSampAD7Ix/AXKfWFiYhAFeQ1o+HseHjWGnz8j////fxAg"
    "gFguAB16GmiANjD+DUREGM4ICzNUAfkSQE0T/v5lEPnzh8EAyE58+JDBA+jKh0B1rwQFmbne"
    "vr367efPVwABxGLHyPhdChj3LECJ38CAOsHKynAa6Fehd+8Ynp45wyDy8yeDAFBzHzAwXwIN"
    "DmBkfPDx169PAnx8McC0Mg8ggBh82dlfX2Bk/H8ZmAzus7H9P8rL+99GRuZ/pqDg/69AsV9A"
    "/AGIgUH+PwmU+hgZIxlZWWU5OTkPMzEx5QEEEAMjM/PRAAaGh2uACs4D8RMgBhn2GIjfAvFV"
    "IF4BNNiFiekzMNRLkBKuABCrAgQQKCZkgfinFgNDnRUDQwQwigTFQLECDLTnHByM537+/HUO"
    "aMCTX7/mAPNBDgMaAAggRhiDF4i/MDCIAZO0CzACJf+wslr/4+b2/vb583ygRmC6YdgLxSgA"
    "IIDgSASUM6CYHWw0YzsQnwWyJKFK2LDpAwggZhiDC4iBAcbwD5o3oGAnEF+Gsv9iMwAgwABT"
    "rSdfbFNpAwAAAABJRU5ErkJggg==")
index.append('bug_icon16x16')
catalog['bug_icon16x16'] = bug_icon16x16

#----------------------------------------------------------------------
folder_blue_light_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9wBHQ8AJ5htIlAAAAFc"
    "SURBVDjLpdK9alRBGMbx39EjgpXoXWjhWlmrWFl7A6fwjizdVlCCVxCsrD2CwSJYGiFRV1eT"
    "zfmYGYuZ3SQbwQ0+MJwZ5jz/92PeCpc0bbCu6eQ2dhFx/n6lpk1/k6ZNeIBrqMu6jOq0vYZP"
    "385z0/M7Ku32vzKr4XdXTKgqYuLdXoasq9LumE7u4+0KMOvYnfH02XsbqWnfnECaNr36kGu+"
    "iEqP7tWwf5TBHw/yd0zEyBAZI32kD4TE1osfHj+5vszlUQ1fCuBXn1scUjYOkeNQ9iGD5j0/"
    "+1UxN2vYK4B5l1sbI0NiDHSRrpiHwGHP9+6kpzV8PsynWZdfIqQcfSipd6f2i4H5sDYHyxIO"
    "FhmQZMMy8phKBpGj4UwJGfB1kQ/7HSnlORiXWaQceYj5ny6cARxXuKVpd1xU08lLvK5wBXfx"
    "EDc2tC/QYrtaTShXC2wTBfQYq5SS/9EfwAcEYKGWevMAAAAASUVORK5CYII=")
index.append('folder_blue_light_icon16x16')
catalog['folder_blue_light_icon16x16'] = folder_blue_light_icon16x16

#----------------------------------------------------------------------
arrow_down_with_status_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAIaSURBVDhPpZNdSJNRGMdf7aLb"
    "rqOboJVEahNv7IN2GVEI3QwNLCgRFJkYURSjlRQ08CIaLiPoKhgLhXSuZGlsLT+yWqZt+YVb"
    "Y+7DKbk5tsr269273Oa2uqjn8D/nfQ//8+N5zkcJYgj/ExKgoguhUi+pVK6nur6HWpWZUy39"
    "okzUtg5Q09BLaVU3wkHRl5Loldame3HI0TO7R5qOfk+yFt+Qvt98Chb4/giwjroZCUCTKUmD"
    "YR3D+ziTTt9fABoF5Mg6ukjfHBy+H6PqTgjt4CqOae8Wj+TPlJAHsNgW6JvZ4MjdMPIOD9qB"
    "ABMOsaw8XwaQtwU4Jr0YHDGqb3vZf+UzN4xuXDO+/K3KZpACWN/5WY0kCKysk0zE0FmCVKjn"
    "2aP6QNsDJ19DYZyzPhbdS7yd+iLBtmSgbB8U/5N0v4pQ99DPic4FKi9PUdYyjrzVzqHmYc52"
    "2IishGhSWwoBKeLjfhezwTjl6jlk7R8pb5tA1mhjx2kz244a6RtyYXrhypSSyUCj0bApj3eZ"
    "ntdL7GocZ/d5GzvrnlOiMHK9awy/z5/xpfxFAccvPOXHtyjqR9NsP2lGOGZEeW2IaDhE/UVz"
    "cYAgXsjcprr1kp+JCM2dYyivDrMmLtbo7Hmu35c4nUZhu6S1shwMEQ4GuakfKerJnsIB8THl"
    "quwewl4dNWeeoDjXi7BPhyAT51IPbvMxiWP2JqaP9J/iFxbmq1DuKHvvAAAAAElFTkSuQmCC")
index.append('arrow_down_with_status_icon16x16')
catalog['arrow_down_with_status_icon16x16'] = arrow_down_with_status_icon16x16

#----------------------------------------------------------------------
export16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAALkSURBVHjaYvz//z+DubnVanNz"
    "2wB2NpZff/8xMPz99xeI/zH8B2IGoPzf/0D2/78MDEDunz9/mH7/YWA5emT/gbt3r7sCBBAL"
    "AxC4uPiGZGZmM4iIcrFwsLMwAJUy/AIqBulnZmRgYGUG0gwQ8P37T4ZTJ68z8PIIuYD4AAE4"
    "HGMcAEAQiF0IMqn/f6ZBHMTECO5t735ARK7ZpNoAlo7tsRQVP3EgobCZkgPUFlQHSuGb7hNA"
    "YAP+/fv3H+QVZiZGBqA6OAbZ/hto0E8g/gtyApD/889/BmERUQZubi6wiwACiAXiMEYGRkYI"
    "ZgI5GSQKtPo/VBPQzwygsAG5Cuwkhn8MMAAQQGADmJkhmpmZmSCCzGB9DIxALhNQLcjQX0DN"
    "f6COZoRaCAIAAQQ2gBGokomJiYGFhRluMjPzf4b/YOuACplBMfEf7Ip/QMjIhDAAIICgBjAy"
    "/AXKvnz5ARQewKj6B6b/gaLv338wDQwmoEYmht+/fgLVM8MNAAggsAGswHj68eMXw717z4HR"
    "9A3smv9gTQzQwPgP9P9foNwPBgEBQSAWgrsWIIDABjAxMQNt/Q805CuDlpYsg4iIEFjxb2Do"
    "MTL+Z2Bn52B4+vQFw+LF6xlMTc2BBoiAvQ0CAAHEBPMCyMnMzCwM/PwCDLt3H2Z4//4zg4yM"
    "FIOwsCjQQBGwzUJCgkCbWYBq/8K9ABBAUBdAnPz37x+Gd+8+MezadQiokJ1BSkqG4evXL8AE"
    "9IPh8+fvQEMEwK79C/QOExPEAIAAgkYjE9gFv4Bx9e7dVwYxMTEgn4nh9euPQG/9ABrGCjTg"
    "BwMfHw9Y469ff4B6IGEAEECwMGDm4uIBewGU6kNCQkDhxvDt2y9wNILCh4mJlUFaWgYozgp0"
    "ERdILdj7AAEENuDChROHOTi4LT9//vT/+/d3wGTKDcp1YFeBkjjIi58+fWL48uUbUAyUBhgY"
    "Dx/eebWkJJUBIIAYQQooAQABBgBAdRuySum4oQAAAABJRU5ErkJggg==")
index.append('export16x16')
catalog['export16x16'] = export16x16

#----------------------------------------------------------------------
new22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQLSURBVHjaYvz//z8DLQBAALHA"
    "GLy2hUtc3Zwi+Xk4GP/8/UfQtv9IjP+MDAysLMyMFy/f+njh1GlfBiamIwABxAByMQi7lC//"
    "8fM/8eAfEv4LFdt08fV/NpvS9Vx2ZQwAAQR3MTcnByMrCV4FhSAM/wPy2ZiBZgAJZjY2FmYm"
    "RgaAAIIb/Bfo/b/IYYPD+//+QQxCNhQkBjL4159/DEAvgMMRIIBYiHUhSDXYYHRDYTQQMzEx"
    "MDAyQjBAABFl8H8Y/o8IBpgFMEkQ9fUXyOcQcYAAYiEmLGEuQ3YpjA0y6CcQMwJNevYFGBw/"
    "gZLAYAEIIBZCLv2HFlEwl4Ms+vkHaBDQ0D9ANus/qA8YIRgggFgIhul/hEFwHwAN+fEXYuh/"
    "oOv+/YGI/f+H0A8QQEw4kxIyjeR6YMQzfPsNMRSUhH59/8nwFxQk/2FxAFENEEBYDf6L7PV/"
    "CPYfoMt+gBM90LdcDAxfbt9k+NQfyvD98HoGBg6gJeycwKTB9Y+BhZsBIICYcIbtf0RSAlkE"
    "cuEPoOpPj+8z3J/exfD53gOGH/duMcjq3mNg+36B4ePH/wxSRxYyFL3cJLD27GQGgABiwZe0"
    "QAb/BmFQBAExEw8Dw6v9mxlkRS8xvN/3luHDG1EGRTsZht+3gJYubGUIc5zB4Hj8k8bt9Z+Z"
    "AAKICdn7sGQFSkK/YRH0BxKuYNcD+SJuYQxvP0kzKNrwM3DcWwUUZ2dgvr2ZQcN+M8P5+9oM"
    "bYdUzgHT2z+AAIK7+OlnYJgBDeFlhHj773+k4PgHif03l68xfHt4i+HDUwGGbzM2M5jVOTAw"
    "cv5k4NdnZ3i/9jrDsfUmDEzSv3VE/zDEAwQQUlkBMewvA6qhMPbv378YXu2awKBuJM6g7M3N"
    "8P+rBTACuRmYRKWBin4ysOoKMGSa7mdg4/wm92Kd9DyAAIIbzIhcyCAb+g9iGSMzCwO3ji/D"
    "05fnGP5c+8jAdv8og7CmC8P/H+8YGNl+Mvy6/JFh+3kDBubf974Zn720CyCAmFAiDepiFENh"
    "lvxmYhBx9GXgcqsE5jQuBqUoZ4Y7844xfDl3ieHpzPMMAhZcDH+EmRg2b+Hb/OvG90CAAIK7"
    "mImZmYmTDZxEga5DSmqwIAJyGIHJ9O36pQzGOq8ZvjzjZnjwSIdB7M89hjcfrBl+HfnKEJlw"
    "mMHw5Du7lzsYmAACCG7w80f33k7d/1pCCGj6zz//UQobWLZmYGdg+PxBhuHYvh0Mz3QyGKTV"
    "3jPYs15neCwtxbCPIY5BtaOAQfHB3lPAVPkPIIAYYZUpo0GiDQOrcDEDCws70JR/KImbESm8"
    "WNkYGH//YfjPwffH5OVpkS0+Zyw6Tyg+6GfwvMLw7f2NuovLq3z+Pv4HEGAAV49RwueAC9gA"
    "AAAASUVORK5CYII=")
index.append('new22x22')
catalog['new22x22'] = new22x22

#----------------------------------------------------------------------
markuncompleted16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANJSURBVHjaYvz//z8DJQAggBgv"
    "yjMwMP9kYBAEYs8odxfPSPOJsjJiCitWbFrfueTQOitZFgmGj9+eM3z+t57pMkRTiigDw5p3"
    "DAzv/vxnAAgglv9c3K58LsYZ3znFJaxl2XWfSTzmfcxwj4HJ7Xf0nL+20b+1MhkUme4xcMyf"
    "OO3/5cfZjEADlNgYGNQ4IIYBBBDDwySTh+9eH/o/68ib//uuPvjvON/jv+gkqf9SLUr/S5f0"
    "/3/6/v//DWfe/n+yIOu/MQeDISOS80HeBwgghvveXJ9/XZv+f+m+a/9B4Pu3H//vPb73/83r"
    "d/9h4N7b///3bFz9f7mH+DIGNAMAAojpzsVvq1jvnWbgZXzGcO/1PwYOTnYGRRlFBmERQYbf"
    "/xgYvv5mYBAWYGB4wm3NYOhgGqzByqCJbAhAADE9esWw8OPhE/90BJ4xXLv7Ciz4DxgxIM1/"
    "gPjHHwaG7z8YGDTVJRmui3my5ZhyFiEbABBATB9/MRy7dvzuFUXWewyvX95n+P0fYsDf/xDN"
    "P/8CaaArJIUZGN5I+jEYW2hGyTEzKMAMAAggJqC6P1ev/tzA8OIagxDjXYY7z4DWMgFt/Q0x"
    "AOQKkGu+f2dgUNGSYbgvG8yVoMGQCzMAIICYwDHxlmHdizPX/uuLPGQ4e/UpA1Av2GawZij9"
    "C0hLCAEN1U1iMDTSSBRnYJAG6QUIICZQtAANv3z24MMLCoKPGd4/vcLwEehnRqDRzMD4ZmYH"
    "GgJU9PoLA8PFa38Z7jzlZvgs7yCYpstQAjIAIIBYfoCSI9DbN05/XuP95amhHPM5hoPn3RjE"
    "gSH/AOi0N89fMvx8/5yB/e8LBjHOdwyuwl8ZTCN4GS68U48Eai0ECCCWr1C/vH7DsO7+uSeN"
    "7jbPWNYc6GJ4zsTMoCr8nsFF8TuDjAkwnXMCrfr1GZh+PzAwnHvHsHX/s0PWQH0AAcTYCPEC"
    "AzC8GOwC+Y/5Lgq2ZPgGDIUv7xkYXn1gePfgPcOjW+++3rv96cXte9/v337678ad1wyXT35j"
    "WP/9///XAAHEGA7UCAwfcLy85mYK9Inn6X/77u+HKze+X7755N/l+28Zrj7/z3Ab6NKnQCVf"
    "0VMiQAAxUpqdAQIMAM5oca3d9yYGAAAAAElFTkSuQmCC")
index.append('markuncompleted16x16')
catalog['markuncompleted16x16'] = markuncompleted16x16

#----------------------------------------------------------------------
folder_blue_light_open_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9wBHQ8BGUAXDroAAAGY"
    "SURBVDjLpdO/ahRRFAbw38RRIWLjK9jtouALWJjawk58gBWsJC/gExirFFl7sRCxELERkohJ"
    "KtFiBoxd/pgQzUJIdDO7M3Mt7iSu0SSCH1wOh3u/7zvn3HsTjOlklaPottv4jKET0cnC36CT"
    "BdzAOFKcQXKUnsL67p+6YaYlkb05raoU+mVM6sBYEuNyL4qMIpFluu0JLKGPkMJOwaDm3Fg8"
    "OPOJx0/y41qOVa18nPT6zkwK2/u/9j/0Ivn7dMt4evzokrumkKewNSLwaj3G5R5VTRko67ie"
    "P9t189bFUZ2rKWz2Y/Z+m7mXuYUHLXuDhlxR1FFsb5/tgttTOd32C6ymsNEIzDbuO0V0HAYG"
    "VYxlTW+fXnHovoL5FNZ+kPfYWMg9nWz5VsSbKGuKikrTRsW96UP3RWylML/G5mKc+teGXId4"
    "M8PAsI5rd/Cb+1uUcQaLucvXW+5fceheNcSDOAzM7aweuC/hguZptnSyzL+i236Eh/iCkOAs"
    "rmECl06h72EW71Aa+RwJzjdiJ6HC4IAMSQjB/+AnG8TfTdDZEucAAAAASUVORK5CYII=")
index.append('folder_blue_light_open_icon16x16')
catalog['folder_blue_light_open_icon16x16'] = folder_blue_light_open_icon16x16

#----------------------------------------------------------------------
close16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANWSURBVHjaYvz//z8DJQAggFhA"
    "hIeHJ8Pfv38Y/v//w/D583cNdna2fHFxSX8BAUGRv3//Mrx48fLdo0f3tv348bX//3/Gyz9+"
    "/ACq/wcUf8EAEEAsMJMYGRkYvn37ka6nZ9gXFhbKpaenx8DPz8/w/fsvoMLX4hcvXkhcuXJZ"
    "7OnTR6sZGP73AbX8AekDCCAmmAEgzQ4ObjM6Otq5TE1NGcTERBh+/PgJFP/GwMnJwaCsrM5Q"
    "VFTD4uLi2/nr168imD6AAAK74MePb1ra2joTCgsLGF6/fsvw4cMHBlZWdqAhogxfvnxnePjw"
    "KdA17Aw/f/5hCA9PZHj8+EHbmTPH9wC1ngMIILAL/v37nxMSEsoBsunLly8MurqqQAMYGW7f"
    "fsjw6NFTBhERXgZDQw0GcXF+hvfvPzM4Ofkys7KyVoL0AgQQ2AX8/MKBHz78ZWBiYgY6l5vh"
    "/v1nDOrqisBwecjw588/oIHKDPfuPWM4c+Y2w+/f/4He+s4gICDsCtILEEBgF/Dw8As/fvyS"
    "Yc2a7Qzc3FxAL/xguHPnKYOSkizQIAWw5kOHLgE1/ma4du0Uw5s3Txm4uHh5QHoBAgjsAlAU"
    "srKyADU9YHjw4CGDmpoq0P9aDBwc7EBXMAFdxsLw7t1bhqtXzwBd8JWBj08U6DJwJDAABBDY"
    "BR8/fnj/8+cPoOJ/wID6xaChoQA0RJ4B5Ko7dx4zqKjIMfj7OwBdygXU+BuI/zJ8+vTxG0gv"
    "QACBDXj79uW2Z88eA21iY4iJCWHQ19dkePbsNcP06fMYpkyZBQyTp0CvKDOUl5eAbQcloDdv"
    "Xh0E6QUIICZINH7pv3r11B9gbDDcvHkHqPkVw8yZC4D0I4ZXr54yTJgwBRyV586dA3qXgeHG"
    "jUv/fv781g7SCxBAjKC8oK+vx/D16+dibW3zHh0dM2C0/gJqfARM2v+Afv4FSicMQkIS4Bi4"
    "dOkcw9mzR1qAeuuAev8DBBALJBkzggJr4pkzh1hfvXrepqKiyygkJAz2LxvbLyDmY3j69DnQ"
    "5ssMd+9e7wBqaQfqAedCgAACu8DAQB8YRV9ByRmY9r+ZMDOzlAoIiLhzcHDzgEIbGMjf3r17"
    "uf/Hj+/dQD1HYJb++/ePASCAGCnNzgABBgCb8HiZFej7+gAAAABJRU5ErkJggg==")
index.append('close16x16')
catalog['close16x16'] = close16x16

#----------------------------------------------------------------------
calculator_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAkvSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAs6AIBASGcf/78yRUUFIzj5OTgBgbQP1AgMTIyATHCzYxgDiOYRrAh8hAaJgbiMzL/"
    "/Pnz9d27t+cfOrRz/p8/v7/DQh4ggDAc8Pv3n0xPT/fO0NAgBiYmZgZmZiYgzcTw/99/BqBb"
    "GP4BaRAGCoBdBhKH0TD5/yC5fyDT/jH8+vUHyP/L8PHjJ/mdOw+bPHr0gO3OnSvTgJK/QCoA"
    "AgjDAYKCAhEBAX4M4uLiDN9//GD4/uUbw9f33xj+Q/30D8QCccA+gKaf/xACBMH+/g/B/5kY"
    "GQQE+BhYWVkZpKQkGb5+/cGwdeuWIKCSTUB8D6QLIIAwHMDBwcHDysoCdPkvBqb/jAxd03sZ"
    "1j5azyAgzM/w7w/Mp7BQ+A/1MdBhQF//BwcMKERAzvnL8O/ld4aGoHqGgMAAhh9Az3BwsDOw"
    "sDDzAK0RhjkAIIBYMJMF43+Iof8ZgBHA8OzLawZ+tw8MKsqsDF++/WX4/RcYdlAMYf9n+PWP"
    "keH3H5jYf7D4T2DUfT7xiuHDh/cMzIzMkID6D3YsE3LiBwggDAdAEtR/sI+Y/jMxsADTATsz"
    "BwMbGP9jYEQEPDz0wZgFwQa6B5x2GFg4gemHERo6EF1MaPkOIICwOgCsGBrpf37+Yrh75x3Q"
    "h8wMP74DQwCYuP5AffkX5mOQ2L//YPaffyBxoAeAFjM8ecvwT/Y/1FNwG1DsAwggFnxFw4/f"
    "PxhCnf0ZlO6rMLC8Y2X4B7QRZBgLyHCgI/8AUzfIpYzMjAzMrIzg9PAXpAYkBvK5BQODobEx"
    "w/fv33A6ACCAMBwACSJIwvr15xeDgZ4eg56GLoOYiCgwNbMDLfnD8Pzlc7BDxEXFgRaBEuw3"
    "hhevXjFwcXAwCAuJMoCi+eO3rwxvHj9iYBcVBIYQMDoZsJe4AAGELRGCLWdlYWK4eOsuw7FT"
    "v4BZT4BBTuo2g5ebCcORE5cZrt7gA+cEbbX7DOYmmgw79lxieP5KhoGJ8SWDjdUrBgkpKYZT"
    "1TUM0mfOMDxWUWHQaGlhkAJma0QaQwCAAGLCDAGI0N9/vxgeP/3NoCJrzGCsocPw7Zs4w7Pn"
    "jxieP//HoK9qyKCrpM/w9i0Xw6MnDxi+fRViMNPWZpCX0mN48uwnw6u7txhUPn1isExLY9AB"
    "+ubNpUsgH0GKDgZUKwECsFHuOgDBAAC8qJLoQPUhYuz//1JN9rJYEE03if2mS+5+DZTGMzj7"
    "jiMlZHsz9k+elGOZLkSdqMSNMwJrR4zOqcmTRu4Yqxi8InrPGiOb1tgQeHKnZVJfAbwCCGcu"
    "+A1M2hoakgzPnn1k+PXzE4OBgiAwa3Ey6OpJMDx9+gJskISkOAM7GweDgT4Dw8uXDxhkZBmA"
    "JagIAxMLG4Mk0PePb95kEFFSYhCTlWX4++snOP2hOwAggHCXA8CEw8zMxqCgIMoArJyAbFZg"
    "afaTgZ2dk0FRkRWaS5nBJSa/AD8DF/dPoBoWoNp/DH+AYpKyMgwiUuIMLECxH99+AHMKE7yC"
    "QgYAAYTFAUzwtPDu3TuGM8CE9PXrVwYdHR0GFWCCevbsGcPZs2fBDjUwMACW8VIMt27dYrh2"
    "7RoDLy8vg4mJCQMXJwfD6TMXGa4+eMugJMnDYKqvycDIwsEAqVVRHQAQQFjTAAM0rq5fv84g"
    "LS3NICwszHD16lUGISEhhhs3bjDo6+uDQwhkMQg8efKEwc7ODphAnwPlrzPIAn1/7o0IwxcJ"
    "Z4Yvn28wiAGjTE1FEamaRgCAAGLCFgIgl/758xdYcbAwSEhIgDEPDw+wQAEmNDY2sK9BmIuL"
    "iwFYz4PlJCUlwepYgEH989d/Bk4eIQZ7dQYGAUFhhh/AUpQBXhyjOgAggLBEAaTSAAWVgoIC"
    "OMhfvnwJzAGC4CoaVNLdBCYukBqQhSCLf//+zXDx4kUwLSevwMDDzcWg+PYdw8O7DAySzG8Z"
    "FKUEwcU3uuUgABBAOEIAUqQKCAiAExkoaEHRABIHOQSUNt6/fw8WA6UVkBgoGkCJlYeHF5hF"
    "WRgUhb4zCH47zqAk8ImBHVhC/gO3UP5jpAGAAGLClQZABl8CFiAgX4HSwblz58COuXDhAthh"
    "/Pz8YLFPwAIH5HtFRUWwPCh0Pn/6wLD7+l+Gi0y+DLtusYFDkYWJCd6EQwYAAYSlJIS07UC+"
    "AVmuDSzhQBhk6StgeQ/yCShHaGlpgdPDixcvwHK6urrAckOD4dePbwzvPnxnYOaXYzBV52Tg"
    "ElNjePsF5PP/WItigADCWhuCSkKQQ0Bx/PDhQ4bXr1+DLQPxQQnx7t274Ojg4+MDpnhZMB/s"
    "88+fGMQkpBgE+PkYhF+/Z/j4moeB99czBhkZXnB1ja02BAggnGkAFAJycnJgF4OCUAlYooHY"
    "IPrt27fgNABig9p7oMR6//59sF5QomTn4GQwVwCmoQ/7GMxkvjIIAh0EKqAYsBTFAAHEhKsk"
    "BJVqoMLlzZs34OwGKnxADgPFN6x5df78eXC8g2hQmgCFFCg0fnz7wrD38nuGsz8tGHZf/QY0"
    "4xWkhYQlBAACCEciZAK6+Dc4gekB2wOGhoYMnJycwDrgKbhUNDIyApeCoPTw+PFjsJypqSk4"
    "rXx4/47h5duPDN84lRnkFaUZ/gnqMjx/9wNYVWOPAoAAwkgDzMzAtgwjJM+CEheoDAD5EgRE"
    "RUWB7fuPYDGQ5cAWNLBskGD48uULOKuCQksQWFoK8vEwcDz9wsDN8Ifh88/3DELinOAQA+Us"
    "JrRGIUAAYYTAly8/Xv/48QvUfAaW/crgOH7z5i2Dmpoq2KdqampANZ+B7YNvYDYvLzeDMrC6"
    "BeUGUHNeXl4eGB28DPaqQAd+usBgo/SNQVpCGNxU//z5G6jk/I3crgUIIIwQuHnz6pIdOw7b"
    "OjtbgR3ByQn0DQcvyGHAJvZXcCLi5hYAq30LDOq/f/+BfcXLKwjOPc+fv4GUpEAxBWEmYGP2"
    "H8OTp2+B0fmV4fjxqwx37ly+A/InzD6AAGJE7x1zcHByKCpqZAkLi4dAOxGMiH4fA1J/ELn/"
    "xwhtR0DKEUgiRYgDxRiBVfnvW7cu3Xr27P5qoPROoL2fQeoAAgjDAdCCgg2I5aA9GGr1oP9D"
    "ff4YiD/C7AUIMAArL9m2UmiLGAAAAABJRU5ErkJggg==")
index.append('calculator_icon32x32')
catalog['calculator_icon32x32'] = calculator_icon32x32

#----------------------------------------------------------------------
earth_green_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQnSURBVHjaYmRQ0WZgZPnP8J/l"
    "DwMDyw8eLuk/JVpaipFG2joyEiIibJ9/fv53+f7lj2cOnz7x4fiXDkZBhmOsDCwMv24C1QMB"
    "QAAxgAxgUNNiYNBUNNfMM7075Vz//xd/Hv7////PfxD49//3//f/X/3f9GrVf+dZlj9YrRmm"
    "MMsyMcAAQAAxMwiIAal/5gaR4juai/Mk3SS8Gf5/Z2b4/e8vw9vv7xiev3/J8OvbXwZlDnUG"
    "Q0MjlldyD83unbqv+vc5wzqQAQABAEEAvv8DAA8VEh5CCLtlYk8YNAg9APgA+gAJCgkADQkM"
    "AOnx6ADo8ecA8fjyAO327AAI8w0AGBIbAOIZ2S3nyOv6//UAxAKIiV+WM59fm1Hh1fPvDAwC"
    "DAys3CwMf9n+Mjz/8pzh0fMXDL9+MTAAPcPw9ccvhlcf3zI8OvWB4cd7Lka5QNFioAMkAQIA"
    "QQC+/wQUJAlY/hH4ALwNuQD67fIANwk4AAniEQDgzO4AAAH/AAMFAwD8/v0A7O/vAM3Z1ADB"
    "xMkAAAn+ACtSHRPmGe5eAoiFR5JF9sNDVobvQMcfZz/BcJ//DgPrc0GGy7s+M/z7wsjAwPyP"
    "4du37wxvPwFtZvnGwCzCysDFyM3w+ysTA5c4uy1AALGwsDGzvr/0h+HLfkaGb+b/GP75vWfg"
    "EfvKwB/zj+Hz898M9zZ/Zfj1E+gSThYGXiFOBilrDgZ+Dn4GSSF2hmdsrMIAAcTy7zfDPw4J"
    "JoZfjH8YPt75y8B7/z8DE99vBhYpZgYhOQagGDvDp5vAdAJ0ibIXD4O0qgCD6DdZBklxZoZD"
    "3/7+Awggpn+f/n7kl2JjYBH6yyDvwM0gpy3AwPqHneHvDwaG/78YGTglGRj+cvxikDbjZDC2"
    "UWL4CrTg6uWHDK8evgUG5vfXAAHEzMbGYSvjIKkupyLGYOukycDGxM7AwcDJwMPCw8DLysvA"
    "x8vNwCvAxWBqqcogwizCwPmLj4GNjYXhyeVr/+5teTIFIIAYmdjYLSwrbQ64Jzixy/JIMbz4"
    "+oLh67+vwKj7w/CP4T8DGzMLAw870BAGPgbO3zwMlpLmDGee7WfIcC+89/XGT1OAAGL+//f/"
    "k7f33onommlYBBj4MAgzizMIswgziLOJM0iySTJIs8kwSDHLMChxKjPYi9oyvPp6nyG/sOTX"
    "8/1vi4Dp4CRAAAHjmpWBgZ2dgVNFcEndqqp/93/c/n/y98n/J7+f/H/xx8X/t37f/v/9/zcg"
    "/vx/y52V/zUiFH8CNVbB8gJAADGCDGAUYGf4/wco/ud3uYG/fgkwQwizy3Iw8vFzA0OQgUH6"
    "myTDvVO3/22Yt/Xhz9t/G4H6FsIMAAgghAEs/xgYXn8DiYkzcDMncQixWDHxMAv//vXr/++3"
    "f14xfGDYDZRbAcTvGJAAQIABABoRfoj98pqeAAAAAElFTkSuQmCC")
index.append('earth_green_icon16x16')
catalog['earth_green_icon16x16'] = earth_green_icon16x16

#----------------------------------------------------------------------
palette_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAZMSURBVHjaYvz//z8DLQBAALEU"
    "CzIyMPxjYPj7m4GBiRmI2YBcIPvvXwaw+I/fDDw//jK4AaV82PlZTXmEWMS4uBh4mVj+Mv39"
    "/fvnu9cMb968Zjjyj+F/LxMDwyUOqMEAAcSCzbZ/QEP//GKQ/fOHoVxImidYw1JWTNVEiElC"
    "iYNBUJiZgYfjBwMLwxeGv99/sr95ysp38+xXpe0bn4bdfPitCWhwO8gMgACCG8zICCIYGH5/"
    "ZxD7/5dhoqK5kr+5nwGnroUow3/eXwx8QNuYmf6DFbx6J81w4Kkhw6mPugx/2LkZLK0uMmRY"
    "beXYPO1465FDL4U5GRhKAAIIbjAoqP98Z4gXkhHtdykMFLTwVWZg/POCYfKD8wxbn99iEH35"
    "hyFLSJXBRliQoeJUNsMRdgcGcaAJLEB87r0egyjvE4asus+Mbwp+F9688u4GQAAxWwLd/g8Y"
    "ln++MdTreJj1xswo4dK24mJgfnWTYca5iww1Ai8YvvJ9YLjI/o7h9OPXDF7/RIEW/mG48pib"
    "4TenCAOXFAsD06e3DO5/NzLIqfxi4GBnYzx/+IUyQACBDf7zkyFVx8OmJ7SliEno7yWG349v"
    "MrCwcjD8/vaP4c71xwyML78wyH1gYvD4Ic7gIKfCYK79isGXaycD55vrDG/eMTHoftzP4KN0"
    "iYGLm42Bhe0/w6UTL3kAAogxn5tBXlBc9FxkZ6KQlNolhl8vXjEwfhVl+PeDhYFXTJjh6+//"
    "DM9ff2Dg52BjkBTlYWDiBIUzMNmAo/87w7d3PxjYOBkZWJiBYfKHieHd5/cMfVWnfgMEEAsz"
    "E0OtaUiwkKDgKwYexXsMDKLiDAxvmBk+3fzB8OLBWwZpa1UGQXlBoCZgUvn9k4Hh5y9gwLIw"
    "/Prxh+HunpcMjBeeM3AKczHwu8kzCCjzMfx68wuo7P8/gABiEZISddH392b49mA2w58PvxlY"
    "fv1m+HjrJcPTu9wMEmYqwJQATMHAhMzwF+jSf4xgQ/8Ak83p/rMM/9bdYtAGuhuY9BlunH3N"
    "oNtvyfD27ReGj2/+fAQIIBZBaQlREXVRhjdPeRgerGRmYGX9wvCTWYpB1k2fgVeYHWgoxIWg"
    "pMjw/x+Y/vHpF8Pnsy8ZLIFCPED8AoifXXzNwLTrNsOjn78ZPr39dR4ggFhYWZlYGIFZTNpY"
    "leGHghgDAzDS2CQEgWHJysDw6TMDAzPQsP+Q3MnADHL5PwYeAQ4GpVR9hmszLzD8ffWV4Q0w"
    "0hh1BBl+8/5mOLv19W8mhn+9AAHE8uPj+6+/v31jY2PmYOAQ+sHAwCfA8OnNR4aP9xkZxFTt"
    "GNiZrjIwfHsJzOvADPuPCZKLfv5lUPVSYhAxFGX4cP89g7oIN4OA/E+GTYtuMDy9/OkA0Em7"
    "AQKI5f2TR3efX7tlomKnzvDzwX6G6ztuMDw8dJaB8c3TX1qR39lUvB2Bkf8SmjVBwcEA9gEj"
    "MNyFRDkZhKR4gJnrA8Ox3Q8Yjq99+YGN+X8qEzDkAAKI6dtnhqU3dmz4/+3TX4b9S/YznJ+z"
    "+QfT4yfVPP/+e32/ffrn/79Qw8BhAXLwfwgGZdXf4ABnOHP4FsPeBU//fH/3Mwto6EOQMoAA"
    "YuHgZphzfd+Bwvcvb8h9vfPkLj/nv9C/zAznP39jCJfgF2dmBBd9v8BhCy6d/v2H2MHCDGR+"
    "Yjh78hbD/sVP/n19+bWem59hOawQBgggFmY2hi/fPvxKenzsUaCwCEMDAxPDmz+gYvMHgw6r"
    "sAwLw//f4KwJDl9Q3gfpZGVm+PbtFcPJgzcYTq9/8vfzs28lLGwME5BLdoAAYgEpBMbLXmYW"
    "hr2MQL1//0AwsEBi/Q9yIdAmht9/QEkCGARANsdfhlfPHzMc2XqD4e7+lz9/vf+RxcrOMO/n"
    "T0jowABAACGKTWjE/IE6Cpgczz49d/qPmr8lCwMPMAsAdX7/8onh+qkHDGd33GN4d+3dfYbf"
    "f2OAYXrsH5ZKCCCAmB0FIKYCgwyYOSDFACgogGqvvX74NI6B4bXQP+bPDHfOP2A4uPEqw+Xt"
    "9/98f/hpIRPTfydg4ngMyjOgIAfVOMjmAwQQSg0CshlcJUHB39//0/ZNObyYmZdd7O/Pv98Z"
    "vv85w8L4vw3oyr3/mRAJBRsACCBGWlWmAAEGAEMGd0ythN3dAAAAAElFTkSuQmCC")
index.append('palette_icon22x22')
catalog['palette_icon22x22'] = palette_icon22x22

#----------------------------------------------------------------------
decpriority16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAK6SURBVHjaYvz//z8DJQAggJgY"
    "KAQAAcQCItjNZh/nYmAw+AN0zLe/zO/+cXGHMLCxH2f494+BgV+agYERqOjvPw2G/0zbWdg5"
    "JP6zsDL8Y2a79W+Zij5AAIENUJLitegqtWL4wczEsP34S6n5i25OYGBldGFgYfsM1v33Hzfz"
    "33/T1XSkFP7zCjN8+MLI8Pr1Bz2QXoAAAnvh1Yuv97mZmBhCLWQY0lIMGQxMZMwYPv+vZOAU"
    "BOoHGvD1T46RqoCDiZEsAwMXD8MvBmaGv38ZX4L0AgQQ2IB3b38UpTUfZ7j1/CsDFyvQoGgD"
    "Bn4x3lyGn3+MGP78UxEW4ShVN1JguPScieHVmz8M71++Z2D4/78EpBcggCCBKMSz4e69z9Nq"
    "p15h4Pj5n0FSio/B3EmNh+E/43Rmhv9ztAykhe9/52Z4+paB4dO7Twz///1bxsDEvASkFSCA"
    "IAawMDMwCHI2rt797Mr6vU8ZpIEhqqguwyAuL2rGJyJg/19QguHBawaGzx+/Mfz5/uMeAzNL"
    "PdAJYK0AAQQx4N9foF//v/rPzlLdvfDun1cvvjHICjEyKOopM0hrKDI8fc/I8O7DH4afn4Fh"
    "yszcCNRxBxI1DAwAAQQxgI0biIHWsrFtevvx7/Spqx4xSPH9Y+DhYmP48ped4e2n/wzfP34G"
    "qVzMwMi0CIgZgF4AawUIIIgB/DIMDHziwBDmZWDgZS8/cfHjpYOn3jHoyzMwvP4EjIRP34FR"
    "+fsuUFMeekICCCCIAf+BXvj/DxSyoGj7zsDGkr1699sPf379ZNCW+svw99PXHwxMTCDNH9AN"
    "AAggqAH/ERiU+liYjnz78X/m5oMfGb59+gJSMQ9o8nZsSRkggBhBmYkx+i66MMglnMBEvo+R"
    "lZ3zPxOjPVDoI7rm/3OEGAACiJHS3AgQQBTnRoAAAwBe7eXig4nCewAAAABJRU5ErkJggg==")
index.append('decpriority16x16')
catalog['decpriority16x16'] = decpriority16x16

#----------------------------------------------------------------------
paste32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAaZSURBVHjaYvz//z/DQAKAAGLB"
    "EDFhZAC56c8/BoZ/QPwXyGb5y8BwlUeMIVPfk+EbI0s6L8//nh8s7/99+vPq3+d/rxm+/vvA"
    "+OPnZ5Y/nxg6hf6wNc+49plB99M/hi9A44BGMPwFmQOkQV61fYLqYYAAYiHGlT+ZGBgM379i"
    "UHt1R/COgUnN9Mw0Hg52VoaPfz4yfPv7DehQZoYXX54wZE0sL1A9+3mu2cd/z94zQW0kAAAC"
    "CJsDlBl+MXgy/GFgAhoAiSFQaPxn+M3745ONmb2CzEfxVwyv/vxmYARDJgYmxn8MSpIyDHbO"
    "tkLPj22e+fsfw+Z/vxjYgXrAPv/7l4HxLzBgmTgY9gO5V5AtAwggDAf8/8cwi8FS0YlZXoKB"
    "8dcPiBWMjAyPP/xkeM6mwGCo9Yth5/PdGL5jYWZlkNbjZzgtL+jz3kjWR0WCjeHHb5DbgQpZ"
    "2Rh+PfnI8GT3jfNApUbI+gACCMMBQFerMjlrMzAJfWJgun2JgYENqITpL8N6RlmG94aiDMc+"
    "HWD48f0XAyPQUaguByrlZmb4as/PsOD8V4ap6g8ZGH4B4+E3MAyUdRn+/jJmeLb3hhS6fQAB"
    "hOEAYKL7xfQTqOnzJwaGH1+BCQCYAD+wMKxhEmL4wHGL4dG5RwzMjMwQGxmhwcAIcQETMxPD"
    "X/FfDAveMjMkXPrJYCr1CyL36RPQ3N8gs3+j2wcQQNhCAALY/zHc+iXEUP9+AsNXnn8MNz43"
    "M3zZ+ZTh5zdg3P9mhqTMP0D6DyvQ1UA2MCGCaEZmRoZ/n1kYcl92Mki/5meoka9kMFQFJqLv"
    "SGYjAYAAwnQAyC9MQIPYfzKseW3DYOAdy5Br94/h5E0lht//fkCDCajtDxD/Y4Lj//+Bqewf"
    "IzjrApMDg5G+IcPCAywM8xbuYJhse5rh/3dIlkYHAAGEGQUgApSFmP4DfcPMIMf7j4GLhYnB"
    "UduW5EJGTvAvw0NQWmH8C/ccOgAIIOwOAMYlA1CGkfkvMAvhzswgGVA2/YeEwbkWaBMnGzAN"
    "AhMgIzsnAwO3AAPD2/9YowAggLBGAQMojbEANTKzA9MQHgf8h6gHWwyzHInNBEx4jHwaDAxi"
    "QNc8fA0uEdEBQABhTwPMoDwlAnSEEDCh/8Nq+T+Y5f8QPgcphZZbwHKMgeHDd6CvQUHKyASW"
    "wBYCAAGE4YA/jNA0wCoMDAEBoAP+4rUc5ltYaPwCEj//QDLIx28gtWBZuMPQAUAAsWCLV1D8"
    "MzCyYS2p/zEg4vkfkuWg4P3xB+IAEBtUgYHTH8zbQKP+MmI6ACCAsJYDoGzIAK0EGInwOSid"
    "fgda/vsfIlH+hUkywBIEI9YoAAggzFzACIkykC6Q62GJ8D9aqmdAtvw3pPoGV+NQsX/IGkAU"
    "I3YHAAQQtqKY4T8w34O9/v8/PATA4v8QloPoX8BgBpXaf6Hif6Dth79QNgMsCkBxwcKINQoA"
    "Agh7GgAGF5SBCIH/qFHwE4h//UFkwz9IQQ+jURpbjNgzNEAA4SiIgJiDG1gfcMFN+QttIf3+"
    "D6ng/vxHOArmc7gD/kFDABqNYACsI/5gcQBAAOEoikFBxgoOhR+/IQaAghpmGayw+YvF5zDx"
    "v//RWkTM2BtIAAGEoyBiRElk0FYNPLhhbUVsPgezoeniP7jGhrjqPxP2NAAQQNhDgIURniBA"
    "MfcX1rj8j4r/ocX33/+IBig8xcMSAsgBWEIAIIBw1wUMEMuZgBpBwfznL8IyuCUwGpom/iGL"
    "gwsipDwLKgewhABAAOFsD4DAr9//wfU8F1DjP14kB/xHWAijf8NCAxotbJwgc5gZfv+CtphA"
    "DmDCdAFAAGGJAkZIIgRGmLnKe4ZJa84zPLgrD0yEfxn+Q+P27z9QJQPEQBrk0/9AGhxC/yCV"
    "D4gGmXEL2HzLs/4ANus/qN2CJQQAAgh7FIAKom//Gdz03jMIsy9lOP+AAyj0D5qgkFL3/3+w"
    "5iCijQhKP0B1f4GlY6r/bwZLU6CJX9mB0coMdAemCwACCFtdwMjIJ8TAIOIHzHtfGIzV/zMY"
    "s/1DWMoIsxCLd9CFfgMFvgMxOzcD48O9QG2Y9SFAAGFWx3+ZWX9/O8LA/OcGw39QifOTAV7t"
    "/UdpgaDTEDmQGlDhA25GQEOMkZWJ4eezd8DKioUN3T6AAMJwwI8Pf1edT76RysDx//+/P+Do"
    "hdR+DJC4h6Xyf9DUj2iU/McoD5CaaYw/fjAwfXr9dyu6fQABxDjQvWOAAGJiGGAAEGAAxXKM"
    "7ztpgQYAAAAASUVORK5CYII=")
index.append('paste32x32')
catalog['paste32x32'] = paste32x32

#----------------------------------------------------------------------
bomb_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAL4SURBVHjaYmDAAxiZmBlYOTgY"
    "rkQxM/xPY2T4X8o2aZYj2wUedkaRJj1mBhM+RgaAAGLCawAQ//7LwDDvNlCZEJAnzhS07cV/"
    "8WRLFvY0K9bEzz/+ywIEEAteA0AmMP5jWHH6H0P+N0aBu8o/nwvzMHBlWrGf3Hv2v/CbXwzZ"
    "AAHEiEUfGxCLA/EvoAEvOQSFbAwFBZzr791/sez/f15xa0bHr58ZtQ7dZeh+8/f/IoAAQvaC"
    "JBMT0xwuLq67zMzMV4H8G6ysbBdsLSw3BUQGNSw1NWjaxMBwmI/h/+1waxaBaQEcwVpcjHoA"
    "AcQM1ezm4OCwp66uzsbf359PXV2d/fXr1xx///6V4Obg4Nxz9CTD7Z9/GD98fD/3ySeGI68/"
    "Mqi/+/Rf79Szf38AAgjkBS1HR8cjM2bMEOTk5GS4evUqw6NHjxiuXLnCsG7dOoanT5/CXPgS"
    "iM15WBgeigM9efcbAxeQzwEQQEwcHBwdnp6egh8+fGA4fvw4w507dxgePHjA8OnTJwZ2dnZo"
    "YDKCsBiQmf0NGCvPf4KFvwHxO4AAYgL61+bly5cM58+fZ7h16xZYM8jW+/fvMwC9AVb5//9/"
    "EAa5NujffwYukCEwABBALN++fePZvXs3w+fPnxl+/frF8O7dO7DGu3fvMnz58oWBlZUVrhgY"
    "JsL//v2TBjJvw8QAAohRQEDgO9DvHLy8vAzc3NwMP378YPj69SvYMGCswJwPVvzmzZv3QHlT"
    "IPMuzACAAGIBeuGxnJycKkgDyKnAMGEARiVYM1wRCwsD0KUMr169AgXkc+REAxBALB8/flwM"
    "pBuVlJQYQbaDDEFRANQMtITh4sWL/4GWbIYGHhwABBDIbSxA5++ws7NzBroE5E8whmkG+hmk"
    "meHMmTMngGxPoPAHZAMAAgiUkP4BTV4DDHlloEZ1YHgws7GxgV0CdDLD2bNn/wDTxD6g5hig"
    "2lfo6R4ggNDzgiswDGKBWAFoADMwMJ/++fNnA1B8FRD/wZbhAAIMAIHdM3X03RvoAAAAAElF"
    "TkSuQmCC")
index.append('bomb_icon16x16')
catalog['bomb_icon16x16'] = bomb_icon16x16

#----------------------------------------------------------------------
newsub22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAACXBIWXMAAA7DAAAOwwHHb6hk"
    "AAAEG0lEQVR4nLXUa2xTdRjH8b+QECYjmOgLE4Im+AbDTchgExruXoPiTDQiJBtTQ0ImMJUC"
    "ZUzkMmCLY1MhaMCEBJAsKd3GEBwKdNMpFcPATWBjbCC7tLRru+605/71xWm7TRj4xid5kpOT"
    "nE9+53n+5whA/B+dvEi1rT2cubXKyC6pMZcXnzYe1ssSXXTaeLf4tJG154z5XM7eoJiUbRNT"
    "cvrhReu/kxX+e5kD2ojfq7p8lxG2da5H59j74SVbXIoJLH0nC1034o8MXYYJugGaDopu3fvp"
    "ag8pCzadGL0ovx9evNkpa3E4Ub4Vo5KdSKkboBoWJmsQ1aAv/qqn/vQzcp6jMnXBpvvDibZA"
    "gW/FKCuheR9UhV7FGscPTX5S5jsqRy8cAk7U7SXjk4l1E7R42gQqqVbaXhl04PglPyPmOCpH"
    "zhsC1o175/vvpAk0HAVfBCI6fFXnR6Q7KsWsIWBFUdF1E9mEGCANQGNxNKKAX4LOXrgdhqAK"
    "X9b6ERmOSjF7CFiWVVTdRDasVH2qBcY0iKoQkeGuBB29cEeC9jAEZPji/ENgAKNPGjSKqAaS"
    "Zs3TG7HQjijc9Mu0hsAXg7JzfkTGxkox2/Hg5V19f3pyeZIK4Rj4ZPCa0KnD9cvXaNj0Go0n"
    "juMDiuslxKxtLjF3x4OP263lY5LHLRgDrwI3Wm7y297dXG9u47KrisCxiTSXF9ASMjlStI+P"
    "nplxzpn65L1worS7gUEfSHcEfED9N2X8Xb6MK/vtuLcX01v7Ek0HCvijbBva72PpzBzd5RZi"
    "WBJ+Jd8pK1gfQTS+9Z6YtaTuPmv73To0t3fRcMBOpHEHF3JmEHK/zrX105AaZuIpf5Hcx6ed"
    "dIvh/YmnrHHKAQ1UHUIxCEQt1BtHOyRovNSEp8JFzWeF1Oc+j96+DsO7GkJv0XNwEqWLV7F6"
    "yvxbf40ZnpWEJ+Y6ZV/8WA1CI3AnDG0hhbO7P6DjTD6Bizvxu/MwfFuAUtBXEvHYUK48Cy1P"
    "07X7KSMJT8p1yl4VJOVe9FYY2iMGF05W4Tm8hfqyDVxcOxN8DsxYARjZBA5Opjx3Kc6V6VJ7"
    "WoprUOJuxfqhDEJD0BaC1h5ojUKjT6G2dCs9no207JpH7y9vcGffVIymdI5+uozssS8ca04V"
    "/TOevKZCDWP9pSI6hDUIauCXwRuDTgm6AM+Rb5F+/BBvxQbOrMol7H6VhsJV3Pw6C9rGc/Xl"
    "xzrrBp6KJzJLugrP+tj/a4iyuiCltUFKaoN87g5SfD5I0bkgRfVBNu+tofS9t7HvOUtZqRO1"
    "fiEnDm0n79Bt9q18k1MTxlTUiQGJxdRsm0j72CUy1n8v0u3Vgzoj3un2amHLr34kfUO1mFtY"
    "kTYh8+fuT8bpebY5N4RtZ4WYbt9VMHzcMI8Q4h9Ycc5rAmiRuwAAAABJRU5ErkJggg==")
index.append('newsub22x22')
catalog['newsub22x22'] = newsub22x22

#----------------------------------------------------------------------
keys_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANESURBVHjaYvz//z8DJQAggFhA"
    "RHh4OgPj+vnqYorCMZoWcu7qJvKqj2+/uH1u96099268rBJA08QExfOBlgMEEAtM0Eifr1g3"
    "Pyn1F5ciA5vodwbngPemFmbvjCZXfHvN/p/x8/+///79/8/4n5GBkeHb59/vv3z/vgGkDyCA"
    "4AY8/8HPLPBLk+H7e0aGR8+4GZ69VGIQ4T3G3LLjSx8LMzMDI9C2/38YGZhZGBne3f3FsLDm"
    "3VqgthCAAAIbAAwH+b9WQUb3775g0NPRYODl5WBgZGJmePezluHWjVgGY4V7DB8eMjCwsDMw"
    "vALSXFwMDL+/ML0F6QUIILABurqaB7R0tBXOnznP8PXrJwZXV0uGw4fPMLz/8Jrh1RnZbyu2"
    "v67/8frHfY6/f0Tff2c4w8HAIP+Z4R/IBQwAAQQ24MeP71fYWFgUzC2MGUREeIH8bwz6+hoM"
    "Bw+eYBB2rmV7KnDB6OH5Cxv/PXt0m+PvT4bfj+6dYf30Bux1gAACBSbDkyfP579+/ZKBlRVo"
    "NjCQPnz4CDIUiH8ycHAwsrh72EWGJoRfkXN1nfhT20jih1sgPEYAAghswNevPzZfuXJt7YcP"
    "bxm+ffvN8PTpW4a7dx8yXLx49cjp02cvPn70gEFaUowtxM89z8bM4DQjI6MMzACAAGKC0r8f"
    "P36ZeubMuSO3bt0ABupfYEDyMSgoSP86c+aa2aFDp3p2795/7P37dwza2uoy/Hw8gX88w8Ea"
    "AQKIBZE8/r+/f/+Z3cuXb5M/ffrUpaOjK6inp+l069b9mDdvPpS+ffuR5c2bd90sLKwvgF7b"
    "+eDbZ7AugAACuwAoCOYwMzP9//bt55wLF26FX7x44TcPDy+DgYFmC1BKiIWF+c+bNx8LX7x4"
    "3fn3799bRw9uA+sBCCCwAUeP7gQGIBsD0G8MTEyMDH/+/N396NHzHbdu3WZQV1eRlJISqwGK"
    "MQItAFrCjJKsAQIIGgv3GK5fPw8RYGICGsTA8Pbt56Dz5y/e+vPnD4OQEL87I8h0mCagGhAG"
    "AYAAggUiw4ULxxkuXz7F8OnTBzAfqP7Px4/fJxw6dOLGu3efzgCF/sE0v3r1jOHfPzCXASCA"
    "GCnNzgABBgADTVmW+94X5wAAAABJRU5ErkJggg==")
index.append('keys_icon16x16')
catalog['keys_icon16x16'] = keys_icon16x16

#----------------------------------------------------------------------
clock_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAArrSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsMAYj12IQycDw7w8Dw8/PQPYPIAaGDiMTAzMfK8Pfv0C3srIY8svwWkgr81uISfFp"
    "i4jwSLNysLJ/+/n7y5uPX5+8eP3h0pP7t079vHfwOMPvl9cZ3p4m6ACAAGKERQFOB4AwG4uD"
    "upl8ppuPmqeXlzyvijw/AycnAwMTI1QJEPz9x8Dw6SsDw4W7nxk2H3v5au+OTVten982neHl"
    "3jP4HAAQQAQc8FNIRFq8IanIIismXodZQICR4dnjnwz3739i+PLpO8OvH78Z/v/7z8DEwsTA"
    "zsnGICjExaCiJMAgKMzEcO8FA8PUdXe+blw6u+/n+Vk9DH8+fMLmAIAAwuEAkNqvGgaWigvr"
    "JnqZ6ejzMVy6+JXhxsWnDP8+vmNQleVgkJXkZmBi+sPw6tVzBhkZBYZPn34w3H30jeHJOwYG"
    "HgkxBiNTKQZ5aRaGDSe+MHRNXrXj1c76XIYfT+6gOwAggHA44J2qlafG1pZp7qosHMwMOzY/"
    "ZPj39DaDl6M4g7q6FAMbBysDKysrw6ePnxhu37nFYGhoDAyJv8B08pfh25dfDMeO32Y4evUf"
    "g7ShFoOrvTDDpQf/GCo7lp94tqEwkeH36xvIDgAIILRcAHTMzz/C2pZqS6v63VXffGVkWDz7"
    "CoOx4HuGpCh5Blb27wwcXFxAdcwM/4BBz8nFzaCqqg52+H+gGBMzGwMHNweDmMQfhoI4OQa2"
    "Z3cY5i57yCArysRQnR9mwW9fNpGBiVMC2UaAAEIKgWUMDL/+MghJc01uneeewyPBw7BjxRWG"
    "BJv/DGYWcgx//jIDffibgYODg4GZmQkY/IxADMwhzMwMf/78YQCZA3LU799/wBgUQv//fGNY"
    "vf4ew7FPkgyh/rIMu8+8Z5hUldr7797aGmgiYwAIIEQIMLECsxmrrW+iYRavNA/Dpg2PGMJN"
    "/jNYWCmCLWcBJjQeHh4GdnZWBjY2VjDNysoMdggbGwtYDIQ5ONgYuLg4IA5iZGcICVRmUGN4"
    "zrBu/0cGSz1BBuPAghQGVlFnmLUAAYRwwH8WJlk9iVwzd0Wm3ce+Mwi/u8LAyfGa4SMwtXMD"
    "gxVkMMgRrKwsYJqRkRGKGcA0yCEQeVawGpCjWFiAUcXIyuBmycTw/tp1hssP/zN4OFnwc2p6"
    "xwFtFAZZCxBACAewcevrW8t5vfzByPDy9jOGMHcRhtvAvLR27S6GixevA4P3L9jHIAvxFq1A"
    "E0EWg6IGFCIfP3xkYOZgZAg1Y2M4euo1MN2wMMibBrswMHPqg2IeIIDgDuAWYbNQNZbiPnH1"
    "B4My61sGI2NlhogIHwZdXRWGI0fOMSxdupXh7t1HiJIHX+ECdCQ7OwvQEYwMIiLCDMqq2gxW"
    "JiIMHJ9eMTwEZlNNTT0hBn5tUDRwAgQQ3AFC0jwWn5g4GB7e/8xgp8sGLFxYgQawMjg4mDMk"
    "JQUwCAvzM6xbtw8YIrsZXr9+i2rjr18MDO/eoQh9+vQZKPwbHGp/gIlSQJiHQZX7M8OJq78Z"
    "xCUlGFiEFHRA0QAQQHAHcAnyaj//Ckqb3xmUZbnBDgD55MePXwwCAvwMAQHODOHhbgwfP35h"
    "WLJkK8OBAycZvnz5xvDz+w+Gf7NmMny7eIHhFVDu/PlrDKtX72BYvHgrw9mzV4AhwQbOLSxs"
    "7AwGKpwMz59/ZPjwh42BX0BMFGibAEAAwSsjNh4u6WfAApARmNVYWX4DE98nBi5ObnBqB6Vo"
    "UEKTk5MChkYQOE0cPnye4e69xwyMP78x/Lr6iIGbH1gaXt8CjiExMQEGS0sdBkVFWWAW/QvM"
    "lr8YvgIdKyHCxPDl22+G19+AYc/NxwvyN0AAwR3AwsLK8eknkAbm5devXzB8+PYWXMjw8nJD"
    "SkgkoK+vyaCoqsiweN4KhhevXzN8FhBmcBXiYXB2NmPg5+cB5hpuuNrfv/8xvHnzBujzx8BS"
    "Ehgdf8UZvvwGFmWMwCAGZnyAAII74Mfvf585WRgEWNmZGKSkFBhklCTAFoMKGVCWQncE3+MH"
    "DO5MvxneBnozvHr0FOhjAwYhIUHUchUYciD90tLSDFLAeD9xEhhSbP8ZWIFG/fr27R/IUIAA"
    "gjvg29dfj+W4GGR/cbAD4/kTg/T/v0ADmMGGgApLlOz3+DHD9xUrGRTS0hiUJCUZmAy0sOYG"
    "UMn4//8/cB3BzPSP4f7DbwyCglIMvEBbv7x/+x2o5A9AAMET4bePX6/wgQpDbk6G2w++Mfz/"
    "+xdevP779w9h6sePDD+WLWNgioxkYAFZjic7gvRBHMEITKy/GB69BlZQMrwMfz6+Z/jy5jEo"
    "K30DCCC4/g/Pr5/8+QGYgGT5GO68+M/w89tPYHz9BZfrf4GtDbATgMH5G2j5PycnBnY1Nbxl"
    "AUjP37//gVnxD7hwevXiM8NbZgFgAmVmeHnrPsP/j7dAhcongACCO+D3g6nHH93+84ZfiIXh"
    "A4c4ML7uMFy9ch4chyCD/gJ98mfdOoZfysoMXKameC2HxD0kBD9//sxw4fxJhmMnXzD8F5Jg"
    "AKW8p+f3vGX49+QykPkWIIAQIfjrwo3HZ7du/gwMBT5FCYaLN/8xyAATIwuwbP/FwsLwd89e"
    "hm/AhMDh5kbA5xBf//79l+Hnz1/AOOcHxr8ww6PP3AxCinwMTy8/Z/hwfdUVoNKLQPwZIIAQ"
    "Dvj34f/3OwunnD8KDHxOVgZ+TW2GA/tfMPwC1gHMZ04zfH/yhIElMIThD7B0AxkOidv/cBoU"
    "5JCq+DcDKMmA2CDw+tUHhtPnfzGIGWowvPzIwHB1w8L3f3+cPwKUug4KLIAAQk1Dn9efe7B3"
    "Ru/JU8B8KivE8F9EmmHP8jMMf+/eZPjl5c3w8tlzYGL6DS5iQSXkz5+Q6PkBbBuCfAvy+ffv"
    "P4HNtFfgmvHt648M27c/ZpDUV2V4DSwRjy468v/djanHgL7dA3IbyEqAAGLBCMJHbT0n1+ib"
    "8As7eMY5yjAwsv5mWPmYg0Hz9lNgyfiXQUxYCGwRqFHy69d3hrdv3zBISkqBUzywvASL37t3"
    "k+HuHWGG69d/MqhaqDPc4RRg2DrlLsOtzRXAeH+yAWjNeVCYg+wDCCDMXPTv1aefl9Pzds87"
    "dnLebiBfW5FB306L4erlHwx3gE3KB/dfARPYL3Ct+P37d4anT59As+lfhh/fvjPcvPmS4fYt"
    "DoanzxgYrHz1GW5zCDLMnfKQ4fz84psM/4+uACrcAcrMMOsAAgjRJEOv6JlkNJg1+icYBIW4"
    "xwYxMLjL/2f48uA1w+N7rxhevPzMICLKycDDywRM5R8ZBPiFGV6++Ap0wD8GWTkBBlkVCYYf"
    "ovwMK4A9grVTTzPc31J5ieHf3uVAU9cAMUrLGCCAcDsA0kqQZBBOKBayKk208pAX8rFmYLBV"
    "ZGDgBcb3l3efGH5//Q3UBwxExn8M7LwcDFxAS5/8ZmLYC0xeWza/Y7i4btmH73cmHgXauR5o"
    "2C5QGYpuA0AAEXAAGHAwMCm4MEhExHBrhLjI62oIG+lzMyjJMzAA7WMAVRNfgM3LV8DmwPXr"
    "3xgunXnK8Ozs3je/7i27yvDnODC1/9kDjfOP2AwHCCBiHABvswBDxJCBw8qJgVNZh4FXRJSF"
    "i5eXiZmJ9S8w7P9+Apbtnx6+ZfgJrJsZ7oAKmQtAfBOIX8ESHDYAEECkOADedoE2KPlBcQSq"
    "UqFVJbCSZQBVMKBuFaic/0xM+w0gwAANGVpo2ifr1AAAAABJRU5ErkJggg==")
index.append('clock_icon32x32')
catalog['clock_icon32x32'] = clock_icon32x32

#----------------------------------------------------------------------
magnifier_glass_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAK/SURBVHjaYvz//z8DJQAggFhA"
    "RElJMQMLCyuDsJCgp7y8QpmgsIgZIyMDy6ePH2+8f/dm2tdvP2bevHmTYdq0aQyenp4Me/fu"
    "Zfj16xcDyHKAAAIbAGKIi4nk6hhZ9kooqLHKiPAyMDExMjx9+1Xvyf27M17cv2r66dOndKDS"
    "v1ZWVgyMQNN37doFdgFAAIEN4OXhdlXVM5/IoWDOyCXEwvAPKAryGBc/OwOvihDDbyb25D/b"
    "t90ACvUAAQMbGxsDMzMz2ACAAGICEbJy8jU/BTUZfzCyMLz/wsDwEorffmVg+P6HgeGnoBaD"
    "ubVjARsrK8/Hjx8ZXr9+zfDz50+wAQABBDaAW0DM+P0/PoaP34GagPg1UOOrrxD2h28MDO//"
    "sDAISKlIi4oIqbGjBSJAALFAqP9MrEAXvQNq+PaLgYEd4jqGX38ZGH4AXcDKysDAzASOLfa5"
    "QCIFiH9ADQAIILALPr99cVGc7QuDABcDw+9/DAyffkLwLyCbl5OBQYrrL8Pbx7feP33z7p4U"
    "UH0XEHNADQAIILAB27Zu7fj99CKDHB8Dg5IIMEwEGBhkgFhRGIL/v7jMsGXjhnnWv3+//A1U"
    "LwrEVVADAAIIbMCGTZs3Tp/Q0frxxlEGQcavDNL8QAOAWJjlB8PbaycZvnx+xeBrbagzU5jJ"
    "DeQRoK8YBKAGAAQQIygNgOIVBLS1NOO8fXzzVNQ1NZiYmVkeP7j3bMf2bQtCRXiVIhvd43nf"
    "bfl7NvlC9KPHn1a+A6ovAOoFCCAGLEmZmZ2NVZmTg10LZlEoE4PSjTiW5/9PmP3/fMzv91pZ"
    "3vA+aAIECCAGYvOCEgOD4ZUQ9mf/j5v+/3DE5/c8dcEEkF6AAGIgJTPJMjLonw5ie/j/qOn/"
    "Nbla90B6AQKIgdTcKMfEYNRmyLZdnYc5AqQXIIAYKc3OAAEGABLI/zfRn+acAAAAAElFTkSu"
    "QmCC")
index.append('magnifier_glass_icon16x16')
catalog['magnifier_glass_icon16x16'] = magnifier_glass_icon16x16

#----------------------------------------------------------------------
computer_laptop_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAO3SURBVHjaYvz//z8DJQAggBid"
    "nZ0YXr16z8DNzcsgIS4l+frVO3lGJlYdPh4eGwF+Pk0FeQWV81duft6+fYk9A8O/h+gGAAQQ"
    "CxMzi4eyonyinIy0iri4lAIvnwCfoJgMCwePEMP7PzwMbNzCDMwca4R271rm/Ofvv3noBgAE"
    "EIuuttaSmIR04fe/uRm+/mNlePWNk+HeFw6GD98YGR5+YmL49PgfgzmbCAMfN7fJu08fgQYw"
    "oRgAEEBMrEwMXG/+8jIwCMkw8AmLM/Dw8zMI8bMz/GFmY3j5mYXh4Zu/DP/YRRgEBYS1IFr+"
    "I+F/DAABxHT/wYMXPz++ZhBiZ2TgZGZkYAYK33rFwLDyNAPD0QtfGe7ffs/wk5GfQVREXJEB"
    "bD2yAQwMAAHE9Pz5izvfPr5nePTuL8OP3wwMt98yMKy5zMDwCshnYPzB8OfXT4ZnnzkYREWl"
    "BYHqpWFOFxBQZODiEmUACCCmn79/33z3+gnDl++/GV58Y2DYcZuB4fUnkJJfDAx/gfjPT4bX"
    "nxkYhMWkuYCCuiAZXl4FBhERbQYODj4GgABi+vHj9/Xv71/+//TpG8PWOwwMd94A9f0GOu8X"
    "0Dk/gfjrT4bnr/8ysHAKMTMz8uqqqdkz8PFJMPz+/Z3h//9/DAABxPSPgeHKhzfPfr96/4Xh"
    "IDCW3wBt///5DwP7r88M0uwfGAwV3zOYKX5iUFaWYdA3ME1lZGQN+vzlC8+fP38ZGBkZGQAC"
    "AEEAvv8ECQgJ2+7w8vQi+u0SD/UOACUO8AAsH/AAJRL2ACITAwARDBAACQkMAAgMMQBFQjsj"
    "wsPFq8TFzujf3uuy3NzdggKI5eHDB89FREVf+0v8EJqn9ZpBhOcPAyODKMPbj3wMv/6xM7Bz"
    "sjLw8DACMTAchFgZOLlMmJSVBaX27buQ8eDBKTGAAGJmZWVh+Pf3n7aQII+JsABQAxcnAxMr"
    "N8Pvf9wMLKxsDNxcX4Fh+Yzh0aPLDGfOnGA4duw0w8GD5xmuX7/N8OzZ/fsAAcQoKirK8O3b"
    "Vw5gFJfKyiomGBmZyJuaGDCrqcsDQ1qU4dPHTwx37z1iuHXrIVDTPYb79x/+ePHiyc2PH19v"
    "+f//xyKAAAIb8PPnTwYmJmaGDx/eAxMKY4CwsHi6pqaulYGBHs+3bz8YLl68+vv+/buP3r9/"
    "s/3//++rgTF5hIGB+R8j438GgABCMeDz588M//79ZQBlcWZmDjUWFuaUf/8ZxH7/+roBqGkP"
    "EH+B5IV/wBhgAdMAAQYA9zFXbMNEp5QAAAAASUVORK5CYII=")
index.append('computer_laptop_icon16x16')
catalog['computer_laptop_icon16x16'] = computer_laptop_icon16x16

#----------------------------------------------------------------------
delete22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAARHSURBVHjaYvz//z8DLQBAALHA"
    "GLx2xYucnG0jeLjZmf79xW8ZzC0wVUxMjAw/fv35f+jAse1v97T7gcQAAogB5GIQdilb/uP7"
    "fwj4hwP/R5L/C8S/gfgHEP8E4q9AHNq+7TfMPIAAgruYm4uDkQ3KZiTg2n8g/A+IgXyQ50Au"
    "Z2JlYODiZIerAwgguMF///77/xekAJeBDBAD/wJN+88MUfjzN0gf1CFAk/79/QcPQ4AAYmLA"
    "Y9DjQ4cY3t69C+aDXAgylAlo6OfHTxie7NnH8I8J6G1GiDjI+YxIXgUIIBZshoLU3Vm9muHn"
    "xIkMr4WFGRgmT2IQkpMHi394+YrhcUMDA/e3rwzPHj9kEIuJBWpgYfgJdPmHHwgzAAIIq4uf"
    "nz7N8LeqikG3pYVB3dGR4XZSMsPbZ88YPr9/z3AnK4tBWUOdQa2sjIF323aGN/sOMTByMjD8"
    "+MPA8OILwgyAAMLqYm5paYb3cnIM36ZMYeAOD2fQlJdnuBQXz/CfnY1BR0SYgU9GhuHP8mUM"
    "31jZGJiB7L+/IJHKhBQUAAGE1cUCUlIMEgsXMVx/9pzhc1s7Az8/P4Pu718MOu/fMQgJCjF8"
    "X7aU4dqFKwwM1VUM3BpqDL+/Q1MHUvIHCCCsBoMUCctIM8hv2cJwjY+f4dOKFQwCQIOFgJH+"
    "bccOhqtfvjNwTJ/OwKelxfD7MzTZ/UM1AyCAsAbFH6AiZmDsM3EA0yU3N8Pr588ZPr55w/Cf"
    "lZXhHwcHw395RQZGDk5wUgMbyIzqWhAACCAm5OT1nwGRpN4DDXoYEsogu30rKIkyvP39m+HN"
    "t28M/3h5GWQf3mX4kZnB8OnBPYb/3EA9fxAZBQYAAghuMCjgGaEiH1+/ZngdF8egsH0bAygv"
    "gTLOV2Mzhq/Wdgx/371jYOYTYJB784KBpaaW4du9B0BfYAYFQADBDf4KjNlf/yCGf37wiIFj"
    "/wEGXiAbmLkY3hgaM4ivX8cgvWE9w3tTc4bfb14zcAgIMgg8usfw6/ZdUDIGW47sZIAAghv8"
    "6ivQcKApv4EqhI0NGX5MncpwD5icXunpMwiuWcvAJSXNwMonxMA3fxHDG1UNhhd37zA8Do1m"
    "YLO3ZfjzGZLdkYMCIIBYkAsXcPYElQd/mBjEkhIZXkjKMHDKyDLwKMkz/HoPkWOTkGT40zeN"
    "4eWVywyc7j4M/4FqQZH4Dy19AQQQSqr4Cy2xfgMj4x8Qi3i4MvwD+uLnB4g4GH8Eljeyigzc"
    "iooMf79ACiEmRszIAwgguMFMwNKamQmiCESDTPn3BTVHgTQyguR+AmlgucAIlQMVPiAaZAbM"
    "PIAAghv86/ev/5+BhrEDy9WfTJDI+McILdVgpdt/JPH/ED5InAmo/hvIvl+/4S4GCCC4wfcv"
    "n9oe3Szky8fDyQgqm1Gqn/+oaZ0Bydv/oVXT95+/GG+dOXKUgcEdLA4QQIy0qkwBAgwArdTo"
    "b+osmOAAAAAASUVORK5CYII=")
index.append('delete22x22')
catalog['delete22x22'] = delete22x22

#----------------------------------------------------------------------
clock_stopwatch_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAPwSURBVHjaYvz//z+DM5smAyMb"
    "CwMjKzPDf4Z/DJIfLjOocHEetxQWsvj99++/zz9+/Cl89yHnBQPDbAY0ABBALCBiH3cpA7MM"
    "HwOjMDfDP0YGBv1XKxkYbu9kkPnwjeEXCwfTiw+fWX+LqIgw/uZlYOXgYGD88YLh58f7YAMA"
    "AghsAMP/7wws/1gZ/n37x8DMz2n4P7i2nFswWeO94D8GTm4+hu9PHjCaP3qfcHzfuReff/yb"
    "z8V+A24AQAAxgrzAKDiVgUGIk0FWW6IlPkav1ECanU2Eh5Xh9/evDP/+/mbg4BFi+PaXkeHW"
    "qy//V63cfPHkppkBf96dewgyACCAIC74+ZdBUoa/KyvVskST/xej4J9XDOpiUgzMLBwMv38z"
    "MXBzMjLcufuU4RcrJ2NicoLB3y+vD5xYc8GCgeHfS4AAYgC5gF18uk3ixAe/J629///h3Wf/"
    "///78//Pn7//YQAYjv9//fz5//2bt//X7Hjwv2rtj/9yesH7QHYDBBATiJBRE2sQFBVhkWF+"
    "yfDg0X2GR49fMTAxMcJDGsRmZWNj4BcSZDBR+c/A9ucng4Jlkg0jI6cjQACBvcAtLW3K/Pcb"
    "g7udGsPHH78Zbty4z3D37iMGTT0NBpFvnxm+//3P8BwUPX/+MKgpSjCoPvjCsINXm5WdVyIM"
    "IIDABnzlEOBi/POV4f9vNgZJSSkGcXERhoePXzKcOX6W4eXlywyyWtoMAjKSDJJiQgy/fv9i"
    "YGf4zPDhNxcDI5OAPEAAgQ3495vxHxszKwMLGzsw1P8zMDEzMShysTDw8zIzPA/wYpCXl2Xg"
    "4eIAe+fPz68MbKwcDCxAz//9/4cBIIDAYfDz+Z3nP5gEGF6//cLwh/E/w/+PHxl+A20W0NFh"
    "0NZURWj+85fh25dvDK++cjAwfX/D8OfbywcAAQQ24O2VNQuffGBhOH//B8Ov168Zft+4wcCo"
    "q8vAJCwMSWf/ga7895/h798/DLfvvGG4/Zmf4c2VTT/+/X61EiCAIC54daH3xOYNj+6wqjM8"
    "uPCQ4SU7J8MfoOYfP34yfP/2neHP798Mb169Znh46znDg38KDCdPX2F4dnzaDqDWgwABBElI"
    "DMyfHuya5z+Ti3kXc6avqN6vDwy/771hYPz/A5iQfjJw8woxPH7xg+H+T3GGJTvvMBycnnCa"
    "4e+TdJBOgACCJGUmYKLilGRg+P5eTVDHeL5DQLiZlYEai6QQMECBbvzwlYnhEtDpuzav/35v"
    "z9QtDP8f5AL1vgQZABBAEAMYzRkYuKQZGBh/MzB8O8fAwGvtwcb+LJpPSEaJiYWZ/eunD9+/"
    "vrpxjeHng6XA0DiEnJ0BAgwAzAGTepRlltwAAAAASUVORK5CYII=")
index.append('clock_stopwatch_icon16x16')
catalog['clock_stopwatch_icon16x16'] = clock_stopwatch_icon16x16

#----------------------------------------------------------------------
print16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOOSURBVHjaYvTwiGaAgd+/fzOY"
    "mRnMCQnxDgdy//z//x9IMTIwMjKCaJYtW3ZvOXbsZDQbG9s/mB6AAGL59es3lPmf4cePHwx8"
    "fLzCRkZ6PAxYwPHjZ0SAapiBBv4DGQwCAAHE8v//OwZmZm4GJiZeBlZWRgagpX/+/fsH5DOB"
    "HAHEf6GYFSzHxsb1n4WFg+HXr/dA/jcGgABigij6wsDGBqJ/A2mOXzAXITSD5P4BLeD4/e/f"
    "7z+srL+Bmr8C8R8GgABiAjnl799/QPyW4e3bm8L///+SwbT9Lzgs/v//LfXy5TWZnz9fA9VD"
    "xAACiAXoXE4+Pr54OTmFQCMjS1VpKXH57z9+Ab31B+iG30AlQIMYgfgfI4OYqJBRYGDIsa9f"
    "vtx+8ODhpg+f3s0FCCAWCQmpuuzs/AodXT2G79++MBw7f5Ph2tOvDBxsjAxMQJsZGf8DDWJi"
    "+PUHGNhcgowxUUmyXLx8snfu3HSaP2+GJkAAsZibm0VaWJoz/P75k+Htt68MDOKyDO85+Rl+"
    "/P7P8OHnf4bPvxgZfv0FxtB/ZqABvxhkv3xgePT1LYOItDSDqaF5AEAAsSgpKQqwsjAw/Pj8"
    "k+HLr38MJ74KM1y7xMRw5z0Dw+c//xn+/YemEaAXBP7yADV9Z9AVE2Bg5uBjUFBW5AQIIJb3"
    "778xPn36joGX/R/Dxz8cDOuPczLceweNeEZGKA2hPv7jYnirwcMgwMvK8OLDD4bPX379Bwgg"
    "lnfvPv27evUh0M/AcGfhYshQ+M3wTY6N4dtfZqA3IBrZgS7kZP3LwMv0C+iKnwynLnxl+P6T"
    "gQGolwEggFgYGf9+FxcXEXj06AHDurVTGAS5WRg4ONgZWIBBxwO1GhSQf4Dsl8BUu+TDdwZf"
    "/ygGVRVNhrt3f/8GCCCW798/vZCUFJb88eM9g5i4GIOdvS0DLw8vMKX9Aiel/8BU+Q+YBJkY"
    "mYB55RfD4cOHGcSBYSAjI8bw7dundwABxPLgwd25N29e6hIWFuOKj09j0NJSZADqASr+C05g"
    "f/6A8F8g/gNMpawMSko64ER2586V7/fu3VoIEEDMiooK54GMs1ev3mWwt7fS4+PjAWoGJiKg"
    "raA8gYxByZuVlZlh5sx52/ft213+9ev7xQABBAwDUFLj2M7Kymn98eOHL1++fP4Lywug3Pz/"
    "PygJ/wOzQYCVlYWZiYn12evXPzZzcHAyAAQYAGroh6Cis+h6AAAAAElFTkSuQmCC")
index.append('print16x16')
catalog['print16x16'] = print16x16

#----------------------------------------------------------------------
newsub32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAACXBIWXMAAA7DAAAOwwHHb6hk"
    "AAAH5klEQVR4nMWXe3CNZx7HH2VHx3U7s4Nu1G3qVolLFRtkolWU2qZF4rJyRVG3LUlZEqVU"
    "2cal1ekM2ukypC21g5PIxrUSREIkESIXCZKTE8k5J+ec91ze99zez/5xThBpuszuTt+Z37z/"
    "PDOfz/f3/J7nvEcA4res3xTeTEAExfTtPH7tqQGx31gGxu4zDozdZ/ifVNw+wytx3xj7x+w1"
    "tQ+J/1EExXQVQ+KEGBLXVKBdSHzKV+dr0DnhrhUqn7UkqJDgjr/KJSi3QJkZyixQaYekoyW0"
    "GbFkuxgUJcTg2KYCAeHbbxQb+L88qup7ny630XF84kkRGCPEsAVNBf44PbmgoM630mio53pe"
    "HqiepwJ4VfCo4PaCywtODyhukN3gcIPd5VuXVizRYfw6jQiKEeLV5gL5jQInTmgIC5uFXt/Q"
    "cqrH0jXCGwUUv0Aj3O7yrdfckujwxjpNSx14KJCWdpKIiGjMZslHcTgo2/wpNxLXcz1pNZf+"
    "upCaixfxquD+heSNcJsfblXA+ywCJ0+mEx4eRWXlfUxmiaqfL1MwIxTHv95GzhzA7U2Cy2vW"
    "Nm27tync7vIJWJ0gKeB5FoHTp8/w1lthREW+z9x5S1g0MZySWYFwMgCudEP7VTuurUt62PZf"
    "gtsfg0sKuP0CHZ9mBtLTMwgLi+DE8TTOZmZx+uvvqJzcCfcXbfHsb0fVKkF+whpcjXv+K8kl"
    "BRocYPfA7ksSrcb6Bf7TDMycGYPFYvMNmtFE5dSx3B3fj7K3B5I3uh8V+w+2mNzmhxvt8MAK"
    "WgkanPDpOQkx+qkE0oiIiMZg8J0Cl8eLbLRgrdZjqTJgrq7HZpdbTG6SfeAaCaolqLaA0Qlb"
    "zkmIMc8g0HgMXU43ikdF9oDs9mJ3eZBs8sOz/vieG+1Qa4UaK2gtPvh9C+jl/0JAUVzIDgW7"
    "TcZmdSBZbFgs9qbJFdDbQWcFneSDV5nhnhnumqDOAZvPPqVAamoaERFR6PVGvF6QZSdOQAEc"
    "gEMFsyQ/TC4pUN8I9yevcUM1cE+CygZ48KwC4eFR6PUNeL0qDoeC9sx5yg4dozhFw82UE9RW"
    "3MfuArMCdTaotfn2XGsBrQMKNafI2bGLsrsmKiygs8GmRoHB84R4ddGvCaQ2OQXW0kqyQ16j"
    "eOVoitb05uyUNmQlbXw4bHVOqAdq3aBVoOhSMdfi+lO7I4DcrZu5I0ONAomXZUTIJ8cD+k0T"
    "3V+Z2Vwg/7EOTJjwDp9t2UXyl/vYsWQtJVHBcGUIlPSm9h8dyVy1mgc2qHNB6Y1SMlYtJefQ"
    "EWqBa4czKNsaDPoYrm9YTKkHSsr1HFqQxLTe40/N7zJEvN9tWMsCOTm5rFiximXLVrJoRTwJ"
    "06K4E/4S/PQHONOFuuS2XF65mlo76GRIWzIHw9lF5MUHk70/hevpOZRtC4aa2eR/vorCCjtX"
    "oycj5wZg3tZTuSpa/elaq+daFpBlF06nF1l2oXihobCM8tfaU/+eoDayNUVjBbmrE9FJoFPg"
    "VNJqqo59AJzjZtIYjk1fQNn218EYQ1bUNK7EvIu9IBDkkRTNevH+j6JDzyO/69hUoNu05Pyr"
    "Op+AU3HiVJzIDhm7XUYyW6lLy6B6/2Eq9x+h7LsfqC4qQWuBWi9UOdyc+jASnWY+uA6TM/0l"
    "SjcOBuNirk9qjzV/LPAmWUtHEdRr5vEDbX4vrrRq3VSgzaTk/KMlPgGbEywKmB2+e9zggHrb"
    "E9MugdauUnjtDnlZBWSfKiQlNITKvVNB2oH71mxU42K82j/jqpnK7bCeJEQcoPvAFeeOida9"
    "C4R4volAq4nJ+Sm3fAIWxXe8GuF6+yO4TgKtGWpckHnoIBkrJpK7NYq8rbPJTXqPoo8G461Z"
    "DXwNJAKLMB0bRunKfpQmj6D0414u7aYA2+3hnQ83EXhuYnL+9zdVVPxwGfQO0PuT62yPrtcq"
    "M1QrcHxNLPW5icBVsB4G+QBYvgTjJmAXsB5c88H8DngngCcQ6A8MQLeht76ZQMpNFS/+5Pam"
    "yRsvmSqL73q9b4e87Jukxs/hdMIULiTNIvXdkeRO7wSGvwE7wbMGWIj5pyCujeuBZmk0h6YM"
    "ra8e0ykj74W2C5oJHCpScasttP0x+F0TVDRAhQ3KzVDqgIs/l5I+ewy225/guBGP6fg41LoF"
    "SOnDgZkYvghizYydiOCtx/7efZAoaiVEM4GDN1Sc3kdwnfXRwFWZ4Z4JKv3wciOUGqDMCjlX"
    "SjgdNwqndjfO6s/RDOpE8ccjwLSIC8NfoGZvIPAmNXteZmH3URl7nu8icsQTAq0n7Sj84Zbv"
    "E9vo70C9/4OitvG33QL3H+vAHZPvT4dmcTi227txV+4ja34IqRu+pXznOKiby8W1yzmfsAXD"
    "tz2BYAxbesrZQozIfFJAhK4/s/6cjXsOKDbBTSMUGaHQAPkGuK6HvHq4Wgc5dZD9wFeXjPD9"
    "xm3c+mwsqTGT0ORqSf3nRd9VXPsXLmxcjkYPmo+S0W57meI5L1ZkCtErq5nA0NipnWfs0fZZ"
    "dYk+KzPVlqr3kxV/Ue3z4QV18LgktW/0UbXHxmJ15ORdasmWUWCOJnHqdLomFKi9EnLoO2T5"
    "7ZgOgcGXhBDNOzA0ToigqB4iMPJ1ERg5TgRGhj51BUWFihEfhIqhcaEiKDakR8CkN9JHdD5/"
    "d15X44KuA5eJoIXBInBuqBi9smtEwBiR7Rf4N+S6EKV4OunSAAAAAElFTkSuQmCC")
index.append('newsub32x32')
catalog['newsub32x32'] = newsub32x32

#----------------------------------------------------------------------
windows16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMlSURBVHjaYvz//z8DJQAggFgi"
    "Mxa/k9JWE/j4+d//X38ZGH7+AWIQ/ZuB4TeI/Y+B4dc/Robff5kZH+yacu397S02SspKH7S1"
    "xBmOHz/JABBALBI6Wry/lfQY/77/w8gINJEJqJkNqIkRSLMANbL+/c/wl4GJ4TcjCwPzaXF5"
    "KXEBUWMjhQ9fvnwFuwAggFi+v/v2L875NwOX2l8GkAkgQ0CYAcwGeu//PwYmZiaGCw//MmR+"
    "+f5PVoRF9Pefn2///PnDCvT+S4AAYmFhZ2EwUOVi4GLB5kOgAf+ATmFiYHj64QeDsYU1r4O0"
    "xG5uPoF/7z98/H/t2o1egABi+fOfEexfZANAAfv37z8g/s8AtImBC+inT19/MQiKSDKqawhw"
    "sXPyMLx9946Bg4PTFSCAWH4DDUCOCJDmf//+QQ0A4j9/GX4z/GP4+es3g5CYAIOJmRwDGxs7"
    "w7HjHxh+/fr1ByCAWP78Y2KARCUjhubfwGj4AzSAGRgOQLUMfHx8DMoqCmC1fFdvgdUABBDT"
    "7/9McNv//QM5+T8w+kD4Lxj/AhoCNggYFL9+A13y4wcwWIDyQDYjMNQBAnBABjkAgCAMGwwS"
    "//9dAZ2edlubhgYrfneQB5RumOiiUjbejbRC+mCeoZpslGzfgeMKIJZfjByMD179ZuDnYGD4"
    "CwzxP79+MfwF2/gP7PzfQA3sQM1P3/1j+PGHCRyozEAD/kG9DRBALLeevPubNesh6y9gIP0B"
    "Jpx/wED9B0o4/yAx+B/oXEag2Kv3fxmirIDhBHTVT2ByBToADAACiOXakpQqoGFODN8+fxVT"
    "UDLoaO9QNzDWBQcQSBUoUP/8/gs07C+DGD8zw5fvQAOByRWWhwACiOX/v9/9DD9f90uICTPo"
    "Kgm26CpwV+spcIMVgNSA/ArS/AeIv377xfDt+x8GNlZgWDGzgDATQABBkg9QUkNDnUFT24Dp"
    "7IWbDNdvPQL7H2QAyAXg6AWn6v/g1MnKysrw7t0HUNJnBgggRg5+ZQYpITYGDk4OkAbVDx/e"
    "NgI1SADZv0F5A+LS/7AcAmYDQx8kw/L+/as1AAEGAKblpIpcMYqLAAAAAElFTkSuQmCC")
index.append('windows16x16')
catalog['windows16x16'] = windows16x16

#----------------------------------------------------------------------
symbol_plus_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAPSSURBVHjaYmRIYWJg+M3MwPCX"
    "jYGB4R+Q/hurLKI0WVtCk/vfX4b/ICFmRlbGey8e/7ly/1bjfybmTgYmxv8Mv4D6/jAzAAQQ"
    "I8QARiBmAer/K6YqrnxnY9kmXk0pNQZk8Pz9K4aQljiGY5ePGzMwsZxj+MsOVM/MABBAQF1A"
    "zf+B1vz7CbSdQVhDUoMbpLnmQDXDmednGJhZmRlk+GUZprhOZjDXtGQ4du6QHMN/xnMMTP/B"
    "WgECiAVs/L//0kCnqjH8ZFD68v3Lv9//fjMdfniE4dDdQwwMQIuURJUYvv79wvD9x3cGhu/f"
    "dRjY2T8AvfqUgYX5NkAAsQAZUapSarMN5Yw4//z4/d9UwYyJlYmFgYeNh4GZg5mBjY2NgZed"
    "F+hKRgYLNSOGV65RTWwcfAw3Ht75e/HujRqAAGKQL5J/ee3ptf8g8Ov/bzD97vu7/66LXP+z"
    "tLL85+zm/K8/X///rXe3gDL//v8FYhB4+ublf+NUt18AAcSiJKrMpymlydCwu4Fh7429DCyc"
    "QEcx/WW4+/EuAzcbNzC8mBhef3vNkLE7g4GTnZPhGxAaiOsz9Fn0M2jIaTICBKCIjHEABKEo"
    "VolKwr//jZiY2N2NJ2Agfr7P9gbtHkuzRL879apganNuWDHykUly+qQ9DVRsxCDS4tf9jU8A"
    "lsggBwAYBGFS9/8vMzSGC4Fb+3yapCCNUaF47wDW7k6mA0sdc988VV8AsTAzMTODFKoKqTLo"
    "S+oxMHOzAKP3H8Pzb88ZfgKjFqSRg4mDQUFAgYGVjZXh279vDMr8ykC7mBlYWFgYAQIAQQC+"
    "/wAkdCPu3+zg//7//v/z+/L/8/ry/+Px4/9CokL/BYUF/wqLCv8uny7/0uvS//P88v/1//T/"
    "/P/6//D08v84aEv7Aojl7tt7lf79AZPMlC3Z/vz6w2Akq8+Q65XNwMHCAfYn2AtA78kJyDJs"
    "O3qQYdvJXQzsHFwMV+9e/3f61oUOgABi+c/EOPPSgwu7Lt2+oMfwnUHhgZFzb4ZHGvO3318Z"
    "/v78CxT6zvD111ewYfsuHWJYuHZyGwM77wmG/2z3GJjZrgIEECQlMjHcB2MOBg1eLp5eViZW"
    "BiMJI4bff/8wMLEyMsgD/c/FwsXAwwlMUJycJ4FmbWYABR3QZQABxAIKTWDQAw0BMf9+vPvi"
    "zq+Hbx5z9rr1MfwBQlCIMwPhx6+fGc7eOg2MStYXDEyg3MsIzrwAAcTIkMyIkp0Z//7N0pbW"
    "7daT1WcDZef//xgZWBhZGO48u8dw+uaFvn//GKuA2fkfwy9GcHYGCDAAV016+26QlyMAAAAA"
    "SUVORK5CYII=")
index.append('symbol_plus_icon16x16')
catalog['symbol_plus_icon16x16'] = symbol_plus_icon16x16

#----------------------------------------------------------------------
fileopen32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAZESURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAD7gCAABpwBwAEEAuIYAzZx8AASgv/8OF/UBqo4f9fBoYz+QycJsExMaF+c4T4ONhB"
    "0vgALKUxA7384v337ysWzEr7cWH5EoAAAjuAgZHBgOHXv0CG3//YgJb8x7D0L9whjEC1vxlY"
    "mVYBNV2Rk5fznJDrwM5FpG//QYP86Q8GzsPbVoUBmUsAAgjigN//dukq8YrKCLEz/P3zHxEa"
    "yDQQMwOVPnn9m+HyhRtRDH9/6jMzMv6Cm4rLUqRABBnDxAQJQGYWNrAugACCOODnX9EEd2WG"
    "OCdRhq8/cAehEA8Dw+5j7xhCMq5JM/x6o/r/56ffwDDBqR7ZD8iOAMcWNP8DBBDEAUD2rVf/"
    "GC49BbrlFyK+UAwECooJMDB8+AJU8IODlUGsYvZvJjlpxv/gKCTOcqg5f4Ah8AdqCUAAQR3w"
    "j+EH0NwvP4GxAXUAEyOmAz5+ZWCQk+RhyEgzZmZitDB5+uUxw6/f/xm42Bkx1MIdwYDAv/9C"
    "5F8CzXn3HcIGCCAWWER9/fmf4TNQ8M9vSDz9BDrx/7//KL56D9QowMXD4Ouvw8AItHPLgU/A"
    "EPvH8J+NGcXn//5jhgBQGdBMSC74g5RjAAIIHgVfgb7/DIx/kMDpa68Z1u69wfAVqIsRyRSQ"
    "g/7BcgTQNBnZXwyxLloMb979Bwbrf3C0gi3+B3P0f4bfQEt/AkOJn4+dQUyUE+4gGAAIIHgI"
    "fAOGwBegA0CJ6tC1NwzSgkwMVWEqQM1IYcqACFMmoC1Xn71i4OViZRDgYIQ6AKIUFDrvPv9h"
    "ePvpF4OMGBfDuw+/GN5//MUgLMQJNh/ZAQABhBQFwDQAjAJgLmP49OsvgxQHJ4OetiTQIlQN"
    "0HKDgRmInx76zsDCwsrAAXYAtIwCqmUGxsj3j/8Ynn/6x6CiwMbA/h2Yxn7+BRcnIKP+IkUB"
    "QADBo+AbNAq+AIOW8c8fhjdADecfMTBws6BqgJfhIAe8+83w7RfI8QgHwCz5+A0oDrQY5LFf"
    "wPT0D+iLv0jpAgYAAgg1EQId8O3HH4a/vyGmgRIlAxt2B4BC4DswtIDKGX4AaWCggS0AJTBQ"
    "FIAS3C+o3B+oHMgc9CgACCB4NgQVQJ++wxwAxP+YGD58A3kJuwNYgKn55WcGhjfAnAFMBmCH"
    "wBIY2AHAhAfKoiBxsH/+McBD4C+SAwACCB4C339BfIxwACvYASz/UbMNsgO+/YJYDAoJkI9h"
    "GYSRCRoqQAeA5P6CHfAfawgABBDcAf9+/wMXRD+AOv4B08CvvxAHcICrCuxRAPLlb2hQ/0IK"
    "AVAi/fMHJAeRB5X9sDoNPQQAAgjJAcB0APIJ0KS/IAcANYMcwMWEPQRANclvoHp2YMnCzgQR"
    "gIUAiM0KjAdgzAArTqAY0LVMwKzBzgoptdlANBML2G6AAILngv9/IA74BQzLf8BU8xfIBjmA"
    "jxm7A0BZ7R9Qz85zzxikhbiA8fwfXvqBitJ3wMTxFoj/cHMyMP/5Acxlvxlu/uEE556Hb38w"
    "fH99/xnIHIAAQoQA0JYvoBQLDQGQraBy4TMLdgeA4llaWo3h5MXrwJD4gaiRGEGJENho+PqL"
    "4Qew4vr4+wvDxwfPGU5v2/OOgfnofGBQPGX4CSzy3l7bBlIOEEBQB/yDR8E/UIEBzTffQTmD"
    "FXsuYAAnNnYGOSUDrI77+v4Hw+d3HxmUtcQZHn3kZWB4tf0DA8OpGUDpO8hqAQIIJQpAIQBK"
    "TX+g9eUPUOmIoxzAB0AO+P79DzBrA4vjL0AzfvwDRQsTsGzhRlcLEEDwKPgPtPw3ME9DEsJv"
    "sDAoC32G5mHCtqIyf/5jZvjxl5nhIzA2v/1hgiYOTAAQQNA2IdCbXz4yMNx8AaTfQmzk/M3w"
    "5PpLhmdM/xAtCXjr8j+i1fIfjQ11wv/vXxn+ffsKjE5gDnj1CSgErDSwuAEggCAO+Hp3GsOP"
    "d3EM74C18V+gMjag8L8fDL8+3ENrESNhWJ77j9FwRWkMPLr8H1KeMz+/C4wCjLAECCBGUB3O"
    "CIo0FgEboDMVIIUvA6Q8RU5x/3G0sxn+o/HR1fyHGPb/IzAR/rsK5DxEtJz+MwAEEONA9w0B"
    "AmjAe0YAATTgDgAIoAF3AECAAQDxkcMYZC6i7QAAAABJRU5ErkJggg==")
index.append('fileopen32x32')
catalog['fileopen32x32'] = fileopen32x32

#----------------------------------------------------------------------
taskcoach22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAZ5SURBVHjaYvz//z8DLQBAALGA"
    "iAuKjAxszAwMLIwMDHw/GRjMXAI4NXQ4Ot1dzKN//v/+b96ctTN+TTnbf1+NgYeZmUEY6JYn"
    "DJwMr7/8YmAQvsnA8LuGg4FBGqjxH8SR/9P+MwAEEAuExcDw9yuDPCsrgwIbE4O4Oe+7GMNI"
    "D19Grn8MzP8ZGeyzLGpEfv6MZ1bT4GaQEOBmvHHzBcPOo80//v6bi+FUJggFEEBgg/8wsoWL"
    "50TO+amuy/OEVYjB8M19hg3ndjCIq/IxvPr8keE3yw+Ge/qfZLU1Exjk1dQZftp8l7cU7ZhT"
    "XL3sClD7SYjjgK5jZNBh/cmiAuRtAAggsMFsUjxlkjlRPH2nRRk0ZHUYfP7+Yjh99BHD9ivr"
    "GH7+/cbA8JaNweCvHYORuxmDhKAow6e/DAzHTZMYbFU3Fi299DWc4e9fBnYmzigXU43ZZ+c+"
    "ugEyGCCAwAZz/H+nz3D3NIMalzkwwL4zaHLyMczwmM5w4G4Yw68/HxnE2eQYDJVNGPiE2Bm+"
    "/frFIMbMxPBc2Y5B1dbPX+X1cqP3iuIxRX5GhTvW3HnyYubbOIaJDAwAAQBZAKb/AP///wDC"
    "IADE/5xK/+Xftf+Fsdf/9OX9/+/Y+v/5+///+/f//+3x+f9ouXL/wdnN//bz///u7/j/+Pj/"
    "/4mopP9YmCb0p/xg/1KrOPEACQA5AAAAAP///wACCBzUz94yLPlw5gyDFMcHht9fnjO8ZvzL"
    "8PvTb4YvX74zfP7xk+HPz78MPxlfMMy818Bw+ektBhYGdob5p5cxPFbcwpCY7sW9dceDL+vS"
    "TycwfGc4DotDgAACB8WnXwybH1161Sbw+ragEj87w8UXXxnsOdgYfn74x8DE/5+Bi52LwZ4n"
    "hOEJ31aGxXcnMTBeFWDgV3jEEK6nzrD04GWGHXkXlzK8+7seOXEABBDYxX8YGJ7ee8BwhOH6"
    "eQYz6a8MF+++ZGAQYmH4/uY/wy9g+vny+xeDsbgNw/sn3AwOxmIMGnqfGVw1tRiWHTzHwHPf"
    "kSE8uM0alAYYwmbBDQYIICYY48kbho0fL11iUBP8xPDp7SOG1yxMDL++MzD8Aqb7b3/+MYjw"
    "izLw/tBmePjuGYOciADDihPHGH5c0Gco8cpgiM5L0+Ez8gtlkDeFGwwQQHCDP/xi2H370pPv"
    "jL+fM8hxPgWG5XcGRh5mhh/v/jP8ZWJi+P7/H4OltCfD+RuvGLbfOM7w8aQqQ5JjAcO5x98Y"
    "vr3/wmCX3t7AcHEDB8w8gACCGwwMjkfXrvw+wfDsNoO59BuGS7eeMzALszF8evWX4cdfRoYv"
    "wAhUkpNl4H1jznBjnRRDinMZwx92boYnn/8yiMvxMsj/eK0itqsR7mSAAAIb/OoXhHP/EcOG"
    "T9dvMhhKf2Z4+fgWw08uZoZ//xgYfgLzyPffjAwffvxhCLVIZ6iO6mIQlRdn4ONnBRYeQEcB"
    "5fgZrjOkyjFkwgwGCCCwwb+hBdyXvwzbbpx59IuN+yODBNMthhtPvjH852Vl+Pn+LwMzJzPD"
    "X0YmBg5+AQY2fjGGt0CXMgBLJA1FfoYDV38zCAq9Z0gK4PEXZWDQBJkFEEBggxmhtgDpO9dP"
    "fTzL8PUlg7nUE4ZzV54y8EqzM7x68ovh3af/DD/+MDGwswMTjAADAzvQoitPvjKsPHaHYev+"
    "rQymwu8ZlLz0udyEWNNAZgEEEAvM6f+g9MOb/zd+vffY0kqZm2HZ5ssMf71UGcRluBg4eJgY"
    "Xnz8w3Dy+keGm/efM/z4fJ9BkfMeg53UK4YKz+8MAgLMDD/PSTG8+s4ADliAAGKBuRhW3H/5"
    "ybDl5qkHrUZZhswCv28wbNr9nuHzt08M9zYBi2DGlwwGEk8ZMpUfA9P1KwZG5jfA3PUOnFb/"
    "nX3LsG/Vu9/HvzOAMwpAADGC8nUFIyMwkwIjCBoi+vasZ6M3hxteuqzDMH3dAwZrjecMjqpv"
    "GKT5PgIN+szw6+EXhke3vzPcu/Prz90H/17dfvb/wb23DDdPf/2//RkDwxqgmf8BAghssB/Q"
    "YGOgiSA//AZiXm6m8tQFkh2iSuwMn+5+AabAnwzXr/x+f/XW72c3H/+9c/ftv+tP/jJce8vA"
    "cAuYfx4BtbyGpFgGBlghBBBAYIM5gAaLQwXDgXgVAwOXAhtjPT8nk/zdj38vA11x5T0wYkEZ"
    "FIg/EqrvQGYCBBAjrSpTgAADANIrjEmyTHLrAAAAAElFTkSuQmCC")
index.append('taskcoach22x22')
catalog['taskcoach22x22'] = taskcoach22x22

#----------------------------------------------------------------------
document_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAIVSURBVHjaYqyqqjrt4eFhzMDA"
    "8J+BAGBkZARjNjY2xgMHDnxvaWkpAAgghh07dvz4DwT//v37//v3bxT848eP/79+/QKzP3/+"
    "/P/Dhw//3759C1Z7+fLl//Ly8ocAAvA0xigAwDAIdCjk/88VFKoZMop3+up2PQAC30skSOob"
    "ZmZzWNgGye2avwAcTjkKADAMEqlTlvz/kyGQva1x8QDFY2HMDLobEbEDVbWcmSAJSVtyZv9f"
    "eOA+AQRyAdgAFhYWsOCrV6/AikA2gfggGuQCkAEgABODGsAAEEAssADi4uJikJWVBUvAFIIU"
    "gDCI/e3bNzANMhxZHiCAWEAMkNOAAQQ2GQRATgSxQeLMzMxgMViYgFzCz88PjxmAAGKC2QLS"
    "BFMMcwXMryAAUgPSDPIOzFUgABBATMhOhgUoLAxgIQ1zGcwVMANANEAAscA4IAyyEaQIBEDR"
    "B9KI7AWYwbBABAGAAAKHAUgRyHkgCZBGWFTBYgaEQWyQGHogAgQQC8yJwFQHNgikCCQGSlTo"
    "MQICIDWwcAMBgACCpwOYYpjzQDaB/AtzCcxVIDGYN0B6AQKIBSjICApZcXFxlMCBYZDC79+/"
    "w8MGZAhIPScnJ8hQJoAAYjl37tx3CwsLDpACZM2wqAOJweRg/gcBYGYC5Z9fAAHECEz7iVJS"
    "UmlAMVZYxkLOvsg0zGAgn+X169dfgfllEkCAAQCVXrTAqK/PHQAAAABJRU5ErkJggg==")
index.append('document_icon16x16')
catalog['document_icon16x16'] = document_icon16x16

#----------------------------------------------------------------------
note_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAASxSURBVHjaYvz//z8DLQBAADHS"
    "ymCAAGIBEQmMjEamHBy9GtzcRsJSUt+23Xu4e+H/T61dmgw3/31hYBA3FmZQl2JmeP/4C8OD"
    "Z78Y/n74wyAH1PnlPQMDkxoDAyMrAwPbEyANVHvjEwNDLlAcIIDABpuys0/wFRKy5WBkZOBh"
    "ZeVTFhCIlfn3z0IlyOAE24fXr5h+f3rz5+fXD////Hr+//efx/9+Mbz9/4/h27+fDF8YfjF8"
    "ZwR6+v8vCP73B+JigAACG6zJy6vHDuL8/cvwm5eX4efLVwy6n/6ocrmXqSoaqTIwfL3D8Pf7"
    "Mwbhb4/+yn68+/3Pq0c//7x59unXm3fvvr37+eTb01/PWdn/v2H6yPCO+wXjLaYv//cBBBDY"
    "YD4Fhc+8LCz8v7m4GD6zszP8f/uG4f2PnwzMb4HWMyoyMPAIMDDz/AWq/MHMwvCNh4HhIxC/"
    "F2ZgeA2UfGkMZDP8//KWgfHrawaGLSd/s2S8CgQIILDBG2/d3azCx5/5H+i3fy9eMbz995fh"
    "g4keg44+MAAZ3gAxMPAYfgPxLygGRTgvEHMAsQQQ/2Rg5AGK87xk+MJ06c/3PwxCAAHEBDL4"
    "Ft/Hhc/0uL7cYmFmuM3LxfDI0phBc2obg5i4MNTQP1ADfwLxdyD+BsTAWAK6lIHhLRC/Agbu"
    "CyD9kOHLg6/fgSo+AAQQ2MU1+kyM6r3FjG9Y7Bh+fn7GwKcozSDAzwWU+Qg18DfUUBiGWfID"
    "gv9+BmKgcczvGd4/+f0TKPIJIIDABktwMgv8Z2NiF1dUAvJEoa78iOTSX0iGIWOooX+Arv8P"
    "9MXPTwyvX/39DnTGJ4AAAhv8m4uVh5GdiRli2HuoC38jhesPNIN/oBoKwqDg+fyF4cnrvyBD"
    "PgIEENhgDnYWAQZONkaEt3F5H2ogKJz/An31G2jgXygG5Y6PXxmevf73GhQBAAEENvg/C4Mw"
    "ExszFm//wmIokkvBYfsJYgHrV4a/r34wvH73F2Twd4AAAhv8h41JAJQtUb2PHEE/kVyK5P0/"
    "UINBYqzATPgKmCW//AOF5S+AAGKB5D8mQQaWf0gG/EIzEJrM/kC9DTP4/2cIBgXrn68M3178"
    "Y3j55z8o4f8BCCCIwRz/+RmY/mGPdRD9/zsiLGGG/wMa+ANo4C8gZgLyef4xfPrAzvCa4f8r"
    "UOgCBBDYYCb+/3xAlWguxOJ9mLeB+DeQ/+vqRwbup8BijQUYrL84GI7vZHsPVPEUZCZAAIEN"
    "ZhNlBGYxdiiXGYoZIVn3H9Dw/18g+C/U20AL/n4H0j//g3zL8OvMH4YJS7486nzBMB0YiMdB"
    "ZgIEENhgxo/f+P48msrAzCwCzPNsQPOBljADY5MZaDgoiBiBGYUFqIUR6Pp/38CZgeM3kM/M"
    "yXBz18//dSsZjqz6yjABaNRWqFcZAAIIXIMEijP6/2NmcObiZODi4WLg4uNi4BPiZhAU4mLi"
    "4eNk5+DhYmPn5mFj4+LjYGXjZGVl52Jh5WBiZTu95c37qv0vVzxkYJgONOs6cg0CEEBggxkZ"
    "GcFBDcSsUJoNWnSBCgxuIIcbKMEJw0A+ByvQb8+ABSzQ3QehRSAKAAggmtV5AAEGAHgLIOfz"
    "nEN3AAAAAElFTkSuQmCC")
index.append('note_icon22x22')
catalog['note_icon22x22'] = note_icon22x22

#----------------------------------------------------------------------
calendar_icon32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAqbSURBVHjafM3BDYQgFEXRJwKG"
    "EONyqpiJVViBlbijE/pyYQe60UwQw+rjVxvwFHBvwcx4OOdARGAusSwHjDEQQiClhLpusO9/"
    "aF3BWosYI6TUn/OkPM/T2ra/Sin1zURdKWUfQhi998Pd2/DieV8CiIWBCABSCMXS//79VwEy"
    "1WWkZcoZGP9+YGH5chroUDVhYWFjTU1NPnEJCYbdu3YZioiIHHvx4sV8oPZf+MwGCCAW/Bb/"
    "A2FBRkYmZxYWlt/8/HzNCgqKuq9ePfknLMLOxMbGzaClpWCkb6jNIC8nx8DJwQHWd/f+fQYF"
    "BQUPoAM2AbnP8dkBEEAsuH3MGM3NzWPNwMCoqqOj6fL58weG378+MXBw/mNQ1dBg0tNXA0YJ"
    "JwMbMzuDhpocWN+fv3+BUfWJQUxMgkFeXl7n5MmTSkCz8DoAIIBY0C3m5GQBxjOTk5CgwGw5"
    "OVnO9x/fMDCyfGMQkxJjkFcwYdA31GJ4ePcFg6a6HAMrMxPDm7cfGJ49fw5MJ3wMP3/8YHjy"
    "9AmDiqYWg66evuLRo0dtnzx5cgJo9F9cDgAIIGQHcPz7+9fx29evbkJCXMHiUgKcIrKCDNLK"
    "SgyWVoYMjEDHff3whUGEl4eBWUGc4cWLpwy8fLwMnFysDI+fvGTgA7JFRITBGBTpSmrqzNJS"
    "UjZABywGcp/icgBAAMEd8O/fvzWyikreimoaDPLKqgwy0lIMwlxsDE+fvGUQ4uZg+P37F8Pb"
    "7x+BPmZh4ADGNSh3CAuLMPADLRbg52cA5aW/QIIRSLMCCVlFNQZtbS3zc+fPa//+/RunAwAC"
    "COGAv3+MLZw9GEy1VRlggj9+/GR49+4FAxv7PwYeHh6Gf8BEycrKxsADzIo62toM/0DxDrUU"
    "mpvBDmEBOUBJnkFZXQsYIEKWz1+8PAIU/obNAQABxIQUAn///PzF8A0YWw+Aqfjps+cMv37/"
    "BqYHTnD8gsoFbS0NBh6gj39DLQbqAREM/xmRTYI4QlCAl0FOTYtBWUnJFsgVxRUCAAEEDwFg"
    "ML36++OL9PuvwMKIkREYv3xAX3Mz8AHjHGIZEAMt+Q8srBhfv2FgfvOGgeHrF2C4Ax3Awc7w"
    "T0kJaKsgAyhYgEIMHEC1Stq6DGoamobnL1w0/Prt20NsDgAIILi7v3z5evQlMAX/ZmRmkFJQ"
    "AFsOMuj3P0jw/gep/PCRgfX0GQaW06cYGO/eZmB89oyB8eVzBqZ79xiYjx5lYPj4CaIOCJiB"
    "WEJSlkFaUVVIVEQEmJ0ZuLA5ACCA4A64fuP6gSePHjN8BKb0Dz+AFsIshlrO+PYtA8uxYwwM"
    "L4DZWkCQ4a+uPsMfY2OG/zy8YEWM798zMD54AA7+f/8hGU+ImxEYCvoM0tKS5kARGWwOAAgg"
    "uAMePHh4+tmTh/deP33G8PkPJB6RI/U/OwfDP2Ehhn/SMgy/LMwZ/gCz5195eYY/evqQ+Pn5"
    "k4HxGyKdgRIoJ9B0WXUDBk1tPUNOTg4jBkh6RQEAAQR3ALA8f/T02ZOzzx/eZ3gLNOfHP2jq"
    "ZoCExj9gyv9tasbw28SE4T8wJ8BdCAoioOUMTIwMf4FZ9z9Uzz9oAhOXFGGQUdXmEYdEAx+6"
    "AwACCO6AV69eMTx58vTwq8f3Gd5/+MnwARgKwLSICAmQiaDk/h8qCLSU+eo1BpaDB4EZ7Cs4"
    "KkAOA6uHuhzkcDE+VgZJVR0GRWUlG1CyQHcAQADBHQCqih8+fLjhwd3Lr149fsTw+idS8KNj"
    "oC6m588YWHbuYGAEpYlfvxkYgcUx26qVDGxr1jIA62uG/yyQ3MAHTI0K2nrAMkFbC1hg2aBm"
    "WAYGgABC4Tx5+uzxq5dPj724d5vhxUdgzvgDDll4sP5HipK/wArnr4Ehw28HJ4bf1jYM/7m4"
    "waHCdPUKA/Oxo/BoYAYS4hKiDGKKGmzCQoIgBwgg2wkQQHAH8PLyMnADS7v3795teXX/GsOL"
    "178YXv1Ccy5SdPzn4GT45eTI8MdAD5gujBl++vpC0gbIUqAjGD9/g2gGukQS6DY5PXMGBXl5"
    "C2ZmFnVkowACCG4+IzjC/wOz463971/cePQamCWff4NWY9iiAYR/QzDIQcAGAsN/bkgoMH7/"
    "AQylf+BQ+AOKBqDR0mrANoOGjhovNycoS7LC7AUIILgDvn//Diz7fzB8+/7z3qsXD/e/vnOF"
    "4T6wQfX+NyQaUADIrcA4/s8KxUBTmK9fB6YLYHr4+pXhj5Iyw38+HrDr/0Nzg4wYD4OoqgGT"
    "tKSEI3LRDBBA8KKYjQ0SfN+AKfrR4yf7pJ5eiX/61JPhqQQbgwg/qLGB5OSf3xnYjp5j+McD"
    "bAExMjEw37vPwHLuLLCkfM/wD1iK/nJzg9QPUABKjIpA98jomTIoqKhZ3rp7T+c3sDkJkgMI"
    "ILgDvgJdDgMvX73d/P7Jhfuc924p3gNmIS0+pDIBmKpZr99gYF29DOhtJojpINcB09AfKyuG"
    "X94+DP8EBRgYgSHHzAjBoMTICCSk1XUYpLVNRGWuXPK///DxMVANABBAWJtkL1+9fv/u7bNd"
    "nHcvpt98rsNgLAJ0PTswt8HrBRaGfxraQIt/g+P9j5IKw191VYb/UpJAC5kYWIG55y/Q4q9A"
    "dz0FpqOHwDrr5Zd/DJ9/MjPwCEgA2xA8JtCi+QZAAOFslL56/Wad4NMz6Y/u+zHcUuBlkGaD"
    "Rj/QZ7/1dRl+62qDGhEMTCwswIBgBMfzX6DFH4HyD4CB+eAz0Aygpe8+/AVWdP8Yvn5iYvj0"
    "Axhd70DtzX/ssOwIEEA4HXD3/uPjSvI3jr+/dcnyio41gwEwGvhZoQ2Qv6DQZwL2DZgYfgJD"
    "5fUPiE/vfPjP8PTTf4YPn4EWfvwPtJgB2EhlZvj8jZXh95evDKxf7zAIf3wJLvihMcoAEEA4"
    "HfDj56/Pr18/2y748JTl9UfWDE+AASbBCYmG30CtL74DfQm04DawwHr5+T/Qp/8YvgAt//SR"
    "meHDFxZQzczwD1g5cX+9wcD97SwD96dTDN/e3fpx8/Xrl/cfPT0Jin+QPQABhNMBwAYqw5MX"
    "b3YIS50rfHrzleA5NTEGPhZI0N4BWvro/X+Gj0BffgVa9P4TI8PbD8wM34Byf7++B1p6nYHv"
    "+1UG9q8XGb69Of/p8asXz968/3Tv1dv35/78/nsVVPsD8X2QPQABhLdj8vjJ89NyUlePMt4+"
    "7XP2sTfDHWC58Pbtf1BuA1rOyPDqPSPDl88gv3xlEPhxmUHo20kGzq9nGX58vPP12cunD168"
    "envj46cfl4D9hStAVfegnZR3oAYYzA6AAGIh1CX79uPbWsGXe32unHVk+MfOxfD8NSPDd6DP"
    "mb+8ZBD6cZ5B9tdFBrZvl///eHft/ZNXLx68//D19ruPny8Du3AgS+9Cm+SfcPUNAAKIYN/w"
    "zr0nO20krjy7deWO1If/SgyiP84wqPw8xMD14/L/n5/ufHwJtPT56w9Xvn77ffnv37+XoEH7"
    "BtSAw9chgQGAACLogK/ffjz/9OXLCk22CUU/v7798+vTvXcvXr28+/rtp8ufv/4E+fIOFL8A"
    "KYdWgkQDgABihHXPwZURDgDsE8hzcjDHAPsJwl+//boDbI7fAAqDWrmvyLEUOYoBAgwAKL9+"
    "+iNoOGsAAAAASUVORK5CYII=")
index.append('calendar_icon32x32')
catalog['calendar_icon32x32'] = calendar_icon32x32

#----------------------------------------------------------------------
next16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAKfSURBVHjaYvz//z8DJQAggFhg"
    "DDaz2ccVJHgt3rz8cu/9+z9FDGKCGxnYeRkY/rMxMLBzMTDwANlMnAwMzNwMDNxcDP9n84P1"
    "AQQQE8wA9v8MBqV5Vgyz+9yVlFRFNjB8+DWV4e8/MQZG/C4ACCC4Af+APnn3n4lB20qWYU6/"
    "J0NYoHoW089/exn+/vfDZwBAAMEN+MHA+u7I8VcM+6/+Y3jwn5shM8OUoavCREdChGstw+//"
    "k4A2cGIzACCAGGGByGi5xJLhF/MEFV05Myt3PQY+IV4GI2kGBsH/3xlmrnnKsOv010v/2Lmy"
    "GTh5jjDwAMNgFiQMAAIIYYDNMiDBwcvwg6mSW0IoV8lAhYdLQpJBV4aRwVvzH8OFax8YJq/7"
    "/OHdF5aZDMI8jcBA/A7SBxBAaAawQzAnrxEDM/t0bjERMxYxeQZeblaGRFsGBlX+3wwzdnxl"
    "OHaX6cT/eXyWIH0AAQSPRgYxDWBIMkAx4ydeLsbv7CzMDB9//Gf4+JuBYdYRBgZPbWag2RwM"
    "TKwM8PAACCAWtDDhZvj9L0dCgqvUQFtM+M53YYZvXxgZlET/M/z5+Yth2a4/P379YJrHwM1c"
    "BdMAEEAIA/7/12D8zzBdUUXQQVJRhuHcO3aG70DfSQn+ZXj36hfD2w9/7/77z5zHwMa4HaQa"
    "pg0ggJBcwLRdQEZM4Y+YJMPpZ0wMPBwMDHysvxiePP7J8OMX82IGFpY8oPc+gL2IBAACCG4A"
    "IxuHxDdmAYbvX5kYBDj/Mvz58Z3h2cu/9/4zsjQysDAuwpWQAAIIbsB/FnYGJiYmBm7GXwyf"
    "3n4H2sqwjIGZuR4odQdfSgQIIHhKZGRmv/X3+3eG92+/vgRqjmVgYY4G5oM7DAQyK0AAMVKa"
    "nQECDADMwNCYef7LugAAAABJRU5ErkJggg==")
index.append('next16x16')
catalog['next16x16'] = next16x16

#----------------------------------------------------------------------
star_red_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAALkSURBVHjaYvz//z8DJQAggBjw"
    "GfDDktP1QyPzUy02Jg0glwObGoAAYsJn+B+Vf4H8wUxS7Hz/o4FcESBmQVcDEEA4Dfgow8HB"
    "qsnkw3BciSHKgsEbKCQOxOzo6gACCKcB3wT+WLCK8ckyPLBhCDdl1gQqVAEKc6GrAwggsJOe"
    "KrHp/WT6z/eThYEJGCgM/7/+Z2Ax+JPN+EuOgUE3lEH20WoOb8VPwX5fmf6b8jK9Yv/P+I/h"
    "LwPDp18MDwECiBEUiJ8bWJ7x+v+T/LObiYGRh4+BgZODgfH7dwbGv6kMjPqRDP+35zMwCpxn"
    "+PudheH/hx8MTCy/GZgU/zH0TmfcAxBAYAN0uJmsmiIZJgSpMZj+O6zGwJjQwcAor8zA8Pk9"
    "A8P71wwMz14xMLx8xfDv0mkGpi+HGBj0vzKU7P1/ZuLl//MAAghsACMjIycjA4Ncgj5jTWfM"
    "/yihUwJMTBZlDIwahkDNTxkY3r1n+HtsCwMz+1GG24J/foasYNx+6eP/jUDfPwIIIJgBjNAQ"
    "Fg7hZQydX/e/i2EpFysjrxJIjuHfh88MnKKPGY7L/f3ouoBh9o//DKeBah+CMEAAweIVlJp+"
    "APHLcFbGS/9eMDC/u/uTgenfFQY2YJD8+MDIwPWZiYFLmJH55///d4HqQPgeEH8CCCD0aPxj"
    "rPc/8MMFBqZP3/4xiPgyMvDGMzLwmTAwPL7/j0HzNwOPBg+DHkgjEH8B4r8AAYRiwAVGRjYu"
    "OQYvpm//GVTS/jOUXPt/Ub7n/6QtLP+vq/v8Z/jz8j9DkByDBVApLxCzgfQABBBKXrjOy+j8"
    "fzrD/8upDJ91+BhWAYXigdgViK0SpRkWvU9m+P0kkuE3MwMDKGmLgvQABBCKAZ3cDK4TrBiO"
    "czMyVAG5IUBsBsSSQAwMCQZpbTaGgskaDNeAVmcB+cBUxsAMEECwWICZwQrEEkAsBkrNQPwc"
    "iD+D/ApNtXxQA0Ex9gyIXwMEGABgbOjPeKFyjAAAAABJRU5ErkJggg==")
index.append('star_red_icon16x16')
catalog['star_red_icon16x16'] = star_red_icon16x16

#----------------------------------------------------------------------
edit32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAUFSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsyBxG/UR1QQm5fjFpGROG////UNMiRiZmlrcvn998/eR+8f8Lc0/BxAECCMUB3IJi"
    "LZ312Z7+pmIMv34zMBAdOYwQ6v8/CBsWq/+gBIjLxsrAcPzuN9Hcyr4OINcJphUggFAcICAm"
    "rm6jLsYgBlTMwEodn4Mdwwhxo7kSF4OkjJwssjxAAKGkgf///v/99RfV30cv3GE4ffUBQUv+"
    "AfGffxAMAjcevGLYeuQqwy+g+A9gZP4B0b+A6oAAWS9AABFMhAvXHmRYuvkYXstBToY5G2b6"
    "kTM3GHpmb2ZgBspwsEBCAFuGAwggFozoZIRE6J1HrxhYWJkZXn/8xqAkwIvX8n+wOIeGBAPQ"
    "W4ysLAyff/1iuHrvBdhMRSkRII1hHQNAAGGIsDAzMHz7/p2hqHMZw7XHrxj+//zN4OOgj9ty"
    "GB/NhzLiQgxvgY4PKZ7KwMXOxrC0M52BT1Icll7hACCAMBzw5y8DAxcnJ8P89lSGP0AOyPX8"
    "PJwEfY4cFd+AOcjaSI3hxNIaYJz/BwUrgwAPF8OdzwwMP/+i2gcQQCyYPoMkW2E+bvzBjsXn"
    "/5HkmRiZGAT4eRhAaRrkKVChcu8DA8PTT6jmAQQQC6lZCp/PYWwQ8RuIv/6C5AqQpzlZsCdC"
    "gABiIdlyPD6HcUDB/AtkMZLi/zhyAUAAEXYAkg8J+RxE/wSG9e9/qGLgcuIfQj8yAAggFiLs"
    "J8rn/6CW/4FaBIsKWNb8h8N8gABiIeTzf2iW/McSIqCg/vEX4ktYqQj3PbQugIUAejYECCAW"
    "Qj7H5ltk9m9QfP9F+BReIsKCHskT2Co3gABiwWYxAwGfw3wJshge30gOgIXEX+Q0wIQ9GgAC"
    "iAVHzYpSpSL7HGQJKMhBloPo/4xIQc6AiG90/j8cuQAggDAc8A89mGHxDMJQH//9h2TBP0yf"
    "/2NADZV/0HSCzQEAAcSC7nvkEPgHtQik+fc/NN8wEPY5LBH/+487BAACCKU6fvudgeHJJ6Sg"
    "B7rmLyOkLocnMrR8/Q/mUHQ+uGYFFckQ/B9HVgQIIJQQAOXjz79gqfsf3EfABh12n6OFyH8k"
    "h4Is/wt0FSzomYFmYAsBgADCiAI2FkaGr99/MCRWLmC49ew1WLww3pMh3MuE4eMXtOD9jxq8"
    "MD6w9mU4dek+Q9Pk5WCFnMAGYVtZAgMbhyiGAwACCCMRgvI1G9CE5AgnoMH/GXpmbGA4f+0e"
    "Q5S3CSSBIgUnNstBtR/QswwPHj1n+PntK0NpTiRQARODILBR8+ILZggABBCGA/4CTWFkYmJw"
    "NFdj4ABatmqzGMPP33/gFjLg8fm//4i08BtYB4sI8DM4W2sDq2NGhr+gmvEjpgMAAggjCv5D"
    "8/nnH8C8DmwZW5rrMnBysDJ8/4UWz8ilHTIflJaADRJFBRkGB3tzhk9fIOaxMWMvCQECCDUE"
    "mJiY2VgZGdiZIBr+AhNlPDDoQcH69ScwbpmhljEhUjuKQ6CF0j9gNJroKDKYGygyfPkGyWvA"
    "5iUDK6hxysSEkvMAAgjFAX8/v35w7PY7XSVRIYbvP5EqI6hFyLUbclQghwxKJQXrrIAsAtp0"
    "4fEvhh/vX7xCthMggBiRO6eMBgnGrGLqUwVkNVQZ///9g1FBMKLxGQiIo4QuM8vnlw+efn96"
    "peT/+bm7YMIAAcQ40L1jgAAa8N4xQIABAMHeEOw8KMSCAAAAAElFTkSuQmCC")
index.append('edit32x32')
catalog['edit32x32'] = edit32x32

#----------------------------------------------------------------------
folder_yellow_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAGnSURBVHjaYmRgYGCa3Sz4lwEN"
    "pNa+1wZSt4H4HxBjyMMAQAAxADX///PjGQYGiQPlnYCYC4hZoJgZiBmR9QMEEEiQ4feXixgG"
    "J5ZeBZLaewm5DCCAwAb8+/MBIvsfKMbIBKZ/vt8PNQQdaF8FGuIIZBwFGQAQQBADfr1h+PP9"
    "HsOaVQsYiAFA7+2HGQIQQFAXvANrxm4jLqANMsQCIIAgBvx+Axb69ekE1Ct/Gf7//wOkfwN9"
    "8wuIf4LZIG9/e7GCgVMsEGaKK0AAQQ14ywChPzAwAsMAofkPRDMDiP4NEfv7FajuI8wAYYAA"
    "ghrwHmrxJ4b/QFv+//sH1gR2CdAAuCsYIAbCAx0Y5AABBDXgHcQFfz5CYgJoDEgTKP38//cD"
    "ajvEFWDDgK6AAYAAQolGcFj8/w9xDVIYQAz6Azbk79/fKAYABBALxOmfGZBdAnEBRAOIDXH6"
    "P7DM338MUO+AwQ+AAAIlSy1gvJISf7AUuQpIrQcIIJABrEBsCMTOQCxEpP7vQHwBiPcCBBAs"
    "Y4BodqhhxABQ7gQFzh+AAGL8Dw00cgFAgAEAnPQAMnOYXgYAAAAASUVORK5CYII=")
index.append('folder_yellow_icon16x16')
catalog['folder_yellow_icon16x16'] = folder_yellow_icon16x16

#----------------------------------------------------------------------
newsub16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAACXBIWXMAAA7DAAAOwwHHb6hk"
    "AAADN0lEQVR4nHWRW2zTdQCFfxpNiOGBeCEGtvgkUSOKIZIgEpMxL8EHjIGJRmFKDBczM4i3"
    "OUSXQWE6raNzTnC1c7oLWbe0a8e6KkvpQFMmS52hXUvXy2bvXWvvl//6+TCjkuhJzuP5cnKO"
    "AMSGvfKREwOWuGzAEjvR/98+3m+JHe+zxGTnphY/7P0pvm6nTCUeqL1NAOJ9lTnpnvPhcvlI"
    "pIokUhLx5BJ/pIv8n/Z+rAuKjfX3CUA0KC+GR7Q6tNpRAOKXLhIYV5P2zVEug1SG4hLkS1Aq"
    "QwnY0zLiFZuOrBOAeLfLFDaMGTna1MJQmxJv00MEum7G2dNKqQwFCXIlyBQhL0EBeOWU1ise"
    "PfwPYOy8AVlrO+a+Yfyv3o7/kMDzbQdFCXISZEuQzkO2CHEJdnyk9YoN/wLodKNoRsYASE5N"
    "E74wSfz3IPnScihVgGgaolnwZaG6QesVD94A0DM4qAGgyHLNZGGJXAlSOQinYaEAgRJ4UvBk"
    "g8Yr1tff2ECvNwAQtFhxGS8R9CyQLEK0DP4COK2/4v7Nxlweth770SMeOXrv3wCtVofsEwXm"
    "wTFcHzyB+7M7menpxB/PYL9yGbvDj0P1Ju5z7zDjz1C3/6T/wB0b7xeAeE9pCs/a7Mg7ztDZ"
    "KCNYv4rUKcG1s3Kcjlls3XVYz8pwfP0yc92v49Q34GquyP1yi3hYAKLuK1N4+V0gnybV30W4"
    "s5Xo1Skc3giWbz7F+UUNQfVzLOqr8A9tQrb9mYhOrHxMAOKlz01hCUjmIJKBUBoCOZi97uVq"
    "bwu2/mbm1a9B6SSU9xPSbMHcsYXZA3dfFoB4UW4K54HFDARSsJAEXwZs7gBXhhRMqxrxfreb"
    "cvIYZGrxqzcz2ryteG336nEBiD2KyQhAVoJEAWI5CGUgVARXJIF9uI1gTy3z3c8SHagiYdiK"
    "ct+28LBYuV4A4umm87HpUBmLf4nJBQnzvMSER2IiCJpJOxdOH0Z9+gzWnkNY+95C09vGzNuV"
    "+Z9v/WvEFY8fqa+saZ+ofKH9h4pdCmPFLoVxbY3CuLam3bhmZ5txTVXj+Oqn5IbvD1ZfV72x"
    "3X3XDqXh+Xuq931506oVfwLuSQii3BRsGwAAAABJRU5ErkJggg==")
index.append('newsub16x16')
catalog['newsub16x16'] = newsub16x16

#----------------------------------------------------------------------
taskcoach32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAv3SURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsIIKRkRGnAplYV34h6X8ldpaGEeIiwuJ3Xtx8vmXdweVvl97vAEr/AuJ/lDgAIIAY"
    "QVGAywFqcaEaqhYsy/2D7A0EBYQY/vz9y/CP6S/D9cd3GFbkL969996DqxL8TGKMHByiDFwc"
    "An/evLv35uPfRXK3GXYDtf8BYoLxCxBAcAdckGGQ+PubwY+FlUGBjZNRUoCDU2yRmqb+gTRJ"
    "aU9zF4Zb728Dg4qB4fe/Pwws7EwM954/YRBb8ZFhQVQ4A4MAOwMDOxsDw4tnDAwrNzE833Js"
    "utQ1hjKg+d8IhRBAAIGj4IQkgzSjvOwBhcx4FXYpSYZPzJwM37l4GWw+/WPoOdLMICx1guEz"
    "20+G19/eMvz795/hz79/DD8YPjNs53zMoHU1mUFZX4/h77u/QC9rMcinGDBYszdnOt8+ennv"
    "b4YlQOO/YjhiBkM44yvG6//r/l8FCCCwA75+YUhWiotQEYqOYVh6+DXDkScyDOZKnAwc7KwM"
    "0YpvGKasL2FQ1JVhePf7G8PXr78Zfv34wfDv1ScGtpsyDMr22gymCgYMLMDQYQYm6WXX3zGI"
    "2vgwFCqfzN174892aDr5Cba4i0FJSIJ/kpGRkvfhtmsHfjL8DAYIILADWHkYYvm/XgDG2mcG"
    "CYYXDCz/ZBgSzMUZGK4yMAR5ZDJw7/nLMPPAFIafTN8Z2Bn/M4h+lWFQZTJmiA+OYnAz1QNG"
    "4R9wyPwH+jNCVYhh3jdvhgrP1ZraN875AI1YCnZED0esmZZcT6i1jujiDRe//9z2cx9QXBAg"
    "gBhAaeCICsPPf4U8//8/2fb/5/O1//17jvy///rX//d3gfjE7/+/fv////jxm//Xb935f/vu"
    "/f9Pgeyfn4E6//z//xlIv33/A4h/gvHnT7/+73r09//SeRP+r5VkOM2gwGzBMUV0WfpF+/9b"
    "/if+N12u9YuBj6ETaLkFEMsBBBC4HHj+nmHXh4dfGBjunmJgE+RmUON7zXD41hcGZjFGhh8f"
    "fzN8fP+TgYdPgEFKQolBTFSegV9ImGH/42MM4WszGR6/e8zAycYOjeb/DD///GWwE2Fi+K4T"
    "z3DBXd9Qt1B6W3eYY2SknipD86pz/05nX5/D8IlhE1DxKyB+CxBAYAe8/MGw8MkTIOPaHmA0"
    "sDJ4q35i2H3pFQMHBxMDqxAw8b/6x/Cf+TfD7z/AWPv1k+HPn/8M117dYLjNs42h5Gguw80n"
    "jxnYWdiBofmPgZudg+Ht5/cMZ5/3MRyOFWLuTPcV1BYWYqhec5HhZM61xQzv/i8HWQl1wHeA"
    "AAI74Oxfht33njHc+3/zLFD4DYOF0l+Gl29eMDx4/YvhvwQwAp/+BXruPzCR/WdgY/3PwML0"
    "h8FTwY1B7ocug5kpC0PF8XyGu88fM3CxcDJsvnCEIW5nAgOH3DmGdhtbBsZ/jAwNay/+OZp/"
    "eRXD69/zoJa/hGVRgAACJ8L5wDxl8p5hx+v737PE7p1lYDczYNDkf8lwCBgNkea8DK/+/Gb4"
    "8wIYErygUGZkYGL8xSAjKMOgzmTDwP37CYOPLStD6Z5SBgNBU4az/7YwRDjpMOgLKzI8//GI"
    "YcLuqwwPa289YHj2YwbU1y+QsyZAAMHqgn9vfjGsvfcYyLp9CBgNbAy+qm8Ytp97xcDMzMzA"
    "Kc/MwPqGgYGLj4WBHVgIMQHz2/c/fxg85IIZdp2+x2AjY8BgayHA8FxoH0OJuxuDGq8yw8Nv"
    "jxkmbTnHIHzDmsHFpUGGh4EBmK0YPqCXCwABBK+MTv5nOHHrGcPdv48uA5PGRwZr5R8Mb948"
    "Z7j78jfDF15GhpdP/jC8e/uP4QswR/8GhgIwTBhM1VUZFP6ZMay/sp/BWdGYIdbIieEPMNc/"
    "/nmPYe6uCwxC140Z+mIKGeJrcjh4DX1jgNYwM4DLUwQACCC4A7b9Yfj+5A3D5mcPgQ58doWB"
    "Q5CdQUvgGcPeq58ZpOU4GPjEmBlYPjIwMLExMfz4zcDw8ct/hhcf/jK4KgQznLjwguHet9sM"
    "r76+YXj+5wHD0j1XGZhP6zJURlQw/OcVYtDg+s8Q0zjBlYFH0hRoFSeyIwACCLk6/v/2D8Oa"
    "G3eAeenZSQaGvxwMfmrPGHaff8nw4f1fhrc8TAyvbv9i+PmXkYGZlYmBm4eFgYPnP4ONvi6D"
    "0EcThpMPrjK8/v+IYeOhuwz/jmsyFIeWMLDwCDPcffiO4fSNtwymYgJs4pmLahi4hARhVQAI"
    "AAQQSntgFwPD6Wt3GK7/fnKNgeHzNwY75c8Mr14+YXjz9Q+DlDonAzsTI8MfoM8///jP8Aro"
    "+2ev/jC8/fqfwU01kOHKpW8Me04+Yfh7WJOhJqGUQUpBjmHL0ScMtz4xMnxn52KQleVnkONl"
    "N2L+/lkfORQAAgjFAVeA0fv4HcPGB7eBaeXDHQYuARYGHYGHDNvPvAO65z/Dew5Ghvd3fjKw"
    "cDIz8PKyMAgJszJwAkPBwtiAQeiRE8PHLZoMsZ5ZDBzCCkBH/mGQleRmkBDiYtCT5wKmPGYG"
    "7nvnGV0YfwcDreKBpgcGgABCbxH9B0bzmksXGf4yvDsPVuev9pBh95lnDKJ8TAxi2twMTJ//"
    "Mfz6xcjw7hMwpl7/ZXj8/DfDO6DjCqKKGNpLWxhMbLSA0fMf6DAWBiUZfob7994yAJMTw81n"
    "fxhUZe8zFOgwBgBtlgcaDio+GQACCKNJtoWB4fL1a4yXfzy5BazD/jI4qbxjeP3iEcO9p98Y"
    "vgDbDa9+/GN4/+gXAzMHMwMfMIRExdgY+PmBISLMw8DGJ8rwFJhrnrz4A46ef8wsDOIiXEC9"
    "XxluPPnIYCT7lMEhQJDfjIEhGmgVNygUAAIIwwHAUuLPq8//19+9/B5UBDJwC4Ki4R7DjlOv"
    "GSSEmRgkdXkYWN//ZmAE5gZQTnj6EhgKQAufv/7D8Pn7X2BjhYVBABg1MpKsDLLizAyWOgIM"
    "p+7/ZNhy4SWDiYwoA4e9GEO8CnMIMBTkQKEAEEDYGqX/PzMwrL90Ctg++gqsTJkFGIK17jDs"
    "PAEspYDZ7wPQggcPfzK8eAxsFwBLaDZgehAQZGWQEGdhkJVgZpAWBuYSYLF9/v4Phv4tLxiS"
    "p59jmLdjD4MTwzIGHWFg6SsuyeDgriQOjAM/UCgABBALtmbSAmC1pHDp39mvj29bcIvaMTir"
    "vGSo3PmA4e4jQwYeUR6GP4ocDFxA20WAvmQDtvp+AqPl4fMfDGfufGQ4ffMFw5NnDxn4/t1l"
    "MBF7ypCv/J3B3A4UFfzAQkQHiPUZnj+b9/8DpL3ICRBALDiaav/efPm/9vaZdxYGpm+B8cvM"
    "oC94g2HXyRcMeXEqDG8MeYHp4gvD3iMvGY5f/whspH5k+PfjDYO68BsGL6VPDPYWPxg0xH4z"
    "sLAKMDD8AFrx+TUDw/2bQPZzYDH5jGHP/ncv3wHTGsgigADC5YD/wNbBxssHfzUbJF/jYBD0"
    "YIg0uMSQs+Uew/fv/xmOXX7J8PbDWwYJnvfAIvsTQ5LtJwZTqdcMgtxvgQUY0GigYxheANnf"
    "gXkKWHb/+/SX4RtQ+ONrRoYrh38zrPnAAKxwGB6CqmOAAMLXLGeu4WTcW7ZF2p5XLxrYFtBj"
    "yO2/wfDqxQMGH+P3DA4a7xiUJIHOZAXG6+8fQF8CK4F3wIT4EphLngGL6Uf/GJ49/MPw6PHf"
    "Hw9e//nw4OO/18Dq5Nk9BgZgucgAbHgw3AC1hQACCG+/IIuBITupUWKKcREfsPZmhbQvvwFr"
    "I2Dq//3sL8Ob+38Znt/9zfD0zm+GB/d//7r77M+H++/+Ai36/wKYm4ARw/DkO7Bm+QepgoH2"
    "ApvSkAbqV2jN+AUggPA6AFhkiSkJsuwOSBfW4wTWiM9u/mR4eOfXz7uPfn28/+rPm8e/wBY9"
    "AZr0FGgRqNkCamgAwx/Y6IJY9BvaKkam/yDRfwECCK8DQAUVMJEoSDEwZABpaaAXXgCdDgxJ"
    "BmBqYngN9cVPLJbA8F9o3f8X3mhE6y0BBBAhB4Bb7UAMagvxQxPtX6hFyD76Q8giXAAggIhx"
    "AAO04mCBFt3/kSz6R6xFuABAgAEAFFPKTSk5QagAAAAASUVORK5CYII=")
index.append('taskcoach32x32')
catalog['taskcoach32x32'] = taskcoach32x32

#----------------------------------------------------------------------
cut16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAL7SURBVHjaYvz//z8DJQAggFgY"
    "GRkxBEFisrKyDLy8vGD+79+/maKjo7vFxcUD29razHh4eN5wcnKC5QACiAlEcAA5IiIiYMzN"
    "w8OgqKgIZoPA379/GZKSkqbV1tYWXbhw4dSjR4/eMDMzwy0DCCAWETExBll5eQZWqEv+AfGn"
    "jx9BtjKAvGdubp6Qn5+ffuLECYbVq1fPZGFhYUB2NUAAMYmIi4PcDLYJhEGaXr1+DZbk4OAQ"
    "TE9Pb/v27RvD1q1bz799+/awpqYmWN3Pnz/BNEAAsfwDEmCnAjEoOFlApjIxgQ2ytLRMVldX"
    "l3zw4AHDqlWrJnFzc/8B2s7Oz8+vrKKiYv7p0ychgABiAWn6/usXgzwzs9vff//YXjEwbAF5"
    "4x/QEE8Pj7RfQLkrV678AIaLWkRExAolJSVjIFsFJF5aWloGEABBAL7/BAP+/hdbDw+gOQoK"
    "SCgKCgBs0dF/5f39oksgII4cICAl4+PjAPj4+Ah6enp5CAgIAPf39wDNzc3nmJiYfNvb254C"
    "iLHH3X0PMF4c3NavN5+mo9NnLCRkfu/du19qqam85xUUGD4/ewZ0zj+Ql74BXXL+6NGjay9e"
    "vLgc6MgXIK8DBBCLl7y886uvX98ysLF95+Tn59BjZ2ff8OTJhdvPn3PHWlrqTNm798HCNWvK"
    "X716dQ7o7DvoaQYggBhKzMyW/ygv//8gPf3/dVfX/67c3J18vLx2e0JD7+91dHwrxcISjC8l"
    "AgQQg6q+Ppu1sHBeorz8XEkODk9xERGdY2lpL/63t/+fqKl5S15WVg6ffoAAYtAyNYWzNdTV"
    "hfYmJj6/np7+rc3A4MTboKA/U1VVzwOl2HHpBwggFgakzKTJzR3qpKkpETVhwprlr1/HKPHy"
    "npT+/18RKMUGxD+xGQAQQCygRAMDb37/vvn00SOGSY6OQbG3bt2W/PRJNurRo2yg1GdcLgAI"
    "IGZmYDL+8ukTmMMuJvbg4JUrn9i/f1f++vnz/6Lbt+uv//o1FV8QAAQYADwHKjC+q+7PAAAA"
    "AElFTkSuQmCC")
index.append('cut16x16')
catalog['cut16x16'] = cut16x16

#----------------------------------------------------------------------
calculator_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAWOSURBVHjaYvz//z8DLQBAALHA"
    "GH5+QdYqKsqdMjLSCgwMjL8ZGJkYGBn+AyEjA5AAISCLEUozIMQh7mL58OHj261bN/VduHBy"
    "EcixAAEEN1hZWamxtrbaWlBQgOHHlx8Mv7//BRsLUvT/H8iEf2D6P4hmgNL/gBrZgKZysDB8"
    "+/Zd5seP781Xrly4CRQ9CRBAcIPFxcXlQIZ+/fiVoWRaGcNNpssM3GzcDL///GP4BzT8L9DQ"
    "f0CD/v6FsoFiX75/YdBlUmXoyehiEBYXYhAVFRZhZGRUBhkMEIAHMlYBEAZi6BMOV3/J/5+d"
    "RXB3kdKW3h0GRIdAAiEJ+YPl8/DQ0uRkZ14PzBaadAuoDk2o40XPiavc2CZeugo1ION7ikcA"
    "JsjgBmAQhoEmoKqw/2wswBCUBAUMr34sv3y20/9w57YgC7E/aFVQ0sJnJDJkqEMnPQRqDmPQ"
    "IeTxQmK4l7Ht7XhkCyC4wYzAyPr79y8DEycjQ6FbLsO7z18YWFk4gRp+Mfz78wPoei6wut+/"
    "vzIwM7IwMDGyMXz//5eBg4uHgV2Ih+HXr18wx4IBQAChuJiJiYHh8ZMXDC+eSzGICbMwmBoJ"
    "M5y/+Jbh+UdGBm09VgY2NmaGk2e+MvAKMjIYGoszPFu1luHb6a0MzyL/M8ibmYNdDHU1A0AA"
    "MSEMBTH/Mjx//p2BlUGe4c0bNoarN28zPH/xn4GDWYHh5p1XDNdvP2ZgZpBiePeBk+HKtWsM"
    "/y6fZZABhvPX8xeAQQQzDmIwQAChuPg3MDxVVYQZnnG+ZWDnYGCQkJBh4OT4zfDt63MGEVER"
    "oOWMDK9evGdg5fjDICYpz/DOzYPh6eNHDKJW1gzMf4BJnxGR2QACCCmMGcExy8/PBQyvlwwc"
    "HJwMnJzcDMLCnxnY2D8xCAiIMYB8+evXK4gcKxuDkLEBwxctVQYBPkGGf79/o4QxQAAhuZiJ"
    "gZmZieHOnfsMjx8/ZmBnZ2dQVFRkePDgAcPPnz8YPgHTNysrK8OzZ8+AYc3KoCAvw3Di9g+G"
    "26/+M7hofmfQVZOBhjHENIAAYkJNFf+AOegbg66uLtClwgwvX75k4OPjYzA2NmH48eMHw8eP"
    "H4FyegxioqIMr1+/ZfjNLsEgoazH8OYbGzDl/IKZBCYBAogJOYxBaVFOTo7h7t27wMT+j0FN"
    "TQ3sykuXLjHIysoyKCkpMdy6dROYpn8xqKioMKjyvGEQ+XyGQVuGjeE/IysDcnkGEEAoYQwy"
    "+M2b12D+ly9fGN69e8fw6dMnYBpmYXj79i045YDUAcsFoBwwGX7+z/D4Gx+DxLvPDMICPGBf"
    "w1wMEEBIyY0R7EqQJpDLQEHw+vVrYERxMGhoaDB8/foVbAnIpQL8/Azv3r5n+MIozMAppsrw"
    "+ivQxf/+ooQxQAAxIUceqMQCeRk5KNjY2BiuXLkCDiJQZN6+fRtYMP1mUFJWZtDke8sg+e0U"
    "g44sO8M/BhZYbgYDgABCShUQq169egX2OsiFIBeDIgzkalBEMjMzQ+W+AYPsFcOD94wMj78I"
    "Mwi8+cQgyMcNNQNiDkAAwV0M1MQMCmNQ7MvIyDDw8PAwvH//HpzsFBQUgEnuJzjFgHzEy8sD"
    "tPAzwzdQUIgoM7wDFlrAXMDAArQYFBog8wACCO7ib99+fGBiYgaGpzowW78AZgh+cFi/f/+O"
    "4cOH98BgUQWH4atXr4EG8zLIywEtePON4d2XRwyKEgLAkGRn+PL1x39gEILDAyCA4AZv3Lhu"
    "BrDgVJWRkeQFpud/INeDwhkWIf/AtQcDNOYhNQhIHBgzDI+usTK+ev2RcdOmDRd+/vx6C2Qe"
    "QAAxwipTZmZWYGCwmAJdrQEKIkZw9YbIopBSC50PiRtgimL88+f3nx8/vlwHClwEmvkLIMAA"
    "DmQ6FmXRrtcAAAAASUVORK5CYII=")
index.append('calculator_icon22x22')
catalog['calculator_icon22x22'] = calculator_icon22x22

#----------------------------------------------------------------------
cogwheel_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAOjSURBVHjaYvz//z8DJQAggFhK"
    "SzsY+PmFGNjY2BkgZjEFMDD8v/Pjx/cr/Pz8nXJyUpHv3384/+LFy2U1NWkry8s7Gbi4eBi+"
    "fv3M8O3bRwaAAGLi5xcAm/Tv339JAQH+xa6ulus1NZW3AYXy5OWl4gID7WRjY739Pn58XQRS"
    "9+fPHwaYq0GWAgQQs4ODFwMjIxPQVO4lFhaGYYaGKgxyctL86uqKnvr6KjysrMwMzMxMDJ8+"
    "/eRjY+Nj0NbWKfr796/p9+/fd/3//5cBIIBYGBgYGX7//sXNxMSg8+HDF4b3778x8PBwAA0R"
    "AbqKAewtRkYGhuBgF15HR8s2Pj5Ohhs3HjCsXr3l0devvyYBBBDL379/NCQkJFeKigqLi4kJ"
    "gG37+fMPw6NH74GGfQE6k5lBRISXQVpamEFIiBvsdCEhYYbHj+8Gb9y4fApAALEAQaWOjqae"
    "lpYS1H8MQBvuMyxbtuL6u3evHmlo6BiYmVmIg1wjJycMNmDdus1f5s+fnKmnZ/gPIICYAwKi"
    "JoiIiAn8A6oA+ZeVlY3h2LGTvyoqMpwuXTo9Yd++rcuBMaOqoaGnLiEhzMDExMjw9+9/pv//"
    "Wfm0tQ3eAgQQ04cPb1nevXvP8OTJa4YvX74DXfGbQUZGhk1DQ18nJCSWQU/P6Onfv79jr169"
    "8uX/f0awCywt9VmmTJkQ/+HDhz6AAGI+derQ6jdvXn7T1TWyExcXASsAhgeDsrKG6+fPX/5K"
    "SUlz8fMLxlhaWtgpKEgxg7zy7dsvho8fvzIsWrTgKEAAsXh4+D9WUFDj4efnYWBhYQbGyB8G"
    "UMzY2VkLaGqqd3/69IlBXFyUQUpKHCj3D+jSVwyvX38BhtM1hqtXz+0ACCAWcXEJAWAYhIMC"
    "DxSNf/78BSUroCsEwS6RlpYAR+PPn78YODnZgSnwG8PcuZPvnjhxYNHTp4+XAAQQy69fvz8s"
    "WDC5QFy8bcXr12+AUfeWQVFRCZiwOIEBygLU+Jvh82dQ2PxgkJWVBEahCFDNm2dXr15qAnkX"
    "IIBYQMStW9dXtrdXcH/79u0PDw/357y82qVycvKc37//Yrhz5x5Q/gaDjo4+MGGxMhw8uOf/"
    "4cN7ZsIyE0AAsTABkyAI3Llzcx6IDgkJAyWSxw8eKKhxcnIxzJw54eCePZvrw8ISeoDS7OvX"
    "L6sFhstGmAEAAcQQFhaPkj3Dw6MZBAUFDRUVlQssLW3ncXNz20Ol+IBYHD07AwQYAPMmY7/a"
    "Z8krAAAAAElFTkSuQmCC")
index.append('cogwheel_icon16x16')
catalog['cogwheel_icon16x16'] = cogwheel_icon16x16

#----------------------------------------------------------------------
treeview32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAbzSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsIIKRMTmNh4upjoOdifvf//9/QWLggGEEMcCIASaGHF7/GdDkYfqQ5TH0MTL+/fOP"
    "4c/Pv5P+/5/XCBBAjKAo4ONNe9rYEy/FKy/F8PXbf4a//xgYgGoY/gJ1/QU65w+I/gfFUPYf"
    "qByyWhD9D5kPlAcywTRIPUiOgYWR4cPT1wwX5s579evrDHGAAAKHADcXK5eEjjLDD3EJBvZv"
    "EENZkAxi+YswHOYAmOV/kSz/i4RBan+D6P+oav8DbfzNIcjAxM7BCrIbIIDADvj3j+nvly9/"
    "GB4D2R8/QcIOZOB/mK/+o1r+D2YwTA4phGDs/1D1cL1QzMwBNP/Hb6DZTKDwYAAIIEgaYGJi"
    "+PWHkWHlmgcM3z9+Z2BgwoxfbOA/ckQjMxkZMDSCuExA8X/MbAzSsnxgO0EAIIBYIBqYGT58"
    "/cvgLPWBoTBXFujS/wzouRNkJhMzEwMzMzPD79+/iUrhjMgOAHLYgPG/cPdLhiXXfwOTIjNY"
    "HCCAWCCuY2L4DUwlwiI8DKpKwjgNfPf2NcPDhw8YDA31wY4mDvwHx+U/YDwwMbMwSIr/ZPhz"
    "BeQbSAgABBATxHVM0FT9D69RR4+dZGhs6mZ48/YT3mj5/esPwy8g/gGM6+/ffzN8A+Kv30Ch"
    "9huYI/6C0wbMAQABBIkCJmZgsDPij3AgAAU/Ozs7AysrRBvQdIbbff1Ai4C+YvzJ8P3zBwaF"
    "kFgGCSsrsCNABoKyOQxDzGcEJ8z/TJAQBAggSBQwMoJDgIVQnALVgQx69+492KzPF64wfLuw"
    "i0EthYeBkesuw4ODNxgebhFmkLS2RrH43z8IhiQsIBuUIBghKQQggCAOAAbHX6AgIQewsrIy"
    "fP78kaGhvpXhHxsrA8/jVwwFQm8ZOP/dAaawvwy8glwMX38wgwsfUJz/+wdzAIT/H5w/odmU"
    "CeIAgACCRgETOL+zE3DAH2CRBoqCkBB/Bm5BPoZ/V+4wsG3eyfDn9k8GRgGgxfe/MTAz/QY7"
    "ABzi//9D6X+IaGCAlA0MzBCrAQIIGgKMEFcRSAP//v1l4ODgYrB3sGfg4+Nm+K+tw/Bg21yG"
    "J5teAUs3ZobP7/8yCKTrQAokIAEphP6BHQBK4BD2f7ADYVEAEEDQcoAFmAYYGf4TkaVABvz+"
    "/QtUgDP84edjkFy4leHPt1/geBVj/MfAIsTL8PvbdyAfMw2A0wEDJAqAJRGYDRBASGmA+CoU"
    "HlJAkxj5eRlY+BnBZS/I0r+/foN9/x9aBMPSAiw9QGpMRrCdIAAQQHAH/PmHpfyEW/gfnAMw"
    "ogRk4K9fSIkNaDEom/1jgAc7LAT+AqPv719msOsh5QAkGwIEEEoi/M+Ix7dwNjQr/YM4jImT"
    "HVzkgkKVCSj168sPoNwvaOJjgPoe6BCQI/9BQgBcLUNzAUAAscBKQlA2RHbAf6SaBjkE/v9H"
    "1DYg8dd7DzB8e/kRWBCBsh+wODfXZ2AVFGL49/sPUvD/Q8qWDOD08h+aBgACCBICwEoGVAEx"
    "MGHUdXCfwMSYgKHFxsYGpIE55/EzhseNpQz8pmwMf9meM7y69Jjh/fVaBu3qOobf795B0sRf"
    "iMV//oCC/i88ETJBowAggBBp4D9yUfwfno8h2e8/sBiG+Pwd0OBpU+cwsPJwMzDdvM/gpcjK"
    "oB72lYFBgIHhxQlOhtuXv8Gz3F+476H0X2ha+c8Ar44BAggaBcwMyPUQuuWwhCQqKsqgpaXK"
    "8Oz5I4bfQAN4n79iYGF+wsDwBNiG+MLEwPTmFwMLcrwjZUFwYfTvHywJwSsjgABCTYRIcQ4r"
    "xWBx/fPnLwY9PQMGAwNjcIHECCyKv125zfA2eS3D66VfGf7yMDO8uf+XgcWaFZLiQSEA9TEo"
    "RP78hYQIrHEAy4YAAYSSCGHBAopfJmiMgJpWYDZQDlKh/AEL/v0BtExekoGvcR3DzzcfgRUt"
    "IwM70GIBEwOGf1+/APWAEhqkbGCEtgmALWBgcmOEZBloIgQIIHgIgBzwHZiFvnz+zvD7739E"
    "mQCKL1DwQaMDHCLQfA5WYW3PwMoIasRCo+rHd4a/P39BKz5GiKVAx7AwgVpTwKbfz59AcU54"
    "CAAEEKwuYOLlZWc4f4+dIaL3Ptgx4FrrPwO88viPaNwAPQCR/wevbBB9ArB/kdqSsFr4P9j3"
    "3xhefAZmOj5Ohr/QfA0QQGAH/P7H+Ofdm7cM3PISDO++/AVrgrWK4a1gBmjzGqk1DOZDC6F/"
    "SE30f0hNdWQz/oP7BcDE+uM9PMEDBBDYAb/+MUw8tvFgPhOo0fb//z9srVuMHhE2/n/88lDj"
    "GP/9B5b7//4sBIkBBBDjQHdOAQJowDunAAE04A4ACDAASqlW/oTQwvEAAAAASUVORK5CYII=")
index.append('treeview32x32')
catalog['treeview32x32'] = treeview32x32

#----------------------------------------------------------------------
redo16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJWSURBVHjaYmYgADyNjef7WmtO"
    "4OEUOnDv+fOX6PIAAcRMyAAtKamCBUvaDBgYeCK+vP34AmjIRWR5gADCa4CDsbE/BwNDRESE"
    "o4C+rTuHuqJ4wNfnTxT4hCUPPH7+/CdIDUAAMWLT6GJsHO9opjDB1cVGwNTaiIGBkQmKmRle"
    "P3/G0N0x98LRmy8Sjp09exEggDAMiLQzmJ+fYJ1gbqnDwMDEAsRARzJCaSj/67ffDIvmb/+w"
    "YuelYIAAQvFCpL1+f1uKbIaOmhADw68PDAy/PwJpIP7zCcj+DKQ/g8XY2D4ziKrzcVy8+EwI"
    "IIBYYJodjY3to43eFCiwARW/fMrAwMrDwMDCzsDAzAbEIBqoFOReKWmG/VeZGCb2H128cf/J"
    "QoAAghugxP9rgbc+0NY/QMXSygwMigYM115wM1w7+5whRO0RA8NfBoav8oYM85Y+/DxvyZny"
    "C7duLQBq+w4QQGDNVkDb95VK/P+/gP///zvp/zctzfkfZG+2SU9TM8nbyOjS/xNB/189bPtf"
    "EOt+Q1VR0RM59gACCIy8jI37v0zj+v//dsr/KU1Rn3SVlWOAwmwgOT9j4/0n9lf+DwAaKCUu"
    "rowe6AABBEZNQSrn/+8y+b90Ztp/DTk5f2QbgFGqb6aungZkcmLTCxBAYDQzXvb/q5uV/+20"
    "tacSkzqRAUAAsQD9L88nxcywYuXDz7devZrIAA4u4gFAAAGTF4OCsqUBw4G9dw68eP36LgOJ"
    "ACCAQAYwPHzOwPDgxcctpNoOAgABBDLgwfLll06++PBhLwMZACDAAGyQxmsDWqtQAAAAAElF"
    "TkSuQmCC")
index.append('redo16x16')
catalog['redo16x16'] = redo16x16

#----------------------------------------------------------------------
lamp_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMvSURBVHjaYvz//z8DCNQzMjJ8"
    "Z2Cw4JdmqJU1ETIS0ZDm5xYUYGVgYGb89e3z3zf3Xny7cOHZk0vX/y+4/Jdh4isGhj+/gHoB"
    "AogBZAAI1zIweC/35fzy8nLs/79/NgKFrgHxcyB+BcS3////t+//z+c1//fXSP+35GBYx8LA"
    "wALSBxBAcANmajFeeX87Bsg8/P//u4NAfcf+//8G1Pzj/f//b87+//963///v4////+34/+S"
    "GP6/fAwMWSB9AAHEwgAF0nosygIq/xgYnh1kYHj7iYGBU5KB4e8bBgZGoJJvDxkYvj5iYPjA"
    "zMCg8ptB14GDSWjlR3egtmkAAQQ3gJPxNxPD910MDG9uAQ0AKmQTZGD4KAaUYWJg+PWWgeEH"
    "0NccvxkYpN4zsDG9ZmBlYuAG6QMIILgBr+8xvGd490acgQmo+BNQ03+gFBM7UIaRgeHfTyD/"
    "FwODCNCFfxkYXl4Gmv2T4R1IH0AAMcEM2HKeYfXj5UCGFDBWRIGqWICa/gG98vcj0KAfDAx8"
    "QM1KDAy/zzAwrFvD8Avonq0gfQABBA9EAwYGrmx+huPP+oDcB0B8GogPQPERIL7H8P/3Nob/"
    "rVoMfzkZGOaALAfpAwgguAFyQBEJoCHBHAzbrlZBNPy/CsSXIAa+Xcrwv1SO4SfQUxNgLgfp"
    "AwgguAFmQAE9IFYB4ulyDA/BrvgIxG+A+B/D/52BDP+BqWoGAxIA6QMIIHggpjJBmH+AXpVj"
    "Yvr86QMbAzMbSICBgfUfMJX+/vX/L8PPy8wMqAAggOAGnNDXB9P//v5juC8rwSj4KJSB4w0H"
    "kP+XgZuTk+GMyFqGfwzLWdENAAgguAG3pMUhLvj7h+evgobwN157hl/MrAx///1l+MPJy/Bf"
    "+QkjF9cGtW/fvqMYABBAcAPYoBJMf37zcv/7zc3G/I2BhZWF4d+/fwwswKTAx8bIwMbGpoJu"
    "AEAAwQ34+u0bLGCe37x5/cq9e9ctJMSlGP4CvfABaMDp08f+f/785SyaDxgAAogRlp1FRUUg"
    "AkDFP3/+1hIREeliZWUVBrrgLxMTE+vz58/uffr0uQio5DlyLAAEGABF2H3q9ig4swAAAABJ"
    "RU5ErkJggg==")
index.append('lamp_icon16x16')
catalog['lamp_icon16x16'] = lamp_icon16x16

#----------------------------------------------------------------------
newtmpl32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAGiSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAD7gCAABpwBwAE0IA7ACCABtwBAAHEAmO0CzMulzJmsAYyf8FlGYEYlEn+AvEfKP0X"
    "jQ+i/+Hh/2VgBrLZYfqY/jAwvvjLcKb0/39fkBUAAQR3gLQpg23cDgZpuOX/oAb8BOKvQPwF"
    "iUbHX7HwP0Ppr0h8qNzmHwzmMGsAAgjuAGBx8APM+AVV/A1J83coHyb2DU38OxZxdPo71DN/"
    "wYH6E2YvQAAh0sB/JJ/D8H8sGB3gk8MBkJUCBNCAJ0KAABpwBwAE0IA7ACCABtwBAAE04A4A"
    "CKABdwBAAA24AwACaMAdABBAA+4AgAAacAcABNCAOwAggAbcAQABNOAOAAggJrxV1j8S8H8i"
    "MRoACCB4e4CJBdhqAQEOKBaFGvyDhIbIF6SGCBsQs0K9yIzkUGB7gPk3WBYMAAII7oAXlxgu"
    "bExjYIE2piDNMVhI/IaK/oLSv7HgP2jq0MWhGNgkY773n+EGzF6AAGIc6J4RQAANeCIECKAB"
    "dwBAAA24AwACaMAdABBAA+4AgAADABm2q1BWHwJuAAAAAElFTkSuQmCC")
index.append('newtmpl32x32')
catalog['newtmpl32x32'] = newtmpl32x32

#----------------------------------------------------------------------
cogwheel_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAXZSURBVHjaYvz//z8DLQBAADHC"
    "GC0tsxjY2NgZGBkhQn/+/DdmZma6+vfvnx+MjP+BchxLeHl5rD9//nL2+/cfZ4FyWysqEi6V"
    "l3cyCAgIMfz48Z3h16+fDOvWLWS4efMKA0AAMUlIyDBERKTBbfrz5688ExPzbFVVhSMcHGwb"
    "/vz5I/bjxy81MTHhoPBwN4XoaK9gLy/Htjdvnk3A5tLAwBgwDRBALCkpJQzc3DwMf//+A2F/"
    "CQmxuVZWBsJKSlIM9++/cD9w4MRZYGj9MzDQ4OTlZWcAYT4+PpBaOZAB//79Y/j9+7ckkP4C"
    "5H7+9+8vg6ysIgNAADE7OHgygMIZ6EoBXl6+9crKSlKqqjJAPhODsDAvg4aGMp+JiRa/pKQA"
    "3FUg9f//swoICEjpCAuLBCgoKEz4/v27KzA4VgCt+mNmZs8AEEAssMj78+e3IBcXp+z37z8Z"
    "Hj58yaCiIg32BQcHKwMLC6p3WVmZGLy9rZiMjLRCgfYziIsLMJw7d9tm/fotc/79Y4z//v3L"
    "H4AAYra39wC51h8YGSG/fv3SFBUV4ZKXlwBGFis4IkGGv379keH58/cMHz9+BbuWkxMUyQzA"
    "YOFg4OHhAFsmKSnM8O7dD91jxw48ZGFhPA8QQCAXS/PxCcyztDQR4uJiB7qQHRjmHEAD/4Ii"
    "kuHOnacMb958AFsCjEyGFy9YGISEeBmUlSWBlrNAffuP4fTpWwwHDuy+vXLlvMtZWSUMAAHE"
    "wsTEmAt0pZCwMD/QBVxgF4EMZGdnZbh79wnD+vXrn1+9eu4IMFL+qKhoGltY2KoxMsoyPHr0"
    "Gmw4NHUyXLp0hWHJkhllYmLip0F8gABiAQJLdnYOhrdvPwFdCfImKwMzMzOY/ezZS4bly2e1"
    "Pnp0dyo0ePkPHDDNSU0tbLa2tmD8+vUnMCjYgXHAxKCtrcrg6RmSISQk8A8Y8XsBAojZxyd0"
    "opSUHPvPn38YPnz4DA4/kGtBqeLdu48M+/fvPigoKHKUl5cXlAF+Pnny8LCEhBSvlJS8lby8"
    "FFAtJDhkZSUYnJycVJiYOCLPnj0lARBAzFxc3PJfvnyW4+QEhiwHJzDGWRi4uDjAQcLDw83w"
    "7dtvDWBwbTUzs36rqanNoKamATSM4wc/v1CioaEO0HeM4OAAYU5ONoafP/8zLFgw+ypAADHf"
    "uHF568OHd5Z8//5VVVfXSENUVBComIkBlApBkaWsrMr/5w9DxNu3r/n///8nxMnJ4y4tLd/i"
    "7u4sJCoqAFTHCMwgfxnev/8KpP8zXLx4nWHDhpUrAQII7A8/v7BfwsISeoKCvGDDQEkMlAp+"
    "//7DAIrUkJBA0WfPXlR/+PAB6AsuBiUlBWD5wAOWZ2RkYnj69DUwKf4ABtVfhhMnjv94/fr5"
    "SYAAAhvMyclVq66uo/jlyxeGz585gV5lY/j06QtQnAOc9EA+UFSUAUaSItjrwNwF9PIvIJ8Z"
    "mKX/ADPHFWD2v/MbmMkY169fPPHz588HAAKIGWQwHx+/nJycki+olPr37z/Q5v9gg4FhD44Y"
    "UNqGlCV/wa78+fM3OKhAFoLkQGDhwllXFy+eHvHixfP1QO43gAACG3z37s1z79+/5tXSMrIC"
    "JfZz507+//nzJyMPDx8wE7CBIxOSC/8Dw/ILOLW8efMO7COQQ96//8Zw7dql/6dPH68FGvcB"
    "ZCZAAMFLgRMnDpcCKTZgYcJ49+6NdcrKak6JiQU1lpbWQJd/ByfD9+8/Ab38EBgELMCwFwRm"
    "83dAC1kZ9u7dwbBjx/ouUOkGMw8ggJCLl/9Aw/NgHHl5uUufP7+vZGRkZgbF9u/fPxgOHTrE"
    "MHdu3wwpKVnF2Ngsd35+QYZduza+2bFj7ZTnz5/PBgc+FAAEEAuuqgWYvr9dunTqwNevX8yB"
    "BvGAyuytW1fsuXnzegEQ/3337lUnKyu7zPnzp3uA4X8aXT9AAOFEERHR4AQDjHlDQUHBSCUl"
    "5SZgarFAU8aKSz9AADHSqjIFCDAAdoRPhwDclxkAAAAASUVORK5CYII=")
index.append('cogwheel_icon22x22')
catalog['cogwheel_icon22x22'] = cogwheel_icon22x22

#----------------------------------------------------------------------
save32x32 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAZfSURBVHjaYvz//z/DQAKAAGJi"
    "GGAAEEAsyBxGxi4JBlbOJjYxEQsWRoZ///8DETyEUEPqP6YQA0zpfwiLnZGBkYuJmfnXP0bG"
    "n7/efbjB8O1T/f//5deR9QAEEIoDGDi5u52KfWL0rGUZ5P69YdCUYGL4z8zB8PffPwagW4Au"
    "YmD4ByT+Q21jhDmFEYT+Q9ggivE/AxMjIwPTv58Mp+98Yrj4T5PhxcP3umcnbhADyjogWwkQ"
    "QCgO4JWVcrX3kmcQYmdg0GD4w+Cix8fAwMLD8A8o9/cvxId/gZw//yB2MQEjkJkRQjMh0Qjw"
    "jeHbh+cM9159ZmBTZWO4o6VijB4FAAG4IqMUAGEYhga7ifc/nR+7R4VS9U0HMj9CKZTkhU4A"
    "pSxnhuSYBz/wgyYbgdmbI2YOiAdIb6D1YEP6IIxdcXFb1doux2Ctln+AWwChJkJGSOyBgpoJ"
    "6DUmoEkgH4J9CQpmJJ+CLAWGMiRa/kMcB4kihBjYrH+/GaQU9RlkNYwZ/v/9g5HlAAJwRQZJ"
    "AIAgCNQsp/8/WJPkUh4dBnR5CMAUCphYpdg9ogJZrzA0Q7r/aDogkkSfo/eqrcET213mIpVv"
    "jgBCywVQB/xnAFvOyIjwMUic+R/EcGDsMPxlRITfX6ivGf9DHPsPZPk/EBvkASagPCgqIKGJ"
    "DgACCMMBjFAHMIHCG6qBhQkSAiCDwT6F0oywSPyPSBsQAyDR9R8ckoyQqGBAT6AQABBAKA4A"
    "+fQf1CBw8DMgkggLMPj+wF0KcSwoN4B9DRX7A02cMG3MQFcyA8P9LzRtMGGJAoAAYsGWIsHR"
    "jSW8QCEBcgTIZ/+gqR8U1H+gCRQcEtAoAUfnf0gagBVSzFjKXYAAwkiEsOAEJUJGRjRHMEJC"
    "AiT6F5Jp4Ib8g6Xvf9BsCk6ojOAogKURFiwOAAggjDTAAM0FLLC8hwWAUjMsCzJBEyQ4EUKj"
    "Biz2H+JKkANgBSQ24wACCCMNwHIBEzDCvn75wfDr3y9g8P0DVQwMoHrhP6xYBmZ8kNi/fxAM"
    "kvsL9CpYLSjVA/nMwKKYlZWFAVRo44oCgABCdQAjIhHse8jFcPDRf4jF0DiEFVJwx/xHhBhM"
    "DcRR4AoJnFX+/Gdm+AlMOEws2EMAIIAwHAAKQlCcPfwE1AxMaZ+/A0vU34jcAaP/QtMKmA8t"
    "BUGFEih62FkRav5B6w1eTkgIowOAAMKaCEEFJjNQ5sX93wzc794zyIgCHfEXUdkhV7+IKpiB"
    "ARjaDPdfMDEcv8rOwC/EzCBpwMnACExLf/7gToQAAYSRBmBBCnI9498/DLnePxmMdLjBhoB8"
    "B6L/wpI8WpByAH25ad93hqNngMb+ghZA0BwBqz3RAUAAsaDlMrBP4EEM5P0E+vzXPxaGtx9/"
    "MvwCSnCyMjNwcLBhzR2swKADhZ6gIAMDPx8TvHKCRR22ohgggJjQo4CRATn+/jP8AqYgUMOi"
    "edYXBrvQfwzr930ClnC/GL5/x8Tfvv0C5wQ+XiYGbm4mePsBlkixhQBAAGGtjP5CS7r/QOLX"
    "rz8Mf37/Ynj2jpnh0WNOho+fvwIT5W+gOKZ3mMA54A/QAYwMXNwMDG+hJeMfqAOwpQGAAEJN"
    "hDAHQBPbH6CBP4Eh8P07MBRAxR4HJP9///GH4ccPLOEJKgeA8vzAEGDnZmR4DU3Qf6GJlRGL"
    "FoAAYsEVBeAaDxgFv4Eh8PMn0Mc/IVXg7z8Q/s+fTFgdAHIgHw8TAyswCv59Y4Q0VKAtKmwh"
    "ABBA2NsD/yCpA5Rwvv34xfDmAzODt9k3Bj2FTwxqkn8Z3r5nBud5dPAb2GAApwE+YB3ACcwF"
    "30AlIiO8nsCWBgACCNUBTIwsoBYN0NPA1jCE/vL5B8PbdwwM2goMDAbKvxm+/2IG8r9iDU5m"
    "xr/A9MICTAPMDP/ZmID6GRl+AAux338gtSgHMyMLuh6AAEIReHLr1eX/7z46SUnxgy1n+sLC"
    "ICzMxiAjzcvwA5ro+LD0ERAO+MfwCWjVbx5Whv9AkyU/Axu4vyEJWprzF8OlK08foesBCCBG"
    "5K4ZI2OHoZKW9AQtc1k9YBPq7/8/vxk4vj8DFq2gcp+RYC8HGNgMP/6xM/zjkQAmF2AO+gtt"
    "wrEyMD++/uLh5ROPy///L92JrAcggBgHum8IEEAD3jcECKABdwBAAA24AwACDAB+IPC0x2Wb"
    "gwAAAABJRU5ErkJggg==")
index.append('save32x32')
catalog['save32x32'] = save32x32

#----------------------------------------------------------------------
calendar_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAPsSURBVHjaYiwpLmL4+vUHw38G"
    "PoafP/8y/Pnzi42Xl9vy31+GUAbG3+ICAszfdHV0XJ89f87T1dWV+/r164UMSAAggFj+A4m/"
    "f/8y/Pv3J5SNlTWSn5/LWECQT1JWVppVRlqSQUFRmkFNVZnh0uUrDEuXLctBNwAggFiA2ICD"
    "g71AUkYuSkFRmVVaRoJBXEyYgZeHm4GTgx0o/Z/hz+9fDHIKCgyOTs7aF86fNwcKnoQZABBA"
    "DH0TJ32+eP3m/7efPv1/9fb9/09fvv1/+/7D/8/fvv3/8vPn/1///v//+x8CNmzb8V9ERHQ2"
    "sgsAAohJQFCYXVZegYGdiYmBnY2VgY2NhYGPm4uB+9NnBq4HDxmYnjxm+PfnD1ixvpExg6GJ"
    "qTuQyQMzACCAmC5dOP/01cevDBzc3Aw8QGez/PnLwHLhIgPDjRsM/1+9ZGA+foyB8dJlhn9A"
    "xTLiIgzWjk7SrMxMESzMTAxsrCwMAAHEDGQYqRta6yvISzAwgwL0928GcMBqajD8k5JiYLpz"
    "l4EBqPCPvDwDK1D8Nwsn45H9e3nfvn278O+/fwwAAcT0+cO75aeOH//96hsDAyNQwT8OToY/"
    "CvJAGxkZmA8fZmB8+pSB4eVLhv/PnoOdrKmtw2BgYWMKZOqA+AABxCwmwvcIGP/hsvp2Iori"
    "3AyMQLf+AzkBaPp/oM1/lVUYmK9fZ2AGeuOvpiYDFy8Xw/NPP1iP793x/devX7sAAojp06ev"
    "v57cvrzh7JnL/1/9ZmBgATrjP8gQYID+UVRk+KOsBDSIlYHx40eG38DwYQOabWFtxaikqR0E"
    "ZPIBBBCziCAfw7cvHx4xsAklyRo6sSkLMjL8AQYG84kTDOxbNjOwHj3GwPzlMwNDUAADs6ws"
    "2JusfPwMd5+/4b90ZP9TgAAAQQC+/wMOCwgA/fz77vz390EHITwjLCgUAAsBAAAB8/MABf/9"
    "AAEtKgD5SkkA6RYbANruAADD5AAAwuMAAPPp8wHhzbYHAojl719QqP9nePX46uzzh/Y5XbBR"
    "Z/aQBCrQ0mS4q6bJcA+o6dZNBobbQI2vXvxj+P7iPsPf5+cZvt3e/P/bjx9vAAKIUVFaGJxc"
    "///5ySCs4nzTqni2mp6+GMPVewwMd578Z3j37CPDz2c3Gf49O8rw+/mpP1+en335/s2jc5+/"
    "/toBDKqNAAHEqCQjAo6ef39/MXBw8tYKuPQ3vRFwZGB6eJSB6cWh/z9fnv727c21u18/vj34"
    "9TfDTmAEHQYq/wRLiQABxMjHzQJm/AcGPRsbm7SQpM4GYLTKf3l949L3b993/frLsPsvA8Nl"
    "oJI/jGC3QjVC2QABBgCmxoa5RuwDVgAAAABJRU5ErkJggg==")
index.append('calendar_icon16x16')
catalog['calendar_icon16x16'] = calendar_icon16x16

#----------------------------------------------------------------------
wrench_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAASOSURBVHjaYvz//z8DLQBAALGA"
    "CE9PDwYmJmaG////8bCwsE8TFBRxExIS5v/58+evZ88e33737lU3CwvrSiYmRrA6QmDXrl0M"
    "AAEENpiRkZmBlZWD4efPH3Ly8uoxiYlpjCwsLAzfv//k+PjxnfGRI/uWHj261+nfv7/pxBgM"
    "AgABxAQigMHB8f37d72fP79fu3Pn8pojR/aDpX7+/M0gJCTCEB2dyOzq6p8GVFdGbFAABBDY"
    "YFZWtlkaGnrn2NnZV3z9+ils/fql83bv3srAy8vFAHQl0OK/DN7efgwKCuoVf/78EgLFCz4M"
    "AgABBDZYQEDEIzExg9nHJywcGCzzfv36nrx588r5Bw7s+s/Hx8PAzMzIICoqyKCnZyTw48dP"
    "72/fvgGDCTcGAYAAAhvMx8fP+/fvPwYHBzcGH5/wRAYG5oXPnj2cfvny+R+gCOPm5mDg5GQH"
    "Gi7GCATqf//+BvrkHxhDXPkPBYMAQACBI+/bt6+/v3//wfHt23cGJyd3BqAlcdeuXQjz8grg"
    "ABnKxgaKXGaGN2/eMHz+/OktMzMz2FCQY/78+csAshxoIUoYAwQQ2ODHj+/ffv/+jZGEhCRQ"
    "wx8GX98AhoCAAA4eHk6goUwMXFzsDL9+/QYGhR6DtLR8ye3bN1YADX/++/cfYAT/BBrMBMbI"
    "ACCAwLzXr5917d277R8HBycwwrgZODiYgMHDBaRZwMHw48dvhnfvPjJYW5syNDW1SSkoqJ4D"
    "GqgKSddMGBgEAAIITAJtX7lv37Z5y5YtYPj69TswPLmBruQC09+//2F4//4TOIUwM/9nMDXV"
    "Z2ht7ZRQUFDZ/+PHd0VcyQ0ggBhBgW9iYgQ04AfQu7+q1NR0io2MzAVkZOSY3r9/y6ClpcPg"
    "7GzHwM7OyMDCgsgcp06dZygtLXp57dolYy4unqfIQXH//n0GgACCGwwKQ1B4AQ3nA0aEH1Bc"
    "FZgTv8nLqxS0tvZIODlZYLjqzJlLDLm5Wc/v3r3lwM3NfQsUgSDzQAYDBBCYYWxsyKCrq8Og"
    "rq7GoKqqwqChocEgIyPFIC8vxyAiIixpbGz2dO/eY/+RATA1gOmTJy/8t7CweQEMOlkBAQEG"
    "YWFhsJkAAcSEM0sCvQZyAVAD0EU37AoLs5/t23cCLAdKZu/efWZ49eoDOMxnzJgprqWle/rz"
    "549qoGQIAgABxEQoz4Ns5+Livvvo0V0roOGvDhw4CUz3f4H4BzgNv3nzkUFfX4th1qy5QMMN"
    "9gHLG2WQPoAAIspgkMu5uXke3rx52SwzM/npsWOngJbxgF0OLAGBYX0DGHzqDEuWLJHW1TUE"
    "lWAMAAHERErhzcnJ9fDGjatOhYXpLy9fvgo2HOR1kKvfvP3IICEhAUyW/PwgtQABRJLBINdz"
    "cnLeun79qmlWVsLL8+fPA33CC4x4NYafP34wREVFfd27d2coSC1AADGRYigoC4MAMEIf37x5"
    "3S4jI/H59evXgTnzF0NqasI3oKFBrKysu0BqAAKIhdw6DWjALWD6NU1Pjz8OrAx4jh07BHLp"
    "XlhhBBBAjLSqTAECDACLIiFQsfjR6AAAAABJRU5ErkJggg==")
index.append('wrench_icon22x22')
catalog['wrench_icon22x22'] = wrench_icon22x22

#----------------------------------------------------------------------
linux16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAAoLQ9TAAABd1BMVEUAAAABAQFhRQDepwCO"
    "chI3LQifeQACAQAAAAAAAAAREREAAAABAQEAAAAAAACkpaYAAAACBQwAAAAAAAIFBQUAAAD/"
    "///Ly8sAAAAAAAD+/v4YGBgAAAAAAACXl5cAAAAAAAAAAAAAAABzWgAAAADbpADjrACshwAI"
    "AAAAAADxvgn+0DPwxCtALAD8ygb/0Tf/0RD/0xn/0zolHwOpgACleQAYCAAAAAAAAAApGABi"
    "RgAAAAAAAABCIgBEJwAAAABmPAAhDgAAAAAGBgZ9foAVFx2Fd1HFpTRrZVAFBgj/xAD/zQj/"
    "zAAAAQvBtqDHp0r8/f9EREQFBQVwcHD////Pz8/9/f3x8fHX19dSUlL+/v6qqqvv7+/BwcO0"
    "tLfq6uq/uJwAAAICAwn/2zaeorDv8PrfpgDcrg7/0jL/3y4rHQDi4+bV1d70uwD/0gv/1Tf/"
    "1DL/0zbQztLi4uKbm5wWDAH/2jXSqiTMmAH5xhD4vwLzugXttw3ccsBSAAAAQnRSTlMAAAAA"
    "AAAAAIbcxChJBnD+YmduT54Z/v633f7+UR/+57L+JPsvG5f+/iu5/tIGt/7+/v5KU9f43tPs"
    "vhUBVYcNf3COC2g3AAAAAWJLR0QWfNGoGQAAAAlwSFlzAAALEwAACxMBAJqcGAAAAMZJREFU"
    "GNNjYAADDk4ubgYkwOPk5OzEiyTA5+LK7ySAJCDo5u7hKYQkIOzl7eMrgqzFzz8gMAjBFw0W"
    "CwkRd5KAchkZJEOlwsIjpJ1koCKykWIhUSEhcs7yYC4Tg0J0SAxQJFbRSQnIZ2ZgUI4LiQcK"
    "JCQmqYBUqKolq6cA+alpGumaDCwMDFoZ2plZ2SEhObl5+Tq6QBV6BYX6BkXFJaWGRmXlxkAB"
    "E9OKyiozcwvL6horawZWBjYGBhtbO3ugjIMjAwM7AwC0diXgkD5FqAAAAB10RVh0Q29tbWVu"
    "dABDcmVhdGVkIHdpdGggVGhlIEdJTVDvZCVuAAAAJXRFWHRkYXRlOmNyZWF0ZQAyMDEzLTA2"
    "LTAyVDE2OjAxOjIyKzAyOjAw9OZXtQAAACV0RVh0ZGF0ZTptb2RpZnkAMjAxMy0wMy0wNFQx"
    "OToyMDowMCswMTowML9xtScAAAAASUVORK5CYII=")
index.append('linux16x16')
catalog['linux16x16'] = linux16x16

#----------------------------------------------------------------------
envelope_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAATHSURBVHjaYvz//z8DLQBAADHS"
    "ymCAAGIBEYGB8xm+//jHwMjwjwFk0b9/jCA7Gf7+BeH/YPb//0xA9j8w/fXrbw1hYZ5oQX5R"
    "78NHuya+erVnIbrBAAHEQqwLQIb++vXHmJOLO9rcUjkoLFhV/vMXUYb3X+7U7du9/zBQxT1k"
    "9QABRNDgP3/+Ag1lMOXm5kkxNpH3t7RWFbe3E2N48/I/w39uJobgMB+lU8cWRH35eqoFWR9A"
    "AOE0+N+/f0y/fv635uPjjdPQUoiws1fj0dERYRARYWBg+vef4eDNbwwKOtwMZhayDEamEbGH"
    "DpydD3T1U5h+gABCMRgUvmAX/mH04ufnj7ewVHW3sFLk19QUZuDjZWBgZWZg4OVmYDhx9DuD"
    "gCgLAxcrIwOfIAODf0CA6qkTS6J+/DjXDTMLIIDgBv/8+Qeo5b+HsJBQmoGRup2ZhRKHijIf"
    "xEBWBgYWoKGcHAwMb179YXj28jeDvgkf0FtA1/9hYHB1UWRcYxiZcPz4uWVAo8CuBgggsMGM"
    "jAw+WpoSeTp6qq66+goM8rJcDJycQANZIIaCaGagwUxMDAwXzn1lkJNlY+BmB6YYoKFAIQZZ"
    "GaCrgwK1zpxdFPL71+WJIDMBAggkzsDBzjlV30DK1cREhEFS7A8DO8tPBk62vww8QD9wcUIM"
    "Bxn66MFPhh8//zIoKHCADeRgZ2BgYwNaCuS4uCgyaGoGRwOFRUFmAgQQ2MXPn1/bxS1gkXLp"
    "zgeG759+MbAAky4XByuDmBgng6AAJwMfHzsDDw8bw7lznxi0dPgYODgYgXEBsQzoW4b/wCDR"
    "1mRi8PYNM71+dUXI7z83pgMEENjFV65un/nq2cePDk5qDBb2OgzmdhoM6royDFzcXAyfPv5j"
    "uHXrC8OxI6+BFrEySElxgOME5FJw2AOdxgg0hQPIDwnXBFocFQ8UFQMIILCLP3++e27t6uUb"
    "bey043iFWRgY/7AwCPOwMLBLczOwsUJc9e/fH6BhzJAYZ4GIgcOSGRIkICAITCFq6jYmly7y"
    "2gAEENjFzMws/27e2D99/65Ln/m5gHyQGCgn/wcZCDGEnZ0F6HVGsPdBGBSZXFwQQ588YmCY"
    "MeMZQ2rC7C8H97cfAjr1PUAAgdOukJAyAze3JJOzU/uqa9d+/3/56v//58///3/z5v//9+//"
    "///06f//L1/+///+/f//X7/+w8HTJ///z5/3+L+765zPfLwuQAM5CoHOUQeFFEAAgQ3m45MD"
    "Ro4Eg7Cwqd3kSdd/ff4MNPQtqqE/fyIMfPDg//+pU+/+d3Sc8J6Pz2Y70JxMqIFMsHwBEEAQ"
    "g/nlgIaDIkua1dGhY/2dW//+f/32///HjxBXwsDNG3//93Rd+29p0fiCk9N4LVB/HBDLYSsS"
    "AAIIbDC/gAIY8/BKMIiJ23tOn3Lzx38kcOXy3//tbef+W5jXPOPk0FkF1BcJxJL4Ci+AAIIH"
    "BQxzcIgyeLj173x4////a1d//6+tOfzf0KDgISur0lxQ0Q3E4sQUswABBK5BuLlFkcrd3wyi"
    "onrujvaZ867fPPHz4sWNO37/frgFKHUSiN8SW34DBBDYYDY2bmQ/AH3xj/Hffzbrf38/A5M9"
    "wzkg/kBq1QQQQDSr8wACDACbz95UGztbIgAAAABJRU5ErkJggg==")
index.append('envelope_icon22x22')
catalog['envelope_icon22x22'] = envelope_icon22x22

#----------------------------------------------------------------------
arrow_up_with_status_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAIcSURBVDhPpZJdSJNRGMff6qZb"
    "IegiCoPAuxisG7soRcKgtHI3SVih6wPBKIQJirQQrG7CyjCbNMmiRMitD0aW4crmN5irbbic"
    "fahLt6ULc3td2693H063mRf1HP6Hczj/8zvPOedZhxTC/0QUcDVJV6R5LbvkzezOvBcZC8Ll"
    "FF9kb7iTVlJ0rqyDX94FRL9IZWXXqp6/Ak6fMkAwSOsoaEYiR1ChWgUSz0CdBTFl5TwkKAZo"
    "tUHugxB77ixww/hT4i2iULTHfRF/MiBt001GLS6GpiHvUYhDLX5yG38gr52grc/FzJSHrem3"
    "lyFLgKUn0Nx9j9MPBW2wr1nkgGaOnLpJZBc/seP8MP12NzqdJf5k8QzCgIMKHaEQ3P8AJe2L"
    "FLbMkVXnJLPGQXaNlb3Vw5TdGkFc8FJU/CwCSQD0DDrxib+Z9wUg4KP+lRvZJQcZ5WYuNNnw"
    "ut18mXAxO+vBbJ1MBRQon5N/4gn5R9uxfJxC2+0hQ2Vh29kBKrRW7LZJjhzXs7/wMYeL9YkA"
    "tVrNSnV2jaF9O0N66SCbi4yopL809TkSPGF//ArJgDfvHNQbvpJ2zMjGPAOl1/rpHVgDIEgF"
    "ubJ1mxw0GT6z/WQnWxRPKb/eS8/gWJIrVsTRNBLba9M4om8eq92J2fKN785pTEPjawB2NiDE"
    "tF7WQLZSh7L6JcVVLyIqqeog94yeDfJGBGl9ScuVGP3Sf4o/P1CtNhfMEbgAAAAASUVORK5C"
    "YII=")
index.append('arrow_up_with_status_icon16x16')
catalog['arrow_up_with_status_icon16x16'] = arrow_up_with_status_icon16x16

#----------------------------------------------------------------------
symbol_minus_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAIDSURBVHjaYvz//z8DJQAggJgY"
    "KAQAAUSxAQABxMiQyGAhwCqwVElYRYrhP8Pf//+AXvrLyMAA8tk/ZiANZP8DYSYwZmRgYmD6"
    "z8T88Pmzb68/fUkFCCAGvnS+M3MPzfv/49fP/79+//n/4+ev/99/QPH330D8C4K/gfDv/z++"
    "/fn/88ff/9tP7f8vFaT/FCCAWLSkdbQTrOMZLr28xPDw/UMGdhY2BgYmoI0gBPIgIyMQATFI"
    "AMj/C4TCnEIMHqYODGZaphIAAcTCycLxj4mJiaF7XzfDuvPrGFi4WRgYgGYwAF3PyMLIwMzM"
    "zMDEDHQ2CwR/+/+NwUbGmmGr9zYGTjbOvwABqCJjFABgGARKKPn/cwMh2pqto+DgnYea/bHV"
    "KBVSiZhXjmUFxU+as1dYEjm4AogJKTghzoY5GcnpjP+hNBKEAYAALJJBEgAwDAGD/v/Fqamm"
    "Pbqwg0VwFNI00v4F2kxG3JUVWJ/Gj6jdyfc8QFQdAcTy8+9vJpAfCu0LGHx1fRk4WNnBAQdU"
    "DdYIcwkkUBkZ/vz/wyDKKcYAitCff34wAQQQg1Cm0JU1Z9f/xwn+IeG/UAwEx6+d/S8favwG"
    "IIBACclVlEt8saakJv9/UEIChQ800YATENDZkMSE4LMwsTLdffTg9+NXL4oAAoiR0swEEEAU"
    "5wWAAKLYAIAAAwBc7vIKtZwbEwAAAABJRU5ErkJggg==")
index.append('symbol_minus_icon16x16')
catalog['symbol_minus_icon16x16'] = symbol_minus_icon16x16

#----------------------------------------------------------------------
pencil_icon22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAPpSURBVHjaYvz//z8DLQBAADEx"
    "0AgABBALiGBkZCSskhnoBkNFBgZWTgaGv0Btf5gZGP4xAgmG+Yw8f4L+f2SdwcD6r4yBkeHf"
    "/zNnGAACiDgXcwINMpUD0uwIsT+MfExM/1dYxz2PlS54zs1i/aaY4S/jPIb/DCDLGAACiIWg"
    "oUIcDAwGkkAGKwPDL2h8/GKUZ+f7tcoz44mZhtkXBpbXPAwffN4w/Pj/P/7PMdF7QBVNAAGE"
    "38Wy3AwM5hLAYAAG1T+ooT+ZTPilvx0IqbxrpmL6keHj5z8MvqIGDNEKfgxCLt8ZGAT+VICU"
    "AQQQbherAA3VFmIAOoOBAZZwfjL6iWt+WuSX/pSfT/Q3w9cvjAxafF4MThJZDBq/jzCc3CbA"
    "8OrZpwMgpQABhN1gIx4GBkU+oKH/IAnnP9j7WdKm7yd5x79k5uH5x/DjOxODqVAkg6VYDMPZ"
    "jxsYuqfvYbg48+dMhldPC0BGAAQQC0bis+NlYBAHhut3kKHMEEP/MrTL2r6tsA14y8ABShS/"
    "2BjsRNMY9IQ9GfY+m8vQ23eK4f6SH3UMn141w4wCCCBUg72ALuUFRtJPqN//MXIyMP6bLe30"
    "LlrX4TMDB1A1238eBnvxAgZlQWOGNbf6GCZ2XP/+ZsO3LIbvHxYgGwUQQAiDuYCu42dCxPw/"
    "BnEG1j/LRWw+OsoZfAcayMAgyibK4C5ZziDCK8Ew+0wrw6z2xy+/7v4Sw/D76x700AQIIIjB"
    "wkDKgQnkZQj4y6jNwP17NZf5J01hld/AdM/AoMSrwBAgU8XAwvGXoWdvM8PSjrdX/x77HMHw"
    "7+cVbNEEEEAQgx2AQcD1lYHhN9ilTgyCP5czGn4V4xb/CzZUX0iPIUqxguEb8wOGlvVTGbb2"
    "fN3LcOFzDNAFL3AlKoAAghgs8AWYAoBp9f9/OQa+rxsZTH/zsHD+Z2AHet9Bwp4hUS2f4cXf"
    "YwzNS+YxHJ/8exHDrS8ZQMXf8WUBgACCZBCmf2CKkel/gbOxBg8D638GTqA9QYoBDAkaxQxX"
    "P29mKJ42neF4549Whluf4wkZCgIAAcQALjZTgOZHMUmaV2p9fHlj5X+7tXr/4w4F/j/9ccv/"
    "9it+/yUL2X4yiLCnEVuygcwECCAkgxmaZ60t/f//6+3/Xz9f/99/qeV/6Dar/9yJzG8Y+Fi8"
    "SCkyQWYCBBAkjJn+SzJ850mfv/ERg7LqOYajdy8wrN279eXF45cOMFxjamP4/vcSqeUxQABB"
    "XJzDMJkhQuA/gwv3bQYThjkMwgwhQDlJcgt5kJkAAQQhTBnKGaQY/IEloyA1ag+QmQABxEir"
    "Og8ggGhW5wEEGACl001MOq0YDwAAAABJRU5ErkJggg==")
index.append('pencil_icon22x22')
catalog['pencil_icon22x22'] = pencil_icon22x22

#----------------------------------------------------------------------
saveas16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAODSURBVHjaYvz//z8DI2MbAxCI"
    "MjAwA6m//xnAgJEBAUBCTEDM+89WeqdJbtiFGY8e8Fw/d40nDyCAWEDS7IJCZbLakn5CbP+A"
    "av///wc09D9ID+N/iF6GfwyMzDwM3189/BMTwKIbWuYr/PXCJsV5U//wAQQQ2AA9f5Pq1AId"
    "PhexPwyc7OwMf4C6f4HNAroJaAg7CwvD+1cvGd5e3cNgbmcBFLzPwK3LwCCuyCQNEEBgA3iF"
    "uf/ysjMzSAkCXcPByvD7LwMY//rNwMDGwcDw/dNXhj8PpjOYWIswsPE+Z2D4uYlh1wr+V3u3"
    "MKUABBDYgD+///77+RNo659/DOxA/u8/QDEgZgQGybt33xkenpzGoKrBwMAp9AUYRKsZDh1T"
    "ZSjodFjz88XzfQABBDaAERReTIwMTMxM4CBjBYqysjIwfP36h+HJmdkMiupfGcRUgE75MI3h"
    "7Blxhinn2hjusb7/8/PHRQaAAGKChTdQOwMzEyTkWUGRAXTNg3PLGKTkXzJIKbAxMDyaxfDs"
    "GAvDuSd1DB/ErRh42T8BVX1lAAggJpgLGIHEvz9/Gf7++cXw7cdfhuvHVzHw8V1nkJPmZWC4"
    "MJnh2f5XDC/+NDHIm3gysDL+AkbMP7C1AAEEiUaW/wzPvjAxzDnDyvDhNxPDn9dPGbz4HzPo"
    "yQND8Wgvw92LbxhmfJvE8M8smuHvPaBeYACxMENSCEAAgV3ABuS8+s7AcPb2P4Z/b34w8H59"
    "xaCm68Dw//4Nhpcn3jBcZa9huMmRw7B+HxPDmRcsoFQBjF6IAQABBHYByOtffzIwKAr8YSiw"
    "+shw8uBphkd3ORgevrVm+KHhwuDin8jwaudHhls72RhYGLgY/gJdzwQJbwaAAIKHAUjwHzAg"
    "r99+yjB1zimGM+e+MvDohDEouGcy/AYmiP//vzGIi7Mw/PoLUQtzAUAAQVwA5IHi/vv3bwyv"
    "3zMwvOfJZRDVFmFg52ZmeP/yOcNnht/AGGJjEBRgYngEdCnI/0xM4MhnAAggsAuYmP6zM3Ey"
    "MHwDpl8REQ6GukIeBknBbwzv3nxm+PnjB8PPXz8Z+PkYGaRkWRh+g6xkYwK6+j9YL0AAgV1w"
    "dd/Ng7q8XNb/vnz6d+3fZwY+XlaGZz8hTgVZw8zwl+HJl28MP3/+YZD8ysT07cW736/uvjnF"
    "wMDBABBAjP///2egBAAEGAAVFDp8a+OtjwAAAABJRU5ErkJggg==")
index.append('saveas16x16')
catalog['saveas16x16'] = saveas16x16

#----------------------------------------------------------------------
print22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAW8SURBVHjaYvT3T2RAB3/+/BGx"
    "t7e4aWSky/f379//CBlGOIuJiYnxxo073/bsOWjOysp6A90MgABi+fXrN5D6jyL48+cvTjU1"
    "JT5nZ1sWBjyAlZWNZ+vWXfyMjIxoMowMAAHEwsgIMpgVVZiR8d/v339QbQNb/heI/wExSA8X"
    "w69fv/4D1f5B9dF/hv//fzIABBDL//8fGP79YwXaLgw0kAkswcz8D8XbEMNg+C+cBgYHELMC"
    "1bOA9f7794/h27eXQLHfDAABBDSJCWwDA8M7YNh+YmBj4wZawsnAwsKC5FJMQ0E0ExMzAzs7"
    "FxDzMHz//h6o/w3D37/fwZYABBATzAugYPr69QUQPwXyf3wEugQaFH9xuPgf2LVAgz59/PiI"
    "4dOnp2AzIL5mYAAIICbkMPzz5x/D27cPWT5/fprExMzIigiGv0gYxgcFGSPTp0/Pkl6+vMcB"
    "TElAnyOiBSCAwP79/fu3ODAScqSlZVyEhcVkxMXFpfl4OBkRQfEfzdWQiOTiYGW0s7Mvf/v2"
    "ZcKbN6+efvr05RAwQiezsrLcBwgglh8/fmjo6uof8fUNFJaVlWMQFRVnuHz5AsOdp58YpF58"
    "A/riO9Al/4BehGAGMPsvAzC+GG4/+chgYW7JaG5hIfH+7XuJF8+fGW/auj75/PkzngABxCIg"
    "INCelJQmrKGhyfD5yycGVqCGH79/M/DJKzG8+8XI8P8vOwMzMMCYgQEIdjeQ+AXMM/9/MTEw"
    "S8ozfHl5k+HNy3cMn//8ZOCVlmQIC43le/Dgfg9AAAbpLgUgKIqi8OpGyLNSopMnTEDmPwOT"
    "8KL8PciVc50J7NauLxKRsWlqY3Lh7Hz4PD7O0aJijzLUBU4P2xPYDc9r0Z+tGzZuLRnSlcQp"
    "87nwHspUdoi0/S+AWKSlpQR5eLiAyeU7MKz/Mvz8/o3h5T8ehlsf2Rg+vmZguPKGgeHZd5Bh"
    "yJmaEYy+/mZhCOPmZPDh+M9gJKkMluPgYmeQl1fgBgggFmA6ZAVlSWagf3/8+AUOw8dfWBi2"
    "PWRmuAE0+O9fRlBSZ2BAz7agcAHmuXNcbAwM5kwMvJzsQCFGYD5gY+Di4mYCCCCW168/MRw7"
    "do1BUJCTQVCAm4GDjYnh4gNehrt3gHoZkRIEMgCFOTMDAztQ7q+AAMMPhp8MrL//MLx9/5Xh"
    "/fsfDC9evGcACCBgWfH/3/fvP5k/fPjI8JiFmUFYWIAhy4ibIUb/LVA5O9C7TEDMCIwwaMED"
    "NJSb7T8DF+s/Bk7GXwx8TN8Znj/7xvD23XuGXz//MHBwcIJc8x8ggICl249vfHxc/P/+cQBL"
    "tR8M27atZ/j++R3QUX/AZQEoRTCBgwGSKkDke2BGAIU5qGz4x8DCwMrJy2Bn58ogIiIALgr+"
    "/PnxGyCAmH79+v6eg4OFQUCAD2jQX4bDh3cy3H10h4GVm5vhPzDt/QEm2F/AMuEX0JI/QPwb"
    "GAZ/QeUIGysDOy83w5MXjxgOHtwGNOwbMDj5GTg52YAp7NNngABi+fjx/RUWlv8KoqICQBd8"
    "YwBmFAY9PT0GQ0N9cEr59w9UDDKAMwmIDcH/wJiDg53hyZMnwDg6ziAtLQ4MRj6Gjx8/APH7"
    "ewABxPL+/bvpN25c8hIVFWOSkpJisLX1YNDQkGPg5uZg4OHhBaaKf3DDQJUJiA+sVaCG/2cA"
    "5lqgBcIMMjKSQP5fhps3r/wHZu/ZAAHEwszMvG3Llo3xFy+eD/33j03R3t5Rh4dHjfHLl+/g"
    "QgXZYFAhBbMA5ANgGQNMXqxA+htDf3//LRaWvzcfPXq4FRgT8wACiAVYdQFrgp9Lbtx4uISV"
    "VbAgN1e9HxT7nJwcYINhGGQ4yEBYMMAsBRXy6upKDAsWfDv86tXjFE5OUAXAyAAQQCx//nCB"
    "qiJgivgHzDHCgQ8fPv4Hwpj1GKLqQa9cgQYxSUgIOzx9+pyJjY0DVFIxAAQYAKcBgQoSDnct"
    "AAAAAElFTkSuQmCC")
index.append('print22x22')
catalog['print22x22'] = print22x22

#----------------------------------------------------------------------
folder_blue_arrow_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAALYSURBVHjaYmRgYGBicNv6lwEd"
    "7PLWBpK3gfgfEGPKQwFAADEANf/HBkDiQHknIOYCYhYoZgZiRmTtAAEEEmQ48RDT3P87vRgY"
    "3bfuJeQygAACG/DuK1QTyHig+f+AjC3XIIagA6ChV4GGOAKZR0EGAAQQ2IBXXxgYrr9kYFh4"
    "Am4/EP2HGgnXCjZd3ARoqMn//S/bGMGGAAQQ2ICXnyCaX7QyMJy+d5Jh6ok+BnY2Doa/fyFh"
    "x8LMzPDu7XuGMptaBkMNIwZmIJ/x6bf9DAu5LAACCGLAZ4gd++8CvXjpAoO3iw+DspAqw++/"
    "v8HiXOxcDFvObmKonLkA6F8eBq8MTZizXAECCOIFqAHvvjMwfP3+l+HtrzcMXF+5GH7/+wMM"
    "dmYGzr+cDB9+vWd4/fI5A/vvDwwfvsP9JQwQQGADXn+B8N5++c/w9Tcjw8e/74EK2YGB+Y/h"
    "w+93DLdf3mC4+vIqw69fSgz/f/0GBjo8bBgBAghswBuoC+68vsnw7MMdBpE/Pxj+/P7N8Ozn"
    "Y4brn68wfP72meEPMGp+sX0EGnKB4dN3K7gTAAKICez0bxDOiWe1DALGNxn+sv5meP7zCcOH"
    "P+8YONk4GTg5ORnYhVkZ+IIfMjDpzWV4/eIpPIYAAghswCeoAcIMlQzvbzMxiLKIM0iwSzOI"
    "skowCLGIMHCyczIw8zAx/HrJyvBulxLDh/dvYQ74ARBAzAx3l63+zmOW/fW/DMONRwIMDy7x"
    "M3z8sJFBX1uJgY2NjeE3EH7//ZXh4ZavDHcXqzB8eWvD8IRVi4FhhfQqoAEHAAIIlK5ZgdgQ"
    "iJ2BWAjkKmZuJmndHBW3wEx/wbf/3jDsnr/vx63+x4/+fvl3BygPjGyGD0B8AYj3AgQQIyKZ"
    "MbBDDQOnHRZBZm2tPMVZQLbA1d57B4GatwDZe4AYFIl/gPgXiAYIIEbc2RSc86SBGJR5QJ6+"
    "DNWMAgACDABdJhjEdDtVBQAAAABJRU5ErkJggg==")
index.append('folder_blue_arrow_icon16x16')
catalog['folder_blue_arrow_icon16x16'] = folder_blue_arrow_icon16x16

#----------------------------------------------------------------------
wrench_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANmSURBVHjaYvTw8GRgYPjLwMbG"
    "VaakpFEoJCQo8OHDh683b15Z8/HjuxYWFuYnjIyMDLgAQACx/P//n+HXr78MnJxsau7uvhJq"
    "aloM796953jz5ln6ypULbO7cuebJysr6GJcBAAHErK2tU8LNzZP0+vWziXfv3pARF5dV5+fn"
    "Z5CUFGdQUdEQu3DhjMyvX99Xg1wBcgg6BgggZk9P/02RkUnWt25dl7p161rugwd3VNTVNdRF"
    "RUWAhogyPH78VPzatUubgMrf/v37jwGG//37x/Dnzz8GgAAAQQC+/wIMDA4LIiInAF1daE8b"
    "Gx6X2dnTveXl4gAPDwwACgoIABEREQf6+voa+/v7APr6+QD5+fgA+Pj4APX19gD29vYAAojl"
    "3bvX39++fcshLS3LkJpa6PXs2SMva2tbBhaW/wysrMwMv3//YXr//r0wMKiANv4GGvAT6IL/"
    "DB8/fgSqYWIACCBmTk52ZR0dfWMZGVmgk8UYDA0NgLZzAWOFmeHTp28M6upqbG/fvjO/evXK"
    "aaCfH4MCHegdoMG/gAawMAAEEDMXF/uFhw8fuqqqaolJS0sCnc0OtJkVaMMXBpBiWVkxBgsL"
    "a4FHj576Xb9+5dC/f3+fggwAuYSJiYkBIICYpaWlPj1//nTbsWMHZe7duy905crVv0eOHGVS"
    "VVVnVlKSBEcV0JUMlpbWXMAADbxy5fJeoEueA8MFbABAADHo6+syaGlpMcjLyzEAXaAqKirs"
    "yMfHHRMdnfj0xYt3/5HBu3cf/yckpL3k5OS0ZGJiZODgYGMACCBmCQlxYJT8BwbMHwZmZpZ3"
    "LCysD4BqL126dP7I7dv3/Wxt7Xl4ebkYvn//CfQzG4OLixP3s2cvvc+dO3uCmZnxMUAAwQ0A"
    "hTAIgPz99+9fEOsp0M8H7917FGRlZcfFwMACDJfPDAICvAzu7m48Dx488b927fIhgADCasCf"
    "P3/AYkAvPjt//uyBO3fuB9nbu3Lx8fEwAG1n+Pz5J0NcXDTXzZs3gwECiIkBD2BkZAJ6i/nU"
    "pk1r/MvK8l98+vQVGEvcQBkmhnv3HjA8f/7iGkAA4TUABECBBQRH161bEZSXl/GKlZUNGB5f"
    "GCIigg8fOrQ/FCCAWBiIB8c3b17nlZDwa+eHD+8vXb58MRqYwZ4DBBgAlD55km0lw0cAAAAA"
    "SUVORK5CYII=")
index.append('wrench_icon16x16')
catalog['wrench_icon16x16'] = wrench_icon16x16

#----------------------------------------------------------------------
save22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAQwSURBVHjaYvz//z8DLQBAALHA"
    "GMzsE7rEDNSyhAXZWDgY//z/B7bvPwPMXiALRDDAeVAuM8M/5q9/WRjefWf58+HSzdV/vubE"
    "g1QABBAjzMUKDjt/JHS4s5vwfGQwV2Rn+MsAxH//M/wF2vDnH9gsBmam/wxMjAxA/J+BEUgD"
    "EZD9m2HrmZcM298oMpxYcOzv461WYMcCBBDcxQLi/Kx8nAwMXP9+M4hycjD8B5rw+zcjw5+/"
    "DGCD/wIxE0gDM9ACEAZymJggNPuvrwwvHlxnYOVjY4SZBxCABzJGAQCEYWAU8QOO7v7/byJq"
    "K9WI4BCaLEcT/8uZ7TkJ2Q7GT3UBQqgSqPcyNwHqoPpTo5dhbBeQckEM8W98BBDcxSDb/4G9"
    "y8TAxMwIDDuI62BeBjsFSPz9C3E9SBykHhRULEAvsLMxM7CywB3MABBAcINBQiCFTEwQk4Bm"
    "QyLuP1SSCWrBf0TQgAz/CyQYGSFBxoiUKgACCMXF4HAEMaBKQC5mQLaZEZowgDQ47IFMNqAL"
    "mIAB/fc/xAwYAAgghMFQr4GCAewUmAJYcIAshbqS+R/E8H//QHqYwI759w/VYIAAQjEY5GIO"
    "FpDsX6jzIEmRGehOJiD+Bw6R/1DZ/xBX/v3JwA60CewopLAACCAUg0FG3v/AxvD7zh+Gn39/"
    "AxUDDfsHwv/AGJTmQWkbTEP5TP//MDz6KgB2AzOSwQABhMh5TBBvHn/GyvDpDivD56//wN4D"
    "+QLkMjAG8xnBNMgVoPAF0SAfMLFD0jcMAAQQisEgzd+AMcL4+jODi+Q3BjZOZmjyAmdoSDYG"
    "YlZgcr16l4FhxV4OBhl9TgYucRYGFqBlLEhhDBBAqJEHlPwOTPRqIr8YUjyAkcIGySz/GVjh"
    "6RjsZaDr9hz9zrD54F8GVkZIuQFOUUhBARBAqMkNlEaB+Acwu33+9puhof8/w7ev/xmaC1mB"
    "PmKFhi8DAw8w63/7+Y9BXJyJgY2dieEH0JesLKguBgggjFQBit3fQIN//vjOcO4GF8P3z/8Y"
    "fgC9wQxMa7//QIIEFKj/gJErLAQ0lJWB4dNfSMZhR3IxQAChhPE/qJf+ALPWt6+/gb74y8DM"
    "BmQDXc8EzV1gg4Hy//7+YRAVZGJ4AQyu38Byg+UvajoGCCCEwUB//IEWNt9+/GZ48eobQ4j9"
    "N4Zfv/8zvH0DDGOmXwywIvYr0PU/fzIxCAFd/OYHI8N3YOHE/BtiBgwABBDc4O9fvv/n5AZm"
    "EH5gMvrFzMACLFDcrNiBlgHD+QfEj4zQHMkCpHl4GRkUBDkYXr0CqgeWitwCQN/e/gE3GCCA"
    "4Aaf23FtHQ8/VxCbADfTqw+vGdbdfw9Ml6wMsJoEGTAB8/enn6wM37l/M3wFGirzi4mB8c7P"
    "/+c3XTjEMNkYrAYggBhpVecBBBATA40AQIABALGctR+2AHwHAAAAAElFTkSuQmCC")
index.append('save22x22')
catalog['save22x22'] = save22x22

#----------------------------------------------------------------------
box_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANpSURBVHjaYvz//z/DX2fp8j8m"
    "qhEsGmIa/lXbJwRx/L/r76lQfOzSu+3/GRgWrTv1/IK0viSDFhdQLSsTw12gno6Drxl+/vnN"
    "ABBALAxA8NBMrUSpPUyE4fdbhp5rt4q5xJj/Cccbs7vc+aHx5PSLdG9JgfNvXn1Z+/3r12VM"
    "XKwvGVgYGWAAIIAYQS6I0hSqndLt2vjq+W3GV7fvM8irCzDwKSkyCCjJMzAKCDMw/GRl+HD3"
    "y/93V55/fH7jwand5x9MbD/0cRvIBQABBDZAWUBZ1dLE+4IExzMuLpaLDOrcjxjYmP8xCMlJ"
    "MsjpqDJI6ygycMmLMTBwcjAwfOJjWFh/+2fm1OVB3/692wYQQGAv8Cl7M7xgUPsvKp/A8PLN"
    "I4bPXy4xCP47w8Dy8jLDqZNnGQSErjIoGGgwcKpEMty/q8+w4tAH5r8MDGpArdsAAghswO8f"
    "3xk4mX8yxicqMPz8rcRw6qghw5M7Pgw/39xlYP1+muHFixsMF467MYi+DGDYtrPj8/kbk3MZ"
    "GP6uBOkFCCCwAQz/GBj+/2diEBHhZJBR4GQwNuFjePFMguHyOSWGm2fNGL5+Y2AQUpRlYGL8"
    "xfBh87nHQM0LGRggAQkQQBADGIEcYHz9+w8JWRZgVMnIs4OxkQkHw+r5Xxg42JgYfgPdzczM"
    "zgxRBVEMEEBMYP1MTAxMQMzIgIgeGODgAsox/mH4/fsfWBMTwz8URQABxIRgMjL8+/cPwwCQ"
    "RYwMTAwMWAwHAYAAAnuBiZERrOj79x8MP37/Z/jzC8hjZAZ75dfvXxDXMUJ8ysD4H8UAgAAC"
    "G8DFycHKwcbFdPbQT4Z3b34wiEkzMLBx/gfr+PLuD5DiAmKgJUDNjGgOAQggsAFPnp949O3L"
    "4wsMGxnNFc7qMyopijPIqjAziCn8BIbPL6DrOBn+A33Hyg4JK2QAEEBgA37+/PzlzruDjjfu"
    "7oyRFDGIU5K2NVSXsedRV9ZhkJLmZWBn4WJg5QV6lQUcUqzIBgAEEDgpS0oaMHz+/ALo38/A"
    "0P7OwMLMriXAJxcnLWIQqqcYIG9kZMf8mfUFw9nz63/v3T9rw5cvr8PA4QvUCxBAYEJCQp+B"
    "m1ucgY2NB+xXTk5BBm4uUVAY8LGx8UbKSBpsFOCXWQfUEwrEAvAIAuoFCDAAizocdnpS40EA"
    "AAAASUVORK5CYII=")
index.append('box_icon16x16')
catalog['box_icon16x16'] = box_icon16x16

#----------------------------------------------------------------------
folder_grey_open_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABmJLR0QA/wD/AP+gvaeTAAAA"
    "CXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH1AICAg01dSsBcgAAAbtJREFUeF6lUj1rwlAU"
    "vYkZiiAU2sV28xdkUZxdHDu4iP6CInQX3Lo6ddBfYJc6dKjQRXBwKJgiWBAqEipaSNAhiPED"
    "jem9l74Qq4LQByf3fdxz7nm5T3JdF/4zpHQ6fSPL8vNfEdM0b+v1+hPuW1jEOVokk8ng+f6g"
    "/Wg0eo/EK0SAnB6CRInFYnGvQDgchmw2u7N/yJVCGcvlkhOpgiRJHHVdh0qlsiOAgmV0dd1q"
    "tcqYZ9LVWMC2bdhsNqAovIR2uw29Xm/PVSQSAUQhmUwWms3mHYo8MmM2m3nJaJPJ+XwegsHg"
    "0X+Xy+UeGo1G33MgMkejEU8HgwE4jgPb7daLmqaBqqp+0diOwGQyAcMwIJVKwWKx8MhCiK45"
    "n8+hWq0CXuGdlFiANmmMx2OOh8jkZLVa8RmNTqfTxdD1HEynU/4XiUSCBakTRKKqYk6xVquJ"
    "6l8o8MkCVJmsh0IhjyxIJCKwXq/91T9w8c0ClmUxOR6P85sQL46IfqHhcAilUumFKlPzCAq9"
    "LpyUqcf49o+2zXdg4FxDvCFcCT8XiNgvzk9Q6GPOK0JHyCQQQBDxEnF2ggBZN/BqNuX+AKkB"
    "WKfphYp9AAAAAElFTkSuQmCC")
index.append('folder_grey_open_icon16x16')
catalog['folder_grey_open_icon16x16'] = folder_grey_open_icon16x16

#----------------------------------------------------------------------
next22x22 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABYAAAAWCAYAAADEtGw7AAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAN9SURBVHjaYvz//z8DLQBAADEx"
    "0AgABBALugCj8axt/Jwsbt+//Prx6z/jZAZunkoGNg4GBhZOCAaxWYE0K1QMyv4/iw/FHIAA"
    "wnTxt1/uCREGzFNaXbmFeTgqGL7+PgoUFSDVxQABhGkwMMz/sLIyuHspMyyf7sdgrC9hxfDh"
    "1z2GfwzeDIzEGwwQQJgGMzIx/mViZrj6hoGBWViQYUa7I0N0qKYg049/Gxh+/W0j1nCAAMI0"
    "mJPz6+lzLxnuP/nFcOUtA8Old+wMRen6DC1Fhix8XOyVDN/+HAOqEiFkMEAAYTGYPfzsqWdv"
    "29sOM1y99oHh6isGhlXnGRnMLaUZZjcYMGipCFkyfPl3neHffzd8BgMEELbkto2Bh0PrydPP"
    "5xbOO8Vw+vRThntAl0/ax8DwiYmXYVqFOoO/k6gIw0+mbQy//7fhMhgggBjRMwij9VJg4LID"
    "kxAXAwMTWweQUyytKsMipqzI8O0vC4ODKgNDhBkDw+5Tnxgmb/jM8PkPywkGHk7f/zP53iCb"
    "AxBAmAbbLIMaDEqzXCC2BcN/5vUc/HwSAgpKDG++sTHoyjAwlLgzMPz8/puhc8M3hpsvmF7+"
    "n8srgWwOQABhZBAGEXWg6dCoZwKG1D/Gd8z/GX4wMjIxfPz6H8gFRuhTBoaJBxkYfHWYGXh5"
    "2RgYXjGyoxsDEECYBoMMhRn8l6GAhZGh3d1CiOMpgyTDhWcsDML8DAxSwDTx7uMvhqblvxl+"
    "/2W+xcDG5ItuDEAAseAIex6G3//WCfCxu9oDU8Ozn4IMlx8zMIgJMjCI8v9nePPqB8OL13+A"
    "gci6gIGVKQmbAQABhMXFDIYMf/5tkpTklVHXkWE4+4aL4QkwVUiKMTDwsv1lePLoB8PHL/+/"
    "MLCyZgF9thiYVcGa0AFAAGEa/J9hL4eQoCCPgizDmaesDF9/MDDIigPF//xkePDgJ8OvP8w3"
    "GFhZvIAi9/GlY4AAwjSYiVWAhUeQ4eF7VgZGoGOkRf4xfP/0neHtuz///jOxLGZgYUoBWUMo"
    "5wEEEKbBLKz/v/5iYuTmBXn9D8P7V98Yvn7/952BhSUH6OV5xBZCAAGEaTAzGwMHBxMDx/+f"
    "DG+efwXGIfM1oKE+QEPvk1JsAgQQZpZmYT/169uPf2/ffP0JNHQeAzOTLjDc75NaHgMEECOt"
    "6jyAAKJZnQcQYAAiHgTG/ByQRQAAAABJRU5ErkJggg==")
index.append('next22x22')
catalog['next22x22'] = next22x22

#----------------------------------------------------------------------
house_red_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAO5SURBVHjaYvz//z8DJQAggBjD"
    "gcR/CBYwkJK+qinMJ/nz55d/f/79Z2D69///P6DMn79//jP/Z2D8/efv/wmvPs69/OdPJswA"
    "gABiARH/gJiPjWOaNBOjlIU4H8O/32zMf/78ZeBgBkn+Zfj5h4GBCaiKk4mJ4dN/ptSy56/X"
    "/WZg2A3SCxBATMyMjAycrKxu6qxcobKafxi41P4wfPr0l+Hz738Mb3/8Y/j46z+Q/Z/hI1DH"
    "m1//GHwlhJgD+HhnA/VygQwACCBmLQYGNmUu3m1mEmwiVqlqDLxKPAw/rn9j+PfzNwMLE9Ab"
    "QC8wA21nYfwPdisXCyODAguLwOH3X0Q//P+/BSCAmD24uCa6MrC66/pJMwhayjEwcHIzsLGy"
    "MPy++oHhPxsDxACgVxiBmoHhwPD33z8GKTZWBo7/zLpnv30/DhBATEx//4U/+f2NgUFRnIGB"
    "mQeokoeBw1qRgVtFhOHni58MXz8BvfD+H8OXjwwM33+zMfxm5mb4xMHJ4CQjxWbCxTUTIAAA"
    "QQC+/wPW/fzb0QMBGFJQMiATHBYAAAAFAAENEAACEQoA9eTlAA0lBwAIGPoACR79ABYj9QAX"
    "H+sA3vrzAHLM7fDX9/fKAgBBAL7/BPAAALArAwIXCgcKP+wCBgH7/wEAFv8JAAc3MAAgOicA"
    "1M3RABAA+QAfEwEAwdUCAOH6DAAk9bUA0PPnpfMAAG4CiMmal4PBiYeD8w+rAMPUb5cZrK4s"
    "Zph28QYDA480wz8Jb4aGF6kMq7nSGBiktBgOC/gybJVNY2BiBoYBKHiAoQoQQExO/JwMX4CJ"
    "5g8LNwMX/1+Gr4rPGA7+eAqUFmD4//cvg7UKN4OctCAw0qQYtHm+McSJPmFgYuNh+MsAQgwM"
    "AAHE9AdoECsTEyMrnyDDH1ZOBpF3DAyy//gZGLh5GP4zczAwvrvKIMv+hYFBUo/BTfkvg5fE"
    "bQZGMQVw4gMBgABi+fXjF8Pfn3/+fH/zgYFZkYHhNTB5/PzDCQxLLgY2TnGG3GAbBjYePnCi"
    "5dbjZPjz/CLD21d/GGA5CCCAWHa//cwAzE+rLhRNdFe0ZnFRMGBgFzLgBbpADhj3PEDNXAyf"
    "79xkuHvuKMORXXv/nzx55tvJmw/vAANwO8gAgABiNGYCJRIGhnNANwEDxkqamaHczUbNNaug"
    "gPP7NyaGA9t2/Nt76vzHi7cfXnnGwLCDAaLxIjQLMQAEENyAm0DuT2jOBOYwPUEOluzff/8J"
    "vv79bx9QaCcQ38eWnQECDACg6DSOPOv+CAAAAABJRU5ErkJggg==")
index.append('house_red_icon16x16')
catalog['house_red_icon16x16'] = house_red_icon16x16

#----------------------------------------------------------------------
led_blue_information_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAANkSURBVHjaYvz//z8DJQAggFhA"
    "BKPRDAaGP38ZGP7+Y2D4+VeDkYM1X1RcyF9QkFfk728GhvdvPr57//TVtn8/fvQzMDBdZvgN"
    "VPuPkeH/+0oGgABiQTHu1990JR2FPl9ffS5VFRmGT19+MXz59o/h5x9W8Zu3XiSe3nsm9uX1"
    "m9VAK/uAqv+AtAAEEMKAn3/TzRwNZuRmWDHwcbIxPH3ymcFC8jvDr9//GHZdZWCQkxNm4Azx"
    "Yjm1Q6Dz4bFjQA2sXSBtAAHEBNb847eWipb8hNx0awZuVjaG+09+M/z8/pXBTJOfwVpHkOH/"
    "n+8MT568Y/jx8TWDgpkFg4CSehvDn59GIK0AAQQJA0bmHG9vAw5hPlaGh0DNf4B+fPCKmaFv"
    "/SuGbz/+Mtx+BlQEDOyPn78zfAOGuYCmAfPHe3crgaKhAAEENoBfVCDwJaMkw50nfxn+//rL"
    "8Pz1FwYZgb8MP34yMfxn+MfAxvKX4cXbv0DDgIZ8ec/wm5mJgYNfwBWkFyCAwAYICvAJn3/0"
    "n+HMtQ8MoSYsDN9//GH4w/6boTBAmIEJ6Mm8ac8ZLr3/xfD/3x+Gp6/fMLBzszMwcXLygPQC"
    "BBDYgP/A6GNh/MNw8/E3hunv/jDoSjEzXGP4xRBg9pNBWZqF4fP3Pwwv33wGBtVnht8/vzKw"
    "cwC1/fkHDj6AAAIb8PXDp/dC/36KMbL8Y/gAjLrDZ38wiPIDPcsoxABKaO/ff2X4/OoNAwPI"
    "zv//wBb+/vL1G0gvQACBY+Hji7fbGD+9BEYfIzAx/QJq/M3w+/dvhn//gIpB+C8wyv8D8Z9f"
    "DMzMjAyMQEv+vP9wEKQXIIDABvz69rP/2YXLf+REWcBeYfj9g4GT8R8DMDkAAxBoMStI0Q9g"
    "6vvJwMEtwPDz7qN//379aAfpBQggSDpgYbn0/Nq9ik8XzzPIS/ExcPGyMYR5STL8+MvMcPv5"
    "b4YALyUGYUVRBjYeUYZ/914w/Hp8vw2YpI+DtAIEECQlMgKdzsQw8eGJ86xCr963iampMO67"
    "+Jlhw9G3wDTxm4GTlZnh/3egumv3GL4/edgBZLUD9YBzIUAAIZIyI9DtjIwd727c3fPp7qPS"
    "/1z87gxs7Dz/gAH2/xMwwL5/2g/0WjdQ5RGgYrg2gABipDQ7AwQYAC91W9M/JkHXAAAAAElF"
    "TkSuQmCC")
index.append('led_blue_information_icon16x16')
catalog['led_blue_information_icon16x16'] = led_blue_information_icon16x16

#----------------------------------------------------------------------
cd_icon16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0"
    "RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAMhSURBVHjaYvz//z8DJQAggFhA"
    "hJHhUgZmFiYGJiZGhp8//ipLSvPmGxqJ+8jI8cn9/v3v3/07H+5cvvRy9du33yezsbG8+ffv"
    "P8PZs5EMIMsBAogFZhJIkJmZKSo+Sbc3JFRVQlSMk4GRiYHhz28GZiDWfPf6c93ChdfDN226"
    "k87IyHgQpg8ggMCmGBosYTAzWxG6at3D/3+BAp++/f//5sP//6+B+Pnr///fvv///9fv/2Aw"
    "Y8b1L5KSERYwvQABxASxnUHGP1hjqoeXHMOr1wwMnz8xMPz8ycDw+zfYBQx//zAwfPzIwPAB"
    "iIHquC2s3WYzMDBzg/QCBBDYAHFJ7lzfAHXRz18gGv/8BSp+z8Bw9QIDw+1rDAzfvwHFgIa8"
    "fcfAcOcxA4N7YJiOsLBFGEgvQACBDdDTE/MXE+dg+AXU+I+ZgeEL0KDr5/8xsLMAGf++Mpw/"
    "xcDw5gMDw4uvQMMYGRjk1LkZ1LWdAkF6AQIIbIC0DK8sMzPIU0DMysDw9CkDg5jED4YpM2Zd"
    "2Lpz421+/t8MD54BXQFUAwpYVqAacSklZZBegAACx8Lfv///gk0DSv79DwocBoZfvxgZlJWk"
    "eUVFhdh//QLKsQM1/4dY8p8RpOcPOAEBBBDYgIcPPtwCBpjxN6ATvwKxjAwDw71rnAyuDsHK"
    "oID89JOFQUIJEjYgF/z8DnTlwxt3QHoBAgjshcuX3iy7duErw1+gTeJCQD9KMzCYWANjgY2F"
    "gVmYhUFGB6iRAWI7O9Al92++Zrh5dccGkBBAAIENePr07px1aw5fVwXaIsgPdN4/BoZvQFsE"
    "JIB8cYhmkLeYWSDRum7x/NNfvl5dDdILEEBgA/78+fRp3ry8xOlTzn5i52Bg4ADij5+hKRQU"
    "OkATODiBBgBVL5q8+c2B3W1ZQNGvIHmAAAKnJiWlOjCbmVnOPDl5wbmjxz78P3n+//+rj/7/"
    "vwzEJ6///7943cv/Hl7dJ4DuMkNOxQABxAgilJXrGe7da4KJcwkJWkcbGDr7KygqKv8BRtG9"
    "OzfuXLywe+PnLxdXwG2GGgAQQIyUZmeAAAMAocFQytX0VvAAAAAASUVORK5CYII=")
index.append('cd_icon16x16')
catalog['cd_icon16x16'] = cd_icon16x16

#----------------------------------------------------------------------
sort16x16 = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAZiS0dE"
    "AP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB94EChEPM1txJcUAAAAi"
    "dEVYdENvbW1lbnQAQ3JlYXRlZCB3aXRoIEdJTVAgb24gYSBNYWOHqHdDAAAASUlEQVQ4y+2Q"
    "uw3AUAgDjyj7MBYSAyF5WlIxQESU6lG5wZ+D5dmIiOg3j+5umcm1CG9gZcAnBvcISVZV/RtE"
    "SbaaMG0PxAMR4AF3dSBKU3dbTAAAAABJRU5ErkJggg==")
index.append('sort16x16')
catalog['sort16x16'] = sort16x16

#----------------------------------------------------------------------
splash = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAAToAAAC5CAMAAACPzwOsAAAAAXNSR0IArs4c6QAAAwBQTFRF"
    "BAIECIkNkohij91YjAYE/McsBEkZhouTKbs0RGYU1MWYBE2GxfCZhFIMWYdx0Yrp0ocVocms"
    "C2cWiaqRLI58jVzc1qcl/OWdFGOiEKAWsamO+smWc7nmV74+yxsEKGk9RYnTzkoJ5e34+qgW"
    "qulqyPpqNGWVruqoUQUE+6diTFyC6sh6z2gPXaLUx8zvtsa9KIc8MEo0mSkGrareB3kTJqMl"
    "+7gciKbpXNlW6fP7Bi0f6YsUydfyxbydz27muPRltsTvNs038Mr2BUm10en8CGjJibjm1CUF"
    "TnKM+2wEMHOj/IhQ19vyudj4qRAEB3jUmN98aYu9/Nd0dthHRLxB6NqpKXpDBVfBBloTtOqG"
    "qbntLqNB/PbZsYjm/PO//NcnFDpk+5oQGIhIVW5m9SsECJYLeaHbKpc+Fncs+70wzKfuaniI"
    "qYcvF2g09UkT+qgzoHXf2fXw+9yWTKdPGorV/Oq1KlhUqrexc+BztbzoOYbR+/z7luhVjJDe"
    "F60ct8zyq5BVkHkwRDJE/OYsPDoUr0gLvG8UcaOHdoaR8az2ZGqMLQQESFRj7ob1/AIEcW9P"
    "vOL8zq1fsyUHrmXg7nX08LSOeCIc+3YFeQIEzHgTdHbYX8pAH0mPc5evBCw6bGZUlbaiqPSo"
    "iJSiyJkex1kS5Lpky7XovJcps6BuBDikSHvHBRgLkOuQ7VcQ1NbEoMXxGFw/9HdJnjYN0jgL"
    "UJbY8Nzz+ZdSs1gP75b2dL50kRgFtprqIFitb5aP/OKBdHhms3TkGUVCLmZYK2m8LEpc3HJU"
    "sjoGjZzTcK54ufKDc5bYdGLcbcxGc67Y0Pe6qnwU9mgzhH5cMD5Yzpns/DIE8bv5SJRgTWeJ"
    "3LpwkEJI9B4ENpfXT2RrJFWXBTojSNNFKq4fV3pxLMQ0WK7klaiwp6mxLnnJWZ9vLa4+tdO9"
    "HHKsVHyWysa1znnot/i3XI70+uvh+8tkNoZLl6zr/IgPmrfttRgEOXZUtoYbubaglZmd25kc"
    "2FcZ/LlpDDk8/L6EFQjUvgAAAAFiS0dEAIgFHUgAAAAJcEhZcwAACxMAAAsTAQCanBgAAAAH"
    "dElNRQfYChkMGBDDrkUPAAAb90lEQVR42u2dDXxT9bnHU9tA78adrlxaKi/F0+KKSumFEjgY"
    "aTAJFUwbSIC0IZR1tqUVCzaSYnuw0et1Q8GPEdyoQ5ioQwpKq1BfgFUEnINBRXG6C45iqV3F"
    "6lL6ooDO+zz//0mapHlPihDPc/J6cpIcvv39nuf5n5cgOiJEkCESEIQX3b8EMP6hEwkRcAjo"
    "BHQCOgGdgE4IAZ2ATkAnoBPQCegEdAI6AZ2ATggBnYBOQDewUZZLIi4uV0AXELf63Lik9PNL"
    "zqeXxOXmDsoV0PkZ9fVlC+6b84umpvKmpq/mvJmee3qQgM6fSKpPOn/0qyag1vQVBNxELxh0"
    "eoeAzlccOFCfPmrLV7bYQqf2QdMc2X1oCwGdA7mKpIX3IywSL5HrS3AZlTttR+GHOLmEgM6u"
    "uQXteV/94iVKjFz5GJX774mEVSG9YIROL3LQAbnYeU1NLznH/eQyL3fzxMJCHtmyZS8ff+X4"
    "Ml6EP3p0WVknP6k4f+bMnHJERSanaMnd/Orx43feOe63348wJpjyTSbj968sCw3eVY0uyyEK"
    "1x34+kz1m+UuzDCmwuXMoH9fyCdhgkiACejdSeFd0ei0NMJJbXQWXvg4iZe1SYmTzzRtcSA2"
    "tS+aE0+//RPKjJ8gTCNeCUF3btFpvcQVgW50Fj+NPomXk6MLdy379g8HllQnzmlyBEagTW3G"
    "qI7a/InRDg0ekQemcSi8rB8POmQGkbULgB1/BxLYiBHzTYYLFSWJibFNW5rtuBxiRnVR7rRt"
    "BJyJRAIN0/fLfkzokNyudzDhm6wGGvmG/Pz5Xx8oSiy6v+l+Gy2c7FFdlAGyy8+3J7z5EMju"
    "O3Bs6o8EHQruHSMlZkcBk2EoyK4occuWqXZeLTjRKCqKOv326pHHxo8ff+rUqRUY6hXj5zcm"
    "JLxS+OEub99XBBEx6HrfcaBm19DOlQeS4F9ZveWlqS19EdsSC5fqRES3+adqNTIDdmfPnh05"
    "cqT6FOjue6+yQ3BFiRkRgW507+hdJoOJSq1n0+6qqu3n2trUaqVSOTypBAhNvv+lqTNiIarx"
    "gjfV1UVRUWDYQ63S8nKW64uR6mMgu+OFnrMdcmuJTSzaEXBzog3PPzqs6Eb3fssZEJw1v1in"
    "U+qUanWbuqqqavfZ8QcqMoBd0bz7pza3xBJm1dWJiYkALqooatC0uy5wzsGuUHc0JnwHA4s1"
    "HiVX3YI1Zlp40HnToPvXwomud3TvcQ5UZzUl5G9S6sYrGIaxoZhQdiADOUUlEmbwD4+yRcag"
    "zf++2OGCjrOoTxkbUXbrPJFrmbFqVXN14ukwoPPqYE+vuX+qDZJc1iyTwQpuTcjvUOqqbBRu"
    "b4Cb1vqyjCg3kZF7evO01StVrujAsuNBdoXush3qFySXmTkDasyOgUDnD1bnpyFlAUC3a9Z3"
    "IDvSm7XplFIewuNv4O3Q+voSAs92kxGVERc36PS0aW8PXtlh7YeOPaWe0Nh4/OSHE12/CCxe"
    "FDtDOxbIJRYNEg0IOq1vqk5PQ8ufgG7ZrDsNBhzCg2N1umM8hEefQtlxzybVl8Vl0IgDarm5"
    "gwadPn367U8/69hJKwTToFCpVFJpWppMpuKy1SuMjeM+LCxc0w9cYkvzqrGZzaC5XNEAodP6"
    "CzUc5Hp7R8+aNcvI5ZNR/E6lro1HZ33qUXK/82dlZfVxwCwu1w6u4rHBrZ9NqpSlVcrgRpoG"
    "6FQNDGu3bOPLLtmOSm7q2LFjm0FzuaKwoXP/qra/Ld2gC7Fmo183zOpz7GydMtsmu8epH5nW"
    "T5Pq63PL4srKSpKSvv506eqLiE0FqFiOc2fZlQnjCh1lRwoN1AcgN9WT5oJB5/FlN4W1P7pQ"
    "ux1Q3TJA94qBojOBY4s5jhjwxPoTMhmRVVrlZxAXIDo6OnbubFAxBs5zgGU1ILvCPQ7kiqqJ"
    "5LyRC7Wv846u/2Ih94m9vTkvz3JxLMulSdGBTz3epyrWCj2LlWGtnJXzGixadnfjuKwPedlR"
    "yTWvouQSo+JElw2d1stioXfYgG4XohtH0aFjdWa7Yxs4v4JlGkidAJFWVkrRss82LuNlh+Ta"
    "Z0zNRHJab+SCQ+e+HHgZr4VjMOdUJmZ1Ww0EHTp2PE+kgS8UbmmxDLo6DQtFZWVamhR02sCy"
    "tsZ4xfzXski2A3BLYkmW80UuCHT+l1hP7wllLJHTm9NLHDsCZQfooCs+Z+PzxuNWZ3FRWgTW"
    "VrMUcTGsu1IBltUsyyrcMwj6wPYWKjlKbkcY0WkD6U7cvinEcVhvTtYG3rFkQ+85na6OJ/C3"
    "p/5GKwa1IoiLJEGk9fl9S/7CeLbwCvWzr50s/M8MlJx27NjMTD7P7RCFD533zSmeXgif6ojs"
    "UHUbHBy7m+OIuLau319pJl2bq7h+ngGjivSd7qhZwcxo2Ztf+I+oqMmxzSi5zLGZqLmyHaIB"
    "QOdpXO8PupDZYXsCjkXVGU0dOp2StCfQt73xlIdC8VHUmdjJGVGfu2a/NGpnFiz7X+lRv4pt"
    "ic5EbpljtTN8kQsUnY9xvafNJ9owssvJoTX2NZtj2/oc61Io7Mpj0qNim1uWZKRP4KwNtvRX"
    "uTWNChSXUPwqI2Ny7MZVaFYI0FxJyQ7RAKDzvTXJLTpRONgBuhybY0F0xgRw7Bgbo8dthcLq"
    "OHKwsulR7dFaYFeySSZDYLZRmN3Q6RlR7S1aFBxetTMmZ/giFyI6P1o59y1xiOxuQHbfo2ON"
    "CQnoWIZXGRYKOzCGOBIEpq/8Z8bk5kxkt2BCX46z3zP/zMhYEtu8CrAdRs2tavGDXGjoXPo6"
    "P9CFgx2gIzX2NYMhgbCDrrjH7tg3IIURR+rBkkRhWDJ2lmTEaqHdACQTKGQrjjg4Mqz9eXpc"
    "+hmU3NjDh4nmWiaXlEwTDQw61x6476l3w4aFHTr2ZWC3wcQhOJtjacrfv36vjAfm1OP9M2Nh"
    "c0qmNhZ093Necbxlmb/ElSS2N0dnZh4eknk4My8zGshl+HE0YxibEy9di9vFwuJY3I+PjlUB"
    "Oqm0gb19+qOOVcJK056V+7zsVyA7wq5kJUl/VJ2VQ9PjStpjj67KHAKSQ7tq5/mluR8CXejs"
    "AN06RHenwUBUZzyo0820ZS6+UFiJJ1mOv0B7Ejd5Y0oTsisraaV+lqWp0n5TVrYkdh60JJRc"
    "nv/kQhxNiHyg8ydFBoEuhjp2lhEdC/Q26eRVtsT/6PTb+3jhLN6cz5aA7JqQ3cKMsj8x1M87"
    "z8eVVENLAl49DOjyUjKj/SYX2hhW5AOdyEdhDppdTgxu70THEnS36XQ6FW1zsVBwrIGl2FiO"
    "QVvKKidV6heULNyYCeyOxi4pIZ7lmKElcUva52lTDg9BzeVhnosFcqdFA4TOXdJ3J0lv+2FD"
    "ZGdz7CtQY428Y4+RiskZ6IjCShOZfpIeR7IqEBn3bFx6bPSQ/KYUbfuSsrKhMKwFyS2J3Rid"
    "ieAO5x3OS0khmvtfUVjQXZkRA5blN3iajBBYY7cTHTEN0rvXL9JD4D4Ie5k1gAQZlF1KUxOw"
    "I/luaFJZevu8oyA5mAAc2BXJlf13zJoIRmd37Djq2EZ0rJRTTQJkZtn+6Q2sfUwBHsbxPRp4"
    "KC+7ISlH54HuykrArNFIDsHlZRLNLSir/+WePZGMDmRHNp90Q41F1RnP6eTFKDq6V/F2hGWg"
    "zAg9FB2nWlDWTmQH7EB36QtjtatQcnlILiWPkvv6mZ+uPZkayaoDdi/zm0+MjUZj4yY57qJA"
    "ibHc7dNPkFECS9sTgo/U29+Upc/TDmmCadXR9oXto6JT8sCqecAO0hwld+Ca59Tq/+uNYHQo"
    "u10bNmwgjgV2CdSxZGhlYN+Y3gAtL9Ec3X/IQp1NM8ta60sWHk0h7MCzG1cBOcKN19zCpLKv"
    "3/+sYaRanZUawehyYmJyZgE6dCyqDmosOpZIjf3b9EdZ2tqxUGmx/YUkCFWD+VP9knnRQ0is"
    "inYAB6GdtxDc+tbFSpY7pf44J4LRIbtvQXYbwLEao8bYOFeOGzypypjHr2UYBY5oJ/V0YaVV"
    "kL3XLPd5PRZZyi7PFil5c/Kio0ctXFBff9Pvn9MznAIsmxrB6HjHbhjH5QM5I6mxMmJN6IAX"
    "DVtEuxPs52i6Y9C97M9QdrzabPDmoOaQXNJbgzugh2a5mWr17yIZHTh2Q/eGDd35UGOBnfGg"
    "XD6eYysndell5q3Tr2WpzhiW4alBAWG5CfULFh6d06c44AbTHKK5A29dTCMLc2jZNZGLDmV3"
    "Q7eDY6HGHmQ4hliTufbLBoKrj5qVPv+0Ph16uc/zfpH3ORp1TgrCgwphI4dvIZb9IJLRxcSs"
    "6wbdjTOgYzUJF3Q6eR06Fnk9Ov1uzhEcX2thNIayi4YENwcvqLg50TbNTWDIW1B3YNkdEY2O"
    "OtZqMGo0WGPl8t0cD6zhy2sJQivFh3MZqVmm7+o6UAHZjlJDbjy5igqorQzDs2bQssrIRYfs"
    "buju7gbHJjg4lnfpiem3szZoCqkMocEYTapiW6nsCDXMckBuyYKKpKWDJxBuWGR6uqRo2Z9G"
    "NLqJ6NjlXL4GwnibXCfvgc6EsIMRBdkMLNPruybpZTKpTVGKioolo7RILSUawRFyFW89N0EF"
    "hHnAmDDr1OrNkYwuZhbIrtuEjh3RqAHHjiFbzsGdafvXd0FAtVXx0GilZbjfHELZITae3PmK"
    "iruWPkcWhgawHJcth0V3+7TsVXw+LDq2gDjW1Ml3xTqosTJwXFfPA8NO8MzKGSABV7jBbCat"
    "OACyi6ZxlGhu7WPPVVJZkl5GgZv6dsrUanUko9uDqivgrOhYzQXiWAU4DkYPDdNhRAHMKDK4"
    "xSuh+Ke3z38Esot+M/pNLdXc4JUKIAyLKRRSKCa447br3eIVvtj5gS5Mhyj5ioC/hDi2gHcs"
    "6m44cSwpsgxzYnqaTWs8OHLLcmbIdvOORh+N1mpHfXS+4tBdgz9TITZptgypTeoqnjumavfZ"
    "kZDtlCGhC+eBNmFGB+xqHkF034NjoU5oNsnlOhVHfFnObp1+gtBiFDw10BWoyqzvuuNQ+n0b"
    "gd3RUfctIOSk5QBOP6lyUuuRuVVVY3aPrLMoSKEYHgK6cJ1eNzDoaqC16wZ2yQaDBtg1Xg+O"
    "nYkbPNGbDV9+qSqn4IBauUJlxhzYpTcrVh6CbHcUyH2UjuRacbHyNH3X9rYxY471ZGOpAOuy"
    "I331J97Qhe/MxAFSXU1MzS0FBd0FRs7YiZaFGrsdDKtgFADj7vVbWayXCtp1dPXos1UKmFPO"
    "3HFoAchuI+S5Q58MbpUCKEAlaz0IYoOwWBR4X35Krf6fYNFpr3R0yO7PgI53rMa4Ghwr5Uh2"
    "U7Cq6deqFGZ9T1/bgbNhYle+DbIjee6T1Z9JQZhYI8zvHixWWLKzsy0WJMdYoEr8LjU4dNor"
    "Hx2obk93AQQ6tlPTeEEul0/hOGI4qf7eYQ9M6ZFhvS2nzPipXPHxofML7wO33oSaK2cwAU7p"
    "eveBtjoFopOi9lgzpLo9OSGjc5xzhaGLuQXRoWNRd/vk8tkcp+rpARR1e6G1K6fYHLih7Fo/"
    "qDi/4BC49aLMLOuaAoCzgZbsyBhwK9EcLDNSqX4mprcmGHSezmu9ktAhO3BsQfd36FiQHThW"
    "LuVUUAtAaqov96soLyI1eqWyu+ODikOH1j722BEwczYui/PN+u0zy4EcqA56mt1q9dqc3nUh"
    "oXOZ7X4pPzTr6+P7H4PhB7qamhhUXTJ1rOZ6QIe7KKhBT6zfaqNFL3iDKlTIbvrg0Nqb5k6y"
    "oApxNonKKW0KhoADta5Qqp9Y1zs6NXB0/uQ2v88o1npNo/3QBZBYscYmE8c2dnZ2No4Axx5k"
    "OYqjPG36IoKlnGdXroAmJRt6FLDou4MHt6YxNmqUr1Q/dzyLT6FzzlYr1fe83tubEzw6P8H5"
    "f7qxx2Oy+x1v7A87kN1ERPedwdSJhWIpyM7McTyrL/enkewGMFBsFpl+ClCrA2Fhy2KjZled"
    "QtbVls2SzXbcTKX63DfJWTleZOcBne911/o+sCmwU7WDKujo2O7kZLtjocbWHuEMDIVy9/oT"
    "5ehRpEa0RhIbYxdaHzQyp1xReaQKGkHoBrmzSvUdz4tvyfEiOx/oAiHn7XRsP07VDq4XAnbg"
    "2GR0rBjZ/QMcy9gcK4VCoVCYe3qmTIFyYEE4noLiPfL7gzPJnm92hVL5RLx4ubdsFzI6dxL1"
    "40wLh4IQKrqJySC7UqixnWKN5otauVwGjsXMplAsgtZuypQ67D3KnSTm+BhSIGCbCXjrzPop"
    "VQrcm6tQqtUPxYtRdnvCjE7r9YcQPJ9u7K6yumUaiGNvBXbJVnRsJ4xj5bVzOZRdHWjtgWH3"
    "Ziu8BWLL1qMq68iCUv2/xuOe2zql8tw9klJxcq/nbDcg6Lw0bFqfJ6kE+OMKwO4FCchOwxmp"
    "Y2vl+9CxFj0kNumi9WneuFnMPbRwWGw6hEqBp9eeVSpnf1NaWir+Nqc3JqzofJ065hGd1je6"
    "AFtk3rHJpVx+J7JbDY7Vcyzv2L3rT3jAZuGxmS38HDrqt+iLcZtflVL9RTyQEy/vzelNvczo"
    "vB/lLgonuppbQXbJJg6SXSd0xbW1Y/gaCwbcv9+GBsnwjy2Wui7Elm1xecWCRWXMSE6hVipX"
    "/1Es8Sq7gUHn8wQBPwZifnfFNTWvIzoNl1AKhaJzHzhWgcmO0Fi0fqsTNYUlGwruu5Da+tRm"
    "u5KlLJVdVYxZqVRf09mJji3Iyeldc/nQaS8zuj3JEglxbCk4FmpsbQ9xLLKoXL/IxkZBUlsx"
    "pLY+tdkmHhuO+1UdxeOPQZXYptGUisGyL3iSXXAtsXd02suLLoY4VoKOFaNjAd12zlpOiYFj"
    "aaGw1PUUAzZztgM0u+ToE3JcFMdKi9uUSuUXT2qMYl52OWvCNxDzD52XMhFGdCi7R8QgO5tj"
    "/1Fbq1NhjSU89q7fa8mum1lsqwgOOnMQHQ75kZvCfKwKfwlPqVz6d6NRU4rsbvAgu+CG/17R"
    "BfmrRKGgQ8dKSg0GMTp2KciuFR1LK+Zf/wpq02dn2zeeu05EbnhEo3nkGDX+Hh5G2zU3G1F2"
    "MF0C2QWFzu1GJ3/QBdmcBIWu5jpAJzEZQCbo2Nra7SwOKCzQfhT/ddheKrZ+0Ghyo9ikM3e3"
    "KXUUm3L2F+9f8+TNGg0pFKWl69w7NshNnd5aYq+nGw8AOsx26FhJJzgWsh06Vi7lGIvFAhVB"
    "sRULhcXOysJfyHZ0etSnqm68DZtOee6Opdc8+GD8k8sh0UF3IpGIjYGi876B3X902gAHYsGg"
    "4x0rNhg6UXbo2CnUsTBl33uvzA6Mx2ZXG6eQFVcBNh3Bppy99P177nnx+XhAJiaC6+xM4H6b"
    "kxNQrvO1W8fL8N/b6ca+h//BbHBHx6LsTBxx7DZw7GyWZXhMe4ftdaRmoduB8eBP85QxB3U6"
    "8nufOuXwu5546Jtvnr8UX8pD0ySY8g0cl/9CTE5NOHcmBvya1s+NTkGiA8eKiWNLocY+A45N"
    "Y1kLdWjl+nuzLRQaNCtEbSxJbgSbDq06/MYnHnrvmxcvXZIQxxNo9MfH8kcE3JwEs03Ovx24"
    "A4MuBsiJxVhjobX7FGRXDI5FWHBdNGyrXW3kcEVp3ZE2nU7Ok7vxrifeew/kthyU1kh+YJue"
    "kpdvMnZ+l1wwKyYnoJY4mE3BnnZa+Hpj6OiQ3dNisGwCOFYiJo7dB47lxaavNPPJDQ8GMEPH"
    "K7dxG/4xyu26+D+W2pVm4KEtX16AR2UU3BCTkxr2w3X8esXtTwIOALpXF0sWSzTEsZ3o2Foz"
    "cSzqDvdIE7kpzDPHDJfLKTbd8Bvveui9916M//v8hHz+pwENBpNR813p8oK+6J4Yk7MmKHT+"
    "bSjSems1vP8sWxiaE+rYZDFEPjgWfHcHoNsEjs2mtRTTG5PdM+agnHKD6z8+eeKhex588ifz"
    "rXZoCZrO5ZLkApfY4MmvkfIfxAC7Xy8GdI1YY3nHMiyPjYXkNvegrpaAg8u+G5e+/+C22zoa"
    "DHxOA2ily3HnEE54Tx+SPbx/9uTXCEL3qhjYLeZMpLH4JbCT8Qdey47M3kewEXQHv1h9cduF"
    "jp1WAyrNBNCQmkMU9LGDaVaMJ79GELqY+MUAL5/rhErbuRTQzeVY6Ny274OHwAyv+2avfu56"
    "wNZgIJWgsxS38yVLkvtFge3+lhgPTV3koCM7FRFdI2eC4ZMYHftMF8GG4PB64xeDL164rWOn"
    "ESoBQMPNVBIcg+ANfdyHDh9233rLDROR3JrIRzdxMbBbbMB9FBLJL2sdQr7v49UXr79w2803"
    "d3aKSyVOkezIjsxILrj16Rf+PHFPDI3AD9e5+tDVxCM7E6SwfKNmtZ3bMzcuHbzt+ief/CMO"
    "5wGqCzoeHw1naBhBHK5zdTqWZ4c/znET5fbx+9dsezE+XgIJEC5icocAxY7ocEPprdcRaHjI"
    "Hh81NalrvJ3bGTnoYmr2LCbRaEJ281evXfuHh77BzSBiW0js7Ag8MkeSfOm6W16duKeGBEVW"
    "k5qa6vuE2EhSHTqWhkbzsEYc//zz8fFiFCKZxORqC5wjAWiPoNIotRoiNFBaRP9EjKedig8v"
    "dgrx4j5wi/uYuUJDahRaQF8YUej2LHZht7iPHf9Mknzdrx2g1VB7BggtAtHVXHoY2D3sKjwb"
    "tEuoNEdm/uW0yEdHNngCOjrZAh+DP59+3Q4tpiYmVGiRiK7m1w8/TOnREEvin37kVbvSamrC"
    "Ay3i0BF2j0gotMWotIGCFono0Iqvvv60E7SY8EOLPHSimlQ++w+Y0iIWnUi0JpUMCWifFlzP"
    "8WNFR+gRpa0Z8C+KPHSXLQR0AjoBnYBOQCeEgE5AJ6AT0GHsENAJIaAT0AnoBHRCCOgEdAI6"
    "AZ09yBFXzs/9epPfy3p++5WDro9BAKsmGQh0vj7jikNnXyOJ/6vWf1F/3ttHLih0Pxw5z4al"
    "6yQJTHTBG+7qE50PdJI+gJL+/068czQ2v5h9tqNy7Y+cc4HE6SP7K7DvU53SgfMnOgPs+w6X"
    "5T2tgtPqeF7pAMqEIwqH1XcxmXt0jmvh8Hb7Y7dPHP9S/VbCaWk37+u33p6+yfcLHlY6YHQi"
    "kcutx/QkcZ0t6YfDCY1zRuj/ioshJaJ+4nQrCYnj5zou6e6v0+9DJa53XnKWl+akv9IkPtCJ"
    "XNG5CNU9OlH/pTzmsj5ZizyYSeLpj+BJ2P2ShfuVDgKdG8e7z+zOxJz+mc4OdrOqHtH1TwgS"
    "V7+6X41+f2Q3q+D4gtPau1vpQNGJfgB07kUncZ/r/ELnaRUcXxhAdP3zgd/o3H+i+8+SeM4C"
    "zl/Zv2C4QecmJ3hKFk5JyFvK8LdMiPyvEiKvVYJkZEmo6PiPdPkzeKgSbr9J4v0FiUs36y3V"
    "eUfnvrUKRHR+5Jl+vvRU5j01SxI3FTlQv3puWjz33L7RiSRemljf6EQSDw21yN1n9fsbS9w0"
    "G66fKHFuBN1WCU8Z1sWrHlf6yt/o9AOOqwL+/isLneQHQycJ/OuvJHSSH46cU+a9StFdRX83"
    "YQO7sG9CQCegE9AJIaAT0AnoBHQCOgGdgE5AJ6AT0AkhoBPQXXHohAgqBHQCussf/w89C40O"
    "4uNs8QAAAABJRU5ErkJggg==")
index.append('splash')
catalog['splash'] = splash

